/* File: api16.cpp
 * Purpose: Contains external API for the Bt848-based boards
 * Copyright (c) 1996 Brooktree Corporation. All Rights Reserved.
 * $Log: API16.CPP $
 * Revision 2.8  1997/09/30 22:07:13  rudyl
 * Renamed BlockRead() & BlockWrite() to MultiRead() & MultiWrite().
 * Revision 2.7  1997/09/11 00:10:16  dtharp
 * Mods required for name change of Bt848_32.dll to BtVid_32.dll.
 * Revision 2.6  1997/09/10 20:40:18  dtharp
 * Mods required for name change bt848_16.dll -> BtVid_16.dll.
 * Revision 2.5  1997/08/23 01:41:55  rudyl
 * Added sub-addr to I2CHWBlockRead & Write.
 * Revision 2.4  1997/08/14 18:08:27  rudyl
 * Graft from 2.3.2.2. Added Gemini support.
 * Revision 2.3.2.2  1997/07/30 01:09:41  rudyl
 * Gemini: Added support and I2CHWBlockRead() and I2CHWBlockWrite().
 * Revision 2.3.2.1  1997/07/22 23:54:19  rudyl
 * Added APIs to get dev ID, etc.
 * Revision 2.3  1996/06/26 22:16:58  rudyl
 * Added I2CGetLastError() and changed return types for I2CInitHWMode() and I2CInitSWMode().
 * Revision 2.2  1996/06/21 05:15:19  rudyl
 * Added ReadDecoder() and WriteDecoder().
 * Revision 2.1  1996/06/10 20:11:52  ssm
 * MarkBeta1AsRev2
 * Revision 1.3  1996/05/31 21:55:39  rudyl
 * Added software control of I2C.
 * Revision 1.2  1996/05/02 21:27:58  sasha
 * Seems to be a working version
 * Revision 1.1  1996/04/30 02:11:57  sasha
 * Initial revision
 */

#include "bt848api.h"


/* Internal thunk prototypes */

extern "C" {
//---------------------------------------------------------------------------
// I2C DATA/CONTROL REGISTER API
//---------------------------------------------------------------------------
bool      WINAPI Thunk_I2CIsInitOK( void );
ErrorCode WINAPI Thunk_I2CInitHWMode( long );
ErrorCode WINAPI Thunk_I2CInitSWMode( long );
void      WINAPI Thunk_I2CSetFreq( long );
int       WINAPI Thunk_I2CReadDiv( void );
ErrorCode WINAPI Thunk_I2CHWRead( BYTE, BYTE * );
ErrorCode WINAPI Thunk_I2CHWWrite2( BYTE, BYTE );
ErrorCode WINAPI Thunk_I2CHWWrite3( BYTE, BYTE, BYTE );
ErrorCode WINAPI Thunk_I2CHWMultiRead( BYTE, BYTE, BYTE *, int );
ErrorCode WINAPI Thunk_I2CHWMultiWrite( BYTE, BYTE, BYTE *, int );
ErrorCode WINAPI Thunk_I2CSWStart( void );
ErrorCode WINAPI Thunk_I2CSWStop( void );
ErrorCode WINAPI Thunk_I2CSWRead( BYTE * );
ErrorCode WINAPI Thunk_I2CSWWrite( BYTE );
ErrorCode WINAPI Thunk_I2CSWSendACK( void );
ErrorCode WINAPI Thunk_I2CSWSendNACK( void );
ErrorCode WINAPI Thunk_I2CSWSetSCL( Level );
int       WINAPI Thunk_I2CSWReadSCL( void );
ErrorCode WINAPI Thunk_I2CSWSetSDA( Level );
int       WINAPI Thunk_I2CSWReadSDA( void );
ErrorCode WINAPI Thunk_I2CSetSync( State );
int       WINAPI Thunk_I2CReadSync( void );
int       WINAPI Thunk_I2CGetLastError( void );


//---------------------------------------------------------------------------
// GPIO RELATED REGISTERS API
//---------------------------------------------------------------------------
bool      WINAPI Thunk_GPIOIsInitOK( void );
void      WINAPI Thunk_SetGPCLKMODE( State );
int       WINAPI Thunk_GetGPCLKMODE( void );
void      WINAPI Thunk_SetGPIOMODE( GPIOMode );
int       WINAPI Thunk_GetGPIOMODE( void );
void      WINAPI Thunk_SetGPWEC( State );
int       WINAPI Thunk_GetGPWEC( void );
void      WINAPI Thunk_SetGPINTI( State );
int       WINAPI Thunk_GetGPINTI( void );
void      WINAPI Thunk_SetGPINTC( State );
int       WINAPI Thunk_GetGPINTC( void );
ErrorCode WINAPI Thunk_SetGPOEBit( int, State );
void      WINAPI Thunk_SetGPOE( DWORD );
int       WINAPI Thunk_GetGPOEBit( int );
DWORD     WINAPI Thunk_GetGPOE( void );
ErrorCode WINAPI Thunk_SetGPIEBit( int, State );
ErrorCode WINAPI Thunk_SetGPIE( DWORD );
int       WINAPI Thunk_GetGPIEBit( int );
DWORD     WINAPI Thunk_GetGPIE( void );
ErrorCode WINAPI Thunk_SetGPDATA( GPIOReg *, int, int );
ErrorCode WINAPI Thunk_GetGPDATA( GPIOReg *, int, int );
ErrorCode WINAPI Thunk_SetGPDATABits( int, int, DWORD, int );
ErrorCode WINAPI Thunk_GetGPDATABits( int, int, DWORD *, int );


//---------------------------------------------------------------------------
// DECODER REGISTER API
//---------------------------------------------------------------------------
ErrorCode WINAPI Thunk_ReadDecoder( WORD, BYTE * );
ErrorCode WINAPI Thunk_WriteDecoder( WORD, BYTE );


//---------------------------------------------------------------------------
// HARDWARE INFORMATION API
//---------------------------------------------------------------------------
WORD WINAPI Thunk_GetHWDeviceID( void );
int  WINAPI Thunk_GetHWRevisionID( void );
BOOL WINAPI Thunk_IsBt848( void );
BOOL WINAPI Thunk_IsBt848A( void );
BOOL WINAPI Thunk_IsBt849A( void );
BOOL WINAPI Thunk_IsBt848Family( void );
BOOL WINAPI Thunk_IsBt878( void );
BOOL WINAPI Thunk_IsBt879( void );
BOOL WINAPI Thunk_IsBt880( void );
BOOL WINAPI Thunk_IsBt878Family( void );

}

extern "C" BOOL  WINAPI I2C_GPIO_ThunkConnect16( LPSTR pszDll16,
   LPSTR pszDll32, WORD hInst, DWORD dwReason );

extern "C" BOOL WINAPI _export DllEntryPoint( DWORD dwReason, WORD hInst,
   WORD, WORD, DWORD, WORD  )
{
   if ( !I2C_GPIO_ThunkConnect16( "btVid_16.dll", "btVid_32.dll", hInst, dwReason ) )
      return FALSE;
   return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// I2C DATA/CONTROL REGISTER API
/////////////////////////////////////////////////////////////////////////////

bool __BT848FUNC I2CIsInitOK( void )
{
   return Thunk_I2CIsInitOK();
}

//---------------------------------------------------------------------------
ErrorCode __BT848FUNC I2CInitHWMode( long freq )
{
   return Thunk_I2CInitHWMode( freq );
}

//---------------------------------------------------------------------------
ErrorCode __BT848FUNC I2CInitSWMode( long freq )
{
   return Thunk_I2CInitSWMode( freq );
}

//---------------------------------------------------------------------------
void __BT848FUNC I2CSetFreq( long freq )
{
   Thunk_I2CSetFreq( freq );
}

//---------------------------------------------------------------------------
int __BT848FUNC I2CReadDiv( void )
{
   return Thunk_I2CReadDiv();
}

//---------------------------------------------------------------------------
ErrorCode __BT848FUNC I2CHWRead( BYTE address, BYTE *value )
{
   return Thunk_I2CHWRead( address, value );
}

//---------------------------------------------------------------------------
ErrorCode __BT848FUNC I2CHWWrite2( BYTE address, BYTE value1 )
{
   return Thunk_I2CHWWrite2( address, value1 );
}

//---------------------------------------------------------------------------
ErrorCode __BT848FUNC I2CHWWrite3( BYTE address, BYTE value1, BYTE value2 )
{
   return Thunk_I2CHWWrite3( address, value1, value2 );
}

//---------------------------------------------------------------------------
ErrorCode __BT848FUNC I2CHWMultiRead( BYTE address, BYTE subAddr, BYTE * pBlock, int len )
{
   return Thunk_I2CHWMultiRead( address, subAddr, pBlock, len );
}

//---------------------------------------------------------------------------
ErrorCode __BT848FUNC I2CHWMultiWrite( BYTE address, BYTE subAddr, BYTE * pBlock, int len )
{
   return Thunk_I2CHWMultiWrite( address, subAddr, pBlock, len );
}

//---------------------------------------------------------------------------
ErrorCode __BT848FUNC I2CSWStart( void )
{
   return Thunk_I2CSWStart();
}

//---------------------------------------------------------------------------
ErrorCode __BT848FUNC I2CSWStop( void )
{
   return Thunk_I2CSWStop();
}

//---------------------------------------------------------------------------
ErrorCode __BT848FUNC I2CSWRead( BYTE * value )
{
   return Thunk_I2CSWRead( value );
}

//---------------------------------------------------------------------------
ErrorCode __BT848FUNC I2CSWWrite( BYTE value )
{
   return Thunk_I2CSWWrite( value );
}

//---------------------------------------------------------------------------
ErrorCode __BT848FUNC I2CSWSendACK( void )
{
   return Thunk_I2CSWSendACK();
}

//---------------------------------------------------------------------------
ErrorCode __BT848FUNC I2CSWSendNACK( void )
{
   return Thunk_I2CSWSendNACK();
}

//---------------------------------------------------------------------------
ErrorCode __BT848FUNC I2CSWSetSCL( Level scl )
{
   return Thunk_I2CSWSetSCL( scl );
}

//---------------------------------------------------------------------------
int __BT848FUNC I2CSWReadSCL( void )
{
   return Thunk_I2CSWReadSCL();
}

//---------------------------------------------------------------------------
ErrorCode __BT848FUNC I2CSWSetSDA( Level sda )
{
   return Thunk_I2CSWSetSDA( sda );
}

//---------------------------------------------------------------------------
int __BT848FUNC I2CSWReadSDA( void )
{
   return Thunk_I2CSWReadSDA();
}

//---------------------------------------------------------------------------
ErrorCode __BT848FUNC I2CSetSync( State sync )
{
   return Thunk_I2CSetSync( sync );
}

//---------------------------------------------------------------------------
int __BT848FUNC I2CReadSync( void )
{
   return Thunk_I2CReadSync();
}

//---------------------------------------------------------------------------
int __BT848FUNC I2CGetLastError( void )
{
   return Thunk_I2CGetLastError();
}


/////////////////////////////////////////////////////////////////////////////
// GPIO, GPOE, GPIE, and GPDATA REGISTER API
/////////////////////////////////////////////////////////////////////////////

bool __BT848FUNC GPIOIsInitOK( void )
{
   return Thunk_GPIOIsInitOK();
}

//---------------------------------------------------------------------------
void __BT848FUNC SetGPCLKMODE( State s )
{
   Thunk_SetGPCLKMODE( s );
}

//---------------------------------------------------------------------------
int __BT848FUNC GetGPCLKMODE( void )
{
   return Thunk_GetGPCLKMODE();
}

//---------------------------------------------------------------------------
void __BT848FUNC SetGPIOMODE( GPIOMode mode )
{
   Thunk_SetGPIOMODE( mode );
}

//---------------------------------------------------------------------------
int __BT848FUNC GetGPIOMODE( void )
{
   return Thunk_GetGPIOMODE();
}

//---------------------------------------------------------------------------
void __BT848FUNC SetGPWEC( State s )
{
   Thunk_SetGPWEC( s );
}

//---------------------------------------------------------------------------
int __BT848FUNC GetGPWEC( void )
{
   return Thunk_GetGPWEC();
}

//---------------------------------------------------------------------------
void __BT848FUNC SetGPINTI( State s )
{
   Thunk_SetGPINTI( s );
}

//---------------------------------------------------------------------------
int __BT848FUNC GetGPINTI( void )
{
   return Thunk_GetGPINTI();
}

//---------------------------------------------------------------------------
void __BT848FUNC SetGPINTC( State s )
{
   Thunk_SetGPINTC( s );
}

//---------------------------------------------------------------------------
int __BT848FUNC GetGPINTC( void )
{
   return Thunk_GetGPINTC();
}

//---------------------------------------------------------------------------
ErrorCode __BT848FUNC SetGPOEBit( int bit, State s )
{
   return Thunk_SetGPOEBit( bit, s );
}

//---------------------------------------------------------------------------
void __BT848FUNC SetGPOE( DWORD value )
{
   Thunk_SetGPOE( value );
}

//---------------------------------------------------------------------------
int __BT848FUNC GetGPOEBit( int bit )
{
   return Thunk_GetGPOEBit( bit );
}

//---------------------------------------------------------------------------
DWORD __BT848FUNC GetGPOE( void )
{
   return Thunk_GetGPOE();
}

//---------------------------------------------------------------------------
ErrorCode __BT848FUNC SetGPIEBit( int bit , State s )
{
   return Thunk_SetGPIEBit( bit, s );
}

//---------------------------------------------------------------------------
ErrorCode __BT848FUNC SetGPIE( DWORD value )
{
   return Thunk_SetGPIE( value );
}

//---------------------------------------------------------------------------
int __BT848FUNC GetGPIEBit( int bit )
{
   return Thunk_GetGPIEBit( bit );
}

//---------------------------------------------------------------------------
DWORD __BT848FUNC GetGPIE( void )
{
   return Thunk_GetGPIE();
}

//---------------------------------------------------------------------------
ErrorCode __BT848FUNC SetGPDATA( GPIOReg *data, int size, int offset )
{
   return Thunk_SetGPDATA( data, size, offset );
}

//---------------------------------------------------------------------------
ErrorCode __BT848FUNC GetGPDATA( GPIOReg *data, int size, int offset )
{
   return Thunk_GetGPDATA( data, size, offset );
}

//---------------------------------------------------------------------------
ErrorCode __BT848FUNC SetGPDATABits( int fromBit, int toBit, DWORD value, int offset )
{
   return Thunk_SetGPDATABits( fromBit, toBit, value, offset );
}

//---------------------------------------------------------------------------
ErrorCode __BT848FUNC GetGPDATABits( int fromBit, int toBit, DWORD *value, int offset )
{
   return Thunk_GetGPDATABits( fromBit, toBit, value, offset );
}


/////////////////////////////////////////////////////////////////////////////
// DECODER REGISTER API
/////////////////////////////////////////////////////////////////////////////

//---------------------------------------------------------------------------
ErrorCode __BT848FUNC ReadDecoder( WORD regOffset, BYTE * value )
{
   return Thunk_ReadDecoder( regOffset, value );
}

//---------------------------------------------------------------------------
ErrorCode __BT848FUNC WriteDecoder( WORD regOffset, BYTE value )
{
   return Thunk_WriteDecoder( regOffset, value );
}


/////////////////////////////////////////////////////////////////////////////
// HARDWARE INFORMATION API
/////////////////////////////////////////////////////////////////////////////

//---------------------------------------------------------------------------
WORD __BT848FUNC GetHWDeviceID( void )
{
   return Thunk_GetHWDeviceID();
}

//---------------------------------------------------------------------------
int __BT848FUNC GetHWRevisionID( void )
{
   return Thunk_GetHWRevisionID();
}

//---------------------------------------------------------------------------
BOOL __BT848FUNC IsBt848( void )
{
   return Thunk_IsBt848();
}

//---------------------------------------------------------------------------
BOOL __BT848FUNC IsBt848A( void )
{
   return Thunk_IsBt848A();
}

//---------------------------------------------------------------------------
BOOL __BT848FUNC IsBt849A( void )
{
   return Thunk_IsBt849A();
}

//---------------------------------------------------------------------------
BOOL __BT848FUNC IsBt848Family( void )
{
   return Thunk_IsBt848Family();
}

//---------------------------------------------------------------------------
BOOL __BT848FUNC IsBt878( void )
{
   return Thunk_IsBt878();
}

//---------------------------------------------------------------------------
BOOL __BT848FUNC IsBt879( void )
{
   return Thunk_IsBt879();
}

//---------------------------------------------------------------------------
BOOL __BT848FUNC IsBt880( void )
{
   return Thunk_IsBt880();
}

//---------------------------------------------------------------------------
BOOL __BT848FUNC IsBt878Family( void )
{
   return Thunk_IsBt878Family();
}

