// $Header: G:/SwDev/W95/Video/pisces/dvlpmnt/Bt848_32/rcs/GPIO.CPP 2.5 1997/08/14 18:23:13 rudyl Released $
// $Log: GPIO.CPP $
// Revision 2.5  1997/08/14 18:23:13  rudyl
// Graft from 2.4.1.2. Added Gemini support.
// Revision 2.4.1.2  1997/07/30 01:08:21  rudyl
// Added support for Gemini.
// Revision 2.4.1.1  1997/07/17 19:33:24  rudyl
// Add support for Gemini.
// Revision 2.4  1997/04/16 17:36:37  rudyl
// Removed reg names.
// Revision 2.3.1.2  1996/12/17 18:02:56  rudyl
// Removed reg names
// Revision 2.3.1.1  1996/12/17 18:00:31  rudyl
// remove reg names
// Revision 2.3  1996/08/02 00:48:54  rudyl
// Added comments only.
// Revision 2.2  1996/06/11 23:04:40  rudyl
// only changed comments
// Revision 2.1  1996/06/10 20:11:59  ssm
// MarkBeta1AsRev2
// Revision 1.2  1996/04/28 01:25:16  rudyl
// ver 1.00 - both I2C and GPIO are now objects of register class.
// Created a set of API wrapper functions to invoke member functions of the classes.
// Revision 1.1  1996/04/26 23:52:48  rudyl
// Initial revision
// Revision 1.6  1996/04/24 17:16:09  rudyl
// Need to generate a 16 bits DLL. Added typecast to remove warning msg.
// Added #include <mem.h> for memcpy().
// Revision 1.5  1996/04/23 22:11:47  rudyl
// Removed codes inside SetAudio() and GetAudio() from GPIO to SetAudioSource() and GetAudioSource() in Grappler.DLL
// Revision 1.4  1996/04/17 20:21:07  rudyl
// To set audio, need to set GPOE to 0x0F (instead of 0x03). Don't know why.
// Revision 1.3  1996/04/12 21:05:01  rudyl
// Added overloaded SetGPDATA() and GetGPDATA() to set a range
// of bits of GPDATA register.
// Added SetAudio() and GetAudio() for audio source.
// Revision 1.2  1996/04/09 22:33:26  rudyl
// Added copyright info.

/////////////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 1996 Brooktree Corporation. All Rights Reserved.
//
//  Module:
//
//    Gpio.cpp
//
//  Abstract:
//
//    Implementation file for Pisces GPIO API
//
/////////////////////////////////////////////////////////////////////////////

#include <mem.h>
#include "gpio.h"
#include "bt848api.h"

/////////////////////////////////////////////////////////////////////////////
// Constructor
/////////////////////////////////////////////////////////////////////////////
GPIO::GPIO( void ) :
   // construct all GPIO related register and register fields
   decRegGPIO ( 0x10C, RW ),                    // GPIO register
   decFieldGPCLKMODE( decRegGPIO, 10, 1, RW ),
   decFieldGPIOMODE( decRegGPIO, 11, 2, RW ),
   decFieldGPWEC( decRegGPIO, 13, 1, RW ),
   decFieldGPINTI( decRegGPIO, 14, 1, RW ),
   decFieldGPINTC( decRegGPIO, 15, 1, RW ),
   decRegGPOE( 0x118, RW ),                     // GPOE register
   decRegGPIE( 0x11C, RW ),                     // GPIE register
   decRegGPDATA( 0x200, WO ),                   // GPDATA register
   initOK( false )
{
   initOK = true;
}

/////////////////////////////////////////////////////////////////////////////
// Destructor
/////////////////////////////////////////////////////////////////////////////
GPIO::~GPIO()
{
}

/////////////////////////////////////////////////////////////////////////////
// Method:  bool GPIO::IsInitOK( void )
// Purpose: Check if GPIO is initialized successfully
// Input:   None
// Output:  None
// Return:  true or false
// Note:    Before calling other GPIO functions, application should call
//          this function and check for return value is true to ensure
//          initialization is successful.
/////////////////////////////////////////////////////////////////////////////
bool GPIO::IsInitOK( void )
{
   return( initOK );
}

/////////////////////////////////////////////////////////////////////////////
// Method:  void GPIO::SetGPCLKMODE( State s )
// Purpose: Set or clear GPCLKMODE
// Input:   State s - On to set; Off to clear
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
void GPIO::SetGPCLKMODE( State s )
{
   decFieldGPCLKMODE = s;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int GPIO::GetGPCLKMODE( void )
// Purpose: Get value of GPCLKMODE
// Input:   None
// Output:  None
// Return:  int - On (1) or Off (0)
/////////////////////////////////////////////////////////////////////////////
int GPIO::GetGPCLKMODE( void )
{
   return ( decFieldGPCLKMODE );
}

/////////////////////////////////////////////////////////////////////////////
// Method:  void GPIO::SetGPIOMODE( GPIOMode mode )
// Purpose: Set GPIO mode
// Input:   GPIOMode mode - GPIO_NORMAL, GPIO_SPI_OUTPUT, GPIO_SPI_INPUT,
//                          GPIO_DEBUG_TEST
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
void GPIO::SetGPIOMODE( GPIOMode mode )
{
   decFieldGPIOMODE = mode;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int GPIO::GetGPIOMODE( void )
// Purpose: Get GPIO mode
// Input:   None
// Output:  None
// Input:   int - GPIO_NORMAL, GPIO_SPI_OUTPUT, GPIO_SPI_INPUT, GPIO_DEBUG_TEST
/////////////////////////////////////////////////////////////////////////////
int GPIO::GetGPIOMODE( void )
{
   return( decFieldGPIOMODE );
}

/////////////////////////////////////////////////////////////////////////////
// Method:  void GPIO::SetGPWEC( State s )
// Purpose: Set or clear GPWEC
// Input:   State s - On to set; Off to clear
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
void GPIO::SetGPWEC( State s )
{
   decFieldGPWEC = s;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int GPIO::GetGPWEC( void )
// Purpose: Get value of GPWEC
// Input:   None
// Output:  None
// Return:  int - On (1) or Off (0)
/////////////////////////////////////////////////////////////////////////////
int GPIO::GetGPWEC( void )
{
   return ( decFieldGPWEC );
}

/////////////////////////////////////////////////////////////////////////////
// Method:  void GPIO::SetGPINTI( State s )
// Purpose: Set or clear GPINTI
// Input:   State s - On to set; Off to clear
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
void GPIO::SetGPINTI( State s )
{
   decFieldGPINTI = s;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int GPIO::GetGPINTI( void )
// Purpose: Get value of GPINTI
// Input:   None
// Output:  None
// Return:  int - On (1) or Off (0)
/////////////////////////////////////////////////////////////////////////////
int GPIO::GetGPINTI( void )
{
   return ( decFieldGPINTI );
}

/////////////////////////////////////////////////////////////////////////////
// Method:  void GPIO::SetGPINTC( State s )
// Purpose: Set or clear GPINTC
// Input:   State s - On to set; Off to clear
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
void GPIO::SetGPINTC( State s )
{
   decFieldGPINTC = s;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int GPIO::GetGPINTC( void )
// Purpose: Get value of GPINTC
// Input:   None
// Output:  None
// Return:  int - On (1) or Off (0)
/////////////////////////////////////////////////////////////////////////////
int GPIO::GetGPINTC( void )
{
   return ( decFieldGPINTC );
}

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode GPIO::SetGPOE( int bit, State s )
// Purpose: Set or clear a bit in GPOE
// Input:   int bit - bit to be set or clear
//          State s - On to set; Off to clear
// Output:  None
// Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode GPIO::SetGPOE( int bit, State s )
{
   // range checking
   if ( ( bit < 0 ) || ( bit > MAX_GPIO_BIT ) )
      return ( Fail );

   RegField decFieldTemp( decRegGPOE, (BYTE)bit, 1, RW );   // create a reg field
   decFieldTemp = s;

   return ( Success );
}

/////////////////////////////////////////////////////////////////////////////
// Method:  void GPIO::SetGPOE( DWORD value )
// Purpose: Set GPOE value
// Input:   DWORD value - value to be set to
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
void GPIO::SetGPOE( DWORD value )
{
   decRegGPOE = ( value & 0x00FFFFFFL );     // bits [23:0]
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int GPIO::GetGPOE( int bit )
// Purpose: Get value of a bit in GPOE
// Input:   int bit - bit to get value from
// Output:  None
// Return:  int - On (1), Off (0), or -1 for parameter error
/////////////////////////////////////////////////////////////////////////////
int GPIO::GetGPOE( int bit )
{
   // range checking
   if ( ( bit < 0 ) || ( bit > MAX_GPIO_BIT ) )
      return ( -1 );

   RegField decFieldTemp( decRegGPOE, (BYTE)bit, 1, RW );   // create a reg field
   return( decFieldTemp );
}

/////////////////////////////////////////////////////////////////////////////
// Method:  DWORD GPIO::GetGPOE( void )
// Purpose: Get value of GPOE
// Input:   None
// Output:  None
// Return:  Content of GPOE (DWORD)
/////////////////////////////////////////////////////////////////////////////
DWORD GPIO::GetGPOE( void )
{
   return ( decRegGPOE & 0x00FFFFFFL );
}

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode GPIO::SetGPIE( int bit, State s )
// Purpose: Set or clear a bit in GPIE
// Input:   int bit - bit to be set or clear
//          State s - On to set; Off to clear
// Output:  None
// Return:  Success, Fail or Err_WrongHardware
/////////////////////////////////////////////////////////////////////////////
ErrorCode GPIO::SetGPIE( int bit, State s )
{
   // GPIE is deleted in Bt878 family
   if ( IsBt878Family() )
      return ( Err_WrongHardware );

   // range checking
   if ( ( bit < 0 ) || ( bit > MAX_GPIO_BIT ) )
      return ( Fail );

   RegField decFieldTemp( decRegGPIE, (BYTE)bit, 1, RW );   // create a reg field
   decFieldTemp = s;

   return ( Success );
}

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode GPIO::SetGPIE( DWORD value )
// Purpose: Set GPIE value
// Input:   DWORD value - value to be set to
// Output:  None
// Return:  Success, Fail or Err_WrongHardware
/////////////////////////////////////////////////////////////////////////////
ErrorCode GPIO::SetGPIE( DWORD value )
{
   // GPIE is deleted in Bt878 family
   if ( IsBt878Family() )
      return ( Err_WrongHardware );

   decRegGPIE = ( value & 0x00FFFFFFL );     // bits [23:0]
   return ( Success );
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int GPIO::GetGPIE( int bit )
// Purpose: Get value of a bit in GPIE
// Input:   int bit - bit to get value from
// Output:  None
// Return:  int - On (1), Off (0), or -1 for parameter error
/////////////////////////////////////////////////////////////////////////////
int GPIO::GetGPIE( int bit )
{
   // GPIE is deleted in Bt878 family
   if ( IsBt878Family() )
      return ( -1 );

   // range checking
   if ( ( bit < 0 ) || ( bit > MAX_GPIO_BIT ) )
      return ( -1 );

   RegField decFieldTemp( decRegGPIE, (BYTE)bit, 1, RW );   // create a reg field
   return( decFieldTemp );
}

/////////////////////////////////////////////////////////////////////////////
// Method:  DWORD GPIO::GetGPIE( void )
// Purpose: Get value of GPIE
// Input:   None
// Output:  None
// Return:  Content of GPIE (DWORD)
/////////////////////////////////////////////////////////////////////////////
DWORD GPIO::GetGPIE( void )
{
   // GPIE is deleted in Bt878 family
   if ( IsBt878Family() )
      return ( -1 );

   return ( decRegGPIE & 0x00FFFFFFL );
}

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode GPIO::SetGPDATA( GPIOReg * data, int size, int offset )
// Purpose: Set GPDATA registers contents
// Input:   GPIOReg * data - ptr to data to be copied from
//          int size       - number of registers to be copied; max is 64
//          int offset     - how many registers to skip (default = 0)
// Output:  None
// Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode GPIO::SetGPDATA( GPIOReg * data, int size, int offset )
{
   // check size and offset for out of bound
   if ( ( offset < 0 ) || ( size < 0 ) || ( size + offset > MAX_GPDATA_SIZE ) )
      return ( Fail );

   // point to offseted register
   GPIOReg * p = (GPIOReg *)((char *)decRegGPDATA.GetBaseAddress() +
                             decRegGPDATA.GetOffset() +
                             offset * sizeof( GPIOReg ));   // pts to offseted register
   memcpy( p, data, size * sizeof( GPIOReg ) );
   return ( Success );
}

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode GPIO::GetGPDATA( GPIOReg * data, int size, int offset )
// Purpose: Get GPDATA registers contents
// Input:   GPIOReg * data - ptr to data to be copied to
//          int size       - number of registers to be copied; max is 64
//          int offset     - how many registers to skip (default = 0)
// Output:  None
// Return:  Success or Fail
// Note:    This function can only retrieve the last value written to GPDATA
/////////////////////////////////////////////////////////////////////////////
ErrorCode GPIO::GetGPDATA( GPIOReg * data, int size, int offset )
{
   // check size and offset for out of bound
   if ( ( offset < 0 ) || ( size < 0 ) || ( size + offset > MAX_GPDATA_SIZE ) )
      return ( Fail );

   // point to offseted register
   GPIOReg * p = (GPIOReg *)((char *)decRegGPDATA.GetBaseAddress() +
                             decRegGPDATA.GetOffset() +
                             offset * sizeof( GPIOReg ));   // pts to offseted register
   memcpy( data, p, size * sizeof( GPIOReg ) );
   return ( Success );
}


/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode GPIO::SetGPDATA( int fromBit, int toBit,
//                                     DWORD value, int offset )
// Purpose: Set GPDATA contents with specified range of bits
// Input:   int fromBit - starting bit
//          int toBit   - ending bit
//          DWORD value - value to set to
//          int offset  - how many registers to skip (default = 0)
// Output:  None
// Return:  Success or Fail
// Comment: By specifying a range of bits to be set, it allows a portion of
//          the GDATA register to be modified. For example,
//             SetGPDATA( 8, 10, 5, 0 )
//          would set the first GPDATA register's [10:8] to value 0x101
/////////////////////////////////////////////////////////////////////////////
ErrorCode GPIO::SetGPDATA( int fromBit, int toBit, DWORD value, int offset )
{
   // check size and offset for out of bound
   if ( ( fromBit < 0 ) || ( fromBit > MAX_GPIO_BIT ) ||
        ( toBit   < 0 ) || ( toBit   > MAX_GPIO_BIT ) || ( fromBit > toBit ) ||
        ( offset  < 0 ) || ( offset  > MAX_GPDATA_SIZE ) )
      return ( Fail );

   // make sure value can "fit" into range of bits specified
   if ( value >= ( 0x00000001L << ( toBit - fromBit + 1 ) ) )
      return ( Fail );

   RegisterDW reg( decRegGPDATA.GetOffset() + offset * sizeof( GPIOReg ), RW, "temp" );
   RegField field( reg, (BYTE)fromBit, (BYTE)(toBit - fromBit + 1), RW );
   field = value;

   return ( Success );
}


/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode GPIO::GetGPDATA( int fromBit, int toBit,
//                                     DWORD * value, int offset )
// Purpose: Get GPDATA contents with specified range of bits
// Input:   int fromBit - starting bit
//          int toBit   - ending bit
//          int offset  - how many registers to skip (default = 0)
// Output:  DWORD * value - value retrieved
// Return:  Success or Fail
// Comment: By specifying a range of bits to be set, it allows a portion of
//          the GDATA register to be retrieved. For example,
//             GetGPDATA( 8, 10, &data, 0 )
//          would set *data to the first GPDATA register's [10:8] content
// Note:    This function can only retrieve the last value written to GPDATA
/////////////////////////////////////////////////////////////////////////////
ErrorCode GPIO::GetGPDATA( int fromBit, int toBit, DWORD * value, int offset )
{
   // check size and offset for out of bound
   if ( ( fromBit < 0 ) || ( fromBit > MAX_GPIO_BIT ) ||
        ( toBit   < 0 ) || ( toBit   > MAX_GPIO_BIT ) || ( fromBit > toBit ) ||
        ( offset  < 0 ) || ( offset  > MAX_GPDATA_SIZE ) )
      return ( Fail );

   RegisterDW reg( decRegGPDATA.GetOffset() + offset * sizeof( GPIOReg ), RW, "temp" );
   RegField field( reg, (BYTE)fromBit, (BYTE)(toBit - fromBit + 1), RW );
   *value = field;

   return ( Success );
}


