// $Header: G:/SwDev/W95/Video/pisces/dvlpmnt/Bt848_32/rcs/GPIO.H 2.2 1997/08/14 18:23:52 rudyl Released $
// $Log: GPIO.H $
// Revision 2.2  1997/08/14 18:23:52  rudyl
// Graft from 2.1.1.2. Added Gemini support.
// Revision 2.1.1.2  1997/07/30 01:08:39  rudyl
// Added support for Gemini.
// Revision 2.1.1.1  1997/07/17 19:33:45  rudyl
// Add support for Gemini.
// Revision 2.1  1996/06/10 20:11:59  ssm
// MarkBeta1AsRev2
// Revision 1.2  1996/04/28 01:25:16  rudyl
// ver 1.00 - both I2C and GPIO are now objects of register class.
// Created a set of API wrapper functions to invoke member functions of the classes.
// Revision 1.1  1996/04/26 23:52:49  rudyl
// Initial revision
// Revision 1.6  1996/04/23 22:11:47  rudyl
// Removed codes inside SetAudio() and GetAudio() from GPIO to SetAudioSource() and GetAudioSource() in Grappler.DLL
// Revision 1.5  1996/04/12 21:05:01  rudyl
// Added overloaded SetGPDATA() and GetGPDATA() to set a range
// of bits of GPDATA register.
// Added SetAudio() and GetAudio() for audio source.
// Revision 1.4  1996/04/09 22:33:27  rudyl
// Added copyright info.

/////////////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 1996 Brooktree Corporation. All Rights Reserved.
//
//  Module:
//
//    Gpio.h
//
//  Abstract:
//
//    Header file for Pisces GPIO API
//
/////////////////////////////////////////////////////////////////////////////

#ifndef __GPIO_H
#define __GPIO_H

#ifdef _BUILD_GPIODLL
  #define __GPIOCLASS __export
  #define __GPIOFUNC __export
#else
  #define __GPIOCLASS __import
  #ifdef __WIN32__
    #define __GPIOFUNC __import
  #else
    #define __GPIOFUNC
  #endif
#endif

#ifndef CLASSLIB_DEFS_H
#include <classlib/defs.h>
#endif

#include "regField.h"
#include "viddefs.h"
#include "gpiotype.h"
#include "retcode.h"


//===========================================================================
// Constants
//===========================================================================
const int MAX_GPDATA_SIZE = 64;   // maximum number of DWORDs in GPDATA
const int MAX_GPIO_BIT    = 23;   // maximum bit number for GPIO registers


/////////////////////////////////////////////////////////////////////////////
// CLASS GPIO
//
// Description:
//    This class encapsulates the register fields in the GPIO register of the
//    Bt848. A complete set of functions are developed to manipulate all the
//    register fields in the GPIO for the Bt848.
//
/////////////////////////////////////////////////////////////////////////////

class __GPIOCLASS GPIO
{
private:
   bool initOK;

public:
   //  Constructor
   GPIO( void );
   ~GPIO();

   // Member functions
   bool      IsInitOK( void );
   void      SetGPCLKMODE( State );
   int       GetGPCLKMODE( void );
   void      SetGPIOMODE( GPIOMode );
   int       GetGPIOMODE( void );
   void      SetGPWEC( State );
   int       GetGPWEC( void );
   void      SetGPINTI( State );
   int       GetGPINTI( void );
   void      SetGPINTC( State );
   int       GetGPINTC( void );
   ErrorCode SetGPOE( int, State );
   void      SetGPOE( DWORD );
   int       GetGPOE( int );
   DWORD     GetGPOE( void );
   ErrorCode SetGPIE( int, State );
   ErrorCode SetGPIE( DWORD );
   int       GetGPIE( int );
   DWORD     GetGPIE( void );
   ErrorCode SetGPDATA( GPIOReg *, int, int offset = 0 );
   ErrorCode GetGPDATA( GPIOReg *, int, int offset = 0 );
   ErrorCode SetGPDATA( int, int, DWORD, int offset = 0 );
   ErrorCode GetGPDATA( int, int, DWORD *, int offset = 0 );

protected:
   RegisterW decRegGPIO;
   RegField decFieldGPCLKMODE;
   RegField decFieldGPIOMODE;
   RegField decFieldGPWEC;
   RegField decFieldGPINTI;
   RegField decFieldGPINTC;
   RegisterDW decRegGPOE;
   RegisterDW decRegGPIE;
   RegisterDW decRegGPDATA;
 
};

#endif  // __GPIO_H

