// $Header: G:/SwDev/W95/Video/pisces/dvlpmnt/Bt848_32/rcs/I2C.CPP 2.11 1997/09/30 18:37:07 rudyl Released $
// $Log: I2C.CPP $
// Revision 2.11  1997/09/30 18:37:07  rudyl
// Renamed BlockRead() & BlockWrite() to MultiRead() & MultiWrite().
// Revision 2.10  1997/08/23 01:43:01  rudyl
// Added sub-addr to I2CHWBlockRead & Write.
// Revision 2.9  1997/08/14 18:24:37  rudyl
// Graft from 2.8.1.2. Added Gemini support.
// Revision 2.8.1.2  1997/07/30 01:02:07  rudyl
// Gemini: Added support and I2CHWBlockRead() and I2CHWBlockWrite().
// Revision 2.8.1.1  1997/07/17 19:34:08  rudyl
// Add support for Gemini.
// Revision 2.8  1997/04/16 17:37:20  rudyl
// Removed reg names.
// Revision 2.7.1.2  1996/12/17 18:03:30  rudyl
// Removed reg names
// Revision 2.7.1.1  1996/12/17 18:00:56  rudyl
// remove reg names
// Revision 2.7  1996/08/02 00:48:54  rudyl
// Added comments only.
// Revision 2.6  1996/07/25 21:14:39  rudyl
// Place a check in I2C::IsInitOK() if register's base address returns 0, return false.
// Revision 2.5  1996/07/24 22:57:01  rudyl
// Didn't quite fix reading multiple bytes problem. Let's try again.
// Revision 2.4  1996/07/09 17:03:39  rudyl
// Fix the problem of not able to read more than 1 byte from slave in SW mode.
// Revision 2.3  1996/06/25 21:57:42  rudyl
// Added I2CGetLastError() and changed return types for I2CInitHWMode() and I2CInitSWMode().
// Revision 2.2  1996/06/11 23:04:50  rudyl
// only changed comments
// Revision 2.1  1996/06/10 20:12:00  ssm
// MarkBeta1AsRev2
// Revision 1.11  1996/06/05 00:40:49  rudyl
// Added checking ot SCL to wait for SCL goes hi or low after setting it since the slave may have control of it.
// Revision 1.10  1996/05/30 20:21:07  rudyl
// Cleanup the code and added comments for software control of I2C.
// Added comments for bug with register class and to do list.
// Revision 1.9  1996/05/30 19:08:24  rudyl
// Removed vxd related calls from Bt848api.h to vxdcall.h.
// Moved define of __BT848CLASS & __BT848FUNC to def__.h so both Bt848api.h and vxdCall.h can include it.
// Also added software control of I2C; seems working properly but need to cleanup the codes and add comments.
// Revision 1.8  1996/05/24 18:38:47  rudyl
// Added software control of SCL & SDA. Code compiled OK. No testing done.
// Revision 1.6  1996/05/23 19:20:01  rudyl
// Added functions skeleton (no code inside) for software control of I2C.
// Revision 1.4  1996/04/30 01:07:23  sasha
// Removed base address initialization
// Revision 1.3  1996/04/29 17:51:32  rudyl
// A few parameter type changes in I2C (thus in API) to make it consistent with GPIO.
// Revision 1.2  1996/04/28 01:25:17  rudyl
// ver 1.00 - both I2C and GPIO are now objects of register class.
// Created a set of API wrapper functions to invoke member functions of the classes.
// Revision 1.1  1996/04/26 23:52:51  rudyl
// Initial revision
// Revision 1.5  1996/04/24 17:21:58  rudyl
// Need to generate a 16 bits DLL. Added codes to remove warning msg. Nothing changed.
// Revision 1.4  1996/04/09 21:36:49  rudyl
// Added copyright info.
// Revision 1.3  1996/03/20 02:17:56  rudyl
// Should be no changes.
// Revision 1.2  1996/03/12 18:54:14  rudyl
// Deleted GetBt848LinearAddress(). Already in 848lin.cpp.
// Revision 1.1  1996/03/12 18:16:08  rudyl
// Initial revision
// Revision 1.4  1996/03/12 00:35:52  rudyl
// Return ErrorCode type for some functions.
// Revision 1.3  1996/02/28 01:36:11  rudyl
// All functions checked OK. Added I2CReadSync().

/////////////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 1996 Brooktree Corporation. All Rights Reserved.
//
//  Module:
//
//    I2C.cpp
//
//  Abstract:
//
//    Implementation of I2C register DLL
//
//  Note:
//    Due to certain behavior and operation requirements of the I2C
//    (listed below), an I2C register structure is created and acts as a temp
//    storage for I2C.
//
//    Behavior of I2C:
//       1) When modifying more than 1 field of I2C, must set the content of the
//          whole register.
//       2) Be very careful of the register type; RO, RW, RR, & WO.
//          WO does not read from hardware whereas RW does.
//       3) Since changing channel requires writing to I2C and read back the
//          status from I2C, care must be taken not to modify any other bits
//          except the ones supposed to be altered.
//
//    ***** IMPORTANT NOTE for software control of I2C: *****
//       1) There is a bug somewhere in Register class and a problem exists
//          when try to read from the SDA line. The problem is after sending
//          the tuner address for read operation, software set SDA to high
//          so that slave can pull it low for ACK signal. Even using a logic
//          analyser, we never see SDA goes high due to the fact that slave
//          probably have pull it low already. That's ok. Next step we do is
//          a read SDA to make sure slave did ACK (by checking SDA=0). Then
//          we generate a clock pulse to let slave to release pulling the SDA
//          line low. As soon as slave releases SDA, we should see SDA goes
//          back to high (since we wrote 1 to the register field SDA before).
//          We never see SDA goes high thus we are unable to read valid bits
//          from the slave.
//          The fix for it for right now is to use pointer to I2C register and
//          set SDA and SCL directly to 1 or 0 without going through register
//          class. In the future, register class should be used again after
//          finding out what the problem is.
//       2) Everytime we releases the SCL line (i.e. let it go high), we need
//          to check and make sure SCL really goes high before we continue
//          because the slave could pull it low for whatever reason
//          (for example, slave is not ready) and releases it later. What to
//          do is to have a timeout loop and check if SCL really goes high.
//
/////////////////////////////////////////////////////////////////////////////

//===========================================================================
// I2C DATA/CONTROL REGISTER API
//===========================================================================

#include "i2c.h"
#include "bt848api.h"

//**************************************************************************
//	DEFINES
//**************************************************************************
#define I2C_WRITE         0         // write operation
#define I2C_READ          1         // read operation

#define PCI_FREQ          33000000L // frequency for PCI in hz
#define I2CDIV_MAX        15        // I2C maximum divider

#define TIMEOUT           500       // timeout value (500ms) to wait for an I2C operation
                                    // passing 3 values to I2C takes ~450ms

//**************************************************************************
//	Structures
//**************************************************************************
struct _i2c_reg    // I2C register structure
{
	unsigned int sda:1;
	unsigned int scl:1;
	unsigned int w3b:1;
	unsigned int sync:1;
	unsigned int div:4;
	unsigned int byte2:8;
	unsigned int byte1:8;
	unsigned int addr_rw:8;
} i2cShadow;

struct _Gemini_i2c_reg           // I2C register structure for Gemini
{
	unsigned int sda:1;
	unsigned int scl:1;
  	unsigned int w3bra:1;
	unsigned int sync:1;
   unsigned int nos1b:1;
   unsigned int nostop:1;
   unsigned int rate:1;
   unsigned int mode:1;
	unsigned int byte2:8;
	unsigned int byte1:8;
	unsigned int addr_rw:8;
} * pGeminiI2CShadow;

//===========================================================================
// Bt848 I2C Class Implementation
//===========================================================================

/////////////////////////////////////////////////////////////////////////////
// Constructor
/////////////////////////////////////////////////////////////////////////////
I2C::I2C( void ) :
   // construct I2C register and register fields
   decRegINT_STAT ( 0x100, RW ),                // Interrupt Status register
   decFieldI2CDONE( decRegINT_STAT, 8, 1, RR ),
   decFieldRACK( decRegINT_STAT, 25, 1, RO ),
   decRegI2C ( 0x110, RW ),                     // I2C Data/Control register
   decFieldI2CDB0( decRegI2C, 24, 8, RW ),
   decFieldI2CDB1( decRegI2C, 16, 8, RW ),
   decFieldI2CDB2( decRegI2C,  8, 8, RW ),
   decFieldI2CDIV( decRegI2C,  4, 4, RW),
   decFieldSYNC( decRegI2C, 3, 1, RW),
   decFieldW3B( decRegI2C, 2, 1, RW),
   decFieldSCL( decRegI2C, 1, 1, RW),
   decFieldSDA( decRegI2C, 0, 1, RW),
   decFieldI2CMODE( decRegI2C, 7, 1, RW),   // new for Gemini
   decFieldI2CRATE( decRegI2C, 6, 1, RW),
   decFieldI2CNOSTOP( decRegI2C, 5, 1, RW),
   decFieldI2CNOS1B( decRegI2C, 4, 1, RW),
   decFieldI2CW3BRA( decRegI2C, 2, 1, RW)
{
   initOK = false;
   cycle  = 0L;
   errNum = I2CERR_OK;
   mode   = I2CMode_None;

   // Gemini: points to same location as in old I2C
   pGeminiI2CShadow = (_Gemini_i2c_reg *)&i2cShadow;
}

/////////////////////////////////////////////////////////////////////////////
// Destructor
/////////////////////////////////////////////////////////////////////////////
I2C::~I2C()
{
}


/////////////////////////////////////////////////////////////////////////////
//  Method:  bool I2C::IsInitOK( void )
//  Purpose: Check if I2C is initialized successfully
//  Input:   None
//  Output:  None
//  Return:  true or false
//  Note:    Before calling other I2C functions, application should call
//           this function and check for return value is true to ensure
//           initialization is successful.
/////////////////////////////////////////////////////////////////////////////
bool I2C::IsInitOK( void )
{
   // make sure hardware and driver are installed correctly
   if ( RegBase::GetBaseAddress() == 0 )
      return( false );
      
   // initialize I2C register shadow
   I2CResetShadow();

   initOK = true;
   return( initOK );
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2C::I2CInitHWMode( long freq )
//  Purpose: Initialize I2C for hardware control of SCL and SDA
//  Input:   long freq - frequency (hz) to run SCL at
//  Output:  None
//  Return:  None
//  Note:    After calling I2CIsInitOK(), application should call this
//           function and check for return value is Success before starting
//           hardware communication.
/////////////////////////////////////////////////////////////////////////////
ErrorCode I2C::I2CInitHWMode( long freq )
{
   // initialization was successful?
   if ( initOK != true )
   {
      errNum = I2CERR_INIT;
      return Fail;
   }
   
   // initialize I2C register shadow
   I2CResetShadow();

	decFieldSCL = i2cShadow.scl = 1;   // must be 1 for hardware mode
	decFieldSDA = i2cShadow.sda = 1;   // must be 1 for hardware mode

	I2CSetFreq( freq );   // set frequency for hardware control

   // I2C is running hardware mode
   mode = I2CMode_HW;
   return Success;
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2C::I2CInitSWMode( long freq )
//  Purpose: Initialize I2C for software control of SCL and SDA
//  Input:   long freq - frequency (hz) to run SCL at
//  Output:  None
//  Return:  None
//  Note:    After calling I2CIsInitOK(), application should call this
//           function and check for return value is Success before starting
//           software communication.
/////////////////////////////////////////////////////////////////////////////
ErrorCode I2C::I2CInitSWMode( long freq )
{
   // initialization was successful?
   if ( initOK != true )
   {
      errNum = I2CERR_INIT;
      return Fail;
   }
   
   // initialize I2C register shadow
   I2CResetShadow();

	decFieldSCL = i2cShadow.scl = 1;
	decFieldSDA = i2cShadow.sda = 1;

	I2CSetFreq( 0 );        // set frequency to 0 for software control

   // need to calibrate in order to generate the correct clock cycle
   // the approach is to calculate how many dummy loop we need in order to
   // generate a cycle that is 2 * freq * 1000 Hz
   cycle  = 10000L;        // use a large number to start
   DWORD elapsed = 0L;
   while ( !elapsed )      // loop until delay is long enough for calculation
   {
      cycle *= 10;
      DWORD start = GetTickCount();
      for ( DWORD i = cycle; i > 0; i-- )
         ;
      elapsed = GetTickCount() - start;
   }
   if ( freq > 1 )
      cycle = cycle / elapsed * 1000L / freq / 2;

   // I2C is running software mode
   mode = I2CMode_SW;
   return Success;
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  void I2C::I2CSetFreq( long freq )
//  Purpose: Set frequency for SCL
//  Input:   long freq - frequency (hz) to run SCL at. (137.5khz to 2.0625Mhz)
//             PCI frequency 33Mhz: SCL = (412.50Khz to 33.81Khz)
//                           25Mhz: SCL = (312.50Khz to 25.61Khz)
//  Output:  None
//  Return:  None
//  Note:    For Gemini, frequency in hardware mode can be either 100KHz or
//           400KHz.
/////////////////////////////////////////////////////////////////////////////
void I2C::I2CSetFreq( long freq )
{
	// avoid division errors
	if( freq > 1 )
      if ( IsBt878Family() )
      {
         pGeminiI2CShadow->mode = 1;     // hardware mode
         pGeminiI2CShadow->rate = ( freq == 400000 ) ? 1 : 0;
         pGeminiI2CShadow->nos1b = pGeminiI2CShadow->nostop = 0;
      }
      else
      {
	      unsigned int i2cdiv;
         i2cdiv = (unsigned int) (PCI_FREQ / (64 * freq));
         if ( i2cdiv > I2CDIV_MAX )
            i2cdiv = I2CDIV_MAX;
         i2cShadow.div = i2cdiv;
      }
	else
		i2cShadow.div = 0;

	decFieldI2CDIV = i2cShadow.div;
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  int I2C::I2CReadDiv( void )
//  Purpose: Obtain value of programmable divider
//  Input:   None
//  Output:  None
//  Return:  Value of programmable divider
/////////////////////////////////////////////////////////////////////////////
int I2C::I2CReadDiv( void )
{
	return decFieldI2CDIV;
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2C::I2CHWRead( BYTE address, BYTE *value )
//  Purpose: Perform a hardware read from the I2C
//  Input:   int address - address to be read from
//  Output:  int *value  - retrieved value
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode I2C::I2CHWRead( BYTE address, BYTE *value )
{
   // check if correct mode is selected
   if ( mode != I2CMode_HW )
   {
      errNum = I2CERR_MODE;
      return Fail;
   }

   _i2c_reg i2cTemp = i2cShadow;   // obtain previous settings that didn't change

   // see above for reasons of doing all these.
   i2cTemp.addr_rw = address | I2C_READ;
   i2cTemp.byte1   = 0;
   i2cTemp.byte2   = 0;
   i2cTemp.w3b     = 0;    // Cheated for Gemini: w3b is same bit as w3bra

   decRegI2C = *(DWORD *)&i2cTemp;

   if ( I2CHWWaitUntilDone( TIMEOUT ) == Fail )
   {
      errNum = I2CERR_TIMEOUT;
      return Fail;
   }

   *value = decFieldI2CDB2;   // returned value is in 3rd byte

   if ( I2CHWReceivedACK() == true )
      return Success;
   else
   {
      errNum = I2CERR_NOACK;
      return Fail;
   }
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2C::I2CHWWrite2( BYTE address, BYTE value1 )
//  Purpose:  Perform a hardware write of two bytes to the I2C
//  Input:   int address - address to be written to
//           int value1  - value of 2nd byte to be written
//  Output:  None
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode I2C::I2CHWWrite2( BYTE address, BYTE value1 )
{
   // check if correct mode is selected
   if ( mode != I2CMode_HW )
   {
      errNum = I2CERR_MODE;
      return Fail;
   }

   _i2c_reg i2cTemp = i2cShadow;   // obtain previous settings that didn't change

   // see above for reasons of doing all these.
	i2cTemp.addr_rw = address | I2C_WRITE;
	i2cTemp.byte1   = value1;
	i2cTemp.byte2   = 0;
   i2cTemp.w3b     = 0;    // Cheated for Gemini: w3b is same bit as w3bra

   decRegI2C = *(DWORD *)&i2cTemp;

	if ( I2CHWWaitUntilDone( TIMEOUT ) == Fail )
   {
      errNum = I2CERR_TIMEOUT;
      return Fail;
   }

   if ( I2CHWReceivedACK() == true )
      return Success;
   else
   {
      errNum = I2CERR_NOACK;
      return Fail;
   }
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2C::I2CHWWrite3( BYTE address, BYTE value1, BYTE value2 )
//  Purpose: Perform a hardware write of three bytes to the I2C
//  Input:   int address - address to be written to
//           int value1  - value of 2nd byte to be written
//           int value2  - value of 3rd byte to be written
//  Output:  None
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode I2C::I2CHWWrite3( BYTE address, BYTE value1, BYTE value2 )
{
   // check if correct mode is selected
   if ( mode != I2CMode_HW )
   {
      errNum = I2CERR_MODE;
      return Fail;
   }

   _i2c_reg i2cTemp = i2cShadow;   // obtain previous settings that didn't change

   // see above for reasons of doing all these.
	i2cTemp.addr_rw = address | I2C_WRITE;
	i2cTemp.byte1   = value1;
	i2cTemp.byte2   = value2;
   i2cTemp.w3b     = 1;    // Cheated for Gemini: w3b is same bit as w3bra

   decRegI2C = *(DWORD *)&i2cTemp;

	if ( I2CHWWaitUntilDone( TIMEOUT ) == Fail )
   {
      errNum = I2CERR_TIMEOUT;
      return Fail;
   }

   if ( I2CHWReceivedACK() == true )
      return Success;
   else
   {
      errNum = I2CERR_NOACK;
      return Fail;
   }
}


/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2C::I2CSWStart( void )
//  Purpose: Generate START condition using software control
//  Input:   None
//  Output:  None
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode I2C::I2CSWStart( void )
{
   // check if correct mode is selected
   if ( mode != I2CMode_SW )
   {
      errNum = I2CERR_MODE;
      return Fail;
   }

   // SendStart - send an I2c start
   // i.e. SDA 1 -> 0 with SCL = 1

   if ( I2CSWSetSDA( Hi )  != Success ) { errNum = I2CERR_SDA; return Fail; }
   if ( I2CSWSetSCL( Hi )  != Success ) { errNum = I2CERR_SCL; return Fail; }
   if ( I2CSWSetSDA( Low ) != Success ) { errNum = I2CERR_SDA; return Fail; }
   if ( I2CSWSetSCL( Low ) != Success ) { errNum = I2CERR_SCL; return Fail; }
   return Success;
}                  

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2C::I2CSWStop( void )
//  Purpose: Generate STOP condition using software control
//  Input:   None
//  Output:  None
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode I2C::I2CSWStop( void )
{
   // check if correct mode is selected
   if ( mode != I2CMode_SW )
   {
      errNum = I2CERR_MODE;
      return Fail;
   }
   
   // SendStop - sends an I2C stop, releasing the bus.
   // i.e. SDA 0 -> 1 with SCL = 1

   if ( I2CSWSetSCL( Low ) != Success ) { errNum = I2CERR_SCL; return Fail; }
   if ( I2CSWSetSDA( Low ) != Success ) { errNum = I2CERR_SDA; return Fail; }
   if ( I2CSWSetSCL( Hi )  != Success ) { errNum = I2CERR_SCL; return Fail; }
   if ( I2CSWSetSDA( Hi )  != Success ) { errNum = I2CERR_SDA; return Fail; }
   return Success;
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2C::I2CSWRead( BYTE * value )
//  Purpose: Read a byte from the slave
//  Input:   None
//  Output:  BYTE * value - byte read from slave
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode I2C::I2CSWRead( BYTE * value )
{
   // check if correct mode is selected
   if ( mode != I2CMode_SW )
   {
      errNum = I2CERR_MODE;
      return Fail;
   }
   
   *value = 0x00;   

   // read 8 bits from I2c into Accumulator
   for( BYTE mask = 0x80; mask > 0; mask = (BYTE)( mask >> 1 ) )
   {
      if ( I2CSWSetSCL( Low ) != Success ) { errNum = I2CERR_SCL; return Fail; }
      if ( I2CSWSetSCL( Hi )  != Success ) { errNum = I2CERR_SCL; return Fail; }
      if ( I2CSWReadSDA() == TRUE )
         *value = (BYTE)( *value | mask ); // set the bit
   }
   return Success;
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2C::I2CSWWrite( BYTE value )
//  Purpose: Write a byte to the slave
//  Input:   BYTE value - byte to be written to slave
//  Output:  None
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode I2C::I2CSWWrite( BYTE value )
{
   // check if correct mode is selected
   if ( mode != I2CMode_SW )
   {
      errNum = I2CERR_MODE;
      return Fail;
   }

   // generate bit patterns by setting SCL and SDA lines   
   for ( BYTE mask = 0x80; mask > 0; mask = (BYTE)(mask >> 1) )
   {                                          
      if ( I2CSWSetSCL( Low ) != Success ) { errNum = I2CERR_SCL; return Fail; }

      // Send one data bit.
      if ( value & mask ) // Put data bit on pin.
      {
         if ( I2CSWSetSDA( Hi ) != Success ) { errNum = I2CERR_SDA; return Fail; }
      }
      else
      {
         if ( I2CSWSetSDA( Low ) != Success ) { errNum = I2CERR_SDA; return Fail; }
      }
          
      if ( I2CSWSetSCL( Hi ) != Success ) { errNum = I2CERR_SCL; return Fail; }
   }

   return I2CSWWaitForACK();
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2C::I2CSWSendACK( void )
//  Purpose: Send ACK to slave
//  Input:   None
//  Output:  None
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode I2C::I2CSWSendACK( void )
{
   // check if correct mode is selected
   if ( mode != I2CMode_SW )
   {
      errNum = I2CERR_MODE;
      return Fail;
   }
   
   // Generate ACK signal
   // i.e. SDA = 0 with SCL = 1

   if ( I2CSWSetSCL( Low ) != Success ) { errNum = I2CERR_SCL; return Fail; }
   if ( I2CSWSetSDA( Low ) != Success ) { errNum = I2CERR_SDA; return Fail; }
   if ( I2CSWSetSCL( Hi )  != Success ) { errNum = I2CERR_SCL; return Fail; }
   if ( I2CSWSetSCL( Low ) != Success ) { errNum = I2CERR_SCL; return Fail; }
   if ( I2CSWSetSDA( Hi )  != Success ) { errNum = I2CERR_SDA; return Fail; }
   return Success;
}                  

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2C::I2CSWSendNACK( void )
//  Purpose: Send NACK to slave
//  Input:   None
//  Output:  None
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode I2C::I2CSWSendNACK( void )
{
   // check if correct mode is selected
   if ( mode != I2CMode_SW )
   {
      errNum = I2CERR_MODE;
      return Fail;
   }
   
   // Generate NACK signal
   // i.e. SDA = 1 with SCL = 1

   if ( I2CSWSetSCL( Low ) != Success ) { errNum = I2CERR_SCL; return Fail; }
   if ( I2CSWSetSDA( Hi )  != Success ) { errNum = I2CERR_SDA; return Fail; }
   if ( I2CSWSetSCL( Hi )  != Success ) { errNum = I2CERR_SCL; return Fail; }
   if ( I2CSWSetSCL( Low ) != Success ) { errNum = I2CERR_SCL; return Fail; }
   if ( I2CSWSetSDA( Low ) != Success ) { errNum = I2CERR_SDA; return Fail; }
   return Success;
}                  

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2C::I2CSWSetSCL( Level scl )
//  Purpose: Set software SCL value
//  Input:   Level scl - Hi releases the SCL output; Low forces the SCL output low
//  Output:  None
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode I2C::I2CSWSetSCL( Level scl )
{
   // check if correct mode is selected
   if ( mode != I2CMode_SW )
   {
      errNum = I2CERR_MODE;
      return Fail;
   }
   
	i2cShadow.scl = scl;
   decRegI2C = *(DWORD *)&i2cShadow;

   // loop until SCL really changes or timeout
   DWORD maxWait = 500; // 500 mSec
   DWORD startTime = GetTickCount();

   while (1)
   {
      // has SCL changed yet?
      if ( I2CSWReadSCL() == scl )
         break;

      // timeout?
      if ( GetTickCount() - startTime > (DWORD)maxWait )
      {
         errNum = I2CERR_TIMEOUT;
         return Fail;
      }
   }

   I2CSWBitDelay();
   return Success;
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  int I2C::I2CSWReadSCL( void )
//  Purpose: Read software SCL value
//  Input:   None
//  Output:  None
//  Return:  SCL value
/////////////////////////////////////////////////////////////////////////////
int I2C::I2CSWReadSCL( void )
{
	return decFieldSCL;
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2C::I2CSWSetSDA( Level sda )
//  Purpose: Set software SDA value
//  Input:   Level sda - Hi releases the SDA output; Low forces the SDA output low
//  Output:  None
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode I2C::I2CSWSetSDA( Level sda )
{
   // check if correct mode is selected
   if ( mode != I2CMode_SW )
   {
      errNum = I2CERR_MODE;
      return Fail;
   }

	i2cShadow.sda = sda;
   decRegI2C = *(DWORD *)&i2cShadow;

   I2CSWBitDelay();
   return Success;
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  int I2C::I2CSWReadSDA( void )
//  Purpose: Read software SDA value
//  Input:   None
//  Output:  None
//  Return:  SDA value
/////////////////////////////////////////////////////////////////////////////
int I2C::I2CSWReadSDA( void )
{
	return decFieldSDA;
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2C::I2CSetSync( State sync )
//  Purpose: Set I2C sync value
//  Input:   sync: On  - allow slave to insert wait states
//                 Off - slave cannot insert wait states
//  Output:  None
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode I2C::I2CSetSync( State sync )
{
   // check if correct mode is selected
   if ( ( mode != I2CMode_HW ) && ( mode != I2CMode_SW ) )
   {
      errNum = I2CERR_MODE;
      return Fail;
   }

	i2cShadow.sync = sync;
   decRegI2C = *(DWORD *)&i2cShadow;
   return Success;
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  int I2C::I2CReadSync( void )
//  Purpose: Read I2C sync value
//  Input:   None
//  Output:  None
//  Return:  Sync value
/////////////////////////////////////////////////////////////////////////////
int I2C::I2CReadSync( void )
{
	return decFieldSYNC;
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  int I2C::I2CGetLastError( void )
//  Purpose: Obtain last I2C error number
//  Input:   None
//  Output:  None
//  Return:  Last I2C error number
/////////////////////////////////////////////////////////////////////////////
int I2C::I2CGetLastError( void )
{
   return errNum;
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2C::I2CHWMultiRead( BYTE address, BYTE subAddr, BYTE * pBlock, int len )
//  Purpose: Perform a hardware read of multi bytes from I2C without stopping
//  Input:   int address   - chip address to be read
//           int subAddr   - sub-address to be read from
//           BYTE * pBlock - pointer to buffer to receive data
//           int len       - number of bytes to be read
//  Input:   None
//  Output:  None
//  Return:  Success, Fail or Err_WrongHardware
/////////////////////////////////////////////////////////////////////////////
ErrorCode I2C::I2CHWMultiRead( BYTE address, BYTE subAddr, BYTE * pBlock, int len )
{
   // check if hardware supports multi-byte write
   if ( !IsBt878Family() )
      return ( Err_WrongHardware );

   // check if correct mode is selected
   if ( mode != I2CMode_HW )
   {
      errNum = I2CERR_MODE;
      return Fail;
   }

   // check if reading at least 2 bytes
   if ( len < 2 )
   {
      errNum = I2CERR_PARAMETER;
      return Fail;
   }

   // first send the sub-addr which will be read from later
   if ( I2CHWWrite2( address, subAddr ) != Success )
      return Fail;

   _Gemini_i2c_reg i2cTemp = *pGeminiI2CShadow;   // obtain previous settings that didn't change

   // continuously read 1 byte at a time until 1 byte left to be read
   bool lastByte;
   for ( int i = 0; i < len; i++ )
   {
      lastByte = ( i == len - 1 );
      i2cTemp.addr_rw = address | I2C_READ;
	   i2cTemp.byte1   = 0;
   	i2cTemp.byte2   = 0;
      i2cTemp.nos1b   = ( !i || lastByte ) ? 0 : 1;  // first and last byte, set to 0
      i2cTemp.w3bra   = lastByte ? 0 : 1;  // last byte, set to 0
      i2cTemp.nostop  = lastByte ? 0 : 1;  // last byte, set to 0

#pragma message("Delete me...")
char s[80];
wsprintf(s, "I2CHWBlockRead setup: i2cTemp = 0x%lX\n", i2cTemp);
OutputDebugString(s);

      decRegI2C = *(DWORD *)&i2cTemp;

      if ( I2CHWWaitUntilDone( TIMEOUT ) == Fail )
      {
         errNum = I2CERR_TIMEOUT;
         return Fail;
      }

      *( pBlock + i ) = decFieldI2CDB2;   // returned value is in 3rd byte
wsprintf(s, "I2CHWBlockRead byte #%d --> i2cTemp = 0x%lX  READ = 0x%lX\n", i, i2cTemp, *(pBlock+i) );
OutputDebugString(s);
   }

   return Success;
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2C::I2CHWMultiWrite( BYTE address, BYTE subAddr, BYTE * pBlock, int len )
//  Purpose: Perform a hardware write of multi bytes to the I2C without stopping
//  Input:   int address   - chip address to be written
//           int subAddr   - sub-address to be written to
//           BYTE * pBlock - pointer to data to be sent
//           int len       - number of bytes to be sent
//  Input:   None
//  Output:  None
//  Return:  Success, Fail or Err_WrongHardware
/////////////////////////////////////////////////////////////////////////////
ErrorCode I2C::I2CHWMultiWrite( BYTE address, BYTE subAddr, BYTE * pBlock, int len )
{
   // check if hardware supports multi-byte write
   if ( !IsBt878Family() )
      return ( Err_WrongHardware );

   // check if correct mode is selected
   if ( mode != I2CMode_HW )
   {
      errNum = I2CERR_MODE;
      return Fail;
   }

   // check if writing at least 2 bytes
   if ( len < 2 )
   {
      errNum = I2CERR_PARAMETER;
      return Fail;
   }

   _Gemini_i2c_reg i2cTemp = *pGeminiI2CShadow;   // obtain previous settings that didn't change

   // see above for reasons of doing all these.
	i2cTemp.addr_rw = address | I2C_WRITE;
	i2cTemp.byte1   = subAddr;
	i2cTemp.byte2   = 0;
   i2cTemp.nostop  = 1;
char s[80];
wsprintf(s, "I2CHWBlockWrite setup: i2cTemp = 0x%lX\n", i2cTemp);
OutputDebugString(s);

   decRegI2C = *(DWORD *)&i2cTemp;

	if ( I2CHWWaitUntilDone( TIMEOUT ) == Fail )
   {
      errNum = I2CERR_TIMEOUT;
      return Fail;
   }

   if ( I2CHWReceivedACK() == false )
   {
      errNum = I2CERR_NOACK;
      return Fail;
   }

   // write 1 byte at a time to the device until 1 byte left
   for ( int i = 0; i < len; i++ )
   {
   	i2cTemp.addr_rw = *( pBlock + i );
	   i2cTemp.byte1   = 0;
   	i2cTemp.byte2   = 0;
      i2cTemp.nos1b   = 1;
      i2cTemp.nostop  = ( i == len - 1 ) ? 0 : 1;  // last byte, set to 0

wsprintf(s, "I2CHWBlockWrite byte #%d: data byte = %X i2cTemp = 0x%08lX\n", i, *( pBlock + i ), i2cTemp);
OutputDebugString(s);

      decRegI2C = *(DWORD *)&i2cTemp;

   	if ( I2CHWWaitUntilDone( TIMEOUT ) == Fail )
      {
OutputDebugString("I2C timeout - not done\n");
         errNum = I2CERR_TIMEOUT;
         return Fail;
      }
OutputDebugString("I2C done\n");

      if ( I2CHWReceivedACK() == false )
      {
OutputDebugString("I2C timeout - no ACK received\n");
         errNum = I2CERR_NOACK;
         return Fail;
      }
OutputDebugString("I2C - ACK received\n");
   }
   return ( Success );
}

//============================================================================
// Following functions are used internally
//============================================================================

/////////////////////////////////////////////////////////////////////////////
//  Method:  void I2C::I2CResetShadow( void )
//  Purpose: Reset register shadow
//  Input:   int maxWait - maximum waiting time in milliseconds
//  Output:  None
//  Return:  Success if done; Fail if timeout
/////////////////////////////////////////////////////////////////////////////
void I2C::I2CResetShadow( void )
{
   // initialize I2C register shadow
	i2cShadow.addr_rw = 0;
	i2cShadow.byte1   = 0;
	i2cShadow.byte2   = 0;
	i2cShadow.div     = 0;
	i2cShadow.sync    = 0;
	i2cShadow.w3b     = 0;
	i2cShadow.scl     = 0;
	i2cShadow.sda     = 0;
}
   
/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2C::I2CHWWaitUntilDone( int maxWait )
//  Purpose: Wait for hardware I2C to finish
//  Input:   int maxWait - maximum waiting time in milliseconds
//  Output:  None
//  Return:  Success if done; Fail if timeout
/////////////////////////////////////////////////////////////////////////////
ErrorCode I2C::I2CHWWaitUntilDone( int maxWait )
{
   DWORD startTime = GetTickCount();

   // loop until either I2CDONE is set or timeout
   while (1)
   {
      if ( I2CHWIsDone() == true )
         return Success;

      // timeout?
      if ( GetTickCount() - startTime > (DWORD)maxWait )
      {
         errNum = I2CERR_TIMEOUT;
         return Fail;
      }
   }
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  bool I2C::I2CHWIsDone( void )
//  Purpose: Determine if I2C has finished a read or write operation by
//           checking the I2CDONE bit in the interrupt status register
//  Input:   None
//  Output:  None
//  Return:  true if done; else false
/////////////////////////////////////////////////////////////////////////////
bool I2C::I2CHWIsDone( void )
{
   if ( decFieldI2CDONE != 0 )
   {
      // Need to clear the bit when it is set; don't want to alter any other bits
      // Writing a 1 to the bit clears it.
      decFieldI2CDONE = 1;
      return true;
   }
   else
      return false;
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  bool I2C::I2CHWReceivedACK( void )
//  Purpose: Determine if ACK is receieved
//  Input:   None
//  Output:  None
//  Return:  true if ACK received; else false
/////////////////////////////////////////////////////////////////////////////
bool I2C::I2CHWReceivedACK( void )
{
   return ( ( decFieldRACK != 0 ) ? true : false );
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  void I2C::I2CSWBitDelay( void )
//  Purpose: Insures minimum high and low clock times on I2C bus.
//  Input:   None
//  Output:  None
//  Return:  None
//  Note:    This routine must be tuned for the desired frequency.
/////////////////////////////////////////////////////////////////////////////
void I2C::I2CSWBitDelay( void )
{
  //  unsigned int n;
  for ( DWORD i = cycle; i > 0; i-- )
    ;
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2C::I2CSWWaitForACK( void )
//  Purpose: Determine if ACK is receieved in software mode
//  Input:   None
//  Output:  None
//  Return:  Success if ACK received; Fail if timeout
/////////////////////////////////////////////////////////////////////////////
ErrorCode I2C::I2CSWWaitForACK( void )
{
   if ( I2CSWSetSCL( Low ) != Success ) { errNum = I2CERR_SCL; return Fail; }
   if ( I2CSWSetSDA( Hi )  != Success ) { errNum = I2CERR_SDA; return Fail; }

   // loop until either ACK or timeout
   DWORD maxWait = 500; // 500 mSec
   DWORD startTime = GetTickCount();

   while (1)
   {
      // SDA pin == 0 means the slave ACK'd
      if ( I2CSWReadSDA() == 0 )
      {
         if ( I2CSWSetSCL( Hi )  != Success ) { errNum = I2CERR_SCL; return Fail; }
         if ( I2CSWSetSCL( Low ) != Success ) { errNum = I2CERR_SCL; return Fail; }
         return Success;
      }

      // timeout?
      if ( GetTickCount() - startTime > (DWORD)maxWait )
      {
         if ( I2CSWStop() != Success ) return Fail;
         errNum = I2CERR_TIMEOUT;
         return Fail;
      }
   } // while
}



