// $Header: G:/SwDev/W95/Video/pisces/dvlpmnt/Bt848_32/rcs/i2c.h 2.5 1997/09/30 18:37:09 rudyl Released $
// $Log: i2c.h $
// Revision 2.5  1997/09/30 18:37:09  rudyl
// Renamed BlockRead() & BlockWrite() to MultiRead() & MultiWrite().
// Revision 2.4  1997/08/23 01:43:03  rudyl
// Added sub-addr to I2CHWBlockRead & Write.
// Revision 2.3  1997/08/14 18:25:21  rudyl
// Graft from 2.2.1.2. Added Gemini support.
// Revision 2.2.1.2  1997/07/30 01:02:32  rudyl
// Gemini: Added support and I2CHWBlockRead() and I2CHWBlockWrite().
// Revision 2.2.1.1  1997/07/17 19:34:21  rudyl
// Add support for Gemini.
// Revision 2.2  1996/06/25 21:57:43  rudyl
// Added I2CGetLastError() and changed return types for I2CInitHWMode() and I2CInitSWMode().
// Revision 2.1  1996/06/10 20:12:04  ssm
// MarkBeta1AsRev2
// Revision 1.9  1996/05/30 20:21:08  rudyl
// Cleanup the code and added comments for software control of I2C.
// Added comments for bug with register class and to do list.
// Revision 1.8  1996/05/30 19:08:25  rudyl
// Removed vxd related calls from Bt848api.h to vxdcall.h.
// Moved define of __BT848CLASS & __BT848FUNC to def__.h so both Bt848api.h and vxdCall.h can include it.
// Also added software control of I2C; seems working properly but need to cleanup the codes and add comments.
// Revision 1.7  1996/05/24 18:38:48  rudyl
// Added software control of SCL & SDA. Code compiled OK. No testing done.
// Revision 1.5  1996/05/23 19:20:02  rudyl
// Added functions skeleton (no code inside) for software control of I2C.
// Revision 1.3  1996/04/29 17:51:33  rudyl
// A few parameter type changes in I2C (thus in API) to make it consistent with GPIO.
// Revision 1.2  1996/04/28 01:25:22  rudyl
// ver 1.00 - both I2C and GPIO are now objects of register class.
// Created a set of API wrapper functions to invoke member functions of the classes.
// Revision 1.1  1996/04/26 23:52:52  rudyl
// Initial revision
// Revision 1.6  1996/04/09 21:36:50  rudyl
// Added copyright info.
// Revision 1.5  1996/03/20 02:16:25  rudyl
// Added  __I2CFUNC in declaration.
// Revision 1.4  1996/03/12 18:54:52  rudyl
// Removed extern C. Decided to use .cpp for all source files.
// Revision 1.3  1996/03/12 00:36:02  rudyl
// Return ErrorCode type for some functions.
// Revision 1.2  1996/02/28 01:36:18  rudyl
// All functions checked OK. Added I2CReadSync().


/////////////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 1996 Brooktree Corporation. All Rights Reserved.
//
//  Module:
//
//    i2c.h
//
//  Abstract:
//
//    Header file for I2C register DLL
//
/////////////////////////////////////////////////////////////////////////////

#ifndef __I2C_H
#define __I2C_H

#ifdef _BUILD_I2CDLL
  #define __I2CCLASS __export
  #define __I2CFUNC __export
#else
  #define __I2CCLASS __import
  #ifdef __WIN32__
    #define __I2CFUNC __import
  #else
    #define __I2CFUNC
  #endif
#endif

#ifndef __DEFS_H
#include <classlib/defs.h>
#endif

#include "regField.h"
#include "viddefs.h"
#include "retcode.h"
#include "i2cerr.h"


/////////////////////////////////////////////////////////////////////////////
// CLASS I2C
//
// Description:
//    This class encapsulates the register fields in the I2C register of the
//    Bt848. A complete set of functions are developed to manipulate all the
//    register fields in the I2C for the Bt848.
//
/////////////////////////////////////////////////////////////////////////////

class __I2CCLASS I2C
{
private:
   // define which mode the I2C is selected
   enum I2CMode { I2CMode_None, I2CMode_HW, I2CMode_SW };
   
   bool    initOK;      // initialization is successful?
   DWORD   cycle;       // software control of frequency
   int     errNum;      // error number
   I2CMode mode;        // which mode the I2C is running in

protected:
   RegisterDW decRegINT_STAT;
   RegField decFieldI2CDONE;
   RegField decFieldRACK;
   RegisterDW decRegI2C;
   RegField decFieldI2CDB0;
   RegField decFieldI2CDB1;
   RegField decFieldI2CDB2;
   RegField decFieldI2CDIV;
   RegField decFieldSYNC;
   RegField decFieldW3B;
   RegField decFieldSCL;
   RegField decFieldSDA;

   // new for Gemini
   RegField decFieldI2CMODE;
   RegField decFieldI2CRATE;
   RegField decFieldI2CNOSTOP;
   RegField decFieldI2CNOS1B;
   RegField decFieldI2CW3BRA;

public:
   // constructor and destructor
	I2C( void );
	~I2C();

   // member functions
   bool      IsInitOK( void );
   ErrorCode I2CInitHWMode( long freq );
   ErrorCode I2CInitSWMode( long freq );
   void      I2CSetFreq( long freq );
   int       I2CReadDiv( void );
   ErrorCode I2CHWRead( BYTE address, BYTE *value );
   ErrorCode I2CHWWrite2( BYTE address, BYTE value1 );
   ErrorCode I2CHWWrite3( BYTE address, BYTE value1, BYTE value2 );
   ErrorCode I2CSWStart( void );
   ErrorCode I2CSWStop( void );
   ErrorCode I2CSWRead( BYTE * value );
   ErrorCode I2CSWWrite( BYTE value );
   ErrorCode I2CSWSendACK( void );
   ErrorCode I2CSWSendNACK( void );
   ErrorCode I2CSWSetSCL( Level );
   int       I2CSWReadSCL( void );
   ErrorCode I2CSWSetSDA( Level );
   int       I2CSWReadSDA( void );
   ErrorCode I2CSetSync( State );
   int       I2CReadSync( void );
   int       I2CGetLastError( void );

   // new for Gemini
   ErrorCode I2CHWMultiWrite( BYTE address, BYTE subAddr, BYTE * pBlock, int len );
   ErrorCode I2CHWMultiRead( BYTE address, BYTE subAddr, BYTE * pBlock, int len );

private:
   void      I2CResetShadow( void );      // reset register shadow
   ErrorCode I2CHWWaitUntilDone( int );   // wait until I2C completes operation
   bool      I2CHWIsDone( void );         // check interrupt bit for operation done
   bool      I2CHWReceivedACK( void );    // check interrupt bit for received ACK
   void      I2CSWBitDelay( void );       // insert delay to simulate frequency
   ErrorCode I2CSWWaitForACK( void );     // wait for ACK from receiver using software
};


#endif // __I2C_H

