/* File: Capchip.h
 * Purpose: Contains definition of the CaptueChip class
 * Author: sasha
 * $Log:
 */

#ifndef __CAPCHIP_H
#define __CAPCHIP_H

#ifndef __DEFS_H
#include <classlib/defs.h>
#endif

#if !defined __vtoolscp_h && !defined TESTING
#include <vtoolscp.h>
#endif

#undef FAR

#ifndef __WINDOWS_H
#include <windows.h>
#endif


#ifndef __MYTYPES_H
#include "mytypes.h"
#endif

#ifndef __VXDFUNC_H
#include "vxdfunc.h"
#endif

#ifndef __VIDDEFS_H
#include "viddefs.h"
#endif

#ifndef __COLFRMAT_H
#include "colfrmat.h"
#endif

#ifndef __STRMINFO_H
#include "strminfo.h"
#endif


/* Class: CapChip
 * Purpose: Provides consistent interface to the underlying capture HW
 * Attributes:
 * Operations:
 */
class CaptureChip
{
   private:
      bool Inited_;

   public:
      void Init() { Inited_ = true; }
      bool IsInited() { return Inited_; }

      virtual bool DoesSupportBothFields() { return false; }

      // decoder 'set' group
      virtual void SetBrightness( DWORD value ) = 0;
      virtual void SetSaturation( DWORD value ) = 0;
      virtual void SetConnector ( DWORD value ) = 0;
      virtual void SetContrast  ( DWORD value ) = 0;
      virtual void SetHue       ( DWORD value ) = 0;
      virtual void SetSVideo    ( DWORD value ) = 0;
      virtual void SetFormat    ( DWORD value ) = 0;

      // scaler group
      virtual ErrorCode SetAnalogWindow( TRect &r, StreamInfo & which )  = 0;
      virtual ErrorCode SetDigitalWindow( TRect &r, StreamInfo & which ) = 0;

      // color space converter group
      virtual void SetPixelFormat( ColFmt aFormat, StreamInfo & which ) = 0;

      virtual DWORD Start( StreamInfo & ) = 0;
      virtual void Stop ( StreamInfo & ) = 0;

      virtual void SetFrameRate( DWORD rate = 30 ) = 0;

      // resource allocation group
      virtual int  GetNumberOfStreams() = 0;
      virtual void GetStreamInfo( StreamInfo  &aStream ) = 0;
      virtual void FreeStream( StreamInfo &StrInf )     = 0;
      virtual ErrorCode AllocateStream( StreamInfo &aStream ) = 0;


};

#endif