 /* File: Pisces.cpp
 * Purpose: Contains implementation of the BtPisces class
 * Author: sasha
 * $Log: PISCES.CPP $
 * Revision 2.67  1998/06/01 18:05:02  mdimeo
 * added rcs ident string
 * Revision 2.66  1998/05/22 21:35:05  dtharp
 * Reduced R3 queue sizes to 10. Queues now grow dynamically
 * if needed. Also upgraded buffer instrumentation.
 * Revision 2.65  1998/05/21 23:54:34  dtharp
 * Increased queue sizes to 100. Fixes capture stall problem
 * Revision 2.64  1998/05/19 17:42:41  dtharp
 * Added comment for potential bug.
 * Revision 2.63  1998/05/14 23:38:01  dtharp
 * Graft from DT_VBI branch: VBI now functional.
 * Revision 2.62.1.1  1998/05/14 23:25:33  dtharp
 * Modifications required to bring VBI back online. Also fixed Starter 
 * SYNC problem.
 * Revision 2.62  1998/05/12 20:45:32  dtharp
 * Added hack to disable sync filip if both fields are starting.
 * Revision 2.61.1.2  1998/05/12 00:58:10  dtharp
 * First checkpoint: "graceful degredation" mode QS capture from overlay.
 * Revision 2.61  1998/04/22 16:04:07  dtharp
 * Protected riscProg Pointer in interupt. Added DeallocVxdFrames()
 * Revision 2.60  1998/04/21 00:54:03  dtharp
 * No functional changes--Changed debug level on numerous
 * debug prints
 * Revision 2.59  1998/04/17 18:43:08  dtharp
 * Interrupt handler no longer uses Register class.
 * Revision 2.58  1998/04/16 23:45:09  dtharp
 * Added protection for pointer access in ProcessAddBuffer.
 * Revision 2.57  1998/04/08 19:14:04  dtharp
 * Fixed Quartsight Overlay fps < 30 bug.
 * Revision 2.56  1998/04/07 16:24:41  dtharp
 * Interlaced capture, preview, and overlay now work properly.
 * Revision 2.55  1998/04/06 20:27:06  dtharp
 * NumVxdFrameBuffers is now configurable via registry key 
 * '..Video\NumVxdFrameBuffers'.
 * Revision 2.54  1998/04/06 19:42:13  dtharp
 * Vxd Frame Buffers are allocated on the fly instead of at init time.
 * Revision 2.53  1998/04/06 16:15:21  dtharp
 * Removed dead declaration
 * Revision 2.52  1998/04/05 20:28:33  dtharp
 * RestartFrameServer() no longer takes a parameter.
 * Revision 2.51  1998/04/05 20:03:41  dtharp
 * Removed unused flag Update_ and unnecessary counter nCreated_.
 * Revision 2.50  1998/04/03 18:34:01  dtharp
 * General Clean-up
 * Revision 2.49  1998/04/02 21:49:17  dtharp
 * Merge DT_VIDEOHDR branch. Implements a new buffer 
 * management scheme.
 * Revision 2.48.1.8  1998/04/02 21:25:19  dtharp
 * Cleanup before merge
 * Revision 2.48.1.7  1998/04/02 17:37:04  dtharp
 * Fixed overlay mode
 * Revision 2.48.1.6  1998/04/02 16:40:27  dtharp
 * Fixed yuv9 problem
 * Revision 2.48.1.5  1998/04/01 21:19:27  dtharp
 * Incremental checkin
 * Revision 2.48.1.3  1998/03/27 16:33:27  dtharp
 * Commit to new prog array and no skipper array.
 * Revision 2.48.1.2  1998/03/27 15:30:11  dtharp
 * Incremental check-in
 * Revision 2.48.1.1  1998/03/18 19:34:12  dtharp
 * Branch for passing videoheaders around
 * Revision 2.48  1998/02/27 18:18:02  dtharp
 * Capture Controler constructor now takes devID as a param. Added
 * check for 878 series before setting Triton compatibility bit in IMASK.
 * Revision 2.47  1998/02/24 21:01:43  rudyl
 * Merged 2.45.2.2 & 2.46.
 * Revision 2.45.2.2  1998/02/24 20:57:50  rudyl
 * Added SetVScaleMode() and GetVScaleMode().
 * Revision 2.45.2.1  1997/11/26 00:15:46  rudyl
 * Duplicate revision
 * Revision 2.46  1998/02/24 17:56:51  dtharp
 * Merge VVL AEC Code
 * Revision 2.45.1.3  1998/02/13 00:39:51  dtharp
 * Re-worked VPAEC Init() and iterate() call conditions
 * Revision 2.45.1.2  1998/02/10 03:32:55  dtharp
 * Added VVL AEC Initialization
 * Revision 2.45  1997/11/26 00:15:46  peter
 * Graft reg key move to tip.
 * Revision 2.44.1.2  1997/11/26 00:14:40  peter
 * Removed commented-out line.
 * Revision 2.44.1.1  1997/11/25 17:32:38  peter
 * Revision 2.44  1997/09/16 16:40:35  dtharp
 * Merged AVI processing branch.
 * Revision 2.43  1997/09/04 22:44:43  dtharp
 * Merged VSYNC Callback changes
 * Revision 2.42.2.1  1997/08/21 01:40:01  dtharp
 * Branch for vsync callback
 * Revision 2.42  1997/08/14 21:55:28  dtharp
 * Tentative fix for Bug ID 726: avi player crash after capture 640x480.
 * NEEDS REGRESSION TEST.
 * Revision 2.41  1997/08/08 11:17:59  dtharp
 * Fix for bug 714: NotifyArg now passed by reference.
 * Revision 2.40  1997/07/25 23:54:39  dtharp
 * Bug id 671.  Fixed Netmeeting freeze. Uncommented block at line 1440.
 * Revision 2.39  1997/07/21 23:18:42  dtharp
 * Fix for Bug ID 675: Suspend/Resume IRQ problems.
 * Revision 2.38  1997/06/27 23:05:29  dtharp
 * Kludge for Skipper program not having sync regulation for variable
 * frame rate.
 * Revision 2.37  1997/06/27 15:54:52  dtharp
 * New sentinel based buffer management scheme.  Also passes skipcount
 * down to ring 3 (instead of using the pointer method).
 * Revision 2.36.1.1  1997/06/10 18:00:17  dtharp
 * Branch for new Buffer mgmnt scheme.
 * Revision 2.36  1997/05/30 21:43:52  dtharp
 * Fix for various VBI/modem related bugs. Initialized InterruptToIdx_
 * array to -1 in AssignIntNumbers().  0 was being returned when a
 * non-interrupting program position was queried. ) happens to be
 * the Array index of the first VBI program: result was that the first
 * VBI program's Notify would be called any time a non-interrupting
 * program was quiried in the ISR.
 * Revision 2.35  1997/05/23 00:50:01  dtharp
 * Bug ID 591:  Protected against skipping (chaining) of a program
 * that is not currently chained.
 * Revision 2.34  1997/05/22 15:52:55  dtharp
 * Initialized GPIO_CB_ and I2C_CB_ to null .
 * Revision 2.33  1997/05/16 16:21:14  dtharp
 * Bug Id 562. Disabled risc engine while unskipping riscprogram. This
 * fix probably fixes numerous other primary CC problems.
 * Revision 2.32  1997/05/14 17:46:04  dtharp
 * Cast some bools to avoid compiler warnings
 * Revision 2.31  1997/05/06 17:31:34  dtharp
 * Moved GPIOINIT registry key to:
 * HKEY_LOCAL_MACHINE\SOFTWARE\Brooktree\Bt848\GpioInit
 * Revision 2.30  1997/05/02 17:26:43  dtharp
 * Added support for interrupt count instrumentation.
 * Revision 2.29  1997/04/28 20:54:29  dtharp
 * Merged CC Primary Surface branch from 2.24.2.3 Bug ID 475
 * Revision 2.28  1997/04/26 00:39:56  trantq
 * - Fix function declaration, missing variable.
 * Revision 2.27  1997/04/25 22:13:56  dtharp
 * Restored GetSVideo and SetSVideo code from rev 2.19. This
 * code got blasted during the WDM back port.
 * Revision 2.26  1997/04/25 17:09:04  dtharp
 * Merged DT_GPIO_INIT branch (2.24.1.1)
 * Revision 2.25  1997/04/25 16:58:45  dtharp
 * Merged DT_GP_INT work (modem fix- 2.23.2.1)
 * Revision 2.24  1997/04/07 21:36:40  dtharp
 * Grafted "new vpres" branch from 2.23.2.1. Bug Id 436.
 * Revision 2.23.2.1  1997/04/04 23:27:19  dtharp
 * Bug Id#436  Added initialization of bit 7 of register at offset 0x78. This enables
 * new vpres circuitry.
 * Revision 2.23  1997/04/03 17:14:22  dtharp
 * Added GPIO and I2C callback calls to ISR. They were inadvertantly
 * left out during the WDM transplant.
 * Revision 2.22  1997/03/26 00:24:48  dtharp
 * Removed debugBuffer; added debug print.
 * Revision 2.21  1997/03/21 21:59:45  dtharp
 * Fix for out of disk space blue screen crash.
 * Revision 2.20  1997/03/20 22:06:41  dtharp
 * Grafted WDM Buffer managment branch.
 * Revision 2.19.1.6  1997/03/20 22:05:58  dtharp
 * Added band-aid to pSkipCount.
 * Revision 2.19.1.5  1997/03/20 18:11:38  dtharp
 * PAL Mode Implemented.
 * Revision 2.19.1.4  1997/03/18 16:13:29  dtharp
 * General cleanup.
 * Revision 2.19.1.3  1997/03/17 16:41:25  dtharp
 * Works @ 30fps NTSC.
 * Revision 2.19.1.2  1997/03/12 18:11:58  dtharp
 * Single field capture kinda works.
 * Revision 2.19  1997/02/19 21:17:04  dtharp
 * Grafted DT_PAL_CAPTURE_RATE branch (bug id 384)
 * Revision 2.18.2.1  1997/02/10 21:47:12  dtharp
 * Branch for bug #384
 * Revision 2.18  1997/01/17 00:56:10  rudyl
 * Graft from 2.17.1.2: Added codes in SetSVideo() and GetSVideo() to call decoder.
 * Revision 2.17.1.2  1997/01/16 02:37:41  rudyl
 * Added codes in SetSVideo() and GetSVideo() to call decoder.
 * Revision 2.17.1.1  1997/01/16 02:14:51  rudyl
 * Add codes in SetSVideo() and GetSVideo().
 * Revision 2.17  1997/01/16 01:04:01  dtharp
 * Merged DT_FRAME_DROP branch (2.15.2)
 * Revision 2.16  1997/01/09 23:51:53  ssm
 * graft from revision 2.15.1.2, fix clientY < 0 problem
 * Revision 2.15.1.2  1996/12/25 03:14:37  sasha
 * Call RISCProgram::SetAboutToClip( true ) if clip data exists. This prevents
 * creation of risc program when it will be destroyed by the clipping.
 * Revision 2.15.1.1  1996/12/25 02:50:55  sasha
 * Branch for the above the screen fix
 * Revision 2.15  1996/12/06 23:29:42  dtharp
 * Added 'DidWe = On'  in Interrupt for GPIO and I2C callbacks per
 * Alex's instructions.
 * Revision 2.14  1996/11/07 21:38:37  dtharp
 * Added initialization of new static vars VBIField::VBI_PacketSize
 * and VBIField::VBI_HDelay.
 * Revision 2.13  1996/10/25 00:37:15  sasha
 * Finished off the ring 0 I2C and GPIO callbacks ( enable and disable
 * respective interrupts ).
 * Removed SetRate() member function ( not needed ).
 * Revision 2.12  1996/10/11 10:25:06  sasha
 * For more accurate frame counting: enable VSYNC interrupts and count
 * them. Calculate number of dropped frames based on given frame rate,
 * number of VSYNCs and RISC interrupts. Changed how program is updated
 * when processing RISC interrupt. Needs good testing.
 * Revision 2.11  1996/10/02 04:49:06  sasha
 * Added code to invoke ring0 GPIO and I2C callbacks. Moved event
 * callbacks to the callback.cpp
 * Revision 2.10  1996/09/29 23:26:23  sasha
 * To support 32 bit interrupt notification: added code to the ISR.
 * Revision 2.9  1996/09/26 23:58:49  sasha
 * To resolve bug ID 254.
 * Added parameter to the CreateProgram() member of RISCEng to indi-
 * cate that the program to be created should insert resynchs. Removed
 * SetResync( false ) for the sync programs.
 *
 * Unrelated issues: Removed WaitingFault global. Do not disable interrupts
 * in ProcessAddBuffer() as this is done ( and better ) in AddBuffer() of
 * device.cpp. Update skipped counter differently.
 * Revision 2.8  1996/09/23 00:46:45  sasha
 * Added static dwPlanarAdjust_ member. Use it in the CreateProgram().
 * Added Update_ memebr to prevent buffer update when starting full
 * size buffer.
 * Revision 2.7  1996/09/20 02:53:58  sasha
 * Added Fault_.SetParent( NULL ) to prevent page faults on unload.
 * Revision 2.6  1996/07/20 03:51:50  rudyl
 * fixed dos box freezing video problem
 * Revision 2.5  1996/07/15 19:09:47  sasha
 * Added Continue() to the Start() to undo what Pause() did in case of
 * PABORT ( after quickres ).
 * Revision 2.4  1996/07/14 03:20:15  sasha
 * Added Pause() when PABORT is detected
 * Revision 2.3  1996/07/02 21:04:24  sasha
 * Added test to ProcessAddBuffer() to test if the stream was stopped.
 * Revision 2.2  1996/06/14 23:59:00  sasha
 * Used explicit cast to get rid of warnings. Enhanced Pause() and Continue()
 * to use Paused_ flag.
 * Revision 2.1  1996/06/10 20:12:15  ssm
 * MarkBeta1AsRev2
 * Revision 1.28  1996/05/17 21:15:58  sasha
 * Call decoder to set video format
 * Revision 1.27  1996/05/02 21:20:29  sasha
 * Moved constructor here. Do casts in AllocateStream to avoid compile errors.
 * Added couple of function headers
 * Revision 1.26  1996/04/20 03:41:44  sasha
 * Changed the way buffers are added to a running program.
 * Changed how buffers are handled in the ISR. Added some debugging
 * code as well
 * Revision 1.25  1996/04/17 21:32:18  sasha
 * Added SetRate() function. Subject to change
 * Revision 1.24  1996/04/17 19:20:48  sasha
 * Removed write to the GPDATA in the Interrupt() function.
 * Revision 1.23  1996/04/11 23:03:56  sasha
 * Added #ifdef DEBUG around calls to sprintf
 * Revision 1.22  1996/04/11 18:06:04  sasha
 * Set triton fix bit on. Moved setting of IMASK above the Start()
 * Revision 1.21  1996/04/01 20:53:36  sasha
 * Moved dtor here. Added decoder functions. Plus many more
 * Revision 1.19  1996/02/19 22:47:45  sasha
 * Moved disable() lower in the ProcessAddBuffer()
 * Revision 1.18  1996/02/18 23:59:14  sasha
 * Added 2 VBI streams to the PresentStreams. Put VBI window rectangles here
 * Moved decoder functions to the end of file. Changed ProcessSyncPrograms
 * Moved ProcessSyncPrograms() before LinkThePrograms() in the
 * ProcessPresentPrograms()
 * Revision 1.17  1996/02/14 02:39:50  sasha
 * Added extra member to hold interrupt/program association. Changed the way
 * skipped frames are counted.
 * Revision 1.16  1996/02/08 23:51:30  sasha
 * Deleted SetClipping function. Added time-out in the Start function when a fault
 * created. Set clipping based on info stored in the stream. Created a few helper
 * functions that are called from Restart().
 * Revision 1.15  1996/02/06 01:33:15  sasha
 * Changed ProcessSyncPrograms(). Added Restart() which is called from ISR
 * and Stop(). Added skipped frames counting.
 * Revision 1.14  1996/01/31 22:39:28  sasha
 * Changed argument of the Skip function and changed it.
 * Changed ProcessSyncProgs(), Start() and Add Program. Do not call
 * CreateSyncCodes from ProcessPresentPrograms.
 * Revision 1.13  1996/01/30 19:19:27  sasha
 * Added synchronization code to block ISR when adding buffers
 * Revision 1.12  1996/01/17 00:14:19  sasha
 * Changed the ProcessAddBuffer() function. Field's Start function wasn't always
 * called before.
 * Revision 1.11  1996/01/16 01:26:33  sasha
 * Modified ProcessAddBuffer() to fix jump to jump condition
 * Revision 1.10  1996/01/16 00:25:01  sasha
 * Changed DEBUGTRACE to OUTPUT_MESS
 * Revision 1.9  1996/01/16 00:13:15  sasha
 * Removed the check for stopped channel in BtPisces::Stop(). It was causing problems
 * when Stop was called on channels that were stopped because they were
 * skipped
 * Revision 1.8  1996/01/15 20:12:12  sasha
 * Added SetSVideo function. Moved some functions around. Modified Skip()
 * to stop the field. Modified ProcessAddBuffer to restart the field if it was stopped
 * by the Skip. Removed commented out code
 * Revision 1.7  1996/01/06 02:04:26  sasha
 * Changed ColorFormat to ColFmt
 * Revision 1.6  1996/01/06 01:45:25  sasha
 * Changed to use new decoder
 * Revision 1.5  1996/01/02 19:06:20  sasha
 * Fixed bug in the ProcessAddBuffer. There was no check for the empty queue
 * Added Engine_.SetChain() call after setting clipping
 * Revision 1.4  1995/12/08 22:13:53  sasha
 * Fixed multi-buffering related problems as per Mike's testing
 * Revision 1.3  1995/11/07 00:56:33  sasha
 * Moved some code out of Interrupt function to the new function
 * ProcessBufferAtInterrupt. Idea is for some channel classes to call it,
 * for some to ignore. As a result channel is still notified about interrupt, but RISC
 * program is can be left unmodified
 * Revision 1.2  1995/11/02 22:01:50  sasha
 * First major check-in for testing
 * Revision 1.1  1995/10/07 08:41:08  sasha
 * Initial revision
 */

static char rcsid[] = "$Id: PISCES.CPP 2.67 1998/06/01 18:05:02 mdimeo Exp $" ;
#include "pisces.h"

#include <stdlib.h>

#include "dbgprint.h"

extern "C" {
  #include "vpaec.h"
}

#include "vxdid.h"
#include "bt848vxd.h"


#define MAX_VXD_FRAME_BUFFERS (10)
#define MAX_FRAME_BUF_SIZE (640*480*4)
LPVIDEOHDR VxdFrameBuf[MAX_VXD_FRAME_BUFFERS];

extern PDWORD pSkipCount ;
extern InterruptCounter gIntrCntr ;
extern BufferInstrumentation gBufInstrumentation ;

DWORD BtPisces::dwPlanarAdjust_;
TRect VBIField::AnalogWin_( 0, 0, 0, 0 );
TRect VBIField::DigitalWin_( 0, 0, 0, 0 );
WORD VBIField::VBI_PacketSize ;
BYTE VBIField::VBI_HDelay ;
HANDLE BtPisces::NotifyEvent_;
DWORD  BtPisces::IntStatus_;
DWORD  BtPisces::VSYNC_CB_ = NULL;
DWORD  BtPisces::GPIO_CB_ = NULL;
DWORD  BtPisces::I2C_CB_ = NULL;
BtPisces *BtPisces::me_;
bool   BtPisces::VsyncTaken_ = false ;

template <class T> inline T Max( const T a, const T b )
{
   return a > b ? a : b;
}

inline bool IsMaster( int idx )
{
   return bool(idx == EvenStartLocation  ||
               idx == EvenStartLocation + DistBetweenProgs) ;
}

inline bool IsSlave( int idx )
{
   return bool(idx == OddStartLocation  ||
               idx == OddStartLocation + DistBetweenProgs) ;
}


inline bool IsFirst( int idx )
{
   return bool( idx == OddStartLocation || idx == VBIOStartLocation ||
      idx - DistBetweenProgs == OddStartLocation ||
      idx - DistBetweenProgs == VBIOStartLocation  );
}

inline bool IsLast( int idx )
{
   return bool( idx == VBIEStartLocation + DistBetweenProgs ||
      idx == EvenStartLocation + DistBetweenProgs );
}

inline bool IsEvenField( int idx )
{
   return bool( idx == EvenStartLocation  ||
                idx == EvenStartLocation + DistBetweenProgs );
}

inline bool IsVBI( int idx )
{
   return bool( idx == VBIOStartLocation ||
                idx == VBIOStartLocation + DistBetweenProgs ||
                idx == VBIEStartLocation ||
                idx == VBIEStartLocation + DistBetweenProgs);
}


//
//  CProgArray member functions
//

   DWORD CProgArray::GetIndex(RiscPrgHandle hRp) {
      for (int i = 0 ; i < MAX_PROGS ; i++)
         if (rpArray[i] == hRp)
            return i ;

      return (DWORD) -1 ;
   }

   DWORD CProgArray::Count() {
      DWORD n = 0 ;
      for (int i = 0 ; i < MAX_PROGS ; i++)
         if (rpArray[i])
            n++ ;
      return n ;
   }

   RiscPrgHandle CProgArray::First() {
      RiscPrgHandle hRp = NULL ;
      for (dwCurrNdx_ = 0 ; dwCurrNdx_ < MAX_PROGS ; dwCurrNdx_++) {
         if (rpArray[dwCurrNdx_]) {
            hRp = rpArray[dwCurrNdx_++] ;
            break ;
         }
      }
      return hRp ;
   }

   RiscPrgHandle CProgArray::Next() {
      RiscPrgHandle hRp = NULL ;
      for ( ; dwCurrNdx_ < MAX_PROGS ; dwCurrNdx_++) {
         if (rpArray[dwCurrNdx_]) {
            hRp = rpArray[dwCurrNdx_++] ;
            break ;
         }
      }
      return hRp ;
   }

/*
*/
BtPisces::BtPisces(WORD devId) : devId_(devId), Engine_(), AllocatedStreams_(), PresentStreams_(),
   Even_( CAPTURE_EVEN, VF_Even,&COLOR_EVEN, &WSWAP_EVEN, &BSWAP_EVEN ),
   Odd_( CAPTURE_ODD, VF_Odd, &COLOR_ODD, &WSWAP_ODD, &BSWAP_ODD ),
   VBIE_( CAPTURE_VBI_EVEN ), VBIO_( CAPTURE_VBI_ODD ),
   Paused_( false ),
   OldImageSize_(0),
   NumVxdFrameBuffers_(5),
   Starter_(       ),
   SyncEvenEnd1_(  ),
   SyncEvenEnd2_(  ),
   SyncOddEnd1_(   ),
   SyncOddEnd2_(   ),
   PsDecoder_(),
   R3VidInQueue_ (10),
   R3VbiQueue_ (10),
   AvailableQueue_ (10),
   FilledQueue_ (10),
   //   dwPlanarAdjust_( 0 ),
   CONSTRUCT_COLORCONTROL,
   CONSTRUCT_INTERRUPTSTATUS,
   CONSTRUCT_INTERRUPTMASK,
   CONSTRUCT_CONTROL,
   CONSTRUCT_CAPTURECONTROL,
   CONSTRUCT_COLORFORMAT,
   CONSTRUCT_GPIOOUTPUTENABLECONTROL,
   CONSTRUCT_GPIOREGISTEREDINPUTCONTROL,
   CONSTRUCT_GPIODATAIO,
   CONSTRUCT_RES_1E
{
//   LastRiscIntrTime_ = Get_System_Time() ;
   Init();
}

BtPisces::~BtPisces()
{
   Engine_.Stop();
   InterruptMask = 0;                 // We're going out of scope. Disable all interrupts
                                      //   (Yes, even GPIO and I2C)
   InterruptStatus = AllFs;

   // prevent risc program destructor from gpf-ing
   SyncEvenEnd1_.SetParent( NULL );
   SyncEvenEnd2_.SetParent( NULL );
   SyncOddEnd1_.SetParent( NULL );
   SyncOddEnd2_.SetParent( NULL );

   // free the association array now
   int ArrSize = sizeof( InterruptToIdx_ ) / sizeof( InterruptToIdx_ [0] );
   while ( --ArrSize >= 0 )
      delete InterruptToIdx_ [ArrSize];

   // delete vxd frame buffers
   for (int i = 0 ; i < NumVxdFrameBuffers_ ; i ++)
   {
       if (VxdFrameBuf[i]) { 
         if (VxdFrameBuf[i]->lpData)
            delete [] VxdFrameBuf[i]->lpData ;

         delete VxdFrameBuf[i] ;
       }
   }
}

/* Method: BtPisces::GetIdxFromStream
 * Purpose: Returns starting index in the program array for a field
 * Input: aStream: StreamInfo & - reference
 * Output: int : Index
 */
int BtPisces::GetIdxFromStream( StreamInfo &aStream )
{
   switch ( GetField(aStream).GetStreamID() ) {
   case VS_Field1:
      return OddStartLocation;
   case VS_Field2:
      return EvenStartLocation;
   case VS_VBI1:
      return VBIOStartLocation;
   case VS_VBI2:
      return VBIEStartLocation;
   default:
      return 0;
   }
}

void BtPisces::CreateSyncCodes()
{
   if (SyncEvenEnd1_.Create( SC_VRE ) != Success)
      DebugPrint((0,"SyncEven1 Create Failed\n")) ;
   if (SyncEvenEnd2_.Create( SC_VRE ) != Success)
      DebugPrint((0,"SyncEvenEnd2_ Create Failed\n")) ;

   if (SyncOddEnd1_.Create( SC_VRO ) != Success)
      DebugPrint((0,"SyncOddEnd1_ Create Failed\n")) ;
   if (SyncOddEnd2_.Create( SC_VRO ) != Success)
      DebugPrint((0,"SyncOddEnd2_ Create Failed\n")) ;

//   if (Starter_.Create( SC_VRO ) != Success)
//      DebugPrint((0,"Starter_ Create Failed\n")) ;
}

/* Method: BtPisces::InitGPIOFromRegistry
 * Purpose: Set up GPIO According to Registry Key string values
 * Input: None
 * Output: None
 */
void BtPisces::InitGPIOFromRegistry()
{
   // All string values (even the key itself) are optional.
   // If the value is not present in the registry, no action is
   // taken (no writes are issued to these registers).
   //
   // Reg Key is "Bt848\GpioInit" and has following string values:
   //
   // For the GPIO and DMA Control register (0x10C):
   //   GPCLKMODE   [bit  10]
   //   GPIOMODE    [bits 12:11]
   //   GPWEC       [bit  13]
   //   GPINTI      [bit  14]
   //   GPINTC      [bit  15]
   // For the GPIO Output Enable Control register (0x118):
   //   GPOE        [bits 23:0]
   // For the GPIO Registered Input Control register (0x11c) :
   //   GPIE        [bits 23:0]
   // For the GPIO Data I/O register (0x200):
   //   GPDATA      [bits 23:0]

   struct {
     LPSTR tag ;          // match from registry
     RegField * reg ;     // pointer to associated register
   } gpKeys [] = {
        {"GPCLKMODE"    ,&GPCLKMODE} ,
          {"GPIOMODE"     ,&GPIOMODE}  ,
          {"GPWEC"        ,&GPWEC}     ,
          {"GPINTI"       ,&GPINTI }   ,
          {"GPINTC"       ,&GPINTC }   ,
          {"GPOE"         ,&GPOE   }   ,
          {"GPIE"         ,&GPIE   }   ,
          {"GPDATA"       ,&GPDATA }
   } ;
   #define NUM_GP_KEYS (sizeof(gpKeys) / sizeof(gpKeys[0]))

   char valName[80], valData[80] ;
   DWORD cbValName, cbValData;
   REGISTRYVALUETYPE rvType ;

   VRegistryKey vkey( PRK_LOCAL_MACHINE, "SOFTWARE\\Brooktree\\Video\\GpioInit" );
   if ( vkey.lastError() != ERROR_SUCCESS )
      DebugPrint((1,"No GpioInit key\n"));

   cbValData = sizeof(valData) ;
   cbValName = sizeof(valName) ;
   bool bFound = (bool) vkey.firstValue(valName, &cbValName, &rvType, valData, &cbValData);
   if (!bFound) {
      DebugPrint((1,"No Value keys\n"));
   }

     while (bFound)
     {
      bool bWrote =false ;
      for (int i = 0 ; i < NUM_GP_KEYS ; i ++)
      {
          if (!stricmp(valName,gpKeys[i].tag))
          {
             DebugPrint((1,"%s (idx %d) = [%s]\n", valName, i, valData));
             *(gpKeys[i].reg) = strtoul( valData, NULL, 0 ); // handles '0x' prefix if present
             bWrote = true ;
             break ;
          }
      }

      if(!bWrote) {
         DebugPrint((1,"Invalid: [%s] = [%s]\n", valName, valData));
      }

      // find next string value
      cbValData = sizeof(valData) ;
      cbValName = sizeof(valName) ;
      bFound = (bool) vkey.nextValue(valName, &cbValName, &rvType, valData, &cbValData) ;
     } // End while
}

/* Method: BtPisces::Init
 * Purpose: Performs all necessary initialization
 * Input: None
 * Output: None
 */
void BtPisces::Init()
{
   me_ = this; // some static functions need access to the data

   InterruptStatus = AllFs;
   InterruptStatus = 0;
   GAMMA = 1;

   InitGPIOFromRegistry() ;

   // Invoke new vpres circuitry by setting MSB of reserved register
   // at offset 0x78
   ENABLE_NEW_VPRES = 1;

   PresentStreams_.Add( new StreamInfo( 0, VS_Preview, &Odd_ ) );
   PresentStreams_.Add( new StreamInfo( 1, VS_Capture, &Even_ ) );
   PresentStreams_.Add( new StreamInfo( 2, VS_VBIO, &VBIO_ ) );
   PresentStreams_.Add( new StreamInfo( 3, VS_VBIE, &VBIE_ ) );

   // initialize the array
   int ArrSize = CreatedProgs_.Size();
   while ( --ArrSize >= 0 )
      CreatedProgs_ [ArrSize] = ActiveProgs_ [ArrSize] = NULL;

   // fill in constant elements; see the table in the .h file
   CreatedProgs_ [2] = &SyncOddEnd1_;
   CreatedProgs_ [8] = &SyncOddEnd2_;

   CreatedProgs_ [5]  = &SyncEvenEnd1_;
   CreatedProgs_ [11] = &SyncEvenEnd2_;

   DebugPrint((2,"SyncOddEnd1_ %08lx",&SyncOddEnd1_)) ;
   DebugPrint((2,"SyncOddEnd2_ %08lx",&SyncOddEnd2_)) ;
   DebugPrint((2,"SyncEvenEnd1_ %08lx",&SyncEvenEnd1_)) ;
   DebugPrint((2,"SyncEvenEnd2_ %08lx",&SyncEvenEnd2_)) ;

   // set corresponding sync bits
   CreateSyncCodes();

   // initialize association array now
   ArrSize = sizeof( InterruptToIdx_ ) / sizeof( InterruptToIdx_ [0] );
   while ( --ArrSize >= 0 )   {
      InterruptToIdx_ [ArrSize] = new IntrIdxAss();
      DebugPrint((1,"Init InterruptToIdx[%d] = %x\n",ArrSize,InterruptToIdx_ [ArrSize])) ;
   }

   Odd_.SetStreamID( VS_Field1 );
   Even_.SetStreamID( VS_Field2 );
   VBIO_.SetStreamID( VS_VBI1 );
   VBIE_.SetStreamID( VS_VBI2 );

   // How many frame buffers are we going to be using?
   // default is 5 frame buffers (set in constructor)
   VRegistryKey vkey( PRK_LOCAL_MACHINE, "SOFTWARE\\Brooktree\\Video\\NumVxdFrames" );
   if ( vkey.lastError() != ERROR_SUCCESS )
   {
      DebugPrint((1,"No NumVxdFrames reg key present\n")) ;
   }
   else
   {
      char anonValue[80];
      DWORD cbValue = sizeof(anonValue);

      if (vkey.getValue(anonValue, &cbValue))
      {
         NumVxdFrameBuffers_ = strtoul( anonValue, NULL, 0 ) ;
         if (NumVxdFrameBuffers_ >  MAX_VXD_FRAME_BUFFERS)
            NumVxdFrameBuffers_ = MAX_VXD_FRAME_BUFFERS ;
      }
   }
   DebugPrint((1,"Using NumVxdFrames =  %d\n",NumVxdFrameBuffers_)) ;

   // zero out the vxd frame buffer pointer array
   for (int i = 0 ; i < MAX_VXD_FRAME_BUFFERS ; i ++)
   {
      VxdFrameBuf[i] = NULL ;
   }    

   // hook up buffer instrumentation
   gBufInstrumentation.pR3QueueCount        = R3VidInQueue_.GetCountAddress();
   gBufInstrumentation.pAvailableQueueCount = AvailableQueue_.GetCountAddress();
   gBufInstrumentation.pFilledQueueCount    = FilledQueue_.GetCountAddress();

   gBufInstrumentation.pR3Size        = R3VidInQueue_.GetSizeAddress();
   gBufInstrumentation.pAvailableSize = AvailableQueue_.GetSizeAddress();
   gBufInstrumentation.pFilledSize    = FilledQueue_.GetSizeAddress();

}

/* Method: BtPisces::AssignIntNumbers
 * Purpose: Assigns numbers to RISC programs that generate interrupt
 * Input: None
 * Output: None
 */
void BtPisces::AssignIntNumbers()
{
   int IntrCnt = 0;
   int limit = ActiveProgs_.Size() ;
   int idx;

   // initialize InterruptToIdx_ array
   for ( idx = 0; idx < limit; idx++ ) {
         IntrIdxAss item( idx, -1 );
         DebugPrint((1,"Assiging %x to InterruptToIdx_ [%d]  limit = %d\n",&item, idx, limit)) ;
         *InterruptToIdx_ [idx] = item;
   }

   // assign numbers in front of starting program
   bool first = true;
   DebugPrint((1,"AssignIntNumbers:\n")) ;
   for ( idx = 0; idx < limit; idx++ ) {

      RiscPrgHandle pProg = ActiveProgs_ [idx];

      //if not skipped and generates an interrupt assign number
      if ( pProg && pProg->IsInterrupting() ) {
         if ( first == true ) {
            first = false;
            pProg->ResetStatus();
         } else {
            pProg->SetToCount();
         }
         DebugPrint((1,"    IntrCnt %d, idx %d\n",IntrCnt, idx)) ;
         IntrIdxAss item( IntrCnt, idx );
         *InterruptToIdx_ [IntrCnt] = item;
         IntrCnt++;
      }
   }
   DebugPrint((1,"AssignIntNumbers Done\n")) ;
}

/* Method: BtPisces::LinkThePrograms
 * Purpose: Creates links between the created programs
 * Input: None
 * Output: None
 */
SyncCode scLastStarterSync = SC_Reserved ;

void BtPisces::LinkThePrograms( bool bNewStarter )
{
   DebugPrint((1, "*** Linking Programs\n"));
   RiscPrgHandle hParent    = ActiveProgs_.First(),
                 hChild,
                 hVeryFirst = NULL,
                 hLastChild = NULL ;

   if (hParent) {

      while (hParent) {

         if (!hVeryFirst)
            hVeryFirst = hParent ;

         if ( NULL != (hChild = ActiveProgs_.Next()) ) {

            hLastChild = hChild;
            DebugPrint((1,"      Link Parent  %x (%d) and Child %x (%d)  ",
                                  hParent,  CreatedProgs_.GetIndex(hParent), hChild, CreatedProgs_.GetIndex(hChild))) ;
            Engine_.Chain( hParent, hChild ) ;

         }
         hParent = hChild ;
      }

      // Create new starter prog if directed. Starter should SYNC VRO unless an 
      // even field is linked in.
      if (bNewStarter)
      {
         SyncCode Sync = SC_VRO ;

         if (ActiveProgs_[4] || ActiveProgs_[10]||    // Even Field
             ActiveProgs_[3] || ActiveProgs_[9] )     // VBI Even
            Sync = SC_VRE ;

         // why create another on if the last one created will do?
         if ( scLastStarterSync != Sync ) 
         {
            DebugPrint((1,"  Creating a new starter: Sync = %s\n ",Sync == SC_VRE ? "SC_VRE" : "SC_VRO"))
            Starter_.Create( Sync, true );
            scLastStarterSync = Sync ;
         }
      }
        
      // initial jump
      DebugPrint((1,"      Link Starter %x     and First %x (%d)  ",
                            &Starter_,  hVeryFirst, CreatedProgs_.GetIndex(hVeryFirst))) ;
      Engine_.Chain( &Starter_, hVeryFirst ) ;

      // now create the loop
      #if 0
      if (hLastChild) {
         // Debug: single shot loop
         DebugPrint((0,"   Created single shot loop  ")) ;
         Engine_.Chain( hLastChild, hLastChild ) ;
      }
      else  
      #endif
      {
        DebugPrint((1,"      Link Last    %x (%d) and 1st   %x (%d)  ",
                              hLastChild,  CreatedProgs_.GetIndex(hLastChild), hVeryFirst, CreatedProgs_.GetIndex(hVeryFirst))) ;
        Engine_.Chain( hLastChild ? hLastChild : hVeryFirst, hVeryFirst ) ;
      }
   }
}

/* Method: BtPisces::ProcessSyncPrograms()
 * Purpose: This function unlinks the helper sync programs
 * Input: None
 * Output: None
 */
void BtPisces::ProcessSyncPrograms()
{
   int limit = ActiveProgs_.Size() ;

   for ( int i = 0; i < limit; i += ProgsWithinField ) {
      // If Video program is interrupting, we need to make sure all sync programs are linked in.
      //  [bugbug]  expound... Possibly need to do vbi progs too.
      if ( !ActiveProgs_ [i] && !ActiveProgs_ [i+1] 
//           && !ActiveProgs_ [i+1]->IsInterrupting()
      )
         ActiveProgs_ [i+2] = NULL;
      else
         ActiveProgs_ [i+2] = CreatedProgs_ [i+2];
   }
}

/* Method: BtPisces::ProcessPresentPrograms
 * Purpose:
 * Input: None
 * Output: None
 */
void BtPisces::ProcessPresentPrograms()
{
   // link in/out helper sync programs
   ProcessSyncPrograms();
   // and now is time to cross link the programs
   LinkThePrograms(true);
   // and now figure out the numbers programs use for interrupts
   AssignIntNumbers();
}

/* Method: BtPisces::AddProgram
 * Purpose: Creates new RISC program and inserts it in the chain at a proper place
 * Input: aStream: StreamInfo & - reference to the stream to add a program for
 *   NumberToAdd: int - number of programs to add
 * Output:
 * Note:   Basically this internal function performs a loop 2 times
 *   //4. Tries to get another buffer to establish double buffering
 *   //5. If buffer is available it creates another RISC program with it
 *   //6. Then it has to link the program in...
 */
RiscPrgHandle BtPisces::AddProgram( StreamInfo &ToStart, int NumberToAdd )
{
   int StartIdx = GetIdxFromStream( ToStart );
   SyncCode Sync;
   int SyncIdx;
   bool rsync;
   if ( StartIdx <= OddStartLocation ) {
      Sync = SC_VRO;
      SyncIdx = OddSyncStartLoc;
      rsync = false;
   } else {
      Sync = SC_VRE;
      SyncIdx = EvenSyncStartLoc;
      rsync = bool( StartIdx == EvenStartLocation );
   }
    // have to know what is the size of the image to produce
   TRect r;
   GetField(ToStart).GetDigitalWindow( r );
   // RISC engine operates on absolute sizes, not rectangles
   TSize s = r.Size();

   int BufCnt = 0;

   DebugPrint( ( 1,"\nBtPisces::Add Program:\n"));

   int Idx = StartIdx;
   for ( ; BufCnt < NumberToAdd; BufCnt++ ) 
   {
      LPVIDEOHDR lpvh ;

      // init sync programs with a premise tha no data program exists
      CreatedProgs_ [SyncIdx]->Create( Sync, true );

      // obtain the next buffer from queue ( entry is removed from container )
      if ( ToStart.GetPaired() )
      {
         if (IsMaster(Idx)) {
           if(!ToStart.Interrupt_)
              lpvh = R3VidInQueue_.Remove() ;            // Not interrupting... must be overlay
           else
              lpvh = AvailableQueue_.Remove() ;

           DebugPrint((1,"  got MASTER lpvh(%x) for idx %d\n",lpvh,Idx));
         }
         else
         {
           // Slave: obtain buffer from master prog
           lpvh = CreatedProgs_[Idx+ProgsWithinField]->GetVideoHdr() ;
           DebugPrint((1,"  got Slave lpvh(%x) for idx %d from master idx %d\n",
                               lpvh,Idx,Idx+ProgsWithinField));
         }
      }
      else
      {  
         // Not interrupting... must be overlay or vbi
         if(!ToStart.Interrupt_)
         {
            // overlay
            lpvh = R3VidInQueue_.Remove() ;              
            DebugPrint((1,"   (single overlay): R3 queue remove = %x\n",lpvh));
            if (!lpvh)
            {  
               // this can happen if we're running both fields to the overlay surface (QuartzSight)
               // since we're given only enough buffers (2) for a single stream to start an
               // overlay. We obtain the buffer address (actually the VIDEOHDR pointer)
               // from the other field in this case.
               DebugPrint((1,"   (single overlay): Overlay mode QS? Attempt to steal...\n"));
               DWORD idxToSteal ;
               if (IsEvenField(Idx))
                  idxToSteal  = Idx - ProgsWithinField ; 
               else
                  idxToSteal  = Idx + ProgsWithinField ; 
           
               if (CreatedProgs_[idxToSteal])
                  lpvh = CreatedProgs_[idxToSteal]->GetVideoHdr() ;
            }
         }
         else
         {
            if (IsVBI(Idx))
            {
               lpvh = R3VbiQueue_.Remove() ;      // vbi stream        
               DebugPrint((1,"   (single capture VBI): Available queue remove = %x\n",lpvh));
            }
            else
            {
               lpvh = AvailableQueue_.Remove() ;  // Interrupting: capture
               DebugPrint((1,"   (single capture): Available queue remove = %x\n",lpvh));
            }
         }
         DebugPrint((1,"   (single): got lpvh(%x) for idx %d\n",lpvh,Idx));
      }
      DebugAssert((bool(lpvh != NULL),"*** No available buffer\n")) ;

      // can create a RISC program now.
      Field & fld = GetField(ToStart) ;
      RiscPrgHandle hProgram = 
          Engine_.CreateProgram( s,                    // Size
                                 fld.GetBufPitch(),    // pitch
                                 fld.GetOffset(),      // offset. Zero unless this is an interlaced frame.
                                 fld.GetColorFormat(), // color format
                                 lpvh,                 // pointer to videohdr containing buffer to fill
                                 GetFormat(),          // 
                                 ToStart.Interrupt_,   // Is program going to generate risc interrupt?
                                 dwPlanarAdjust_,      //
                                 rsync );              //

      // store this program
      CreatedProgs_ [Idx] = hProgram;
      DebugPrint((1,"Created: idx=%d  hProgram=%08lx  lpvh = %x Intr:%x\n",Idx,hProgram,lpvh,ToStart.Interrupt_)) ;

      if ( !hProgram ) {
         Idx -= DistBetweenProgs;
         if ( Idx >= 0 ) {
            // clean up previous program
            Engine_.DestroyProgram( CreatedProgs_ [Idx] );
            CreatedProgs_ [Idx] = NULL;
         }
         return NULL;
      }
      // make sure we unskip the program when buffer becomes available
      if ( !lpvh ) {
         hProgram->SetSkipped(true,Get_System_Time());  // do not have enough buffers to support double buffering
      }

      // assign stream to program; makes it easy during interrupt
      hProgram->SetTag( &ToStart );

      // check out the clipping
      const RGNDATA *const clip = ToStart.GetClipping();
      if ( clip )
         Engine_.SetClipping( hProgram, *clip );

      SyncIdx += DistBetweenProgs;
      Idx     += DistBetweenProgs; // skip the location intended for the other program

   } /* endfor */
   return CreatedProgs_ [StartIdx];
}

/* Method: BtPisces::Create
 * Purpose: This functions starts the stream.
 * Input: aStream: StreamInfo & - reference to a stream to start
 * Output: Address of the Starter_
 * Note: After the Start 2 entries in the CreatedProgs_ are created. Starting
 *   location is 4 for even and 1 for odd. Increment is 6. So if it is the first
 *   invocation and there is enough ( 2 ) buffers present entries [1] and [7]
 *   or [4] and [10] will be filled with newly created RISC programs. When programs
 *   exist for one field only they are doubly linked. When programs exist for
 *   both fields they alternate, i.e. 0->2->1->3->0... When one of the fields
 *   has 1 program only, programs are linked like this: 0->2->1->0->2...(numbers
 *   are indexes in the CreatedProgs_ array ). Alternating programs makes for
 *   maximum frame rate.
 */
ErrorCode BtPisces::Create( StreamInfo &ToCreate )
{
   // who is starting ?
//   Field &ToStart = GetField( aStream );
   DebugPrint((1,"--> Creating %08lx\n",(DWORD) ToCreate.tag )) ;
   // running full-steam, nothing to create
   if ( GetField(ToCreate).IsStarted() == true )
      return Success;

   int StartIdx = GetIdxFromStream( ToCreate );
   if ( CreatedProgs_ [StartIdx] )
      return Success; // not running yet, but exists

   // call into internal function that adds new RISC program
   if ( ! AddProgram( ToCreate, MaxProgsForField ) )
   {
      DebugPrint((0,"   *** AddProgram Failed!\n" )) ;

      return Fail;
   }

   DebugPrint((1,"   done Create\n" )) ;
   return Success;
}

void BtPisces::Start( StreamInfo & ToStart, StartType st )
{
   DebugPrint((1,"--> Starting %08lx\n",(DWORD) ToStart.tag )) ;

   Paused_ = false ;

   if ( GetField( ToStart ).IsStarted() == true )
      return;

   // all we need to do at this point is to create a proper starter
   // and link the programs in.
   int idx = GetIdxFromStream( ToStart );
   // this loop will enable LinkThePrograms to see programs for this stream
   for ( int i = 0; i < MaxProgsForField; i++, idx += DistBetweenProgs ) {
      ActiveProgs_ [idx] = CreatedProgs_ [idx]->IsSkipped() ? NULL : CreatedProgs_ [idx];
   }
   // all I want to do at this point is call Restart.
   if (st != START_MASTER)
      Restart();
   else
      DebugPrint((1,"*** DID NOT RESTART : StartMaster\n")) ;
}

/* Method: BtPisces::Stop
 * Purpose: This function stops a stream. Called when PAUSE SRB is received
 * Input: aStream: StreamInfo & - reference to a stream to start
 * Output: None
 */
void BtPisces::Stop( StreamInfo &ToStop )
{
   DebugPrint((1,"Bt848: BtPisces::Stop\n"));

   ToStop.SetClipping( NULL );

   int StartIdx = GetIdxFromStream( ToStop );

   // prevent unneeded syncronization interrupts
   IMASK_SCERW = 0;

   // it is time to pause the stream now
   GetField(ToStop).Stop();
   bool Need2Restart = false;
   // go through the array of programs and killing ones for this field (stream)
   for ( int i = 0; i < MaxProgsForField; i++, StartIdx += DistBetweenProgs ) {

      RiscPrgHandle ToDie = CreatedProgs_ [StartIdx];
      if ( !ToDie ) // this should never happen
         continue;

      DebugPrint((1,"Bt848: about to destroy idx = %d\n", StartIdx ) );
      Engine_.DestroyProgram( ToDie );
      CreatedProgs_ [StartIdx] = NULL;
      ActiveProgs_  [StartIdx] = NULL; // in case Pause wasn't called
      Need2Restart = true;
   } /* endfor */

   // nobody's around anymore
   if ( !CreatedProgs_.CountDMAProgs() ) {
      Engine_.Stop();

      // Flush all queues
      FilledQueue_.Flush() ;
      AvailableQueue_.Flush() ;
      R3VidInQueue_.Flush() ;

      // set all bits to 0 preserving state of GPINT & I2C bits
      DWORD tmpImsk = (DWORD) InterruptMask ;
      InterruptMask = tmpImsk & (GPINT_I | I2CDONE_I);

      // clear all pending interrupts except GPINT and I2C
      InterruptStatus = (AllFs & ~(GPINT_I | I2CDONE_I));
   } else {
      if ( Need2Restart ) {
         Restart();   // relink the programs and start ones that are alive
         IMASK_SCERW = 1; // re-enable the sync error interrupts
      }
      else
         DebugPrint((1,"*** DID NOT RESTART : BtPisces::Stop()\n")) ;
        
   }
}

/* Method: BtPisces::Pause
 * Purpose: This function stops a stream. Called when PAUSE SRB is received
 * Input: aStream: Field & - reference to a stream to start
 * Output: None
 */
void BtPisces::PauseStream(StreamInfo &ToPause )
{
   DebugPrint((1,"Bt848: BtPisces::PauseStream\n"));

   if ( !GetField(ToPause).IsStarted() )
      return;

   int StartIdx = GetIdxFromStream( ToPause );

   // prevent unneeded syncronization interrupts
   IMASK_SCERW = 0;

   // go through the array of programs and killing ones for this field (stream)
   for ( int i = 0; i < MaxProgsForField; i++, StartIdx += DistBetweenProgs ) {
      ActiveProgs_ [StartIdx] = NULL;
   } 

   Restart();   // relink the programs and start ones that are alive
}

/* Method: BtPisces::PairedPause
 * Purpose: This is a hacky function that pauses 2 streams at once
 * Input: idx: index of the second program in the second field
 * Output: None
 */
void BtPisces::PairedPause( int idx )
{
   // go through the array of programs and killing ones for this field (stream)
   for ( int i = 0; i < MaxProgsForField; i++, idx -= DistBetweenProgs ) {
      ActiveProgs_ [idx] = NULL;
      ActiveProgs_ [idx-ProgsWithinField] = NULL;
   } 

   Restart();   // relink the programs and start ones that are alive
}

/* Method: BtPisces::GetStarted
 * Purpose: Figures out the channels that are started
 * Input:
 * Output: None
 */
void BtPisces::GetStarted( bool &EvenWasStarted, bool &OddWasStarted,
   bool &VBIEWasStarted, bool &VBIOWasStarted )
{
   VBIEWasStarted = bool( ActiveProgs_ [VBIEStartLocation] );
   EvenWasStarted = bool( ActiveProgs_ [EvenStartLocation] );

   VBIOWasStarted = bool( ActiveProgs_ [VBIOStartLocation] );
   OddWasStarted  = bool( ActiveProgs_ [OddStartLocation] );
}

/* Method: BtPisces::RestartStreams
 * Purpose: Restarts streams that were started
 * Input:
 * Output: None
 */
void BtPisces::RestartStreams( bool EvenWasStarted, bool OddWasStarted,
   bool VBIEWasStarted, bool VBIOWasStarted )
{
   // vbi programs are first to execute, so enable them first
   if ( VBIOWasStarted )
      VBIO_.Start();
   if ( OddWasStarted )
      Odd_.Start();
   if ( VBIEWasStarted )
      VBIE_.Start();
   if ( EvenWasStarted )
      Even_.Start();
}

void  BtPisces::DeallocVxdFrames() 
{
   for (int i = 0 ; i < NumVxdFrameBuffers_ ; i ++)
   {
       if (VxdFrameBuf[i]) { 
         if (VxdFrameBuf[i]->lpData)
            delete [] VxdFrameBuf[i]->lpData ;

         delete VxdFrameBuf[i] ;
         VxdFrameBuf[i] = NULL ;
       }
   }
   OldImageSize_ = 0 ;
}

void  BtPisces::RestartFrameServer(DWORD dwImageSize) 
{
   // reallocate frame buffers
   if (dwImageSize && dwImageSize != OldImageSize_)
   {
      // create 'available' bufers
      DebugPrint((-1,"Creating VxdFrameBuf... %u  %u\n",dwImageSize, OldImageSize_)) ;

      for (int i = 0 ; i < NumVxdFrameBuffers_ ; i ++)
      {
          if (VxdFrameBuf[i]) { 
            if (VxdFrameBuf[i]->lpData)
               delete [] VxdFrameBuf[i]->lpData ;

            delete VxdFrameBuf[i] ;
          }

          VxdFrameBuf[i] = new VIDEOHDR ;
          DebugAssert((bool(VxdFrameBuf[i] != NULL),"Bad alloc in RestartFrameServer\n")) ;
          if (VxdFrameBuf[i]) 
          {
             memset((PVOID) VxdFrameBuf[i], 0, sizeof(VIDEOHDR)) ;
             // [bugbug] We should probably use dwImageSize for the buffer allocation...
             VxdFrameBuf[i]->lpData = new BYTE[MAX_FRAME_BUF_SIZE] ;
             DebugAssert((bool(VxdFrameBuf[i]->lpData != NULL),"Bad alloc in RestartFrameServer\n")) ;
             VxdFrameBuf[i]->dwReserved[0]= (DWORD) (VxdFrameBuf[i]->lpData ) ;
             if (!VxdFrameBuf[i]->lpData)
             {
                DebugPrint((1,"Unable to alloc data buf for VxdFrameBuf[%d]",i)) ;
                delete VxdFrameBuf[i] ;
                VxdFrameBuf[i] = 0;
             }
          }
          else
          {
             DebugPrint((1,"Unable to alloc VxdFrameBuf[%d]",i)) ;
          }
      }
      OldImageSize_ = dwImageSize ;
   }

   // Flush available and filled queues
   AvailableQueue_.Flush() ;
   FilledQueue_.Flush() ;

   // fill up the available queue
   DebugPrint((1,"Filling up the AvailableQueue_ ImageSize = %d!\n",dwImageSize)) ;
   for (int i = 0 ; i <  NumVxdFrameBuffers_; i++)
      if ( VxdFrameBuf[i] )
         AvailableQueue_.Add(VxdFrameBuf[i]) ;
}

/* Method: BtPisces::Restart
 * Purpose: Restarts the capture process. Called by ISR and Stop()
 * Input: None
 * Output: None
 */
void BtPisces::Restart()
{
   bool EvenWasStarted, OddWasStarted, VBIEWasStarted, VBIOWasStarted;
   GetStarted( EvenWasStarted, OddWasStarted, VBIEWasStarted, VBIOWasStarted );

   Odd_.Stop();
   Even_.Stop();
   VBIE_.Stop();
   VBIO_.Stop();
   Engine_.Stop();

   // this will never happen, probably
   if ( !EvenWasStarted && !OddWasStarted && !VBIEWasStarted && !VBIOWasStarted )
      return;

   InterruptStatus = AllFs; // clear all the status bits

   ProcessPresentPrograms();

#if 1
   Engine_.Start( Starter_ );

   RestartStreams( EvenWasStarted, OddWasStarted, VBIEWasStarted, VBIOWasStarted );

   //
   // Leave the GPIO bit alone
   //
   // set the appropriate interrupt mask bits (preserving GPINT bit)
   DWORD tmpImsk = (DWORD) InterruptMask ;
   InterruptMask = RISC_I | FBUS_I | OCERR_I | SCERR_I 
                   | (tmpImsk & (GPINT_I | I2CDONE_I)) ;

   if (GetVsyncTaken())
   {
      tmpImsk = (DWORD) InterruptMask ;
      InterruptMask = tmpImsk | VSYNC_I;
   }

   // Only set the triton bit if this is an 848 derivative
   if ( devId_== 0x350  || devId_== 0x351)
   {
      DebugPrint((0,"Setting triton Bit in IMASK (848)\n")) ;
      tmpImsk = (DWORD) InterruptMask ;
      InterruptMask = tmpImsk | EN_TRITON1_BUG_FIX;
   }
   else
      DebugPrint((0,"NOT Setting triton Bit in IMASK (878)\n")) ;

#endif
}


void  BtPisces::ProcessRISCIntrVBI(int idx, Field &InterruptingField)
{
   NotifyArg na(idx, (DWORD) 0 ) ;
   InterruptingField.Notify( na );
}

void  BtPisces::ProcessRISCIntr()
{
   //
   // Who is interrupting?
   // 
   int ProgCnt      = RISCS;
   int nCreated = CreatedProgs_.CountDMAProgs() ;
   IntrIdxAss *item = InterruptToIdx_ [ ProgCnt % nCreated ];
   int idx          = item->Idx;

   RiscPrgHandle ResponsibleProg = (idx == -1) ? NULL : ActiveProgs_ [idx];

   // Get out if bogus interrupt
   if (ResponsibleProg == NULL)
      return ;

   StreamInfo * thisStream = (StreamInfo*)ResponsibleProg->GetTag() ;
   Field &InterruptingField = GetField( *thisStream );

   DWORD dwTimeNow = Get_System_Time();
   extern DWORD gdwStreamStartTime ;

   DebugPrint((1,"Risc Interrupt for prog %x ndx %d at %d intr? %x\n",
                   ResponsibleProg , idx,dwTimeNow, ResponsibleProg->IsInterrupting())) ;

   //[bugbug]  HACK required since I can't easily disable slave interrupt for capture modes because
   // the flip in riscprog currently depends on whether or not the prog is set to interrupt.
   if(thisStream->GetPaired() == true && IsSlave(idx))
      return ;

   if ( ResponsibleProg && ResponsibleProg->IsInterrupting() ) 
   {
      // if this is a VBI program, deal with it in specialized function and return
      if (IsVBI(idx))
      {
         ProcessRISCIntrVBI(idx, InterruptingField); 
         return ;
      }

      // obtain video hdr from risc prog
      LPVIDEOHDR usedBuffer = ResponsibleProg->GetVideoHdr() ;
      if (!R3VidInQueue_.IsEmpty())
      {
         DebugPrint((1,"   Servicing r3queue during interrupt\n")) ;

         // - Remove r3 buffer from queue
         LPVIDEOHDR r3Buf = R3VidInQueue_.Remove() ;

         // - copy data from vxd buffer to r3 buffer
         DebugPrint((1,"   Src %x  dest %x  bufSize %d\n",
                        usedBuffer->dwReserved[0],r3Buf->dwReserved[0],r3Buf->dwBufferLength)) ;

         memcpy((PVOID)r3Buf->dwReserved[0],(PVOID)usedBuffer->dwReserved[0],r3Buf->dwBufferLength);

         // Fill out video header
         r3Buf->dwTimeCaptured = dwTimeNow - gdwStreamStartTime ;
         r3Buf->dwFlags &= ~VHDR_INQUEUE;
         r3Buf->dwFlags |= ( VHDR_DONE | VHDR_KEYFRAME );

         // - Place used vxd buffer in 'available' queue
         AvailableQueue_.Add(usedBuffer) ;
       
         // - Notify
         DebugPrint((1,"   Notify %d @ %ld:\n",idx,dwTimeNow)) ;
         NotifyArg na(idx, (DWORD) r3Buf->dwReserved[1]) ; // dwReserved[1] holds original 16:16 ring 3 addr
         InterruptingField.Notify( na );

         // instrumentation
         gBufInstrumentation.NotifyCount++ ;
         
      }
      else
      {
         // Fill out video header
         DebugPrint((1,"   Adding completed buffer to filled queue\n")) ;

         usedBuffer->dwTimeCaptured = dwTimeNow - gdwStreamStartTime ;
         usedBuffer->dwFlags &= ~VHDR_INQUEUE;
         usedBuffer->dwFlags |= ( VHDR_DONE | VHDR_KEYFRAME );

         usedBuffer->dwReserved[3] = idx ;  // needed for ProcessAddBuffer Notify call

         FilledQueue_.Add(usedBuffer) ;
      }

      LPVIDEOHDR bufToAttach ;
      // - If buffer in 'available' queue
      if (!AvailableQueue_.IsEmpty()) {
         // - Fetch next available buffer
         DebugPrint((1,"   Attaching an available buf\n")) ;

         bufToAttach = AvailableQueue_.Remove() ;
      }
      // - Else (no buffer in available queue)
      else 
      {
         DebugPrint((1,"   Attaching a filled buf (skip condition)\n")) ;

         // - Fetch buffer from filled queue
         bufToAttach = FilledQueue_.Remove() ;

         // - Increment skip count
         *pSkipCount++ ;

         // instrumentation
         gBufInstrumentation.SkippedCount++ ;

      }
      
      // - Attach buffer to risc prog
      Engine_.ChangeAddress( ResponsibleProg, bufToAttach );
      if(thisStream->GetPaired() == true)
      {
         // paired stream. Only the master prog interrupts. We need to attach the same buffer
         // to master and slave programs

         // figure out slave index
         int slaveNdx = idx - ProgsWithinField ; 

         // get handle of slave program
         RiscPrgHandle SlaveProg =  ActiveProgs_ [slaveNdx];

         // attach the buffer
         Engine_.ChangeAddress( SlaveProg, bufToAttach );
      }
   }
   DebugPrint((1,"Done with risc intr\n")) ;

}

void  BtPisces::ProcessAddBuffer( LPVIDEOHDR r3Buf )
{
   // - If there's a filled buffer available
   if(!FilledQueue_.IsEmpty())
   {
      DebugPrint((1,"ProcessAddBuffer: Servicing %x in ProcessAddBuffer\n",r3Buf)) ;
        //- critical section (already in a crit section from
        //  PsDevice::AddBuffer()
        // 
        
        //- Remove vxd buffer from filled queue
        LPVIDEOHDR usedBuffer = FilledQueue_.Remove() ;

        // grab index of risc prog which filled buffer
        // use it to get RiscProgram and Field info
        int idxRiscProg = usedBuffer->dwReserved[3] ;
        DebugPrint((1,"   idxRiscProg %d\n",idxRiscProg)) ;
        RiscPrgHandle ResponsibleProg = (idxRiscProg == -1) ? NULL : ActiveProgs_ [idxRiscProg];
        DebugPrint((1,"   ResponsibleProg %x\n",ResponsibleProg)) ;

        if (ResponsibleProg)
        {
            Field &InterruptingField = GetField( *(StreamInfo*)ResponsibleProg->GetTag() );
            DebugPrint((1,"   &Field %x\n",&InterruptingField)) ;

            // - copy data from vxd buffer to r3 buffer
            memcpy((PVOID)r3Buf->dwReserved[0],(PVOID)usedBuffer->dwReserved[0],r3Buf->dwBufferLength);

            // Fill out video header
            r3Buf->dwTimeCaptured = usedBuffer->dwTimeCaptured ;
            r3Buf->dwFlags = usedBuffer->dwFlags ;

            //- Place used vxd buffer in 'available' queue
            AvailableQueue_.Add(usedBuffer) ;
                        
            //- end critical section ( see above )
        
            // - Notify
            NotifyArg na(idxRiscProg, (DWORD) r3Buf->dwReserved[1]) ; // dwReserved[1] holds original 16:16 ring 3 addr
            InterruptingField.Notify( na );

            // instrumentation
            gBufInstrumentation.NotifyCount++ ;
        }
        else
        {
            //- Place used vxd buffer in 'available' queue
            AvailableQueue_.Add(usedBuffer) ;

            // - Add incoming buffer to r3 queue
            DebugPrint((1,"ProcessAddBuffer: Adding %x to r3queue at %u\n",r3Buf,Get_System_Time())) ;
            R3VidInQueue_.Add(r3Buf) ;
        }
   }
   // - else (no filled buffer available)
   else
   {
      // - Add buffer to r3 queue
      DebugPrint((1,"ProcessAddBuffer: Adding %x to r3queue at %u\n",r3Buf,Get_System_Time())) ;
      R3VidInQueue_.Add(r3Buf) ;
   }
}

/* Method: BtPisces::Interrupt
 * Purpose: Called by ISR to initiate the processing of an interrupt
 * Input: None
 * Output: None
*/
State BtPisces::Interrupt(bool bPowerDownInEffect)
{
   DebugPrint((2,"Bt848: Enter interrupt\n"));

   // Since the register class is not interrupt safe, we need to go directly to the 
   // hw to play with the interrupt status reg
   #define IMASK_OFFSET    0x104
   #define ISTATUS_OFFSET  0x100
   DWORD * pIMask   = (DWORD *) (RegBase::GetBaseAddress() + IMASK_OFFSET);
   DWORD * pIStatus = (DWORD *) (RegBase::GetBaseAddress() + ISTATUS_OFFSET);

   // what was the reason ?
   DWORD IntrStatus = *pIStatus;
   DWORD IntrMask   = *pIMask;

   State DidWe = Off;

   if ( IntrStatus & VSYNC_I ) {
      DebugPrint((0,"**VSYNC\n"));

      if(IntrMask & VSYNC_I) {
      *pIStatus |= VSYNC_I ;  // reset the status bit

      if (!bPowerDownInEffect)
      {
        if( VSYNC_CB_) 
        {
           DebugPrint((0,"**CallbackVSYNC\n"));
           CallbackVSYNC();
        }
        else if( VPAEC_Is_Initialized() )
        {
           // the VVL Vxd must have initialized so call the iterator
           DebugPrint((1,"**VPAEC_Iterate\n"));
           VPAEC_Iterate() ;
        }
      }

      DidWe = On;
      gIntrCntr.dwVsync ++ ;
      } 
   }

   if ( IntrStatus & RISC_I ) {
      *pIStatus |= RISC_I ;  // reset the status bit

     if ( !bPowerDownInEffect )
        ProcessRISCIntr();

      DidWe = On;
      gIntrCntr.dwRisc ++ ;
   }

   // see if need to call back GPIO
   if ( IntrStatus & GPINT_I ) {
      *pIStatus |= GPINT_I ;  // reset the status bit
      if (bPowerDownInEffect) {
         DidWe = On;
      }
      else
      if ( GPIO_CB_ ) {
         CallbackGPIO();
         DidWe = On;
      }
      gIntrCntr.dwGpio ++ ;
      DebugPrint((2,"Bt848: GPINT\n"));
   }
   // and I2C
   if ( IntrStatus & I2CDONE_I ) {
      *pIStatus |= I2CDONE_I ;  // reset the status bit
      if (bPowerDownInEffect) {
         DidWe = On;
      }
      else
      if ( I2C_CB_ ) {
         CallbackI2C();
         DidWe = On;
      }
      gIntrCntr.dwI2c ++ ;
   }

   if ( IntrStatus & FBUS_I ) {
      *pIStatus |= FBUS_I ;  // reset the status bit
      DidWe = On;
      DebugPrint((2,"Bt848: FBUS\n"));
      gIntrCntr.dwFbus ++ ;
   }
   if ( IntrStatus & FTRGT_I ) {
      DebugPrint((0,"Bt848: FTRGT\n"));
      *pIStatus |=  FTRGT_I;  // reset the status bit
      gIntrCntr.dwFtrgt ++ ;
   }
   if ( IntrStatus & FDSR_I ) {
      DebugPrint((2,"Bt848: FDSR\n"));
      *pIStatus |=  FDSR_I;  // reset the status bit
      gIntrCntr.dwFdsr ++ ;
   }
   if ( IntrStatus & PPERR_I ) {
      DebugPrint((2,"Bt848: PPERR\n"));
      *pIStatus |=  PPERR_I;  // reset the status bit
      gIntrCntr.dwPperr ++ ;
   }
   if ( IntrStatus & RIPERR_I ) {
      DebugPrint((-1,"Bt848: RIPERR\n"));
      *pIStatus |=  RIPERR_I;  // reset the status bit
      if (!bPowerDownInEffect) {
        Restart();
      }
      DidWe = On;
      gIntrCntr.dwRiperr ++ ;
   }
   if ( IntrStatus & PABORT_I ) {
      DebugPrint((2,"Bt848: PABORT\n"));
      Restart();
      *pIStatus |=  PABORT_I;  // reset the status bit
      DidWe = On;
      gIntrCntr.dwPabort ++ ;
   }
   if ( IntrStatus & OCERR_I ) {
      DebugPrint((-1,"Bt848: OCERR\n"));
      DidWe = On;
      if (!bPowerDownInEffect)
         Engine_.Stop();
      *pIStatus |=  OCERR_I;  // reset the status bit
      if (!bPowerDownInEffect)
         Restart();
      gIntrCntr.dwOcerr ++ ;
   }
   if ( IntrStatus & SCERR_I ) {
      DebugPrint((1,"Bt848: SCERR\n"));
      DidWe = On;
      *pIStatus |=  SCERR_I;  // reset the status bit
      if (!bPowerDownInEffect)
         Restart();
      gIntrCntr.dwScerr ++ ;
   }

   return DidWe;
}

// resource allocation group \\\

/* Method: BtPisces::GetNumberOfStreams
 * Purpose:
 * Input:
 * Output:
 *
*/
int  BtPisces::GetNumberOfStreams()
{
   return PiscesStreams;
}

/* Method: BtPisces::AllocateStream
 * Purpose: This function allocates a stream for use by a video channel
 * Input: StrInf: StreamInfo & - reference to the stream information structure
 * Output:
 */
ErrorCode BtPisces::AllocateStream( StreamInfo &ToAllocate )
{
   // search the allocated streams first
   if ( AllocatedStreams_.Find( &ToAllocate ) )
      return Fail;

   // brand new stream allocation. Add to the list of allocated.
   StreamInfo *ToAdd = new StreamInfo( ToAllocate.StreamNo, ToAllocate.VidStrm,
      ToAllocate.StreamNo == 0 ? (PVOID)&Odd_  :
      ToAllocate.StreamNo == 1 ? (PVOID)&Even_ :
      ToAllocate.StreamNo == 2 ? (PVOID)&VBIO_ : (PVOID)&VBIE_ );
   // do not forget to init it
   GetField( *ToAdd ).Init( ToAllocate.callback );

   // make sure we know this stream ( field ) is allocated
   AllocatedStreams_.Add( ToAdd );
   return Success;
}

/* Method: BtPisces::FreeStream
 * Purpose:
 * Input:
 * Output:
 */
void BtPisces::FreeStream( StreamInfo &ToFree )
{
//   DebugBreak();
   StreamInfo *FoundStream = AllocatedStreams_.Find( &ToFree );
   AllocatedStreams_.Detach( FoundStream );
   delete FoundStream;
}


/* Method: BtPisces::FreeStream
 * Purpose:
 * Input:
 * Output:
 *
void BtPisces::FreeStream( StreamInfo &toFree )
{
//   AllocatedStreams_.Detach( toFree );
}
*/
/* Method: BtPisces::SetBrightness
 * Purpose: Changes brightness of the captured image
 * Input:
 * Output:
 */
void BtPisces::SetBrightness( DWORD value )
{
   PsDecoder_.SetBrightness( value );
}

/* Method: BtPisces::SetSaturation
 * Purpose:
 * Input:
 * Output:
 */
void BtPisces::SetSaturation( DWORD value )
{
   PsDecoder_.SetSaturation( value );
}

/* Method: BtPisces::SetConnector
 * Purpose:
 * Input:
 * Output:
 */
void BtPisces::SetConnector( DWORD value )
{
   PsDecoder_.SetVideoInput( Connector( value ) );
}

/* Method: BtPisces::SetContrast
 * Purpose:
 * Input:
 * Output:
 */
void BtPisces::SetContrast( DWORD value )
{
   PsDecoder_.SetContrast( value );
}

/* Method: BtPisces::SetHue
 * Purpose:
 * Input:
 * Output:
 */
void BtPisces::SetHue( DWORD value )
{
   PsDecoder_.SetHue( value );
}

/* Method: BtPisces::SetSVideo
 * Purpose:
 * Input:
 * Output:
 */
void BtPisces::SetSVideo( DWORD value )
{
   PsDecoder_.SetCompositeVideo( ( value == On ) ? false : true );
}

/* Method: BtPisces::
 * Purpose:
 * Input:   value: DWORD
 * Output:
 */
void BtPisces::SetFormat( DWORD value )
{
   PsDecoder_.SetVideoFormat( VideoFormat( value ) );
   // let the scaler know format has changed
   Even_.VideoFormatChanged( VideoFormat( value ) );
   Odd_.VideoFormatChanged( VideoFormat( value ) );
}

/*
 * Method:  BtPisces::SetVScaleMode
 * Purpose: Select vertical scaling mode
 * Input:   int VSMode - 0 = non-interlace
 *                       1 = interlace
 *                       2 = field aligned (Gemini family only)
 * Output:  None
 * Return:  None
 */
void BtPisces::SetVScaleMode( DWORD value )
{
   PsDecoder_.SetVScaleMode( value );
}

/* Method: BtPisces::GetSaturation
 * Purpose:
 * Input:   pData: PLONG
 * Output:
 */
void BtPisces::GetSaturation( PDWORD pData )
{
   *pData = PsDecoder_.GetSaturation();
}

/* Method: BtPisces::GetHue
 * Purpose:
 * Input:   pData: PDWORD
 * Output:
 */
void BtPisces::GetHue( PDWORD pData )
{
   *pData = PsDecoder_.GetHue();
}

/* Method: BtPisces::GetBrightness
 * Purpose:
 * Input:   pData: PDWORD
 * Output:
 */
void BtPisces::GetBrightness( PDWORD pData )
{
   *pData = PsDecoder_.GetBrightness();
}

/* Method: BtPisces::GetSVideo
 * Purpose:
 * Input:   pData: PDWORD
 * Output:
 */
void BtPisces::GetSVideo( PDWORD pData )
{
   *pData = ( PsDecoder_.IsCompositeVideo() ? Off : On );
}

/* Method: BtPisces::GetContrast
 * Purpose:
 * Input:   pData: PDWORD
 * Output:
 */
void BtPisces::GetContrast( PDWORD pData )
{
   *pData = PsDecoder_.GetContrast();
}

/* Method: BtPisces::GetFormat
 * Purpose:
 * Input:   pData: PDWORD
 * Output:
 */
void BtPisces::GetFormat( PDWORD pData )
{
   *pData = PsDecoder_.GetVideoFormat();
}

/* Method: BtPisces::GetConnector
 * Purpose:
 * Input: pData: PDWORD
 * Output:
 */
void BtPisces::GetConnector ( PDWORD pData )
{
   *pData = PsDecoder_.GetVideoInput();
}

/* Method:  void BtPisces::GetVScaleMode( PDWORD pData )
 * Purpose: Get vertical scaling mode
 * Input:   None
 * Output:  int VSMode - 0 = non-interlace
 *                       1 = interlace
 *                       2 = field aligned (Gemini family only)
 * Return:  None
 */
void BtPisces::GetVScaleMode( PDWORD pData )
{
   *pData = PsDecoder_.GetVScaleMode();
}

// scaler group
/* Method: BtPisces::SetAnalogWindow
 * Purpose:
 * Input:
 * Output:
 */
ErrorCode BtPisces::SetAnalogWindow( TRect &r, StreamInfo &aField )
{
   return GetField(aField).SetAnalogWindow( r );
}

/* Method: BtPisces::SetDigitalWindow
 * Purpose:
 * Input:
 * Output:
 */
ErrorCode BtPisces::SetDigitalWindow( TRect &r, StreamInfo &aField )
{
   return GetField(aField).SetDigitalWindow( r );
}

// color space converter group
/* Method: BtPisces::SetPixelFormat
 * Purpose:
 * Input:
 * Output:
 */
void BtPisces::SetPixelFormat( ColFmt aFormat, StreamInfo &aField )
{
   GetField(aField).SetColorFormat( aFormat );
}

/* Method: BtPisces::GetPixelFormat
 * Purpose:
 * Input:
 * Output:
 */
ColFmt BtPisces::GetPixelFormat(StreamInfo &aField )
{
   return GetField(aField).GetColorFormat();
}


void BtPisces::TurnVFilter( State s )
{
   Even_.TurnVFilter( s );
   Odd_.TurnVFilter( s );
}

static bool EvenWasStarted, OddWasStarted, VBIEWasStarted, VBIOWasStarted;

/* Method: BtPisces::Pause()
 * Purpose: Stops Pisces ( in cases such as full-screen DOS box
 * Input: None
 * Output: None
 * Note: It may be desireable to continue capture VBI data
*/
void BtPisces::Pause()
{
   DebugPrint((DebugLevelVerbose,"Bt848: BtPisces::Pause (Paused_ = %d)\n",Paused_));
   if ( Paused_ )
      return;
   GetStarted( EvenWasStarted, OddWasStarted, VBIEWasStarted, VBIOWasStarted );

   Odd_.Stop();
   Even_.Stop();
   VBIE_.Stop();
   VBIO_.Stop();
   Engine_.Stop();
   Paused_ = true;
}

/*  Method: BtPisces::Continue()
 * Purpose: Restarts Pisces after it was paused
 * Input: None
 * Output: None
*/
void BtPisces::Continue()
{
   if ( !Paused_ )
      return;
   if ( EvenWasStarted || OddWasStarted || VBIEWasStarted || VBIOWasStarted ) {
      Restart();
      RestartStreams( EvenWasStarted, OddWasStarted, VBIEWasStarted, VBIOWasStarted );
   }
   Paused_ = false;
}

/* Method: BtPisces::SetVSYNCCB
 * Purpose: Sets the ring 0 callback for the VSYNC interrupt
 * Input: cb: DWORD, callback address
 */
void BtPisces::SetVSYNCCB( DWORD cb )
{
   DebugPrint((0,"SetVSYNCCB\n"));
   if (!GetVsyncTaken()) 
   {
     VSYNC_CB_ = cb;
     if ( cb ) {
        me_->IMASK_VSYNC = 1;
        SetVsyncTaken( true ) ;
     }
     else {
        me_->IMASK_VSYNC = 0;
        SetVsyncTaken( false ) ;
     }
   }
}

/* Method: BtPisces::SetGPIOCB
 * Purpose: Sets the ring 0 callback for the GPINT interrupt
 * Input: cb: DWORD, callback address
 */
void BtPisces::SetGPIOCB( DWORD cb )
{
   GPIO_CB_ = cb;
   if ( cb )
      me_->IMASK_GPINT = 1;
   else
      me_->IMASK_GPINT = 0;
}

/* Method: BtPisces::SetI2CCB
 * Purpose: Sets the ring 0 callback for the I2C interrupt
 * Input: cb: DWORD, callback address
 */
void BtPisces::SetI2CCB( DWORD cb )
{
   I2C_CB_ = cb;
   if ( cb )
      me_->IMASK_I2CDONE = 1;
   else
      me_->IMASK_I2CDONE = 0;
}
/* Method: BtPisces::GetStreamInfo
 * Purpose:
 * Input:
 * Output:
 */
void BtPisces::GetStreamInfo( StreamInfo &StrInf )
{
   if ( StrInf.StreamNo >= PiscesStreams )
      return;

   // have to go through all these troubles because for some reason can not
   // use array classes. Get linker and VxDLoader errors if do.
   StreamsInUseIter tmpIter( PresentStreams_ );
   while ( tmpIter ) {
      StreamInfo *CurElement = tmpIter++;
      if ( *CurElement == StrInf ) {
         StrInf = *CurElement;
         return;
      }
   }
   // notify that no good stream was found
   StrInf.VidStrm = VS_Below;
}


DWORD oddBuffAddr ;

void BtPisces::SkipProgram(int ndx)
{
#define nDO_SKIP
#ifdef DO_SKIP
  Skip(ndx) ;
#else
  Pause() ;
#endif
}

void BtPisces::UnSkipProgram(int ndx)
{
#ifdef DO_SKIP
   // get the skipper program
   RiscPrgHandle Skipper = ActiveProgs_ [ndx];

   RiscPrgHandle pUnSkipper = CreatedProgs_ [ndx];
   if ( !pUnSkipper->IsSkipped() )
      return;

   //unskip by linking the CreatedPRog in instead of the skipper
   RiscPrgHandle SkipeeParent = Skipper->GetParent();
   RiscPrgHandle SkipeeChild  = Skipper->GetChild();

   Engine_.Chain( pUnSkipper, SkipeeChild );
   // disable the riscengine
   RISC_ENABLE = 0 ;
   Engine_.Chain( SkipeeParent, pUnSkipper );
   // re-enable the risceng
   RISC_ENABLE = 1 ;


   pUnSkipper->SetSkipped(false,0);

   DebugPrint((1,"Bt848: BtPisces::UnSkipped %d \n", ndx ) );
#else
   Continue() ;
#endif
}
