/* File: Pisces.h
 * Purpose: Contains definition of the BtPisces class
 * Author: sasha             
 * $Log: PISCES.H $
 * Revision 1.54  1998/05/22 21:35:33  dtharp
 * Added queue dynamic growth implementation.
 * Revision 1.53  1998/05/14 23:40:09  dtharp
 * Merge from DT_VBI branch: VBI now functional.
 * Revision 1.51.1.3  1998/05/14 23:24:01  dtharp
 * Added VBI queue and some member functions to access it.
 * Revision 1.51.1.2  1998/05/13 16:42:14  dtharp
 * Merged graceful degradation stuff
 * Revision 1.51.1.1  1998/05/11 17:26:50  dtharp
 * Branch to restore VBI
 * Revision 1.51  1998/04/22 16:04:21  dtharp
 * Added DeallocVxdFrames()
 * Revision 1.50  1998/04/06 20:27:09  dtharp
 * NumVxdFrameBuffers is now configurable via registry key 
 * '..Video\NumVxdFrameBuffers'.
 * Revision 1.49  1998/04/06 19:42:16  dtharp
 * Vxd Frame Buffers are allocated on the fly instead of at init time.
 * Revision 1.48  1998/04/05 20:45:15  dtharp
 * Added Peek() Method to CQueue class
 * Revision 1.47  1998/04/05 20:28:35  dtharp
 * RestartFrameServer() no longer takes a parameter.
 * Revision 1.46  1998/04/05 20:03:44  dtharp
 * Removed unused flag Update_ and unnecessary counter nCreated_.
 * Revision 1.45  1998/04/03 18:34:21  dtharp
 * Added FlushR3Queue();
 * Revision 1.44  1998/04/02 21:49:39  dtharp
 * Merge DT_VIDEOHDR branch. Implements a new buffer 
 * management scheme.
 * Revision 1.43.1.5  1998/04/02 21:26:03  dtharp
 * Cleanup before merge
 * Revision 1.43.1.4  1998/04/01 21:19:49  dtharp
 * Incremental Checkin
 * Revision 1.43.1.3  1998/03/27 16:33:30  dtharp
 * Commit to new prog array and no skipper array.
 * Revision 1.43.1.2  1998/03/27 15:30:14  dtharp
 * Incremental check-in
 * Revision 1.43.1.1  1998/03/18 19:34:32  dtharp
 * Branch for passing videoheaders around
 * Revision 1.43  1998/03/12 00:35:30  dtharp
 * Renamed some parameters to reduce confusion.
 * Revision 1.42  1998/02/27 18:18:27  dtharp
 * Capture Controler constructor now takes devID as a param.
 * Revision 1.41  1998/02/24 20:57:01  rudyl
 * Merged 1.39.3.2 & 1.40.
 * Revision 1.39.3.2  1998/02/24 20:52:10  rudyl
 * Added SetVScaleMode() and GetVScaleMode().
 * Revision 1.39.3.1  1997/09/04 22:44:55  rudyl
 * Duplicate revision
 * Revision 1.40  1998/02/24 17:40:33  dtharp
 * Graft from 1.39.2.3
 * Revision 1.39.2.3  1998/02/13 00:39:53  dtharp
 * Re-worked VPAEC Init() and iterate() call conditions
 * Revision 1.39.2.2  1998/02/10 03:33:31  dtharp
 * Added VsyncTaken_ boolean in support of VVL AEC mod.
 * Revision 1.39  1997/09/04 22:44:55  dtharp
 * Merged VSYNC Callback changes
 * Revision 1.38.1.1  1997/08/21 01:40:23  dtharp
 * Branch for vsync callback
 * Revision 1.38  1997/08/08 11:18:32  dtharp
 * Fix for bug 714: Added EventArg Default Constructor.
 * Revision 1.37  1997/07/21 23:18:44  dtharp
 * Fix for Bug ID 675: Suspend/Resume IRQ problems.
 * Revision 1.36  1997/06/27 15:54:54  dtharp
 * New sentinel based buffer management scheme.  Also passes skipcount
 * down to ring 3 (instead of using the pointer method).
 * Revision 1.35  1997/05/23 00:48:41  dtharp
 * Bug ID 591: Made nSkipped_ static.
 * Revision 1.34  1997/05/02 23:17:52  dtharp
 * 848A Merge: from 1.28.1.2
 * Revision 1.33  1997/04/28 20:59:03  dtharp
 * Merged CC Primary branch from 1.31.2.2 Bug ID 475
 * Revision 1.32  1997/04/25 17:05:23  dtharp
 * Grafted DT_GPIO_INIT branch (1.31.1.1)
 * Revision 1.31.1.1  1997/04/09 00:32:16  dtharp
 * Branch for bug id 435. Added gpio initialization support from 
 * registry.
 * Revision 1.31  1997/04/07 21:40:41  dtharp
 * Grafted "new vpres" branch from 1.30.1.1. Bug Id 436.
 * Revision 1.30.1.1  1997/04/04 23:28:17  dtharp
 * Bug Id#436  Added initialization of bit 7 of register at offset 0x78. This enables 
 * new vpres circuitry.
 * Revision 1.30  1997/03/26 00:23:32  dtharp
 * Made Skippers_ array visible to dumper.
 * Revision 1.29  1997/03/20 22:07:39  dtharp
 * Grafted WDM Buffer managment branch.
 * Revision 1.28.2.4  1997/03/20 18:12:05  dtharp
 * PAL Mode Implemented.
 * Revision 1.28.2.3  1997/03/17 16:42:47  dtharp
 * Works @ 30fps NTSC.
 * Revision 1.28.2.2  1997/03/12 18:13:45  dtharp
 * Single field capture kinda works.
 * Revision 1.28  1997/02/19 21:18:06  dtharp
 * Grafted DT_PAL_CAPTURE_RATE branch (bug id 384)
 * Revision 1.27.3.1  1997/02/10 21:49:19  dtharp
 * Branch for bug #384
 * Revision 1.27  1996/10/25 00:45:25  sasha
 * Finished off the ring 0 I2C and GPIO callbacks ( enable and disable
 * respective interrupts ). 
 * Removed SetRate() member function ( not needed ).
 * Revision 1.26  1996/10/11 10:18:49  sasha
 * For the more accurate frame count: added VSYNC counter and RISCI counter.
 * Added milsec per frame member as well.
 * Revision 1.25  1996/10/02 04:46:08  sasha
 * Added members to hold addresses for the I2C and GPIO callbacks.
 * Added member functions that invoke the callbacks. Added functions
 * to set the callback addresses.
 * Revision 1.24  1996/09/29 23:22:09  sasha
 * To support 32 bit interrupt notification: added data members NotifyEvent_ 
 * and IntStatus_; functions InterruptNotifyProc() , StartIntNotification(),
 * StopIntNotification() and GetIntStatLocation().
 * Revision 1.23  1996/09/23 00:45:06  sasha
 * Added static dwPlanarAdjust_ member and function to set it. This is
 * needed for the 640x480 planar AVI capture. Added bool Update_ to
 * prevent buffer updates when starting full size capture.
 * Revision 1.22  1996/08/22 23:22:16  rudyl
 * In responds to class name changed from "Bt848Decoder" to "Decoder".
 * Revision 1.21  1996/06/14 23:57:25  sasha
 * Added Paused_ member
 * Revision 1.20  1996/05/02 21:18:47  sasha
 * Use FieldWithScaler instead of Field. Moved constructor into .cpp
 * Revision 1.19  1996/04/17 21:27:58  sasha
 * Added SetRate() function. Use decoder.h instead of 848decod.h
 * Revision 1.18  1996/04/11 18:02:04  sasha
 * Added definition for the bit 23 of the IMASK
 * Revision 1.17  1996/04/01 18:30:44  sasha
 * Use 848Decoder now. Added Pause(), Continue(). Added return type to
 * the Interrupt(). Added GetDataBuffer(). Added Get() functions for the decoder
 * Revision 1.15  1996/02/19 00:00:42  sasha
 * Use VBIField class for the vbi fields and changed constructor calls for them
 * Revision 1.14  1996/02/14 02:37:14  sasha
 * Added extra interrupt-program association array member
 * Revision 1.13  1996/02/08 23:52:41  sasha
 * Deleted SetClipping() and added a few small helper functions
 * Revision 1.12  1996/02/06 01:27:34  sasha
 * Added Restart() member function
 * Revision 1.11  1996/01/31 22:36:49  sasha
 * Changed argument to Skip function. Rearranged const definitions
 * Revision 1.10  1996/01/30 19:20:44  sasha
 * Added CallbackEvent class ( moved from vxdvidch.h ). Added new member
 * function to BtPisces to be called from a callback event
 * Revision 1.9  1996/01/22 20:38:59  sasha
 * Changed the index of the register array for the even field
 * Revision 1.8  1996/01/15 20:14:16  sasha
 * Moved MaxProgs constant to the field.h. Added 2 decoder functions. 
 * Added one more argument to the Field constructor ( capture_enable)
 * Revision 1.7  1996/01/06 02:04:29  sasha
 * Changed ColorFormat to ColFmt
 * Revision 1.6  1996/01/06 01:45:25  sasha
 * Changed to use new decoder
 * Revision 1.5  1996/01/02 19:08:12  sasha
 * Added a few registers. Added GetColorFormat()
 * Revision 1.4  1995/12/08 22:15:19  sasha
 * Changed programs order + many other little things
 * Revision 1.3  1995/11/07 00:56:34  sasha
 * Moved some code out of Interrupt function to the new function
 * ProcessBufferAtInterrupt. Idea is for some channel classes to call it,
 * for some to ignore. As a result channel is still notified about interrupt, but RISC
 * program is can be left unmodified
 * Revision 1.2  1995/11/02 21:39:36  sasha
 * First major check-in for testing
 * Revision 1.1  1995/10/07 08:40:02  sasha
 * Initial revision
 */

#ifndef __PISCES_H
#define __PISCES_H

#include "preg.h"

#ifndef __CAPCHIP_H
#include "capchip.h"
#endif

#ifndef __FIELD_H
#include "field.h"
#endif

#ifndef __DECODER_H
#include "decoder.h"
#endif

#ifndef __RISCENG_H
#include "risceng.h"
#endif

#include "dbgprint.h"

#include <listimp.h>

#ifndef CLASSLIB_VECTIMP_H
#include <classlib/vectimp.h>
#endif
//
// Code Switches
//



// the following will eventuall be it's own .h file
class CQueue
{
   private :
   DWORD head_ ;
   DWORD tail_ ;
   LPVIDEOHDR * elements_ ;
   DWORD size_ ;
   DWORD initialSize_ ;
   DWORD count_ ;

   void incQptr( DWORD & ptr ) 
   {
      ptr++ ;
      ptr %= size_ ;
   }
   void realloc(int newSize)
   {
      LPVIDEOHDR * tmp = NULL ;
      if(elements_)
         tmp = elements_ ;

      elements_ = new LPVIDEOHDR[newSize];

      if (tmp)
      {
         if (head_ < tail_)
         {
            memcpy(elements_,tmp, (head_+1) * sizeof(LPVIDEOHDR)) ;
            memcpy(elements_ + (tail_+1), tmp + tail_, (newSize - (head_+2)) * sizeof(LPVIDEOHDR)) ;
            tail_ += newSize - size_ ;
         }  
         else
            memcpy(elements_,tmp, size_ * sizeof(LPVIDEOHDR)) ;
         
         delete[] tmp ;
      }
      size_ = newSize ;
   }

   public :
   CQueue (int size): size_(size+1) ,head_ (0), tail_ (0), count_(0) , elements_(NULL), initialSize_(size+1)
   {  realloc(size_); }

   virtual ~CQueue () { delete [] elements_ ;}

   DWORD * GetCountAddress() { return &count_ ; }
   DWORD * GetSizeAddress() { return &size_ ; }

   // queue operations
   BOOL Add(LPVIDEOHDR pItem) 
   {
      BOOL st = FALSE ;
      if (!IsFull())
      {
         elements_[head_] = pItem ;
         incQptr(head_) ;
         count_++ ;
         st = TRUE ;
         // if we're now full, stretch
         if (IsFull())
         {
           realloc(size_ + 1) ;
         }
      }
      else
      {
//         DebugPrint((-1,"------------>Attempt to add to full queue<-----------\n")) ;
      }
      return st ;
   }
   LPVIDEOHDR Remove (void) 
   {
      LPVIDEOHDR ret = NULL ;
      if (!IsEmpty())
      {
         ret = elements_[tail_] ;
         incQptr(tail_) ;
         count_-- ;
      }
      else
      {
//         DebugPrint((1,"------------>Attempt to remove from empty queue<-----------\n")) ;
      }
      return ret;
   }

   LPVIDEOHDR Peek (void) 
   {
      LPVIDEOHDR ret = NULL ;
      if (!IsEmpty())
      {
         ret = elements_[tail_] ;
      }
      return ret;
   }

   void Flush(void)
   {
      head_ = tail_ = count_ = 0 ;
   }
   
   int Count (void) {return count_ ;}
   int Size (void) {return size_ -1 ;}
   BOOL IsEmpty() { return head_ == tail_; }
   BOOL IsFull() 
   { 
      DWORD tmp = head_;
      incQptr(tmp) ;
      return tmp == tail_ ; 
   } 
} ;


// Added for compatibility
typedef struct EventArg {
  PVOID pRefData_ ;
  DWORD extra_ ;
  EventArg(PVOID pRefData, DWORD extra) : pRefData_ (pRefData) , extra_(extra) {}
  EventArg() : pRefData_ (NULL) , extra_(0) {}
} ;
typedef  void  (*EVENTHANDLER)( DWORD dwCB, EventArg *ea );
extern VMHANDLE GetVMHandle();
typedef TIListImp<StreamInfo> StreamsInUse;
typedef TIListIteratorImp<StreamInfo> StreamsInUseIter;

/* Class: CallbackEvent
 * Purpose: Creates an event to call a ring 3 code ( capture driver )
 *
 */
class CallbackEvent : public VPriorityVMEvent
{
   private:
      DWORD dwCB_;
      CallbackEvent();
      EVENTHANDLER EventCallee_;

   public:
      CallbackEvent( EVENTHANDLER callee, DWORD cb, EventArg * pEa ) : dwCB_( cb ),
         EventCallee_( callee ),
         VPriorityVMEvent( GetVMHandle(), TIME_CRITICAL_BOOST, (PVOID)pEa, 0, 0 ) {}

   virtual VOID handler( VMHANDLE hVM, CLIENT_STRUCT* pRegs, PVOID pEa,
      BOOL bTimeOut );
};
// end addition


//typedef  void  (*EVENTHANDLER)( DWORD dwCB, PVOID pTag );

const PiscesStreams = 4;

const VSYNC_I = ( 1 << 1 );
const I2CDONE_I = ( 1 << 8 );
const GPINT_I   = ( 1 << 9 );
const RISC_I    = 0x00000800;  // bit 11
const FBUS_I    = 0x00001000;  // bit 12
const FTRGT_I   = 0x00002000;  // bit 13
const FDSR_I    = 0x00004000;  // bit 14
const PPERR_I   = 0x00008000;  // bit 15
const RIPERR_I  = 0x00010000;  // bit 16
const PABORT_I  = 0x00020000;  // bit 17
const OCERR_I   = 0x00040000;  // bit 18
const SCERR_I   = 0x00080000;  // bit 19
const DMAERR_I  = 0x00100000;  // bit 20

const PARITY_I = 0x00018000;  // bit 15+16

const EN_TRITON1_BUG_FIX = ( 1 << 23 );

/* Class: IntrIdxAss
 * Purpose: Used to assiciate a number program will generate with IRQ and
 *   an index in the array
 */
class IntrIdxAss
{
   public:
      int   IntNo;
      int   Idx;
      IntrIdxAss() : IntNo( 0 ), Idx( 0 ) {}
      IntrIdxAss( int key, int value ) : IntNo( key ), Idx( value ) {}
      int operator ==( const IntrIdxAss &rhs ) { return IntNo == rhs.IntNo; }
};

/* Type: IntrToIdxDict
 * Purpose: Holds values of interrupt number and array index associated
 * Note Use TArrayAsVector instead of TIArrayAsVector ( it contains pointers,
 *   after all ) because compiler insists on calling vector_new which is
 *   non-existent in the VxD libs
 */
typedef IntrIdxAss *IntrToIdxDict [12];

/* Type: ProgramsArray
 * Purpose: Defines an array that holds created programs
 * Note: Elements in the array alternate, i.e. even program, odd, even...
 */

/* the following scheme is used to store the RISC programs:
    ____
0   |VBIO|
1   |Odd |
2   |SYNC| VRO
3   |VBIE|
4   |Even|
5   |SYNC| VRE
   ---------
6    |VBIO|
7    |Odd |
8    |SYNC| VRO
9    |VBIE|
10   |Even|
11   |SYNC| VRE
*/
#define MAX_PROGS 12

class CProgArray {
private:
   DWORD dwCurrNdx_  ;
   RiscPrgHandle rpArray[MAX_PROGS] ;
public:
   CProgArray ()           { Clear() ;}
   virtual ~CProgArray ()  { } 

   // Clear sets all array elements handles to NULL
   void Clear() {
      dwCurrNdx_ = 0 ;
      memset( &rpArray[0], '\0', sizeof( rpArray ) )  ;
   }

   DWORD Size() { return MAX_PROGS; }

   // overload for accessing the array
   RiscPrgHandle & operator [] (DWORD n) { 
      n %= MAX_PROGS ; // prevent invalid accesses
      return rpArray[n] ;
   } 

   // Return the index of the given handle. Assumes that the handle
   // was given by this class and is valid and current.
   DWORD GetIndex(RiscPrgHandle hRp) ;

   //
   // Count Methods
   //

   // Returns the number of elements. Static for now.
   DWORD NumElements() { return MAX_PROGS ; }

   // Returns the number of non-null programs
   DWORD Count() ;

   // Returns the number of video programs
   DWORD CountVideoProgs() {
      DWORD n = 0 ;
      if (rpArray[1])  n++ ; 
      if (rpArray[4])  n++ ; 
      if (rpArray[7])  n++ ; 
      if (rpArray[10]) n++ ;
      return n ;
   }

   // Returns the number of skipped video programs
   DWORD CountSkippedProgs() {
      DWORD n = 0 ;
      if (rpArray[1]->IsSkipped())  n++ ; 
      if (rpArray[4]->IsSkipped())  n++ ; 
      if (rpArray[7]->IsSkipped())  n++ ; 
      if (rpArray[10]->IsSkipped()) n++ ;
      return n ;
   }


   // Returns the number of vbi programs
   DWORD CountVbiProgs() {
      DWORD n = 0 ;
      if (rpArray[0]) n++ ;
      if (rpArray[3]) n++ ;
      if (rpArray[6]) n++ ;
      if (rpArray[9]) n++ ;
      return n ;
   }

   // Returns number of programs which actually transfer data
   DWORD CountDMAProgs() {
     return CountVideoProgs() + CountVbiProgs() ;
   }

   //
   // Find Methods
   //

   // Returns the first non-null riscprogram. Returns Null if array is empty.
   RiscPrgHandle First();

   // Returns the next non-null riscprogram. Returns null if there are none left.
   RiscPrgHandle Next() ;
} ;

const VBIOStartLocation = 0;
const OddStartLocation  = 1;
const OddSyncStartLoc   = 2;
const VBIEStartLocation = 3;
const EvenStartLocation = 4;
const EvenSyncStartLoc  = 5;

const DistBetweenProgs  = 6;

const ProgsWithinField  = 3;

/* Class: BtPisces
 * Purpose: Controls the BtPisces video capture chip
 * Attributes:
 * Operations:
 * Note: Every time any of the Set functions is called, operation must stop.
 *   After all changes are done execution can resume. This means that all RISC
 *   programs are destroyed ( if they exist ), changes made, new RISC programs
 *   are created ( if needed )
*/
class BtPisces 
{
public:
         DECLARE_COLORCONTROL;
         DECLARE_INTERRUPTSTATUS;
         DECLARE_INTERRUPTMASK;
         DECLARE_CONTROL;
         DECLARE_CAPTURECONTROL;
         DECLARE_COLORFORMAT;
         DECLARE_GPIODATAIO;
         DECLARE_GPIOOUTPUTENABLECONTROL;
         DECLARE_GPIOREGISTEREDINPUTCONTROL;
         DECLARE_RES_1E;
   private:
         WORD          devId_;
         Decoder       PsDecoder_;
         DWORD         OldImageSize_ ;
         DWORD         NumVxdFrameBuffers_ ;
         StreamsInUse  AllocatedStreams_;
         StreamsInUse  PresentStreams_;

         // all possible data streams
         FieldWithScaler Even_;
         FieldWithScaler Odd_;
         VBIField        VBIE_;
         VBIField        VBIO_;

         RISCEng       Engine_;

         RISCProgram   SyncEvenEnd1_;
         RISCProgram   SyncEvenEnd2_;

         RISCProgram   SyncOddEnd1_;
         RISCProgram   SyncOddEnd2_;

         // Ring 3 queues (data requests)
         CQueue R3VidInQueue_ ;  // capture and preview
         CQueue R3VbiQueue_ ;    // VBI

         // Ring 0 queues (buffers to attach to riiscprogs)
         CQueue AvailableQueue_ ;
         CQueue FilledQueue_ ;

public:
         RISCProgram   Starter_;
         CProgArray CreatedProgs_;
         CProgArray ActiveProgs_;
private:
         IntrToIdxDict InterruptToIdx_;

         bool          Paused_;

         static bool   VsyncTaken_; 
         static DWORD  dwPlanarAdjust_;

         // data to support interupt notifications
         static HANDLE NotifyEvent_;
         static DWORD  IntStatus_;
         static DWORD  VSYNC_CB_;
         static DWORD  GPIO_CB_;
         static DWORD  I2C_CB_;
         static BtPisces *me_;

         void Init();

         void CreateSyncCodes();
         void ProcessRISCIntrVBI(int idx, Field &InterruptingField) ;
         void ProcessRISCIntr();

   protected:
      // type-unsafe method of getting field associated with a stream
      Field & GetField( StreamInfo &str ) { return *(Field*)str.tag; }
      int GetIdxFromStream( StreamInfo &aField );

      RiscPrgHandle AddProgram( StreamInfo &aStream, int NumberToAdd );
      void ProcessPresentPrograms();
      void AssignIntNumbers();
      void ProcessSyncPrograms();
      void LinkThePrograms( bool bNewStarter );
      void Restart();
      void GetStarted( bool &EvenWasStarted, bool &OddWasStarted,
         bool &VBIEWasStarted, bool &VBIOWasStarted );
      void RestartStreams( bool EvenWasStarted, bool OddWasStarted,
         bool VBIEWasStarted, bool VBIOWasStarted );

      Decoder & GetDecoder() { return PsDecoder_; }

      void PairedPause( int idx );

      void InitGPIOFromRegistry() ;
   public:
      // decoder 'set' group
      virtual void SetBrightness( DWORD value );
      virtual void SetSaturation( DWORD value );
      virtual void SetConnector ( DWORD value );
      virtual void SetContrast  ( DWORD value );
      virtual void SetHue       ( DWORD value );
      virtual void SetSVideo    ( DWORD value );
      virtual void SetFormat    ( DWORD value );
      virtual void SetVScaleMode( DWORD value );

      virtual void GetSaturation( PDWORD pData );
      virtual void GetHue       ( PDWORD pData );
      virtual void GetBrightness( PDWORD pData );
      virtual void GetSVideo    ( PDWORD pData );
      virtual void GetContrast  ( PDWORD pData );
      virtual void GetFormat    ( PDWORD pData );
      virtual VideoFormat GetFormat  (  ) {return (VideoFormat) PsDecoder_.GetVideoFormat();}
      virtual void GetConnector ( PDWORD pData );
      virtual void GetVScaleMode( PDWORD pData );

              static void SetPlanarAdjust( DWORD val ) { dwPlanarAdjust_ = val; }
              void TurnVFilter( State s );

        // scaler group
      virtual ErrorCode SetAnalogWindow( TRect &r, StreamInfo &aField );
      virtual ErrorCode SetDigitalWindow( TRect &r, StreamInfo &aField );

      // color space converter group
      virtual void SetPixelFormat( ColFmt, StreamInfo &aField );
      ColFmt GetPixelFormat( StreamInfo &aField );

      // streaming operation functions
      virtual ErrorCode Create( StreamInfo &aField );
      virtual void  Start( StreamInfo &aField, StartType st=START_SINGLE );
      virtual void  Stop( StreamInfo &aField );

              void  PauseStream( StreamInfo &aField ); // renamed

              void Pause();  // overload to pause all streams
              void  Continue(); // analog to Pause()

              State Interrupt(bool bPowerDownInEffect);
              void  ProcessAddBuffer( LPVIDEOHDR lpvh );
              void SetBufPitch( DWORD dwP, StreamInfo &aStream )
              { GetField(aStream).SetBufPitch( dwP ); }

              void SetOffset( DWORD dwO, StreamInfo &aStream )
              { GetField(aStream).SetOffset( dwO ); }


//       void SetBufQuePtr( StreamInfo &aStream, VidBufQueue *pQ )
//      { GetField(aStream).SetBufQuePtr( pQ ); }
//
//      VidBufQueue &GetCurrentQue( StreamInfo &aStream )
//      { return GetField(aStream).GetCurrentQue(); }

      // resource allocation group
      virtual int  GetNumberOfStreams();
      virtual void GetStreamInfo( StreamInfo &StrInf );
      virtual void FreeStream( StreamInfo &StrInf );
      virtual ErrorCode AllocateStream( StreamInfo &StrInf );

      DWORD   GetDataBuffer( int idx ) { return CreatedProgs_ [idx]->GetDataBuffer(); }

      // interrupt notifications group
      static  void StartIntNotification( HANDLE hEv ) { NotifyEvent_ = hEv; }
      static  void StopIntNotification();
      static  PDWORD GetIntStatLocation() { return &IntStatus_; }
      static  void SetVSYNCCB( DWORD cb );
      static  void SetGPIOCB( DWORD cb );
      static  void SetI2CCB( DWORD cb );
      static  void InterruptEventProc( DWORD, PVOID );
      static  void InterruptNotifyProc( DWORD, PVOID );
      static  void CallbackVSYNC();
      static  void CallbackI2C();
      static  void CallbackGPIO();

      // Access for CC Clipping
      RISCEng & GetRiscEngine() {return   Engine_;}
      CProgArray & GetCreatedProgs () { return CreatedProgs_;}
              void  SkipProgram(int ndx) ;
              void  UnSkipProgram(int ndx) ;

      // Misc
      static void SetVsyncTaken( bool state ) { DebugPrint((0,"VsyncTaken_ = %d\n",state)) ; VsyncTaken_ =  state ; }
      static bool GetVsyncTaken( void )       { return VsyncTaken_  ; }

      void  RestartFrameServer(DWORD dwImageSize) ;
      void  FlushR3VidInQueue(void) {R3VidInQueue_.Flush();}
      void  FlushR3VbiQueue(void)   {R3VbiQueue_.Flush();}
      CQueue &GetR3VidInQueue(void) {return R3VidInQueue_ ; }
      CQueue &GetR3VbiQueue(void) {return R3VbiQueue_ ; }
      void  DeallocVxdFrames() ;

      BtPisces(WORD devId);
      ~BtPisces();

};
void inline BtPisces::StopIntNotification()
{
   if ( NotifyEvent_ )
      VWIN32_CloseVxDHandle( NotifyEvent_ );
   NotifyEvent_ = NULL;
}

#endif
