// $Header: G:/SwDev/W95/Video/pisces/dvlpmnt/CapChip/rcs/Scotch.cpp 1.6 1998/04/14 18:42:08 rudyl Exp $
// $Log: Scotch.cpp $
// Revision 1.6  1998/04/14 18:42:08  rudyl
// VVL performs most init and all we need is to pulse the RESET pin.
// Revision 1.5  1998/04/08 17:28:32  rudyl
// VPAEC_Init() returns BOOL now.
// Revision 1.4  1998/04/02 02:42:56  rudyl
// Put QSight camera in low-power mode after init.
// Revision 1.3  1998/04/01 02:21:47  rudyl
// Added reg key to denote which GPIO data pin is connected to Scotch's RESET pin.
// Revision 1.2  1998/03/31 02:06:27  rudyl
// Fixed getting VPBusy status bug and removed lots of comments.
// Revision 1.1  1998/03/28 01:34:16  rudyl
// Initial revision


/////////////////////////////////////////////////////////////////////////////
//
//  Copyright (C) 1998 Brooktree Division, Rockwell Semiconductor Systems
//
//  Module:
//
//    Scotch.cpp
//
//  Abstract:
//
//    Class definition for BtScotch
//
/////////////////////////////////////////////////////////////////////////////

#include "device.h"
#include <vdd.h>

#include "scotch.h"

extern "C" {
  #include "vpaec.h"
}

#define BIT(n) ((DWORD) 1 << n)


//===========================================================================
// Wait until timeout expires
// Note: not to be used inside ISR
//===========================================================================
void BtScotch::WaitLoop( DWORD howlong )
{
	DWORD dwStart = Get_System_Time();
	while ( Get_System_Time() < dwStart + howlong )
		;
}
   
//===========================================================================
// Wait for a bit to be on or off in VP register with timeout
//===========================================================================
BOOL BtScotch::WaitVP( BYTE reg, BYTE bitNo, BOOL onoff, DWORD timeout )
{
   DWORD dwStart = Get_System_Time();
   while ( 1 )
   {
      //
   	// timeout?
      //
      if ( Get_System_Time() - dwStart > timeout )
			return FALSE;		       	
         
      //
    	// get register value from VP
      //
   	WORD vvlRead = PsDevice::BT848_VVL_Read_Byte( reg );
   
      //
		// wait for bit to be ON or OFF
      //
   	if ( onoff )   // wait for bit to be OFF
      {
      	if ( vvlRead & BIT( bitNo ) )
         	return TRUE;
      }
      else	// wait for bit to be OFF
		{         
      	if ( vvlRead & BIT( bitNo ) )
         	continue;
         else
         	return TRUE;
      }
   }
}


//===========================================================================
// Initialize QuartzSight Video Processor (Scotch)
//===========================================================================
BOOL BtScotch::InitVP( void )
{
	// As it turns out, don't need to check if Scotch exists since VPAEC will
   // eventually find out its nonexistence.

   // initialize connection with VVL VXD
   DWORD dwVer = VPAEC_Get_Version() ;

	if (!dwVer)
   {
      DebugPrint((0,"*** VVL VXD GetVer FAILED!  Ver = %x ***\n",dwVer)) ;
      return FALSE;
   }
   
   DebugPrint((1,"*** VVL VXD GetVer Succeeded!  Ver = %x ***\n",dwVer)) ;

	//------------------------------------------------------
   // reset VP device
	//------------------------------------------------------

   BYTE resetBit = 22;     // which GPDATA bit is VP RESET pin connected to; def is 22
   VRegistryKey vkey( PRK_LOCAL_MACHINE, "SOFTWARE\\Brooktree\\Video\\QuartzSight" );

   // make sure the key exists
   if ( vkey.lastError() == ERROR_SUCCESS )
   {
      char * keyName = "VPResetPin";
      char   key[9];
      DWORD  keyLen = 3;    // up to 2 chars + null

      // get the registry value and verify
      if ( vkey.getSubkeyValue( keyName, key, (DWORD *)&keyLen ) )
      {
         int intKey = atoi( key );
         if ( ( intKey >= 0 ) && ( intKey <= 23 ) )
            resetBit = (BYTE)intKey;
      }
   }

   RegisterDW GPOE( 0x0118, RW );
   RegField GPOEResetBit( GPOE, resetBit, 1, RW);
      
   RegisterDW GPDATA( 0x0200, RW );
   RegField GPDATAResetBit( GPDATA, resetBit, 1, RW);

   GPOEResetBit = 1;       // enable GPOE[ RESETBIT ] to allow GPDATA write

   GPDATAResetBit = 1;     // take GPDATA[ RESETBIT ] high
	WaitLoop( 100 );

   GPDATAResetBit = 0;     // take GPDATA[ RESETBIT ] low
	WaitLoop( 100 );

   GPDATAResetBit = 1;     // take GPDATA[ RESETBIT ] high
	WaitLoop( 100 );

   GPOEResetBit = 0;       // disable GPOE[ RESETBIT ]

	// VVL suggests to set to CIF for image size
   if ( !VPAEC_Init( TRUE,                             //  doAEC  
                     CIF,                              //  ImageSize
                     TRUE,                             //  Load Defaults
                     PISCES_DEVICE_ID,                 //  VxDID  
                     0x9,                              //__BT848_VVL_Read_Byte,  //  ReadServiceID 
                     0xA) )                            //__BT848_VVL_Write_Byte  //  WriteServiceID 
   	return FALSE;

   // put camera in low-power mode until being used
   VPAEC_SetParam( VP_LOW_POWER, TRUE );
   
	return TRUE;
}
       

/////////////////////////////////////////////////////////////////////////////
// Method:  BtScotch::BtScotch( WORD devId )
// Purpose: Constructor
// Input:   None
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
BtScotch::BtScotch() : Inited_( false )
{
	if ( InitVP() )
	{
   	Inited();
      DebugPrint((0,"InitializeQSVP--> Success\n")) ;
	}
	else
      DebugPrint((0,"InitializeQSVP--> Fail\n")) ;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  BtScotch::~BtScotch()
// Purpose: Destructor
// Input:   None
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
BtScotch::~BtScotch()
{
   if ( IsInited() && VPAEC_Is_Initialized() )
      VPAEC_Terminate();
}
