/* File Api.cpp
 * Purpose: Contains external api functions
 * Author: sasha
 * $Log: API.CPP $
 * Revision 2.22  1998/03/28 01:26:16  rudyl
 * Graft from 2.21.1.: Moved QuartzSight init to vxd.
 * Revision 2.21.1.1  1998/03/24 01:54:21  rudyl
 * Moved QuartzSight init to vxd.
 * Revision 2.21  1998/02/24 19:45:30  rudyl
 * Removed unused functions.
 * Revision 2.20  1998/02/24 19:36:17  rudyl
 * Graft from 2.19.1.2.
 * Revision 2.19.1.2  1998/02/24 19:35:28  rudyl
 * Added VVL's VPAEC interface.
 * Revision 2.19.1.1  1998/01/16 21:53:15  rudyl
 * Duplicate revision
 * Revision 2.18  1997/12/13 01:48:43  rudyl
 * Graft from 2.17.1.1. Added SRAM API.
 * Revision 2.17.1.1  1997/11/27 03:11:00  rudyl
 * Moved SRAM table to dialogs.dll.
 * Revision 2.17  1997/11/26 06:29:38  rudyl
 * Added support of Scotch.
 * Revision 2.16  1997/10/29 18:40:38  dtharp
 * Merged TrueView branch.
 * Revision 2.15.1.2  1997/10/29 02:19:28  rudyl
 * Added trueview.
 * Revision 2.15.1.1  1997/10/29 00:06:37  dtharp
 * TrueView Branch.
 * Revision 2.15  1997/10/21 02:01:16  rudyl
 * Added support to expected total line count by decoder.
 * Revision 2.14  1997/08/14 18:52:49  rudyl
 * Consider Bt878 family is also Bt848A since it has all the capabilities.
 * Revision 2.13  1997/05/28 16:54:34  trantq
 * Add Bt848ImageLoad & Bt848UnLoadImage.  Bug ID 431
 * Revision 2.12.1.2  1997/05/28 15:49:16  trantq
 * Add Bt848ImageLoad & Bt848UnLoadImage.  Bug ID 431
 * Revision 2.12.1.1  1997/05/21 00:46:06  trantq
 * Add feature LoadImage of DD Surface.  Bug ID 431
 * Revision 2.12  1997/05/21 00:46:06  dtharp
 * Fix for Bug ID 605: Driver needs to recognize 849 in addition to 848A.
 * Revision 2.11  1997/05/06 00:19:20  dtharp
 * 848A Merge: from 2.7.1.5
 * Revision 2.10  1997/04/16 16:28:57  rudyl
 * Merge 2.9 with 2.7.2.4:
 * Added functions to thunk down from Bt848_32.dll to capture driver.
 * Revision 2.9  1997/03/26 18:00:42  dtharp
 * Merged CC Fix branch. Enabled Closed Captioning.
 * Revision 2.8.1.1  1997/03/25 23:48:09  dtharp
 * Re-enabled  Closed Captioning.
 * Revision 2.8  1997/03/21 02:20:49  dtharp
 * HACK TO DISABLE CC!!
 * Revision 2.7.2.4  1997/03/27 03:13:17  rudyl
 * Modified all fcns in 32 bit dll to thunk to capture driver.
 * Revision 2.7.2.3  1997/03/26 18:20:50  rudyl
 * Added all functions in capture driver available to Bt848_32.dll using thunking.
 * Revision 2.7.2.2  1997/03/20 23:32:53  rudyl
 * Added thunk.
 * Revision 2.7.2.1  1997/03/20 21:43:19  rudyl
 * Thunk from 32 bit dll to capture driver.
 * Revision 2.7  1996/10/02 17:14:20  rudyl
 * Bug #353
 * Problem: Driver depends upon direct draw files
 * Solution: Added "SetPreviewOnly( bool par )" and "IsPreviewOnly()" in VfWChannel class
 * so that when driver is loading, Bt848ddi.dll is not linked in if registry key "PreviewOnly" = 1.
 * Also prevent calling DDI...() in this case.
 * Revision 2.6  1996/10/02 04:27:13  sasha
 * Added SetCCCallback() API and cleaned up compiler warnings.
 * Revision 2.5  1996/09/07 01:15:24  rudyl
 * Added checking of NULL ptrs everywhere.
 * Revision 2.4  1996/08/07 22:57:41  sasha
 * Added call to the DDI_SetVidFormat() in the API that sets video format
 * Revision 2.3  1996/06/26 21:27:08  sasha
 * Added GetOverlayFlags() API
 * Revision 2.2  1996/06/26 19:07:28  sasha
 * Added Pause() and Continue() functions that freeze/unfreeze Pisces
 * Revision 2.1  1996/06/10 20:12:54  ssm
 * MarkBeta1AsRev2
 * Revision 1.11  1996/06/05 19:51:26  sasha
 * Added functions for getting the CC file name and alert token
 * Revision 1.10  1996/06/05 04:10:44  sasha
 * Added functions to set CC file name and alert token
 * Revision 1.9  1996/05/30 21:20:27  sasha
 * Added SetCCStatus() and GetCCStatus()
 * Revision 1.8  1996/05/08 19:08:28  sasha
 * Fixed hue setting ( changed limit chex )
 * Revision 1.7  1996/04/17 22:47:18  sasha
 * Removed range checks.
 * Revision 1.6  1996/04/01 21:11:58  sasha
 * Changed name again
 * Revision 1.5  1996/04/01 20:55:38  sasha
 * Changed names to reflect new Connector names
 * Revision 1.4  1996/03/29 18:11:10  sasha
 * Changed video format names ( as per viddefs.h )
 * Revision 1.3  1996/01/10 21:32:48  sasha
 * Changed prototypes and started using decoder constants
 * Revision 1.2  1996/01/04 23:11:56  sasha
 * Changed SetConnector() to use BOOL instead of bool
 * Revision 1.1  1995/09/07 00:29:56  sasha
 * Initial revision
 */

#include "extvidin.h"
#include "exvidout.h"
#include "vidinchn.h"
#include "api.h"


extern int  HWRevID;
extern bool HWisBt848Aor849;
extern bool HWisBt878Family;

int WINAPI _export GetHWRevisionID( void )
{
   return ( HWRevID );
}

BOOL WINAPI _export IsABt848A( void )
{
   return ( (BOOL)HWisBt848Aor849 || (BOOL)HWisBt878Family );
}

BOOL WINAPI _export IsBt878Family( void )
{
   return ( (BOOL)HWisBt878Family );
}

void WINAPI _export SetCCCallback( CC_CALLBACK cb )
{
   CCEngine::SetCCProcessor( cb );
}

ErrorCode WINAPI _export SetConnector( Connector AConn, BOOL ApplyImm )
{
   if ( !IsABt848A() && AConn == ConCamera )
      return Err_WrongHardware;
   if ( AConn > ConComposite )
      return Fail;
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return Fail;
   ExtVidInChan::GetExtVideoChan().SetConnector( AConn, bool( ApplyImm ) );
   return Success;
}

ErrorCode WINAPI _export SetSVideo( State aState )
{
   if ( aState > On )
      return Fail;
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return Fail;
   ExtVidInChan::GetExtVideoChan().SetSVideo( aState );
   return Success;
}

ErrorCode _export WINAPI SetBrightness( int percentOfFullScale )
{
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return Fail;
   ExtVidInChan::GetExtVideoChan().SetBrightness( percentOfFullScale );
   return Success;
}

ErrorCode WINAPI _export SetContrast( int percentOfFullScale )
{
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return Fail;
   ExtVidInChan::GetExtVideoChan().SetContrast( percentOfFullScale );
   return Success;
}

ErrorCode _export WINAPI SetHue( int degrees )
{
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return Fail;
   ExtVidInChan::GetExtVideoChan().SetHue( LimitRange( degrees, 0, 255 ) );
   return Success;
}

ErrorCode WINAPI _export SetSaturation( int percentOfFullScale )
{
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return Fail;
   ExtVidInChan::GetExtVideoChan().SetSaturation( percentOfFullScale );
   return Success;
}

extern "C" void DDI_SetVidSize( DWORD, DWORD );

ErrorCode WINAPI _export SetVideoFormat( VideoFormat aFormat )
{
   if ( !IsABt848A() && aFormat == VFormat_PAL_NC )
      return Err_WrongHardware;
   if ( aFormat > VFormat_PAL_NC )
      return Fail;
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return Fail;
   ExtVidInChan::GetExtVideoChan().SetVideoFormat( aFormat );

   // don't call DDI_...() if we are using Preview mode only;
   // i.e. no overlay channel is created and Bt848ddi.dll is not linked in
   if ( ExtVidInChan::GetExtVideoChan().IsPreviewOnly() == false )
   {
      int width  = ( aFormat == VFormat_PAL_NC ) ? 640 : IsNTSCCompatible( aFormat ) ? 640 : 768;
      int height = ( aFormat == VFormat_PAL_NC ) ? 576 : IsNTSCCompatible( aFormat ) ? 480 : 576;
      DDI_SetVidSize( width, height );
   }
   return Success;
}

ErrorCode WINAPI _export SetImage( LPRECT lpRect, ColFmt aFormat )
{
   if ( &VidInChan::GetVidInChan() == NULL )
      return Fail;
   return VidInChan::GetVidInChan().SetImage( lpRect, aFormat );
}

ErrorCode WINAPI _export SetCCStatus( int st )
{
   if ( &ExtVidOutChan::GetExtVidOut() == NULL )
      return Fail;
   return ExtVidOutChan::GetExtVidOut().SetCCStatus( st );
}

void WINAPI _export SetCCFileName( LPCSTR name )
{
   if ( &ExtVidOutChan::GetExtVidOut() == NULL )
      return;
   ExtVidOutChan::GetExtVidOut().SetCCFileName( name );
}

void WINAPI _export SetCCAlert( LPCSTR token )
{
   if ( &ExtVidOutChan::GetExtVidOut() == NULL )
      return;
   ExtVidOutChan::GetExtVidOut().SetCCAlert( token );
}

// 'get' group
Connector WINAPI _export GetConnector()
{
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return Connector( 0 );
   return ExtVidInChan::GetExtVideoChan().GetConnector();
}

State WINAPI _export GetSVideo()
{
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return Off;
   return ExtVidInChan::GetExtVideoChan().GetSVideo();
}

int WINAPI _export GetBrightness()
{
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return 0;
   return ExtVidInChan::GetExtVideoChan().GetBrightness();
}

int WINAPI _export GetContrast()
{
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return 0;
   return ExtVidInChan::GetExtVideoChan().GetContrast();
}

int WINAPI _export GetHue()
{
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return 0;
   return ExtVidInChan::GetExtVideoChan().GetHue();
}

int WINAPI _export GetSaturation()
{
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return 0;
   return ExtVidInChan::GetExtVideoChan().GetSaturation();
}

VideoFormat WINAPI _export GetVideoFormat()
{
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return VFormat_AutoDetect;
   return ExtVidInChan::GetExtVideoChan().GetVideoFormat();
}

int  WINAPI _export GetImageFormat( ColFmt *pColFrmt )
{
   if ( &VidInChan::GetVidInChan() == NULL )
      return 0;
   return VidInChan::GetVidInChan().GetImageFormat( pColFrmt );
}

void WINAPI _export GetImageSize( LPRECT lpR )
{
   if ( &VidInChan::GetVidInChan() == NULL )
      return;
   VidInChan::GetVidInChan().GetImageRect( lpR );
}

int WINAPI _export GetCCStatus( void )
{
   if ( &ExtVidOutChan::GetExtVidOut() == NULL )
      return 0;
   return ExtVidOutChan::GetExtVidOut().GetCCStatus();
}

void WINAPI _export GetCCFileName( LPSTR name, int cnt )
{
   if ( &ExtVidOutChan::GetExtVidOut() == NULL )
      return;
   ExtVidOutChan::GetExtVidOut().GetCCFileName( name, cnt );
}

void WINAPI _export GetCCAlert( LPSTR token, int cnt )
{
   if ( &ExtVidOutChan::GetExtVidOut() == NULL )
      return;
   ExtVidOutChan::GetExtVidOut().GetCCAlert( token, cnt );
}

DWORD WINAPI _export GetStatus()
{
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return 0;
   return ExtVidInChan::GetExtVideoChan().GetStatus();
}

void WINAPI _export Pause()
{
   if ( &ExtVidOutChan::GetExtVidOut() == NULL )
      return;
   ExtVidOutChan::GetExtVidOut().Pause();
}

void WINAPI _export Continue()
{
   if ( &ExtVidOutChan::GetExtVidOut() == NULL )
      return;
   ExtVidOutChan::GetExtVidOut().Continue();
}

DWORD WINAPI _export GetOverlayFlags()
{
   if ( &ExtVidOutChan::GetExtVidOut() == NULL )
      return 0;
   return ExtVidOutChan::GetExtVidOut().GetOverlayFlags();
}

ErrorCode WINAPI _export ImageLoad( HINSTANCE hInst, HWND hWnd, LPCSTR szBitmap )
{
    if (&ExtVidOutChan::GetExtVidOut() == NULL )
        return Fail;
    
    return ExtVidOutChan::GetExtVidOut().ImageLoad( hInst, hWnd, szBitmap );
}    

ErrorCode WINAPI _export UnLoadImage( )
{
    if (&ExtVidOutChan::GetExtVidOut() == NULL )
        return Fail;
    
    return ExtVidOutChan::GetExtVidOut().UnLoadImage( );
}    

/////////////////////////////////////////////////////////////////////////////
// 'set' group
/////////////////////////////////////////////////////////////////////////////

// Select signal format
ErrorCode WINAPI _export SetSignalFormat( SignalFormat sFormat )
{
   if ( !IsABt848A() )
      return Err_WrongHardware;
   if ( ( sFormat > Signal_ExtHSYNC_FIELD ) || ( sFormat == Signal_Reserved3 ) )
      return Fail;
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return Fail;
   ExtVidInChan::GetExtVideoChan().SetSignalFormat( sFormat );
   return Success;
}


// selects HS/VS aligned with Cb, Y0, Cr, or Y1
ErrorCode WINAPI _export SetSyncVideoRef( SyncVideoRef syncRef )
{
   if ( !IsABt848A() )
      return Err_WrongHardware;
   if ( syncRef > VRef_alignedY1 )
      return Fail;
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return Fail;
   ExtVidInChan::GetExtVideoChan().SetSyncVideoRef( syncRef );
   return Success;
}

// Enable/Disable sync output for sync video input
ErrorCode WINAPI _export SetEnableSyncOutput( State state )
{
   if ( !IsABt848A() )
      return Err_WrongHardware;
   if ( state > On )
      return Fail;
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return Fail;
   ExtVidInChan::GetExtVideoChan().SetEnableSyncOutput( state );
   return Success;
}

// Select decoder input clock
ErrorCode WINAPI _export SetDecInputClock( DecInputClk iClk )
{
   if ( !IsABt848A() )
      return Err_WrongHardware;
   if ( iClk > DecClk_invGPCLK )
      return Fail;
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return Fail;
   ExtVidInChan::GetExtVideoChan().SetDecInputClock( iClk );
   return Success;
}

// Select GPCLK output clock
ErrorCode WINAPI _export SetOutputClock( GPCLKOutputClk oClk )
{
   if ( !IsABt848A() )
      return Err_WrongHardware;
   if ( oClk > GPCLK_invPLL )
      return Fail;
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return Fail;
   ExtVidInChan::GetExtVideoChan().SetGPCLKOutputClock( oClk );
   return Success;
}

// set timing generator load byte
ErrorCode WINAPI _export SetTimGenByte( BYTE * tgb, int size )
{
   if ( !IsABt848A() )
      return Err_WrongHardware;
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return Fail;
   ExtVidInChan::GetExtVideoChan().SetTimGenByte( tgb, size );
   return Success;
}

// select 64, 48, 32, or 16 CLKx1 for HReset
ErrorCode WINAPI _export SetHReset( int clk )
{
   if ( !IsABt848A() )
      return Err_WrongHardware;
   if ( ( clk != 64 ) && ( clk != 48 ) && ( clk != 32 ) && ( clk != 16 ) )
      return Fail;
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return Fail;
   ExtVidInChan::GetExtVideoChan().SetHReset( clk );
   return Success;
}

// set PLL to desired synthesized frequency
ErrorCode WINAPI _export SetPLL( PLLFreq pll )
{
   if ( !IsABt848A() )
      return Err_WrongHardware;
   if ( pll >= PLL_Limit )    // PLL_Limit is sentinel value
      return Fail;
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return Fail;
   ExtVidInChan::GetExtVideoChan().SetPLL( pll );
   return Success;
}

// set video parameters for scaling purpose
ErrorCode WINAPI _export SetVideoInfo( VSIZEPARMS vsize )
{
   if ( !IsABt848A() )
      return Err_WrongHardware;
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return Fail;
   ExtVidInChan::GetExtVideoChan().SetVideoInfo( vsize );

   // don't call DDI_...() if we are using Preview mode only;
   // i.e. no overlay channel is created and Bt848ddi.dll is not linked in
   if ( ExtVidInChan::GetExtVideoChan().IsPreviewOnly() == false )
      DDI_SetVidSize( vsize.HActive, vsize.VActive );
   return Success;
}

// Set expected total line count per frame for the decoder
ErrorCode WINAPI _export SetTotalLineCount( int lines )
{
   if ( !IsBt878Family() )
      return Err_WrongHardware;
   if ( ( lines < 0 ) || ( lines > 0x3FF ) )    // max 10 bits
      return Fail;
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return Fail;
   ExtVidInChan::GetExtVideoChan().SetTotalLineCount( lines );
   return Success;
}


/////////////////////////////////////////////////////////////////////////////
// 'get' group
/////////////////////////////////////////////////////////////////////////////

// gets signal format
SignalFormat WINAPI _export GetSignalFormat()
{
   if ( !IsABt848A() )
      return SignalFormat( 0 );
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return SignalFormat( 0 );
   return ExtVidInChan::GetExtVideoChan().GetSignalFormat();
}

// gets HS/VS is aligned with either Cb, Y0, Cr, or Y1
SyncVideoRef WINAPI _export GetSyncVideoRef( void )
{
   if ( !IsABt848A() )
      return SyncVideoRef( 0 );
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return SyncVideoRef( 0 );
   return ExtVidInChan::GetExtVideoChan().GetSyncVideoRef();
}

// gets decoder input clock
DecInputClk WINAPI _export GetDecInputClock( void )
{
   if ( !IsABt848A() )
      return DecInputClk( 0 );
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return DecInputClk( 0 );
   return ExtVidInChan::GetExtVideoChan().GetDecInputClock();
}

// Select GPCLK output clock
GPCLKOutputClk WINAPI _export GetOutputClock( void )
{
   if ( !IsABt848A() )
      return GPCLKOutputClk( 0 );
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return GPCLKOutputClk( 0 );
   return ExtVidInChan::GetExtVideoChan().GetGPCLKOutputClock();
}

// checks if sync output for sync video input is enabled or disabled
State WINAPI _export GetEnableSyncOutput( void )
{
   if ( !IsABt848A() )
      return Off;
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return Off;
   return ExtVidInChan::GetExtVideoChan().GetEnableSyncOutput();
}

// gets timing generator load byte
ErrorCode WINAPI _export GetTimGenByte( BYTE * tgb, int * size )
{
   if ( !IsABt848A() )
      return Fail;
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return Fail;
   return ExtVidInChan::GetExtVideoChan().GetTimGenByte( tgb, size );
}

// Get HRESET length (HSFMT)
int WINAPI _export GetHReset( void )
{
   if ( !IsABt848A() )
      return Off;
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return Off;
   return ExtVidInChan::GetExtVideoChan().GetHReset();
}

// obtains the settings of PLL registers
PLLFreq WINAPI _export GetPLL( void )
{
   if ( !IsABt848A() )
      return PLLFreq( 0 );
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return PLLFreq( 0 );
   return ExtVidInChan::GetExtVideoChan().GetPLL();
}

// obtains video parameters for scaling purpose
ErrorCode WINAPI _export GetVideoInfo( VSIZEPARMS * vsize )
{
   if ( !IsABt848A() )
      return Fail;
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return Fail;
   ExtVidInChan::GetExtVideoChan().GetVideoInfo( vsize );
   return Success;
}

// obtains expected total line count per frame for the decoder
int WINAPI _export GetTotalLineCount( void )
{
   if ( !IsBt878Family() )
      return Off;
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return Off;
   return ExtVidInChan::GetExtVideoChan().GetTotalLineCount();
}

ErrorCode WINAPI _export ResetFieldCounter( void )
{
   if ( !IsABt848A() )
      return Err_WrongHardware;
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return Fail;
   ExtVidInChan::GetExtVideoChan().ResetFieldCounter();
   return Success;
}

int WINAPI _export GetFieldCounter( void )
{
   if ( !IsABt848A() )
      return -1;
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return -1;
   return ExtVidInChan::GetExtVideoChan().GetFieldCounter();
}

ErrorCode WINAPI _export SetInterlaceMode( int mode )
{
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return Fail ;
   return ExtVidInChan::GetExtVideoChan().SetInterlaceMode( mode );
}

int WINAPI _export GetInterlaceMode( void )
{
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return Fail ;
   return ExtVidInChan::GetExtVideoChan().GetInterlaceMode();
}

/////////////////////////////////////////////////////////////////////////////
// QuartzSight group
/////////////////////////////////////////////////////////////////////////////

ErrorCode WINAPI _export SetQuartzSightMode( BOOL state )
{
   if ( !IsBt878Family() )
      return Fail;
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return Fail ;
   return ExtVidInChan::GetExtVideoChan().SetQuartzSightMode( state );
}

BOOL WINAPI _export GetQuartzSightMode( void )
{
   if ( !IsBt878Family() )
      return FALSE;
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return FALSE;
   return ExtVidInChan::GetExtVideoChan().GetQuartzSightMode();
}

// Using VVL's VPAEC to communicate with QuartzSight camera controller chip.
ErrorCode WINAPI _export SetQuartzSightParam( BYTE param, BYTE value )
{
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return Fail ;
   return ExtVidInChan::GetExtVideoChan().SetQuartzSightParam( param, value ) ;
}

ErrorCode WINAPI _export GetQuartzSightParam( BYTE param, BYTE * pValue )
{
   if ( !IsABt848A() )
      return Fail;
   if ( &ExtVidInChan::GetExtVideoChan() == NULL )
      return Fail;
   return ExtVidInChan::GetExtVideoChan().GetQuartzSightParam( param, pValue );
}


//=====================================================================
// The following are functions provided for thunking from 32 bit dll
// to capture driver
//=====================================================================
extern "C" int DRV_SetVideoSource( int conn )
{
   return SetConnector( (Connector)conn, TRUE );
}

extern "C" int DRV_GetVideoSource( void )
{
   return GetConnector();
}

extern "C" int DRV_SetBrightness( int brightness )
{
   return SetBrightness( brightness );
}

extern "C" int DRV_GetBrightness( void )
{
   return GetBrightness();
}

extern "C" int DRV_SetContrast( int contrast )
{
   return SetContrast( contrast );
}

extern "C" int DRV_GetContrast( void )
{
   return GetContrast();
}

extern "C" int DRV_SetHue( int hue )
{
   return SetHue( hue );
}

extern "C" int DRV_GetHue( void )
{
   return GetHue();
}

extern "C" int DRV_SetSaturation( int sat )
{
   return SetSaturation( sat );
}

extern "C" int DRV_GetSaturation( void )
{
   return GetSaturation();
}

extern "C" DWORD DRV_GetStatus( void )
{
   return GetStatus();
}


extern "C" int WINAPI DRV_SetSVideo( int state )
{
   return SetSVideo( (State)state );
}

extern "C" int WINAPI DRV_GetSVideo( void )
{
   return GetSVideo();
}

extern "C" int WINAPI DRV_SetVideoFormat( int vFormat )
{
   return SetVideoFormat( (VideoFormat)vFormat );
}

extern "C" int WINAPI DRV_GetVideoFormat( void )
{
   return GetVideoFormat();
}

extern "C" void WINAPI DRV_SetCCCallback( LPVOID cb )
{
   SetCCCallback( (CC_CALLBACK)cb );
}

extern "C" int WINAPI DRV_SetCCStatus( int status )
{
   return SetCCStatus( status );
}

extern "C" void WINAPI DRV_SetCCFileName( LPSTR name )
{
   SetCCFileName( (LPCSTR)name );
}

extern "C" void WINAPI DRV_SetCCAlert( LPSTR token )
{
   SetCCAlert( (LPCSTR)token );
}

extern "C" void WINAPI DRV_GetCCFileName( LPSTR name, int cnt )
{
   GetCCFileName( name, cnt );
}

extern "C" void WINAPI DRV_GetCCAlert( LPSTR token, int cnt )
{
   GetCCAlert( token, cnt );
}

extern "C" int WINAPI DRV_GetCCStatus( void )
{
   return GetCCStatus();
}

extern "C" int WINAPI DRV_SetImage( LPRECT lpRect, int aFormat )
{
   return SetImage( lpRect, (ColFmt)aFormat );
}

extern "C" int WINAPI DRV_GetImageFormat( int *pColFrmt )
{
   return GetImageFormat( (ColFmt *)pColFrmt );
}

extern "C" void WINAPI DRV_GetImageSize( LPRECT lpR )
{
   GetImageSize( lpR );
}

extern "C" void WINAPI DRV_Pause( void )
{
   Pause();
}

extern "C" void WINAPI DRV_Continue( void )
{
   Continue();
}

extern "C" DWORD WINAPI DRV_GetOverlayFlags( void )
{
   return GetOverlayFlags();
}

extern "C" DWORD WINAPI DRV_ImageLoad( HINSTANCE hInst, HWND hWnd, LPSTR szBitmap )
{
   return ImageLoad( hInst, hWnd, (LPCSTR)szBitmap );
}

extern "C" DWORD WINAPI DRV_UnLoadImage( void )
{
   return UnLoadImage( );
}

