/* File: Drvproc.cpp
 * Purpose: Contains DriverProc entry point
 * Author: sasha
 * $Log: DRVPROC.CPP $
 * Revision 2.26  1998/02/27 18:15:03  dtharp
 * Added VIA compatibility code. Added Triton Bit flip if 878 series. Added
 * 2 new SIS checks.
 * Revision 2.25  1998/01/09 17:07:57  peter
 * Graft Trident fixes to tip.
 * Revision 2.24.1.1  1998/01/07 22:40:32  peter
 * Added fns to detect trident 9685 and to enable/disable its graphics engine.
 * Revision 2.24  1997/12/09 22:17:15  peter
 * Removed global instance of RegUtil.
 * Revision 2.23  1997/12/03 21:55:10  peter
 * Fixed regutil 16/32-bit incompatibility.
 * Revision 2.22  1997/11/26 01:45:32  peter
 * Fixed MKS revision problem from graft.
 * Revision 2.20.3.1  1997/11/20 22:32:42  peter
 * Changes to move registry subkey to HKEY_LOCAL_MACHINE.
 * Revision 2.20  1997/09/16 20:09:03  dtharp
 * Changes requrired for BtPciVid.drv name change to BtVid.drv.
 * Revision 2.19  1997/09/11 00:09:53  dtharp
 * Mods required for name change of Bt848_32.dll to BtVid_32.dll.
 * Revision 2.18  1997/09/09 19:36:32  dtharp
 * Mods for Dialogs DLL name change
 * Revision 2.17.1.1  1997/09/04 21:28:42  dtharp
 * Revision 2.17  1997/09/04 15:49:50  dtharp
 * Mods required to change Bt848ddi.dll to BtDDI.DLL.
 * Revision 2.16  1997/09/03 17:52:28  dtharp
 * Mods required for name change of Bt848.drv to BtPciVid.drv.
 * Revision 2.15  1997/08/14 18:55:05  rudyl
 * Consider Bt878 family is also Bt848A since it has all the capabilities.
 * Revision 2.14  1997/05/27 21:25:25  dtharp
 * Bug Id 592.  Added SIS PCI Bridge chip (5596, 5511) compatibility.
 * Revision 2.13  1997/05/22 18:35:55  dtharp
 * Bug id 607: 849 now passes as an 848A. Also merged SIS fix 
 * (bug ID 592) but this is dormant (disabled) until we get revIds
 * for the scummy SIS chips.
 * Revision 2.12  1997/05/21 00:43:02  dtharp
 * Fiix for Bug ID 605: Merged code needs to recognize 849
 * Revision 2.11  1997/05/03 00:11:38  dtharp
 * 848A Merge: from 2.8.1.3
 * Revision 2.10  1997/04/16 16:55:05  rudyl
 * Didn't graft correctly last time. Do it again.
 * Revision 2.9  1997/04/16 16:49:23  rudyl
 * Graft from 2.8.2.4:
 * Added functions to thunk down from Bt848_32.dll to capture driver.
 * Revision 2.8.2.4  1997/04/15 22:40:58  rudyl
 * Removed DRV_Initialize() and DRV_Disconnect() functions.
 * Revision 2.8.2.3  1997/03/26 18:20:52  rudyl
 * Added all functions in capture driver available to Bt848_32.dll using thunking.
 * Revision 2.8.2.2  1997/03/20 23:33:16  rudyl
 * Added thunk.
 * Revision 2.8.2.1  1997/03/20 20:58:57  rudyl
 * Add thunk from 32 bit dll to capture drvier.
 * Revision 2.8  1996/10/10 17:28:13  dtharp
 * Removed reference to api_cc.h
 * Revision 2.7  1996/10/03 16:33:27  dtharp
 * All calls into cc.dll now go through functions in api_cc.cpp. 
 * Revision 2.6  1996/10/02 22:12:11  dtharp
 * Tidied up code block per Alex's request
 * Revision 2.5  1996/10/02 20:08:23  dtharp
 * Changes required to load the newly separated Closed Caption
 * DLL and initialize the api pointers into this dll.
 * Revision 2.4  1996/10/02 17:10:00  rudyl
 * Bug #353
 * Problem: Driver depends upon direct draw files
 * Solution: Added "SetPreviewOnly( bool par )" and "IsPreviewOnly()" in VfWChannel class
 * so that when driver is loading, Bt848ddi.dll is not linked in if registry key "PreviewOnly" = 1.
 * Also prevent calling DDI...() in this case.
 * Revision 2.3  1996/08/07 18:49:07  sasha
 * Set SingleKeyColor_ to true in case of Cirrus driver
 * Revision 2.2  1996/07/19 01:49:19  sasha
 * In high resolution mode, 1280x1024x256 colors, dialog which displays Bt848 is busy removed.
 * Revision 2.1  1996/06/10 20:12:57  ssm
 * MarkBeta1AsRev2
 * Revision 1.13  1996/05/30 21:17:47  sasha
 * Do not support config dialog. GetProcAddress of a single video dialog
 * function ( dispatcher ) and call it instead of specific entry points
 * Revision 1.12  1996/05/29 22:08:02  sasha
 * Added code to call display driver's function to disable PCI retry and screen blits
 * Revision 1.11  1996/05/17 20:29:17  sasha
 * Moved functions around in the file. Added Disconnect() and Initialize() calls
 * to the dialogs DLL
 * Revision 1.10  1996/05/17 02:18:45  sasha
 * Load dialogs DLL explicitly now.
 * Revision 1.9  1996/04/30 16:36:02  sasha
 * Changed names in the thunking call. Removed unused code
 * Revision 1.8  1996/04/09 01:14:30  sasha
 * Removed reference to the debug.h file
 * Revision 1.7  1996/03/15 22:24:58  sasha
 * No change, really
 * Revision 1.6  1996/03/07 22:31:42  sasha
 * Changed _WinAllocFlag to DWORD.
 * Revision 1.5  1996/02/14 02:33:50  sasha
 * Return OK for the Close message all the time
 * Revision 1.4  1996/01/22 20:23:42  sasha
 * Changed Config() prototype
 * Revision 1.3  1996/01/10 21:31:50  sasha
 * Added code to avoid the same error dialogs to be displayed more the once
 * Added more error codes to the exception switch
 * Revision 1.2  1996/01/04 23:16:19  sasha
 * Added thunking code.
 * Added code to connect to the dialogs DLL
 * Revision 1.1  1995/09/06 03:33:58  sasha
 * Initial revision
*/
#include <windows.h>
#include <mmsystem.h>
#include <msvideo.h>
#include <msviddrv.h>
#include <stdio.h>

const DWORD BOGUS_DRIVER_ID = 1L;
const LRESULT DRV_RET_OK    = 1L;

#define DRV_NOTIFYMSG	      0x0010

#include "vfwchan.h"
#include "dialogs.h"
#include "cc_api.h"
#include "pci_io.h"
#include "dbgprint.h"

#include "regutil.h"


const int BtID        = 0x109E;           // Brooktree manufacturer ID
const int SISId       = 0x1039;           // SIS manufacturer ID
const int VIAId       = 0x1106;           // VIA manufacturer ID
const int Bt848devID  = 0x0350;           // Bt848 DeviceId
const int Bt849devID  = 0x0351;           // Bt849 DeviceId
const int Bt878devID  = 0x036E;           // Bt878 DeviceId
const int Bt879devID  = 0x036F;           // Bt879 DeviceId
const int Bt880devID  = 0x0370;           // Bt880 DeviceId

int  HWRevID;                             // hardware revision ID
bool HWisBt848Aor849;                     // Is it a Bt848A or Bt849 chip?
bool HWisBt878Family;                     // Is it a Bt878 family chip?

extern DWORD _cdecl __WinAllocFlag = GPTR | GMEM_SHARE;
static HINSTANCE ghModule;

#define TRIDENT_ESCAPE_FN_ID     0x00001995
#define ZDDI_SUB_GE_DISABLE      0x80000001
#define ZDDI_SUB_GE_ENABLE       0x80000002

BOOL IsATrident9685()
{
   PCI_IO      pci( 0x1023 );
   if( pci.FindDevice( 0x9660 ) )
   {
      DebugPrint(( 0, "Trident 9685 found\n" )) ;
      return TRUE;
   }
   else
   {
      DebugPrint(( 0, "Trident 9685 not found\n" )) ;
      return FALSE;
   }
}


BOOL EnableTridentGE( BOOL bEnable ) 
{

   if( !IsATrident9685() )
   {
      return FALSE;
   }

   HDC         hDC = GetDC( GetDesktopWindow() );
   DWORD       dwData;
   BOOL        bRetVal = FALSE;

   if( bEnable )
   {
      DebugPrint(( 0, "attempt to enable trident ge escape\n" )) ;
      dwData = ZDDI_SUB_GE_ENABLE;
   }
   else
   {
      DebugPrint(( 0, "attempt to disable trident ge escape\n" )) ;
      dwData = ZDDI_SUB_GE_DISABLE;
   }

   if( Escape( hDC, (int)TRIDENT_ESCAPE_FN_ID, sizeof(DWORD), (LPCSTR)&dwData, NULL ) == SP_ERROR )
   {
      DebugPrint(( 0, "no support present for trident ge escape\n" )) ;
   }
   else
   {
      DebugPrint(( 0, "support is present for trident ge escape\n" )) ;
      bRetVal = TRUE;
   }

   ReleaseDC( GetDesktopWindow(), hDC );
   return bRetVal;
}


/* Function: SetSISCompatibility
 * Purpose: Flips the magic bit in certain SIS PCI Bridges. 
 * Note:    Original intention was to check not only the
 *          DeviceId, but also the RevId. This was considered to
 *          be the safest way to deal with a "reserved" register.
 *          We decided to eliminate RevId checking for now however,
 *          since we assume this bit not to change--if it does,
 *          the part will stop functioning and we'll address it then.
 */
void SetSISCompatibility(State s)
{
   int devID1 = 0x5511;          // SIS Part Numbers
   int devID2 = 0x5596;
   int devID3 = 0x5597;
   int devID4 = 0x5598;
   
   PCI_IO pci( SISId );

   if (pci.FindDevice( devID1 ) ||
       pci.FindDevice( devID2 ) )
   {
      long val ;

      DebugPrint((0,"SIS 5511 or 5596\n")) ;

      pci.ReadConfigDWORD( 0x080, &val ) ;
      if (s == On)
         val |= 0x0100 ;           // set the magic bit
      else
         val &= ~0x0100 ;          // clear the magic bit
      pci.WriteConfigDWORD( 0x080, val ) ;
   }
   else if (pci.FindDevice( devID3 ) ||
            pci.FindDevice( devID4 ) )
   {
      long val ;
      DWORD mask = 0x06 ; // bit 1 and 2 set

      DebugPrint((0,"SIS 5597 or 5598\n")) ;

      pci.ReadConfigDWORD( 0x081, &val ) ;
      if (s == On)
         val &= ~mask ;          // clear bits 1 and 2
      else
         val |= mask ;           // reset bits 1 and 2 
      pci.WriteConfigDWORD( 0x081, val ) ;
   }

   else
      DebugPrint((0,"NO SIS\n")) ;

}


/* Function: SetVIACompatibility
 * Purpose:  Turns on the VIA Bit in Fusion PCI config space.
 * Note:     This is a performance hit: Only done if offending chipset 
 *           is present. Only valid on Fusion (not Bt848).
 */
void SetVIACompatibility(State s)
{
   if (HWisBt878Family)
   {
      int devID1 = 0x585;          // VIA Part Number
      PCI_IO pci( VIAId );

      if ( pci.FindDevice( devID1 ) )
      {
         // At this point we've determined that the system is using the
         // offending chipset. We now set the bit in *Fusion's* PCI
         // config space.
         DebugPrint((0,"Found VIA 585\n")) ;
         PCI_IO pciBt( BtID );

         if ( pciBt.FindDevice( Bt878devID ) ||
              pciBt.FindDevice( Bt879devID ) ||
              pciBt.FindDevice( Bt880devID ) )
         {
            long val ;
            DebugPrint((0,"Flipping Via bit in Fusion PCI Config space [%x]\n",s)) ;
            pciBt.ReadConfigDWORD( 0x040, &val ) ;
            if (s == On)
               val |= 0x04 ;           // set the magic bit
            else
               val &= ~0x04 ;          // clear the magic bit
            pciBt.WriteConfigDWORD( 0x040, val ) ;
         }
      }  
      else
         DebugPrint((0,"No VIA 585 Found\n")) ;
   }
}

/* Function: SetTritonCompatibility
 * Purpose:  Turns on the Triton Bit in Fusion PCI config space.
 * Note:     In the 878, the Triton bit is in the PCI config space.
 *           In the 848, the bit is in the IMASK register. So we only
 *           set the bit here if we're on an 878. Otherwise, it is set
 *           in the Vxd in the IMASK register.
 */
void SetTritonCompatibility(State s)
{
   if (HWisBt878Family)
   {
      PCI_IO pciBt( BtID );

      if ( pciBt.FindDevice( Bt878devID ) ||
           pciBt.FindDevice( Bt879devID ) ||
           pciBt.FindDevice( Bt880devID ) )
      {
         long val ;
         DebugPrint((0,"Flipping Triton bit in Fusion PCI Config space [%x]\n",s)) ;
         pciBt.ReadConfigDWORD( 0x040, &val ) ;
         if (s == On)
            val |= 0x02 ;           // set the magic bit
         else
            val &= ~0x02 ;          // clear the magic bit
         pciBt.WriteConfigDWORD( 0x040, val ) ;
      }
   }
}

void SetChipsetCompatibility(State s)
{
   SetSISCompatibility(s) ;
   SetVIACompatibility(s) ;
   SetTritonCompatibility(s) ;
}

/* Function: GetOurInstance
 * Purpose: Returns the module instance handle
 */
HINSTANCE GetOurInstance() { return ghModule; }

/* Function: VideoClose
 * Purpose: Close a VfW video channel.
 * Input: VidChan: VfWChannel - pointer to a channel to be closed
 * Output: WORD: DRV_RET_OK
 * Note: Called in response to the DRV_CLOSE
 */
inline DWORD VideoClose( VfWChannel *VidChan ) { return VidChan->Close(); }

extern "C" BOOL WINAPI _export Connect( HINSTANCE hDriver );

typedef int ( WINAPI *CONNECT_t )( HINSTANCE );
CONNECT_t pCONNECT;

DIALOG_t pVID_DIALOG;

typedef ErrorCode ( WINAPI *INIT_t )();
INIT_t  pINITIALIZE;

typedef void ( WINAPI *DISCON_t)();
DISCON_t  pDISCONNECT;

typedef void ( WINAPI *DRV_FEATURE_t )( WORD );
DRV_FEATURE_t pDRV_FEATURE;


int Connect( HINSTANCE hInst )
{
   return pCONNECT ? pCONNECT( hInst ) : 1;
}

WORD WINAPI VidFormatDialog( HWND hWndParent )
{
   return pVID_DIALOG ? pVID_DIALOG( hWndParent, VIDEO_FORMAT ) : 0;
}

WORD WINAPI VideoSourceDialog( HWND hParent )
{
   return pVID_DIALOG ? pVID_DIALOG( hParent, VIDEO_SOURCE ) : 0;
}

WORD WINAPI VideoDisplayDialog( HWND hParent )
{
   return pVID_DIALOG ? pVID_DIALOG( hParent, VIDEO_DISPLAY ) : 0;
}


ErrorCode WINAPI Initialize()
{
   return pINITIALIZE ? pINITIALIZE() : Fail;
}

void WINAPI Disconnect()
{
   if ( pDISCONNECT ) pDISCONNECT();
}

void DriverFeature( WORD par )
{
   if ( pDRV_FEATURE )
      pDRV_FEATURE( par );
}

void CalmDisplayDriver( State s )
{
   if ( s == On ) {
      DriverFeature( 0 ); // disable screen blit
      DriverFeature( 2 ); // disable PCI retry
   } else {
      DriverFeature( 1 ); // enable screen blit
      DriverFeature( 3 ); // enable PCI retry
   }
}

HINSTANCE hDlg;
HINSTANCE hCCDll;

/* Function: LibMain
 * Purpose: DLL entry point
 */
extern "C" int FAR PASCAL LibMain( HINSTANCE hInst, WORD, WORD, LPSTR )
{
   ghModule = hInst;
   hDlg = LoadLibrary( "BtVidDlg.dll" );
   if ( hDlg > HINSTANCE_ERROR ) {
      pCONNECT = (CONNECT_t)GetProcAddress( hDlg, "Connect" );
      pVID_DIALOG = (DIALOG_t)GetProcAddress( hDlg, "VideoDialog" );

      pINITIALIZE = (INIT_t)GetProcAddress( hDlg, "Initialize" );
      pDISCONNECT = (DISCON_t)GetProcAddress( hDlg, "Disconnect" );
   }
   Connect( hInst );

   // connect to the CC decode dll
   hCCDll = InitCcApi ( ) ;

   // connect to the display driver
   HINSTANCE hDispl = GetModuleHandle( "Display" );
   if ( hDispl )
      pDRV_FEATURE = (DRV_FEATURE_t)GetProcAddress( hDispl, "DrvFeatureCtrl" );

   if ( pDRV_FEATURE )
      VfWChannel::SetSingleKeyColor( true );

   CalmDisplayDriver( On );

   HWRevID = -1;
   HWisBt848Aor849 = HWisBt878Family = false;

   PCI_IO pci( BtID );
   if ( pci.FindDevice( Bt849devID ))
   {
      HWRevID = pci.GetRevisionID();
      DebugPrint((0,"Found 849: RevId: %d\n",HWRevID)) ;
      HWisBt848Aor849 = true ;              // Any rev of an 849 is cool
   }
   else if ( pci.FindDevice( Bt848devID ))
   {
      HWRevID = pci.GetRevisionID();
      DebugPrint((0,"Found 848: RevId: %d\n",HWRevID)) ;
      HWisBt848Aor849 = ( ( HWRevID == 0x02 ) ||   // Rev A
                          ( HWRevID == 0x12 ) ||   // Rev B
                          ( HWRevID == 0x32 ) );   // Rev C
   }
   else if ( pci.FindDevice( Bt878devID ) ||
             pci.FindDevice( Bt879devID ) ||
             pci.FindDevice( Bt880devID ) )
   {
      HWRevID = pci.GetRevisionID();
      HWisBt878Family = true;
      DebugPrint((0,"Found 878 family: RevId: %d\n",HWRevID)) ;
   }
   else
     DebugPrint((0,"** No 848, 849, or Bt878 family\n")) ;

   DebugPrint((0,"HWisBt848Aor849 : [%s]\n",HWisBt848Aor849 ? "true" : "** FALSE **")) ;
   DebugPrint((0,"HWisBt878Family : [%s]\n",HWisBt878Family ? "true" : "** FALSE **")) ;

   DebugPrint((0,"Load Capdrv\n")) ;
   SetChipsetCompatibility(On) ;

   DebugPrint((0,"EnableTridentGE( FALSE );\n")) ;
   EnableTridentGE( FALSE );

   return 1;
}

/* Function: DriverProc
 * Purpose: The entry point for an installable driver.

DWORD dwDriverId  For most messages, dwDriverId is the DWORD
      value that the driver returns in response to a DRV_OPEN message.
      Each time that the driver is opened, through the DrvOpen API,
      the driver receives a DRV_OPEN message and can return an
      arbitrary, non-zero value. The installable driver interface
      saves this value and returns a unique driver handle to the
      application. Whenever the application sends a message to the
      driver using the driver handle, the interface routes the message
      to this entry point and passes the corresponding dwDriverId.
      This mechanism allows the driver to use the same or different
      identifiers for multiple opens but ensures that driver handles
      are unique at the application interface layer.

      The following messages are not related to a particular open
      instance of the driver. For these messages, the dwDriverId
      will always be zero.

          DRV_LOAD, DRV_FREE, DRV_ENABLE, DRV_DISABLE, DRV_OPEN

HDRVR  hDriver  This is the handle returned to the
      application by the driver interface.

UINT  uiMessage  The requested action to be performed. Message
      values below DRV_RESERVED are used for globally defined messages.
      Message values from DRV_RESERVED to DRV_USER are used for
      defined driver protocols. Messages above DRV_USER are used
      for driver specific messages.

LPARAM  lParam1  Data for this message.  Defined separately for
      each message

LPARAM  lParam2  Data for this message.  Defined separately for
      each message

Returns: Defined separately for each message.
*/

extern "C"
LRESULT WINAPI _export DriverProc( DWORD dwDriverID, HDRVR hDriver,
   UINT uiMessage, LPARAM lParam1, LPARAM lParam2 )
{
   switch ( uiMessage ) {
   case DRV_LOAD:
      DebugPrint((0,"DRV_LOAD\n"));

      /*
         Sent to the driver when it is loaded. Always the first
         message received by a driver.

         dwDriverID is 0L.
         lParam1 is 0L.
         lParam2 is 0L.

         Return 0L to fail the load.

         DefDriverProc will return NON-ZERO so we don't have to
         handle DRV_LOAD
      */

      return DRV_RET_OK;

   case DRV_FREE:
      DebugPrint((0,"DRV_FREE\n"));

      /*
         Sent to the driver when it is about to be discarded. This
         will always be the last message received by a driver before
         it is freed.

         dwDriverID is 0L.
         lParam1 is 0L.
         lParam2 is 0L.

         Return value is ignored.
      */

      Disconnect();
      CalmDisplayDriver( Off );
      return DRV_RET_OK;

   case DRV_OPEN:
      DebugPrint((0,"DRV_OPEN\n"));

      /*
         Sent to the driver when it is opened.

         dwDriverID is 0L.

         lParam1 is a far pointer to a zero-terminated string
         containing the name used to open the driver.

         lParam2 is passed through from the drvOpen call. It is
         NULL if this open is from the Drivers Applet in control.exe
         It is LPVIDEO_OPEN_PARMS otherwise.

         Return 0L to fail the open.

         DefDriverProc will return ZERO so we do have to
         handle the DRV_OPEN message.
       */

      //
      //  if we were opened without a open structure then just
      //  return a phony (non zero) id so the OpenDriver() will
      //  work.
      //
      if ( lParam2 == NULL )
         return BOGUS_DRIVER_ID;

      //  Verify this open is for us, and not for an installable
      //  codec.
      if (((LPVIDEO_OPEN_PARMS) lParam2) -> fccType != OPEN_TYPE_VCAP)
          return 0L;

      if ( VfWChannel::GetCreatedChannels() >= 3 ) {
//RL In high resolution mode, 1280x1024x256 colors, it displays Bt848 is busy 
//RL         static bool DoneThis = false;
//RL         if ( !DoneThis ) {
//RL            DoneThis = true;
//RL            ErrorDialog( XMSG( IDS_BUSY ) );
//RL         }
         return 0;
      }
      return VfWChannel::VideoOpen( (LPVIDEO_OPEN_PARMS) lParam2 );

   case DRV_CLOSE:
      DebugPrint((0,"DRV_CLOSE\n"));

      /*
         Sent to the driver when it is closed. Drivers are unloaded
         when the close count reaches zero.

         dwDriverID is the driver identifier returned from the
         corresponding DRV_OPEN.

         lParam1 is passed through from the drvClose call.

         lParam2 is passed through from the drvClose call.

         Return 0L to fail the close.

         DefDriverProc will return ZERO so we do have to
         handle the DRV_CLOSE message.
      */

      if ( dwDriverID == BOGUS_DRIVER_ID || dwDriverID == 0 )
         return DRV_RET_OK;      // return success

      VideoClose( (VfWChannel*)dwDriverID );
      return DRV_RET_OK;

   case DRV_ENABLE:
      DebugPrint((0,"DRV_ENABLE\n"));

      /*
         Sent to the driver when the driver is loaded or reloaded
         and whenever Windows is enabled. Drivers should only
         hook interrupts or expect ANY part of the driver to be in
         memory between enable and disable messages

         dwDriverID is 0L.
         lParam1 is 0L.
         lParam2 is 0L.

         Return value is ignored.
      */
      // Yahoooo - global enable !
      VfWChannel::Enable();
      return DRV_RET_OK;

//      case DRV_EXITSESSION:
   case DRV_DISABLE:
      DebugPrint((0,"DRV_DISABLE\n"));

      /*
         Sent to the driver before the driver is freed.
         and whenever Windows is disabled

         dwDriverID is 0L.
         lParam1 is 0L.
         lParam2 is 0L.

         Return value is ignored.
      */

      // criple all activities that might be
      VfWChannel::Disable();
      return DRV_RET_OK;

   case DRV_QUERYCONFIGURE:
      DebugPrint((0,"DRV_QUERYCONFIGURE\n"));

      /*
          Sent to the driver so that applications can
          determine whether the driver supports custom
          configuration. The driver should return a
          non-zero value to indicate that configuration
          is supported.

          dwDriverID is the value returned from the DRV_OPEN
          call that must have succeeded before this message
          was sent.

          lParam1 is passed from the app and is undefined.
          lParam2 is passed from the app and is undefined.

          Return 0L to indicate configuration NOT supported.
      */

      return 0;        // we don't do configuration

   case DRV_CONFIGURE:
      DebugPrint((0,"DRV_CONFIGURE\n"));

      /*
          Sent to the driver so that it can display a custom
          configuration dialog box.

          lParam1 is passed from the app. and should contain
          the parent window handle in the loword.
          lParam2 is passed from the app and is undefined.

          Return value is undefined.

          Drivers should create their own section in system.ini.
          The section name should be the driver name.
      */

      return 0;

   case DRV_INSTALL:
      DebugPrint((0,"DRV_INSTALL\n"));
      /*
          Windows has installed the driver, and since we
          are loaded on demand, there is no need to
          restart Windows.
      */
      return DRV_OK;

   case DRV_REMOVE:
      DebugPrint((0,"DRV_REMOVE\n"));
      return DRV_OK;

   case DVM_GETVIDEOAPIVER: /* lParam1 = LPDWORD */
      DebugPrint((0,"DVM_GETVIDEOAPIVER\n"));
       if ( lParam1 ) {
          *(LPDWORD) lParam1 = VIDEOAPIVERSION;
          return DV_ERR_OK;
       } else
          return DV_ERR_PARAM1;

   default:
//      DebugPrint((0,"DriverProc default\n"));
      if ( dwDriverID == BOGUS_DRIVER_ID || dwDriverID == 0 )
         return DefDriverProc( dwDriverID, hDriver, uiMessage,lParam1,lParam2 );

      return VfWChannel::VideoProcessMessage(
         (*(VfWChannel*)(LPVOID)dwDriverID), uiMessage, lParam1, lParam2 );
   }
}

int WEP( int )
{
   if ( hDlg > HINSTANCE_ERROR )
      FreeLibrary( hDlg );

   if ( hCCDll > HINSTANCE_ERROR )
      FreeLibrary( hCCDll ) ;

   DebugPrint((0,"Leaving Capdrv\n") ) ;

   SetChipsetCompatibility(Off) ;

   EnableTridentGE( TRUE );

   return TRUE;
}

extern "C" BOOL  WINAPI CAP_ThunkConnect16( LPSTR pszDll16,
   LPSTR pszDll32, WORD hInst, DWORD dwReason );

extern "C" BOOL  WINAPI DRV_ThunkConnect16( LPSTR pszDll16,
   LPSTR pszDll32, WORD hInst, DWORD dwReason );

extern "C" BOOL WINAPI _export DllEntryPoint( DWORD dwReason, WORD hInst,
   WORD, WORD, DWORD, WORD  )
{
   if ( !DRV_ThunkConnect16( "btVid.drv", "btVid_32.dll", hInst, dwReason ) )
      return FALSE;

   RegUtil Storage( BT_VIDEO_REG_SUBKEY );

   // working variable
   DWORD dwTmp = 0;

   // if only Preview mode is to be used; Bt848ddi.dll will not be loaded
   if ( ( Storage.GetValue( &dwTmp, "PreviewOnly" ) == Success ) &&
        ( dwTmp == 1 ) )
   {
      VfWChannel::SetPreviewOnly( true );
      return TRUE;
   }

   VfWChannel::SetPreviewOnly( false );

   if ( !CAP_ThunkConnect16( "btVid.drv", "btddi.dll", hInst, dwReason ) )
      return FALSE;


   return TRUE;
}
