/* File: Errdlg.cpp
 * Purpose: Displays the error dialog from the capture driver
 * Author: sasha
 * $Log: ERRDLG.CPP $
 * Revision 2.5  1997/11/26 01:46:57  peter
 * Fixed MKS graft from branch problem.
 * Revision 2.3.2.1  1997/11/20 22:35:12  peter
 * Changes to move registry subkey to HKEY_LOCAL_MACHINE.
 * Revision 2.3  1997/02/28 19:33:02  trantq
 * - Fix Intel #379  Upon error the error dialog should return to the 
 *   application an error, therefore,  stopping any further capture
 *   which might be faulty or just doesn't capture anything.
 * Revision 2.2  1996/09/06 22:20:56  rudyl
 * If registry key "ErrorDialog" does not exist or = 0, no error dlg displayed.
 * Default=1 which displays error dialog.
 * Revision 2.1  1996/06/10 20:12:58  ssm
 * MarkBeta1AsRev2
 * Revision 1.2  1996/01/10 21:32:15  sasha
 * Added #include "errdlg.h"
 * Revision 1.1  1995/09/03 01:14:36  sasha
 * Initial revision
 */

#include "vfwchan.h"
#include "protos.h"
#include "errdlg.h"

#include <bwcc.h> // for IDHELP


/*^^*-----------------
 * Callback: MessDlgProc
 * Purpose: The dialog procedure to handle the error message dialog
 * Input: Standard for a dialog procedure
 * Output: standard
 * Note : User can open a windows help from this dialog
 *^^*-----------------*/
#pragma argsused
BOOL WINAPI _loadds ErrDlgProc( HWND hDlg, UINT uiMess, WPARAM wParam, LPARAM lParam )
{
   static XMSG FAR *lpMsg;
   static bool CloseHelp = false;

   static char HelpFile [100];

   switch ( uiMess ) {
   case WM_INITDIALOG:
      lpMsg =  (XMSG FAR*)lParam;
      SetDlgItemText( hDlg, ID_TEXT_WIN, (char*)*lpMsg );
      break;
   case WM_COMMAND:
      switch ( wParam ) {
      case IDCANCEL:
      case IDOK:
         if ( CloseHelp == true )
            WinHelp( hDlg, HelpFile, HELP_QUIT, 0 );
         EndDialog( hDlg, 0 );
         return FALSE;
         
      case IDHELP:
         if ( LoadString( GetOurInstance(), IDS_HELPFILE, HelpFile, sizeof( HelpFile ) ) )
            if ( WinHelp( hDlg, HelpFile, HELP_KEY, (DWORD)lpMsg->GetID() ) )
               CloseHelp = true;
         break;
      default:
         return FALSE;
      }
      break;
   default:
      return FALSE;
   }
   return TRUE;
}


void ErrorDialog( XMSG &msg )
{
   RegUtil Storage( "SOFTWARE\\Brooktree\\Video" );

   // working variable
   DWORD dwTmp = 0;
   bool displayErrorDialog = true;  // default = display Error Dialog

   // get Registry key to check error dialog should be displayed or not
   // Registry key "ErrorDialog" = 1 or not exist -> display
   //                            = 0              -> don't display
   if ( ( Storage.GetValue( &dwTmp, "ErrorDialog" ) == Success ) &&
        ( dwTmp == 0 ) )
      displayErrorDialog = false;

   // don't display error dialog unless registry key = 1
   if ( displayErrorDialog )
      DialogBoxParam( GetOurInstance(), MAKEINTRESOURCE( ERR_DIALOG ),
                      GetFocus(), ErrDlgProc, (LPARAM)(LPVOID)&msg );
}
