/* File Extvidin.cpp
 * Purpose: Implementation of the VfW external video in channel
 * Author: sasha
 * $Log: EXTVIDIN.CPP $
 * Revision 2.28  1998/04/14 18:41:23  rudyl
 * Per VVL, modified QSight lighting type default from Backlight to Normal.
 * Revision 2.27  1998/04/06 19:16:29  rudyl
 * Per VVL's suggestion, default value for brightness is modified to be the
 * same as saturation and hue; i.e. 50. Just add 30 to get actual reg value.
 * Revision 2.26  1998/04/01 02:20:12  rudyl
 * Moved storing and restoring QSight settings to constructor and destructor.
 * Revision 2.25  1998/03/31 19:30:34  rudyl
 * Not necessary to call VP_DEFAULTS when app starts for QSight camera.
 * Revision 2.24  1998/03/31 02:08:16  rudyl
 * Added manufacturer default settings for QuartzSight registers.
 * Revision 2.23  1998/03/28 01:30:44  rudyl
 * Graft from 2.23.1.3.
 * Revision 2.22.1.3  1998/03/28 01:30:22  rudyl
 * Use TRUE when calling VP_DEFAULTS.
 * Moved QuartzSight init to vxd.
 * Revision 2.22.1.2  1998/03/24 01:56:12  rudyl
 * Moved QuartzSight init to vxd.
 * Revision 2.22.1.1  1998/03/24 01:36:41  rudyl
 * Modifed to use HW I2C to interface with Scotch. Also removed polling status bit.
 * Revision 2.22  1998/03/04 23:29:16  rudyl
 * Need to store 2 more QSight values to registry.
 * Revision 2.21  1998/02/28 01:53:00  rudyl
 * Removed FrameRate selection for QuartzSight.
 * Revision 2.20  1998/02/26 20:18:46  rudyl
 * Not storing QSight param in Settings structure. Interface to VPAEC directly.
 * Put QSight camera to low power mode when not in used.
 * Revision 2.19  1998/02/24 20:33:23  rudyl
 * Graft from 2.18.1.3.
 * Revision 2.18.1.3  1998/02/24 20:33:09  rudyl
 * QSightParam[] changed to DWORD. Also removed unused codes.
 * Revision 2.18.1.2  1998/02/24 20:24:26  rudyl
 * Added VVL's VPAEC interface to QuartzSight.
 * Revision 2.18.1.1  1998/01/21 19:56:24  rudyl
 * Duplicate revision
 * Revision 2.17  1998/01/16 21:53:21  rudyl
 * Gemini Rev D: HSFMT changes from 1 to 2 bits. Also API changed.
 * Revision 2.16  1997/12/13 01:51:50  rudyl
 * Graft from 2.15.1.3. Added SRAM API.
 * Revision 2.15.1.3  1997/12/13 01:50:27  rudyl
 * GetBinValue() is not returning ErrorCode yet and need to pass buffer size.
 * Revision 2.15.1.2  1997/12/13 00:30:26  rudyl
 * Use SetBinValue() and GetBinValue() to store and retrieve TimGenByte binary data.
 * Revision 2.15.1.1  1997/11/27 03:12:01  rudyl
 * Moved SRAM table to dialogs.dll.
 * Revision 2.15  1997/11/27 00:48:14  rudyl
 * Scotch: added pulse to GPDATA[22] to reset Scotch.
 * Revision 2.14  1997/11/26 06:30:04  rudyl
 * Added support of Scotch.
 * Revision 2.13  1997/11/26 01:11:24  peter
 * Revision 2.11.1.1  1997/11/20 22:35:45  peter
 * Changes to move registry subkey to HKEY_LOCAL_MACHINE.
 * Revision 2.11  1997/10/29 18:41:22  dtharp
 * Merged TrueView branch.
 * Revision 2.10.1.1  1997/10/29 02:21:27  rudyl
 * Added trueview.
 * Revision 2.10  1997/10/21 02:01:24  rudyl
 * Added support to expected total line count by decoder.
 * Revision 2.9  1997/06/02 17:25:46  dtharp
 * Bug id 626: Fix for previewOnly 848A/849 crash.  Protected
 * calls into EVO::StopOverlay() && EVO::StartOverlay() from
 * null EVO pointer.
 * Revision 2.8  1997/05/06 00:19:02  dtharp
 * 848A Merge: from 2.6.1.2
 * Revision 2.7  1997/03/18 20:00:27  rudyl
 * Graft from 2.6.2.2. PAL/SECAM uses 288 lines per field.
 * Revision 2.6.2.2  1997/03/08 02:47:47  rudyl
 * Now PAL uses 288 lines per field.
 * Revision 2.6.2.1  1997/03/08 02:29:19  rudyl
 * Use 288 lines for PAL.
 * Revision 2.6  1997/02/19 21:34:25  dtharp
 * Grafted Branch RL_SvideoNoColor -- Bug id 391
 * Revision 2.5.1.2  1997/02/09 22:55:00  rudyl
 * Bug# 391
 * Problem: SVideo input comes up black and white
 * Solution: Don't call SetSVideo() when app starts since
 *   SetVideoFormat() already takes care of that depends on the video source.
 *   Users can call SetSVideo() separately if SetVideoFormat() doesn't do what they want.
 * Revision 2.5.1.1  1997/02/09 22:50:58  rudyl
 * Bug# 391: SVideo input comes up black and white
 * Revision 2.5  1996/12/06 22:29:21  meierdw
 * Grafting 2.4.1 back into trunk; NTSC-Japan support added.
 * Revision 2.4.1.5  1997/02/20 01:58:21  rudyl
 * Changed DDI_SetVidFormat() to DDI_SetVidSize().
 * Revision 2.4.1.4  1997/02/14 23:40:12  rudyl
 * Removed all #ifdef BT848A. Use same binary files for both Bt848 and Bt848A.
 * Revision 2.4.1.3  1997/02/03 23:41:07  rudyl
 * First Pisces+ check in.
 * Revision 2.4.1.2  1996/11/08 18:53:16  rudyl
 * Added NTSC-Japan, PAL-M & PAL-N formats.
 * Revision 2.4  1996/10/02 17:10:18  rudyl
 * Bug #353
 * Problem: Driver depends upon direct draw files
 * Solution: Added "SetPreviewOnly( bool par )" and "IsPreviewOnly()" in VfWChannel class
 * so that when driver is loading, Bt848ddi.dll is not linked in if registry key "PreviewOnly" = 1.
 * Also prevent calling DDI...() in this case.
 * Revision 2.3  1996/09/06 22:52:42  rudyl
 * Return different maximum video size depends on NTSC or PAL.
 * Revision 2.2  1996/08/08 01:16:57  sasha
 * Call DDI_SetVidFormat() in the RestoreValues() to make sure DDI
 * portion knows what sizes it must use.
 * Revision 2.1  1996/06/10 20:12:58  ssm
 * MarkBeta1AsRev2
 * Revision 1.19  1996/06/07 20:01:45  sasha
 * Added GetStatus() function to read register 0
 * Revision 1.18  1996/06/06 18:18:21  rudyl
 * changed registry key from Bt848 or APisces --> Software\Brooktree\Bt848
 * Revision 1.17  1996/06/06 01:38:21  sasha
 * Finished off with removing exceptions.
 * Revision 1.16  1996/06/06 01:15:50  sasha
 * Removed EH. Use return codes now
 * Revision 1.15  1996/06/04 21:49:52  ssm
 * ch registry name from APisces to Bt848
 * Revision 1.14  1996/05/21 21:57:18  sasha
 * Set static member CurVidFormat_ in the SetVideoFormat().
 * Revision 1.13  1996/05/16 21:22:26  sasha
 * Modified default fo rthe hue
 * Revision 1.12  1996/05/16 21:17:38  sasha
 * Changed defaults for the decoder values when run first time
 * Revision 1.11  1996/05/12 21:12:52  sasha
 * Moved guts of OpenVxDChannel into VfWChannel class
 * Revision 1.10  1996/04/01 21:12:08  sasha
 * Changed connector names again
 * Revision 1.9  1996/04/01 20:57:14  sasha
 * Changed names to reflect new Connector names
 * Revision 1.8  1996/03/29 18:12:54  sasha
 * Changed video format names ( as per viddefs.h )
 * Revision 1.7  1996/03/15 23:39:22  sasha
 * And now fixed problem with getting registry values when they do exist!
 * Revision 1.6  1996/03/15 22:15:40  sasha
 * Fixed problem with getting registry values when they don't exist
 * Revision 1.5  1996/02/21 22:49:43  sasha
 * Added code to initialize decoder with the last stored values. Do it in the
 * init stream message handler
 * Revision 1.4  1996/01/22 20:28:28  sasha
 * Added decoder related function calls to VxD
 * Revision 1.3  1996/01/10 21:23:06  sasha
 * VideoSourceDialog function does not have an instance as an argument anymore
 * Revision 1.2  1996/01/04 23:21:08  sasha
 * First real check-in. Too many changes since previous version
 * Revision 1.1  1995/09/04 23:17:26  sasha
 * Initial revision
 */

#include "extvidin.h"
#include "vpaec16.h"
#include "bt848api.h"

const DWORD AllFs = 0xFFFFFFFFUL;

ExtVidInChan *ExtVidInChan::ExtChan_ = 0;
bool ExtVidInChan::Created_ = false;

/* Method: ExtVidInChan::Create
 * Purpose: Creates the object of type ExtVidInChan and returns pointer to it
 * Input: None
 * Output: ExtVidInChan *
 * Note: This creation method is used to prevent multiple openings of the same
 *   video channel by different applications
 */
ExtVidInChan *ExtVidInChan::Create()
{
   if ( Created_ == true ) {
      ErrorDialog( XMSG( IDS_EXTVIDIN ) );
      return NULL;
   }

   Created_ = true;
   return new ExtVidInChan();
}


/* Method:  ExtVidInChan::DstRect
 * Purpose: Handles the DVM_DST_RECT message
 * Input: lpRect: LPRECT - far pointer to the RECT structure
 *    Flags: LPARAM - flag bits
 * Output: DV_ERR_OK or error code
 */
WORD ExtVidInChan::DstRect( LPRECT lpRect, LPARAM Flags )
{
   switch ( Flags ) {
   case VIDEO_CONFIGURE_SET:
   case VIDEO_CONFIGURE_SET | VIDEO_CONFIGURE_CURRENT:
      // store the position and dimentions of the image to be digitized
      // they will be used for the video in channel and optionally
      // for the external video out channel ( preview )
      VfWChannel::SetMasterAnalogRect( lpRect );

/*
      // at this point we can assume VxD is connected, so it is safe to pass
      // the call down to it
      if ( CallVxD( FNAnalogRect, &SmallRect( *lpRect ) ) != Success )
         return DV_ERR_BADFORMAT;
*/
      break;
   case VIDEO_CONFIGURE_GET | VIDEO_CONFIGURE_CURRENT:
      VfWChannel::GetMasterAnalogRect( lpRect );
      break;
   case VIDEO_CONFIGURE_GET | VIDEO_CONFIGURE_MAX: {
      if ( IsABt848A() )
      {
         if ( GetCurSignal() != Signal_Analog )
         {
            TRect r( 0, 0, (int)Settings_.videoInfo.HActive, (int)Settings_.videoInfo.VActive );
            *lpRect = r;
            break;
         }
         else  // analog signal
         if ( GetCurFormat() == VFormat_PAL_NC )
         {
            TRect r( 0, 0, 640, 576 );
            *lpRect = r;
            break;
         }
      }
      TRect r( 0, 0, ( IsNTSCCompatible( GetCurFormat() ) ) ? 640 : 768,
                     ( IsNTSCCompatible( GetCurFormat() ) ) ? 480 : 576 );
      *lpRect = r;
//         VfWChannel::GetMasterAnalogRect( lpRect );
      break;
      }
   default:
      return DV_ERR_NOTSUPPORTED;
   }
   return DV_ERR_OK;
}


/* Method:   ExtVidInChan::Caps
 * Purpose:
 * Input:
 * Output:
 */
WORD ExtVidInChan::Caps( LPCHANNEL_CAPS lpCaps, LPARAM lSize )
{
   // structure should be of the right size
   if ( lSize != sizeof( CHANNEL_CAPS ) )
      return DV_ERR_NONSPECIFIC;

   // For this device, scaling happens during digitization
   // into the frame buffer.

   lpCaps-> dwFlags = VCAPS_CAN_SCALE | VCAPS_SRC_CAN_CLIP;

   //BUGBUGBUG: have to check the data format for the channel in order to
   // set the granularity properly. I.E. granularity is 1 for RGB modes
   // and 4 for YUV9 ( in both directions )

   // for the external in channel the size of the source rectangle is size
   // of the current standard ( PAL, etc. ) and can not be changed
   lpCaps-> dwSrcRectXMod = 1;	 // Src undefined at present.
   lpCaps-> dwSrcRectYMod = 1;
   lpCaps-> dwSrcRectWidthMod = 1;
   lpCaps-> dwSrcRectHeightMod = 1;

   // for the external in channel dst rectangle defines the cropping and scaling
   // of the _digitized_ image
   lpCaps-> dwDstRectXMod = 1;
   lpCaps-> dwDstRectYMod = 1;
   lpCaps-> dwDstRectWidthMod = 1;
   lpCaps-> dwDstRectHeightMod = 1;

   return DV_ERR_OK;
}

/* Method:  ExtVidInChan::VidConfigStorage
 * Purpose:
 * Input:
 * Output:
 */
#pragma argsused
WORD ExtVidInChan::VidConfigStorage( UINT uiMsg, LPARAM lParam1, LPARAM lParam2 )
{
   return DV_ERR_OK;
}

/* Method:  ExtVidInChan::StreamMsg
 * Purpose:
 * Input:
 * Output:
 */
#pragma argsused
WORD ExtVidInChan::StreamMsg( UINT uiMsg, LPARAM lParam1, LPARAM lParam2 )
{
   //  lParam1	 -   LPVIDEO_STREAM_INIT_PARMS
   //
   //  lParam2	 -   sizeof (LPVIDEO_STREAM_INIT_PARMS)
   //
   switch ( uiMsg ) {
   case DVM_STREAM_INIT:
      SetConnector( (Connector)Settings_.CurConn, true );
      SetBrightness( (int)Settings_.Bright );
      SetContrast( (int)Settings_.Contrast );
      SetHue( (int)Settings_.Hue );
      SetSaturation( (int)Settings_.Satur );
      SetInterlaceMode( (int)Settings_.Interlace );
      if ( IsABt848A() || IsBt878Family() )
      {
         SetSignalFormat( (SignalFormat)Settings_.SigFmt );
         SetVideoFormat( (VideoFormat)Settings_.VFormat );
         if ( IsBt878Family() )
            SetTotalLineCount( (int)Settings_.TotalLineCount );
         if ( (SignalFormat)Settings_.SigFmt != Signal_Analog )
         {
            SetSyncVideoRef( (SyncVideoRef)Settings_.SyncRef );
            SetEnableSyncOutput( (State)Settings_.SyncOutput );
            SetTimGenByte( Settings_.TimGenByte, Settings_.TimGenSize );
            SetHReset( (int)Settings_.HReset );
            SetPLL( (PLLFreq)Settings_.PLLFreq );
            SetDecInputClock( (DecInputClk)Settings_.InClk );
            SetVideoInfo( Settings_.videoInfo );
            if ( Settings_.QuartzSight )
               SetQuartzSightParam( VP_LOW_POWER, FALSE );  // power up QSight camera chip
         }
      }
      else
         SetVideoFormat( (VideoFormat)Settings_.VFormat );
      break;
   case DVM_STREAM_FINI:
      // disconnect from VxD here
//      CloseVxDChannel();

      SetQuartzSightParam( VP_LOW_POWER, TRUE );   // power down QSight camera chip
      break;
   default:
      return DV_ERR_NOTSUPPORTED;
   }
   return DV_ERR_OK;
}

/* Constructor: ExtVidInChan::ExtVidInChan()
 * Purpose: Constructs and initializes the ExtVidInChan object
 */
ExtVidInChan::ExtVidInChan() : VfWChannel()
{
   ExtChan_ = this;
   // 0xFF means non-valid
   memset( &Settings_, 0xFF, sizeof( Settings_ ) );
   RestoreValues();
   RestoreQSightParam();
}

extern "C" void DDI_SetVidSize( DWORD, DWORD );

/* Method: ExtVidInChan::RestoreValues
 * Purpose: Initializes the state of the channel
 * Input: None
 * Output: None
 */
void ExtVidInChan::RestoreValues()
{
// here we should get all the entries from the registry and initialize properly
   RegUtil Storage( "SOFTWARE\\Brooktree\\Video" );

   // working horse variable
   DWORD dwTmp = 0;

   if ( Storage.GetValue( &dwTmp, "connector" ) != Success )
      Settings_.CurConn = ConTuner;
   else
      Settings_.CurConn = dwTmp;

   if ( Storage.GetValue( &dwTmp, "SVideo" ) != Success )
      Settings_.SVideo = Off;
   else
      Settings_.SVideo = dwTmp;

   if ( Storage.GetValue( &dwTmp, "Brightness" ) != Success )
      Settings_.Bright = 128;
   else
      Settings_.Bright = dwTmp;

   if ( Storage.GetValue( &dwTmp, "Contrast" ) != Success )
      Settings_.Contrast = 128;
   else
      Settings_.Contrast = dwTmp;

   if ( Storage.GetValue( &dwTmp, "Hue" ) != Success )
      Settings_.Hue = 128;
   else
      Settings_.Hue = dwTmp;

   if ( Storage.GetValue( &dwTmp, "Saturation" ) != Success )
      Settings_.Satur = 128;
   else
      Settings_.Satur = dwTmp;

   if ( Storage.GetValue( &dwTmp, "Video Format" ) != Success )
      Settings_.VFormat = VFormat_NTSC;
   else
      Settings_.VFormat = dwTmp;

   if ( Storage.GetValue( &dwTmp, "SignalFormat" ) != Success )
      Settings_.SigFmt = Signal_Analog;
   else
      Settings_.SigFmt = dwTmp;

   if ( Storage.GetValue( &dwTmp, "SyncRef" ) != Success )
      Settings_.SyncRef = VRef_alignedCb;
   else
      Settings_.SyncRef = dwTmp;

   if ( Storage.GetValue( &dwTmp, "SyncOutput" ) != Success )
      Settings_.SyncOutput = 0;
   else
      Settings_.SyncOutput = dwTmp;

   if ( Storage.GetValue( &dwTmp, "InClk" ) != Success )
      Settings_.InClk = 0;
   else
      Settings_.InClk = dwTmp;

   if ( Storage.GetValue( &dwTmp, "OutClk" ) != Success )
      Settings_.OutClk = 1;
   else
      Settings_.OutClk = dwTmp;

#pragma message("GetBinValue() is not returning ErrorCode yet...")
   dwTmp = 100;   // size of buffer
   Storage.GetBinValue( (int *)&dwTmp, Settings_.TimGenByte, "TimGenByte" );
   Settings_.TimGenSize = (int)dwTmp;

   if ( Storage.GetValue( &dwTmp, "PLLFreq" ) != Success )
      Settings_.PLLFreq = 0;
   else
      Settings_.PLLFreq = dwTmp;

   if ( Storage.GetValue( &dwTmp, "HReset" ) != Success )
      Settings_.HReset = 64;
   else
      Settings_.HReset = dwTmp;

   if ( Storage.GetValue( &dwTmp, "RefFreq" ) != Success )
      Settings_.RefFreq = 28;
   else
      Settings_.RefFreq = dwTmp;

   if ( Storage.GetValue( &dwTmp, "HActive" ) != Success )
      Settings_.videoInfo.HActive = 160;
   else
      Settings_.videoInfo.HActive = dwTmp;

   if ( Storage.GetValue( &dwTmp, "HDelay" ) != Success )
      Settings_.videoInfo.HDelay = 0x80;
   else
      Settings_.videoInfo.HDelay = dwTmp;

   if ( Storage.GetValue( &dwTmp, "VActive" ) != Success )
      Settings_.videoInfo.VActive = 120;
   else
      Settings_.videoInfo.VActive = dwTmp;

   if ( Storage.GetValue( &dwTmp, "VDelay" ) != Success )
      Settings_.videoInfo.VDelay = 0x1A;
   else
      Settings_.videoInfo.VDelay = dwTmp;

   if ( Storage.GetValue( &dwTmp, "LinesPerField" ) != Success )
      Settings_.videoInfo.linesPerField = 80;
   else
      Settings_.videoInfo.linesPerField = dwTmp;

   if ( Storage.GetValue( &dwTmp, "TotalLineCount" ) != Success )
      Settings_.TotalLineCount = 0;
   else
      Settings_.TotalLineCount = dwTmp;

   if ( Storage.GetValue( &dwTmp, "Interlace" ) != Success )
      Settings_.Interlace = 1;
   else
      Settings_.Interlace = dwTmp;

   if ( Storage.GetValue( &dwTmp, "QuartzSight" ) != Success )
      Settings_.QuartzSight = 0;
   else
      Settings_.QuartzSight = dwTmp;


   // other VfWChannel can use these values also
   VidInfo_ = Settings_.videoInfo;

   // don't call DDI_...() if we are using Preview mode only;
   // i.e. no overlay channel is created and Bt848ddi.dll is not linked in
   if ( IsPreviewOnly() == false )
   {
      if ( Settings_.SigFmt != Signal_Analog )
         DDI_SetVidSize( Settings_.videoInfo.HActive, Settings_.videoInfo.VActive );
      else
      {
         int width  = ( Settings_.VFormat == VFormat_PAL_NC ) ? 640 : IsNTSCCompatible( (VideoFormat)Settings_.VFormat ) ? 640 : 768;
         int height = ( Settings_.VFormat == VFormat_PAL_NC ) ? 576 : IsNTSCCompatible( (VideoFormat)Settings_.VFormat ) ? 480 : 576;
         DDI_SetVidSize( width, height );
      }
   }
}

/* Method: ExtVidInChan::StoreValues
 * Purpose: Initializes the state of the channel
 * Input: None
 * Output: None
 */
void ExtVidInChan::StoreValues()
{
   // here we should put all the persistant info into the registry
   RegUtil Storage( "SOFTWARE\\Brooktree\\Video" );

   Storage.SetValue( Settings_.CurConn,      "connector" );
   Storage.SetValue( Settings_.SVideo,       "SVideo" );
   Storage.SetValue( Settings_.Bright,       "Brightness" );
   Storage.SetValue( Settings_.Contrast,     "Contrast" );
   Storage.SetValue( Settings_.Hue,          "Hue" );
   Storage.SetValue( Settings_.Satur,        "Saturation" );
   Storage.SetValue( Settings_.VFormat,      "Video Format" );
   Storage.SetValue( Settings_.Interlace,    "Interlace" );

   if ( IsABt848A() )
   {
      // No need to store "RefFreq" since it is set by board manufacturer and
      // shouldn't be changed
      Storage.SetValue( Settings_.SigFmt,                  "SignalFormat" );
      Storage.SetValue( Settings_.SyncRef,                 "SyncRef" );
      Storage.SetValue( Settings_.SyncOutput,              "SyncOutput" );
      Storage.SetValue( Settings_.InClk,                   "InClk" );
      Storage.SetValue( Settings_.OutClk,                  "OutClk" );
      Storage.SetBinValue( Settings_.TimGenSize, Settings_.TimGenByte, "TimGenByte" );
      Storage.SetValue( Settings_.PLLFreq,                 "PLLFreq" );
      Storage.SetValue( Settings_.HReset,                  "HReset" );
      Storage.SetValue( Settings_.videoInfo.HActive,       "HActive" );
      Storage.SetValue( Settings_.videoInfo.HDelay,        "HDelay" );
      Storage.SetValue( Settings_.videoInfo.VActive,       "VActive" );
      Storage.SetValue( Settings_.videoInfo.VDelay,        "VDelay" );
      Storage.SetValue( Settings_.videoInfo.linesPerField, "LinesPerField" );
   }

   if ( IsBt878Family() )
   {
      Storage.SetValue( Settings_.TotalLineCount,          "TotalLineCount" );
      Storage.SetValue( Settings_.QuartzSight,             "QuartzSight" );
   }
}

/****************************************************************************
 * Method:  void ExtVidInChan::StoreQSightParam( void )
 * Purpose: Store QuartzSight controller chip values to registry
 * Input:   None
 * Output:  None
 * Return:  None
 * Note:    Using VVL's VPAEC
 ****************************************************************************/
void ExtVidInChan::StoreQSightParam( void )
{
   BYTE value;
   RegUtil Storage( "SOFTWARE\\Brooktree\\Video\\QuartzSight\\User" );

   GetQuartzSightParam( VP_AEC_TC, &value );
   Storage.SetValue( value - 30, "Brightness" );   // dlg value is 30 less actual reg value

   GetQuartzSightParam( VP_SATURATION, &value );
   Storage.SetValue( value, "Saturation" );

   GetQuartzSightParam( VP_HUE, &value );
   Storage.SetValue( value, "Hue" );

   GetQuartzSightParam( VP_GAMCOR, &value );
   Storage.SetValue( value, "Gamma" );

   GetQuartzSightParam( VP_SHARPNESS, &value );
   Storage.SetValue( value, "Sharpness" );

   GetQuartzSightParam( VP_AEC_MAINS, &value );
   Storage.SetValue( value, "LightFreq" );

   GetQuartzSightParam( VP_AEC_LIGHTING, &value );
   Storage.SetValue( value, "LightType" );
}

/****************************************************************************
 * Method:  void ExtVidInChan::RestoreQSightParam( void )
 * Purpose: Restore values from registry and send to camera controller chip
 * Input:   None
 * Output:  None
 * Return:  None
 * Note:    Using VVL's VPAEC
 ****************************************************************************/
void ExtVidInChan::RestoreQSightParam( void )
{
   DWORD dwTmp = 0;

   RegUtil regUser( "SOFTWARE\\Brooktree\\Video\\QuartzSight\\User" );
   RegUtil regManufacturer( "SOFTWARE\\Brooktree\\Video\\QuartzSight\\ManufacturerDefaults" );

   // Looks for user's setting. If not found, look for manufacturer's setting.
   // If nothing exists, default to VVL's default.
   if ( regUser.GetValue( &dwTmp, "Brightness" ) != Success )
      if ( regManufacturer.GetValue( &dwTmp, "Brightness" ) != Success )
         dwTmp = 50;
	SetQuartzSightParam( VP_AEC_TI, (BYTE)dwTmp + 15 );
   SetQuartzSightParam( VP_AEC_TC, (BYTE)dwTmp + 30 );
   SetQuartzSightParam( VP_AEC_TH, (BYTE)dwTmp + 45 );

   if ( regUser.GetValue( &dwTmp, "Saturation" ) != Success )
      if ( regManufacturer.GetValue( &dwTmp, "Saturation" ) != Success )
         dwTmp = 50;
	SetQuartzSightParam( VP_SATURATION, (BYTE)dwTmp );
   
   if ( regUser.GetValue( &dwTmp, "Hue" ) != Success )
      if ( regManufacturer.GetValue( &dwTmp, "Hue" ) != Success )
         dwTmp = 50;
	SetQuartzSightParam( VP_HUE, (BYTE)dwTmp );
   
   if ( regUser.GetValue( &dwTmp, "Gamma" ) != Success )
      if ( regManufacturer.GetValue( &dwTmp, "Gamma" ) != Success )
         dwTmp = 8;
	SetQuartzSightParam( VP_GAMCOR, (BYTE)dwTmp );
   
   if ( regUser.GetValue( &dwTmp, "Sharpness" ) != Success )
      if ( regManufacturer.GetValue( &dwTmp, "Sharpness" ) != Success )
         dwTmp = 3;
	SetQuartzSightParam( VP_SHARPNESS, (BYTE)dwTmp );
   
   if ( regUser.GetValue( &dwTmp, "LightFreq" ) != Success )
      if ( regManufacturer.GetValue( &dwTmp, "LightFreq" ) != Success )
         dwTmp = HZ_60;
	SetQuartzSightParam( VP_AEC_MAINS, (BYTE)dwTmp );
   
   if ( regUser.GetValue( &dwTmp, "LightType" ) != Success )
      if ( regManufacturer.GetValue( &dwTmp, "LightType" ) != Success )
         dwTmp = LIGHTING_NORMAL;
	SetQuartzSightParam( VP_AEC_LIGHTING, (BYTE)dwTmp );
}

/* Method: ExtVidInChan::SetConnector
 * Purpose:
 * Input: None
 * Output: None
 */
void ExtVidInChan::SetConnector( Connector AConn, bool )
{
   Settings_.CurConn = AConn;
   CallVxD( FNDecoder, &Settings_.CurConn, DecConn );
}

/* Method: ExtVidInChan::SetSVideo
 * Purpose:
 * Input: None
 * Output: None
 */
void ExtVidInChan::SetSVideo( State aState )
{
   Settings_.SVideo = aState;
   CallVxD( FNDecoder, &Settings_.SVideo, DecSVid );
}

/* Method: ExtVidInChan::SetBrightness
 * Purpose:
 * Input: None
 * Output: None
 */
void ExtVidInChan::SetBrightness( int percentOfFullScale )
{
   Settings_.Bright = percentOfFullScale;
   CallVxD( FNDecoder, &Settings_.Bright, DecBrt );
}

/* Method: ExtVidInChan::SetContrast
 * Purpose:
 * Input: None
 * Output: None
 */
void ExtVidInChan::SetContrast( int percentOfFullScale )
{
   Settings_.Contrast = percentOfFullScale;
   CallVxD( FNDecoder, &Settings_.Contrast, DecContr );
}

/* Method: ExtVidInChan::SetHue
 * Purpose:
 * Input: None
 * Output: None
 */
void ExtVidInChan::SetHue( int degrees )
{
   Settings_.Hue = degrees;
   CallVxD( FNDecoder, &Settings_.Hue, DecHue );
}

/* Method: ExtVidInChan::SetSaturation
 * Purpose:
 * Input: None
 * Output: None
 */
void ExtVidInChan::SetSaturation( int percentOfFullScale )
{
   Settings_.Satur = percentOfFullScale;
   CallVxD( FNDecoder, &Settings_.Satur, DecSat );
}

/* Method: ExtVidInChan::SetVideoFormat
 * Purpose:
 * Input: None
 * Output: None
 */
void ExtVidInChan::SetVideoFormat( VideoFormat aFormat )
{
   Settings_.VFormat = aFormat;
   CallVxD( FNDecoder, &Settings_.VFormat, DecFmt );
   CurVidFormat_ = aFormat;

   if ( IsABt848A() && ( Settings_.SigFmt == Signal_Analog ) )
   {
      // if the frequency required is the same as the reference frequency,
      // XTAL is used; otherwise PLL is used to synthesized the desired freq
      //
      //  Ref Freq    Signal          Action
      //-----------------------------------------------
      //    14         NTSC           PLL, 28
      //    14         PAL            PLL, 35
      //    14         PAL-Nc         PLL, 28
      //    28         NTSC           XTAL
      //    28         PAL            PLL, 35
      //    28         PAL-Nc         XTAL
      //    35         NTSC           PLL, 28
      //    35         PAL            XTAL
      //    35         PAL-Nc         PLL, 28

      // takes care of using XTAL first
      if ( ( IsNTSCCompatible( aFormat )   && ( Settings_.RefFreq == 28 ) ) ||
           ( IsPALCompatible( aFormat )    && ( Settings_.RefFreq == 35 ) ) ||
           ( ( aFormat == VFormat_PAL_NC ) && ( Settings_.RefFreq == 28 ) ) )
         SetDecInputClock( DecClk_XTAL );
      else
      // takes care of using PLL with 35.46895MHz
      if ( IsPALCompatible( aFormat ) )
      {
         SetPLL( KHz35469 );
         SetDecInputClock( DecClk_PLL );
      }
      else
      // rest are using PLL with 28.63636MHz
      {
         SetPLL( KHz28636 );
         SetDecInputClock( DecClk_PLL );
      }
   }
}

/* Method: ExtVidInChan::GetStatus
 * Purpose: Returns decoder's register 0
 * Input: None
 * Output: None
 */
DWORD ExtVidInChan::GetStatus()
{
   DWORD dwStatus;
   CallVxD( FNStatus, &dwStatus );
   return dwStatus;
}

/* Method: ExtVidInChan::OpenVxDChannel
 * Purpose: Opens the connection to the VxD Channel
 * Input: DWORD: device node
 * Output: void
 */
ErrorCode ExtVidInChan::OpenVxDChannel( DWORD dnDevNode )
{
   if ( VxDHandle_ )
      return Success;

   return VfWChannel::OpenVxDChannel( VS_Capture, dnDevNode );
}

/* Method: ExtVidInChan::VidDialog
 * Purpose: Displays a configuration dialog
 * Input:
 * Output:
 */
WORD ExtVidInChan::VidDialog( HWND hParent, LPARAM lFlags )
{
   if ( lFlags & VIDEO_DLG_QUERY )
		return DV_ERR_OK;	// Support the dialog.
   return VideoSourceDialog( hParent );
}

/****************************************************************************
 * Method:  void ExtVidInChan::SetSignalFormat( SignalFormat sFormat )
 * Purpose: Select input signal format
 * Input:   SignalFormat sFormat - video signal format
 *             Signal_Analog         - Analog
 *             Signal_CCIR656        - CCIR 656
 *             Signal_ByteStream     - ByteStream
 *             Signal_ExtHSYNC_VSYNC - External HSYNC, VSYNC
 *             Signal_ExtHSYNC_FIELD - External HSYNC, FIELD
 * Output:  None
 * Return:  None
 ****************************************************************************/
void ExtVidInChan::SetSignalFormat( SignalFormat sFormat )
{
   CurSigFormat_ = sFormat;
   Settings_.SigFmt = sFormat;
   CallVxD( FNDecoder, &Settings_.SigFmt, DecSigFmt );

   // need to stop and restart overlay channel so odd registers also get updated
   if ( &ExtVidOutChan::GetExtVidOut() != NULL ) {
      ExtVidOutChan::GetExtVidOut().StopOverlay();
      ExtVidOutChan::GetExtVidOut().StartOverlay();
   }
}

/****************************************************************************
 * Method:  void ExtVidInChan::SetSyncVideoRef( SyncVideoRef syncRef )
 * Purpose: Select sync video reference
 * Input:   SyncVideoRef syncRef - sync video reference
 *             VRef_alignedCb - HS/VS aligned with Cb
 *             VRef_alignedY0 - HS/VS aligned with Y0
 *             VRef_alignedCr - HS/VS aligned with Cr
 *             VRef_alignedY1 - HS/VS aligned with Y1
 * Output:  None
 * Return:  None
 ****************************************************************************/
void ExtVidInChan::SetSyncVideoRef( SyncVideoRef syncRef )
{
   Settings_.SyncRef = syncRef;
   CallVxD( FNDecoder, &Settings_.SyncRef, DecSyncRef );
}

/****************************************************************************
 * Method:  void ExtVidInChan::SetDecInputClock( DecInputClk iClk )
 * Purpose: Select decoder input clock
 * Input:   DecInputClk iClk - decoder input clock
 *             DecClk_XTAL      - normal XTAL0/XTAL1
 *             DecClk_PLL       - PLL
 *             DecClk_GPCLK     - GPCLK
 *             DecClk_invGPCLK  - GPCLK- inverted
 * Output:  None
 * Return:  None
 ****************************************************************************/
void ExtVidInChan::SetDecInputClock( DecInputClk iClk )
{
   Settings_.InClk = iClk;
   CallVxD( FNTimGenInClk, &Settings_.InClk );
}

/****************************************************************************
 * Method:  void ExtVidInChan::SetGPCLKOutputClock( GPCLKOutputClk oClk )
 * Purpose: Select GPCLK output clock
 * Input:   GPCLKOutputClk oClk - GPCLK output clock
 *             GPCLK_CLKx1   - pixel rate
 *             GPCLK_Xtal0   - XTAL
 *             GPCLK_PLL     - PLL
 *             GPCLK_invPLL  - PLL - inverted
 * Output:  None
 * Return:  None
 ****************************************************************************/
void ExtVidInChan::SetGPCLKOutputClock( GPCLKOutputClk oClk )
{
   Settings_.OutClk = oClk;
   CallVxD( FNTimGenOutClk, &Settings_.OutClk );
}

/****************************************************************************
 * Method:  void ExtVidInChan::SetEnableSyncOutput( State state )
 * Purpose: Enable/Disable syncs output for synchronizing video input
 * Input:   State state - On to enable; Off to disable
 * Output:  None
 * Return:  None
 ****************************************************************************/
void ExtVidInChan::SetEnableSyncOutput( State state )
{
   Settings_.SyncOutput = state;
   CallVxD( FNDecoder, &Settings_.SyncOutput, DecSyncOutput );
}

/****************************************************************************
 * Method:  void ExtVidInChan::SetTimGenByte( BYTE * tgb, int size )
 * Purpose: Load time generator bytes into SRAM
 * Input:   BYTE * tgb - pointer to BYTE array contains SRAM table
 *          int size   - size of SRAM table
 * Output:  None
 * Return:  None
 ****************************************************************************/
void ExtVidInChan::SetTimGenByte( BYTE * tgb, int size )
{
   if ( size <= sizeof( Settings_.TimGenByte ) )
   {
      Settings_.TimGenSize = size;
      memcpy( Settings_.TimGenByte, tgb, size );
      CallVxD( FNTimGenLB, tgb, size );
   }
}

/****************************************************************************
 * Method:  void ExtVidInChan::SetHReset( int clk )
 * Purpose: Set HRESET length (HSFMT)
 * Input:   int clk - 64, 48, 32 or 16 pixels (default is 64)
 * Output:  None
 * Return:  None
 ****************************************************************************/
void ExtVidInChan::SetHReset( int clk )
{
   Settings_.HReset = clk;
   CallVxD( FNDecoder, &Settings_.HReset, DecHReset );
}

/****************************************************************************
 * Method:  void ExtVidInChan::SetPLL( PLLFreq pll )
 * Purpose: Set PLL to synthesize desired frequency
 * Input:   PLLFreq pll - desired synthesized frequency
 *             NoPLL    - not using PLL
 *             KHz28636 - 28.63636 MHz
 *             KHz35469 - 35.46895 MHz
 *             KHz27000 - 27.00000 MHz
 *             KHz24545 - 24.54545 MHz
 *             KHz29500 - 29.50000 MHz
 *             KHz19636 - 19.63636 MHz
 * Output:  None
 * Return:  None
 * Note:    If no reference frequency is specified, 14.31818MHz is assumed
 *          being installed on the board. Windows registry key "RefFreq"
 *          contains the reference frequency information.
 ****************************************************************************/
void ExtVidInChan::SetPLL( PLLFreq pll )
{
   // depends on the reference frequency, we'll load the correct resource

   int resID;
   if ( Settings_.RefFreq == 28 )
      resID = RC_PLL_28MHZ_BASE;
   else if ( Settings_.RefFreq == 35 )
      resID = RC_PLL_35MHZ_BASE;
   else
      resID = RC_PLL_14MHZ_BASE;  // default is 14.31838 MHz as reference

   // NoPLL uses res ID (RC_PLL_BASE + RC_PLL_NOPLL),
   // other PLL freq are just offsets into each ref frequency's group
   if ( pll == NoPLL )
      resID = RC_PLL_BASE + RC_PLL_NOPLL;
   else
      resID += pll;  // offset into ref freq's base ID

   // the PLL reg settings are in RCDATA
   HANDLE hResource = LoadResource( GetOurInstance(),
      FindResource( GetOurInstance(), MAKEINTRESOURCE( resID ), RT_RCDATA ) );

   if ( hResource == NULL )
      return;     // resource not found

   Settings_.PLLFreq = pll;
   LPBYTE pRes = (LPBYTE)LockResource ( hResource );

   PLLInfo param( *pRes, *(pRes+1), *(pRes+2), *(pRes+3), *(pRes+4) );
   CallVxD( FNPLL, &param, SetFunc );

   UnlockResource( hResource );
   FreeResource( hResource );
}

/****************************************************************************
 * Method:  void ExtVidInChan::SetVideoInfo( VSIZEPARMS vsize )
 * Purpose: Set video size parameters
 * Input:   VSIZEPARMS vsize - parameters to set video size
 * Output:  None
 * Return:  None
 ****************************************************************************/
void ExtVidInChan::SetVideoInfo( VSIZEPARMS vsize )
{
   // convert parameters to VideoInfo in order to pass them to VxD
   VideoInfo param( vsize.HActive, vsize.HDelay, vsize.VActive, vsize.VDelay, vsize.linesPerField );
   Settings_.videoInfo = param;
   CallVxD( FNScaler, &param, SetFunc );
   VidInfo_ = param;
}

/****************************************************************************
 * Method:  void ExtVidInChan::SetVideoInfo( VideoInfo info )
 * Purpose: Set video size parameters
 * Input:   VideoInfo info - parameters to set video size
 * Output:  None
 * Return:  None
 ****************************************************************************/
void ExtVidInChan::SetVideoInfo( VideoInfo info )
{
   Settings_.videoInfo = info;
   CallVxD( FNScaler, &info, SetFunc );
}

/****************************************************************************
 * Method:  void ExtVidInChan::GetVideoInfo( VSIZEPARMS * vsize )
 * Purpose: Get video size parameters
 * Input:   None
 * Output:  VSIZEPARMS * vsize - ptr to video size parameters obtained
 * Return:  None
 ****************************************************************************/
void ExtVidInChan::GetVideoInfo( VSIZEPARMS * vsize )
{
   VSIZEPARMS out = { (WORD)Settings_.videoInfo.HActive,
                      (WORD)Settings_.videoInfo.HDelay,
                      (WORD)Settings_.videoInfo.VActive,
                      (WORD)Settings_.videoInfo.VDelay,
                      (WORD)Settings_.videoInfo.linesPerField };
   *vsize = out;
}

/****************************************************************************
 * Method:  void ExtVidInChan::ResetFieldCounter( void )
 * Purpose: Reset field counter
 * Input:   None
 * Output:  None
 * Return:  None
 ****************************************************************************/
void ExtVidInChan::ResetFieldCounter( void )
{
   CallVxD( FNFldCntr, NULL, SetFunc );
}

/****************************************************************************
 * Method:  int ExtVidInChan::GetFieldCounter( void )
 * Purpose: Obtain field counter
 * Input:   None
 * Output:  None
 * Return:  None
 ****************************************************************************/
int ExtVidInChan::GetFieldCounter( void )
{
   DWORD counter;
   CallVxD( FNFldCntr, &counter, GetFunc );
   return (int)counter;
}

/****************************************************************************
 * Method:  ExtVidInChan::SetTotalLineCount( int lines )
 * Purpose: Set expected number of horizontal video lines per frame
 * Input:   int lines - expected number of horizontal video lines per frame
 * Output:  None
 * Return:  None
 ****************************************************************************/
void ExtVidInChan::SetTotalLineCount( int lines )
{
   Settings_.TotalLineCount = lines;
   CallVxD( FNDecoder, &Settings_.TotalLineCount, DecLineCount );
}

/****************************************************************************
 * Method:  ErrorCode ExtVidInChan::SetInterlaceMode( int mode )
 * Purpose: Select vertical scaling interlace mode
 * Input:   int mode - 0 = non-interlace
 *                     1 = interlace
 *                     2 = field aligned (Gemini family only)
 * Output:  None
 * Return:  ErrorCode
 ****************************************************************************/
ErrorCode ExtVidInChan::SetInterlaceMode( int mode )
{
   Settings_.Interlace = mode;
   return CallVxD( FNSetInterlaceMode, NULL, Settings_.Interlace );
}

/****************************************************************************
 * Method:  ErrorCode ExtVidInChan::SetQuartzSightMode( BOOL state )
 * Purpose: Select QuartzSight camera mode
 * Input:   BOOL state - flag to indicate whether QuartzSight camera is in use
 * Output:  None
 * Return:  ErrorCode
 ****************************************************************************/
ErrorCode ExtVidInChan::SetQuartzSightMode( BOOL state )
{
	// If already in QuartzSight mode, don't call VP_LOW_POWER because
   // it'll reset Scotch reg
	if ( Settings_.QuartzSight == state )
	   return Success;
   
   Settings_.QuartzSight = state;
   SetQuartzSightParam( VP_LOW_POWER, !state );
   RegUtil Storage( "SOFTWARE\\Brooktree\\Video" );
   Storage.SetValue( Settings_.QuartzSight, "QuartzSight" );
   return Success;
}


// Using VVL's VPAEC to communicate with QuartzSight camera controller chip.

/****************************************************************************
 * Method:  ErrorCode ExtVidInChan::SetQuartzSightParam( BYTE param, BYTE value )
 * Purpose: Set QuartzSight camera controller chip register value through VVL'x vxd
 * Input:   BYTE param - parameter to be modified
 *          BYTE value - value to be written
 * Output:  None
 * Return:  ErrorCode
 * Note:    Using VVL's VPAEC
 ****************************************************************************/
ErrorCode ExtVidInChan::SetQuartzSightParam( BYTE param, BYTE value )
{
   if ( param < (BYTE)VP_SENTINEL_LAST )
      return( VPAEC_SetParam( (T_VP_PARAM)param , value ) ? Success : Fail );
   return( Fail );
}

/****************************************************************************
 * Method:  ErrorCode ExtVidInChan::GetQuartzSightParam( BYTE param, BYTE * pValue )
 * Purpose: Get QuartzSight camera controller chip register content through VVL's vxd
 * Input:   BYTE param - parameter to be retrieved
 * Output:  BYTE * value - value retrieved
 * Return:  ErrorCode
 * Note:    Using VVL's VPAEC
 ****************************************************************************/
ErrorCode ExtVidInChan::GetQuartzSightParam( BYTE param, BYTE * pValue )
{
   if ( param < (BYTE)VP_SENTINEL_LAST )
      return( VPAEC_GetParam( (T_VP_PARAM)param , pValue ) ? Success : Fail );
   return( Fail );
}

