/* File: Extvidin.h
 * Purpose: Contains definition of the ExtVidInChan class
 * Author: sasha
 * $Log: EXTVIDIN.H $
 * Revision 2.12  1998/04/01 02:20:13  rudyl
 * Moved storing and restoring QSight settings to constructor and destructor.
 * Revision 2.11  1998/03/28 01:31:54  rudyl
 * Graft from 2.10.1.1: Moved QuartzSight init to vxd.
 * Revision 2.10.1.1  1998/03/24 01:56:29  rudyl
 * Moved QuartzSight init to vxd.
 * Revision 2.10  1998/02/26 20:13:14  rudyl
 * Added StoreQSightParam().
 * Revision 2.9  1998/02/24 20:37:15  rudyl
 * Graft from 2.8.1.3.
 * Revision 2.8.1.3  1998/02/24 20:37:01  rudyl
 * Removed unused codes.
 * Revision 2.8.1.2  1998/02/24 20:34:27  rudyl
 * Added VVL's VPAEC interface to QuartzSight.
 * Revision 2.8.1.1  1998/02/06 00:41:34  rudyl
 * Duplicate revision
 * Revision 2.7  1998/01/16 21:53:24  rudyl
 * Gemini Rev D: HSFMT changes from 1 to 2 bits. Also API changed.
 * Revision 2.6  1997/12/13 01:52:15  rudyl
 * Graft from 2.5.1.1. Added SRAM API.
 * Revision 2.5.1.1  1997/11/27 03:12:11  rudyl
 * Moved SRAM table to dialogs.dll.
 * Revision 2.5  1997/11/26 06:30:10  rudyl
 * Added support of Scotch.
 * Revision 2.4  1997/10/29 18:41:41  dtharp
 * Merged TrueView branch.
 * Revision 2.3.1.1  1997/10/29 02:21:44  rudyl
 * Added trueview.
 * Revision 2.3  1997/10/21 02:01:27  rudyl
 * Added support to expected total line count by decoder.
 * Revision 2.2  1997/05/03 00:13:38  dtharp
 * 848A Merge: from 2.1.1.3
 * Revision 2.1.1.3  1997/02/14 23:40:12  rudyl
 * Removed all #ifdef BT848A. Use same binary files for both Bt848 and Bt848A.
 * Revision 2.1.1.2  1997/02/03 23:41:27  rudyl
 * First Pisces+ check in.
 * Revision 2.1.1.1  1996/11/08 23:14:58  rudyl
 * Start Pisces+ work.
 * Revision 2.1  1996/06/10 20:12:59  ssm
 * MarkBeta1AsRev2
 * Revision 1.6  1996/06/06 01:37:43  sasha
 * Finished off with removing exceptions.
 * Revision 1.5  1996/06/06 01:15:50  sasha
 * Removed EH. Use return codes now
 * Revision 1.4  1996/01/22 20:29:45  sasha
 * Changed DWORDs to ints in the decoder functions
 * Revision 1.3  1996/01/10 21:20:52  sasha
 * Changed return values for the decoder functions to int from DWORD
 * Revision 1.2  1996/01/04 23:21:09  sasha
 * First real check-in. Too many changes since previous version
 * Revision 1.1  1995/09/03 02:07:52  sasha
 * Initial revision
 */

#ifndef __EXTVIDIN_H
#define __EXTVIDIN_H

#ifndef __VFWCHAN_H
#include "vfwchan.h"
#endif

#ifndef __VIDDEFS_H
#include "viddefs.h"
#endif

#ifndef __VIDSETT_H
#include "vidsett.h"
#endif

#ifndef __RESDEF_H
#include "resdef.h"
#endif

#ifndef __EXVIDOUT_H
#include "exvidout.h"
#endif

/* Class: ExtVidInChan
 * Purpose: Implements functionality of the Video_EXTERNALIN channel
 * Operations: see VfWChannel for the responses to the VfW messages
 *    void RestoreValues() - reads values from registry
 *    void StoreValues() - writes values to the registry
 */
class ExtVidInChan : public VfWChannel
{
   private:

      static ExtVidInChan  *ExtChan_;

      static bool     Created_;

      VideoSettings  Settings_;

      virtual WORD VidDialog( HWND, LPARAM );
      virtual WORD DstRect( LPRECT, LPARAM );
      virtual WORD Caps( LPCHANNEL_CAPS, LPARAM );
      virtual WORD StreamMsg( UINT, LPARAM, LPARAM );
      virtual WORD VidConfigStorage( UINT uiMsg, LPARAM lParam1, LPARAM lParam2 );

      void RestoreValues();
      void StoreValues();

      void StoreQSightParam( void );
      void RestoreQSightParam( void );


      // private constructor - hahaha - call Create !
      ExtVidInChan();

   protected:
      virtual ErrorCode OpenVxDChannel(  DWORD dnDevNode );

   public:

      // used by external api functions
      static ExtVidInChan & GetExtVideoChan() { return *ExtChan_; }

      // image rect manipulation
      void SetImageRect( LPRECT lpRect ) { VfWChannel::SetMasterAnalogRect( lpRect ); }
      void GetImageRect( LPRECT lpRect ) { VfWChannel::GetMasterAnalogRect( lpRect ); }

      // 'dialog' functions
      void SetConnector( Connector AConn, bool ApplyImm = false );
      void SetSVideo( State aState );
      void SetBrightness( int percentOfFullScale );
      void SetSaturation( int percentOfFullScale );
      void SetContrast  ( int percentOfFullScale );
      void SetHue       ( int degrees );

      void SetVideoFormat( VideoFormat aFormat );

      // 'get' group
      Connector GetConnector() { return (Connector)Settings_.CurConn; }
      State GetSVideo()   { return (State)Settings_.SVideo; }
      int GetBrightness() { return (int)Settings_.Bright; }
      int GetContrast()   { return (int)Settings_.Contrast; }
      int GetHue()        { return (int)Settings_.Hue; }
      int GetSaturation() { return (int)Settings_.Satur; }
      DWORD GetStatus();

      VideoFormat GetVideoFormat() { return (VideoFormat)Settings_.VFormat; }

      void SetSignalFormat( SignalFormat sFormat );
      void SetSyncVideoRef( SyncVideoRef syncRef );
      void SetDecInputClock( DecInputClk iClk );
      void SetGPCLKOutputClock( GPCLKOutputClk oClk );
      void SetEnableSyncOutput( State state );
      void SetTimGenByte( BYTE * tgb, int size );
      void SetHReset( int clk );
      void SetPLL( PLLFreq pllFreq );
      void SetVideoInfo( VSIZEPARMS vsize );
      void SetVideoInfo( VideoInfo info );
      void SetTotalLineCount( int lines );
      ErrorCode SetInterlaceMode( int mode );
      ErrorCode SetQuartzSightMode( BOOL state );
      ErrorCode SetQuartzSightParam( BYTE param, BYTE value ); // Using VVL's VPAEC

      SignalFormat   GetSignalFormat() { return (SignalFormat)Settings_.SigFmt; }
      SyncVideoRef   GetSyncVideoRef() { return (SyncVideoRef)Settings_.SyncRef; }
      DecInputClk    GetDecInputClock() { return (DecInputClk)Settings_.InClk; }
      GPCLKOutputClk GetGPCLKOutputClock() { return (GPCLKOutputClk)Settings_.OutClk; }
      State          GetEnableSyncOutput() { return (State)Settings_.SyncOutput; }
      ErrorCode      GetTimGenByte( BYTE * tgb, int * size ) {
         if ( Settings_.TimGenSize > *size )
         {
            *size = 0;
            return Fail;
         }
         memcpy( tgb, Settings_.TimGenByte, Settings_.TimGenSize );
         *size = Settings_.TimGenSize;
         return Success; }
      int            GetHReset() { return (int)Settings_.HReset; }
      PLLFreq        GetPLL() { return (PLLFreq)Settings_.PLLFreq; }
      void           GetVideoInfo( VSIZEPARMS * vsize );
      int            GetTotalLineCount() { return (int)Settings_.TotalLineCount; }
      int            GetInterlaceMode() { return (int)Settings_.Interlace; }
      BOOL           GetQuartzSightMode() { return (BOOL)Settings_.QuartzSight; }
      ErrorCode      GetQuartzSightParam( BYTE param, BYTE * pValue );  // Using VVL's VPAEC

      void           ResetFieldCounter( void );
      int            GetFieldCounter( void );

      // this function creates the object
      static ExtVidInChan *Create();

      ~ExtVidInChan() { StoreValues(); StoreQSightParam(); Created_ = false; }
};

#endif  // __EXTVIDIN_H

