/* File Exvidout.cpp
 * Purpose: Implementation of the VfW external video out channel ( preview )
 * Author: sasha
 * $Log: EXVIDOUT.CPP $
 * Revision 2.52  1998/04/16 23:45:54  dtharp
 * Added shutdown of preview frame server upon overlay start.
 * Revision 2.51  1998/04/06 19:42:23  dtharp
 * Vxd Frame Buffers are allocated on the fly instead of at init time.
 * Revision 2.50  1998/04/02 21:40:23  dtharp
 * Merge DT_VIDEOHDR branch. Implements a new buffer 
 * management scheme.
 * Revision 2.49.1.2  1998/03/27 15:30:16  dtharp
 * Incremental check-in
 * Revision 2.49.1.1  1998/03/18 19:35:57  dtharp
 * Branch for passing videoheaders around
 * Revision 2.49  1998/02/24 00:10:07  dtharp
 * Cleaned up compiler warnings and made some explicit casts the BC5
 * wants.
 * Revision 2.48  1998/01/19 22:56:54  peter
 * Graft PL_IMAGELOAD_BUG_FIX to tip.
 * Revision 2.47.1.2  1997/12/20 01:49:13  peter
 * Revision 2.47.1.1  1997/12/19 23:00:01  peter
 * Work-in-progress for bug #567: ImageLoad() not working.
 * Revision 2.47  1997/12/16 23:10:52  dtharp
 * Merged WHQL (update delay) fix.
 * Revision 2.46  1997/11/26 01:48:31  peter
 * Fixed MKS graft from branch problem.
 * Revision 2.44.1.1  1997/11/20 22:36:08  peter
 * Changes to move registry subkey to HKEY_LOCAL_MACHINE.
 * Revision 2.44  1997/10/22 23:42:09  dtharp
 * Added hook to pass video window handle through to the Vxd
 * Revision 2.43  1997/08/01 04:29:35  dtharp
 * Bug ID 684. Fixed resize problem induced by leo page fault fix.
 * Revision 2.42  1997/07/10 16:17:52  dtharp
 * Fix for bug ID 665: leo page fauult in primary surface. This was a latent 
 * bug.
 * Revision 2.41  1997/05/28 17:24:56  trantq
 * Remove Debug statements
 * Revision 2.40  1997/05/28 17:00:18  trantq
 * Add support for Bt848ImageLoad & Bt848UnLoadImage.  Bug ID 431
 * Revision 2.39.1.2  1997/05/28 16:24:02  trantq
 * Add support for Bt848ImageLoad & Bt848UnLoadImage.  Bug ID 431
 * Revision 2.39.1.1  1997/05/20 21:31:40  trantq
 * Add feature LoadImage of DD Surface.  Bug ID 431
 * Revision 2.39  1997/05/20 21:31:40  trantq
 * - Merged in fix for GetOverlayFlag function.  Bug ID 565
 * Revision 2.38.1.1  1997/05/14 16:39:42  trantq
 * - Fix GetOverlayFlag function Bug ID 565.
 * Revision 2.38  1997/05/14 16:39:42  dtharp
 * Fixed a merge error in ExtVidOutChan::Start(). Problem had to do 
 * with calculating size limit of digital window.
 * Revision 2.37  1997/05/06 00:11:20  dtharp
 * 848A Merge: from 2.33.1.3
 * Revision 2.36  1997/04/28 21:15:27  dtharp
 * Merged CC Primary Branch from 2.34.1.1 Bug ID 475
 * Revision 2.35  1997/04/22 18:41:28  trantq
 * -Merge in Magenta Bar Fix onto tip.
 * Revision 2.34  1997/03/18 19:59:00  rudyl
 * Graft from 2.33.2.3. PAL/SECAM uses 288 lines per field.
 * Revision 2.33.2.3  1997/03/18 19:57:50  rudyl
 * NTSC uses 480 lines. Had 484 for testing.
 * Revision 2.33.2.2  1997/03/08 02:47:50  rudyl
 * Now PAL uses 288 lines per field.
 * Revision 2.33.2.1  1997/03/06 20:30:07  rudyl
 * Use 288 lines for PAL/SECAM
 * Revision 2.33.3.2  1997/03/18 01:14:11  trantq
 * - Fix Magenta bar on left-side of screen.
 * Revision 2.33.3.1  1997/03/18 01:11:24  trantq
 * - Fix Magenta bar on left-side of screen.
 * Revision 2.33  1996/12/06 23:58:58  rudyl
 * Merged file from branch 2.30.1.2: Added NTSC-Japan, PAL-M & PAL-N formats.
 * Note DWORD alignment fix is not merged into this release.
 * Revision 2.32  1996/12/06 22:04:33  sasha
 * Simple copy of the 2.31.1.2 to propagate the DD3 changes
 * Revision 2.31.1.2  1996/11/21 20:56:55  sasha
 * Changes for the DD3
 * Changed StopOverlay(). Simplified DoDDI().
 * Revision 2.31.1.1  1996/11/21 18:29:57  sasha
 * Revision 2.31  1996/11/13 23:12:30  sasha
 * Fixed a stupid bug: In the GetScreenParams() replaced DeleteDC()
 * with ReleaseDC().
 * Revision 2.30  1996/09/18 22:11:08  rudyl
 * Changed the registry key from "Preview" to "PreviewOnly".
 * Revision 2.29  1996/09/17 20:26:54  rudyl
 * Added checking of Registry key "Preview" to see if Preview mode is to be used;
 * = 0 or not exist -> attempt to create true overlay
 * = 1              -> don't create overlay, use Preview
 * Revision 2.28  1996/09/10 20:36:02  rudyl
 * Set ExternalOut_ to NULL after delete.
 * Revision 2.27  1996/09/07 01:48:10  rudyl
 * 1. Added checking of ExternalOut_ != NULL, thus deleted the line to destory ExternalOut_ in Create().
 * 2. If registry key "VideoDisplayDialog" does not exist or = 0, no Video Display Dialog is supported.
 * Default=1 which support the dialog.
 * Revision 2.26  1996/08/15 06:07:55  sasha
 * Check clip value for above 2 instead of 1
 * Revision 2.25  1996/08/10 23:39:19  sasha
 * Added dwStopFlag_ and code to prevent starting of the channel when
 * VxD detects full-screen DOS box
 * Revision 2.24  1996/08/07 22:59:07  sasha
 * Removed commented ou t line
 * Revision 2.23  1996/08/07 18:50:47  sasha
 * Use SingleKeyColor_ to see if need to recalculate the key color.
 * Call DDI_SetKeyColor() in every Update() call.
 * Revision 2.22  1996/07/29 21:54:43  sasha
 * Added code to round-down the width of a rectangle to be DWORD-aligned
 * Revision 2.21  1996/07/19 01:50:58  sasha
 * fixed GPF problem in high resolution, say 1280x1024x256 colors
 * Revision 2.20  1996/07/14 03:19:52  sasha
 * Added SetFocus() where quickres was detected.
 * Revision 2.19  1996/07/14 02:58:40  sasha
 * Checked in previously without saving the editor !
 * Revision 2.18  1996/07/14 02:52:36  sasha
 * Added code to the Update() to check for the Quickres activity.
 * Revision 2.17  1996/07/13 22:37:36  sasha
 * Turned moved_ into int. Changed Update() to start in overlay mode.
 * Changed RestartOverlay() to call Update() instead of other functions.
 * Added code to SrcRect() to make sure video is restarted when source
 * rectangle changes and that UpdateOverlay() will be called when source
 * rectangle is passed in ( for ET600 ).
 * Revision 2.16  1996/07/13 00:47:52  sasha
 * Do not alter top and bottom of the rectangles; do horisontal scale only.
 * Revision 2.15  1996/07/09 01:31:06  sasha
 * Removed some cropping-related code that set moved_ to true in overlay mode
 * Revision 2.14  1996/07/09 00:32:02  sasha
 * Display dialog when surface creation fails.
 * Revision 2.13  1996/07/08 22:36:55  sasha
 * Use the scaling information from DDI to set rectangles
 * Revision 2.12  1996/07/03 00:55:43  sasha
 * Use current color key value instead of zero in the call to DDI_CreateSurface()
 * in ExtVidOutChan::StopOverlay(). This fixes wrong color key set
 * Revision 2.11  1996/07/02 21:03:21  sasha
 * Added code to WorkOutColorKey() to support 24 bit overlay surfaces.
 * Revision 2.10  1996/07/01 20:23:45  sasha
 * Return NULL in the Create() when DDI_Init() fails
 * Revision 2.9  1996/06/29 00:31:45  sasha
 * Put a fix for the 5446 pink screen ( figure color key once only ).
 * Put code which forces UpdateOverlay() all the time ( set moved_ to
 * true )
 * Revision 2.8  1996/06/26 19:07:29  sasha
 * Added Pause() and Continue() functions that freeze/unfreeze Pisces
 * Revision 2.7  1996/06/22 00:21:50  sasha
 * Utilize source rectangle now. May be changed a bit
 * Revision 2.6  1996/06/19 23:52:49  sasha
 * Added PauseOverlay() and RestartOverlay() to let capture frames bigger
 * then CIF
 * Revision 2.5  1996/06/14 18:51:29  sasha
 * Disabled CC in sizes less than CIF. Returned back code to disable CC
 * in on-screen mode.
 * Revision 2.4  1996/06/14 18:09:32  sasha
 * Do not create logical palette anymore - use DDI_DoPalette(). Changed
 * DoDDI() to reduce flashing and detect primary->overlay transitions.
 * Call DDI_DoPalette() in the Update()
 * Revision 2.3  1996/06/12 01:45:48  sasha
 * Put back call to the CreateSurface() after clip list changed
 * Revision 2.2  1996/06/12 00:52:07  sasha
 * Create logical palette for 8bpp mode. Added code to Update() to select
 * palette in case of 8bpp mode and primary surface. Disable/enable CC
 * depending if a primary surface used
 * Revision 2.1  1996/06/10 20:13:00  ssm
 * MarkBeta1AsRev2
 * Revision 1.30  1996/06/04 23:59:19  sasha
 * Added PauseCC() and RestartCC(). Also completely stop CC stream
 * when user disabled it in the dialog.
 * Revision 1.29  1996/06/04 01:06:42  sasha
 * Removed commented code. Now re-initialize the CCEngine every time
 * window size changes.
 * Revision 1.28  1996/05/30 21:12:13  sasha
 * Removed dialog code. Do CC display only when enabled. Added SetCCStatus
 * and GetCCStatus
 * Revision 1.27  1996/05/23 03:31:48  sasha
 * Initial code to enable CC decode and display
 * Revision 1.26  1996/05/21 22:02:32  sasha
 * Take current video format in account when figuring out max rectangle in Start().
 * Revision 1.25  1996/05/21 20:36:59  sasha
 * Always use 0,0 as left top for the digital rectangle
 * Revision 1.24  1996/05/17 20:37:21  sasha
 * Do not use analog rectangle when start the channel
 * Revision 1.23  1996/05/14 20:18:46  sasha
 * Changed granularity of the rectangles from 1 to 4
 * Revision 1.22  1996/05/12 21:10:07  sasha
 * Moved guts of OpenVxDChannel into VfWChannel class
 * Revision 1.21  1996/05/06 21:37:27  sasha
 * Fixed stupid error in Create() : Called CreatePalette() with non-existing object
 * Revision 1.20  1996/05/02 21:24:31  sasha
 * Added some code to avoid flickering, but not using it yet ( commented out )
 * Revision 1.19  1996/04/17 20:54:40  sasha
 * Added frame rate parameter to the StartParms structure
 * Revision 1.18  1996/04/09 21:06:34  sasha
 * Use full brightness magenta for the key color
 * Revision 1.17  1996/04/04 00:13:12  sasha
 * Added some rectangle verification code to Start()
 * Revision 1.16  1996/03/30 01:28:40  sasha
 * Added check to the Update() to paint only in overlay mode
 * Revision 1.15  1996/03/27 23:53:38  sasha
 * Added function that figures out the key color. Added one more argument
 * ( key color ) to the DDI_CreateSurface(). Added code to the Update()
 * to paint the window with the key color. May change
 * Revision 1.14  1996/03/26 18:34:50  sasha
 * Added (int) cast to the moved_ in the Update() function.
 * Revision 1.13  1996/03/19 22:15:20  sasha
 * Added a hack to the Update() to avoid some redundancy in the DDI_GetData()
 * Revision 1.12  1996/03/14 20:38:18  sasha
 * Added Overlaying_ = true; to the StartOverlay().
 * Add linear buffer twice, so no overlay fields are lost when doing AVI capture.
 * Changed Update() to check for Overlaying_.
 * Added comments to the Start() function
 * Revision 1.11  1996/03/14 02:16:22  sasha
 * Removed code from Start() back to DoDDI(). Changed Update().
 * Revision 1.10  1996/03/07 18:57:52  sasha
 * Moved calls to add linear address and clip info from DoDDI into Start().
 * Moved line moved_ = false; out of if() in DoDDI(); Call Start() in Update()
 * when initializing for the first time.
 * Revision 1.9  1996/03/05 20:25:18  sasha
 * Added call to the DDI_GetData() with NULL to hide the overlay surface
 * in the StopOverlay()
 * Revision 1.8  1996/02/27 01:55:53  sasha
 * Added config dialog.
 * Revision 1.7  1996/02/24 02:48:49  sasha
 * Changed DoDDI function a bit ( to avoid clipping in overlay cases )
 * Revision 1.6  1996/02/19 02:33:59  sasha
 * Removed old commented code and changed OutputDebugString to D1
 * Revision 1.5  1996/02/14 02:35:22  sasha
 * Changed how clipping information is passed down to VxD
 * Revision 1.4  1996/02/06 01:25:56  sasha
 * Moved palette creation out of here. Add extra arg to the DDI_CreateSurface()
 * Revision 1.3  1996/01/22 20:30:45  sasha
 * Changed calls to VxD ( added an arument )
 * Revision 1.2  1996/01/04 23:21:11  sasha
 * First real check-in. Too many changes since previous version
 * Revision 1.1  1995/09/03 01:06:42  sasha
 * Initial revision
 */

#include "exvidout.h"
#include "protos.h"
#include "ddiflags.h"
#include "vidinchn.h"

static bool bUpdateImage=false;
bool  ExtVidOutChan::bDoIt_=false;  
HINSTANCE ExtVidOutChan::hInstance_ = NULL;
HWND  ExtVidOutChan::hImageWin_ = NULL;
LPSTR ExtVidOutChan::lpBitmap_ = NULL;
bool  ExtVidOutChan::bImageLoaded_ = false;
bool  ExtVidOutChan::Created_ = false;
HWND  ExtVidOutChan::OverlayWin_;
int   ExtVidOutChan::moved_;
UINT ExtVidOutChan::Timer_;
ExtVidOutChan *ExtVidOutChan::ExternalOut_;

HBITMAP ExtVidOutChan::hbm_ = NULL;

const COLORREF Magenta = RGB( 0xff, 0, 0xff );             // magenta
COLORREF KeyColor;

HWND ghwndVideo = NULL ;

const NTSC_Width  = 640;
const NTSC_Height = 480;
const PAL_Width   = 768;
const PAL_Height  = 576;




#include <io.h>

#define HDIB         HANDLE
#define PALVERSION   0x300

HANDLE         AllocRoomForDIB( BITMAPINFOHEADER, HBITMAP );
HANDLE         ReadDIBFile( int );
BOOL           SafeRead( int, LPSTR, DWORD );
HDIB      FAR  BitmapToDIB( HBITMAP, HPALETTE );
HPALETTE  FAR  CreateDIBPalette( HDIB );
WORD      FAR  DIBNumColors( LPSTR );
HBITMAP   FAR  DIBToBitmap( HDIB, HPALETTE );
LPSTR     FAR  FindDIBBits( LPSTR );
HDIB      FAR  LoadDIB( LPCSTR );
WORD      FAR  PaletteSize( LPSTR );





/* Method: ExtVidOutChan::GetScreenParams
 * Purpose: Obtains desktop size and bpp
 * Input: bpp: int &, reference to the current bpp
 */
void ExtVidOutChan::GetScreenParams( int &bpp, int &width )
{
   HDC ScreenDC = GetDC( NULL );         // get screen DC

   bpp = GetDeviceCaps( ScreenDC, BITSPIXEL );
   ReleaseDC( NULL, ScreenDC );

   TRect r;
   GetWindowRect( GetDesktopWindow(), &r );
   width = r.Width();
}


/* Method: ExtVidOutChan::Create
 * Purpose: Creates the object of type ExtVidOutChan and returns pointer to it
 * Input: None
 * Output: ExtVidInChan *
 * Note: This creation method is used to prevent multiple openings of the same
 *   video channel by different applications
 */
ExtVidOutChan *ExtVidOutChan::Create()
{
   RegUtil Storage( "SOFTWARE\\Brooktree\\Video" );

   // working variable
   DWORD dwTmp = 0;

   // get Registry key to check if Preview mode is to be used
   // Registry key "PreviewOnly" = 0 or not exist -> attempt to create true overlay
   //                            = 1              -> don't create overlay, use Preview
   if ( ( Storage.GetValue( &dwTmp, "PreviewOnly" ) == Success ) &&
        ( dwTmp == 1 ) )
   {
      ExternalOut_ = NULL;
      return NULL;
   }

   // not forcing to use Preview, try to create true overlay now...
   if ( Created_ == true ) {
      ErrorDialog( XMSG( IDS_EXTVIDOUT ) );
      return NULL;
   }
   Created_ = true;
   ExternalOut_ = new ExtVidOutChan();
   ExternalOut_->CreatePalette();

   if ( DDI_Init() != Success ) {
      static bool DidDialog = false;
      if ( !DidDialog ) {
         ErrorDialog( XMSG( IDS_LAMEDDI ) );
         DidDialog = true;
      }
      delete ExternalOut_;
      ExternalOut_ = NULL;
      return NULL;
   }
   // let VxD know the address of the flag that is set to true when full-screen
   // DOS box is up
   ExternalOut_->CallVxD( FNStopFlag, &ExternalOut_->dwStopFlag_ );
   return  ExternalOut_;
}

/* Method: ExtVidInChan::OpenVxDChannel
 * Purpose: Opens the connection to the VxD Channel
 * Input: None
 * Output: void
 */
ErrorCode ExtVidOutChan::OpenVxDChannel( DWORD dnDevNode )
{
   if ( VxDHandle_ )
      return Success;
   return VfWChannel::OpenVxDChannel( VS_Preview, dnDevNode );
}

/* Method: ExtVidOutChan::SrcRect
 * Purpose:
 * Input:
 * Output:
 */
WORD ExtVidOutChan::SrcRect(  LPRECT lpRect, LPARAM Flags )
{
   switch ( Flags ) {
   case VIDEO_CONFIGURE_SET:
   case VIDEO_CONFIGURE_SET | VIDEO_CONFIGURE_CURRENT: {
      // store the position and dimentions of the image to be digitized
      // they will be used for the video in channel and optionally
      // for the external video out channel ( preview )

      TRect tmpRect( *lpRect );
      ColorSpace tmp( CF_RGB16 );
      SIZE ImageSize = tmpRect.Size();
      if ( tmp.CheckDimentions( ImageSize ) == false )
         return DV_ERR_NONSPECIFIC;

      if ( SourceRect_ != tmpRect ) {
         SourceRect_ = tmpRect;
         moved_ = 2;
      } else
         if ( DDI_GetFlags() & USE_OVERLAY && moved_ != 2 )
            moved_ = true;
      break;
   }
   case VIDEO_CONFIGURE_GET | VIDEO_CONFIGURE_CURRENT:
      *lpRect = SourceRect_;
      break;
   case VIDEO_CONFIGURE_GET | VIDEO_CONFIGURE_MAX: {
      // At this point, we don't know which video format (NTSC, PAL...) we will
      // be using since there is no guarentee that External Video In is created.
      // Thus there is no need to create different sizes for NTSC vs PAL.
      TRect r( 0, 0, 640, 480 );
      *lpRect = r;
      break;
   }
   default:
      return DV_ERR_NOTSUPPORTED;
   }
   return DV_ERR_OK;
}

/* Method:  ExtVidOutChan::DstRect
 * Purpose:
 * Input:
 * Output:
 */
WORD ExtVidOutChan::DstRect(  LPRECT lpRect, LPARAM Flags )
{
   switch ( Flags ) {
   case VIDEO_CONFIGURE_SET:
   case VIDEO_CONFIGURE_SET | VIDEO_CONFIGURE_CURRENT: {
      // store the position and dimentions of the image to be digitized
      // they will be used for the video in channel and optionally
      // for the external video out channel ( preview )
      TRect tmpRect( *lpRect );
      ColorSpace tmp( CF_RGB16 );
      TSize ImageSize = tmpRect.Size();
      if ( tmp.CheckDimentions( ImageSize ) == false )
         return DV_ERR_NONSPECIFIC;

      if ( DesktopRect_ != TRect( *lpRect ) ) {
         CCBlock_.Init( OverlayWin_ );
         DesktopRect_ = *lpRect;
         moved_ = true;
         if(DDI_GetFlags() & USE_OVERLAY)
            ExternalOut_->DoDDI();
      }
      break;
   }
   case VIDEO_CONFIGURE_GET | VIDEO_CONFIGURE_CURRENT:
      *lpRect = DesktopRect_;
      break;
   case VIDEO_CONFIGURE_GET | VIDEO_CONFIGURE_MAX: {
      // At this point, we don't know which video format (NTSC, PAL...) we will
      // be using since there is no guarentee that External Video In is created.
      // Thus there is no need to create different sizes for NTSC vs PAL.
      TRect r( 0, 0, 640, 480 );
      *lpRect = r;
      break;
   }
   default:
      return DV_ERR_NOTSUPPORTED;
   }
   return DV_ERR_OK;
}

/* Method:   ExtVidOutChan::Caps
 * Purpose:
 * Input:
 * Output:
 */
WORD ExtVidOutChan::Caps( LPCHANNEL_CAPS lpCaps, LPARAM lSize )
{
   // structure should be of the right size
   if ( lSize != sizeof( CHANNEL_CAPS ) )
      return DV_ERR_NONSPECIFIC;

// this 'preview' channel can scale and clip
   lpCaps-> dwFlags = VCAPS_OVERLAY | VCAPS_CAN_SCALE | VCAPS_SRC_CAN_CLIP;

   // Pisces can handle DWORD aligned addresses only
   lpCaps-> dwSrcRectXMod = 4;
   lpCaps-> dwSrcRectYMod = 1;

   lpCaps-> dwSrcRectWidthMod  = 4;
   lpCaps-> dwSrcRectHeightMod = 1;

   lpCaps-> dwDstRectXMod = 4;
   lpCaps-> dwDstRectYMod = 1;

   lpCaps-> dwDstRectWidthMod  = 4;
   lpCaps-> dwDstRectHeightMod = 1;

   return DV_ERR_OK;
}

/* Method:  ExtVidOutChan::VidConfigStorage
 * Purpose:
 * Input:
 * Output:
 */
#pragma argsused
WORD ExtVidOutChan::VidConfigStorage( UINT uiMsg, LPARAM lParam1, LPARAM lParam2 )
{
   return DV_ERR_OK;
}

/* Method:  ExtVidOutChan::StreamMsg
 * Purpose:
 * Input:
 * Output:
 */
#pragma argsused
WORD ExtVidOutChan::StreamMsg( UINT uiMsg, LPARAM lParam1, LPARAM lParam2 )
{
   //  lParam1	 -   LPVIDEO_STREAM_INIT_PARMS
   //
   //  lParam2	 -   sizeof (LPVIDEO_STREAM_INIT_PARMS)
   //
   switch ( uiMsg ) {
   case DVM_STREAM_INIT:
      StartOverlay();
      break;
   case DVM_STREAM_FINI:
      StopOverlay();
      break;
   default:
      return DV_ERR_NOTSUPPORTED;
   }
   return DV_ERR_OK;
}

extern "C" void WINAPI _export PreviewVxDCallback()
{
   //asm int 3;
}

/* Function: MassageRectangle
 * Purpose: Scales the rectangle
 * Input: dwScale: DWORD, scale factor multiplied by a 1000
 * Output: None
 */
void MassageRectangle( TRect &r, DWORD dwScale )
{
   if ( !dwScale )
      return;
   r.left   = int( (DWORD)r.left   * 1000 / dwScale );
   r.right  = int( (DWORD)r.right  * 1000 / dwScale );

   // width must be DWORD-aligned
   r.right &= ~3;
}

static LPVIDEOHDR AllocVideoHdr()
{
   HGLOBAL hTmp = GlobalAlloc( GPTR, sizeof(VIDEOHDR) );
   LPVIDEOHDR lpTmp = (LPVIDEOHDR)GlobalLock( hTmp );
   GlobalPageLock( hTmp );
   return lpTmp;
}


LPVIDEOHDR OddLive = NULL ;
LPVIDEOHDR EvenLive = NULL ;

/* Method: ExtVidOutChan::Start
 * Purpose: Start digitization
 * Input: None
 * Output: None
 */
void ExtVidOutChan::Start()
{
   if ( Overlaying_ == true )
      return;

   // need to shutdown frameserver if already in preview mode
   VidInChan::GetVidInChan().StreamReset(true) ;


   Overlaying_ = true;

   OddLive = AllocVideoHdr() ;
   EvenLive = AllocVideoHdr() ;

   OddLive->lpData  = (LPBYTE) Surface_.dwAddress ;
   EvenLive->lpData = (LPBYTE) Surface_.dwAddress ;

   OddLive->dwFlags &= ~VHDR_DONE;
   OddLive->dwFlags |= VHDR_INQUEUE;

   EvenLive->dwFlags &= ~VHDR_DONE;
   EvenLive->dwFlags |= VHDR_INQUEUE;

   CallVxD( FNAddBufLin, OddLive );      // FNAddBufLin: add buffer noting that buffer address (lpData) does not need to be translated.
   CallVxD( FNAddBufLin, EvenLive );     //              The pointer originates in ddi.cpp, a 32 bit component. Thus the vxd
                                         //              can use the pointer without translation.   

   // make sure we do not ask RISC core to generate too big program

   int width, height;
   VideoFormat vidFmt = GetCurFormat();
   if ( IsABt848A() )
   {
      // width of PAL-NC is the same as NTSC
      width = ( GetCurSignal() != Signal_Analog ) ? (int)GetVidInfo().HActive :
                 ( IsNTSCCompatible( vidFmt ) || ( vidFmt == VFormat_PAL_NC ) ) ? NTSC_Width : PAL_Width;

      // height of PAL-NC is the same as PAL
      height = ( GetCurSignal() != Signal_Analog ) ? (int)GetVidInfo().VActive :
                  ( IsNTSCCompatible( vidFmt ) ? NTSC_Height : PAL_Height );
   }
   else
   {
      width  = IsNTSCCompatible( vidFmt ) ? NTSC_Width : PAL_Width;
      height = IsNTSCCompatible( vidFmt ) ? NTSC_Height : PAL_Height;
   }

   TRect digital( SourceRect_.TopLeft(),
                  TSize( min( width, DesktopRect_.Width() ),
                         min( height, DesktopRect_.Height() ) ) );

   ColorSpace SurfaceColSpace( Surface_.FourCC, (int)Surface_.dwbpp );

   MassageRectangle( digital, Surface_.dwScale );

   StartParms strt( SmallRect( digital ), SmallRect( digital ),
      SurfaceColSpace.GetColorFormat() );

   strt.dwPitch  = Surface_.dwPitch;
   strt.callback = (DWORD)PreviewVxDCallback;
   strt.tag      = (DWORD)this;
   strt.FrameRate = 30;   // in millisec
   strt.ImageSize = GetBMI().biSizeImage;

   CallVxD( FNStart, &strt );

   // need to StartMonitor() here because it could've been stopped in DoDDI()
   StartMonitor();
}

/* Method: ExtVidOutChan::StartOverlay()
 * Purpose: Begins the overlay
 * Input: None
 * Output: None
 */
void ExtVidOutChan::StartOverlay()
{
   if ( Overlaying_ || dwStopFlag_ )
      return;

   Overlaying_ = true;

   StartMonitor();

   videoCallback( MM_DRVM_OPEN, 0L );        // Notify app we're open.
}

void ExtVidOutChan::Stop()
{
   if ( Overlaying_ == false )
      return;

   Overlaying_ = false;

   if ( Timer_ ) {
      KillTimer( NULL, Timer_ );
      Timer_ = NULL;
   }

   // stop Pisces and delete all the risc programs
   CallVxD( FNStop );
   // get rid of memory pointers
   CallVxD( FNFlushQue );

   if ( OddLive ) {
      GlobalPageUnlock( (HGLOBAL)GlobalHandle( HIWORD( OddLive ) ) );
      GlobalFree( (HGLOBAL)GlobalHandle( HIWORD( OddLive ) ) );

   }
   if ( EvenLive ) {
      GlobalPageUnlock( (HGLOBAL)GlobalHandle( HIWORD( EvenLive ) ) );
      GlobalFree( (HGLOBAL)GlobalHandle( HIWORD( EvenLive ) ) );
   }
   OddLive = EvenLive = NULL ;

}

/* Method: ExtVidOutChan::StopOverlay()
 * Purpose: Begins the overlay
 * Input: None
 * Output: None
 */
void ExtVidOutChan::StopOverlay()
{
   if ( Overlaying_ == false )
      return;

   // stop CC decode and display
   CCBlock_.StopDecode();

   Stop();

   if ( !( DDI_GetFlags() & USE_OVERLAY ) )
      InvalidateRect( NULL, NULL, true );   // clear out trash behind

   // hide the overlay
   DDI_GetData( NULL );

   OverlayWin_ = NULL;
   moved_ = true;
   videoCallback( MM_DRVM_CLOSE, 0L );       // Notify app we're closed

   D1( "\n\rStopped overlay\n" );
}

/* Method: ExtVidOutChan::DoDDI
 * Purpose: Interacts with DDI to obtain the cliplist and restarts the stream
 * Input: None
 * Output: None
 * Note: Called from the EnumWindows callback
 */
WORD ExtVidOutChan::DoDDI()
{
   static bool inside = false;

   if ( inside || dwStopFlag_ )
      return DV_ERR_OK;

   inside = true;

   Surface_.Clip = moved_;

   DDI_GetData( &Surface_ );

   // Surface_.Clip is a dual purpose variable. Valuies of 0,1, and 2 are special.
   // Values greater than 2 are assumed to be valid pointers to clip lists.
   if ( Surface_.Clip > 1 ) {

      Stop(); // need to stop in both overlay and primary

      if (Surface_.Clip > 2) {
         DWORD dwFlags = DDI_GetFlags();
         if ( !( dwFlags & USE_OVERLAY ) ) {
            CallVxD( FNClip, NULL, Surface_.Clip ); // clip info needed for primary only
            InvalidateRect( NULL, NULL, true ); // repaint in primary mode only
         }
      }
      Start();
   }
   moved_ = false;
   inside = false;
   return DV_ERR_OK;
}

/* Function:
 * Purpose: Determines the appropriate color key
 * Input: hWin: HWND
 * Output: COLORREF: color key found
 */
COLORREF WorkOutTheColorKey( HWND hWin )
{
   COLORREF cref, ColorKey;
   cref = Magenta;

   HDC ScreenDC = GetDC( NULL );         // get screen DC

   int bpp = GetDeviceCaps( ScreenDC, BITSPIXEL );

   if ( bpp == 8 ) {
      HDC  hDC;
      LPLOGPALETTE pPal;
      HPALETTE hPal;
      HGLOBAL  hgl;

      hgl = (LPLOGPALETTE)GlobalAlloc( GHND,
         sizeof( LOGPALETTE ) + 256 * sizeof( PALETTEENTRY ) );
      pPal = (LPLOGPALETTE)GlobalLock( hgl );

      pPal->palVersion = 0x300;
      pPal->palNumEntries = 256;

      hDC  = GetDC( hWin );

      GetSystemPaletteEntries( hDC, 0, 256, pPal->palPalEntry );
      hPal = ::CreatePalette( pPal );

      ColorKey = GetNearestPaletteIndex( hPal, cref );

      DeleteObject( hPal );
      GlobalUnlock( hgl );
      GlobalFree( hgl );
      ReleaseDC( hWin, hDC );

   } else {
      HBITMAP  hbitmap;
      HDC      cdc;
      DWORD    pixel;

      hbitmap = CreateCompatibleBitmap( ScreenDC, 1, 1 );
      if ( NULL == hbitmap )
         return 0;

      cdc = CreateCompatibleDC( ScreenDC );
      SelectObject( cdc, hbitmap );

      SetPixel( cdc, 0, 0, cref );
      GetBitmapBits( hbitmap, sizeof( pixel ), &pixel );
      DeleteDC( cdc );
      DeleteObject( hbitmap );

      ColorKey = bpp == 16 ? (COLORREF)(WORD)pixel : pixel;
   }
   ReleaseDC( NULL, ScreenDC );

   return ColorKey;
}

/* Function: PaintWindow
 * Purpose: Paints video window magenta
 */
void PaintWindow( HWND win )
{
   
   if (!win)	//TT - Prevent magenta bar on left-side of screen.
	  return;

   ghwndVideo = win ;

   TRect Client;
   GetClientRect( win, &Client );

   const COLORREF ColorKey = Magenta;

   HBRUSH CKBrush = CreateSolidBrush( ColorKey );
   HDC dc = GetDC( win );
   FillRect( dc, &Client, CKBrush );
   DeleteObject( CKBrush );
   ReleaseDC( win, dc );
}

const int IL_NO_UPDATE_1=0;  
const int IL_NO_UPDATE_2=1;
const int IL_UPDATE=2;
static HWND hForeGrndWnd;
static HWND hPreviousWnd=NULL;
WORD ExtVidOutChan::ShouldImageScreenUpdate()
{
    TRect rcImage;
    GetClientRect( hImageWin_, &rcImage );

    HDC hImageDC = GetDC( hImageWin_ );
    DWORD dwIOffset = GetDCOrg( hImageDC );
    ReleaseDC( hImageWin_, hImageDC );

    // Get endpoints of Image
    WORD wIxs = LOWORD(dwIOffset);
    WORD wIys = HIWORD(dwIOffset);
    WORD wIxe  = wIxs  * rcImage.Width();
    WORD wIye  = wIys  * rcImage.Height();

    TPoint pt[4];
    pt[0].x = wIxs-1;
    pt[0].y = wIys-1;

    pt[1].x = wIxe+1;
    pt[1].y = wIys-1;

    pt[2].x = wIxs-1;
    pt[2].y = wIye+1;

    pt[3].x = wIxe+1;
    pt[3].y = wIye+1;

    hForeGrndWnd = NULL;
    for (int i = 0; i< 4; i++ )
    {
        if ((hForeGrndWnd = WindowFromPoint(pt[i])) != NULL )
            break;
    }

    int nMsg;
    if ( (!hPreviousWnd || !hForeGrndWnd) )
        nMsg = IL_UPDATE;
    else
    if ( !hPreviousWnd && hForeGrndWnd ){
        nMsg = IL_NO_UPDATE_2;
        hPreviousWnd = hForeGrndWnd;
    }
    else
    if ( hPreviousWnd && !hForeGrndWnd ){
        nMsg = IL_NO_UPDATE_1;
        hPreviousWnd = hForeGrndWnd;
    }
    else
    if ( hPreviousWnd && hForeGrndWnd && (hForeGrndWnd == hPreviousWnd))
        nMsg = IL_NO_UPDATE_2;
    else
	{
       nMsg = IL_UPDATE;
       hPreviousWnd = hForeGrndWnd;
       TRect rcFG;
       GetClientRect( hForeGrndWnd, &rcFG );

       HDC hFGDC = GetDC( hForeGrndWnd );
       DWORD dwFOffset = GetDCOrg( hFGDC );
       ReleaseDC( hForeGrndWnd, hFGDC );

       // Get endpoints of Image
       WORD wFxs = LOWORD(dwFOffset);
       WORD wFys = HIWORD(dwFOffset);
       WORD wFxe  = wIxs  * rcFG.Width();
       WORD wFye  = wIys  * rcFG.Height();
	   if ( (wFxs <= wIxs) && (wFxe >= wIxe) && (wFys <= wIys) && ( wFye >= wIye ))
		nMsg = IL_NO_UPDATE_2;
	}
    return nMsg;
}

/* Method: ExtVidOutChan::Update
 * Purpose: Handles DVM_UPDATE message
 * Input: DispWin: HWND - handle of the overlay window
 *   UpdDC: HDC - DC to be repainted
 * Output:  error code
 */
WORD ExtVidOutChan::Update( HWND DispWin, HDC )
{
   static bool inside = false;

   if ( (ExternalOut_->ShouldImageScreenUpdate()==IL_UPDATE) && ExternalOut_->bImageLoaded_)
        bDoIt_ = true;
   else if (ExternalOut_->bImageLoaded_ )
   {
        if ((ExternalOut_->ShouldImageScreenUpdate()==IL_NO_UPDATE_2) && !IsWindowVisible(hImageWin_) )
        {
            if (IsWindowVisible(hImageWin_))
            {
            	bDoIt_ = false;
					return DV_ERR_OK;
				}
      		else
            	bDoIt_ = true;

		 }
   }

   if ( ExternalOut_ && ExternalOut_->bImageLoaded_ && bDoIt_)
   {
      bUpdateImage = true;
      ExternalOut_->ImageLoad(hInstance_, hImageWin_, lpBitmap_ );
	  	bDoIt_=false;
   }
   if ( inside || dwStopFlag_ ) 
      return DV_ERR_OK;
   inside = true;

   Surface_.hWin = DispWin;

   // recreate the primary surface every time, so all windows get updated
   if ( OverlayWin_ != DispWin && Overlaying_ ) {
      if ( !KeyColor || !SingleKeyColor_ )
         KeyColor = WorkOutTheColorKey( DispWin );

      if ( DDI_CreateSurface( DispWin, KeyColor,
           palCurrent.palVersion ? palCurrent.palPalEntry : NULL ) != Success ) {
         ErrorDialog( XMSG( IDS_DDIFAIL ) );
         StopOverlay();
         // note that variable 'inside' is not reset to prevent extra calls
         PaintWindow( DispWin );
         return DV_ERR_NONSPECIFIC;
      }
      moved_ = 2;
      OverlayWin_ = DispWin;

      CCBlock_.Init( DispWin );
      if ( !DoCC_ )
         CCBlock_.StopDecode();
      else
         CCBlock_.StartDecode();

      // obtain all the information from the DDI
      DoDDI();

      // if using overlay need to start here
      if ( DDI_GetFlags() & USE_OVERLAY ) {
         Stop();
         Start();
      }
   }
   if ( !KeyColor || !SingleKeyColor_ ) {
      KeyColor = WorkOutTheColorKey( DispWin );
      DDI_SetKeyColor( KeyColor );
   }

   int bpp, width;
   // check for the quickres change
   GetScreenParams( bpp, width );

   if ( bpp != OrigBpp_ || width != OrigWidth_ ) {

      Stop();

      OrigBpp_   = bpp;
      OrigWidth_ = width;

      SetFocus( OverlayWin_ );
      DDI_Close();

      DDI_Init();
      KeyColor = WorkOutTheColorKey( OverlayWin_ );

      if ( DDI_CreateSurface( OverlayWin_, KeyColor,
           palCurrent.palVersion ? palCurrent.palPalEntry : NULL ) != Success ) {
         ErrorDialog( XMSG( IDS_DDIFAIL ) );
         return DV_ERR_NONSPECIFIC;
      }
      DoDDI();
      // if using overlay need to start here
      if ( DDI_GetFlags() & USE_OVERLAY ) {
         Stop();
         Start();
      }
   }
   DWORD dwFlags = DDI_GetFlags();
   if ( dwFlags & USE_OVERLAY ) {
      PaintWindow( OverlayWin_ );
     // RestartCC();
   }
   if ( dwFlags & USE_PRIMARY ) {
      if ( palCurrent.palVersion )
         DDI_DoPalette();
      //PauseCC();
   }

   int height;
   if ( IsABt848A() )
      // height of PAL-NC is the same as PAL
      height = ( GetCurSignal() != Signal_Analog ) ? (int)VidInfo_.linesPerField :
                 ( IsNTSCCompatible( GetCurFormat() ) ? 240 : 288 );
   else
      height = IsNTSCCompatible( GetCurFormat() ) ? 240 : 288;

   if ( DesktopRect_.Height() < height )
      PauseCC();
   else
      //if ( !( dwFlags & USE_PRIMARY ) )
         RestartCC();
   inside = false;
   return DV_ERR_OK;
}

void _export ExtVidOutChan::MonitorProc( HWND, UINT, UINT, DWORD )
{
   if ( OverlayWin_ )
   {
      ExternalOut_->DoDDI();
      if ( ExternalOut_->DoCC_ )
         ExternalOut_->CCBlock_.Display();
   }
}

const MonitorPeriod = 200;

/* Method: ExtVidOutChan::StartMonitor
 * Purpose: Creates timer so we can monitor the windows
 * Input: None
 * Output: None
 */
void ExtVidOutChan::StartMonitor()
{
   if ( !Timer_ )
      Timer_ = SetTimer( NULL, 0, MonitorPeriod, MonitorProc );
}

/* Method: ExtVidOutChan::VidDialog
 * Purpose:
 * Input:
 * Output:
 */
WORD ExtVidOutChan::VidDialog( HWND hParent, LPARAM lFlags )
{
   RegUtil Storage( "SOFTWARE\\Brooktree\\Video" );

   // working variable
   DWORD dwTmp = 0;
   bool hasVideoDisplayDialog = true;  // default = Video Display Dialog available

   // get Registry key to check if video display dialog is available or not
   // Registry key "VideoDisplayDialog" = 1 or not exist -> available
   //                                   = 0              -> not supported
   if ( ( Storage.GetValue( &dwTmp, "VideoDisplayDialog" ) == Success ) &&
        ( dwTmp == 0 ) )
      hasVideoDisplayDialog = false;

   // don't offer video display dialog unless registry key = 1
   if ( lFlags & VIDEO_DLG_QUERY )
      if ( hasVideoDisplayDialog )
	   	return DV_ERR_OK; // Support the dialog.
      else
         return DV_ERR_NOTSUPPORTED;

   if ( hasVideoDisplayDialog )
   {
      VideoDisplayDialog( hParent );
      return DV_ERR_OK;
   }
   else
      return DV_ERR_NOTSUPPORTED;
}

ErrorCode ExtVidOutChan::SetCCStatus( int st )
{
   if ( st & 0x7FFC ) // use bits 15, 1 and 0 only
      return Fail;

   DoCC_ = st & 0x8000;
   int chan = st & 0x0001 ? CCMODE_CC1 : CCMODE_CC2;
   if ( !DoCC_ || CCBlock_.GetChannel() != chan )
      CCBlock_.Erase();
   if ( !DoCC_ )
      CCBlock_.StopDecode();
   else
      CCBlock_.StartDecode();

   CCBlock_.SetChannel( chan );
   return Success;
}

int  ExtVidOutChan::GetCCStatus()
{
   int st = DoCC_ ? 0x8000 : 0;
   st |= CCBlock_.GetChannel() == CCMODE_CC1 ? 0x0001 : 0x0002;
   return st;
}

/* Function: ExtVidOutChan::PauseCC
 * Purpose: Temporarily disables the CC capture
 */
void ExtVidOutChan::PauseCC()
{
   if ( ExternalOut_ )
      ExternalOut_->CCBlock_.StopDecode();
}

/* Method: ExtVidOutChan::RestartCC
 * Purpose: Temporarily disables the CC capture
 */
void ExtVidOutChan::RestartCC()
{
   if ( ExternalOut_ )
      if ( ExternalOut_->DoCC_ )
         ExternalOut_->CCBlock_.StartDecode();
}

/* Function:
 * Purpose: Pauses CC display
 */
void PauseCC()
{
   ExtVidOutChan::PauseCC();
}

/* Function: RestartCC
 * Purpose:  Restarts CC display
 */
void RestartCC()
{
   ExtVidOutChan::RestartCC();
}

static HWND hTmp;
/* Function: PauseOverlay
 * Purpose: Stops overlay to let capture a single frame
 */
void PauseOverlay()
{
   if ( &ExtVidOutChan::GetExtVidOut() == NULL )
      return;
   hTmp = ExtVidOutChan::GetExtVidOut().GetOvlyWin();
   ExtVidOutChan::GetExtVidOut().StopOverlay();
}

/* Function: RestartOverlay
 * Purpose: Restarts overlay channel after a frame is captured
 */
void RestartOverlay()
{
   if ( &ExtVidOutChan::GetExtVidOut() == NULL )
      return;
   ExtVidOutChan::GetExtVidOut().StartOverlay();
   ExtVidOutChan::GetExtVidOut().Update( hTmp );
}

void ExtVidOutChan::Pause()
{
   CallVxD( FNPause );
}

void ExtVidOutChan::Continue()
{
   CallVxD( FNContinue );
}


DWORD ExtVidOutChan::GetOverlayFlags() 
{
    if ( !ExternalOut_ )  // Supposedly take care of the case when we are in PreviewOnly
        return 0;         // Values returned by this function are invalid if
                          // video is in preview mode.  It would only make sense since
                          // the function ONLY gets the status of Overlay or Primary Surface.
                          // TT
    return DDI_GetFlags(); 
}



ErrorCode ExtVidOutChan::ImageLoad( HINSTANCE hInst, HWND hWnd, LPCSTR szBitmap )
{
   bool  bImageLoadNew = bool( ( lpBitmap_ != szBitmap ) ? true : false );

   if( bImageLoadNew )  
   {
      // it's a new load image initial members
      bImageLoaded_  = false;
      hInstance_ = hInst;
      OverlayWin_ = hWnd;
      hImageWin_  = hWnd;
      lpBitmap_ = (LPSTR)szBitmap;
   }
   else  
   {
      // it's either reload the same image or updating
      if( !bUpdateImage )
      {
         return Success;
      }
   }
   // StopOverlay to hold off VXD from churning
   StopOverlay();
   HDC   hdcImage = CreateCompatibleDC( NULL );

   if( !hbm_ )
   {
      HDIB       hDIB = NULL;
      if( ( hDIB = LoadDIB( szBitmap ) ) == NULL )
      {
         return Fail;
      }
      else
      {
         HPALETTE      hPal = NULL;
         hPal = CreateDIBPalette( hDIB );
         if( ( hbm_ = DIBToBitmap( hDIB, hPal ) ) == NULL )
         {
            if( hPal )
            {
               DeleteObject(hPal);
            }
            GlobalFree( hDIB );
            return Fail;
         }
         if( hPal )
         {
            DeleteObject(hPal);
         }
         GlobalFree( hDIB );
      }
   }

   SelectObject( hdcImage, hbm_ );

   BITMAP     bmp;
   GetObject( hbm_, sizeof(bmp), &bmp );

   TRect      rc;
   GetClientRect( hWnd, &rc );
   HDC        hdc = GetDC( OverlayWin_ );

   HDC        h = GetDC( hWnd );
   DWORD      dwOffset = GetDCOrg( h );
   ReleaseDC( hWnd, h );

   if ( !StretchBlt( 
                     hdc, 
                     LOWORD(dwOffset), 
                     HIWORD(dwOffset),
                     rc.Width(),
                     rc.Height(),
                     hdcImage,
                     0,
                     0,
                     bmp.bmWidth,
                     bmp.bmHeight,
                     SRCCOPY
                     ) )
   {
      return Fail;
   }
   ReleaseDC( OverlayWin_, hdc );
   DeleteObject( hdcImage );
   UpdateWindow( OverlayWin_ );
   bUpdateImage = false;  
   bImageLoaded_ = true;  
   return Success;
}



ErrorCode ExtVidOutChan::UnLoadImage()
{

   if( hbm_ )
   {
      DeleteObject( hbm_ );
      hbm_ = NULL;
   }

   // Reset all members back to default state
   bImageLoaded_ = false;
   hInstance_ = NULL;
   hImageWin_  = NULL;
   lpBitmap_ = NULL;
   bUpdateImage  = false;  // As it should be
   StartOverlay();
   return Success;
}    
    

//    DIBUTIL START
//
//    The following functions were adapted from dibutil, a module
//    of wincap.  See MSDN for context and other dibutil fns.

LPSTR FAR FindDIBBits( LPSTR lpDIB )
{
   return ( lpDIB + *(LPDWORD)lpDIB + PaletteSize( lpDIB ) );
}


WORD FAR PaletteSize( LPSTR lpDIB )
{
   return( DIBNumColors( lpDIB ) * sizeof(RGBQUAD) );
}


WORD FAR DIBNumColors( LPSTR lpDIB )
{
   WORD  wBitCount;

   DWORD dwClrUsed;
   dwClrUsed = ((LPBITMAPINFOHEADER)lpDIB)->biClrUsed;
   if( dwClrUsed )
     return (WORD)dwClrUsed;

   wBitCount = ((LPBITMAPINFOHEADER)lpDIB)->biBitCount;

   //    return number of colors based on bits per pixel 
   switch( wBitCount )
      {
   case 1:
      return 2;

   case 4:
      return 16;

   case 8:
      return 256;

   default:
      return 0;
      }
}


HPALETTE FAR CreateDIBPalette( HDIB hDIB )
{
   LPLOGPALETTE lpPal;     
   HANDLE hLogPal;         
   HPALETTE hPal = NULL;   
   int i, wNumColors;      
   LPSTR lpbi;             
   LPBITMAPINFO lpbmi;     

   if( !hDIB )
      return NULL;

   lpbi = (LPSTR) GlobalLock( hDIB );

   lpbmi = (LPBITMAPINFO)lpbi;

   wNumColors = DIBNumColors( lpbi );

   if( wNumColors )
   {
      hLogPal = GlobalAlloc( GHND, sizeof(LOGPALETTE) + sizeof(PALETTEENTRY) * wNumColors );

      if( !hLogPal )
      {
         GlobalUnlock(hDIB);
         return NULL;
      }

      lpPal = (LPLOGPALETTE)GlobalLock(hLogPal);

      lpPal->palVersion = PALVERSION;
      lpPal->palNumEntries = wNumColors;

      for( i = 0; i < wNumColors; i++ )
      {
        lpPal->palPalEntry[i].peRed = lpbmi->bmiColors[i].rgbRed;
        lpPal->palPalEntry[i].peGreen = lpbmi->bmiColors[i].rgbGreen;
        lpPal->palPalEntry[i].peBlue = lpbmi->bmiColors[i].rgbBlue;
        lpPal->palPalEntry[i].peFlags = 0;
      }

      hPal = CreatePalette( lpPal );

      if( !hPal )
      {
         GlobalUnlock( hLogPal );
         GlobalFree( hLogPal );
         return NULL;
      }
   }

   GlobalUnlock( hLogPal );
   GlobalFree( hLogPal );
   GlobalUnlock( hDIB );

   return hPal;
}



HBITMAP FAR DIBToBitmap( HDIB hDIB, HPALETTE hPal )
{
   LPSTR    lpDIBHdr;
   LPSTR    lpDIBBits;  
   HBITMAP  hBitmap;            
   HDC      hDC;                    
   HPALETTE hOldPal = NULL;    

   if( !hDIB )
      return NULL;

   lpDIBHdr = (LPSTR) GlobalLock( hDIB );

   lpDIBBits = FindDIBBits( lpDIBHdr );

   hDC = GetDC( NULL );
   if( !hDC )
   {
      GlobalUnlock( hDIB );
      return NULL;
   }

   if( hPal )
      hOldPal = SelectPalette( hDC, hPal, FALSE );
   RealizePalette( hDC );

   hBitmap = CreateDIBitmap( 
                     hDC, 
                     (LPBITMAPINFOHEADER)lpDIBHdr, 
                     CBM_INIT,
                     lpDIBBits, 
                     (LPBITMAPINFO)lpDIBHdr, 
                     DIB_RGB_COLORS
                     );

   if( hOldPal )
      SelectPalette( hDC, hOldPal, FALSE );

   ReleaseDC( NULL, hDC );
   GlobalUnlock( hDIB );

   return hBitmap;
}


HANDLE AllocRoomForDIB( BITMAPINFOHEADER bi, HBITMAP hBitmap )
{
   DWORD              dwLen;
   HANDLE             hDIB;
   HDC                hDC;
   LPBITMAPINFOHEADER lpbi;
   HANDLE             hTemp;

   dwLen = bi.biSize + PaletteSize( (LPSTR)&bi );
   hDIB  = GlobalAlloc( GHND, dwLen );

   if( !hDIB )
      return NULL;

   lpbi  = (LPBITMAPINFOHEADER)(VOID FAR *)GlobalLock( hDIB );
   *lpbi = bi;

   hDC   = GetDC( NULL );
   GetDIBits(
            hDC, 
            hBitmap, 
            0, 
            (WORD)bi.biHeight,
            NULL, 
            (LPBITMAPINFO)lpbi, 
            DIB_RGB_COLORS
            );
   ReleaseDC( NULL, hDC );

   if( lpbi->biSizeImage == 0 )
      lpbi->biSizeImage = WIDTHBYTES( (DWORD)lpbi->biWidth * lpbi->biBitCount ) * lpbi->biHeight;

   dwLen = lpbi->biSize + PaletteSize( (LPSTR)&bi ) + lpbi->biSizeImage;

   GlobalUnlock( hDIB );

   if( NULL != (hTemp = GlobalReAlloc( hDIB, dwLen, 0 )) )
      return hTemp;
   else
      {
      GlobalFree( hDIB );
      return NULL;
      }
}


HDIB FAR LoadDIB( LPCSTR lpFileName )
{
   HDIB     hDIB;
   int      hFile;
   OFSTRUCT ofs;
   if( ( hFile = OpenFile( lpFileName, &ofs, OF_READ ) ) != -1 )
   {
      hDIB = ReadDIBFile( hFile );
      _lclose( hFile );
      return hDIB;
   }
   else
   {
      return NULL;
   }
}


HANDLE ReadDIBFile( int hFile )
{
   BITMAPFILEHEADER  bmfHeader;
//   DWORD             dwBitsSize;
   UINT              nNumColors; 
   HANDLE            hDIB;        
   HANDLE            hDIBtmp;  
   LPBITMAPINFOHEADER lpbi;
   DWORD             offBits;

//   dwBitsSize = filelength(hFile);

   hDIB = GlobalAlloc(GMEM_MOVEABLE,
       (DWORD)(sizeof(BITMAPINFOHEADER) + 256 * sizeof(RGBQUAD)));
   
   if (!hDIB) return NULL;

   lpbi = (LPBITMAPINFOHEADER)GlobalLock(hDIB);
   if (!lpbi) 
   {
     GlobalFree(hDIB);
     return NULL;
   }

   if (sizeof (BITMAPFILEHEADER) != _lread (hFile, (LPSTR)&bmfHeader, sizeof (BITMAPFILEHEADER)))
     goto ErrExit;

   if (bmfHeader.bfType != 0x4d42)	// 'BM' 
     goto ErrExit;

   if (sizeof(BITMAPINFOHEADER) != _lread (hFile, (LPSTR)lpbi, sizeof(BITMAPINFOHEADER)))
     goto ErrExit;

   if (lpbi->biSize == sizeof(BITMAPCOREHEADER))
     goto ErrExit;

   if (0 == (nNumColors = (UINT)lpbi->biClrUsed))
    {
      // no color table for 24-bit, default size otherwise
      if (lpbi->biBitCount != 24)
        nNumColors = 1 << lpbi->biBitCount; // standard size table 
    }

   if (lpbi->biClrUsed == 0)
     lpbi->biClrUsed = nNumColors;

   if (lpbi->biSizeImage == 0)
   {
     lpbi->biSizeImage = ((((lpbi->biWidth * (DWORD)lpbi->biBitCount) + 31) & ~31) >> 3)
			 * lpbi->biHeight;
   }

   GlobalUnlock(hDIB);
   hDIBtmp = GlobalReAlloc(hDIB, lpbi->biSize +
                        nNumColors * sizeof(RGBQUAD) +
                        lpbi->biSizeImage, 0);

   if (!hDIBtmp) 
     goto ErrExitNoUnlock;
   else
     hDIB = hDIBtmp;

   lpbi = (LPBITMAPINFOHEADER)GlobalLock(hDIB);

   _lread (hFile, (LPSTR)(lpbi) + lpbi->biSize, nNumColors * sizeof(RGBQUAD));

   offBits = lpbi->biSize + nNumColors * sizeof(RGBQUAD);

   if (bmfHeader.bfOffBits != 0L)
      _llseek(hFile, bmfHeader.bfOffBits, SEEK_SET);
   
   if (SafeRead(hFile, (LPSTR)lpbi + offBits, lpbi->biSizeImage))
     goto OKExit;

ErrExit:
    GlobalUnlock(hDIB);    
ErrExitNoUnlock:    
    GlobalFree(hDIB);
    return NULL;

OKExit:
    GlobalUnlock(hDIB);
    return hDIB;
}


BOOL SafeRead(int hFile, LPSTR lpBuffer, DWORD dwSize)
{
   char huge *lpInBuf = (char huge *)lpBuffer;
   int nBytes;

   // Read in the data in 32767 byte chunks (or a smaller amount if it's
   // the last chunk of data read)
   while (dwSize)
   {
      nBytes = (int)(dwSize > (DWORD)32767 ? 32767 : LOWORD (dwSize));
      if (_lread(hFile, (LPSTR)lpInBuf, nBytes) != (WORD)nBytes)
         return FALSE;
      dwSize -= nBytes;
      lpInBuf += nBytes;
   }
   return TRUE;
}

//    DIBUTIL END




