/* File: ExtVidout.h
 * Purpose: Contains definition of the ExtVidOutChan class
 * Author: sasha
 * $Log: EXVIDOUT.H $
 * Revision 2.15  1998/04/22 16:09:13  dtharp
 * Added call to FNCloseDriver to destructor.
 * Revision 2.14  1998/01/19 22:59:26  peter
 * Graft image load fixes to tip.
 * Revision 2.13.4.2  1997/12/20 01:49:31  peter
 * Revision 2.13.4.1  1997/05/28 17:02:52  peter
 * Duplicate revision
 * Revision 2.12.1.2  1997/05/28 16:06:38  trantq
 * Add support for Bt848ImageLoad & Bt848UnLoadImage. 
 * Bug ID 431
 * Revision 2.12.1.1  1997/05/20 16:29:40  trantq
 * Add feature LoadImage of DD Surface.  Bug ID 431
 * Revision 2.12  1997/05/20 16:29:40  trantq
 * -Merged in fix GetOverlayFlag function.  Bug ID 565
 * Revision 2.11.1.2  1997/05/20 16:29:40  trantq
 * - Fix GetOverlayFlag function.  Bug ID 565
 * Revision 2.11.1.1  1997/05/05 19:43:18  trantq
 * - Fix GetOverlayFlag function Bug ID 565.
 * Revision 2.11  1997/05/05 19:43:18  dtharp
 * 848A Merge: from 2.10.2.1
 * Revision 2.10.2.1  1997/02/12 19:31:05  rudyl
 * Pisces+ work.
 * Revision 2.10  1996/08/10 23:40:06  sasha
 * Added dwStopFlag_ and code to prevent starting of the channel when
 * VxD detects full-screen DOS box
 * Revision 2.9  1996/07/14 02:52:12  sasha
 * Removed window subclassing-related members and put new ones
 * that contain info about screen.
 * Revision 2.8  1996/07/13 22:31:33  sasha
 * Made moved_ into int. Made Update() public.
 * Revision 2.7  1996/07/13 00:39:49  sasha
 * Added members to support window subclassing to process 
 * WM_DISPLAYCHANGE message
 * Revision 2.6  1996/06/26 21:27:23  sasha
 * Added GetOverlayFlags() API
 * Revision 2.5  1996/06/26 19:07:30  sasha
 * Added Pause() and Continue() functions that freeze/unfreeze Pisces
 * Revision 2.4  1996/06/19 23:53:08  sasha
 * Added PauseOverlay() and RestartOverlay() to let capture frames bigger
 * then CIF
 * Revision 2.3  1996/06/13 21:53:05  sasha
 * Removed logical palette and use one more call to DDI interface:
 * DDI_DoPalette().
 * Revision 2.2  1996/06/12 00:47:12  sasha
 * Added hPal_ member to hold the palette for 8bpp mode
 * Revision 2.1  1996/06/10 20:13:02  ssm
 * MarkBeta1AsRev2
 * Revision 1.11  1996/06/05 19:51:33  sasha
 * Added functions for getting the CC file name and alert token
 * Revision 1.10  1996/06/05 04:10:39  sasha
 * Added functions to set CC file name and alert token
 * Revision 1.9  1996/06/04 23:58:03  sasha
 * Added PauseCC() and RestartCC() functions.
 * Revision 1.8  1996/05/30 21:13:24  sasha
 * Added SetCCStatus() and GetCCStatus()
 * Revision 1.7  1996/05/23 03:31:07  sasha
 * Added member of CCEngine class to the ext. vid. out channel
 * Revision 1.6  1996/03/27 23:32:02  sasha
 * Changed prototype for the DDI_CreatePrimarySurface()
 * Revision 1.5  1996/02/27 01:55:01  sasha
 * Added prototypes for two more DDI calls
 * Revision 1.4  1996/02/06 01:25:01  sasha
 * Changed DDI_CreateSurface() prototype
 * Revision 1.3  1996/01/22 20:31:18  sasha
 * Changed calls to OpenVxDChannel ( added an arument )
 * Revision 1.2  1996/01/04 23:21:12  sasha
 * First real check-in. Too many changes since previous version
 * Revision 1.1  1995/09/03 01:06:08  sasha
 * Initial revision
 */

#ifndef __EXVIDOUT_H
#define __EXVIDOUT_H

#ifndef __VFWCHAN_H
#include "vfwchan.h"
#endif

#ifndef __COLFRMAT_H
#include "colfrmat.h"
#endif

#ifndef __COLSPACE_H
#include "colspace.h"
#endif

#ifndef __CCENGINE_H
#include "ccengine.h"
#endif

#ifndef __DDI_PAR_H
#define __DDI_PAR_H
#include "ddi_par.h"
#endif

extern "C" DWORD WINAPI DDI_IsClipListChanged();
extern "C" DWORD WINAPI DDI_Init();
extern "C" DWORD WINAPI DDI_CreateSurface( HWND, COLORREF, LPVOID );
extern "C" DWORD WINAPI DDI_GetData( LPSURF_DATA lpData );
extern "C" DWORD WINAPI DDI_Close();
extern "C" DWORD WINAPI DDI_GetFlags();
extern "C" DWORD WINAPI DDI_SetKeyColor( DWORD dwCol );
extern "C" DWORD WINAPI DDI_DoPalette();


/* Class: ExtVidOutChan
 * Purpose: Implements functionality of the VIDEO_EXTERNALOUT channel
 * Attributes: Overlaying_: bool - set to 'true' during overlay process
 *   set to false when time comes to stop preview. Used to break the overlay loop
 * Operations: see VfWChannel for the responses to the VfW messages
 */
class ExtVidOutChan : public VfWChannel
{
   private:

      CCEngine        CCBlock_;
      static bool     Created_;

      TRect  DesktopRect_;
      TRect  SourceRect_;

      SURF_DATA Surface_;

      static HWND   OverlayWin_;
      static int    moved_;
      static UINT Timer_;
      static ExtVidOutChan *ExternalOut_;

	  // ImageLoad related members
	  static bool bImageLoaded_;
	  static HINSTANCE hInstance_;
	  static HWND hImageWin_;
	  static LPSTR lpBitmap_;
	  static bool bDoIt_;

     static HBITMAP  hbm_;

      // quickres related data/functions
      void GetScreenParams( int &bpp, int &width );
      int    OrigBpp_;
      int    OrigWidth_;

      bool   DoCC_;

      DWORD  dwStopFlag_;
      // overlay control data and functions

      void Stop();
      void Start();

      static BOOL CALLBACK _export ExtVidOutChan::WinEnumerator( HWND, LPARAM );
      void StartMonitor();
      WORD DoDDI();

      virtual WORD VidDialog( HWND, LPARAM );
      virtual WORD SrcRect( LPRECT, LPARAM );
      virtual WORD DstRect( LPRECT, LPARAM );
      virtual WORD Caps( LPCHANNEL_CAPS, LPARAM );
      virtual WORD StreamMsg( UINT, LPARAM, LPARAM );
      virtual WORD VidConfigStorage( UINT uiMsg, LPARAM lParam1, LPARAM lParam2 );

      ExtVidOutChan() : VfWChannel(), CCBlock_(), DoCC_( false ), dwStopFlag_( false )
      { GetScreenParams( OrigBpp_, OrigWidth_ ); }

      virtual ErrorCode OpenVxDChannel( DWORD dnDevNode );

   public:
	  // ImageLoad related functions
	  ErrorCode ImageLoad( HINSTANCE hInst, HWND hWnd, LPCSTR szBitmap );
	  ErrorCode UnLoadImage();
	  WORD      ShouldImageScreenUpdate();

      virtual WORD Update( HWND, HDC = 0 );
   
      static ExtVidOutChan *Create();
      static ExtVidOutChan &GetExtVidOut() { return *ExternalOut_; }

      static void RestartCC();
      static void PauseCC();
             void StartOverlay();
             void StopOverlay();
      static HWND GetOvlyWin() { return OverlayWin_; }
      static void SetOvlyWin( HWND hWin ) { OverlayWin_ = hWin; }
      static void MonitorProc( HWND = 0, UINT = 0, UINT = 0, DWORD = 0 );
                       

      ErrorCode SetCCStatus( int st );
      int  GetCCStatus();
      void SetCCFileName( LPCSTR name ) { CCBlock_.SetCCFileName( name ); }
      void SetCCAlert( LPCSTR token ) { CCBlock_.SetCCAlert( token ); }
      void GetCCFileName( LPSTR name, int cnt ) { CCBlock_.GetCCFileName( name, cnt ); }
      void GetCCAlert( LPSTR token, int cnt )   { CCBlock_.GetCCAlert( token, cnt ); }

      void  Pause();
      void  Continue();
	  DWORD GetOverlayFlags();

      ~ExtVidOutChan() { StopOverlay(); Created_ = false; DDI_Close(); CallVxD(FNCloseDriver);}
};


#endif

