/* File: VidChan.cpp
 * Purpose: contains implementation of VfW message processing and channels
 * Author: sasha
 * $Log: VFWCHAN.CPP $
 * Revision 2.8  1998/04/02 21:41:34  dtharp
 * Merge DT_VIDEOHDR branch. Implements a new buffer 
 * management scheme.
 * Revision 2.7.2.2  1998/03/27 16:31:44  dtharp
 * Clean up dead code
 * Revision 2.7.2.1  1998/03/27 15:24:14  dtharp
 * Branch for passing videohdrs around.
 * Revision 2.7  1997/05/19 20:48:21  dtharp
 * Bug Id 511: Added bCapturing_ member var and initialization.
 * Revision 2.6  1997/05/05 19:44:12  dtharp
 * 848A Merge: from 2.5.1.3
 * Revision 2.5.1.3  1997/02/14 23:40:15  rudyl
 * Removed all #ifdef BT848A. Use same binary files for both Bt848 and Bt848A.
 * Revision 2.5.1.2  1997/02/03 23:42:15  rudyl
 * First Pisces+ check in.
 * Revision 2.5.1.1  1996/12/20 01:29:58  rudyl
 * Start Pisces+ work.
 * Revision 2.5  1996/10/02 17:23:54  rudyl
 * Bug #353
 * Problem: Driver depends upon direct draw files
 * Solution: Added "SetPreviewOnly( bool par )" and "IsPreviewOnly()" in VfWChannel class
 * so that when driver is loading, Bt848ddi.dll is not linked in if registry key "PreviewOnly" = 1.
 * Also prevent calling DDI...() in this case.
 * Revision 2.4  1996/09/27 02:16:33  sasha
 * Initialize dwMicroSecPerFrame to 33333 instead of 33 to accomodate
 * the change in vidinch.cpp
 * Revision 2.3  1996/08/07 18:47:08  sasha
 * Added SingleKeyColor_ member. It is needed to support the Cirrus
 * card. May be not needed with newer 5446 drivers.
 * Revision 2.2  1996/06/12 00:49:23  sasha
 * Use PC_NOCOLLAPSE for palette flags. Use palette vesrion 0x300.
 * Revision 2.1  1996/06/10 20:13:03  ssm
 * MarkBeta1AsRev2
 * Revision 1.11  1996/06/06 18:18:21  rudyl
 * changed registry key from Bt848 or APisces --> Software\Brooktree\Bt848
 * Revision 1.10  1996/06/06 01:15:51  sasha
 * Removed EH. Use return codes now
 * Revision 1.9  1996/05/21 21:54:44  sasha
 * Moved constructor here from .h. Added  CurVidFormat_ member
 * Revision 1.8  1996/05/12 21:14:20  sasha
 * Added LoadVxD(). Added overloaded OpenVxDChannel().
 * Revision 1.7  1996/03/14 18:53:18  sasha
 * Added DisableAll() functions to make sure all channels are stopped after
 * DRV_DISABLE message
 * Revision 1.6  1996/02/19 03:40:41  sasha
 * Removed max() from AllocBuf()
 * Revision 1.5  1996/02/06 01:24:03  sasha
 * Moved palette code here
 * Revision 1.4  1996/01/22 20:33:59  sasha
 * Changed calls to VxD ( added an arument )
 * Revision 1.3  1996/01/10 21:27:43  sasha
 * Changed RECT to TRect
 * Revision 1.2  1996/01/04 23:21:13  sasha
 * First real check-in. Too many changes since previous version
 * Revision 1.1  1995/09/07 06:16:46  sasha
 * Initial revision
 */


#include "vfwchan.h"
#include "dbgprint.h"

TRect VfWChannel::MasterAnalogRect_;
bool  VfWChannel::Overlaying_;
bool  VfWChannel::PreviewOnly_;
struct VfWChannel::tagPal VfWChannel::palCurrent;
VideoFormat VfWChannel::CurVidFormat_;
bool VfWChannel::SingleKeyColor_ = false;
SignalFormat VfWChannel::CurSigFormat_;
VideoInfo VfWChannel::VidInfo_;
bool VfWChannel::bCapturing_ = false ;

void VfWChannel::SetMasterAnalogRect( const LPRECT lpRect )
{ MasterAnalogRect_ = *lpRect; }

void VfWChannel::GetMasterAnalogRect( LPRECT const lpRect )
{ *lpRect = MasterAnalogRect_; }


VidChanArray VfWChannel::CreatedChannels_( 10 );

VfWChannel::VfWChannel() : /*VideoBuffers_(),*/ VxDHandle_( 0 ), OpenFlag_( false )
{
   memset( &VidOpenPar_, '\0', sizeof( VidOpenPar_ ) );
   memset( &VidStrmInitPar_, '\0', sizeof( VidStrmInitPar_ ) );
   memset( &bmHeader_, '\0', sizeof( bmHeader_ ) );
   bmHeader_.biSize = sizeof( bmHeader_ );
   bmHeader_.biPlanes = 1;
   VidStrmInitPar_.dwMicroSecPerFrame = 33333;
   AddChannel();
}


void VfWChannel::DisableAllFunc( VfWChannel *&ToDisable, void * )
{
   ToDisable->StreamMsg( DVM_STREAM_FINI, 0, 0 );
}

/* Method: VfWChannel::Disable
 * Purpose:
 * Input: None
 * Output: None
 */
void VfWChannel::Disable()
{
   GlobalEnable_ = false;
   CreatedChannels_.ForEach( DisableAllFunc, NULL );
}

/* Method: VfWChannel::Enable
 * Purpose:
 * Input: None
 * Output: None
 */
void VfWChannel::Enable()
{
   GlobalEnable_ = true;
}

VxDIface  VfWChannel::PiscesEntry_;
bool      VfWChannel::GlobalEnable_ = false;

/* Method: VfWChannel::CheckBuffer
 * Purpose:
 * Input: lpNewBuffer: LPVIDEOHDR - video header describing the buffer
 * Output: WORD - error code
 */
WORD VfWChannel::CheckBuffer( LPVIDEOHDR lpNewBuffer )
{
   // Return error if no node passed.
   if ( !lpNewBuffer )
      return DV_ERR_NONSPECIFIC;

   // Return error if buffer has not been prepared.
   if ( !( lpNewBuffer->dwFlags & VHDR_PREPARED ) )
      return DV_ERR_UNPREPARED;

   // Return error if buffer is already in the queue.
   if ( lpNewBuffer->dwFlags & VHDR_INQUEUE )
      return DV_ERR_NONSPECIFIC;

   if ( lpNewBuffer->dwBufferLength < bmHeader_.biSizeImage )
      return DV_ERR_NONSPECIFIC;

//   VideoBuffers_.Put( lpNewBuffer );
   return DV_ERR_OK;
}

/* Method: VfWChannel::LoadVxD
 * Purpose: Established connection to the VxD
 * Input: None
 * Output: None
*/
ErrorCode VfWChannel::LoadVxD()
{
   if ( PiscesEntry_.IsInited() == false ) {
      PiscesEntry_.LoadVxD();
      if ( PiscesEntry_.IsInited() == false )
         return Fail;
      PiscesEntry_.CallVxD( FNVer, 0 );
   }
   return Success;
}

/* Method: VfWChannel::OpenVxDChannel
 * Purpose: Closes the connection to the VxD Channel
 * Input: DWORD: device node
 * Output: void
 */
ErrorCode VfWChannel::OpenVxDChannel( DWORD )
{
   return Fail;
}

ErrorCode VfWChannel::OpenVxDChannel( VideoStream aStream, DWORD dnDevNode )
{
   ChanInfo parameters( aStream, &VxDHandle_ );
   CallVxD( FNOpenChan, &parameters, dnDevNode );
   if ( !VxDHandle_ ) {
      delete this;
      ErrorDialog( XMSG( IDS_VXDCHANNEL ) );
      return Fail;
   }
   return Success;
}

/* Method: VfWChannel::CloseVxDChannel
 * Purpose: Closes the connection to the VxD Channel
 * Input: None
 * Output: None
 */
void VfWChannel::CloseVxDChannel()
{
   if ( !VxDHandle_ )
      return;
   // pass value itself, as opposed to the open, we address was used
   CallVxD( FNCloseChan, NULL, VxDHandle_ );
   VxDHandle_ = NULL;
}

/* Method:  VidInChan::AllocBuf
 * Purpose: Handles the DVM_ALLOCBUFFER message
 * Input: ppBufAddr: LPBYTE * - pointer to the pointer that holds buffer's address
 * Output: error code
 */
WORD VfWChannel::AllocBuf( LPBYTE *ppBufAddr, DWORD dwSize )
{
   AllocBufStruct abs( ppBufAddr, dwSize + sizeof( VIDEOHDR ) );

   // ask VxD to perform the allocation
   CallVxD( FNAllocBuf, &abs );

   if ( *abs.ppBufAddr == NULL )
      return DV_ERR_NOMEM;  // for some reason allocation failed

   return DV_ERR_OK;
}

/* Method:  VfWChannel::FreeBuf
 * Purpose: Handles DVM_FREEBUFFER message
 * Input: pBufAddr - address of a buffer to free
 * Output: DV_ERR_OK
 */
WORD VfWChannel::FreeBuf( LPBYTE pBufAddr )
{
   AllocBufStruct abs( (LPBYTE*)pBufAddr, 0 );

   // politely return memory to the VxD
   CallVxD( FNFreeBuf, &abs );
   return DV_ERR_OK;
}


/* Method: VfWChannel::VidConfigMsg
 * Purpose:
 * Input:
 * Output:
 */
WORD VfWChannel::VidConfigMsg( UINT, LPARAM, LPARAM )
{
   return DV_ERR_NOTSUPPORTED;
}

/* Method: VfWChannel::VidDialog
 * Purpose:
 * Input:
 * Output:
 */
WORD VfWChannel::VidDialog( HWND, LPARAM )
{
   return DV_ERR_NOTSUPPORTED;
}

/* Method: VfWChannel::SrcRect
 * Purpose:
 * Input:
 * Output:
 */
WORD VfWChannel::SrcRect( LPRECT, LPARAM )
{
   return DV_ERR_NOTSUPPORTED;
}

/* Method:  VfWChannel::DstRect
 * Purpose:
 * Input:
 * Output:
 */
WORD VfWChannel::DstRect( LPRECT, LPARAM )
{
   return DV_ERR_NOTSUPPORTED;
}

/* Method: VfWChannel::Update
 * Purpose:
 * Input:
 * Output:
 */
WORD VfWChannel::Update( HWND, HDC )
{
   return DV_ERR_NOTSUPPORTED;
}

/* Method:  VfWChannel::CaptureFrame
 * Purpose:
 * Input:
 * Output:
 */
WORD VfWChannel::CaptureFrame( LPVIDEOHDR )
{
   return DV_ERR_NOTSUPPORTED;
}

/* Method:   VfWChannel::Caps
 * Purpose:
 * Input:
 * Output:
 */
WORD VfWChannel::Caps( LPCHANNEL_CAPS, LPARAM )
{
   return DV_ERR_NOTSUPPORTED;
}

/* Method:  VfWChannel::StreamMsg
 * Purpose:
 * Input:
 * Output:
 */
WORD VfWChannel::StreamMsg( UINT, LPARAM, LPARAM )
{
   DebugPrint((0,"VfWChannel::StreamMsg (Not Supported)\n"));
   return DV_ERR_NOTSUPPORTED;
}

/* Method:  VfWChannel::VidConfigStorage
 * Purpose:
 * Input:
 * Output:
 */
WORD VfWChannel::VidConfigStorage( UINT, LPARAM, LPARAM )
{
   return DV_ERR_NOTSUPPORTED;
}

// just a prototype
extern "C" BOOL WINAPI DriverCallback( DWORD dwCallback, UINT uFlags,
   HANDLE hDevice, UINT uMessage, DWORD dwUser, DWORD dwParam1,
   DWORD dwParam2 );

/* Method:
 * Purpose: Invokes the user supplied callback
 * Input: msg: WORD, message to send
 *   dw1: DWORD - generic argument
 */
void VfWChannel::videoCallback( WORD msg, DWORD dw1 )
{
   // invoke the callback function, if it exists.  dwFlags contains driver-
   // specific flags in the LOWORD and generic driver flags in the HIWORD
   if ( GetStrInitPar().dwCallback )
      DriverCallback( GetStrInitPar().dwCallback,  // Client's callback DWORD.
         HIWORD( GetStrInitPar().dwFlags ),        // Callback flags.
         (HANDLE)GetStrInitPar().hVideo,           // Handle to the device.
         msg,                                      // The message.
         GetStrInitPar().dwCallbackInst,           // Client's instance data.
         dw1,                                      // First DWORD.
         0 );                                      // Second DWORD not used.
}

void VfWChannel::SetBMI( LPBITMAPINFOHEADER lpbi )
{
   bmHeader_ = *lpbi;
   bmHeader_.biSizeImage =
      WIDTHBYTES( bmHeader_.biWidth * bmHeader_.biBitCount ) * bmHeader_.biHeight;
}

void VfWChannel::CreatePalette()
{
   HDC dc;
   HWND wnd;
   dc = GetDC( wnd = GetDesktopWindow() );
   int i = 16;
   int lim = 241;

   if ( !( GetDeviceCaps( dc, RASTERCAPS ) & RC_PALETTE ) ) {
      i = 0;
      lim = 256;
      palCurrent.palVersion = 0;
      palCurrent.palNumEntries = 0;
   } else {
      GetSystemPaletteEntries( dc, 0, GetDeviceCaps( dc, SIZEPALETTE ),
         palCurrent.palPalEntry );
      palCurrent.palVersion = 0x300;
      palCurrent.palNumEntries = 256;
   }

   for ( ; i < lim; i++ ) {
      int r, g, b, t;

      t = i - 16;

      g = t / 25;
      r = ( t - 25 * g ) / 5;
      b =  t - 25 * g - 5 * r;
      palCurrent.palPalEntry [i].peRed   = r * 255 / 4;
      palCurrent.palPalEntry [i].peGreen = g * 255 / 8;
      palCurrent.palPalEntry [i].peBlue  = b * 255 / 4;
      palCurrent.palPalEntry [i].peFlags = PC_NOCOLLAPSE;

   }
   ReleaseDC( wnd, dc );
}
