/* File: Vidchan.h
 * Purpose: contains declaration of VfW channels
 * Author: sasha
 * $Log: VFWCHAN.H $
 * Revision 2.7  1998/04/07 18:19:45  dtharp
 * Removed include for bufque.h
 * Revision 2.6  1998/04/02 21:42:09  dtharp
 * Merge DT_VIDEOHDR branch. Implements a new buffer 
 * management scheme.
 * Revision 2.5.3.1  1998/03/27 15:24:51  dtharp
 * Branch for passing videohdrs around.
 * Revision 2.5  1997/05/19 20:43:23  dtharp
 * Bug Id 511: Added bCapturing_ member var.
 * Revision 2.4  1997/05/05 17:51:36  dtharp
 * 848A Merge: from 2.3.1.3
 * Revision 2.3.1.3  1997/02/14 23:40:15  rudyl
 * Removed all #ifdef BT848A. Use same binary files for both Bt848 and Bt848A.
 * Revision 2.3.1.2  1997/02/03 23:42:18  rudyl
 * First Pisces+ check in.
 * Revision 2.3.1.1  1996/12/20 01:30:16  rudyl
 * Start Pisces+ work.
 * Revision 2.3  1996/10/02 17:09:30  rudyl
 * Bug #353
 * Problem: Driver depends upon direct draw files
 * Solution: Added "SetPreviewOnly( bool par )" and "IsPreviewOnly()" in VfWChannel class
 * so that when driver is loading, Bt848ddi.dll is not linked in if registry key "PreviewOnly" = 1.
 * Also prevent calling DDI...() in this case.
 * Revision 2.2  1996/08/07 18:47:42  sasha
 * Added SingleKeyColor_ member. It is needed to support the Cirrus
 * card. May be not needed with newer 5446 drivers.
 * Revision 2.1  1996/06/10 20:13:05  ssm
 * MarkBeta1AsRev2
 * Revision 1.12  1996/06/06 01:15:51  sasha
 * Removed EH. Use return codes now
 * Revision 1.11  1996/05/30 21:14:52  sasha
 * #include protos.h instead of dialogs.h
 * Revision 1.10  1996/05/21 21:55:06  sasha
 * Moved constructor to .cpp from here.  Added  CurVidFormat_ member
 * Revision 1.9  1996/05/12 21:13:56  sasha
 * Added LoadVxD(). Added overloaded OpenVxDChannel().
 * Revision 1.8  1996/04/17 20:11:36  sasha
 * Initialize VidStrmInitPar_.dwMicroSecPerFrame to 33.
 * Revision 1.7  1996/04/09 01:14:38  sasha
 * Changed debug.h to the dbg.h
 * Revision 1.6  1996/03/14 18:53:34  sasha
 * Added DisableAllFunc()
 * Revision 1.5  1996/02/06 01:24:17  sasha
 * Moved palette code here
 * Revision 1.4  1996/01/22 20:34:23  sasha
 * Changed calls to VxD ( added an arument )
 * Revision 1.3  1996/01/10 21:27:15  sasha
 * Added #include "dialogs.h" for dialog prototypes
 * Changed RECT to TRect
 * Revision 1.2  1996/01/04 23:21:14  sasha
 * First real check-in. Too many changes since previous version
 * Revision 1.1  1995/09/07 06:19:04  sasha
 * Initial revision
 */

#ifndef __VFWCHAN_H
#define __VFWCHAN_H

#ifndef __WINDOWS_H
#include <windows.h>
#endif

#ifndef __MMSYSTEM_H
#include <mmsystem.h>
#endif

#ifndef __MSVIDEO_H
#include <msvideo.h>
#endif

#ifndef __MSVIDDRV_H
#include <msviddrv.h>
#endif

#ifndef __MEM_H
#include <mem.h>
#endif

DWORD VideoOpen( LPVIDEO_OPEN_PARMS lpVidOpenPar );
HINSTANCE GetOurInstance();


#ifndef __DEBUG_H
#include "dbg.h"
#endif

#ifndef __VXDIFACE_H
#include "vxdiface.h"
#endif

#ifndef __API_H
#include "api.h"
#endif

#ifndef __REGUTIL_H
#include "regutil.h"
#endif

#ifndef __XCEPTION_H
#include "xception.h"
#endif

#ifndef __XMSGID_H
#include "xmsgid.h"
#endif

#ifndef CLASSLIB_ARRAYS_H
#include <classlib/arrays.h>
#endif

#ifndef __COMPAT_H
#include "compat.h"
#endif

#include "protos.h"

class VfWChannel;

/* Type: VidChanArray
 * Purpose: Defines an array that holds pointers to the VfWChannel objects
 */
typedef TArray<VfWChannel*> VidChanArray;

/* Class: VfWChannel
 * Purpose: This channel processes all VfW messages and communicates to the
 *   VxD through the VxDIface class
 * Attributes: VidOpenPar_:
 *   VidStrmInitPar_:
 *   bmHeader_:
 *   VideoBuffers_: VidBufQue - a queue of buffers used for streaming
 *   VxDHandle_: DWORD - handle returned by VxD to be used for the subsequent
 *   calls to it. Most likely address of a VxD video channel
 *   capture
 * Operations:  These operations are direct responses to the VfW messages
      virtual WORD AddBuffer( LPVIDEOHDR lpNewBuffer );
      virtual WORD AllocBuf( LPBYTE *Addr, DWORD dwSize );
      virtual WORD FreeBuf( LPBYTE lpAddr );
      virtual WORD VidConfigMsg( UINT uiMsg, LPARAM lParam1, LPARAM lParam2 );
      virtual WORD VidDialog( HWND, LPARAM );
      virtual WORD SrcRect( LPRECT, LPARAM );
      virtual WORD DstRect( LPRECT, LPARAM );
      virtual WORD Update( HWND, HDC );
      virtual WORD CaptureFrame( LPVIDEOHDR );
      virtual WORD Caps( LPCHANNEL_CAPS, LPARAM );
      virtual WORD StreamMsg( UINT, LPARAM, LPARAM );
      virtual WORD VidConfigStorage( UINT uiMsg, LPARAM lParam1, LPARAM lParam2 );
 */
class VfWChannel
{
   private:
      VIDEO_OPEN_PARMS        VidOpenPar_;
      VIDEO_STREAM_INIT_PARMS VidStrmInitPar_;
      BITMAPINFOHEADER        bmHeader_;

//      VidBufQue               VideoBuffers_;

      bool                    OpenFlag_;

      static TRect            MasterAnalogRect_;
      static VxDIface         PiscesEntry_;
      static bool             GlobalEnable_;
      static bool             PreviewOnly_;

      static VidChanArray     CreatedChannels_;

   protected:

      static bool             bCapturing_ ;

      static struct tagPal {   // Current palette colors.
         WORD		      palVersion;
         WORD		      palNumEntries;
         PALETTEENTRY	palPalEntry [256];
      } palCurrent;

      static bool Overlaying_;

      static VideoFormat      CurVidFormat_;

      static bool SingleKeyColor_;

      void CreatePalette();

      static void DisableAllFunc( VfWChannel *&ToDisable, void * );

      VideoFormat GetCurFormat() { return CurVidFormat_; }

      static VideoInfo         VidInfo_;
      VideoInfo GetVidInfo() { return VidInfo_; }
      static SignalFormat      CurSigFormat_;
      SignalFormat GetCurSignal() { return CurSigFormat_; }

      // state functions ( open/closed )
      void SetOpen( bool State ) { OpenFlag_ = State; }
      bool IsOpen() { return OpenFlag_; }

      // channel validation
      static bool IsRealThing( VfWChannel *LookFor )
      { return bool( CreatedChannels_.Find( LookFor ) != INT_MAX ); }

      void AddChannel() { CreatedChannels_.Add( this ); }
      void RemoveChannel( VfWChannel *ToKill ) { CreatedChannels_.Detach( ToKill ); }

      // invokes user callback
      void videoCallback( WORD msg, DWORD dw1 );

      static void SetMasterAnalogRect( const LPRECT lpRect );
      static void GetMasterAnalogRect( LPRECT const lpRect );
//      VidBufQue &GetBufQue() { return VideoBuffers_; }

      // access functions
      void SetStrInitPar( LPVIDEO_STREAM_INIT_PARMS lpStrInitPar )
         { VidStrmInitPar_ = *lpStrInitPar; }

      VIDEO_STREAM_INIT_PARMS &GetStrInitPar() { return VidStrmInitPar_; }

      // Pisces VxD IFace
      DWORD   VxDHandle_;

              ErrorCode LoadVxD();

      virtual ErrorCode OpenVxDChannel( DWORD dnDevNode );
              ErrorCode OpenVxDChannel( VideoStream aStream, DWORD dnDevNode );
      virtual void CloseVxDChannel();
      ErrorCode    CallVxD( FuncNumber func, LPVOID pData = 0, DWORD dwData = NULL );

      virtual WORD CheckBuffer( LPVIDEOHDR lpNewBuffer );
      virtual WORD AllocBuf( LPBYTE *Addr, DWORD dwSize );
      virtual WORD FreeBuf( LPBYTE lpAddr );
      virtual WORD VidConfigMsg( UINT uiMsg, LPARAM lParam1, LPARAM lParam2 );
      virtual WORD VidDialog( HWND, LPARAM );
      virtual WORD SrcRect( LPRECT, LPARAM );
      virtual WORD DstRect( LPRECT, LPARAM );
      virtual WORD Update( HWND, HDC );
      virtual WORD CaptureFrame( LPVIDEOHDR );
      virtual WORD Caps( LPCHANNEL_CAPS, LPARAM );
      virtual WORD StreamMsg( UINT, LPARAM, LPARAM );
      virtual WORD VidConfigStorage( UINT uiMsg, LPARAM lParam1, LPARAM lParam2 );

   public:
      VfWChannel();
      // format  (mainly) manipulations
      void SetBMI( LPBITMAPINFOHEADER lpbi );
      void SetBMI( BITMAPINFOHEADER &bi ) { SetBMI( &bi ); }
      void GetBMI( LPBITMAPINFOHEADER lpbi ) { *lpbi = bmHeader_; }
      BITMAPINFOHEADER &GetBMI() { return bmHeader_; }

      static  WORD  VideoProcessMessage( VfWChannel &, UINT uiMsg, LPARAM, LPARAM );
      static  DWORD VideoOpen( LPVIDEO_OPEN_PARMS lpVidOpenPar );
      static  void Enable();
      static  void Disable();

      static  void VfWChannel::SetSingleKeyColor( bool par ) { SingleKeyColor_ = par; }

      bool IsEnabled() { return GlobalEnable_; }

      virtual ~VfWChannel()
      {  if ( VxDHandle_ ) CloseVxDChannel();
         RemoveChannel( this );
         if ( CreatedChannels_.IsEmpty() &&PiscesEntry_.IsInited() == true )
            PiscesEntry_.UnloadVxD();
      }
      // called to process DRV_CLOSE message
      virtual LRESULT  Close() { delete this; return 1; }
      static int GetCreatedChannels() { return CreatedChannels_.GetItemsInContainer(); }

      // if Preview mode is used only, no overlay channel will be created
      // and Bt848ddi.dll will not be linked in
      static void SetPreviewOnly( bool par ) { PreviewOnly_ = par; }
      bool IsPreviewOnly() { return PreviewOnly_; }
};

#define WIDTHBYTES( i )     ((unsigned)((i+31)&(~31))/8)

/* Class: ExtVidOutChan
 * Purpose: Implements functionality of the VIDEO_EXTERNALOUT channel
 */
class VidOutChan : public VfWChannel
{
};

/* Methdod: VfWChannel::CallVxD
 * Purpose: Wraps VxD interface around
 * Input: func: FuncNumer - an enum that defines for VxD a function to perform
 *    lpData: LPVOID - far pointer to the data accompanying the function number
 *
inline ErrorCode VfWChannel::CallVxD( FuncNumber func, LPVOID lpData )
{
   return CallVxD( func, (DWORD)lpData );
}
*/
/* Methdod: VfWChannel::CallVxD
 * Purpose: Wraps VxD interface around
 * Input: func: FuncNumer - an enum that defines for VxD a function to perform
 *    dwData: DWORD - far pointer to the data accompanying the function number
 */
inline ErrorCode VfWChannel::CallVxD( FuncNumber func, LPVOID pData, DWORD dwData )
{
   return PiscesEntry_.CallVxD( func, pData, dwData, VxDHandle_ );
}

#endif __VFWCHAN_H
