/* File Vidinchn.cpp
 * Purpose: Implementation of the VfW video in channel
 * Author: sasha
 * $Log: VIDINCHN.CPP $
 * Revision 2.33  1998/05/23 00:01:12  dtharp
 * Leave the HDR_PREPARED bit set for 16 bit vidcap return buffer.
 * Revision 2.32  1998/05/21 17:56:01  dwolfson
 * Changed the 1/4 scale format size from 88x72 to 80x72 for QuartzSight
 * Revision 2.31  1998/05/05 21:50:57  dtharp
 * Re-initialized dwMicroSecPerFrame upon completion of streaming capture.
 * Revision 2.30  1998/04/21 17:07:01  dtharp
 * Fix for Quartzsight Clipping .
 * Revision 2.29  1998/04/16 23:46:28  dtharp
 * Made sure timer is killed on close.
 * Revision 2.28  1998/04/16 17:44:30  dtharp
 * Merge preview rate enhancement branch
 * Revision 2.27.1.3  1998/04/16 16:55:36  dtharp
 * Can now capture from preview mode
 * Revision 2.27.1.2  1998/04/16 06:00:28  dtharp
 * Closer...
 * Revision 2.27.1.1  1998/04/16 04:14:42  dtharp
 * Branch for dealing with preview frame rate
 * Revision 2.27  1998/04/16 00:13:11  dtharp
 * The check for active buffers now gets its info from the vxd, where
 * the buffers are now queued.
 * Revision 2.26  1998/04/06 19:42:25  dtharp
 * Vxd Frame Buffers are allocated on the fly instead of at init time.
 * Revision 2.25  1998/04/02 21:42:27  dtharp
 * Merge DT_VIDEOHDR branch. Implements a new buffer 
 * management scheme.
 * Revision 2.24.1.3  1998/03/27 16:31:02  dtharp
 * Clean up dead code
 * Revision 2.24.1.2  1998/03/27 15:30:20  dtharp
 * Incremental check-in
 * Revision 2.24.1.1  1998/03/18 19:36:14  dtharp
 * Branch for passing videoheaders around
 * Revision 2.24  1998/03/03 01:50:28  dtharp
 * Added some debug prints
 * Revision 2.23  1998/01/09 17:09:47  peter
 * Graft Trident fixes to tip.
 * Revision 2.22.1.1  1997/12/10 00:56:38  peter
 * Merged TRIDENT code.
 * Revision 2.22  1997/12/03 21:56:33  peter
 * Fixed regutil 16/32-bit incompatibility.
 * Revision 2.21  1997/11/26 19:29:01  peter
 * Reg move to HKEY_LOCAL_MACHINE.  Removed spaces from reg values.
 * Revision 2.20  1997/10/22 23:42:10  dtharp
 * Added hook to pass video window handle through to the Vxd
 * Revision 2.19  1997/09/16 16:54:21  dtharp
 * Merged AVI processing branch.
 * Revision 2.18.2.1  1997/08/19 15:56:56  dtharp
 * Branch for gemini Avi post processing
 * Revision 2.18  1997/06/27 16:05:22  dtharp
 * Skip count is now passed in to the callback.
 * Revision 2.17  1997/05/19 20:49:48  dtharp
 * Bug Id 511: Added check for stream capture before allowing frame capture.
 * Revision 2.16  1997/05/06 00:20:54  dtharp
 * 848A Merge: from 2.14.2.3
 * Revision 2.15  1997/03/18 19:52:21  rudyl
 * Graft from 2.14.3.2. Uses 288 lines per field for PAL/SECAM.
 * Revision 2.14.3.2  1997/03/08 02:47:51  rudyl
 * Now PAL uses 288 lines per field.
 * Revision 2.14.3.1  1997/03/06 20:30:43  rudyl
 * Use 288 lines for PAL/SECAM
 * Revision 2.14  1997/02/19 21:29:27  dtharp
 * Grafted Branch DT_TEMPORAL. Bug id 401
 * Revision 2.13.2.2  1997/02/19 21:24:42  dtharp
 * Added missing factor in the modulus calculation for dwFract.
 * Revision 2.13.2.1  1997/02/12 21:34:02  dtharp
 * Branch to fix temporal problem. Fixed blatant '-=' bug.
 * Revision 2.13.1.4  1997/02/14 23:40:16  rudyl
 * Removed all #ifdef BT848A. Use same binary files for both Bt848 and Bt848A.
 * Revision 2.13.1.2  1997/02/03 23:42:59  rudyl
 * First Pisces+ check in.
 * Revision 2.13.1.1  1996/12/21 00:30:47  rudyl
 * Start Pisces+ work.
 * Revision 2.13  1996/12/06 21:45:57  meierdw
 * Grafting 2.12.1 back in per 12/8 mtg; NTSC-Japan support.
 * Revision 2.12.1.2  1996/11/08 18:54:17  rudyl
 * Added NTSC-Japan, PAL-M & PAL-N formats.
 * Revision 2.12.1.1  1996/11/07 23:32:24  rudyl
 * Start work to add NTSC-Japan, PAL-M & PAL-N formats.
 * Revision 2.12  1996/10/11 10:16:19  sasha
 * Changed the way number of frames is incremented in the VxD callback.
 * Revision 2.11  1996/10/11 03:01:19  sasha
 * Moved dwOldSkipped_ to the class definition out of VxD callback. 
 * This lets to have it initialized to zero whan capture is started.
 * Revision 2.10  1996/10/05 22:55:25  sasha
 * For 0 frames captured bug: Added bool AddingBuffer_ and a timer
 * callback to prevent cases when VxD calls us in the middle of adding
 * a buffer to the queue. The VxD callback checks this flags and creates
 * a timer event, so it can exit, but not loose the buffer. Call StreamReset()
 * instead of StreamStop() in the CaptureFrame().
 * Revision 2.9  1996/10/02 10:07:29  sasha
 * Added SET to the processed palette messages.
 * Revision 2.8  1996/10/02 09:07:19  sasha
 * Return OK when driver is asked if it supports palette setting ( to satisfy
 * some applications ). Check if analog rect is empty in the StartStream().
 * Revision 2.7  1996/09/27 02:17:54  sasha
 * Changed how time stamp is calculated ( added code to accumulate
 * fractional part of frame time ).
 * Revision 2.6  1996/09/07 00:10:50  rudyl
 * Can restore video size > 640 for PAL. Default video size also depends on video format.
 * Revision 2.5  1996/08/08 01:15:10  sasha
 * Cleaned up a bit
 * Revision 2.4  1996/07/19 01:46:51  sasha
 * remove display of possible not initialized string
 * Revision 2.3  1996/06/19 23:52:16  sasha
 * Changed CaptureFrame() to let capture images bigger then CIF.
 * Revision 2.2  1996/06/14 18:22:44  sasha
 * Check against CF_RGB8 when checking image size in RestoreValues()
 * Revision 2.1  1996/06/10 20:13:07  ssm
 * MarkBeta1AsRev2
 * Revision 1.31  1996/06/06 18:18:21  rudyl
 * changed registry key from Bt848 or APisces --> Software\Brooktree\Bt848
 * Revision 1.30  1996/06/06 01:37:44  sasha
 * Finished off with removing exceptions.
 * Revision 1.29  1996/06/06 01:15:51  sasha
 * Removed EH. Use return codes now
 * Revision 1.28  1996/06/04 23:58:28  sasha
 * Pause the CC stream when doing capture.
 * Revision 1.27  1996/05/30 21:21:22  sasha
 * Check current video format in the CaptureFrame()
 * Revision 1.26  1996/05/21 21:56:22  sasha
 * Check for the current video format when choosing the rectangle height in
 * StreamOpen()
 * Revision 1.25  1996/05/15 18:12:22  sasha
 * Fixed stupid typo in calculating the pitch
 * Revision 1.24  1996/05/14 20:19:14  sasha
 * Make sure pitch is DWORD-aligned
 * Revision 1.23  1996/05/12 21:11:34  sasha
 * Moved guts of OpenVxDChannel into VfWChannel class
 * Revision 1.22  1996/05/11 22:29:51  sasha
 * Added checks for color format and image size in RestoreValues().
 * Revision 1.21  1996/05/10 00:22:31  sasha
 * Added the same code to StreamOpen() as to CaptureFrame() to check
 * for the full-size image. VfW and/or Vidcap32  apparently chokes with a 
 * page faults if streaming capture fails to start in response to stream start message
 * Revision 1.20  1996/05/06 20:51:23  sasha
 * Do not call ValidateRect() in SetImage() as this caused problems in full
 * screen mode ( when screen size is bigger than max video size )
 * Revision 1.19  1996/04/30 16:35:16  sasha
 * Changed RECT to RECT & in Validate()
 * Revision 1.18  1996/04/17 20:03:28  sasha
 * Added frame rate parameter to the StartParms structure
 * Revision 1.17  1996/04/12 00:53:44  sasha
 * Removed temp buffer for the planar capture. Removed commented lines
 * Revision 1.16  1996/04/04 01:20:16  sasha
 * Added bool argument to StreamStart(). Cleaned up CaptureFrame()
 * Revision 1.15  1996/03/30 02:45:32  sasha
 * Added a kludge to prevent sending the 'start' message down to VxD when
 * overlaying a full-size image. Needed for the Intercast's snapshots
 * Revision 1.14  1996/03/26 20:35:54  sasha
 * Commented out the check for overlaying in the CaptureFrame().
 * Revision 1.13  1996/03/22 22:55:58  sasha
 * Changed StreamStart() function to supply pitch to the VxD in bytes, not
 * pixels.
 * Revision 1.12  1996/03/15 21:52:32  sasha
 * Changed default color mode to RGB15 instead of RGB8
 * Revision 1.11  1996/02/21 19:57:06  sasha
 * Experimenting with the buffer retrieval.
 * Revision 1.10  1996/02/19 03:35:46  sasha
 * Changed XMSG thrown when could not allocate memory
 * Revision 1.9  1996/02/19 02:22:04  sasha
 * Increased size of the local buffer
 * Revision 1.8  1996/02/19 01:30:57  sasha
 * Ooops - did not indicate 'long' for the constants when allocating local buffer
 * Commented out 'if' in the Close() function
 * Revision 1.7  1996/02/18 06:15:03  sasha
 * Do not capture single frames into local buffer for non-planar modes.
 * And was forgetting to restore the VFW pointer... oops
 * Revision 1.6  1996/02/18 05:24:53  sasha
 * Capture single frames into a phys cont. buffer and then copy it to the one
 * given by VFW ( this is needed mainly for planar modes )
 * Revision 1.5  1996/02/14 02:31:01  sasha
 * Seems to be the best in terms of video time stams/skipped frames - modified
 * the callback a lot. Cleaned up CaptureFrame as well. Support 8 bit palettes
 * Revision 1.4  1996/01/22 20:35:02  sasha
 * Changed calls to VxD ( added an arument )
 * Revision 1.3  1996/01/10 21:26:09  sasha
 * Changed ColorFormat to ColFmt ( to avoid collision with the register object )
 * Do not pass our instance to the dialog procedure.
 * Revision 1.2  1996/01/04 23:21:20  sasha
 * First real check-in. Too many changes since previous version
 * Revision 1.1  1995/09/07 06:18:06  sasha
 * Initial revision
 */

#include "vidinchn.h"
#include "dbgprint.h"
#include "pci_io.h"

extern void PauseCC();
extern void RestartCC();
extern void PauseOverlay();
extern void RestartOverlay();

VidInChan *VidInChan::VidIn_ = 0;
bool VidInChan::Created_ = false;

DWORD dwgAddTime ;
DWORD dwgLastAdd ;

/* Constructor: VidInChan::VidInChan()
 * Input: None
*/
inline VidInChan::VidInChan() : VfWChannel(), Streaming_( false ), InCallback_( false )
{
   memset( &ImageRect_, '\0', sizeof ( RECT ) );
   InitCounters();
   RestoreValues();
   VidIn_ = this;
}

/* Method: VidInChan::Create
 * Purpose: Creates the object of type VidInChan and returns pointer to it
 * Input: None
 * Output: ExtVidInChan *
 * Note: This creation method is used to prevent multiple openings of the same
 *   video channel by different applications
 */
VidInChan *VidInChan::Create()
{
   if ( Created_ == true ) {
      ErrorDialog( XMSG( IDS_VIDIN ) );
      return NULL;
   }

   Created_ = true;

   // create the Video In channel
   VidInChan *TheVideoInChannel = new VidInChan();

   // this seems to be the only good place to connect to the VxD:
   // it is called only once and it is called before anything else
   // at this point VxD presence is guaranteed
   return TheVideoInChannel;
}

/* Method: VidInChan::OpenVxDChannel
 * Purpose: Opens the connection to the VxD Channel
 * Input: DWORD: device node
 * Output: void
 */
ErrorCode VidInChan::OpenVxDChannel( DWORD dnDevNode )
{
   if ( VxDHandle_ )
      return Success;
   return VfWChannel::OpenVxDChannel( VS_Capture, dnDevNode );
}

/* Method:  VidInChan::StreamMsg
 * Purpose: Handles streaming messages from VfW run-time
 * Input: uiMsg: UINT - message
 *   lParam1: LPARAM
 *   lParam2: LPARAM
 * Output:
 */
WORD VidInChan::StreamMsg( UINT uiMsg, LPARAM lParam1, LPARAM lParam2 )
{
   switch ( uiMsg ) {

   case DVM_FRAME:
      DebugPrint((0,"DVM_FRAME"));
      return bCapturing_ ? DV_ERR_NOTSUPPORTED : CaptureFrame( (LPVIDEOHDR)lParam1 );

   case DVM_STREAM_INIT:
      DebugPrint((0,"VidInChan::StreamMsg: DVM_STREAM_INIT\n"));
      StreamReset(true) ;

      bCapturing_ = true ;
      return StreamOpen( (LPVIDEO_STREAM_INIT_PARMS)lParam1 );

   case DVM_STREAM_FINI:
      DebugPrint((0,"VidInChan::StreamMsg: DVM_STREAM_FINI\n"));
      CallVxD(FNEndCapNotify) ;
      CallVxD( FNFlushQue );
      GetStrInitPar().dwMicroSecPerFrame = 33333;
      return StreamClose();

   case DVM_STREAM_GETERROR:  // how many frames skipped
      DebugPrint((0,"VidInChan::StreamMsg: DVM_STREAM_GETERROR\n"));
      return StreamGetErr( (LPDWORD) lParam1, (LPDWORD) lParam2 );

   case DVM_STREAM_GETPOSITION:
      DebugPrint((0,"VidInChan::StreamMsg: DVM_STREAM_GETPOSITION\n"));
      return StreamGetPos( (LPMMTIME) lParam1, (DWORD) lParam2 );

   case DVM_STREAM_ALLOCBUFFER:
      DebugPrint((0,"VidInChan::StreamMsg: DVM_STREAM_ALLOCBUFFER\n"));
      return DV_ERR_NOTSUPPORTED;
//      return AllocBuf( (LPBYTE *)lParam1, lParam2 );

   case DVM_STREAM_FREEBUFFER:
      DebugPrint((0,"VidInChan::StreamMsg: DVM_STREAM_FREEBUFFER\n"));
      return DV_ERR_NOTSUPPORTED;
//      return FreeBuf( (LPBYTE)lParam1 );

   case DVM_STREAM_ADDBUFFER:
      DebugPrint((0,"VidInChan::StreamMsg: DVM_STREAM_ADDBUFFER\n"));
      return AddBuffer( (LPVIDEOHDR)lParam1 );

   case DVM_STREAM_PREPAREHEADER:  // Handled by MSVideo
      DebugPrint((0,"VidInChan::StreamMsg: DVM_STREAM_PREPAREHEADER\n"));
      return DV_ERR_NOTSUPPORTED;

   case DVM_STREAM_UNPREPAREHEADER: // Handled by MSVideo
      DebugPrint((0,"VidInChan::StreamMsg: DVM_STREAM_UNPREPAREHEADER\n"));
      return DV_ERR_NOTSUPPORTED;

   case DVM_STREAM_RESET:
      DebugPrint((0,"VidInChan::StreamMsg: DVM_STREAM_RESET\n"));
      bCapturing_ = false ;
      return StreamReset(true);

   case DVM_STREAM_START:
   {
      extern HWND ghwndVideo ;
      DebugPrint((0,"VidInChan::StreamMsg: DVM_STREAM_START\n"));
      CallVxD(FNStartCapNotify, (void *)&ghwndVideo) ;
      return StreamStart();
   }
   case DVM_STREAM_STOP:
      DebugPrint((0,"VidInChan::StreamMsg: DVM_STREAM_STOP\n"));
      bCapturing_ = false ;
      return StreamStop();

   default:
      DebugPrint((0,"VidInChan::StreamMsg: default\n"));
      return DV_ERR_NOTSUPPORTED;
   }
}

/* Method: VidInChan::VidConfigMsg
 * Purpose:
 * Input:
 * Output:
 */
WORD VidInChan::VidConfigMsg( UINT uiMsg, LPARAM lFlags, LPARAM lParam2 )
{
   LPVIDEOCONFIGPARMS	lpcp;
   LPDWORD	            lpdwReturn;// Return parameter from configure.
   LPVOID	            lpData1;   // Pointer to data1.
   DWORD	               dwSize1;   // size of data buffer1.

   lpcp = (LPVIDEOCONFIGPARMS)lParam2;
   lpdwReturn = lpcp->lpdwReturn;
   lpData1 = lpcp->lpData1;
   dwSize1 = lpcp->dwSize1;

   switch ( uiMsg ) {
   case DVM_PALETTE:
      switch ( lFlags ) {
      case VIDEO_CONFIGURE_SET:
      case ( VIDEO_CONFIGURE_QUERY | VIDEO_CONFIGURE_SET ):
         return DV_ERR_OK; // don't support setting, but some apps get upset; so an innocent lie
      case ( VIDEO_CONFIGURE_QUERY | VIDEO_CONFIGURE_GET ):
         return DV_ERR_OK;
      case VIDEO_CONFIGURE_QUERYSIZE:
      case ( VIDEO_CONFIGURE_QUERYSIZE | VIDEO_CONFIGURE_GET ):
         *lpdwReturn = sizeof( LOGPALETTE )
            + ( palCurrent.palNumEntries - 1 ) * sizeof( PALETTEENTRY );
         break;
      case VIDEO_CONFIGURE_GET:
      case ( VIDEO_CONFIGURE_GET | VIDEO_CONFIGURE_CURRENT ):
         return GetDestPalette( (LPLOGPALETTE)lpData1, (WORD) dwSize1 );
      default:
         return DV_ERR_NOTSUPPORTED;
      }
      return DV_ERR_OK;
   case DVM_PALETTERGB555:
      return DV_ERR_NOTSUPPORTED;
   case DVM_FORMAT:
      switch ( lFlags ) {
      case ( VIDEO_CONFIGURE_QUERY | VIDEO_CONFIGURE_SET ):
      case ( VIDEO_CONFIGURE_QUERY | VIDEO_CONFIGURE_GET ):
         return DV_ERR_OK;  // command is supported

      case VIDEO_CONFIGURE_QUERYSIZE:
      case ( VIDEO_CONFIGURE_QUERYSIZE | VIDEO_CONFIGURE_GET ):
         *lpdwReturn = sizeof( BITMAPINFOHEADER );
         break;

      case VIDEO_CONFIGURE_SET:
      case ( VIDEO_CONFIGURE_SET | VIDEO_CONFIGURE_CURRENT ):
         return SetDestFormat( (LPBITMAPINFOHEADER) lpData1, (WORD) dwSize1 );

      case VIDEO_CONFIGURE_GET:
      case ( VIDEO_CONFIGURE_GET | VIDEO_CONFIGURE_CURRENT ):
         return GetDestFormat( (LPBITMAPINFOHEADER) lpData1, (WORD) dwSize1 );
      default:
         return DV_ERR_NOTSUPPORTED;
      } /* endswitch */

      return DV_ERR_OK;

   default:		// Not a msg that we understand.
      return DV_ERR_NOTSUPPORTED;
   } /* endswitch */
}

WORD VidInChan::GetDestPalette( LPLOGPALETTE lpLogPal, WORD wSize )
{
   memcpy( lpLogPal, &palCurrent, wSize );
   return DV_ERR_OK;
}

WORD VidInChan::GetDestFormat( LPBITMAPINFOHEADER lpbi, WORD wSize )
{
   if ( wSize < sizeof( BITMAPINFOHEADER ) )
      return DV_ERR_SIZEFIELD;
   GetBMI( lpbi );
   return DV_ERR_OK;
}

/* Method: VidInChan::SetDestFormat
 * Purpose: Called in response to the DVM_FORMAT message
 * Input: lpbi: pointer to the BITMAPINFOHEADER
 *    wSize: WORD - size of the structure pointed to by lpbi
 * Output: WORD: DV_ERR_OK or error
 */
WORD  VidInChan::SetDestFormat( LPBITMAPINFOHEADER lpbi, WORD wSize )
{
   if ( wSize < sizeof( BITMAPINFOHEADER ) )
      return DV_ERR_SIZEFIELD;

   if ( lpbi->biSize != sizeof( BITMAPINFOHEADER ) )
      return DV_ERR_PARAM2;

   // create a temp colorspace object to perform validations
   SIZE ImageSize;
   ImageSize.cx = (int)lpbi->biWidth;
   ImageSize.cy = (int)lpbi->biHeight;
   ColorSpace tmp( lpbi->biCompression, lpbi->biBitCount );
   if ( tmp.IsValid() == false || tmp.CheckDimentions( ImageSize ) == false )
      return DV_ERR_BADFORMAT;

   // store the current image format
   RECT r;
   ::SetRect( &r, 0, 0, (int)lpbi->biWidth, (int)lpbi->biHeight );
   SetImage( &r, tmp.GetColorFormat() );
   return DV_ERR_OK;
}

/* Method:  VidInChan::DstRect
 * Purpose: Processes the DVM_DST)RECT message
 * Input: lpRect: LPRECT - far pointer to the RECT structure
 *   lFags: LPARAM - set of flags
 * Output: DV_ERR_OK
 */
WORD VidInChan::SrcRect( LPRECT lpRect, LPARAM lFlags )
{
   switch ( lFlags ) {
   case VIDEO_CONFIGURE_SET:
   case ( VIDEO_CONFIGURE_SET | VIDEO_CONFIGURE_CURRENT ): {
         // Where in the frame buffer should we take
         // the image from?

         // verify that the correct size is passed in
         SetImageRect( lpRect->left, lpRect->top, lpRect->right - lpRect->left,
            lpRect->bottom - lpRect->top );
         ColorSpace tmp( CurColFormat_ );
         SIZE ImageSize = ImageRect_.Size();
         if ( tmp.CheckDimentions( ImageSize ) == false )
            return DV_ERR_NONSPECIFIC;
         break;
      }
   case ( VIDEO_CONFIGURE_GET | VIDEO_CONFIGURE_CURRENT ):
      GetImageRect( lpRect );
      break;
   case ( VIDEO_CONFIGURE_GET | VIDEO_CONFIGURE_MAX ):
      GetImageRect( lpRect );
      break;
   default:
      return DV_ERR_NOTSUPPORTED;
   }
   return DV_ERR_OK;
}

/* Method:   VidInChan::Caps
 * Purpose:
 * Input:
 * Output:
 */
WORD VidInChan::Caps( LPCHANNEL_CAPS lpCaps, LPARAM lSize )
{
   // structure should be of the right size
   if ( lSize != sizeof( CHANNEL_CAPS ) )
      return DV_ERR_NONSPECIFIC;

// for the video in channel the source rectangle has no meaning ( because it
// is already implicitly defined by the external video in channel's
// dst rectangle )
   lpCaps->dwFlags = VCAPS_CAN_SCALE;

   lpCaps-> dwSrcRectXMod = 1;
   lpCaps-> dwSrcRectYMod = 1;
   lpCaps-> dwSrcRectWidthMod = 1;
   lpCaps-> dwSrcRectHeightMod = 1;

   // BUGBUG: have to check the current data format to use proper granularity
   lpCaps-> dwDstRectXMod = 1;
   lpCaps-> dwDstRectYMod = 1;
   lpCaps-> dwDstRectWidthMod = 1;
   lpCaps-> dwDstRectHeightMod = 1;

   return DV_ERR_OK;
}

/* Method:  VidInChan::StreamOpen
 * Purpose: Handles the DVM_STREAM_INIT message
 * Input: lpStrInitPar: LPVIDEO_STREAM_INIT_PARMS - pointer to the structure
 * Output: WORD: VfW error code
 */
WORD VidInChan::StreamOpen( LPVIDEO_STREAM_INIT_PARMS lpStrInitPar )
{
   DebugPrint((1,"InStreamOpen\n"));

   // should not open the same thing twice
   if ( IsOpen() == true || !lpStrInitPar )
      return DV_ERR_NONSPECIFIC;

   if ( Overlaying_ ) {
      TRect MasterR;
      GetMasterAnalogRect( &MasterR );

      int height;
      if ( IsABt848A() )
         height = ( GetCurSignal() != Signal_Analog ) ? (int)GetVidInfo().linesPerField :
                    ( IsNTSCCompatible( GetCurFormat() ) ? 240 : 288 );
      else
         height = IsNTSCCompatible( GetCurFormat() ) ? 240 : 288;

      if ( MasterR.Height() > height ) {
         ErrorDialog( XMSG( IDS_FULLCHAN ) );
         return DV_ERR_ALLOCATED;
      }
   }
   // remember the initialization params, so we can perform a callback
   SetStrInitPar( lpStrInitPar );

   // set internal flag
   SetOpen( true );

   InitCounters();

   videoCallback( MM_DRVM_OPEN, 0L );        // Notify app we're open.

   return DV_ERR_OK;
}

/* Method:  VidInChan::StreamClose
 * Purpose: Handles DVM_STREAM_FINI message
 * Input: None
 * Output: WORD: DV_ERR_OK or error code
 */
WORD VidInChan::StreamClose()
{
   // con not finish non-inited or still playing stream
   if ( IsOpen() != true || IsStreaming() == true )
      return DV_ERR_NONSPECIFIC;

   DWORD dwNumActive;
   CallVxD( FNGetR3BufCount, &dwNumActive );

   if ( dwNumActive ) // still have buffers in the queue
      return DV_ERR_STILLPLAYING;

   StreamStop();

   SetOpen( false );

   videoCallback( MM_DRVM_CLOSE, 0L );       // Notify app we're closed

   return DV_ERR_OK;
}

/* Method:  VidInChan::CaptureFrame
 * Purpose: Processed DVM_FRAME message
 * Input: lpVHdr: LPVIDEOHDR - pointer to the VIDEOHDR structure
 * Note: This function has to return only when a buffer is ready. So it
 *   simulates a streaming capture: 1) it adds buffer to the queue 2) Starts
 *   streaming capture 3) Goes into loop 4) Breaks out of the loop upon notification
 *   5) Stops Streaming capture
 */
#define TIMER_VAL 200
UINT tid_ = 0;
VidInChan * pVidInChan = NULL ;
bool bResetting = false ;

VOID CALLBACK _loadds FrameStreamTimerProc(HWND, UINT, UINT, DWORD)
{
   DebugPrint((0,"** Timer Proc\n")) ;
   if (pVidInChan && !bResetting)
   {
       DebugPrint((0,"** TRIGGER!!!!!!!!\n")) ;
       pVidInChan->StreamReset(true);
   }
}

WORD VidInChan::CaptureFrame( LPVIDEOHDR lpVHdr )
{
    // can not do FRAME when streaming
   if ( IsStreaming() )
      return DV_ERR_STILLPLAYING;

   if ( lpVHdr->dwBufferLength < GetBMI().biSizeImage )
      return DV_ERR_SIZEFIELD;

   // just make sure if anybody will check this
   lpVHdr->dwBytesUsed = GetBMI().biSizeImage;

   bool WasOpen = IsOpen();

   SetOpen( true );

   ColorSpace tmp( GetBMI().biCompression, GetBMI().biBitCount );
   GlobalPageLock( HGLOBAL( HIWORD( lpVHdr->lpData ) ) );
   lpVHdr->dwFlags |= VHDR_PREPARED;
   lpVHdr->dwFlags &= ~VHDR_INQUEUE;

   // stop the overlay channel in order to capture full-res image
   bool StoppedOverlay = false;
   if ( Overlaying_ ) {
      TRect MasterR;
      GetMasterAnalogRect( &MasterR );

      int height;
      if ( IsABt848A() )
         height = ( GetCurSignal() != Signal_Analog ) ? (int)GetVidInfo().linesPerField :
                    ( IsNTSCCompatible( GetCurFormat() ) ? 240 : 288 );
      else
         height = IsNTSCCompatible( GetCurFormat() ) ? 240 : 288;

      if ( MasterR.Height() > height ) {
         PauseOverlay();
         StoppedOverlay = true;
      }
   }

   if ( AddBuffer( lpVHdr ) != DV_ERR_OK ) {
      GlobalPageUnlock( HGLOBAL( HIWORD( lpVHdr->lpData ) ) );
      lpVHdr->dwFlags &= ~VHDR_PREPARED;
      if ( !WasOpen )
        SetOpen( false );

      if ( StoppedOverlay )
         RestartOverlay();
      return DV_ERR_NONSPECIFIC;
   }

   InCallback_ = false;

   WORD wRetVal = DV_ERR_OK;

   // if tid_ == 0 it means that the frameserver has not been started. 
   // Go ahead and start it up.
   if(tid_ == 0) {
      if ( FrameStreamStart(  ) != DV_ERR_OK ) {
         CallVxD( FNFlushQue );
         wRetVal = DV_ERR_NONSPECIFIC;
      }

      if ( !WasOpen )
        SetOpen( false );

      if ( StoppedOverlay )
         RestartOverlay();
   }
   else
   {
      // frameserver was already running -- restart the timer. If the timer expires
      // the stream will be stopped.
      DebugPrint((0,"** ReStarting Timer\n") ) ;
      KillTimer(NULL,tid_) ;

      tid_ = SetTimer(NULL,
                      0,                    // timer identifier (ignored since hwnd is NULL)
                      TIMER_VAL,            // time-out value (ms)
                      FrameStreamTimerProc  // address of timer procedure
                      );
      pVidInChan = this ;
   }
   return wRetVal;
}


WORD VidInChan::FrameStreamStart(  )
{
   TRect analog;
   GetMasterAnalogRect( &analog );

   // it is possible that master rect is empty
   if ( analog.IsEmpty() )
      analog = ImageRect_;

   StartParms strt( SmallRect( analog ), SmallRect( ImageRect_ ), CurColFormat_ );

   ColorSpace tmp( CurColFormat_ );

   strt.dwPitch = WIDTHBYTES( ( ImageRect_.Width() * tmp.GetPitchBpp() ) );
   strt.callback = (DWORD)CapVxDCallback;
   strt.tag      = (DWORD)this;
   strt.FrameRate = GetStrInitPar().dwMicroSecPerFrame / 1000;
   strt.ImageSize = GetBMI().biSizeImage;

   // interrupt the CC capture
   PauseCC();

   if ( CallVxD( FNStart, &strt ) != Success ) {
      RestartCC();
   }
   else
   {
      // start up the timer which will stop the frame stream
      // if no frame requests come for .5 sec.
      if (tid_ == 0) {
         DebugPrint((0,"** Starting Timer\n") ) ;
         tid_ = SetTimer(NULL,
                         0,            // timer identifier (ignored since hwnd is NULL)
                         TIMER_VAL,          // time-out value (ms)
                         FrameStreamTimerProc   // address of timer procedure
                         );
         pVidInChan = this ;
      } 
   }
   return DV_ERR_OK;
}

void WINAPI _loadds VidInChan::CapVxDCallback( VidInChan *me, DWORD lpVideoHdr )
{
   DebugPrint((0, "VidInChan::CapVxDCallback().\n" ));

   // free the CaptureFrame function
   me->InCallback_ = true;

   // do the streaming capture processing
   DWORD dwCompleteTime = timeGetTime() ;
   DebugPrint((0,"Completing buffer @ %ld Delta:%ld\n",dwCompleteTime,dwCompleteTime-dwgAddTime)) ;

   LPVIDEOHDR lpTmpHdr = (LPVIDEOHDR) lpVideoHdr;
   lpTmpHdr->dwBytesUsed = me->GetBMI().biSizeImage;

   GlobalPageUnlock( HGLOBAL( HIWORD( lpTmpHdr->lpData ) ) );
//   lpTmpHdr->dwFlags &= ~VHDR_PREPARED;  // For some reason 16 bit vidcap needs this bit set

   me->videoCallback( MM_DRVM_DATA, (DWORD)lpTmpHdr );

   DebugPrint((1, "   gave buffer @ %ld\n", lpTmpHdr->dwTimeCaptured));
}

/* Method:  VidInChan::StreamStart
 * Purpose: Handles DVM_STREAM_START message; starts streaming capture
 * Input: DoThrow: bool, used to control when an exception is thrown
 * Output:  Driver error code
 */
WORD VidInChan::StreamStart( bool DoThrow )
{
   if (tid_) {
      bResetting = true ;
      StreamReset(true) ;
      bResetting = false ;
   }

   // cannot start non-inited stream
   if ( IsOpen() == false )
      return DV_ERR_NONSPECIFIC;

   // apparently, MS wants to see OK code if two ( or more ) starts come in a row
   if ( IsStreaming() == true )
      return DV_ERR_OK;

   // reset the data
   InitCounters();

   CallVxD( FNSkipCount, &dwFramesSkipped_ );

   // call the VxD here to start capture
   TRect analog;
   GetMasterAnalogRect( &analog );
   // it is possible that master rect is empty
   if ( analog.IsEmpty() )
      analog = ImageRect_;

   StartParms strt( SmallRect( analog ), SmallRect( ImageRect_ ),
      CurColFormat_ );

   ColorSpace tmp( CurColFormat_ );

   strt.dwPitch = WIDTHBYTES( ( ImageRect_.Width() * tmp.GetPitchBpp() ) );
   strt.callback = (DWORD)CapVxDCallback;
   strt.tag      = (DWORD)this;
   strt.FrameRate = GetStrInitPar().dwMicroSecPerFrame / 1000;
   strt.ImageSize = GetBMI().biSizeImage;


   // Attempt to find Trident cards which use second aperature for
   // DMA access.  PCI Device Ids are: 9750h, 9850h, 9397h, 9388h
   // Trident vendor ID is: 1023h
   {
       WORD devIds[] = {0x9750, 0x9850, 0x9397, 0x9388 /*,0x9660*/ }; // 9660 is 9685 chip
       #define NUM_IDS (sizeof(devIds)/sizeof(devIds[0]))

       PCI_IO pci(0x1023) ;
       for (int i = 0 ; i < NUM_IDS ; i++)
       {
          if ( pci.FindDevice( devIds[i] ) )
          {
             DebugPrint((0,"Found Trident %x\n",devIds[i])) ;
             pci.ReadConfigDWORD( 0x010, (long *) &(strt.TridentAperture0) ) ;
             pci.ReadConfigDWORD( 0x018, (long *) &(strt.TridentAperture1) ) ;
             DebugPrint((0,"   Ap 0 : %lx\n",strt.TridentAperture0)) ;
             DebugPrint((0,"   Ap 1 : %lx\n",strt.TridentAperture1)) ;
             break ;
          }
       }
       DebugPrint((1,"Done looking for Trident\n")) ;
   }



   SetStreaming( true );

   // interrupt the CC capture
   PauseCC();

   if ( CallVxD( FNStart, &strt ) != Success ) {
      SetStreaming( false );
      RestartCC();
      if ( DoThrow ) {
         videoCallback( MM_DRVM_ERROR, 0 );
         ErrorDialog( XMSG( IDS_FULLCHAN ) );
         return DV_ERR_ALLOCATED;
      }
   }

   return DV_ERR_OK;
}

/* Method:  VidInChan::StreamStop
 * Purpose:
 * Input:
 * Output:
 */
WORD VidInChan::StreamStop(bool bIsFrameStreamReset)
{
   DebugPrint((0,"StreamStop\n")) ;
   // don't have anything to stop yet
   if ( !bIsFrameStreamReset && IsOpen() != true )
      return DV_ERR_NONSPECIFIC;

   // apparently, MS wants to see OK code if number of stops come in a row
   if ( !bIsFrameStreamReset && IsStreaming() == false )
      return DV_ERR_OK;

// call the VxD here to stop capture
   DebugPrint((0,"StreamStop:  Calling Vxd:FNStop \n")) ;
   CallVxD( FNStop );

   SetStreaming( false );

   // let CC re-appear
   RestartCC();
   
   return DV_ERR_OK;
}

/* Method:  VidInChan::StreamReset
 * Purpose:
 * Input:
 * Output:
 */
WORD VidInChan::StreamReset(bool bIsFrameStreamReset)
{
   DebugPrint((0,"StreamReset\n")) ;
   // if tid_ is non-null, the frame server is running.
   if(tid_) 
   {
     KillTimer(NULL,tid_) ;
     tid_ = 0 ;

     if ( bIsFrameStreamReset )
       SetOpen(false) ;
   }
   else
   {
      if (IsOpen() != true || bIsFrameStreamReset)
        return DV_ERR_NONSPECIFIC;
   }

   StreamStop(bIsFrameStreamReset);

   CallVxD( FNFlushQue );

   return DV_ERR_OK;
}

/* Method:  VidInChan::StreamGetErr
 * Purpose:
 * Input:
 * Output:
 */
WORD VidInChan::StreamGetErr( LPDWORD lpErrType, LPDWORD lpFramesSkipped )
{
   if ( lpErrType )
      *lpErrType = (dwFramesSkipped_ ? DV_ERR_NO_BUFFERS : DV_ERR_OK );
   if ( lpFramesSkipped )
      *lpFramesSkipped = dwFramesSkipped_;
   dwFramesSkipped_ = 0;
   return DV_ERR_OK;
}

/* Method:  VidInChan::StreamGetPos
 * Purpose:
 * Input:
 * Output:
 */
WORD VidInChan::StreamGetPos( LPMMTIME /*lpMMTime*/, DWORD dwSize )
{
   if ( dwSize != sizeof( MMTIME ) )
      return DV_ERR_NONSPECIFIC;
// [bugbug]
// We should fill out the following with the last timestamp we received
// currently we just return NOTSUPPORTED 

   // old code (here for reference only)
//   lpMMTime->wType = TIME_MS;
//   lpMMTime->u.ms = dwVideoClock_ * GetStrInitPar().dwMicroSecPerFrame / 1000;

   return DV_ERR_NOTSUPPORTED;
}

/* Method:  VidInChan::VidConfigStorage
 * Purpose:
 * Input:
 * Output:
 */
#pragma argsused
WORD VidInChan::VidConfigStorage( UINT uiMsg, LPARAM lParam1, LPARAM lParam2 )
{
   return DV_ERR_OK;
}

/* Function: ValidateRect
 * Purpose: Checks dimentions of a rectangle
 * Input: r: RECT & - reference to a rectangle to validate
 *        vf: VideoFormat - current video format
 * Output: None
 */
void ValidateRect( TRect &r, VideoFormat vf )
{
   int maxWidth, maxHeight, defWidth, defHeight;

   // figure max and default sizes for different video format
   if ( IsABt848A() && ( vf == VFormat_PAL_NC ) ) {
      maxWidth  = 640;
      maxHeight = 576;
      defWidth  = 160;
      defHeight = 144;
   }
   else
   if ( IsNTSCCompatible( vf ) ) {
      maxWidth  = 640;
      maxHeight = 480;
      defWidth  = 160;
      defHeight = 120;
   } else {
      maxWidth  = 768;
      maxHeight = 576;
      defWidth  = 192;
      defHeight = 144;
   }

   if ( r.Width() > maxWidth || r.Height() > maxHeight ||
        r.right < 0          || r.top < 0 ) {
      r.right = defWidth;
      r.bottom = defHeight;
   }
}

/* Method: VidInChan::RestoreValues
 * Purpose: Initializes the state of the channel by reading the registry
 * Input: None
 * Output: None
 * Note: The function performs validation of values read from the registry
 */
void VidInChan::RestoreValues()
{
// here we should get all the entries from the registry and initialize properly
   RegUtil Storage( BT_VIDEO_REG_SUBKEY );

   // working horse variable
   DWORD dwTmp = 0;

   ColFmt Col;
   if ( Storage.GetValue( &dwTmp, "ColorFormat" ) == Fail )
      Col = CF_RGB15;
   else
      Col = (ColFmt)dwTmp;

   // do some extra checks. registry can be edited manually...
   if ( Col <= CF_BelowRange || Col >= CF_AboveRange )
      Col = CF_RGB15;

   TRect TmpRect;

   if ( Storage.GetValue( &dwTmp, "RectLeft" ) == Fail ) {
      // set default video size depends on video format
      // If no "Video Format" registry key exists, default to NTSC
      if ( IsABt848A() &&
           ( Storage.GetValue( &dwTmp, "SignalFormat" ) == Success ) &&
           ( SignalFormat( dwTmp ) != Signal_Analog ) ) {
         TmpRect.left = TmpRect.top = 0;
         TmpRect.bottom = ( Storage.GetValue( &dwTmp, "VActive" ) != Success ) ? 120 : (int)dwTmp;
         TmpRect.right = ( Storage.GetValue( &dwTmp, "HActive" ) != Success ) ? 160 : (int)dwTmp;
      }
      else
      if ( IsABt848A() &&
           ( Storage.GetValue( &dwTmp, "VideoFormat" ) == Success ) &&
           ( VideoFormat( dwTmp ) == VFormat_PAL_NC ) ) {
         TmpRect.left = TmpRect.top = 0;
         TmpRect.bottom = 144;
         TmpRect.right = 160;
      }
      else
      if ( ( Storage.GetValue( &dwTmp, "VideoFormat" ) == Success ) &&
           ( !IsNTSCCompatible( VideoFormat( dwTmp ) ) ) ) {
         TmpRect.left = TmpRect.top = 0;
         TmpRect.bottom = 144;
         TmpRect.right = 192;
      } else {
         TmpRect.left = TmpRect.top = 0;
         TmpRect.bottom = 120;
         TmpRect.right = 160;
      }
   } else {
      TmpRect.left = (int)dwTmp;

      Storage.GetValue( &dwTmp, "RectTop" );
      TmpRect.top = (int)dwTmp;
      Storage.GetValue( &dwTmp, "RectWidth" );
      TmpRect.right = (int)dwTmp;
      Storage.GetValue( &dwTmp, "RectHeight" );
      TmpRect.bottom = (int)dwTmp;

      // extra checks
      ValidateRect( TmpRect, GetCurFormat() );
      
//      // make sure that somebody ( Intercast ) did not create a wrong combination
//      if ( TmpRect.Height() >= 480 && Col > CF_RGB8 )
//         Col = CF_RGB15;
   }
   SetImage( &TmpRect, Col );
}

/* Method: VidInChan::StoreValues
 * Purpose: Initializes the state of the channel
 * Input: None
 * Output: None
 * Note: The function does not validate data, as at this point it is expected
 *   that data is OK
 */
void VidInChan::StoreValues()
{
   // here we should put all the persistant info into the registry
   RegUtil Storage( BT_VIDEO_REG_SUBKEY );

   Storage.SetValue( CurColFormat_, "ColorFormat" );
   Storage.SetValue( ImageRect_.left, "RectLeft" );
   Storage.SetValue( ImageRect_.top, "RectTop" );
   Storage.SetValue( ImageRect_.Width(), "RectWidth" );
   Storage.SetValue( ImageRect_.Height(), "RectHeight" );
}

/* Method: VideoInChan::AddBuffer
 * Purpose: Adds new buffer to the queue and sends it to the VxD
 * Input: lpNewBuffer: LPVIDEOHDR - video header describing the buffer
 * Output: WORD - error code
 * Note: The buffer management between the VxD and capture driver is the same
 *    as between the capture driver and application - order in which buffers
 *    are returned supposed to be the same as received. I.E. FIFO. No explicit
 *    addresses are dealt with.
 */
WORD VidInChan::AddBuffer( LPVIDEOHDR lpNewBuffer )
{
   // MS does not want to see buffers added to a non-inited stream
   if ( IsOpen() != true )
      return DV_ERR_NONSPECIFIC;

   WORD wRetVal = VfWChannel::CheckBuffer( lpNewBuffer );
   if ( wRetVal != DV_ERR_OK )
      return wRetVal;

   dwgAddTime = timeGetTime() ;
   DebugPrint((0,"\nSending buffer @ %ld  (Delta from last add %ld)\n",
               dwgAddTime,
               dwgAddTime-dwgLastAdd)) ;
   dwgLastAdd = dwgAddTime ;

   lpNewBuffer->dwFlags &= ~VHDR_DONE;
   lpNewBuffer->dwFlags |= VHDR_INQUEUE;

   CallVxD( FNAddBuf, lpNewBuffer );

   return wRetVal;
}

/* Method: VidInChan::SetImage
 * Purpose: Just zeroes out some internal variables
 * Input: None
 * Output: None
 */
ErrorCode VidInChan::SetImage( LPRECT lpRect, ColFmt aFormat )
{
   // create a temp object to perform some validations
   ColorSpace tmp( aFormat );

   // make sure passed value is OK
   if ( tmp.IsValid() != true )
	{
      aFormat = CF_RGB15;
		#ifdef DEBUG
			char	string1[100];
			wsprintf(string1, "invalid color format: changed to CF_RGB15");
			MessageBox( NULL, string1, "VidInChan::SetImage" , MB_OK | MB_ICONINFORMATION | MB_TASKMODAL );
		#endif
	}

   SIZE size;
   size.cx = lpRect->right - lpRect->left;
   size.cy = lpRect->bottom - lpRect->top;

   // make sure that size is OK for this color format
   if ( tmp.CheckDimentions( size ) == false )
	{
		#ifdef DEBUG
			char	string2[100];
			wsprintf(string2, "invalid dimensions: cx=%d cy=%d", size.cx, size.cy);
			MessageBox( NULL, string2, "VidInChan::SetImage" , MB_OK | MB_ICONINFORMATION | MB_TASKMODAL );
		#endif
      return Fail;
	}

   // keep them around
   ImageRect_ = *lpRect;
   CurColFormat_ = aFormat;

   // could have used different structure, but why ?
   BITMAPINFOHEADER &bmH = GetBMI();

   bmH.biCompression = tmp.GetFourcc();
   bmH.biBitCount    = (int)tmp.GetBitCount();

   bmH.biWidth  = size.cx;
   bmH.biHeight = size.cy;

   // store the format for later use
   SetBMI( bmH );

   return Success;
}

/* Destructor: VidInChan::~VidInChan()
 * Note: The base class' destructor takes care of removing from the list of
 *   created channels and closing a channel to a VxD
 */
inline VidInChan::~VidInChan()
{
   StoreValues();
   Created_ = false;
}

/* Method:
*/
LRESULT VidInChan::Close()
{
   // do we have to do the check and return NULL ?
   // IN case of a bad app this will cause driver to hang in memory
//   if ( IsStreaming() == true || !GetBufQue().IsEmpty() ) // still have buffers in the queue
//      return 0;
   if (tid_) {
      bResetting = true ;
      StreamReset(true) ;
      bResetting = false ;
   }

   delete this;
   return 1;
}

/* Method: VidInChan::VidDialog
 * Purpose:
 * Input:
 * Output:
 */
WORD VidInChan::VidDialog( HWND hWndParent, LPARAM lFlags )
{
   if ( lFlags & VIDEO_DLG_QUERY )
      return DV_ERR_OK;	// Support the dialog.

   return VidFormatDialog( hWndParent );
}