/* File: Vidinchn.h
 * Purpose: Contains definition of the VidInChan class
 * Author: sasha
 * $Log: VIDINCHN.H $
 * Revision 2.8  1998/04/16 17:45:06  dtharp
 * Merge preview rate enhancement branch
 * Revision 2.7.1.1  1998/04/16 06:00:06  dtharp
 * Branch for dealing with preview frame rate
 * Revision 2.7  1998/04/16 00:10:48  dtharp
 * Cleaned up some unused variables
 * Revision 2.6  1998/04/02 21:42:44  dtharp
 * Merge DT_VIDEOHDR branch. Implements a new buffer 
 * management scheme.
 * Revision 2.5.2.1  1998/03/27 15:25:20  dtharp
 * Branch for passing videohdrs around.
 * Revision 2.5  1997/06/27 16:14:37  dtharp
 * Skip count is passed into the callback.
 * Revision 2.4  1997/05/05 19:44:38  dtharp
 * 848A Merge: from 2.3.1.1
 * Revision 2.3.1.1  1997/02/12 19:33:08  rudyl
 * Pisces+ work.
 * Revision 2.3  1996/10/11 02:58:38  sasha
 * Added dwOldSkipped_ to the class ( moved it out of VxD callback
 * Revision 2.2  1996/10/05 22:52:32  sasha
 * For 0 frames captured bug: Added bool AddingBuffer_ and a timer
 * callback to prevent cases when VxD calls us in the middle of adding
 * a buffer to the queue.
 * Revision 2.1  1996/06/10 20:13:09  ssm
 * MarkBeta1AsRev2
 * Revision 1.8  1996/04/11 21:31:28  sasha
 * Removed temp buffer for planar mode capture
 * Revision 1.7  1996/04/04 01:02:18  sasha
 * Added a bool argument to the StreamStart() to control when exception is
 * thrown ( do not want it when attempting to capture a frame )
 * Revision 1.6  1996/02/18 05:23:25  sasha
 * Added LPBYTE CapFrameBuf member for the frame capture
 * Revision 1.5  1996/02/14 02:32:37  sasha
 * Moved palette thing out. Plus some little clean-up and extra member for frame
 * counting
 * Revision 1.4  1996/01/22 20:35:11  sasha
 * Changed calls to VxD ( added an arument )
 * Revision 1.3  1996/01/10 21:24:43  sasha
 * Changed ColorFormat to ColFmt ( to avoid collision with the register object )
 * Revision 1.2  1996/01/04 23:21:22  sasha
 * First real check-in. Too many changes since previous version
 * Revision 1.1  1995/09/07 05:42:04  sasha
 * Initial revision
 */

#ifndef __VIDINCHN_H
#define __VIDINCHN_H

#ifndef __VFWCHAN_H
#include "vfwchan.h"
#endif

#ifndef __COLFRMAT_H
#include "colfrmat.h"
#endif

#ifndef __COLSPACE_H
#include "colspace.h"
#endif

/* Class: VidInChan
 * Purpose: Implements functionality of the VIDEO_IN channel
 * Attributes: ImageRect_: RECT - holds size of the final image 
 *   Streaming_: bool - indicates if channel is in a streaming mode
 *   dwVideoClock_: DWORD - basically this is an interrupt counter
 * Operations: see VfWChannel for the responses to the VfW messages
 *   The following functions are called mainly internally, but could be called
 *   from a dialog dll too
 *   void SetImageRect( LPRECT lpRect )
 *   void SetImageRect( int x, int y, int w, int h )
 *   void GetImageRect( LPRECT lpRect )
 *   WORD SetDestFormat( LPBITMAPINFOHEADER lpbi, WORD wSize )
 *   WORD GetDestFormat( LPBITMAPINFOHEADER lpbi, WORD wSize )
 *   virtual WORD AddBuffer( LPVIDEOHDR lpNewBuffer );
 *   Store and restore functions
 *   void RestoreValues() - reads values from registry
 *   void StoreValues() - writes values to the registry
 */
class VidInChan : public VfWChannel
{
   private:

      static VidInChan *VidIn_;

      static bool   Created_;
               
      volatile bool InCallback_;
      TRect  ImageRect_;
      ColFmt CurColFormat_;

      // counters
      DWORD dwFramesSkipped_;
      DWORD dwTimeStream_;
      DWORD dwOldSkipped_;

      // channel state stuff
      bool Streaming_;

      void SetStreaming( bool State ) { Streaming_ = State; }
      bool IsStreaming() { return Streaming_; }

      // function called internally from the VfW response methods
      WORD StreamOpen( LPVIDEO_STREAM_INIT_PARMS lpStrInitPar );
      WORD StreamClose();
      WORD StreamStart( bool DoThrow = true );
      WORD FrameStreamStart() ;
      WORD StreamStop(bool bIsFrameStreamReset  = false);
   public:
      WORD StreamReset(bool bIsFrameStreamReset = false);
   private:
      WORD StreamGetErr( LPDWORD lpErrType, LPDWORD lpFramesSkipped );
      WORD StreamGetPos( LPMMTIME lpMMTime, DWORD dwSize );
      WORD GetDestPalette( LPLOGPALETTE lpLogPal, WORD wSize );
      void InitCounters();

      virtual WORD VidConfigMsg( UINT uiMsg, LPARAM lParam1, LPARAM lParam2 );
      virtual WORD VidDialog( HWND, LPARAM );
      virtual WORD SrcRect( LPRECT, LPARAM );
      virtual WORD CaptureFrame( LPVIDEOHDR );
      virtual WORD Caps( LPCHANNEL_CAPS, LPARAM );
      virtual WORD StreamMsg( UINT, LPARAM, LPARAM );
      virtual WORD VidConfigStorage( UINT uiMsg, LPARAM lParam1, LPARAM lParam2 );
      virtual WORD AddBuffer( LPVIDEOHDR lpNewBuffer );

      static  void WINAPI CapVxDCallback( VidInChan *, DWORD dropCount );

      static void CALLBACK _loadds TimerEventCB( UINT IDEvent, UINT uReserved,
         DWORD  dwUser,	DWORD  dwReserved1, DWORD  dwReserved2 );

      // persistence
      void StoreValues();
      void RestoreValues();

      // can not construct directly - call Create()
      VidInChan();

   protected:
      virtual ErrorCode OpenVxDChannel( DWORD dnDevNode );

   public:

      // main access
      static VidInChan &GetVidInChan() { return *VidIn_; }

      // image rect manipulation
      ErrorCode SetImage( LPRECT lpRect, ColFmt aFormat );
      int       GetImageFormat( ColFmt *pColFrmt );

      void SetImageRect( int x, int y, int w, int h )
         { ::SetRect( &ImageRect_, x, y, w, h ); }

      void GetImageRect( LPRECT lpRect ) { *lpRect = ImageRect_; }

      // called during process the DVM_FORMAT or from a dialog dll
      WORD SetDestFormat( LPBITMAPINFOHEADER lpbi, WORD wSize );
      WORD GetDestFormat( LPBITMAPINFOHEADER lpbi, WORD wSize );

      static VidInChan *Create();
      virtual LRESULT  Close();

      ~VidInChan();// { StoreValues(); Created_ = false; CloseVxDChannel(); }
};

/* Method: VidInChan::InitCounters
 * Purpose: Just zeroes out some internal variables
 * Input: None
 * Output: None
 */
inline void VidInChan::InitCounters()
{
   dwFramesSkipped_ = 0;
   dwTimeStream_    = timeGetTime();
   dwOldSkipped_    = 0;
}

/* Method: VidInChan::GetImageFormat
 * Purpose: Used by the dialogs to initialize entry elements properly
 * Input: pColFrmt: ColFmt * - pointer to the variable to contain the
 *   current color format
 * Output: int: Number of bpp for the current format
 */
inline int VidInChan::GetImageFormat( ColFmt *pColFrmt )
{
   ColorSpace tmp( CurColFormat_ );
   *pColFrmt = CurColFormat_;
   return (int)tmp.GetBitCount();
}

#endif //__VIDINCHN_H
