/* File: Vidmsg.cpp
 * Purpose: Contains functions that process Video for Windows messages
 * Author: sasha
 * $Log: VIDMSG.CPP $
 * Revision 2.10  1998/04/20 23:05:57  dtharp
 * Fixes  video trashing upon format change while in preview (bug 801).
 * Also fixes primary surface mode clipping problem for quartzsight
 * (bug 746).
 * Revision 2.9  1998/04/16 17:52:51  dtharp
 * Merge preview rate enhancement branch
 * Revision 2.8.1.1  1998/04/16 16:56:24  dtharp
 * Moved DVM_FRAME into stream message handler
 * Revision 2.8  1997/11/26 01:49:43  peter
 * Fixed MKS graft from branch problem.
 * Revision 2.6.2.1  1997/11/20 22:37:29  peter
 * Changes to move registry subkey to HKEY_LOCAL_MACHINE.
 * Revision 2.6  1997/07/24 21:08:43  dtharp
 * Bug ids 670 & 587. Random faults when capSetVideoFormat issued.
 * Disallowed CaptureFrame() call if immediately proceeded by a DVM_DST_RECT
 * command.
 * Revision 2.5  1997/05/19 20:50:29  dtharp
 * Bug Id 511: Added check for stream capture before allowing frame capture
 * Revision 2.4  1996/12/13 21:22:00  rudyl
 * Bug #: 257
 * Problem: Bt8848Dlg.dll function Initialize() not being called.
 * Solution: When "PreviewOnly" == 1, only 2 channels will be opened.
 *   Check for 2 channels in that case; otherwise check for 3 channels.
 * Revision 2.3  1996/12/03 03:32:02  meierdw
 * Grafting in version 2.2.1; Check for two channels
 * Revision 2.2.1.2  1996/12/03 03:32:02  sasha
 * Compare if 2 channels are created instead of 3 in order to call Initialize().
 * Some people don't have the overlay channel enabled, so Initialize()
 * was not called. In such situation all functionality exposed by overlay
 * channel will not be available.
 * Revision 2.2.1.1  1996/12/03 03:19:22  sasha
 * Revision 2.2  1996/07/01 20:24:12  sasha
 * Check for a NULL returned from Create() functions.
 * Revision 2.1  1996/06/10 20:13:10  ssm
 * MarkBeta1AsRev2
 * Revision 1.7  1996/06/05 03:23:16  sasha
 * Changed channel limit back to 3 as CC channels are not created until later
 * Revision 1.6  1996/05/23 03:30:20  sasha
 * Check for 5 channels before calling Initialize()
 * Revision 1.5  1996/05/17 20:28:17  sasha
 * Added call to Initialize() when all video channels are opened
 * Revision 1.4  1996/05/12 21:12:22  sasha
 * Moved loading of the VxD into VfWChannel class ( created a function )
 * Revision 1.3  1996/01/22 20:35:43  sasha
 * Changed calls to OpenVxDChannel ( added an arument - devnode )
 * Revision 1.2  1996/01/04 23:21:23  sasha
 * First real check-in. Too many changes since previous version
 * Revision 1.1  1995/09/06 03:36:12  sasha
 * Initial revision
 */

#include <windows.h>
#include <mmsystem.h>
#include <msvideo.h>
#include <msviddrv.h>

#include "extvidin.h"
#include "exvidout.h"
#include "vidinchn.h"
#include "dbgprint.h"

extern ErrorCode WINAPI Initialize();


/* Function: VideoProcessMessage
 * Purpose: Main dispatcher of VfW messages
 * Input: VidChan: VfWChannel & - reference to a channel a messages is intended for
 *   uiMsg: UINT - VfW message
 *   lParam1: LPARAM
 *   lParam2: LPARAM
 */
WORD VfWChannel::VideoProcessMessage( VfWChannel &VidChan, UINT uiMsg,
   LPARAM lParam1, LPARAM lParam2 )
{
   WORD status ;

   if ( ! IsRealThing( &VidChan ) )
      return DV_ERR_NONSPECIFIC;

  // Anything but a DVM_FRAME message must shut down the frameserver prior to actioning.
  // This is done in order to emulate the START-FRAME-STOP behavior of old. Essentailly, we
  // need to make sure the frameserver is stopped because the risc engine was never
  // left running before...
  if (uiMsg != DVM_FRAME)
  {
     VidInChan::GetVidInChan().StreamReset(true) ;
  }

   switch ( uiMsg ) {
   case DVM_GETERRORTEXT: // lParam1 = LPVIDEO_GETERRORTEXT_PARMS
      DebugPrint((0,"DVM_GETERRORTEXT\n"));
      if ( LoadString( GetOurInstance(),
         (WORD) ( (LPVIDEO_GETERRORTEXT_PARMS)lParam1 )->dwError,
         (LPSTR)( (LPVIDEO_GETERRORTEXT_PARMS)lParam1 )->lpText,
         (int)  ( (LPVIDEO_GETERRORTEXT_PARMS)lParam1 )->dwLength ))
         status = DV_ERR_OK;
      else
         status = DV_ERR_PARAM1;
      break ;
   //
   //  lParam1	 -   hWndParent
   //
   //  lParam2	 -   flags
   //
   case DVM_DIALOG: // lParam1 = hWndParent, lParam2 = dwFlags
      DebugPrint((0,"DVM_DIALOG\n"));
      status =  VidChan.VidDialog( (HWND) lParam1, (DWORD) lParam2 );
      break ;

   case DVM_PALETTE:
   case DVM_FORMAT:
   case DVM_PALETTERGB555:
      DebugPrint((0,"DVM_PALETTE DVM_FORMAT DVM_PALETTERGB555\n"));
      status = VidChan.VidConfigMsg(  uiMsg, lParam1, lParam2 );
      break ;

   case DVM_SRC_RECT:
      DebugPrint((0,"DVM_SRC_RECT\n"));
      status = ( lParam1 == NULL )? DV_ERR_PARAM1 : VidChan.SrcRect( (LPRECT) lParam1, lParam2 );
      break ;

   case DVM_DST_RECT:
      DebugPrint((0,"DVM_DST_RECT\n"));
      status =  ( lParam1 == NULL ) ? DV_ERR_PARAM1 : VidChan.DstRect( (LPRECT) lParam1, lParam2 );
      break ;

   case DVM_UPDATE:
      DebugPrint((0,"DVM_UPDATE\n"));
      status =  VidChan.Update( (HWND) lParam1, (HDC) lParam2 );
      break ;

   case DVM_CONFIGURESTORAGE:
      DebugPrint((0,"DVM_CONFIGURESTORAGE\n"));
      status = VidChan.VidConfigStorage( uiMsg, lParam1, lParam2 );
      break ;

   case DVM_GET_CHANNEL_CAPS:
      DebugPrint((0,"DVM_GET_CHANNEL_CAPS\n"));
      status = VidChan.Caps( (LPCHANNEL_CAPS) lParam1,  (DWORD)lParam2 );
      break ;

   default:
      status = VidChan.StreamMsg( uiMsg, lParam1, lParam2 );
      break ;
   } /* endswitch */

   return status ;
}

/* Function: VideoOpen
 * Purpose: Creates an instance of one of the VfWChannel descendants
 * Input: lpVidOpenPar: LPVIDEO_OPEN_PARMS - pointer
 * Output: DWORD: cast pointer to a chanel
 */
DWORD VfWChannel::VideoOpen( LPVIDEO_OPEN_PARMS lpVidOpenPar )
{
   if ( !lpVidOpenPar )
      return 0;

   VfWChannel *NewVidChan = 0;

   // have to check this explicitly as higher bits can be set to fool the switch
   if ( lpVidOpenPar->dwFlags != VIDEO_EXTERNALIN  &&
        lpVidOpenPar->dwFlags != VIDEO_EXTERNALOUT &&
        lpVidOpenPar->dwFlags != VIDEO_IN          &&
        lpVidOpenPar->dwFlags != VIDEO_OUT )
      return NULL;

   // initialize return values
   lpVidOpenPar->dwError = DV_ERR_ALLOCATED;

   switch ( lpVidOpenPar->dwFlags &
      (VIDEO_EXTERNALIN | VIDEO_EXTERNALOUT | VIDEO_IN | VIDEO_OUT ) ) {
   case VIDEO_EXTERNALIN:
      NewVidChan = ExtVidInChan::Create();
      break;
   case VIDEO_EXTERNALOUT:
      NewVidChan = ExtVidOutChan::Create();
      break;
   case VIDEO_IN:
      NewVidChan = VidInChan::Create();
      break;
   case VIDEO_OUT: //break is ommitted on purpose
   default:
      return NULL;
   } /* endswitch */

   if ( !NewVidChan ) // something ( like DDI ) went wrong
      return NULL;

   // now do the magic of loading Pisces VxD portion
   if ( NewVidChan->LoadVxD() != Success ) {
      delete NewVidChan;
      ErrorDialog( XMSG( IDS_VXDGENERAL ) );
      return 0;
   }

   if ( NewVidChan->OpenVxDChannel( lpVidOpenPar->dnDevNode ) != Success )
      return 0;

   // no error occured
   lpVidOpenPar->dwError = 0;

   // Decide to call Initialize() depends 2 or 3 channels will be opened
   RegUtil Storage( "SOFTWARE\\Brooktree\\Video" );
   DWORD dwTmp = 0;  // working variable

   // If "PreviewOnly" == 1, only 2 channels will be opened so Initialize()
   // will be called when both opened; otherwise check for 3 channels
   if ( ( Storage.GetValue( &dwTmp, "PreviewOnly" ) == Success ) &&
        ( dwTmp == 1 ) )
   {
      if ( GetCreatedChannels() == 2 )
         Initialize();
   }
   else
      if ( GetCreatedChannels() == 3 )
         Initialize();

   return (DWORD)(LPVOID)NewVidChan;
}

