/* File: Vxdiface.cpp
 * Purpose: Implementation of the VxDIface class
 * Author: sasha
 * $Log: VXDIFACE.CPP $
 * Revision 2.1  1996/06/10 20:13:11  ssm
 * MarkBeta1AsRev2
 * Revision 1.4  1996/02/18 05:25:56  sasha
 * Removed argument name from Load and Unload functions ( to prevent warnings )
 * Revision 1.3  1996/01/22 20:36:11  sasha
 * Changed calls to VxD ( added an arument )
 * Revision 1.2  1996/01/04 23:21:25  sasha
 * First real check-in. Too many changes since previous version
 * Revision 1.1  1995/09/07 06:16:12  sasha
 * Initial revision
 */

#include "vxdiface.h"

#pragma inline

/* Method: VxDIface::CallVxD
 * Purpose: This function  makes a call to the BtPisces VxD
 * Input: Num: FuncNumber - number of function call
 *   dwpData: DWORD - data expected by the VxD
 * Output: None
 * Note: the calling convention is: AX - function number
 *   EDX - DWORD data
 *   ESI - PVOID data
 *   EBX - channel handle
 */
ErrorCode VxDIface::CallVxD( FuncNumber Num, LPVOID pData, DWORD dwData,
   DWORD hChannel )
{
   ErrorCode retVal = Fail;

   if ( PiscesVxD_ ) {
      VXD_ENTRY Pisces = PiscesVxD_;
      asm {
         push cx
         push bx
         push si
         mov  edx, dword ptr dwData
         mov  esi, dword ptr pData
         mov  ebx, dword ptr hChannel
         mov  ax, word ptr Num
         call dword ptr Pisces;
         mov  retVal, ax
         pop  si
         pop  bx
         pop  cx
      }
   }
   return retVal;
}

/* Method: VxDIface::LoadVxD
 * Purpose: Obtains the entry point of a VxD
 * Input: Name: LPCSTR - constant pointer to the VxD name
 * Output: None
 */
void VxDIface::LoadVxD()
{
   asm     push ds
   PiscesVxD_ = GetVxDEntry( PISCES_DEVICE_ID );

Error:
   asm   pop  ds

   // now have an address to call Pisces
//   PiscesVxD_ = TmpEntry;
}

/* Method: VxDIface::UnloadVxD
 * Purpose: Obtains the entry point of a VxD
 * Input: Name: LPCSTR - constant pointer to the VxD name
 * Output: None
 */
void VxDIface::UnloadVxD()
{
   PiscesVxD_ = NULL;
}


/* Method: VxDIface::GetVxDEntry
 * Purpose: Obtains the entry point of a VxD
 * Input: None
 * Output: VXD_ENTRY: address of the VxD or NULL
 */
VXD_ENTRY VxDIface::GetVxDEntry( const int VxDId )
{
   VXD_ENTRY VxDLoader;
   _DI = 0;
   _ES = 0;                 // es:di = 0 on entry
   _AX = 0x1684;            // get entry point function number
   _BX = VxDId;
   asm int     0x2F;        // get protected-mode API address
   asm mov word ptr VxDLoader, di
   asm mov word ptr VxDLoader + 2, es
   return VxDLoader;
}
