/* File: Dialogs.cpp
 * Purpose: Configuration dialogs
 *
 * $Log: dialogs.cpp $
 * Revision 2.35  1998/05/21 17:55:59  dwolfson
 * Changed the 1/4 scale format size from 88x72 to 80x72 for QuartzSight
 * Revision 2.34  1998/04/22 10:58:48  trang
 * Make the Dialogs DLL to dynamic link with BtVid_16.dll.
 * Revision 2.33  1998/04/14 18:41:20  rudyl
 * Per VVL, modified QSight lighting type default from Backlight to Normal.
 * Revision 2.32  1998/04/06 19:16:27  rudyl
 * Per VVL's suggestion, default value for brightness is modified to be the
 * same as saturation and hue; i.e. 50. Just add 30 to get actual reg value.
 * Revision 2.31  1998/04/01 02:32:42  rudyl
 * Removed set controls back to default when Qsight camera is selected as video source.
 * Revision 2.30  1998/03/31 02:08:13  rudyl
 * Added manufacturer default settings for QuartzSight registers.
 * Revision 2.29  1998/03/28 01:53:40  rudyl
 * Graft from 2.28.1.3: moved scotch to vxd.
 * Read override defaults from qsight.ini file.
 * Will modify it to read from registry.
 * Revision 2.28.1.3  1998/03/28 01:29:01  rudyl
 * Use TRUE when calling VP_DEFAULTS.
 * Revision 2.28.1.2  1998/03/24 01:55:51  rudyl
 * Moved QuartzSight init to vxd.
 * Revision 2.28.1.1  1998/03/18 04:23:42  rudyl
 * For CeBIT: allow user to set own default values in "windows\QSight.ini".
 * Revision 2.28  1998/03/10 23:47:42  rudyl
 * Fixed first time start QSight has black screen problem.
 * Revision 2.27  1998/03/10 20:55:50  rudyl
 * Fixed Bug# 721: Default setting not operating correctly.
 * Also remove QSight setup dlg if non-QS camera is selected.
 * Revision 2.26  1998/03/05 00:12:31  rudyl
 * If Video source dlg is cancelled, restore QuartzSight controls only if
 * different from original values to avoid waiting for AEC to complete which takes a short while.
 * Revision 2.25  1998/03/04 23:31:41  rudyl
 * Don't have a spearate QSight setup dlg anymore. Added to vid source dlg.
 * Revision 2.24  1998/02/28 01:52:57  rudyl
 * Removed FrameRate selection for QuartzSight.
 * Revision 2.23  1998/02/26 20:12:29  rudyl
 * Resolved QuartzSight setup dialog issues.
 * Revision 2.22  1998/02/25 00:46:09  rudyl
 * Use updated SRAM table for QuartzSight with VDelay = 0x08.
 * Revision 2.21  1998/02/24 21:52:01  rudyl
 * Include header file renamed from "vpcamdef.h" to "cam_defs.h".
 * Revision 2.20  1998/02/24 20:23:37  rudyl
 * Graft from 2.19.1.3.
 * Revision 2.19.1.3  1998/02/24 20:23:21  rudyl
 * Removed unused codes.
 * Revision 2.19.1.2  1998/02/24 20:18:30  rudyl
 * Added VVL's VPAEC interface to QuartzSight.
 * Revision 2.19.1.1  1998/02/12 02:51:17  rudyl
 * Implemented Rob Tilton's QuartzSight dialog box. But still unable to talk to Scotch.
 * Revision 2.19  1998/02/03 20:14:21  rudyl
 * Disabled hue control in video source dlg for digital camera.
 * Revision 2.18  1998/02/03 20:12:19  rudyl
 * Gemini Rev D: HSFMT changes from 1 to 2 bits.
 * Revision 2.17  1998/01/27 20:00:22  rudyl
 * Revert to Rev C for Fusion Beta 2 release. Rev D changes are commented out.
 * Revision 2.16  1998/01/16 21:53:19  rudyl
 * Gemini Rev D: HSFMT changes from 1 to 2 bits. Also API changed.
 * Revision 2.15  1997/12/13 01:53:17  rudyl
 * Graft from 2.14.1.1. Added SRAM API.
 * Revision 2.14.1.1  1997/11/27 03:11:48  rudyl
 * Moved SRAM table to dialogs.dll.
 * Revision 2.14  1997/11/26 06:29:49  rudyl
 * Added support of Scotch.
 * Revision 2.11.2.1  1997/11/26 00:35:46  rudyl
 * Added support to Bt832 (Scotch).
 * Revision 2.11  1997/10/29 02:22:11  rudyl
 * Added trueview.
 * Revision 2.10  1997/10/21 02:01:21  rudyl
 * Added support to expected total line count by decoder.
 * Revision 2.9  1997/09/15 18:40:55  rudyl
 * Added support to "Rockwell QuartzSight PCI 1.0" camera.
 * Revision 2.8  1997/07/10 00:15:46  trantq
 * - Add typecast DLGPROC to last parameters to DialogBox().
 * Revision 2.7  1997/07/09 17:49:40  trantq
 * Merge in Revision 2.3.2.3, the branch was never checked into the
 * tip.
 * Revision 2.6  1997/05/06 00:20:52  dtharp
 * 848A Merge: from 2.3.1.10
 * Revision 2.5  1997/03/18 20:01:52  rudyl
 * Graft from 2.4.1.2. Changes sizes since PAL/SECAM use 288 lines per field.
 * Revision 2.4.1.2  1997/03/08 02:47:53  rudyl
 * Now PAL uses 288 lines per field.
 * Revision 2.4.1.1  1997/03/06 20:09:37  rudyl
 * Use 288 lines for PAL/SECAM
 * Revision 2.4  1996/11/08 18:47:16  rudyl
 * Graft from 2.3.1.2: Added NTSC-Japan, PAL-M & PAL-N formats.
 * Revision 2.3.1.2  1996/11/08 18:47:15  rudyl
 * Added NTSC-Japan, PAL-M & PAL-N formats.
 * Revision 2.3.1.1  1996/11/07 22:20:32  rudyl
 * Start work to add NTSC-Japan, PAL-M & PAL-N formats.
 * Revision 2.3.2.3  1996/11/28 01:47:19  trantq
 * Elimination #ifdef by type-casting (DLGPROC) for DialogBox call.
 * Revision 2.3.2.2  1996/11/28 00:59:13  trantq
 * -Make change in WinHelp Call to bring up help file's HELP_CONTENTS,0 instead of bringing up context sensitive help.
 * -Make revision compilable by Borland and Microsoft.
 * Revision 2.3.2.1  1996/11/26 21:22:07  trantq
 * Make dialogs compilable by Borland and Miscrosoft
 * Revision 2.3  1996/09/17 08:10:36  sasha
 * Added support for the raw mode. More bells and wistles could be added
 * for that ( dealing with size buttons ).
 * Revision 2.2  1996/09/06 21:38:22  rudyl
 * Allow all Image Format (in particular YUV9 & YUV12) to go full screen.
 * Revision 2.1  1996/06/10 20:13:16  ssm
 * MarkBeta1AsRev2
 * Revision 1.23  1996/06/05 04:09:36  sasha
 * A bogus check-in to fix that non-updated member crap
 * Revision 1.21.1.1  1996/06/05 04:08:03  sasha
 * Added 2 edit boxes to handle CC file name and alert token
 * Revision 1.21  1996/06/04 02:45:11  ssm
 * add Initial revision line to log section so stripLog.awk will work
 * Revision 1.20  1996/05/30 21:18:33  sasha
 * Put video display dialog here. It controls CC parameters right now.
 * Revision 1.19  1996/05/21 18:50:21  rudyl
 * Uses PAL_BDGHI for default PAL format.
 * Per Ben's suggestion, since we don't have that many lines available to display,
 * changed sizes for PAL/SECAM from 288 to 284 for 1/2, and similarly for 1/4 and full size.
 * Revision 1.18  1996/05/17 00:18:27  sasha
 * Added WINAPI to the dialog functions
 * Revision 1.17  1996/05/02 21:26:49  sasha
 * Added PAL/SECAM sizes and supporting code. Added audio functions
 * which are called when connector changes
 * Revision 1.16  1996/04/24 20:29:44  sasha
 * Fixed image size when 1/4 button pressed
 * Revision 1.15  1996/04/17 22:17:08  sasha
 * Changed scroll bars range.
 * Revision 1.14  1996/04/17 19:23:36  sasha
 * Removed commented code. Changed code to reflect new contents of the
 * video formats and sizes arrays.
 * Revision 1.13  1996/04/09 19:28:40  sasha
 * Added support for disabling hue for PAL and SECAM
 * Revision 1.12  1996/04/01 21:06:20  sasha
 * Use new connector names again
 * Revision 1.11  1996/03/30 01:24:20  sasha
 * Commented out extra color formats and sizes in the video format dialog
 * Revision 1.10  1996/03/29 18:18:06  sasha
 * Changed video format names ( as per viddefs.h )
 * Revision 1.9  1996/03/15 23:20:44  sasha
 * Updated connector values relation to radio button IDs
 * Revision 1.8  1996/03/15 21:02:04  sasha
 * Added handling of video connector radio buttons and changed associations
 * between connector values and radio battons
 * Revision 1.7  1996/02/18 06:15:31  sasha
 * Added validation for the 'full' button as well
 * Revision 1.6  1996/02/18 05:22:01  sasha
 * Added validation for the image size/color format to the video format dialog
 * Revision 1.5  1996/02/14 22:17:26  sasha
 * Added IDCANCEL and IDOK to the big config dialog
 * Revision 1.4  1996/01/22 20:36:29  sasha
 * Revision 1.3  1996/01/10 21:44:51  sasha
 * Revision 1.2  1996/01/10 21:28:51  sasha
 * Functional dialogs

 * Initial revision
 */
// CUT_HERE

//===========================================================================
// Search for <UPDATE REQUIRED> in order to add/modify support of additional
// cameras.
//===========================================================================

#include <stdlib.h>
#include "msvccomp.h"
#include "api.h"
#include "config.h"
#include "exviddlg.h"
#include "dialogs.h"
#include "cam_defs.h"
#include "dlkVid16.h"
#include "audio.h"

#include "regutil.h"

extern HINSTANCE WINAPI GetOurInstance( void );

static int SizeOption;
static int VidSourceUp;
static BOOL bQSDlgUp;

typedef struct tag_combobox_entry
{
   WORD wValue;
   char *szText;
} COMBOBOX_ENTRY;


//
// Loads entries into a combobox, and selects the current index
// Parmameters:
//      hWnd of the parent dialog box
//      ID of the combobox
//      array of text and values
//      Count of entries in the COMBOBOX_ENTRY array
//      Initial value which should match the wValue field of the
//              COMBOBOX_ENTRY.  If no values match, the selection
//              defaults to the first entry in the combobox.
// Returns:
//      Returns the index of the selected item in the combobox.
//

static int WINAPI ConfigLoadComboBox( HWND hDlg, int wID, COMBOBOX_ENTRY *pCBE,
   int nEntries, WORD wInitialValue )
{
   int	j;
   int	nIndex = 0;	// Zero entry should be blank, None, etc.
   HWND hWndCB = GetDlgItem( hDlg, wID );

   // make sure no old strings are present
   int items = (int)SendMessage( hWndCB, CB_GETCOUNT, 0, 0 );
   for ( ; items >= 0; )
      SendMessage( hWndCB, CB_DELETESTRING, --items, 0 );

  // add new strings
   for ( j = 0; j < nEntries; j++ ) {
      SendMessage( hWndCB, CB_ADDSTRING, 0, (LONG)(LPSTR)((pCBE+j)->szText ) );
      if ( pCBE [j].wValue == wInitialValue )
         nIndex = j;
      }
   SendMessage( hWndCB, CB_SETCURSEL, nIndex, 0L );
   return nIndex;
}

const int CameraSizeBase = 4;  // where do the camera sizes start?

static COMBOBOX_ENTRY gwSizeOptions[] [3] =
{
   {  // NTSC compatible
      { 0, "160 x 120" },
      { 1, "320 x 240" },
      { 2, "640 x 480" }
   },
   {  // PAL compatible
      { 0, "192 x 144" },
      { 1, "384 x 288" },
      { 2, "768 x 576" }
   },
   {  // Raw
      { 0, "1024 x 434" },
      { 1, "1024 x 868" },
      { 2, "1024 x 868" }
   },
   {  // PAL-NC
      { 0, "160 x 144" },
      { 1, "320 x 288" },
      { 2, "640 x 576" }
   },
// <UPDATE REQUIRED>
// Following is the available video sizes for each camera
   {  // digital camera: SVI
      { 0, "160 x 120" },
      { 1, "240 x 180" },
      { 2, "320 x 240" }
   },
   {  // digital camera: Logitech VideoMan 1.3
      { 0, "160 x 120" },
      { 1, "320 x 240" },
      { 2, "640 x 480" }
   },
   {  // digital camera: Rockwell QuartzSight PCI 1.0
      { 0, " 80 x  72" },
      { 1, "176 x 144" },
      { 2, "352 x 288" }
   },
};
#define N_SIZEOPTIONS (sizeof (gwSizeOptions [0] ) / sizeof (COMBOBOX_ENTRY))

static COMBOBOX_ENTRY gwFormatOptions[] =
{
   { CF_RGB32, "32 bit RGBA" },
   { CF_RGB24, "24 bit RGB"  },
   { CF_RGB15, "15 bit RGB"  },
   { CF_YUY2,  "YUY2"        },
   { CF_BTYUV, "BTYUV"       },
   { CF_YUV9,  "YUV9"        },
   { CF_I420,  "YUV12"       },
   { CF_RAW,   "Raw ADC data"}
};

#define N_FORMATOPTIONS (sizeof (gwFormatOptions) / sizeof (COMBOBOX_ENTRY))

RECT RectArr [] [3] =
{
   {  // NTSC compatible
      { 0, 0, 160, 120 },
      { 0, 0, 320, 240 },
      { 0, 0, 640, 480 }
   },
   {  // PAL compatible
      { 0, 0, 192, 144 },
      { 0, 0, 384, 288 },
      { 0, 0, 768, 576 }
   },
   {  // Raw
      { 0, 0, 1024, 434 },
      { 0, 0, 1024, 868 },
      { 0, 0, 1024, 868 }
   },
   {  // PAL-NC
      { 0, 0, 160, 144 },
      { 0, 0, 320, 288 },
      { 0, 0, 640, 576 }
   },
// <UPDATE REQUIRED>
// Following is the available video sizes for each camera
   {  // digital camera: SVI
      { 0, 0, 160, 120 },
      { 0, 0, 240, 180 },
      { 0, 0, 320, 240 }
   },
   {  // digital camera: Logitech VideoMan 1.3
      { 0, 0, 160, 120 },
      { 0, 0, 320, 240 },
      { 0, 0, 640, 480 }
   },
   {  // digital camera: Rockwell QuartzSight PCI 1.0
      { 0, 0,  80,  72 },
      { 0, 0, 176, 144 },
      { 0, 0, 352, 288 }
   },
};

#define ARR_SIZE( arr ) sizeof( arr ) / sizeof( arr##[0] )


static COMBOBOX_ENTRY gwStandardOptions[] =
{
   { VFormat_NTSC,      "NTSC-M"        },
   { VFormat_NTSC_J,    "NTSC-Japan"    },
   { VFormat_PAL_BDGHI, "PAL-B,D,G,H,I" },
   { VFormat_PAL_M,     "PAL-M"         },
   { VFormat_PAL_N,     "PAL-N"         },
   { VFormat_SECAM,     "SECAM"         },
   { VFormat_PAL_NC,    "PAL-NC"        }
};
#define N_STANDARDOPTIONS (sizeof (gwStandardOptions) / sizeof (COMBOBOX_ENTRY))

//===========================================================================
// <UPDATE REQUIRED>
// List of supported camera. Update this list when camera models are added or
// deleted.
//===========================================================================
typedef enum { SVI, Logitech, Rockwell } CAMERA;

static COMBOBOX_ENTRY gwCameraOptions[] =
{
   { SVI,      "Silicon Vision 512N" },
   { Logitech, "Logitech VideoMan 1.3"  },
   { Rockwell, "Rockwell QuartzSight PCI 1.0"   }
};
#define N_CAMERAOPTIONS (sizeof (gwCameraOptions) / sizeof (COMBOBOX_ENTRY))

//===========================================================================
// SRAM table values
//===========================================================================
typedef enum { TGB_NTSC624, TGB_NTSC780, TGB_NTSC858, TGB_NTSC392 } TimeGenByte;

BYTE SRAMTable[][ 60 ] =
{
   // TGB_NTSC624
   {
      0x33, // size of table = 51
      0x0E, 0xC0, 0x00, 0x00, 0x90, 0x02, 0x03, 0x10, 0x03, 0x06,
      0x10, 0x04, 0x12, 0x12, 0x05, 0x02, 0x13, 0x04, 0x19, 0x00,
      0x04, 0x39, 0x00, 0x06, 0x59, 0x08, 0x03, 0x85, 0x08, 0x07,
      0x03, 0x50, 0x00, 0x91, 0x40, 0x00, 0x11, 0x01, 0x01, 0x4D,
      0x0D, 0x02, 0x03, 0x11, 0x01, 0x05, 0x37, 0x00, 0x37, 0x21, 0x00
   },
   // TGB_NTSC780
   {
      0x33, // size of table = 51
      0x0e, 0xc0, 0x00, 0x00, 0x90, 0xe2, 0x03, 0x10, 0x03, 0x06,
      0x10, 0x34, 0x12, 0x12, 0x65, 0x02, 0x13, 0x24, 0x19, 0x00,
      0x24, 0x39, 0x00, 0x96, 0x59, 0x08, 0x93, 0x85, 0x08, 0x97,
      0x03, 0x50, 0x50, 0xaf, 0x40, 0x30, 0x5f, 0x01, 0xf1, 0x7f,
      0x0d, 0xf2, 0x03, 0x11, 0xf1, 0x05, 0x37, 0x30, 0x85, 0x21, 0x50
   },
   // TGB_NTSC858
   {
      0x33, // size of table = 51
      0x0c, 0xc0, 0x00, 0x00, 0x90, 0xc2, 0x03, 0x10, 0x03, 0x06,
      0x10, 0x34, 0x12, 0x12, 0x65, 0x02, 0x13, 0x24, 0x19, 0x00,
      0x24, 0x39, 0x00, 0x96, 0x59, 0x08, 0x93, 0x83, 0x08, 0x97,
      0x03, 0x50, 0x30, 0xc0, 0x40, 0x30, 0x86, 0x01, 0x01, 0xa6,
      0x0d, 0x62, 0x03, 0x11, 0x61, 0x05, 0x37, 0x30, 0xac, 0x21, 0x50
   },
   // TGB_NTSC392
   // This table has been modified to be used for Fusion Rev D
   {
      0x2A, // size of table = 42
      0x06, 0x08, 0x04, 0x0a, 0xc0, 0x00, 0x18, 0x08, 0x03, 0x24,
      0x08, 0x07, 0x02, 0x90, 0x02, 0x08, 0x10, 0x04, 0x0c, 0x10,
      0x05, 0x2c, 0x11, 0x04, 0x55, 0x48, 0x00, 0x05, 0x50, 0x00,
      0xbf, 0x0c, 0x02, 0x2f, 0x3d, 0x00, 0x2f, 0x3f, 0x00, 0xc3,
      0x20, 0x00
   }
};

//===========================================================================
// This is the structure of the camera specifications
//===========================================================================
typedef struct tag_cameraSpec
{
   SignalFormat signal;       // which digital signal format the camera has
   VideoFormat  vidFormat;    // video standard
   SyncVideoRef syncRef;      // which sync video reference is used
   State        syncOutput;   // enable sync output for sync video input?
   DecInputClk  iClk;         // which input clock is used
   TimeGenByte  tgb;          // which timing generator byte does the camera use
   int          HReset;       // select 64, 48, 32, or 16 CLKx1 for HReset
   PLLFreq      pllFreq;      // what synthesized frequency to set PLL to
   VSIZEPARMS   vSize;        // video size the camera produces
   int          lineCount;    // expected total number of half-line per frame - 1
   BOOL         interlace;    // interlace signal?
} CameraSpec;

//===========================================================================
// <UPDATE REQUIRED>
// Camera specifications database. Update this table whenever camera spec
// has been changed or added/deleted supported camera models
//===========================================================================
static CameraSpec dbCameraSpec[ N_CAMERAOPTIONS ] =
{  // Silicon Vision 512N
   { Signal_CCIR656, VFormat_NTSC, VRef_alignedCb, Off, DecClk_GPCLK, TGB_NTSC624, 64, KHz19636,
      // Clkx1_HACTIVE, Clkx1_HDELAY, VActive, VDelay, linesPerField; lineCount, Interlace
   {         512,           0x64,       480,    0x13,      240 },         0,       TRUE
   },
   // Logitech VideoMan 1.3
   { Signal_CCIR656, VFormat_NTSC, VRef_alignedCb, Off, DecClk_GPCLK, TGB_NTSC780, 64, KHz24545,
      // Clkx1_HACTIVE, Clkx1_HDELAY, VActive, VDelay, linesPerField; lineCount, Interlace
      {      640,           0x80,       480,    0x1A,      240 },         0,       TRUE
   },
   // Rockwell QuartzSight
   // Note: Fusion Rev D (rev ID 0x02) and later supports 16 pixels for HReset which is preferable.
   //       Use 32 for earlier version of hardware. Clkx1_HDELAY also changed from 0x27 to 0x20.
   { Signal_CCIR656, VFormat_NTSC, VRef_alignedCb, Off, DecClk_GPCLK, TGB_NTSC392, 16, KHz28636,
      // Clkx1_HACTIVE, Clkx1_HDELAY, VActive, VDelay, linesPerField; lineCount, Interlace
      {      352,           0x20,       576,    0x08,      288 },       607,       FALSE
   }
};

//===========================================================================
// Call APIs to use digital camera as video source input
//===========================================================================
static bool SetDigitalCamera( CAMERA index )
{
   CameraSpec spec;
   if ( ( index < 0 ) || ( index >= N_CAMERAOPTIONS ) )
      return false;

   spec = dbCameraSpec[ index ];

   // call APIs to set up appropriately for the selected camera
   SetSignalFormat( spec.signal );
   SetVideoFormat( spec.vidFormat );
   SetSyncVideoRef( spec.syncRef );
   SetEnableSyncOutput( spec.syncOutput );
   SetTimGenByte( SRAMTable[ spec.tgb ] + 1, *SRAMTable[ spec.tgb ] );
   SetHReset( spec.HReset );
   SetPLL( spec.pllFreq );
   SetDecInputClock( spec.iClk );
   SetVideoInfo( spec.vSize );
   if ( DlkIsBt878Family() )
      SetTotalLineCount( spec.lineCount );
   SetInterlaceMode( spec.interlace );

   // Configure to use Rockwell QuartzSight camera if selected
   SetQuartzSightMode( index == Rockwell );

   return true;
}

//===========================================================================
// Return the size option for a video source
// 0:  NTSC-M, NTSC-Japan, PAL-M
// 1:  PAL-BDGHI, PAL-N, SECAM
// 3:  PAL-NC
// 4+: digital cameras
// NOTE: 2 is used by the raw mode and is NOT considered here.
//===========================================================================
static int GetSizeOption( SignalFormat sf, int camIndex, VideoFormat vf )
{
   // PAL-NC has horizontal size of NTSC and vertical size of PAL
   if ( sf != Signal_Analog )
      return CameraSizeBase + camIndex;
   else
      return ( vf == VFormat_PAL_NC ) ? 3 :
             IsNTSCCompatible( vf ) ? 0 : 1;
}

//===========================================================================
// Enable/Disable hue controls since they are only available for NTSC signals;
// not for digital source and PAL/SECAM analog signals
//===========================================================================
static void EnableHueControls( HWND hDlg, bool flag )
{
   EnableWindow( GetDlgItem( hDlg, ID_EBHUE ), flag );
   EnableWindow( GetDlgItem( hDlg, ID_SBHUE ), flag );
}

//===========================================================================
// Save camera model index to Windows registry
//===========================================================================
static void StoreCameraIndex( int index )
{
   RegUtil Storage( "SOFTWARE\\Brooktree\\Video" );
   Storage.SetValue( index, "CameraIndex" );
}

//===========================================================================
// Return camera model index from Windows registry
//===========================================================================
static int RestoreCameraIndex( void )
{
   RegUtil Storage( "SOFTWARE\\Brooktree\\Video" );
   DWORD dwTmp = 0;  // working variable

   if ( Storage.GetValue( &dwTmp, "CameraIndex" ) != Success )
      return 0;
   else
      return ( (int)dwTmp );
}

//===========================================================================
// Enable/Disable QuartzSight camera controls
//===========================================================================
static void EnableQuartzSightControls( HWND hDlg, bool flag )
{
   RECT r;
   GetWindowRect( hDlg, &r );

   // expand/shrink dlg size
   MoveWindow( hDlg, r.left, r.top, ( flag ? 690 : 333 ), r.bottom - r.top, TRUE );

   SetWindowText( GetDlgItem( hDlg, ID_PBCAMSETUP ), ( flag ? "Set&up <<" : "Set&up >>" ) );

   EnableWindow( GetDlgItem( hDlg, ID_SBQSBRIGHTNESS ), flag );
   EnableWindow( GetDlgItem( hDlg, ID_SBQSSATURATION ), flag );
   EnableWindow( GetDlgItem( hDlg, ID_SBQSHUE	     ), flag );
   EnableWindow( GetDlgItem( hDlg, ID_SBQSGAMMA	     ), flag );
   EnableWindow( GetDlgItem( hDlg, ID_SBQSSHARPNESS  ), flag );
   EnableWindow( GetDlgItem( hDlg, ID_EBQSBRIGHTNESS ), flag );
   EnableWindow( GetDlgItem( hDlg, ID_EBQSSATURATION ), flag );
   EnableWindow( GetDlgItem( hDlg, ID_EBQSHUE	     ), flag );
   EnableWindow( GetDlgItem( hDlg, ID_EBQSGAMMA	     ), flag );
   EnableWindow( GetDlgItem( hDlg, ID_EBQSSHARPNESS  ), flag );
   EnableWindow( GetDlgItem( hDlg, ID_RBQS50HZ	     ), flag );
   EnableWindow( GetDlgItem( hDlg, ID_RBQS60HZ	     ), flag );
   EnableWindow( GetDlgItem( hDlg, ID_RBQSNORMAL	  ), flag );
   EnableWindow( GetDlgItem( hDlg, ID_RBQSBACKLIGHT  ), flag );
}

//===========================================================================
// Enable/Disable some controls when analog or digital signal is selected
//===========================================================================
static void AnalogSignalSelected( HWND hDlg, bool flag )
{
   EnableWindow( GetDlgItem( hDlg, ID_CBSTANDARD ),  flag );
   EnableWindow( GetDlgItem( hDlg, ID_CBCAMERA ),   !flag );

   // disable "Setup..." dialog for QuartzSight camera if analog source is selected
   if ( flag )
   {
      EnableWindow( GetDlgItem( hDlg, ID_PBCAMSETUP ), FALSE );
      bQSDlgUp = FALSE;
      EnableQuartzSightControls( hDlg, bQSDlgUp );
   }
}

//===========================================================================
// Obtain manufacturer's default settings
//===========================================================================
void GetQSightManufacturerDefaults( int * nQSBrightness,
                                    int * nQSSaturation,
                                    int * nQSHue,
                                    int * nQSGamma,
                                    int * nQSSharpness,
                                    int * nQSLightFreq,
                                    int * nQSLightType )
{
   DWORD dwTmp = 0;
   RegUtil regManufacturer( "SOFTWARE\\Brooktree\\Video\\QuartzSight\\ManufacturerDefaults" );

   *nQSBrightness = ( regManufacturer.GetValue( &dwTmp, "Brightness" ) != Success ) ? 50               : (BYTE)dwTmp;
   *nQSSaturation = ( regManufacturer.GetValue( &dwTmp, "Saturation" ) != Success ) ? 50               : (BYTE)dwTmp;
   *nQSHue        = ( regManufacturer.GetValue( &dwTmp, "Hue"        ) != Success ) ? 50               : (BYTE)dwTmp;
   *nQSGamma      = ( regManufacturer.GetValue( &dwTmp, "Gamma"      ) != Success ) ? 8                : (BYTE)dwTmp;
   *nQSSharpness  = ( regManufacturer.GetValue( &dwTmp, "Sharpness"  ) != Success ) ? 3                : (BYTE)dwTmp;
   *nQSLightFreq  = ( regManufacturer.GetValue( &dwTmp, "LightFreq"  ) != Success ) ? HZ_60            : (BYTE)dwTmp;
   *nQSLightType  = ( regManufacturer.GetValue( &dwTmp, "LightType"  ) != Success ) ? LIGHTING_NORMAL  : (BYTE)dwTmp;

	SetQuartzSightParam( VP_AEC_TI ,      *nQSBrightness + 15 );
   SetQuartzSightParam( VP_AEC_TC ,      *nQSBrightness + 30 );
   SetQuartzSightParam( VP_AEC_TH ,      *nQSBrightness + 45 );
   SetQuartzSightParam( VP_SATURATION,   *nQSSaturation );
   SetQuartzSightParam( VP_HUE,          *nQSHue );
   SetQuartzSightParam( VP_GAMCOR,       *nQSGamma );
   SetQuartzSightParam( VP_SHARPNESS,    *nQSSharpness );
   SetQuartzSightParam( VP_AEC_MAINS,    *nQSLightFreq );
   SetQuartzSightParam( VP_AEC_LIGHTING, *nQSLightType );
}

inline VOID UpdateSizeDisplay( HWND hDlg, WPARAM wIdx )
{
   SendDlgItemMessage( hDlg, ID_LBSIZE, CB_SETCURSEL, wIdx, 0L );
}

static WORD WINAPI ConfigGetComboBoxValue( HWND  hDlg, int wID,
   COMBOBOX_ENTRY *pCBE )
{
   int	nIndex;
   HWND	hWndCB = GetDlgItem( hDlg, wID );
   nIndex = (int) SendMessage( hWndCB, CB_GETCURSEL, 0, 0L );
   nIndex = max( 0, nIndex );//LB_ERR is negative
   return pCBE [nIndex].wValue;
}

///
#pragma argsused
int WINAPI _loadds VideoFormatDlgProc( HWND hDlg, UINT msg, WPARAM wParam,
   LPARAM lParam )
{
   WORD CurSelSize;
   ColFmt Col;
   RECT r;
   int i;

   switch ( msg ) {
   case WM_INITDIALOG:
      GetImageSize( & r );
      GetImageFormat( &Col );

      if ( !VidSourceUp )
         if ( IsABt848A() )
            SizeOption = GetSizeOption( GetSignalFormat(), RestoreCameraIndex(),
                                        GetVideoFormat() );
         else
            SizeOption = IsNTSCCompatible( GetVideoFormat() ) ? 0 : 1;

      // raw mode has very different sizes
      if ( Col == CF_RAW )
         SizeOption = 2;

      // find our index
      for ( i = 0; i < ARR_SIZE( RectArr [0] ); i++ )
         // have to do 'C' comparison as this should be 'C' compileable
         if ( memcmp( &r, RectArr [SizeOption] + i, sizeof( RECT ) ) == 0 )
            break;
      if ( i >= ARR_SIZE( RectArr [0] ) )
         i = 1;

      ConfigLoadComboBox( hDlg, ID_LBSIZE, gwSizeOptions [SizeOption],
         N_SIZEOPTIONS - ( SizeOption == 2 ), i );
      ConfigLoadComboBox( hDlg, ID_LBIMAGEFORMAT, gwFormatOptions,
         N_FORMATOPTIONS, Col );

      if ( VidSourceUp ) {
         HWND hCtl = GetDlgItem( hDlg, IDCANCEL );
         EnableWindow( hCtl, FALSE );
      }
      break;
      // User will unload this module at exit time, so make
      // sure that the dialog is gone if the user just quits
      // Windows with the dialog box up.
   case WM_ENDSESSION:
      if ( wParam )
         EndDialog( hDlg, IDCANCEL );
      break;

   case WM_COMMAND:
      switch ( wParam ) {
      case IDOK:
         if ( !VidSourceUp )
            if ( IsABt848A() )
               // determine size option for various video format
               SizeOption = GetSizeOption( GetSignalFormat(), RestoreCameraIndex(),
                                           GetVideoFormat() );
            else
               SizeOption = IsNTSCCompatible( GetVideoFormat() ) ? 0 : 1;

         Col = (ColFmt)ConfigGetComboBoxValue( hDlg, ID_LBIMAGEFORMAT, gwFormatOptions );
         if ( Col == CF_RAW )
            SizeOption = 2;

         SetImage( &RectArr [SizeOption] [ConfigGetComboBoxValue( hDlg, ID_LBSIZE, gwSizeOptions [SizeOption] )],
            Col );
         EndDialog( hDlg, IDOK );
         break;
      case IDCANCEL:
         EndDialog( hDlg, IDCANCEL );
         break;
      case ID_LBIMAGEFORMAT:
         if ( HIWORD( lParam ) != CBN_SELCHANGE )
            break;
         CurSelSize = ConfigGetComboBoxValue( hDlg, ID_LBSIZE, gwSizeOptions [SizeOption] );
         if ( CF_RAW == (ColFmt)ConfigGetComboBoxValue( hDlg, ID_LBIMAGEFORMAT, gwFormatOptions ) )
            ConfigLoadComboBox( hDlg, ID_LBSIZE, gwSizeOptions [2],
               N_SIZEOPTIONS - 1, 0 );
         else {
            if ( IsABt848A() )
               // determine size option for various video format
               SizeOption = GetSizeOption( GetSignalFormat(), RestoreCameraIndex(),
                                           GetVideoFormat() );
            else
               SizeOption = IsNTSCCompatible( GetVideoFormat() ) ? 0 : 1;
            ConfigLoadComboBox( hDlg, ID_LBSIZE, gwSizeOptions [SizeOption],
               N_SIZEOPTIONS, CurSelSize );
         }
         break;
      case ID_PBSIZEFULL:
         UpdateSizeDisplay( hDlg, 2 );
         break;
      case ID_PBSIZEHALF:
         UpdateSizeDisplay( hDlg, 1 );
         break;
      case ID_PBSIZEQUARTER:
         UpdateSizeDisplay( hDlg, 0 );
         break;
      case ID_PBSIZEEIGHTH:
         UpdateSizeDisplay( hDlg, ( ARR_SIZE( RectArr ) - 1 ) / 8 );
         break;
      default:
         break;
         }
      break;

   default:
      return FALSE;
   }
   return TRUE;
}

WORD WINAPI VidFormatDialog( HWND hParent )
{
   if ( DialogBox( GetOurInstance(), MAKEINTRESOURCE( DLG_VIDEOFORMAT ),
      hParent, (DLGPROC)VideoFormatDlgProc ) != -1 )
      return 0;
   return 1;
}


#define WM_UPDATEDIALOG  (WM_USER+200)

int ProcessScroll( HWND hDlg, HWND hCtl, WORD wParam, DWORD lParam,
   int iVal, WORD wIDEditBox )
{
   int iPageSize;
   int iMin, iMax;

   GetScrollRange( hCtl, SB_CTL, &iMin, &iMax );

   iPageSize = 1 + ( ( iMax - iMin ) / 10 );

   switch ( wParam ) {
   case SB_LINEDOWN:
      iVal++;
      break;
   case SB_LINEUP:
      iVal--;
      break;
   case SB_PAGEDOWN:
      iVal += iPageSize;
      break;
   case SB_PAGEUP:
      iVal -= iPageSize;
      break;
   case SB_THUMBTRACK:
      iVal = LOWORD ( lParam );
      break;
   default:
      break;
   }
   iVal = LimitRange( iVal, iMin, iMax );
   SetScrollPos( hCtl, SB_CTL, iVal, TRUE );
   SetDlgItemInt( hDlg, wIDEditBox, iVal, TRUE );

   return iVal;
}

DWORD FmtDlgHelpIDs [] = {
   ID_LBIMAGEFORMAT,   IDH_IMAGE_FORMAT,
   ID_PBSIZEFULL,      IDH_SIZE,
   ID_PBSIZEHALF,      IDH_SIZE,
   ID_PBSIZEQUARTER,   IDH_SIZE,
   ID_PBSIZEEIGHTH,    IDH_SIZE,
   ID_LBSIZE,          IDH_SIZE,
};

DWORD SrcDlgHelpIDs [] = {
   ID_RBSOURCE0,  IDH_VIDEO_CONNECT,
   ID_RBSOURCE1,  IDH_VIDEO_CONNECT,
   ID_RBSOURCE2,  IDH_VIDEO_CONNECT,
   ID_RBSOURCE3,  IDH_VIDEO_CONNECT,
   ID_RBNTSC,	   IDH_VIDEO_STANDARD,
   ID_RBNTSC_J,   IDH_VIDEO_STANDARD,
   ID_RBPAL,      IDH_VIDEO_STANDARD,
   ID_RBPAL_M,    IDH_VIDEO_STANDARD,
   ID_RBPAL_N,    IDH_VIDEO_STANDARD,
   ID_RBSECAM,    IDH_VIDEO_STANDARD
//   ID_CBSVHS,     IDH_VIDEO_STANDARD

};

/* Function:  GetAudioSource ( void )
 * Purpose: Get audio source
 * Input:   None
 * Output:  None
 * Return:  audio source - AUDIO_SOURCE_CDROM, AUDIO_SOURCE_EXTAUDIO,
                         AUDIO_SOURCE_TVTUNER, AUDIO_SOURCE_MUTE
*/
DWORD GetAudioSource( void )
{
   // read GPIE from pin (not register)
   DlkSetGPIE( 0x00000000L );

   // ask GPIO what the setting is
   DWORD value;
   return ( DlkGetGPDATABits( 0, 1, &value, 0 ) == Success ? value : -1 );
}


/* Method:  ULONG SetAudioSource ( ULONG audio )
 * Purpose: Set audio source
 * Input:   ULONG audio - AUDIO_SOURCE_CDROM, AUDIO_SOURCE_EXTAUDIO,
 *    AUDIO_SOURCE_TVTUNER, AUDIO_SOURCE_MUTE
 * Output:  None
 * Return:  TV control error value
*/
ErrorCode SetAudioSource( DWORD audio )
{
   if ( audio > AUDIO_SOURCE_MUTE )
      return Fail;

   // enables/disable GPOE drivers for [3:0]
   // analog uses GPIO[3:0] for audio; but they are used in digital so don't touch them
   if ( IsABt848A() )
      if ( GetConnector() == ConCamera )
      {  // digital signal; GPDATA[5:0] remapped
         DlkSetGPOE( 0x00000000L );
      }
      else
      {  // analog signal; GPDATA[5:0] not remapped
         DlkSetGPOE( 0x0000000FL );
         DlkSetGPDATABits( 0, 1, audio, 0 );
      }
   else
   {
      DlkSetGPOE( 0x0000000FL );
      DlkSetGPDATABits( 0, 1, audio, 0 );
   }
   return Success;
/*
   // if already mute, don't change audio source, just remember it
   if ( GetAudioSource() == AUDIO_SOURCE_MUTE ) {
      return Success;
   } else {  // not in mute, change audio source
      if ( DlkSetGPDATABits( 0, 1, audio, 0 ) == Success ) {
         return Success;
      } else
         return Fail;
   }
*/
}

/* Function: SetAudioMute
 * Purpose: Mute or UnMute audio
 * Input:   ULONG mute - 1 to mute; 0 to unmute
 * Output:  None
 * Return:  TV control error value
*/
ErrorCode SetAudioMute( DWORD mute )
{
   if ( mute > 1 )
      return Fail;

   // enables/disable GPOE drivers for [3:0]
   // analog uses GPIO[3:0] for audio; but they are used in digital so don't touch them
   if ( IsABt848A() )
      if ( GetConnector() == ConCamera )
         DlkSetGPOE( 0x00000000L );
      else
         // analog signal; GPDATA[5:0] not remapped
         DlkSetGPOE( 0x0000000FL );
   else
      DlkSetGPOE( 0x0000000FL );

   if ( mute == 1 ) {
      if ( GetAudioSource() == AUDIO_SOURCE_MUTE )
         return Fail;
      else
         return DlkSetGPDATABits( 0, 1, AUDIO_SOURCE_MUTE, 0 );
   }
   return Success;
}


//
// Dialog proc for the video source dialog box (VIDEO_EXTERNALIN channel)
//
#pragma argsused
int WINAPI _loadds VideoSourceDlgProc( HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam )
{
   static int Brightness;
   static int Contrast;
   static int Saturation;
   static int Hue;
   int rbID;
   WORD wID;
   HWND hCtl;
   CAMERA cameraIndex;

   // QuartzSight controls
   static int origQSBrightness, nQSBrightness;
   static int origQSSaturation, nQSSaturation;
   static int origQSHue,        nQSHue;
   static int origQSGamma,      nQSGamma;
   static int origQSSharpness,  nQSSharpness;
   static int origQSLightFreq,  nQSLightFreq;
   static int origQSLightType,  nQSLightType;

   switch ( msg ) {
   case WM_INITDIALOG: {
         // start up audio not mute
         SetAudioMute( 0 );

         Brightness = GetBrightness();
         Contrast   = GetContrast();
         Saturation = GetSaturation();
         Hue        = GetHue();

         hCtl = GetDlgItem( hDlg, ID_SBBRIGHTNESS );
         SetScrollRange( hCtl, SB_CTL , 0, 255, FALSE );

         hCtl = GetDlgItem( hDlg, ID_SBCONTRAST );
         SetScrollRange( hCtl, SB_CTL, 0, 255, FALSE );

         hCtl = GetDlgItem( hDlg, ID_SBSATURATION );
         SetScrollRange( hCtl, SB_CTL, 0, 255, FALSE );

         hCtl = GetDlgItem( hDlg, ID_SBHUE );
         SetScrollRange( hCtl, SB_CTL, 0, 255, FALSE );

         ConfigLoadComboBox( hDlg, ID_CBSTANDARD, gwStandardOptions,
            N_STANDARDOPTIONS, 0 );
         ConfigLoadComboBox( hDlg, ID_CBCAMERA, gwCameraOptions,
            N_CAMERAOPTIONS, 0 );

         // enable "Setup..." dialog for QuartzSight camera
         EnableWindow( GetDlgItem( hDlg, ID_PBCAMSETUP ),
                       DlkIsBt878Family() &&
                          ( GetConnector() == ConCamera ) &&
                          ( RestoreCameraIndex() == Rockwell ) );

         // QuartzSight controls
         bQSDlgUp = FALSE;

         GetQuartzSightParam( VP_AEC_TC,        (BYTE *)&nQSBrightness );
	      GetQuartzSightParam( VP_SATURATION,    (BYTE *)&nQSSaturation );
         GetQuartzSightParam( VP_HUE,           (BYTE *)&nQSHue );
	      GetQuartzSightParam( VP_GAMCOR,        (BYTE *)&nQSGamma );
	      GetQuartzSightParam( VP_SHARPNESS,     (BYTE *)&nQSSharpness );
	      GetQuartzSightParam( VP_AEC_MAINS,     (BYTE *)&nQSLightFreq );
	      GetQuartzSightParam( VP_AEC_LIGHTING,  (BYTE *)&nQSLightType );

         nQSBrightness -= 30;       // dlg value is 30 less actual reg value
         origQSBrightness = nQSBrightness;
         origQSSaturation = nQSSaturation;
         origQSHue        = nQSHue;
         origQSGamma      = nQSGamma;
         origQSSharpness  = nQSSharpness;
         origQSLightFreq  = nQSLightFreq;
         origQSLightType  = nQSLightType;

         hCtl = GetDlgItem( hDlg, ID_SBQSBRIGHTNESS );
         SetScrollRange( hCtl, SB_CTL , VP_MIN_BRIGHTNESS, VP_MAX_BRIGHTNESS, FALSE );

         hCtl = GetDlgItem( hDlg, ID_SBQSSATURATION );
         SetScrollRange( hCtl, SB_CTL , VP_MIN_SATURATION, VP_MAX_SATURATION, FALSE );

         hCtl = GetDlgItem( hDlg, ID_SBQSHUE );
         SetScrollRange( hCtl, SB_CTL , VP_MIN_HUE, VP_MAX_HUE, FALSE );

         hCtl = GetDlgItem( hDlg, ID_SBQSGAMMA );
         SetScrollRange( hCtl, SB_CTL , VP_MIN_GAMMA, VP_MAX_GAMMA, FALSE );

         hCtl = GetDlgItem( hDlg, ID_SBQSSHARPNESS );
         SetScrollRange( hCtl, SB_CTL , VP_MIN_SHARPNESS, VP_MAX_SHARPNESS, FALSE );

         switch ( GetConnector() )
         {
            case ConTuner:
               SetAudioSource( AUDIO_SOURCE_TVTUNER );
               AnalogSignalSelected( hDlg, TRUE );
               rbID = ID_RBSOURCE0;
               break;
            case ConComposite:
               AnalogSignalSelected( hDlg, TRUE );
               rbID = ID_RBSOURCE1;
               break;
            case ConSVideo:
               AnalogSignalSelected( hDlg, TRUE );
               rbID = ID_RBSOURCE2;
               break;
            case ConCamera:
               AnalogSignalSelected( hDlg, FALSE );
               EnableHueControls( hDlg, FALSE );      // no hue control for digital camera
               rbID = ID_RBSOURCE3;
               break;
         }

         CheckRadioButton( hDlg, ID_RBSOURCE0, ID_RBSOURCE3, rbID );

         SendDlgItemMessage( hDlg, ID_CBSTANDARD, CB_SETCURSEL, GetVideoFormat()-1, 0L );
         SendDlgItemMessage( hDlg, ID_CBCAMERA, CB_SETCURSEL, RestoreCameraIndex(), 0L );

         // intentional fall-through *************************************

   case WM_UPDATEDIALOG:
         Brightness = LimitRange( Brightness, 0, 255 );
         SetDlgItemInt( hDlg, ID_EBBRIGHTNESS, Brightness, TRUE );
         hCtl = GetDlgItem( hDlg, ID_SBBRIGHTNESS );
         SetScrollPos( hCtl, SB_CTL, Brightness, TRUE );

         Contrast = LimitRange( Contrast, 0, 255 );
         SetDlgItemInt( hDlg, ID_EBCONTRAST, Contrast, FALSE );
         hCtl = GetDlgItem( hDlg, ID_SBCONTRAST );
         SetScrollPos( hCtl, SB_CTL, Contrast, TRUE );

         Saturation = LimitRange( Saturation, 0, 255 );
         SetDlgItemInt( hDlg, ID_EBSATURATION, Saturation, FALSE );
         hCtl = GetDlgItem( hDlg, ID_SBSATURATION );
         SetScrollPos( hCtl, SB_CTL, Saturation, TRUE );

         hCtl = GetDlgItem( hDlg, ID_SBHUE );
         if ( IsWindowEnabled( hCtl ) )    // HUE control may be disabled
         {
            Hue = LimitRange( Hue, 0, 255 );
            SetDlgItemInt( hDlg, ID_EBHUE, Hue, TRUE );
            SetScrollPos( hCtl, SB_CTL, Hue, TRUE );
            SetHue( Hue );
         }

         // after edit boxes were updated
         SetBrightness( Brightness );
         SetContrast( Contrast );
         SetSaturation( Saturation );

         // disable Bt848A related control if doesn't find the exact hardware
         if ( !IsABt848A() )
         {
            EnableWindow( GetDlgItem( hDlg, ID_RBSOURCE3 ), FALSE );
            EnableWindow( GetDlgItem( hDlg, ID_CBCAMERA ), FALSE );
            SendDlgItemMessage( hDlg, ID_CBSTANDARD, CB_DELETESTRING, 6, 0 );  // No PAL-Nc
         }

         // QuartzSight controls
         nQSBrightness = LimitRange( nQSBrightness, VP_MIN_BRIGHTNESS, VP_MAX_BRIGHTNESS );
         SetDlgItemInt( hDlg, ID_EBQSBRIGHTNESS, nQSBrightness, TRUE );
         hCtl = GetDlgItem( hDlg, ID_SBQSBRIGHTNESS );
         SetScrollPos( hCtl, SB_CTL, nQSBrightness, TRUE );

         nQSSaturation = LimitRange( nQSSaturation, VP_MIN_SATURATION, VP_MAX_SATURATION );
         SetDlgItemInt( hDlg, ID_EBQSSATURATION, nQSSaturation, TRUE );
         hCtl = GetDlgItem( hDlg, ID_SBQSSATURATION );
         SetScrollPos( hCtl, SB_CTL, nQSSaturation, TRUE );

         nQSHue = LimitRange( nQSHue, VP_MIN_HUE, VP_MAX_HUE );
         SetDlgItemInt( hDlg, ID_EBQSHUE, nQSHue, TRUE );
         hCtl = GetDlgItem( hDlg, ID_SBQSHUE );
         SetScrollPos( hCtl, SB_CTL, nQSHue, TRUE );

         nQSGamma = LimitRange( nQSGamma, VP_MIN_GAMMA, VP_MAX_GAMMA );
         SetDlgItemInt( hDlg, ID_EBQSGAMMA, nQSGamma, TRUE );
         hCtl = GetDlgItem( hDlg, ID_SBQSGAMMA );
         SetScrollPos( hCtl, SB_CTL, nQSGamma, TRUE );

         nQSSharpness = LimitRange( nQSSharpness, VP_MIN_SHARPNESS, VP_MAX_SHARPNESS );
         SetDlgItemInt( hDlg, ID_EBQSSHARPNESS, nQSSharpness, TRUE );
         hCtl = GetDlgItem( hDlg, ID_SBQSSHARPNESS );
         SetScrollPos( hCtl, SB_CTL, nQSSharpness, TRUE );

         // requires radio buttons IDs in consecutive orders
         CheckRadioButton( hDlg, ID_RBQS50HZ, ID_RBQS60HZ, ( nQSLightFreq + ID_RBQS50HZ ) );
         CheckRadioButton( hDlg, ID_RBQSNORMAL, ID_RBQSBACKLIGHT, ( nQSLightType + ID_RBQSNORMAL ) );
         break;
      }
      // User will unload this module at exit time, so make
      // sure that the dialog is gone if the user just quits
      // Windows with the dialog box up.
   case WM_ENDSESSION:
      if ( wParam )
         EndDialog( hDlg, IDCANCEL );
      break;
   case WM_COMMAND:
      switch ( wParam ) {
      case IDOK:
         if ( DlkIsBt878Family() )
            SetTotalLineCount( 0 );    // assume normal 525/625 lines unless certain digital camera which sets to different value later
         if ( IsABt848A() )
         {
            // need to "store" dialog selection since they may have changed
            cameraIndex = (CAMERA)ConfigGetComboBoxValue( hDlg, ID_CBCAMERA, gwCameraOptions );
            StoreCameraIndex( cameraIndex );

            // what is the selected video signal format? analog vs digital
            if ( IsDlgButtonChecked( hDlg, ID_RBSOURCE3 ) )
            {
               // digital camera source is selected
               SetDigitalCamera( cameraIndex );
            }
            else
            {
               // analog source is selected
               SetHReset( 64 );     // uses 64 pixels as HReset
               SetInterlaceMode( 1 );     // 1 == interlace
               SetQuartzSightMode( FALSE );
               SetSignalFormat( Signal_Analog );
               SetVideoFormat( (VideoFormat)ConfigGetComboBoxValue( hDlg, ID_CBSTANDARD, gwStandardOptions ) );
            }
         }
         else
            SetVideoFormat( (VideoFormat)ConfigGetComboBoxValue( hDlg, ID_CBSTANDARD, gwStandardOptions ) );

         EndDialog( hDlg, IDOK );
         break;
      case IDCANCEL:
         // Restore QuartzSight controls only if different from original values
         // to avoid waiting for AEC to complete which takes a short while.
         if ( nQSBrightness != origQSBrightness )
         {
            SetQuartzSightParam( VP_AEC_TI , origQSBrightness + 15 );
            SetQuartzSightParam( VP_AEC_TC , origQSBrightness + 30 );
            SetQuartzSightParam( VP_AEC_TH , origQSBrightness + 45 );
         }
         if ( nQSSaturation != origQSSaturation ) SetQuartzSightParam( VP_SATURATION,   origQSSaturation );
         if ( nQSHue        != origQSHue        ) SetQuartzSightParam( VP_HUE,          origQSHue        );
         if ( nQSGamma      != origQSGamma      ) SetQuartzSightParam( VP_GAMCOR,       origQSGamma      );
         if ( nQSSharpness  != origQSSharpness  ) SetQuartzSightParam( VP_SHARPNESS,    origQSSharpness  );
         if ( nQSLightFreq  != origQSLightFreq  ) SetQuartzSightParam( VP_AEC_MAINS,    origQSLightFreq  );
         if ( nQSLightType  != origQSLightType  ) SetQuartzSightParam( VP_AEC_LIGHTING, origQSLightType  );
         EndDialog( hDlg, IDCANCEL );
         break;
      case ID_RBSOURCE0:
         if ( IsABt848A() )
         {
            // if change from digital source, ask for video size
            if ( GetConnector() == ConCamera )
            {
               VideoFormat vf = (VideoFormat)ConfigGetComboBoxValue( hDlg, ID_CBSTANDARD, gwStandardOptions );
               SizeOption = GetSizeOption( Signal_Analog, 0, vf );
               VidFormatDialog( hDlg );
               EnableHueControls( hDlg, ( vf == VFormat_NTSC || vf == VFormat_NTSC_J ) );
            }
         }
         AnalogSignalSelected( hDlg, TRUE );
         SetConnector( ConTuner, TRUE );
         SetAudioSource( AUDIO_SOURCE_TVTUNER );
         break;
      case ID_RBSOURCE1:
         if ( IsABt848A() )
         {
            // if change from digital source, ask for video size
            if ( GetConnector() == ConCamera )
            {
               VideoFormat vf = (VideoFormat)ConfigGetComboBoxValue( hDlg, ID_CBSTANDARD, gwStandardOptions );
               SizeOption = GetSizeOption( Signal_Analog, 0, vf );
               VidFormatDialog( hDlg );
               EnableHueControls( hDlg, ( vf == VFormat_NTSC || vf == VFormat_NTSC_J ) );
            }
         }
         AnalogSignalSelected( hDlg, TRUE );
         SetConnector( ConComposite, TRUE );
         SetAudioSource( AUDIO_SOURCE_EXTAUDIO );
         break;
      case ID_RBSOURCE2:
         if ( IsABt848A() )
         {
            // if change from digital source, ask for video size
            if ( GetConnector() == ConCamera )
            {
               VideoFormat vf = (VideoFormat)ConfigGetComboBoxValue( hDlg, ID_CBSTANDARD, gwStandardOptions );
               SizeOption = GetSizeOption( Signal_Analog, 0, vf );
               VidFormatDialog( hDlg );
               EnableHueControls( hDlg, ( vf == VFormat_NTSC || vf == VFormat_NTSC_J ) );
            }
         }
         AnalogSignalSelected( hDlg, TRUE );
         SetConnector( ConSVideo, TRUE );
         SetAudioSource( AUDIO_SOURCE_EXTAUDIO );
         break;
      case ID_RBSOURCE3:
         if ( IsABt848A() && ( GetConnector() != ConCamera ) )
         {
            CAMERA cam = (CAMERA)ConfigGetComboBoxValue( hDlg, ID_CBCAMERA, gwCameraOptions );
            SendDlgItemMessage( hDlg, ID_CBSTANDARD, CB_SETCURSEL, dbCameraSpec[ cam ].vidFormat-1, 0L );
            SizeOption = GetSizeOption( Signal_CCIR656, cam, (VideoFormat)0 );
            VidFormatDialog( hDlg );

            SetConnector( ConCamera, TRUE );
            SetAudioSource( AUDIO_SOURCE_EXTAUDIO );
            AnalogSignalSelected( hDlg, FALSE );

            // no hue control for digital camera
            EnableHueControls( hDlg, FALSE );

            // enable "Setup..." dialog for QuartzSight camera
            EnableWindow( GetDlgItem( hDlg, ID_PBCAMSETUP ), ( cam == Rockwell ) );
         }
         break;

      case ID_CBSTANDARD:
      {
         if ( HIWORD( lParam ) != CBN_SELCHANGE )
            break;
         VideoFormat vf = (VideoFormat)ConfigGetComboBoxValue( hDlg, ID_CBSTANDARD, gwStandardOptions );
         if ( vf == VFormat_PAL_NC )
         {
            // make sure sizes are OK
            SizeOption = 3;
            EnableHueControls( hDlg, FALSE );
         }
         else if ( IsNTSCCompatible( vf ) )
         {
            SizeOption = 0;
            EnableHueControls( hDlg, ( vf == VFormat_NTSC || vf == VFormat_NTSC_J ) );
         }
         else  // PAL compatible
         {
            SizeOption = 1;
            EnableHueControls( hDlg, FALSE );
         }
         VidFormatDialog( hDlg );
         break;
      }

      case ID_CBCAMERA:
      {
         if ( HIWORD( lParam ) != CBN_SELCHANGE )
            break;
         CAMERA cam = (CAMERA)ConfigGetComboBoxValue( hDlg, ID_CBCAMERA, gwCameraOptions );
         SendDlgItemMessage( hDlg, ID_CBSTANDARD, CB_SETCURSEL, dbCameraSpec[ cam ].vidFormat-1, 0L );
         SizeOption = CameraSizeBase + cam;

         // enable "Setup..." dialog for QuartzSight camera; otherwise disable it
         EnableWindow( GetDlgItem( hDlg, ID_PBCAMSETUP ), ( cam == Rockwell ) );

         // Close QuartzSight setup dlg if not QuartzSight camera is selected
         if ( bQSDlgUp && ( cam != Rockwell ) )
         {
            bQSDlgUp = FALSE;
            EnableQuartzSightControls( hDlg, bQSDlgUp );
         }

         VidFormatDialog( hDlg );
         break;
      }

      case ID_PBDEFAULT:
         Brightness = 128;
         Contrast   = 128;
         Saturation = 128;
         Hue        = 128;

         // Set QuartzSight defaults only if it is selected
         if ( IsDlgButtonChecked( hDlg, ID_RBSOURCE3 ) &&
              ( (CAMERA)ConfigGetComboBoxValue( hDlg, ID_CBCAMERA, gwCameraOptions ) == Rockwell ) )
         {
   	      SetQuartzSightParam( VP_DEFAULTS , TRUE );

            // obtain user default values from win.ini file
            GetQSightManufacturerDefaults( &nQSBrightness, &nQSSaturation, &nQSHue, &nQSGamma,
                                           &nQSSharpness,  &nQSLightFreq,  &nQSLightType );
         }
         
         SendMessage( hDlg, WM_UPDATEDIALOG, 0, 0L );
         break;
         /*
      case ID_CBSVHS:
         if ( IsDlgButtonChecked( hDlg, ID_CBSVHS ) )
            SetSVideo( On );
         else
            SetSVideo( Off );
         break;
         */
/*
      case 0x0053: //WM_HELP
      case WM_CONTEXTMENU:
         WinHelp( ((LPHELPINFO) lParam)->hItemHandle, szHelpFileName,
            HELP_WM_HELP, (DWORD) (LPSTR) aConfigDlgHelpIDs );
         break;
*/
      case ID_HELP:
         {
            char HelpFile [100];
            if ( LoadString( GetOurInstance(), IDS_HELPFILE, HelpFile, sizeof( HelpFile ) ) )
//               WinHelp( hDlg, HelpFile, HELP_CONTEXT, (DWORD)SrcDlgHelpIDs );
               WinHelp( hDlg, HelpFile, HELP_CONTENTS,0 );
            break;
         }
      case ID_EBBRIGHTNESS:
      case ID_EBCONTRAST:
      case ID_EBSATURATION:
      case ID_EBHUE:
         if ( HIWORD( lParam ) != EN_KILLFOCUS )
         return FALSE;

         Brightness = GetDlgItemInt( hDlg, ID_EBBRIGHTNESS, NULL, TRUE );
         Contrast = GetDlgItemInt( hDlg, ID_EBCONTRAST, NULL, FALSE );
         Saturation = GetDlgItemInt( hDlg,ID_EBSATURATION, NULL, FALSE );
         Hue = GetDlgItemInt( hDlg, ID_EBHUE, NULL, TRUE );
         SendMessage( hDlg, WM_UPDATEDIALOG, 0, 0L );
         break;

      case ID_EBQSBRIGHTNESS:
         if ( HIWORD( lParam ) != EN_KILLFOCUS )
            return FALSE;
         nQSBrightness = GetDlgItemInt( hDlg, ID_EBQSBRIGHTNESS, NULL, FALSE );
         SendMessage( hDlg, WM_UPDATEDIALOG, 0, 0L );
			SetQuartzSightParam( VP_AEC_TI , nQSBrightness + 15 );
         SetQuartzSightParam( VP_AEC_TC , nQSBrightness + 30 );
         SetQuartzSightParam( VP_AEC_TH , nQSBrightness + 45 );
         break;

      case ID_EBQSSATURATION:
         if ( HIWORD( lParam ) != EN_KILLFOCUS )
            return FALSE;
         nQSSaturation = GetDlgItemInt( hDlg, ID_EBQSSATURATION, NULL, FALSE );
         SendMessage( hDlg, WM_UPDATEDIALOG, 0, 0L );
         SetQuartzSightParam( VP_SATURATION, nQSSaturation );
         break;

      case ID_EBQSHUE:
         if ( HIWORD( lParam ) != EN_KILLFOCUS )
            return FALSE;
         nQSHue        = GetDlgItemInt( hDlg, ID_EBQSHUE, NULL, FALSE );
         SendMessage( hDlg, WM_UPDATEDIALOG, 0, 0L );
         SetQuartzSightParam( VP_HUE, nQSHue );
         break;

      case ID_EBQSGAMMA:
         if ( HIWORD( lParam ) != EN_KILLFOCUS )
            return FALSE;
         nQSGamma      = GetDlgItemInt( hDlg, ID_EBQSGAMMA, NULL, FALSE );
         SendMessage( hDlg, WM_UPDATEDIALOG, 0, 0L );
         SetQuartzSightParam( VP_GAMCOR, nQSGamma );
         break;

      case ID_EBQSSHARPNESS:
         if ( HIWORD( lParam ) != EN_KILLFOCUS )
            return FALSE;
         nQSSharpness  = GetDlgItemInt( hDlg, ID_EBQSSHARPNESS, NULL, FALSE );
         SendMessage( hDlg, WM_UPDATEDIALOG, 0, 0L );
         SetQuartzSightParam( VP_SHARPNESS, nQSSharpness );
         break;

      case ID_RBQS50HZ:
      case ID_RBQS60HZ:
         // requires radio buttons IDs in consecutive orders
         nQSLightFreq = wParam - ID_RBQS50HZ;
         SetQuartzSightParam( VP_AEC_MAINS, nQSLightFreq );
         break;

      case ID_RBQSNORMAL:
      case ID_RBQSBACKLIGHT:
         // requires radio buttons IDs in consecutive orders
         nQSLightType = wParam - ID_RBQSNORMAL;
         SetQuartzSightParam( VP_AEC_LIGHTING, nQSLightType );
         break;

      case ID_PBCAMSETUP:
         bQSDlgUp = !bQSDlgUp;
         EnableQuartzSightControls( hDlg, bQSDlgUp );
         break;
      }
      break;   // WM_COMMAND

   case WM_HSCROLL:

      hCtl = (HWND)HIWORD( lParam );
      wID = GetWindowWord( hCtl, GWW_ID );
      switch ( wID ) {
      case ID_SBBRIGHTNESS:
         Brightness = ProcessScroll( hDlg, hCtl,
            wParam, lParam, Brightness, ID_EBBRIGHTNESS );
         SetBrightness( Brightness );
         break;
      case ID_SBCONTRAST:
         Contrast = ProcessScroll( hDlg, hCtl,
            wParam, lParam, Contrast, ID_EBCONTRAST );
         SetContrast( Contrast );
         break;
      case ID_SBSATURATION:
         Saturation = ProcessScroll( hDlg, hCtl, wParam,
            lParam, Saturation, ID_EBSATURATION );
         SetSaturation( Saturation );
         break;
      case ID_SBHUE:
         Hue = ProcessScroll( hDlg, hCtl, wParam,
            lParam, Hue, ID_EBHUE );
         SetHue( Hue );
         break;

      // QuartzSight controls
      case ID_SBQSBRIGHTNESS:
         nQSBrightness = ProcessScroll( hDlg, hCtl, wParam, lParam, nQSBrightness, ID_EBQSBRIGHTNESS );
			SetQuartzSightParam( VP_AEC_TI , nQSBrightness + 15 );
         SetQuartzSightParam( VP_AEC_TC , nQSBrightness + 30 );
         SetQuartzSightParam( VP_AEC_TH , nQSBrightness + 45 );
         break;
      case ID_SBQSSATURATION:
         nQSSaturation = ProcessScroll( hDlg, hCtl, wParam, lParam, nQSSaturation, ID_EBQSSATURATION );
         SetQuartzSightParam( VP_SATURATION, nQSSaturation );
         break;
      case ID_SBQSHUE:
         nQSHue = ProcessScroll( hDlg, hCtl, wParam, lParam, nQSHue, ID_EBQSHUE );
         SetQuartzSightParam( VP_HUE, nQSHue );
         break;
      case ID_SBQSGAMMA:
         nQSGamma = ProcessScroll( hDlg, hCtl, wParam, lParam, nQSGamma, ID_EBQSGAMMA );
         SetQuartzSightParam( VP_GAMCOR, nQSGamma );
         break;
      case ID_SBQSSHARPNESS:
         nQSSharpness = ProcessScroll( hDlg, hCtl, wParam, lParam, nQSSharpness, ID_EBQSSHARPNESS );
         SetQuartzSightParam( VP_SHARPNESS, nQSSharpness );
         break;
      }
      break;
   default:
      return FALSE;
   }

   return TRUE;
}

WORD WINAPI VideoSourceDialog( HWND hParent )
{
   Connector CurCon;
   VideoFormat fmt;
   DWORD        audio;
   SignalFormat signal;
   SyncVideoRef syncRef;
   State        syncOutput;
   DecInputClk  iClk;
   int          tgbSize;
   BYTE         tgb[ 100 ];
   int          HReset;
   PLLFreq      pllFreq;
   VSIZEPARMS   vSize;
   int          lineCount;
   int          interlace;
   BOOL         QuartzSight;
   int Brightness;
   int Contrast;
   int Saturation;
   int Hue;

   CurCon = GetConnector();
   fmt = GetVideoFormat();
   if ( IsABt848A() || DlkIsBt878Family() )
   {
      audio = GetAudioSource();
      signal = GetSignalFormat();
      syncRef = GetSyncVideoRef();
      syncOutput = GetEnableSyncOutput();
      iClk = GetDecInputClock();
      GetTimGenByte( tgb, &tgbSize );
      HReset = GetHReset();
      pllFreq = GetPLL();
      GetVideoInfo( &vSize );
      if ( DlkIsBt878Family() )
      {
         lineCount = GetTotalLineCount();
         QuartzSight= GetQuartzSightMode();
      }
   }

   interlace  = GetInterlaceMode();
   Brightness = GetBrightness();
   Contrast   = GetContrast();
   Saturation = GetSaturation();
   Hue        = GetHue();

   VidSourceUp = TRUE;

   switch ( DialogBox( GetOurInstance(), MAKEINTRESOURCE( DLG_VIDEOSOURCE ),
      hParent, (DLGPROC) VideoSourceDlgProc ) ) {
   case IDOK:
      break;
   case IDCANCEL:
      // restore the values
      SetConnector( CurCon, TRUE );
      if ( DlkIsBt878Family() )
         SetTotalLineCount( 0 );    // assume normal 525/625 lines unless certain digital camera which sets to different value later
      if ( IsABt848A() )
      {
         SetAudioSource( audio );
         if ( signal == Signal_Analog )
            SetVideoFormat( fmt );
         else
         {
            SetSignalFormat( signal );
            SetVideoFormat( fmt );
            SetSyncVideoRef( syncRef );
            SetEnableSyncOutput( syncOutput );
            SetTimGenByte( tgb, tgbSize );
            SetHReset( HReset );
            SetPLL( pllFreq );
            SetDecInputClock( iClk );
            SetVideoInfo( vSize );
            if ( DlkIsBt878Family() )
            {
               SetTotalLineCount( lineCount );
               SetQuartzSightMode( QuartzSight );
            }
         }
      }
      else
         SetVideoFormat( fmt );
      SetInterlaceMode( interlace );
      SetBrightness( Brightness );
      SetContrast( Contrast );
      SetSaturation( Saturation );
      SetHue( Hue );
      break;
   case -1:
      VidSourceUp = FALSE;
      return 1;
   }
   VidSourceUp = FALSE;
   return 0;
}

/* CALLBACK:
 * Purpose:
 */
int WINAPI _loadds VideoDisplayProc( HWND hDlg, UINT msg, WPARAM wParam, LPARAM )
{
   static int DoCC;
   switch ( msg ) {
   case WM_INITDIALOG: {
      int RB;
      char str [80];
      DoCC = GetCCStatus();
      if ( DoCC & 0x8000 ) {
         CheckDlgButton( hDlg, IDC_CBCC, 1 );
         RB = DoCC &  1 ? IDC_RBCC1 : IDC_RBCC2;
      } else {
         CheckDlgButton( hDlg, IDC_CBCC, 0 );
         RB = IDC_RBCC1;
         EnableWindow( GetDlgItem( hDlg, IDC_RBCC1 ), false );
         EnableWindow( GetDlgItem( hDlg, IDC_RBCC2 ), false );
         EnableWindow( GetDlgItem( hDlg, IDC_CCTOKEN ), false );
         EnableWindow( GetDlgItem( hDlg, IDC_FILENAME ), false );
      }
      CheckRadioButton( hDlg, IDC_RBCC1, IDC_RBCC2, RB );
      GetCCFileName( str, sizeof( str ) - 1 );
      SetDlgItemText( hDlg, IDC_FILENAME, str );
      GetCCAlert( str, sizeof( str ) - 1 );
      SetDlgItemText( hDlg, IDC_CCTOKEN, str );

      break;
      }
   case WM_COMMAND:
      switch ( wParam ) {
      case IDOK:
         if ( IsDlgButtonChecked( hDlg, IDC_CBCC ) ) {
            DoCC = 0x8000;
            if ( IsDlgButtonChecked( hDlg, IDC_RBCC1 ) )
               DoCC |= 0x0001;
            else
               DoCC |= 0x0002;
            char str [80];
            GetDlgItemText( hDlg, IDC_CCTOKEN, str, sizeof( str ) - 1 );
            SetCCAlert( str );
            GetDlgItemText( hDlg, IDC_FILENAME, str, sizeof( str ) - 1 );
            SetCCFileName( str );
         } else
            DoCC = 0;
         // no break
      case IDCANCEL:
         SetCCStatus( DoCC );
         EndDialog( hDlg, IDCANCEL );
         break;
      case IDC_CBCC:
         if ( !IsDlgButtonChecked( hDlg, IDC_CBCC ) ) {
            EnableWindow( GetDlgItem( hDlg, IDC_RBCC1 ), false );
            EnableWindow( GetDlgItem( hDlg, IDC_RBCC2 ), false );
            EnableWindow( GetDlgItem( hDlg, IDC_CCTOKEN ), false );
            EnableWindow( GetDlgItem( hDlg, IDC_FILENAME ), false );
         } else {
            EnableWindow( GetDlgItem( hDlg, IDC_RBCC1 ), true );
            EnableWindow( GetDlgItem( hDlg, IDC_RBCC2 ), true );
            EnableWindow( GetDlgItem( hDlg, IDC_CCTOKEN ), true );
            EnableWindow( GetDlgItem( hDlg, IDC_FILENAME ), true );
         }
         break;
      default:
         return FALSE;
      }
      break;
    default:
       return FALSE;
   }
   return TRUE;
}

/* Function: VideoDisplay
 * Purpose: Starts Video Display dialog
 */
WORD VideoDisplay( HWND hParent )
{
   return DialogBox( GetOurInstance(), MAKEINTRESOURCE( EXTVIDOUT_DLG ),
      hParent, (DLGPROC) VideoDisplayProc );
}

/* Function: VideoDialog
 * Purpose: Dialog dispatcher
 * Input: hParent: HWND, parent window
 *   dlg: int, dialog number
 */
#ifdef __cplusplus
extern "C"
#endif
WORD WINAPI _export VideoDialog( HWND hParent, int dlg )
{
   switch ( dlg ) {
   case VIDEO_SOURCE:
      return  VideoSourceDialog( hParent );
   case VIDEO_FORMAT:
      return VidFormatDialog( hParent );
   case VIDEO_DISPLAY:
      return VideoDisplay( hParent );
   default:
      return 0;
   }
}





