/****************************************************************************/
/***** Copyright (c) Brooktree Corporation, 1996. All Rights Reserved *******/
/****************************************************************************/
/*                                                                          */
/*                      BROOKTREE CONFIDENTIAL                              */
/*                                                                          */
/* Filename:    CCDEC.CPP                                                   */
/*                                                                          */
/* Description: Closed caption decode in software source file.              */
/*                                                                          */
/*              This module has the real decoding stuff in it.              */
/*                                                                          */
/* Author:      Dave Tharp, based loosely on previous work by               */
/*                Steve Mair, and Steve Glennon.                            */
/*                                                                          */
/****************************************************************************/
/* PVCS Muck:
 * $Header: G:/SwDev/W95/Video/pisces/dvlpmnt/CC/rcs/CCDEC.CPP 2.8 1998/04/02 21:35:42 dtharp Exp $
 * $Revision: 2.8 $
 * $Date: 1998/04/02 21:35:42 $
 * $Author: dtharp $
 * $Archive:  $
 * $Log: CCDEC.CPP $
 * Revision 2.8  1998/04/02 21:35:42  dtharp
 * Merge DT_VIDEOHDR branch. Implements a new buffer 
 * management scheme.
 * Revision 2.7.3.1  1998/03/27 15:27:41  dtharp
 * Branch for passing videohdrs around.
 * Revision 2.7  1997/08/01 08:32:45  dtharp
 * Fixed ambiguous bitwise operation ( '&' and '<<' ).
 * Revision 2.6  1997/06/27 17:03:19  dtharp
 * Moved global variables and functions into class in attempt to solve
 * re-entrancy problem.
 * Revision 2.5  1997/05/28 23:50:08  dtharp
 * Bug Id 558: Gateway report of poor decoder accuracy. Apparently,
 * the speed optimization is not a positive fix . Reverting to rev 2.3
 * Revision 2.3  1997/01/17 01:16:35  dtharp
 * Merged new decoder branch.
 * Revision 2.2.1.11  1997/01/17 01:06:59  dtharp
 * Cleaned up a comment.
 * Revision 2.2.1.10  1997/01/17 01:04:37  dtharp
 * Implemented fft discriminator.
 * Revision 2.2.1.9  1997/01/16 23:02:25  dtharp
 * Added validRunin() function (counts transitions). Also
 * Played with opening instrumentation file in constructor 
 * (experiments are #ifdefed)
 * Revision 2.2.1.8  1997/01/15 23:19:01  dtharp
 * Added discrmination code
 * Revision 2.2.1.7  1996/12/10 23:15:02  dtharp
 * Cleanup: Using new filter, removed dead code. Added TODO:
 *                comment.
 * Functional: Conditionally output Instrumentation (bytes not null)
 * Revision 2.2.1.6  1996/12/09 23:39:08  dtharp
 * Changes to get rid of Page Fault & GPF
 * Revision 2.2.1.5  1996/12/07 00:57:23  dtharp
 * Cleaned up all unneeded variables.
 *
 */

#include <windows.h>
#include <mem.h>
#include <math.h>
#include "mytypes.h"
#include "cc_api.h"
#include "ccdec.h"


#define DO_OPEN_IN_DUMPER


#ifdef INSTRUMENTED
  #pragma message("Closed Caption Instrumented Build!") 
  FILE*   m_fpDbgTxt = NULL;
  FILE*   m_fpDbgBin = NULL;
  BOOL    bFirst=TRUE;
#endif


unsigned char Br_table[] = {
  0x00, 0x80, 0x40, 0xc0, 0x20, 0xa0, 0x60, 0xe0, 0x10, 0x90, 0x50, 0xd0,
  0x30, 0xb0, 0x70, 0xf0, 0x08, 0x88, 0x48, 0xc8, 0x28, 0xa8, 0x68, 0xe8,
  0x18, 0x98, 0x58, 0xd8, 0x38, 0xb8, 0x78, 0xf8, 0x04, 0x84, 0x44, 0xc4,
  0x24, 0xa4, 0x64, 0xe4, 0x14, 0x94, 0x54, 0xd4, 0x34, 0xb4, 0x74, 0xf4,
  0x0c, 0x8c, 0x4c, 0xcc, 0x2c, 0xac, 0x6c, 0xec, 0x1c, 0x9c, 0x5c, 0xdc,
  0x3c, 0xbc, 0x7c, 0xfc, 0x02, 0x82, 0x42, 0xc2, 0x22, 0xa2, 0x62, 0xe2,
  0x12, 0x92, 0x52, 0xd2, 0x32, 0xb2, 0x72, 0xf2, 0x0a, 0x8a, 0x4a, 0xca,
  0x2a, 0xaa, 0x6a, 0xea, 0x1a, 0x9a, 0x5a, 0xda, 0x3a, 0xba, 0x7a, 0xfa,
  0x06, 0x86, 0x46, 0xc6, 0x26, 0xa6, 0x66, 0xe6, 0x16, 0x96, 0x56, 0xd6,
  0x36, 0xb6, 0x76, 0xf6, 0x0e, 0x8e, 0x4e, 0xce, 0x2e, 0xae, 0x6e, 0xee,
  0x1e, 0x9e, 0x5e, 0xde, 0x3e, 0xbe, 0x7e, 0xfe, 0x01, 0x81, 0x41, 0xc1,
  0x21, 0xa1, 0x61, 0xe1, 0x11, 0x91, 0x51, 0xd1, 0x31, 0xb1, 0x71, 0xf1,
  0x09, 0x89, 0x49, 0xc9, 0x29, 0xa9, 0x69, 0xe9, 0x19, 0x99, 0x59, 0xd9,
  0x39, 0xb9, 0x79, 0xf9, 0x05, 0x85, 0x45, 0xc5, 0x25, 0xa5, 0x65, 0xe5,
  0x15, 0x95, 0x55, 0xd5, 0x35, 0xb5, 0x75, 0xf5, 0x0d, 0x8d, 0x4d, 0xcd,
  0x2d, 0xad, 0x6d, 0xed, 0x1d, 0x9d, 0x5d, 0xdd, 0x3d, 0xbd, 0x7d, 0xfd,
  0x03, 0x83, 0x43, 0xc3, 0x23, 0xa3, 0x63, 0xe3, 0x13, 0x93, 0x53, 0xd3,
  0x33, 0xb3, 0x73, 0xf3, 0x0b, 0x8b, 0x4b, 0xcb, 0x2b, 0xab, 0x6b, 0xeb,
  0x1b, 0x9b, 0x5b, 0xdb, 0x3b, 0xbb, 0x7b, 0xfb, 0x07, 0x87, 0x47, 0xc7,
  0x27, 0xa7, 0x67, 0xe7, 0x17, 0x97, 0x57, 0xd7, 0x37, 0xb7, 0x77, 0xf7,
  0x0f, 0x8f, 0x4f, 0xcf, 0x2f, 0xaf, 0x6f, 0xef, 0x1f, 0x9f, 0x5f, 0xdf,
  0x3f, 0xbf, 0x7f, 0xff
};

/*
 *  Sine/Cosine Table for size 256
 *
 *  Sine[n] = sin(x), x = x + (2Pi / 256)
 */

double  Sine[] = {
  0.000000, 0.024541, 0.049068, 0.073565, 0.098017, 0.122411,
  0.146730, 0.170962, 0.195090, 0.219101, 0.242980, 0.266713,
  0.290285, 0.313682, 0.336890, 0.359895, 0.382683, 0.405241,
  0.427555, 0.449611, 0.471397, 0.492898, 0.514103, 0.534998,
  0.555570, 0.575808, 0.595699, 0.615232, 0.634393, 0.653173,
  0.671559, 0.689541, 0.707107, 0.724247, 0.740951, 0.757209,
  0.773010, 0.788346, 0.803208, 0.817585, 0.831470, 0.844854,
  0.857729, 0.870087, 0.881921, 0.893224, 0.903989, 0.914210,
  0.923880, 0.932993, 0.941544, 0.949528, 0.956940, 0.963776,
  0.970031, 0.975702, 0.980785, 0.985278, 0.989177, 0.992480,
  0.995185, 0.997290, 0.998795, 0.999699, 1.000000, 0.999699,
  0.998795, 0.997290, 0.995185, 0.992480, 0.989177, 0.985278,
  0.980785, 0.975702, 0.970031, 0.963776, 0.956940, 0.949528,
  0.941544, 0.932993, 0.923880, 0.914210, 0.903989, 0.893224,
  0.881921, 0.870087, 0.857729, 0.844854, 0.831470, 0.817585,
  0.803208, 0.788346, 0.773010, 0.757209, 0.740951, 0.724247,
  0.707107, 0.689541, 0.671559, 0.653173, 0.634393, 0.615232,
  0.595699, 0.575808, 0.555570, 0.534998, 0.514103, 0.492898,
  0.471397, 0.449611, 0.427555, 0.405241, 0.382683, 0.359895,
  0.336890, 0.313682, 0.290285, 0.266713, 0.242980, 0.219101,
  0.195090, 0.170962, 0.146730, 0.122411, 0.098017, 0.073565,
  0.049068, 0.024541, 0.000000, -0.024541, -0.049068, -0.073565,
  -0.098017, -0.122411, -0.146730, -0.170962, -0.195090, -0.219101,
  -0.242980, -0.266713, -0.290285, -0.313682, -0.336890, -0.359895,
  -0.382683, -0.405241, -0.427555, -0.449611, -0.471397, -0.492898,
  -0.514103, -0.534998, -0.555570, -0.575808, -0.595699, -0.615232,
  -0.634393, -0.653173, -0.671559, -0.689541, -0.707107, -0.724247,
  -0.740951, -0.757209, -0.773010, -0.788346, -0.803208, -0.817585,
  -0.831470, -0.844854, -0.857729, -0.870087, -0.881921, -0.893224,
  -0.903989, -0.914210, -0.923880, -0.932993, -0.941544, -0.949528,
  -0.956940, -0.963776, -0.970031, -0.975702, -0.980785, -0.985278,
  -0.989177, -0.992480, -0.995185, -0.997290, -0.998795, -0.999699,
  -1.000000, -0.999699, -0.998795, -0.997290, -0.995185, -0.992480,
  -0.989177, -0.985278, -0.980785, -0.975702, -0.970031, -0.963776,
  -0.956940, -0.949528, -0.941544, -0.932993, -0.923880, -0.914210,
  -0.903989, -0.893224, -0.881921, -0.870087, -0.857729, -0.844854,
  -0.831470, -0.817585, -0.803208, -0.788346, -0.773010, -0.757209,
  -0.740951, -0.724247, -0.707107, -0.689541, -0.671559, -0.653173,
  -0.634393, -0.615232, -0.595699, -0.575808, -0.555570, -0.534998,
  -0.514103, -0.492898, -0.471397, -0.449611, -0.427555, -0.405241,
  -0.382683, -0.359895, -0.336890, -0.313682, -0.290285, -0.266713,
  -0.242980, -0.219101, -0.195090, -0.170962, -0.146730, -0.122411,
  -0.098017, -0.073565, -0.049068, -0.024541
};

void CCDecoder::scale()
{
  register int  loop;

  for (loop = 0; loop < FFT_SAMPLES; loop++)  {
    Real[loop] /= FFT_SAMPLES;
    Imag[loop] /= FFT_SAMPLES;
  }
}

#define  permute(x)  Br_table[(x)]
#define  sine(x)     Sine[(x)]
#define  cosine(x)   Sine[((x) + 64) % 256]

void CCDecoder::fft()
{
  register int  loop, loop1, loop2;
  unsigned  i1;     // going to right shift this 
  int   i2, i3, i4, y;
  double    a1, a2, b1, b2, z1, z2;

  i1 = FFT_SAMPLES >> 1;
  i2 = 1;

  // perform the butterflys
  for (loop = 0; loop < POWER; loop++)  {
    i3 = 0;
    i4 = i1;

    for (loop1 = 0; loop1 < i2; loop1++)  {
      y = permute(i3 / (int)i1);
      z1 =  cosine(y);
      z2 = -sine(y);

      for (loop2 = i3; loop2 < i4; loop2++)  {

        a1 = Real[loop2];
        a2 = Imag[loop2];

        b1  = z1*Real[loop2+i1] - z2*Imag[loop2+i1];
        b2  = z2*Real[loop2+i1] + z1*Imag[loop2+i1];

        Real[loop2] = a1 + b1;
        Imag[loop2] = a2 + b2;

        Real[loop2+i1] = a1 - b1;
        Imag[loop2+i1] = a2 - b2;
      }

      i3 += (i1 << 1);
      i4 += (i1 << 1);
    }

    i1 >>= 1;
    i2 <<= 1;
  }
}


BOOL CCDecoder::validRunin(BYTE * samples, int len, int midpoint)
{
   int ndx = 0 ;
   int i ;

   // find first crossing
   for( ndx = 0 ; ndx < len ; ndx++)
   {
      if (samples[ndx] > midpoint)
         break ;
   }

   // load data array for fft (zero the imaginary)
   for (i = 0 ; i < 256 ; i++)
   {
      Real[i] = samples[ndx+i]  ;
      Imag[i] = 0.0 ;
   }
   scale() ;
   fft() ;

   // OK, if we have a good run-in sample, we will see at least
   // one peak other than the peak in bin 0 (DC). Only half the buffer
   // is valid, We will define a peak as anythig over 5% of the magnitude
   // of the bin 0 value.
   double target = .05*Real[0] ;
   int peak = 0 ;
   for (i=1 ; i < 128 ; i++)
   {
      if (Real[i] >= target)
      {
         peak = i ;
         break ;
      }
   }
   //char buf[80] ;
   //sprintf(buf,"Peak at %d",peak) ;
   //pMainFrame->statusDisp(buf) ;

   return peak ? TRUE : FALSE ;
}

/////////////////////////////////////////////////////////////////////////////////
CCDecoder::CCDecoder(int hlen/*=640*/,int /*=640*/):
  m_hlen( hlen )
{
  isInitialized = false ;
  m_step = 1; // step size for sample data

#ifdef INSTRUMENTED
#ifndef DO_OPEN_IN_DUMPER
  if (!m_fpDbgBin)
    if(NULL==(m_fpDbgBin=fopen("c:\\cc_data.bin","wb")))
    {
       printf("Open file error!");
    }
  if (!m_fpDbgTxt)
    if (NULL==(m_fpDbgTxt=fopen("c:\\cc_data.txt","w")))
    {
        printf("Open error!");
    }
#endif
#endif
}

CCDecoder::~CCDecoder(void)
{
  isInitialized = false;

#ifdef INSTRUMENTED
#ifndef DO_OPEN_IN_DUMPER
  if (!m_fpDbgBin)
  {
    fclose (m_fpDbgBin);
    m_fpDbgBin = NULL;
  }

  if (!m_fpDbgTxt)
  {
    fclose (m_fpDbgTxt);
    m_fpDbgTxt = NULL;
  }
#endif
#endif
}

/////////////////////////////////////////////////////////////////////////////////
int CCDecoder::Init( int samplesLen, int horizResolution,
                     DWORD decodeMode)
{
  m_hlen        = horizResolution;
  m_samplesLen  = samplesLen;
  m_lines       = m_samplesLen / m_hlen; // # of CC lines

  switch( decodeMode )
  {
    case BT819CC_IN_LUMA: 
      m_step = 1 ; 
      break ;

    case BT819CC_IN_422 : 
      m_step = 2 ; 
      m_lines /= 2;                      // in 422, there is one luma 
      break ;                            //  sample for every 2 bytes

    default:              
      return BT819CC_ERR_MODE;           // unknown mode
  }

  isInitialized = true;

  return BT819CC_ERR_NONE;
}

/////////////////////////////////////////////////////////////////////////////////
int CCDecoder::Decode (const BYTE far * samples, int /*lines*/,  // lines initialized in Init()
   const BYTE far * decodedData, int   far * decodedDataLen )
{
  if (!isInitialized)
    return BT819CC_ERR_INIT ;

#ifdef TESTING
  int m_hlen = GetDocument()->m_nBytes ; // simulated
  int m_lines = 1 ;                      // simulated
  BYTE *  m_pInputSamples ;
  int m_bitWidth ;
#endif

  int nCharsDecoded = 0 ;
  BOOL bParityError = FALSE ;
  int peakArray[7] ;
  int valleyArray[7] ;

#ifdef INSTRUMENTED
  m_sampleNdx = 0 ;
#endif

  // setup output pointer
  BYTE far* pDataOut = (BYTE far *) decodedData;

  for( int n = 0; n < m_lines; n++ ) 
  {
    int nPeaks   = 0 ;
    int nValleys = 0 ;
    int yLast    = 0 ;
    enum {VALLEY, PEAK } search = PEAK ;
    BOOL bFoundCrossing = FALSE ;
    int firstBit ;
    DWORD dwBits = 0 ;
    int bitNum = 0 ;
  
    // set pointer to correct location in sample buffer
    m_pInputSamples = (BYTE * ) (samples + (n*m_hlen)) ;

    // filter the data
  #ifdef TESTING
    if (GetDocument()->bFilter)
  #endif
      theFilter.Apply( m_pInputSamples, m_hlen) ;

    // find the min and max
    int yMin, yMax,i ;
    yMin = yMax = m_pInputSamples[0] ;
    for (i = 1 ; i < m_hlen ; i++) 
    {
      if (yMin > m_pInputSamples[i])
        yMin = m_pInputSamples[i] ;
      if (yMax < m_pInputSamples[i])
        yMax = m_pInputSamples[i] ;
    }

    int midPoint = (yMin + yMax) / 2 ;

    // We need to look at the waveform and determine if 
    // it is a waveform that we can decode. We need
    // to 'continue' if not a CC waveform. Note that nCharsDecoded
    // will be correct if we end up exiting this loop early and
    // the calling function will be notified that no chars
    // were decoded.
    if(!validRunin(m_pInputSamples,m_hlen, midPoint))
    {
      continue ;
    }

    //
    // Walk through the sample points
    //
    for (i = 0 ; i < m_hlen ; i ++)
    {
      // we start by looking for the run-in. There should be 7
      // peaks and 6 valleys.

      if (nPeaks < 7)
      {
        if (search == VALLEY)
        {
          if (m_pInputSamples[i] < midPoint ) 
          {
            if (m_pInputSamples[i] > yLast)
            {
              search = PEAK ;
              valleyArray[nValleys++] = i-1 ;
            }
          }
        }
        else // search == PEAK
        {
          if (m_pInputSamples[i] > midPoint ) 
          {
            if (m_pInputSamples[i] < yLast)
            {
              search = VALLEY;
              peakArray[nPeaks++] = i-1 ;
              if (nPeaks == 7)
              {
                //
                // The following code is here as a last chance
                // discrimination of the waveform. If a 'bad'
                // buffer got through the fft discriminator
                // above, we then check the peak and valley deltas 
                // of the runin. These deltas should be uniform
                // within a few counts.
                //
                #define DELTA_THRESH 10
                int delta[6] ;
                int avgDelta = 0 ;
                int x ;
                BOOL goodRuninPeaks = TRUE ;
                BOOL goodRuninValleys = TRUE ;

                // Checking the peaks...
                for (x = 0 ; x < 6 ; x++)
                {
                   delta[x] = peakArray[x+1] - peakArray[x] ;
                   avgDelta += delta[x] ;
                }
                avgDelta /= 6 ;
                for (x = 0 ; x < 6 ; x++)
                {
                   if (abs(delta[x] - avgDelta) > DELTA_THRESH)
                      goodRuninPeaks = FALSE ;
                }

                // Check the valleys...
                avgDelta = 0 ;
                for (x = 0 ; x < 5 ; x++)
                {
                   delta[x] = valleyArray[x+1] - valleyArray[x] ;
                   avgDelta += delta[x] ;
                }
                avgDelta /= 5 ;

                for (x = 0 ; x < 5 ; x++)
                {
                   if (abs(delta[x] - avgDelta) > DELTA_THRESH)
                      goodRuninValleys = FALSE ;
                }

                // If peaks or valleys failed discontinue decode of
                // this buffer (effect a continue to the outer loop)
                if (!goodRuninPeaks && !goodRuninValleys)
                {
                   break ;
                }

                // we're done characterizing the run-in,
                // now calculate the bitwidth
                m_bitWidth  = 0 ;
                m_bitWidth += peakArray[1] - peakArray[0] ;
                m_bitWidth += peakArray[2] - peakArray[1] ;
                m_bitWidth += peakArray[3] - peakArray[2] ;
                m_bitWidth += peakArray[4] - peakArray[3] ;
                m_bitWidth += peakArray[5] - peakArray[4] ;
                m_bitWidth += peakArray[6] - peakArray[5] ;

                m_bitWidth += valleyArray[1] - valleyArray[0] ;
                m_bitWidth += valleyArray[2] - valleyArray[1] ;
                m_bitWidth += valleyArray[3] - valleyArray[2] ;
                m_bitWidth += valleyArray[4] - valleyArray[3] ;
                m_bitWidth += valleyArray[5] - valleyArray[4] ;
                m_bitWidth /= 11 ;
              }
            }
          }
        }

        yLast = m_pInputSamples[i] ;
      }
      else   // done searching for peaks and valleys
      {
        // We now search for the midpoint crossing. When found, we add
        // 1/2 a bitwidth to center on the start bit.
        if (!bFoundCrossing) 
        {
          if (yLast < m_pInputSamples[i])
            if (m_pInputSamples[i] > midPoint)
            {
              bFoundCrossing = TRUE ;
              firstBit = i + (m_bitWidth/2) ;
#ifdef TESTING
              crossingPt = i ;
#endif
            }
          yLast = m_pInputSamples[i] ;
        }
        else  // gathering data bits
        {
          if (i >= firstBit)
          {
#ifdef INSTRUMENTED
            if (m_sampleNdx < NUM_POINTS -1)
               m_samplePoints[m_sampleNdx++] = i ;
#endif
            // this is a sample point
            if (m_pInputSamples[i] > midPoint)
              dwBits |= BIT(bitNum++) ;  // we have a 1
            else
              bitNum++ ;                 // we have a 0

            firstBit += m_bitWidth ;
          } // if i >= first bit
        } // if-else (bFoundCrossing)
      } // if-else (searching for peaks and valleys)
    } // for all sample points

    // Done processing sample points. dwBits now holds the bit representation 
    // of the data for this line.
    // bits are arranged as follows: (s= start bit, p = parity bit)
    //
    // 16                              0
    //        b2            b1
    // +-------------+ +-------------+
    // p 6 5 4 3 2 1 0 p 6 5 4 3 2 1 0 s

    BYTE b1 = (char) (dwBits >> 1) & 0x7f ;
    BYTE b2 = (char) (dwBits >> 9) & 0x7f ;

    // calculate parity. If parity error (parity not odd) set MSB
    BYTE p1 = 0, p2 = 0;
    for (int j = 0 ; j < 8 ; j++)
    {
      p1+= (b1 & (1 << j)) ? 1 : 0 ;      // add up the set bits
      p2+= (b2 & (1 << j)) ? 1 : 0 ; 
    }

#ifdef INSTRUMENTED
    CC_INST CCI;
    CCI.b1=&b1;
    CCI.b2=&b2;
    // Assume true otherwise
    CCI.bParity1=TRUE;
    CCI.bParity2=TRUE;
    CCI.nHorizLength  =&m_hlen;   
    CCI.dwBitWidth    =&m_bitWidth; 
    CCI.npSamplePoints=m_samplePoints;   // need to deal with this
#endif

    if ((p1 & 1) && dwBits & BIT(8))      // if sum of bits odd and parity bit set then error
    {
      b1 |= 0x80 ;
      bParityError = TRUE ;
#ifdef INSTRUMENTED
      CCI.bParity1=FALSE;
#endif
    }
    if ((p2 & 1) && dwBits & BIT(16))     // if sum of bits odd and parity bit set then error
    {
      b2 |= 0x80 ;
      bParityError = TRUE ;
#ifdef INSTRUMENTED
      CCI.bParity2=FALSE;
#endif
    }

    *pDataOut++ = b1 ;
    *pDataOut++ = b2 ;
    nCharsDecoded += 2;

#ifdef INSTRUMENTED
    if (b1 || b2)
    {
      OutputInstrumentation(&CCI);
      DumpWaveForm(m_pInputSamples,m_hlen);
    }
#endif
  }// for each line

  *decodedDataLen = nCharsDecoded;

  return bParityError ? BT819CC_ERR_PARITY :  
                        nCharsDecoded ? BT819CC_ERR_NONE : BT819CC_ERR_NO_DATA;
}

#ifdef INSTRUMENTED
/////////////////////////////////////////////////////////////////////////////////
void CCDecoder::OutputInstrumentation(FPCC_INST fpCCI)
{
    char szBuf[256];
#ifdef DO_OPEN_IN_DUMPER
     if (NULL==(m_fpDbgTxt=fopen("c:\\cc_data.txt","a")))
     {
         printf("Open error!");
         return;
     }
#endif
    szBuf[0]='\0';
    wsprintf(szBuf,"(%c,%c) (%x,%x) (%s,%s)\n",
        isprint(*fpCCI->b1)?(*fpCCI->b1):' ',
        isprint(*fpCCI->b2)?(*fpCCI->b2):' ',
        *fpCCI->b1,
        *fpCCI->b2,
        fpCCI->bParity1?"GOOD":"BAD",
        fpCCI->bParity2?"GOOD":"BAD");
    fwrite(szBuf,strlen(szBuf),1,m_fpDbgTxt);

    if (bFirst)
    {
        szBuf[0]='\0';
        wsprintf(szBuf,"Horz Length : %d\n",*(fpCCI->nHorizLength));
        fwrite(szBuf,strlen(szBuf),1,m_fpDbgTxt);
        bFirst=FALSE;
    }

    szBuf[0]='\0';
    wsprintf(szBuf,"M Bitwidth: %ld\n", *(fpCCI->dwBitWidth)) ;
    fwrite(szBuf,strlen(szBuf),1,m_fpDbgTxt);

    szBuf[0]='\0';
    wsprintf(szBuf,"Sample Points: "); 
    fwrite(szBuf,strlen(szBuf),1,m_fpDbgTxt);
    for (int i=0; i<NUM_POINTS; i++)
    {
        szBuf[0]='\0';
        if (i==NUM_POINTS-1)
            wsprintf(szBuf,"%d \n",fpCCI->npSamplePoints[i]); 
        else
            wsprintf(szBuf,"%d ",fpCCI->npSamplePoints[i]); 
        fwrite(szBuf,strlen(szBuf),1,m_fpDbgTxt);
    }

    szBuf[0]='\0';
    wsprintf(szBuf,"----------------------------------------------------\n");
    fwrite(szBuf,strlen(szBuf),1,m_fpDbgTxt);
#ifdef DO_OPEN_IN_DUMPER
    fclose(m_fpDbgTxt);
#else
    fflush(m_fpDbgTxt);
#endif
    return;
}

/////////////////////////////////////////////////////////////////////////////////
void CCDecoder::DumpWaveForm(BYTE far * pDbg, WORD hlen)
{
#ifdef DO_OPEN_IN_DUMPER
    if(NULL==(m_fpDbgBin=fopen("c:\\cc_data.bin","ab")))
    {
        printf("Open file error!");
        return;
    }
#endif

    #define START_OFS 0
//    BYTE far *pDbg = m_pInputSamples + START_OFS;
//    WORD nBytes = (m_hlen-4) - START_OFS ;
    WORD nBytes = (hlen-4) - START_OFS ;

    fwrite(&nBytes,sizeof(WORD),1,m_fpDbgBin) ;
    fwrite(pDbg,sizeof(BYTE),nBytes,m_fpDbgBin) ;
#ifdef DO_OPEN_IN_DUMPER
    fclose (m_fpDbgBin);
#else
    fflush (m_fpDbgBin);
#endif
}

#endif

