/* File: CCDispl.h
 * Purpose: Contains definition of the CC display class
 * Author: sasha
 * $Log: CCDispl.h $
 * Revision 2.9  1997/06/02 21:50:45  dtharp
 * Bug id 628: flashing pink box every 5 sec in preview. CC display
 * watchdog timer is now shut down when CC is paused.
 * Revision 2.8  1997/05/28 17:14:20  trantq
 * Merge in Fix for Bug 548.
 * Revision 2.7  1997/05/22 18:38:27  dtharp
 * Bug 560: Implemented a time-out period for cc updates. If the
 * time-out is exceeded, the display is erased. Current timeout is
 * 5 seconds.
 * Revision 2.6.1.2  1997/05/15 01:15:50  trantq
 * - Fix Bug ID 548.  Alert token and filename getting trashed.
 * Revision 2.6  1997/05/02 17:36:00  dtharp
 * Bug Id 531: Fix for CC displaying in the wrong location when window
 * is moved.
 * Revision 2.5  1997/04/21 23:52:07  dtharp
 * Bug Id 508 (Toshiba GPF). Added protection against calling 
 * exported member functions before Init()is called.
 * Revision 2.4  1997/04/18 00:32:22  dtharp
 * Fix for Bug Id 433. Closed Caption Performance (speed, cpu).
 * Revision 2.3.1.14  1997/03/18 17:25:19  tomz
 * added  flag to control primary surface CC display
 * Revision 2.3.1.13  1997/03/04 14:56:15  tomz
 * Attempt to fix italics
 * Revision 2.3.1.12  1997/02/28 14:59:40  tomz
 * Added clipping for non-overlay
 * Revision 2.3.1.11  1997/02/19 17:25:56  tomz
 * safety
 * Revision 2.3.1.10  1997/02/17 19:12:18  tomz
 * Calculate clipping
 * Revision 2.3.1.9  1997/02/14 22:44:52  tomz
 * Stripped out old code
 * Revision 2.3.1.8  1997/02/14 22:05:35  tomz
 * changed a prototype
 * Revision 2.3.1.7  1997/02/13 23:40:16  tomz
 * safety
 * Revision 2.3.1.6  1997/02/13 14:11:21  tomz
 * Looks better, but no pop-on yet
 * Revision 2.3.1.5  1997/02/11 18:38:51  tomz
 * Scrolling works
 * Revision 2.3.1.4  1997/02/11 16:24:59  tomz
 * safety backup
 * Revision 2.3.1.3  1997/02/06 23:45:07  tomz
 * Remove complete repaints.
 * Revision 2.3.1.2  1997/01/31 17:57:15  tomz
 * Revision 2.3.1.1  1997/01/30 15:59:25  tomz
 * New CC (overlay and primary surface)
 * Revision 2.3  1997/01/27 17:32:13  tomz
 * Merged brance 2.2.2
 * Removed overlay mode CC flicker.
 * Revision 2.2.2.5  1997/01/27 17:30:25  tomz
 * Merged brance 2.2.2
 * Fixed EraseDisplay to work with my new non-flickering code.
 * Revision 2.2.2.4  1997/01/23 00:09:22  tomz
 * Backed out previous version.  I screwed something up good.
 * Revision 2.2.2.2  1997/01/22 01:06:38  tomz
 * Revision 2.2.2.1  1997/01/21 18:27:42  tomz
 * Removed flicker
 * Revision 2.2  1996/10/10 20:18:56  dtharp
 * Removed CCMODE constants to cc_api.h.
 * Revision 2.1  1996/06/10 20:12:09  ssm
 * MarkBeta1AsRev2
 * Revision 1.8  1996/06/07 20:02:37  sasha
 * Added AlertMatch_ and removed DataChanged_
 * Revision 1.7  1996/06/05 19:16:24  sasha
 * Moved stdio.h back to .cpp
 * Revision 1.6  1996/06/05 19:12:31  sasha
 * Added functions for getting file name and alert token
 * Revision 1.5  1996/06/05 04:11:59  sasha
 * Added support for writing CC data to a file and displaying a message
 * box
 * Revision 1.4  1996/06/01 03:13:18  sasha
 * Added ShowBitmap() and HBITMAPs for trade mark and music note
 * Revision 1.3  1996/05/31 17:42:22  sasha
 * Very close. Any minute now
 * Revision 1.2  1996/05/30 23:11:53  sasha
 * Just next version.
 * Revision 1.1  1996/05/28 17:51:56  sasha
 * Initial revision
*/

#ifndef __CCDISPL_H
#define __CCDISPL_H

#include "classlib\arrays.h"
#ifndef __WINDOWS_H
#include <windows.h>
#endif
#ifndef __STDLIB_H
#include <stdlib.h>
#endif

const STR_LEN = 40;
const AttrNone = 0xF0;
const TextNone = '\0';


#define MAX_COL         ((unsigned)(32))
#define MAX_ROW         ((unsigned)(15))
#define MAX_CLIP_RECTS  ((MAX_ROW) * (((MAX_COL)/2)+1))

typedef struct {
   DWORD left ;
   DWORD top ;
   DWORD right ;
   DWORD bottom ;
} LRECT ;

typedef struct _RGNDATAHEADER {
    DWORD   dwSize;
    DWORD   iType;
    DWORD   nCount;
    DWORD   nRgnSize;
    LRECT   rcBound;
} RGNDATAHEADER, *PRGNDATAHEADER;

typedef struct _RGNDATA {
    RGNDATAHEADER   rdh;
    char            Buffer[1];
} RGNDATA, *PRGNDATA;

class CCDisplay;

class CC_Char
{
public:
   CCDisplay * CCD;
   char     ch;            // Character for this position
   char     attr;          // Attribute of this character
   bool     visible;
   unsigned column;        // Column position of this character on screen
   unsigned row;           // Row position of this character on screen
   HBITMAP  hbm;           // Handle to bitmap if required, NULL otherwise
   HFONT    hfont;
   COLORREF color;
   // HDC      hdc;           // DC to draw on
   bool     redraw;        // True if ch has changed since the last Draw()

   CC_Char();
   ~CC_Char();
   void Init(char param_ch, int param_row, int param_column, CCDisplay * param_ccd);
   void Draw();
   void Erase ();
   void Replace(char param_ch);
   void Copy(CC_Char* ccc);
};

/*
class CC_Row
{
   CC_Char  *ap_cc_char[MAX_COL];   // Array of pointers to CC_Chars
   bool     bEmpty;                 // TRUE if there are no characters in this row
   int      CurCol;
}
*/

/* Class: CCDisplay
 * Purpose:
 */

class CCDisplay
{
   friend class CC_Char;

   unsigned    row_;
   unsigned    column_;
   COLORREF    mru_color;
   HFONT       mru_hfont;
   bool        NeedToDisplay_;

   WORD        wCapStyle_;    // RCL (pop), RDC (paint), RU2-4 (roll)
   WORD        wLastControl_;
   WORD        wRowIndex_;
   bool        ReadyToDisplay_;
   bool        bGotFirstCR_ ;
   bool        AlertMatch_;
   bool        HaveAlert_;

   WORD        wCapType_;     // Current capture type CC1, CC2, Text1, Text2
   WORD        wStreamMode_;  // Type of capture stream
   WORD        wRUCount_;     // If too many RU are set without a CR
                              // then start treating the RU as a CR

   WORD        wWidth;
   WORD        wHeight;
   bool        bOverlayMode;
   bool        bAllowCC;
   HBITMAP     hbmMusicNote;
   HBITMAP     hbmTradeMark;
   WORD        wXOffset;
   WORD        wYOffset;
   WORD        wMusicWidth;
   WORD        wMusicHeight;
   HDC         hdcScreen;
   HBRUSH      hbrColorKeyBrush;
   HBRUSH      hbrOpaqueBrush;
   HWND        hWnd_;
   HFONT       hFont;
   HFONT       hFontItalic;
   HFONT       hFontUnder;
   HFONT       hFontItalUnder;

   UINT        tid_ ; // timer id

   FILE        *fp;
   char        alarm_ [50];
   char        alert_ [50];
   char        file_  [_MAX_PATH];

   CC_Char     CCScreenArray[MAX_ROW][MAX_COL];

   char        ClipRgnMem[sizeof(RGNDATAHEADER) + (MAX_CLIP_RECTS * sizeof(LRECT))];
   RGNDATA far *ClipRgn;
   LRECT   far *ClipRect;

//   BOOL        bFlashShow;
//   WORD        wFlashCount;


   void        AddChar( char ch );
   void        BackSpace();
   void        SetAttrib(char Attr);

   unsigned long ComputeClip( );
   void        SetClipping( );
   bool        ProcessChar( char ch );
   bool        ProcessPreamble( char NonDisp, char Disp );
   bool        ProcessMidrow( char NonDisp, char Disp );
   bool        ProcessMisc( char NonDisp, char Disp );
   void        ProcessTabOffset( int );
   bool        ProcessSChar( char NonDisp, char Disp );
   void        ShowBitmap( HDC hdc, HBITMAP hBM, WORD x, WORD y, WORD w, WORD h );
   void        DeleteFonts();

   void     WriteCR();
public:
   CCDisplay();

   void Init( HWND win );
   void SetCCFileName( LPCSTR name );
   void SetCCAlert( LPCSTR token );
   void GetCCFileName( LPSTR name, int cnt ) { 
/*       if (!bAllowCC) {
          return;
       }*/
       strncpy( name, file_, cnt ); 
   }
   void GetCCAlert( LPSTR name, int cnt ) { 
/*       if (!bAllowCC) {
          return;
       }*/
       strncpy( name, alert_, cnt ); 
   }
   void Process( LPBYTE data, int len );
   void DrawScreenArray();
   void Display();
   void CreateFonts(bool bDoingInit = false);
   void Erase();
   void SetChannel( int ch );
   int  GetChannel() { 
       if (!bAllowCC) {
          return 0;
       }
       return wCapType_; 
   }
   void EraseDisplay() {
       if (!bAllowCC) {
          return;
       }
       Erase();
   }
   void KillWatchdog() 
   {
      if (tid_) {
         KillTimer(NULL, tid_) ;
         tid_ = NULL ;
      }
   }
   ~CCDisplay();
};


const IndentBit = 0x10;
const PopOn     = 0;
const PaintOn   = 1;
const RollUp    = 2;


// For the Montel Mode
#define RU_MAX                       1

// cntl defines
#define PAC                          1  // Preamble Controls
#define MRC                          2  // Mid-Row Controls
#define MSC                          3  // Miscellaneous Controls

//  Colors
#define CC_WHITE                     0x00
#define CC_GREEN                     0x02
#define CC_BLUE                      0x04
#define CC_CYAN                      0x06
#define CC_RED                       0x08
#define CC_YELLOW                    0x0A
#define CC_MAGENTA                   0x0C
#define CC_ITALICS                   0x0E
#define CC_UNDERLINE                 0x01

// ColorRef Defines
#define CR_WHITE                     (RGB(0xFF,0xFF,0xFF))
#define CR_BGREEN                    (RGB(0x00,0xFF,0x00))
#define CR_BBLUE                     (RGB(0x00,0x00,0xFF))
#define CR_BCYAN                     (RGB(0x00,0xFF,0xFF))
#define CR_BRED                      (RGB(0xFF,0x00,0x00))
#define CR_BMAGENTA                  (RGB(0xFF,0x00,0xFF))
#define CR_BYELLOW                   (RGB(0xFF,0xFF,0x00))
#define CR_DGREEN                    (RGB(0x00,0x80,0x00))
#define CR_DBLUE                     (RGB(0x00,0x00,0x80))
#define CR_DCYAN                     (RGB(0x00,0x80,0x80))
#define CR_DRED                      (RGB(0x80,0x00,0x00))
#define CR_DMAGENTA                  (RGB(0xC0,0x10,0xC0))
#define CR_DYELLOW                   (RGB(0x80,0x80,0x00))

// Indenting
#define CC_INDENT0                   8
#define CC_INDENT4                   9
#define CC_INDENT8                   10
#define CC_INDENT12                  11
#define CC_INDENT16                  12
#define CC_INDENT20                  13
#define CC_INDENT24                  14
#define CC_INDENT28                  15

// Rows from preamble
#define PRE_ROW1                     0
#define PRE_ROW2                     1
#define PRE_ROW3                     2
#define PRE_ROW4                     3
#define PRE_ROW5                     4
#define PRE_ROW6                     5
#define PRE_ROW7                     6
#define PRE_ROW8                     7
#define PRE_ROW9                     8
#define PRE_ROW10                    9
#define PRE_ROW11                    10
#define PRE_ROW12                    11
#define PRE_ROW13                    12
#define PRE_ROW14                    13
#define PRE_ROW15                    14

// Preamble Commands
#define PREAMBLE_DC                  0x4070
#define PREAMBLE_MASK                0x4010
#define ISPREAMBLE(x) ((PREAMBLE_MASK == (PREAMBLE_DC & x)) ? TRUE : FALSE)

// Midrow Commands
#define MIDROW_DC                    0x7077
#define MIDROW_MASK                  0x2011
#define ISMIDROW(x)   ((MIDROW_MASK == (MIDROW_DC & x)) ? TRUE : FALSE)

// Miscellaneous Commands
#define MISC_DC                      0x7077
#define MISC_MASK                    0x2014
#define ISMISC(x)     ((MISC_MASK == (MISC_DC & x)) ? TRUE : FALSE)

// Special Charactes
#define SCHAR_DC                     0x7077
#define SCHAR_MASK                   0x3011
#define ISSCHAR(x)    ((SCHAR_MASK == (SCHAR_DC & x)) ? TRUE : FALSE)

// Miscellaneous Control Codes
#define RESUME_CAP_LOAD              0  // Pop-on
#define BACKSPACE                    1
#define DELETE_TO_END                4
#define ROLLUP_2ROWS                 5  // Roll-up
#define ROLLUP_3ROWS                 6  // Roll-up
#define ROLLUP_4ROWS                 7  // Roll-up
#define FLASH_ON                     8
#define RESUME_DIRECT_CAP            9  // Paint-on
#define TEXT_RESTART                 10
#define RESUME_TEXT_DISPLAY          11
#define ERASE_DISPLAYED_MEM          12
#define CARRIAGE_RETURN              13
#define ERASE_NONDISP_MEM            14
#define EOC_FLIP_MEMORY              15

// Capture modes
#define RCL                          RESUME_CAP_LOAD
#define RDC                          RESUME_DIRECT_CAP
#define RU2                          ROLLUP_2ROWS
#define RU3                          ROLLUP_3ROWS
#define RU4                          ROLLUP_4ROWS

// tab offset commands
#define TAB_OFFSET1_0                0x2117
#define TAB_OFFSET1_1                0x211F
#define TAB_OFFSET2_0                0x2217
#define TAB_OFFSET2_1                0x221F
#define TAB_OFFSET3_0                0x2317
#define TAB_OFFSET3_1                0x231F

// Special character defines
#define SC0                          0x0  // Registered mark symbol
#define SC1                          0x1  // Degree sign
#define SC2                          0x2  // 1/2 symbol
#define SC3                          0x3  // Inverse query
#define SC4                          0x4  // Trademark symbol
#define SC5                          0x5  // Cents symbol
#define SC6                          0x6  // Pounds Sterling symbol
#define SC7                          0x7  // Music note
#define SC8                          0x8  // Lower-case a with grave accent
#define SC9                          0x9  // Transparent Space, do nothing
#define SCA                          0xA  // Lower-case e with grave accent
#define SCB                          0xB  // Lower-case a with circumflex
#define SCC                          0xC  // Lower-case e with circumflex
#define SCD                          0xD  // Lower-case i with circumflex
#define SCE                          0xE  // Lower-case o with circumflex
#define SCF                          0xF  // Lower-case u with circumflex

// Flash On Timing defines
#define FLASH_ON_COUNT               3
#define FLASH_OFF_COUNT              9




#endif
