/* File: CCDecoder.h
 * Purpose: Defines CC decoder class
 * Author: sasha
 * $Log: CCEngine.h $
 * Revision 2.11  1997/07/31 10:48:36  dtharp
 * Cleaned up multiple CC_CALLBACK definitions.
 * Revision 2.10  1997/06/27 16:54:24  dtharp
 * Callback now receives an xtra argument (this is actually the skip 
 * count for the capture callback and is undefined here).
 * Revision 2.9  1997/05/14 17:44:33  dtharp
 * Cast to CC_CALLBACK to avoid compiler warning.
 * Revision 2.8  1997/05/05 17:54:28  dtharp
 * 848A Merge: from 2.7.1.2
 * Revision 2.7.1.2  1997/02/14 23:33:02  rudyl
 * Added #ifdef around CC_CALLBACK typedef to resolve "already defined" compile error.
 * Revision 2.7.1.1  1997/02/14 23:31:54  rudyl
 * Pisces+ work: add #ifdef around CC_CALLBACK def.
 * Revision 2.7  1997/01/17 13:57:36  ssm
 * men ccchannel ccChan
 * Revision 2.6  1996/10/10 20:21:20  dtharp
 * Removed includes for ccdispl.h and ccdec.h.
 * Revision 2.5  1996/10/04 15:02:27  dtharp
 * Changes required for new CC Dll API function names
 * Revision 2.4  1996/10/03 16:37:55  dtharp
 * Modified to use functions in api_cc.cpp rather than explicit pointers
 * into cc.dll.
 * Revision 2.3  1996/10/02 20:07:18  dtharp
 * Implemented modifications required to split the Closed Caption 
 * Decoder into a separate DLL.
 * Revision 2.2  1996/10/02 04:21:18  sasha
 * Added CC_CALLBACK typedef. Added CC_CALLBACK Formatter_
 * member. Added SetCCProcessor() to set the Formatter_. All this is needed
 * to enable extarnal CC callbacks.
 * Revision 2.1  1996/06/10 20:12:10  ssm
 * MarkBeta1AsRev2
 * Revision 1.5  1996/06/05 04:12:06  sasha
 * Added functions to set CC file name and alert token
 * Revision 1.4  1996/06/04 19:19:04  sasha
 * Changed CC channel members to be pointers. A class derived from
 * VfWChannel must be allocated dynamically.
 * Revision 1.3  1996/05/31 17:42:23  sasha
 * Very close. Any minute now
 * Revision 1.2  1996/05/30 23:11:53  sasha
 * Just next version.
 * Revision 1.1  1996/05/28 17:51:57  sasha
 * Initial revision
 */

#ifndef __CCENGINE_H
#define __CCENGINE_H


#ifndef __CCCHANNEL_H
#include "ccChan.h"
#endif

#include "cc_api.h"

#include "common.h"

#define CC_DECODE_BUF_SIZE 300
/* Class: CCEngine
 * Purpose: Implements functionality of a CC decoder
 */
class CCEngine
{
   static CCChannel *Even_;
   static CCChannel *Odd_;
   static BYTE      Decoded_ [CC_DECODE_BUF_SIZE];
   static volatile  int DecodCnt_;
   static bool      Running_;

   static CC_CALLBACK Formatter_;

   static void CALLBACK Format( LPBYTE data, int len ) { CC_Process( data, len ); }

//   static void _loadds WINAPI Callback( DWORD addr, DWORD xtra );
   static void _loadds CALLBACK Callback( DWORD addr, DWORD xtra );

public:

   CCEngine() { SetCCProcessor(); }
   ~CCEngine();

   void Init( HWND hWin );
   void SetCCFileName( LPCSTR name ) { CC_SetFileName( name ); }
   void SetCCAlert( LPCSTR token ) { CC_SetAlert( token ); }
   void GetCCFileName( LPSTR name, int cnt ) { CC_GetFileName( name, cnt ); }
   void GetCCAlert( LPSTR name, int cnt ) { CC_GetAlert( name, cnt ); }

   static void SetCCProcessor( CC_CALLBACK cb = NULL);
   void StartDecode();
   void StopDecode();
   void SetChannel( int ch ) { CC_SetChannel( ch ); }
   int  GetChannel() { return CC_GetChannel(); }
   void Erase() { CC_EraseDisplay(); }

   static void Display();
};

/* Method: CCEngine::SetCCProcessor
 * Purpose: Sets a CC formatter pointer to be either external or local
 * Input: cb: CC_CALLBACK
 */
void inline CCEngine::SetCCProcessor( CC_CALLBACK cb )
{
   if ( cb )
      Formatter_ = cb;
   else
      Formatter_ = (CC_CALLBACK) Format;
}


#endif __CCEINGINE_H
