/* File: CCChannel.cpp * Purpose: Implements the closed captioning channel
 * Author: sasha
 * Copyright  1996 Brooktree Corporation
 * $Log: CCchan.cpp $
 * Revision 2.4  1998/04/02 21:37:09  dtharp
 * Merge DT_VIDEOHDR branch. Implements a new buffer 
 * management scheme.
 * Revision 2.3.2.2  1998/03/23 18:10:58  dtharp
 * Added setting of flags in videohdr struct
 * Revision 2.3.2.1  1998/03/18 19:33:28  dtharp
 * Branch for passing videoheaders around
 * Revision 2.3  1997/01/17 13:57:35  ssm
 * men ccchannel ccChan
 * Revision 2.2  1996/10/10 17:54:38  dtharp
 * Removed unneeded include file ccdec.h
 * Revision 2.1  1996/06/10 20:12:11  ssm
 * MarkBeta1AsRev2
 * Revision 1.4  1996/06/04 19:35:35  sasha
 * Changed destructor
 * Revision 1.3  1996/05/31 17:42:24  sasha
 * Very close. Any minute now
 * Revision 1.2  1996/05/30 23:11:53  sasha
 * Just next version.
 * Revision 1.1  1996/05/28 17:52:11  sasha
 * Initial revision
 */

#include "ccChan.h"

bool CCChannel::Started_ = false;

/* Method: CCChannel::OpenChannel
 * Purpose: Opens channel to the VBI & CC data
 * Input: win: HWND
 * Output: ErrorCode
 */
ErrorCode CCChannel::OpenChannel()
{
   if ( LoadVxD() != Success )
      return Fail;

   if ( OpenVxDChannel( ThisStream_, 0 ) != Success )
      return Fail;

   AddBuffers();

   return Success;
}

/* Method: CCCHannel::CloseChannel
 * Purpose: Finishes off with the CC data
 */
void CCChannel::CloseChannel()
{
   CloseVxDChannel();
   RemoveBuffers();
}

/* Method: CCChannel::Start
 * Purpose: Starts the capture of the CC Data
 * Input: None
 */
ErrorCode CCChannel::Start()
{
   if ( Started_ )
      return Fail;

   StartParms strt( SmallRect( TRect( 0, 0, 0, 0 ) ),
      SmallRect( TRect( 0, 0, 0, 0 ) ), 0 );

   strt.dwPitch = 0;
   strt.callback = (DWORD)ParentCB_;
   strt.tag      = (DWORD)(LPVOID)this;
   strt.FrameRate = 0;

   if ( CallVxD( FNStart, &strt ) != Success )
      printf( "no luck\n" );

   Started_ = true;
   return Success;
}

/* Method: CCChannel::Stop
 * Purpose: Starts the capture of the CC Data
 * Input: None
 */
void CCChannel::Stop()
{
   if ( Started_ == false )
      return;
   Started_ = false;
   CallVxD( FNStop );
}

#define VBI_BUF_SIZE  (768L * 20L * 2L + 3)

LPBYTE AllocBuffer()
{
   HGLOBAL hTmp = GlobalAlloc( GPTR, VBI_BUF_SIZE );
   LPBYTE lpTmp = (LPBYTE)GlobalLock( hTmp );
   GlobalPageLock( hTmp );
   return lpTmp;
}

static LPVIDEOHDR AllocVideoHdr()
{
   HGLOBAL hTmp = GlobalAlloc( GPTR, sizeof(VIDEOHDR) );
   LPVIDEOHDR lpTmp = (LPVIDEOHDR)GlobalLock( hTmp );
   GlobalPageLock( hTmp );
   return lpTmp;
}


/* Method:
 * Purpose: Allocates VBI data buffers and sends them to VxD
 */
void CCChannel::AddBuffers()
{
   VideoHdr1_ = AllocVideoHdr() ;
   VideoHdr2_ = AllocVideoHdr() ;
   VideoHdr1_->lpData = AllocBuffer();
   VideoHdr2_->lpData = AllocBuffer();

   VideoHdr1_->dwBufferLength =  VideoHdr2_->dwBufferLength = VBI_BUF_SIZE;

   Buf1Base_ = GetSelectorBase( HIWORD( VideoHdr1_->lpData ) );
   Buf2Base_ = GetSelectorBase( HIWORD( VideoHdr2_->lpData ) );

   VideoHdr1_->dwFlags &= ~VHDR_DONE;
   VideoHdr1_->dwFlags |= VHDR_INQUEUE;

   VideoHdr2_->dwFlags &= ~VHDR_DONE;
   VideoHdr2_->dwFlags |= VHDR_INQUEUE;

   CallVxD( FNAddBuf, VideoHdr1_ );
   CallVxD( FNAddBuf, VideoHdr2_ );

}

/* Method: CCChannel::RemoveBuffers
 * Purpose: Disposes of data buffers
 * Input: None
 */
void CCChannel::RemoveBuffers()
{
   if ( VideoHdr1_ ) {
      GlobalPageUnlock( (HGLOBAL)GlobalHandle( HIWORD( VideoHdr1_->lpData ) ) );
      GlobalFree( (HGLOBAL)GlobalHandle( HIWORD( VideoHdr1_->lpData ) ) );

      GlobalPageUnlock( (HGLOBAL)GlobalHandle( HIWORD( VideoHdr1_ ) ) );
      GlobalFree( (HGLOBAL)GlobalHandle( HIWORD( VideoHdr1_ ) ) );
   }
   if ( VideoHdr2_ ) {
      GlobalPageUnlock( (HGLOBAL)GlobalHandle( HIWORD( VideoHdr2_->lpData ) ) );
      GlobalFree( (HGLOBAL)GlobalHandle( HIWORD( VideoHdr2_->lpData ) ) );

      GlobalPageUnlock( (HGLOBAL)GlobalHandle( HIWORD( VideoHdr2_ ) ) );
      GlobalFree( (HGLOBAL)GlobalHandle( HIWORD( VideoHdr2_ ) ) );
   }
   VideoHdr1_ = VideoHdr2_ = NULL;
}

/* Destructor: CCChannel::~CCChannel
 * Purpose: Disposes of the CC channel object
 * Note: Closes the channel
 */
CCChannel::~CCChannel()
{
   CloseChannel();
}
