/* $Header: G:/SwDev/W95/Video/pisces/dvlpmnt/CC/rcs/cc_api.h 1.5 1997/06/02 21:50:46 dtharp Released $ */
/****************************************************************************/
/***** Copyright (c) Brooktree Corporation, 1996. All Rights Reserved *******/
/****************************************************************************/
/*                                                                          */
/*                      BROOKTREE CONFIDENTIAL                              */
/*                                                                          */
/* Filename:    CC_API.H                                                    */
/*                                                                          */
/* Description: typedefs required for pointer type function prototypes.     */
/* These typedefs are used to type the pointers returned from               */
/* GetProcAddress.                                                          */
/*                                                                          */
/*                                                                          */
/* Author:      Dave Tharp                                                  */
/*                                                                          */
/****************************************************************************/

/* 
 * $Log: cc_api.h $
 * Revision 1.5  1997/06/02 21:50:46  dtharp
 * Bug id 628: flashing pink box every 5 sec in preview. CC display
 * watchdog timer is now shut down when CC is paused.
 * Revision 1.4  1996/10/10 20:30:31  dtharp
 * Added constants from ccdispl.h and ccdec.h.
 * Revision 1.3  1996/10/10 17:29:16  dtharp
 * Added prototype for InitCcApi.
 * Revision 1.2  1996/10/04 14:58:52  dtharp
 * Added Function prototypes for API functions.
 * Revision 1.1  1996/10/03 16:36:29  dtharp
 * Initial revision
 */

#ifndef CC_API_H
#define CC_API_H

#ifdef CC_DLL
  #define EXPORT _export
#else
  #define EXPORT
#endif

#include <windows.h>

//
// Decoder Constants
//
   #define BT819CC_IN_LUMA      0x00000001UL
   #define BT819CC_IN_422       0x00000002UL

   // The decoder functions can return the following error codes:
   #define BT819CC_ERR_NONE     0  // no error occurred
   #define BT819CC_ERR_PARAM    1  // either an insufficient amount of
                                   //   input data is provided, or the
                                   //   horizontal resolution is invalid,
                                   //   or decodeMode is invalid
   #define BT819CC_ERR_NO_DATA  2  // closed captioning data could not
                                   //   be identified in the video
   #define BT819CC_ERR_PARITY   3  // closed captioning data had a parity
                                   //   error
   #define BT819CC_ERR_INIT     4  // dll not initialized
   #define BT819CC_ERR_INSTANCE 5  // multiple instances not allowed
   #define BT819CC_ERR_MODE     6  // unknown mode

//
// Display Constants
//

   #define CCMODE_UNKNOWN       0
   #define CCMODE_CC1           1
   #define CCMODE_CC2           2
   #define CCMODE_TEXT1         3
   #define CCMODE_TEXT2         4


//
// Typedefs
//

typedef void EXPORT (WINAPI  INITDISPLAY_f) ( HWND win );
typedef INITDISPLAY_f *INITDISPLAY_t;

typedef void EXPORT (WINAPI  KILLWATCHDOG_f) ( );
typedef KILLWATCHDOG_f *KILLWATCHDOG_t;

typedef void EXPORT (WINAPI  SETCCFILENAME_f) ( LPCSTR name );
typedef SETCCFILENAME_f * SETCCFILENAME_t;

typedef void EXPORT (WINAPI  SETCCALERT_f) ( LPCSTR token );
typedef SETCCALERT_f * SETCCALERT_t;

typedef void EXPORT (WINAPI  GETCCFILENAME_f) ( LPSTR name, int cnt );
typedef GETCCFILENAME_f * GETCCFILENAME_t;

typedef void EXPORT (WINAPI  GETCCALERT_f) ( LPSTR name, int cnt );
typedef GETCCALERT_f * GETCCALERT_t;

typedef void EXPORT (WINAPI  PROCESS_f) ( LPBYTE data, int len );
typedef PROCESS_f * PROCESS_t;

typedef void EXPORT (WINAPI  DISPLAY_f) ( );
typedef DISPLAY_f * DISPLAY_t;

typedef void EXPORT (WINAPI  CREATEFONTS_f) ( );
typedef CREATEFONTS_f * CREATEFONTS_t;

typedef void EXPORT (WINAPI  ERASE_f) ( );
typedef ERASE_f * ERASE_t;

typedef void EXPORT (WINAPI  SETCHANNEL_f) ( int ch );
typedef SETCHANNEL_f * SETCHANNEL_t;

typedef int  EXPORT (WINAPI  GETCHANNEL_f) ( );
typedef GETCHANNEL_f * GETCHANNEL_t;

typedef void EXPORT (WINAPI  ERASEDISPLAY_f) ( );
typedef ERASEDISPLAY_f * ERASEDISPLAY_t;

typedef int EXPORT (WINAPI  INITDECODE_f) ( int samplesLen, 
                                            int horizResolution, 
                                            unsigned long decodeMode );
typedef INITDECODE_f * INITDECODE_t;

typedef int EXPORT (WINAPI  DECODE_f) (const BYTE far* samples,
                                int   lines,
                                const BYTE far* decodedData,
                                int   far * decodedDataLen );
typedef DECODE_f * DECODE_t;


//
// Function Prototypes
//
#if defined(__cplusplus)
extern "C" {
#endif

//
// Only found in capdrv 
HINSTANCE InitCcApi(void) ;

// void CC_InitDisplay ( HWND win )
INITDISPLAY_f CC_InitDisplay ;

// void CC_KillWatchdog ( void )
KILLWATCHDOG_f CC_KillWatchdog ;

// void CC_SetFileName ( LPCSTR name )
SETCCFILENAME_f CC_SetFileName ;

// void CC_SetAlert ( LPCSTR token )
SETCCALERT_f CC_SetAlert ;

// void CC_GetFileName ( LPSTR name, int cnt )
GETCCFILENAME_f CC_GetFileName ;

// void CC_GetAlert ( LPSTR name, int cnt )
GETCCALERT_f CC_GetAlert ;

// void CC_Process ( LPBYTE data, int len )
PROCESS_f CC_Process ;

// void CC_Display ( )
DISPLAY_f CC_Display ;

// void CC_CreateFonts ( )
CREATEFONTS_f CC_CreateFonts ;

// void CC_Erase ( )
ERASE_f CC_Erase ;

// void CC_SetChannel ( int ch )
SETCHANNEL_f CC_SetChannel ;

// int CC_GetChannel ( )
GETCHANNEL_f CC_GetChannel ;

// void CC_EraseDisplay ( )
ERASEDISPLAY_f CC_EraseDisplay ;

// int CC_InitDecode ( int samplesLen, 
//                  int horizResolution, 
//                  unsigned long decodeMode )
INITDECODE_f CC_InitDecode ;

// int CC_Decode ( const BYTE far* samples,
//              int   lines,
//              const BYTE far* decodedData,
//              int   far * decodedDataLen )
DECODE_f CC_Decode ;


#if defined(__cplusplus)
} // extern C
#endif

#endif

//EOF