/* File: CCDispl.cpp
 * Purpose:
 * $Log: ccdispl.cpp $
 * Revision 2.15  1997/11/26 00:11:35  peter
 * Graft reg key move to tip.
 * Revision 2.14.2.1  1997/11/20 22:44:22  peter
 * Changes to move registry subkey to HKEY_LOCAL_MACHINE.
 * Revision 2.14  1997/06/27 17:01:06  dtharp
 * Attempt to fix CR handling problem.
 * Revision 2.13  1997/05/28 17:21:20  trantq
 * Merge Rev 2.10.1.3 fix bug 548.
 * Revision 2.12  1997/05/27 23:12:25  dtharp
 * fix for Bugid 609: CC Inop in build 142.
 * Revision 2.11  1997/05/22 18:38:25  dtharp
 * Bug 560: Implemented a time-out period for cc updates. If the
 * time-out is exceeded, the display is erased. Current timeout is
 * 5 seconds.
 * Revision 2.10.1.3  1997/05/15 01:13:20  trantq
 * - Add more to fix bug ID 548.
 * Revision 2.10  1997/05/02 17:36:02  dtharp
 * Bug Id 531: Fix for CC displaying in the wrong location when window
 * is moved.
 * Revision 2.9  1997/04/29 20:54:18  dtharp
 * Reversed polarity of CC Primary surface control registry keys.
 * The new key names are "KillPrimaryCC" and "KillPrimaryCCFull".
 * Revision 2.8  1997/04/28 20:49:05  dtharp
 * Merged CC Primary branch from 2.4.2.5. Bug ID 475.
 * Revision 2.4.2.5  1997/04/26 00:35:37  dtharp
 * Full Screen CC Primary Surface permission is now Via "CC Full" 
 * registry key.
 * Revision 2.4.2.4  1997/04/25 23:55:19  dtharp
 * Now determines Primary surface correctly.
 * Revision 2.4.2.3  1997/04/25 16:36:44  dtharp
 * Interim checkin.
 * Revision 2.4.2.2  1997/04/24 21:09:19  dtharp
 * Stable CC Primary < full screen.
 * Revision 2.7  1997/04/21 23:55:18  dtharp
 * Bug Id 508 (Toshiba GPF). Added protection against calling 
 * exported member functions before Init()is called.
 * Also Bug Id 509 (Font Init problem). Changed CreateFonts() to
 * recognize whether it is being called from Init() or not.
 * Revision 2.6  1997/04/18 15:23:39  dtharp
 * Removed include for (non standard) debug.h.
 * Revision 2.5  1997/04/18 00:33:36  dtharp
 * Fix for Bug Id 433. Closed Caption Performance (speed, cpu).
 * Revision 2.4.1.16  1997/03/18 17:26:21  tomz
 * added  flag to control primary surface CC display
 * Revision 2.4.1.15  1997/03/17 22:53:08  tomz
 * safety
 * Revision 2.4.1.14  1997/03/04 14:57:15  tomz
 * Attempt to fix italics
 * Revision 2.4.1.13  1997/02/28 14:58:54  tomz
 * Added clipping code
 * Revision 2.4.1.12  1997/02/19 17:25:37  tomz
 * safety
 * Revision 2.4.1.11  1997/02/17 19:12:06  tomz
 * Calculate clipping
 * Revision 2.4.1.10  1997/02/14 22:44:27  tomz
 * Stripped out the old code
 * Revision 2.4.1.9  1997/02/14 22:06:09  tomz
 * Fixed more stuff
 * Revision 2.4.1.8  1997/02/13 23:39:38  tomz
 * safety
 * Revision 2.4.1.7  1997/02/13 14:11:52  tomz
 * Looks better, but no pop-on yet
 * Revision 2.4.1.6  1997/02/11 18:39:01  tomz
 * Scrolling works
 * Revision 2.4.1.5  1997/02/11 16:24:40  tomz
 * safety backup
 * Revision 2.4.1.4  1997/02/06 23:46:14  tomz
 * Remove complete repaints.
 * Revision 2.4.1.3  1997/01/31 17:56:41  tomz
 * Revision 2.4.1.2  1997/01/30 16:00:07  tomz
 * Revision 2.4.1.1  1997/01/29 15:58:45  tomz
 * Start a branch
 * Revision 2.4  1997/01/27 17:26:58  tomz
 * Merged branch 2.3.2
 * Removed overlay mode CC flicker, fixed boundary condition bug.
 * Revision 2.3.2.5  1997/01/27 17:23:22  tomz
 * Fix boundary condition bug, removed debug prints.
 * Revision 2.3.2.4  1997/01/23 00:09:41  tomz
 * Backed out previous version.  I screwed something up good.
 * Revision 2.3.2.2  1997/01/22 01:06:39  tomz
 * Revision 2.3.2.1  1997/01/21 18:28:10  tomz
 * Removed flicker
 * Revision 2.3  1996/12/06 00:17:21  trantq
 * Merged branch Rev. 2.2.1
 * Revision 2.2.1.2  1996/12/06 00:15:15  trantq
 * Added processing for CR in PaintOn Mode.
 * Revision 2.2.1.1  1996/12/03 23:02:34  trantq
 * Implement CC insturmentation.
 * Revision 2.2  1996/10/10 20:19:22  dtharp
 * Added include for cc_api.h.
 * Revision 2.1  1996/06/10 20:12:12  ssm
 * MarkBeta1AsRev2
 * Revision 1.12  1996/06/07 20:03:41  sasha
 * Changed where alert box is displayed.
 * Revision 1.11  1996/06/06 01:14:47  sasha
 * Deleted try{} catch{}
 * Revision 1.10  1996/06/05 19:17:52  sasha
 * Save file name for later retrieval. Initialize alert_ and file_.
 * Revision 1.9  1996/06/05 04:12:32  sasha
 * Support CC alerts and CC data file
 * Revision 1.8  1996/06/04 01:05:04  sasha
 * Now displays at proper offsets in both 320x240 and 640x480. Check for
 * non-zero window handle in the Init().
 * Revision 1.7  1996/06/03 19:42:29  sasha
 * Added check for the division sign. Tried to reduce the flicker a bit.
 * It still shows in pop-on.
 * Revision 1.6  1996/06/03 17:57:31  sasha
 * Commented out OutputDebugString()s
 * Revision 1.5  1996/06/01 03:13:29  sasha
 * SHIP IT !
*/
#ifndef __STDIO_H
#include <stdio.h>
#endif
#include <ctype.h>
#include <dos.h>

#include "dbgprint.h"

#include "cc_api.h"

#include "ccdispl.h"
#include "bitmapid.h"
#include <regutil.h>
#include <vxdiface.h>
#include <vxdfunc.h>

#define  nFORCE_PRIMARY_ROLLUP    1

// Millisecond CC display timeout. If time between updates exceeds this value, the display
// is erased.
#define TIMER_VAL 5000

static CCDisplay * me = NULL ;

// #include <winsys\geometry.h>
void GetTextWidth( HDC hdc, WORD &w, WORD &h );
HINSTANCE GetOurInstance();

bool  WasPAC_ = false;
VxDIface PiscesEntry_ ;
unsigned long gctr = 0;

#define ISTAB( c ) ( ( ( c ) & 0x2017 ) == 0x2017 )

CC_Char::CC_Char()
{
   visible = true;
   ch = 0;
   row = 0;
   column = 0;
   attr = AttrNone;
   hfont = 0;
   color = 0;
   CCD = NULL;
}

CC_Char::~CC_Char()
{
}

void CC_Char::Init(char param_ch, int param_row, int param_column, CCDisplay * param_ccd)
{
   CCD = param_ccd;
   ch = param_ch;
   row = param_row;
   column = param_column;
   attr = AttrNone;
   hfont = CCD->hFont;
   color = CR_WHITE;
   visible = false;

   if (ch == SC4 ) {
      hbm = CCD->hbmTradeMark;
   } else if (ch == SC7) {
      hbm = CCD->hbmMusicNote;
   } else {
      hbm = NULL;
   }

   Draw();
}

void CC_Char::Copy(CC_Char* ccc)
{
   int saved_row    = row;
   int saved_column = column;

   *this = *ccc;  // copy all elements exactly

   row    = saved_row;
   column = saved_column;

   visible = false;
   //Draw();
}

void CC_Char::Draw()
{
   DebugAssert((CCD != 0, "CCD == 0\n"));

   if ((visible 
     && CCD->bOverlayMode
   ) || !CCD->ReadyToDisplay_) {
      return;
   }

   HDC hdc = CCD->hdcScreen;
   int x = CCD->wXOffset;
   int y = CCD->wYOffset;

   DebugAssert((hdc != 0, "hdc == 0\n"));

   TRect TextR;

   y += row * CCD->wHeight;
   x += column * CCD->wWidth;

   TextR.left   = x;
   TextR.top    = y;
   TextR.right  = x + CCD->wWidth;
   TextR.bottom = y + CCD->wHeight;

   if (hbm) {
      CCD->ShowBitmap( hdc, hbm, x, y, CCD->wWidth, CCD->wHeight);
   } else {
      if (ch) {
         SelectObject(hdc, hfont);
         SetTextColor(hdc, color);
         SetBkColor( hdc, RGB( 0, 0, 0 ) );

         //SetBkMode(hdc, OPAQUE);
         FillRect( hdc, &TextR, CCD->hbrOpaqueBrush );
         SetBkMode(hdc, TRANSPARENT);

         ExtTextOut(hdc, x, y, ETO_CLIPPED, &TextR, &ch, 1, NULL);
      } else {
         if (CCD->bOverlayMode) {
            FillRect( hdc, &TextR, CCD->hbrColorKeyBrush );
         }
      }
   }
   visible = true;
}

void CC_Char::Erase()
{
   DebugAssert((CCD != 0, "CCD == 0\n"));

   visible = false;
   hfont = CCD->hFont;
   ch = 0;
   color = CR_WHITE;
   hbm = NULL;
   //Draw();
}

void CC_Char::Replace(char param_ch)
{
   DebugAssert((CCD != 0, "CCD == 0\n"));

   visible = false;
   ch = param_ch;

   if (ch == SC4 ) {
      hbm = CCD->hbmTradeMark;
   } else if (ch == SC7) {
      hbm = CCD->hbmMusicNote;
   } else {
      hbm = NULL;
   }

   hfont = CCD->mru_hfont;
   color = CCD->mru_color;

/***********************************************************
* NOTE: Currently, don't support color
***********************************************************/

   // override to white
   color = CR_WHITE;

   //Draw();
}

/* Method:
 * Purpose:
*/

unsigned long CCDisplay::ComputeClip()
{
   int count = 0;

   for (int row = 0; row < MAX_ROW; row++) {
      bool new_row = true;
      for (int column = 0; column < MAX_COL; column++) {
         if (CCScreenArray[row][column].ch != 0) {
            int x = wXOffset;
            int y = wYOffset;

            LRECT TextR;
            y += row * wHeight;
            x += column * wWidth;

            TextR.left   = x;
            TextR.top    = y;
            TextR.right  = x + wWidth;
            TextR.bottom = y + wHeight;

            if (new_row) {
               ClipRect[count] = TextR;
               count++;
               new_row = false;
            } else {
               if (TextR.left == ClipRect[count-1].right) {
                  // The new rectangle is adjacent to the last rectangle.
                  // Just stretch the previous one.
                  ClipRect[count-1].right = TextR.right;
               } else {
                  ClipRect[count] = TextR;
                  count++;
               }
            }
         }
      } // for col
   }

   ClipRgn->rdh.nCount = count;
   return count;
}


/* Method:
 * Purpose:
*/

void CCDisplay::SetClipping( )
{
   if (bOverlayMode) {
      return;
   }

   int count = ComputeClip();

   DebugPrint((1, "Setting (%d) clip rectangles\n", count));
   for (int i = 0; i < count; i++) {
      DebugPrint((1, "\t(%ld,%ld,%ld,%ld)\n",
                 ClipRect[i].left,
                 ClipRect[i].top,
                 ClipRect[i].right,
                 ClipRect[i].bottom)) ;

      ClipRect[i].right ;

   }

   if (count == 0) {
      PiscesEntry_.CallVxD(FNCCClip,0) ;
   } else {
      DWORD linAddr = GetSelectorBase(FP_SEG(ClipRgn)) + FP_OFF(ClipRgn) ;
      PiscesEntry_.CallVxD(FNCCClip,NULL,linAddr) ;
   }
}

void CCDisplay::Process( LPBYTE data, int len )
{
   LPBYTE sentinal = data + len;

   if (!bAllowCC) {
      return;
   }

   while (data < sentinal) {

      char NonDisp = *data;
      char Disp    = *( data + 1 );
      WORD wCode   = *(LPWORD)data;

      if (NonDisp > 0x1F) {

         // normal text characters

         if(bGotFirstCR_ || (wCapStyle_ != PaintOn)) {
            ProcessChar( NonDisp );
            NeedToDisplay_ = ProcessChar( Disp );
         }

      } else {

         // two byte code

         if ( ISPREAMBLE( wCode ) ) {
            NeedToDisplay_ = ProcessPreamble( NonDisp, Disp );
         } else if ( ISMIDROW( wCode ) ) {
            NeedToDisplay_ = ProcessMidrow( NonDisp, Disp );
         } else if ( ISMISC( wCode ) ) {
            NeedToDisplay_ = ProcessMisc( NonDisp, Disp );
         } else if ( ISSCHAR( wCode ) ) {
            NeedToDisplay_ = ProcessSChar( NonDisp, Disp );
         } else if ( ISTAB( wCode ) ) {
            ProcessTabOffset( wCode );
         }

      }

      data += 2;

      if ( (bGotFirstCR_ || (wCapStyle_ != PaintOn)) && ReadyToDisplay_ && 
           ( NeedToDisplay_ || wCapStyle_ == PopOn ) ) {
         Display();
      }
   }

   static bool BoxUp = false;
   if ( AlertMatch_ && !BoxUp ) {
      BoxUp = true;
      AlertMatch_ = false;
      char msg [100];
      wsprintf( msg, "String containing word\n%s\nappeared in CC data", alert_ );
      MessageBox( hWnd_, msg, "CC Alert", MB_OK );
      BoxUp = false;
   }
}

/* Method: CCTimerProc
 * Purpose: Called if time between cc updates > TIMER_VAL. Erases display in this case.
 * Input: None
 * Output: None
*/
VOID CALLBACK _loadds CCTimerProc(HWND, UINT, UINT, DWORD)
{
   if (me)
   {
     me->Erase() ;
     me->DrawScreenArray() ;
   }
   DebugPrint((0,"Timer Proc\n")) ;
}


/* Method: CCDisplay::DrawScreenArray
 * Purpose: Displays text
 * Input: None
 * Output: None
*/
void CCDisplay::DrawScreenArray()
{
   SetClipping();

   for (int row = 0; row < MAX_ROW; row++) {
      for (int col = 0; col < MAX_COL; col++) {
         CCScreenArray[row][col].Draw();
      }
   }
}

/* Method: CCDisplay::Display
 * Purpose: Displays text
 * Input: None
 * Output: None
*/
void CCDisplay::Display()
{
   if (!bAllowCC) {
      return;
   }
   // In case we just set ReadyToDisplay_ to true

   if (NeedToDisplay_) {
      NeedToDisplay_ = false;

      DrawScreenArray() ;

      // Set/Reset timer. If this is the first display of cc text a timer is started. 
      // If this timer expires before the next display of text, the timer handler is called,
      // which in turn erases the display. If new text is to be displayed before the timer has
      // expired, the timer is reset.
      if (tid_ == 0) {
         tid_ = SetTimer(NULL,
                         0,            // timer identifier (ignored since hwnd is NULL)
                         TIMER_VAL,    // time-out value (ms)
                         CCTimerProc   // address of timer procedure
                         );
         me = this ;
      } else {
         KillTimer(NULL,tid_) ;
         tid_ = SetTimer(NULL,
                         0,            // timer identifier (ignored since hwnd is NULL)
                         TIMER_VAL,    // time-out value (ms)
                         CCTimerProc   // address of timer procedure
                         );
         me = this ;
     }

      DebugPrint((1, "Updating display (%lu)\n", gctr++));
      #ifdef   FORCE_PRIMARY_ROLLUP
      ReadyToDisplay_ = (bOverlayMode) ? ReadyToDisplay_ : false;
      #endif
   }
}

/* Method: CCDisplay::ProcessChar
 * Purpose: Handles a printing character
 */
bool CCDisplay::ProcessChar( char ch )
{
   if ( ch == 0 ||
        wStreamMode_ == CCMODE_TEXT1 ||
        wStreamMode_ == CCMODE_TEXT2 ||
        wCapType_ != wStreamMode_ )
      return false;

   // our decoder sets the bit 0x80 in the event of a parity error
   if (ch & 0x80) {
      DebugPrint((0, "character parity error\n"));
      // characters failing parity should be displayed as 0x7F
      ch = 0x7F;
   }

   if ( ch == 0x7C ) //division sign
      ch = 247;
   AddChar( ch );
   return true;
}


/* Method: CCDisplay::ProcessPreamble
 * Purpose: Processes the PAC
*/
bool CCDisplay::ProcessPreamble( char NonDisp, char Disp )
{
   if ( wStreamMode_ == CCMODE_TEXT1 || wStreamMode_ == CCMODE_TEXT2 )
      return false;

   if ( wCapType_ != ( ( NonDisp & 0x08 ) ? CCMODE_CC2 : CCMODE_CC1 ) )
      return false;

   WasPAC_ = true;

   WORD wControl = ( (WORD)Disp << 8 ) | NonDisp;

   if ( wLastControl_ == wControl ) {
      wLastControl_ = 0;
      return false;
   }
   wLastControl_ = wControl;

   int OldRow = wRowIndex_;

   // calculate the row. It is defined by bits 0-2 of nondisp byte and bit 5 of disp byte
   // bit 5 of disp byte increments row count by one
   wRowIndex_ = NonDisp & 7;
   // open up range, so there are different values for different row numbers,
   // i.e. non disp == 0x11 + disp == 0x60 can be separated from
   //      non disp == 0x12 + disp == 0x40.
   switch ( wRowIndex_ ) {
   case 5:
      wRowIndex_ = 6;
      break;
   case 6:
      wRowIndex_ = 8;
      break;
   case 7:
      wRowIndex_ = 10;
      break;
   case 0:
      wRowIndex_ = 12;
      break;
   case 3:
      wRowIndex_ += 10;
      break;
   case 4:
      wRowIndex_ += 11;
      break;
   default:
      wRowIndex_ *= 2;
      break;
   }
   wRowIndex_ += ( Disp & 0x20 ) ? 1 : 0;
   wRowIndex_ -= 2; // zero-base the index

   DebugAssert(((wRowIndex_ < 15), "wRowIndex_ = %x\n", wRowIndex_));

   bool WantRepaint = false;

   if ( OldRow != wRowIndex_ && wCapStyle_ != RollUp )
      WriteCR();

   if ( wCapStyle_ == RollUp && OldRow != wRowIndex_ ) {

      if ( OldRow > wRowIndex_ ) {
         for ( int i = 0, CurIdx = wRowIndex_ - wRUCount_, OldIdx = OldRow - wRUCount_;
            i <= wRUCount_ && CurIdx >= 0 && OldIdx >= 0; i++ ) {

            for (int col = 0; col < MAX_COL; col++) {
               CCScreenArray[CurIdx][col].Copy(&(CCScreenArray[OldIdx][col]));
               CCScreenArray[OldIdx][col].Erase();
            }

            CurIdx++;
            OldIdx++;
         }
      } else {
         if ( OldRow ) // prevent destruction of the second row
            for ( int i = 0, CurIdx = wRowIndex_, OldIdx = OldRow;
               i <= wRUCount_ && CurIdx >= 0 && OldIdx >= 0; i++, OldIdx--, CurIdx-- ) {

               for (int col = 0; col < MAX_COL; col++) {
                  CCScreenArray[CurIdx][col].Copy(&(CCScreenArray[OldIdx][col]));
                  CCScreenArray[OldIdx][col].Erase();
               }
            }
      }
      WantRepaint = true;
   }

   row_ = wRowIndex_;

   if ( Disp & IndentBit ) { // Account for indent 4, 8, 12, ... 28
      column_ = (Disp & 0x0E) * 2;
   } else
      // remember color; italics and underline ( if any )
      SetAttrib( Disp & 0x0F );
   return WantRepaint;
}

/* Method: CCDisplay::ProcessMidrow
 * Purpose:
*/
bool CCDisplay::ProcessMidrow( char NonDisp, char Disp )
{
   WORD wControl;

   if ( wStreamMode_ == CCMODE_TEXT1 || wStreamMode_ == CCMODE_TEXT2 )
      return false;

   if ( wStreamMode_ != wCapType_ )
      return false;
   if ( wCapType_ != ( ( NonDisp & 0x08 ) ? CCMODE_CC2 : CCMODE_CC1 ) )
      return false;

   WasPAC_ = false;

   wControl  = (WORD)( Disp & 0x7F );
   wControl += (WORD)( ( NonDisp & 0x7F ) << 8 );

   if ( wLastControl_ == wControl ) {
      wLastControl_ = 0;
      return false;
   }
   wLastControl_ = wControl;

   SetAttrib( Disp & 0x0F );
   AddChar( ' ' );
   return true;
}

/* Method:
 * Purpose:
*/
bool CCDisplay::ProcessMisc( char NonDisp, char Disp )
{
   int col = 0;
   if ( wCapType_ != ( ( NonDisp & 0x08 ) ? CCMODE_CC2 : CCMODE_CC1 ) ) {
      return false;
   }
   WORD wControl;
   WasPAC_ = false;

   wControl = ( (WORD)Disp << 8 ) | NonDisp;

   if ( wLastControl_ == wControl ) {
      wLastControl_ = 0;
      return false;
   }
   BYTE Command = Disp & 0xF;

   wLastControl_ = wControl;

   int chan = NonDisp & 0x08 ? CCMODE_CC2 : CCMODE_CC1;

   switch ( Command ) {
   case ERASE_DISPLAYED_MEM:
      if ( wCapStyle_ == RollUp || wCapStyle_ == PopOn ) {
         ReadyToDisplay_ = false;
      } else {
         ReadyToDisplay_ = true;
         #ifdef   FORCE_PRIMARY_ROLLUP
         ReadyToDisplay_ = (bOverlayMode) ? ReadyToDisplay_ : false;
         #endif
      }

      mru_hfont = hFont;
      mru_color = CR_WHITE;

      if ( wCapStyle_ == RollUp || wCapStyle_ == PaintOn )
         Erase();
      if ( wCapStyle_ == PopOn )
         WriteCR();
      break;
   case RESUME_CAP_LOAD:
      wStreamMode_ = chan;
      if ( wCapType_ != wStreamMode_ )
         break;
      wCapStyle_ = PopOn; // going to be doing Pop-on type of captioning
      ReadyToDisplay_ = false;
      wRUCount_ = 0;      // don't do scrolling anymore
      Erase();
      break;
   case BACKSPACE:
      if ( wCapType_ != wStreamMode_ )
         break;
      if ( wCapStyle_ == PopOn )
         return false;
      BackSpace();
      break;
   case DELETE_TO_END:
      if ( wCapType_ != wStreamMode_ )
         break;
      if ( wCapStyle_ == PopOn )
         return false;

      mru_hfont = hFont;
      mru_color = CR_WHITE;

      // DeleteToEnd
      for (col = column_; col < MAX_COL; col++) {
         CCScreenArray[row_][col].Erase();
      }
      break;
   case ROLLUP_4ROWS:
   case ROLLUP_3ROWS:
   case ROLLUP_2ROWS:
      ReadyToDisplay_ = true;
      #ifdef   FORCE_PRIMARY_ROLLUP
      ReadyToDisplay_ = (bOverlayMode) ? ReadyToDisplay_ : false;
      #endif
      wStreamMode_ = chan;
      if ( wCapType_ != wStreamMode_ )
         break;
      if ( wCapStyle_ != RollUp ) {
         EraseDisplay();

         // default to row 15
         wRowIndex_ = PRE_ROW15;
         row_ = PRE_ROW15;
      }
      column_ = 0;

      mru_hfont = hFont;
      mru_color = CR_WHITE;

      wRUCount_ = Command - 4; // codes go 5,6,7
      wCapStyle_ = RollUp;

      break;
   case RESUME_DIRECT_CAP:
      wStreamMode_ = chan;
      if ( wCapType_ != wStreamMode_ )
         break;

      mru_hfont = hFont;
      mru_color = CR_WHITE;

      wCapStyle_ = PaintOn;
      wRUCount_ = 0;      // don't do scrolling anymore
      ReadyToDisplay_ = true;
      #ifdef   FORCE_PRIMARY_ROLLUP
      ReadyToDisplay_ = (bOverlayMode) ? ReadyToDisplay_ : false;
      #endif

      break;
   case TEXT_RESTART:  // tbd
      wStreamMode_ = ( chan == CCMODE_CC1 ? CCMODE_TEXT1 : CCMODE_TEXT2 );
      break;
   case RESUME_TEXT_DISPLAY:  // tbd
      wStreamMode_ = ( chan == CCMODE_CC1 ? CCMODE_TEXT1 : CCMODE_TEXT2 );
      break;
   case CARRIAGE_RETURN:
      column_ = 0;
      if ( wCapStyle_ != RollUp ) {

         mru_hfont = hFont;
         mru_color = CR_WHITE;

         #ifdef   FORCE_PRIMARY_ROLLUP
         ReadyToDisplay_ = (bOverlayMode) ? ReadyToDisplay_ : true;
         #endif
         break;
      }

      ReadyToDisplay_ = true;

      {
      // do the upwards scrolling
      for ( int i = wRUCount_; i > 0; i-- )
         if ( wRowIndex_ >= i ) { // avoid array underruns
            for (int col = 0; col < MAX_COL; col++) {
               CCScreenArray[wRowIndex_ - i][col].Copy(&(CCScreenArray[wRowIndex_ - i + 1][col]));
               CCScreenArray[wRowIndex_ - i + 1][col].Erase();
            }
         }
      }
      WriteCR();
      bGotFirstCR_ = true ;
      break;
   case EOC_FLIP_MEMORY:
      ReadyToDisplay_ = true;
      if ( wCapStyle_ == RollUp )
         Erase();
      wCapStyle_ = PopOn;
      break;
   case ERASE_NONDISP_MEM:

      mru_hfont = hFont;
      mru_color = CR_WHITE;

      if ( wCapStyle_ == PopOn ) {
         Erase();
//         InvalidateRect( hWnd_, NULL, TRUE );
      }
      break;
   case FLASH_ON:
      AddChar( ' ' );
      break;
   }
   return true;
}

/* Method:
 * Purpose:
*/

void CCDisplay::Init( HWND win )
{
   memset( alarm_, '\0', sizeof( alarm_ ) );
   strcpy( alert_, "Brooktree" );
   strcpy( file_, "cc.txt" );

   if ( !win || !IsWindow( win ) )
      return;

   if ( PiscesEntry_.IsInited() == false ) {
      PiscesEntry_.LoadVxD();
      if ( PiscesEntry_.IsInited() == false )
      {
         DebugPrint((0, "Failed to load vxd\n")) ;
         return ;
      }
      PiscesEntry_.CallVxD( FNVer, 0 );
   }

   hWnd_ = win;
   row_ = 0;
   column_ = 0;
   NeedToDisplay_ = false;
   bGotFirstCR_ = false ;

   hdcScreen = GetDC( hWnd_ );

   SIZE size;
   RECT clientRect;

   GetClientRect( hWnd_, &clientRect );
   size.cx = clientRect.right - clientRect.left;
   size.cy = clientRect.bottom - clientRect.top;

   hbrColorKeyBrush = CreateSolidBrush(RGB(255,0,255));
   hbrOpaqueBrush = CreateSolidBrush(RGB(0,0,0));

   CreateFonts(true);

   mru_hfont = hFont;
   mru_color = CR_WHITE;

   HDC hdc = GetDC( hWnd_ );
   SelectObject( hdc, hFontItalic );
   GetTextWidth( hdc, wWidth, wHeight );
   SelectObject( hdc, hFont );

   // working variable
   DWORD dwTmp = 0;
   bOverlayMode = true;
   bAllowCC = false;

   RegUtil Storage( "SOFTWARE\\Brooktree\\Video" );
   // get Registry key to check if we are in primary surface mode or not
   //      1 if primary surface
   //      0 if overlay surface
   if ( Storage.GetValue( &dwTmp, "Surface" ) == Success ) {
      if ( dwTmp == 1 ) {
         bOverlayMode = false;
      }
   }

   // get Registry keys to determine if closed caption display is allowed in non
   // overlay modes
   // Registry key "KillPrimaryCC" = 1              -> Disable CC for non overlay
   //                              = 0 or not exist -> Enable CC for non overlay
   //
   // Registry key "KillPrimaryCCFull" = 1              -> Disable CC for non overlay full screen
   //                                  = 0 or not exist -> Enable CC for non overlay  full screen
   //   (note that KillPrimaryCC must be non existant or 0 for Full Screen CC Primary surface)

   bAllowCC = true;  // always assume we allow CC

   if (!bOverlayMode)
   {
       bool bFullScreen =  ( size.cy <= 240 ) ? false : true ; 
       bool bKillPrimCC = false ;
       bool bKillFullScreenPrimCC = false ;

       if ( Storage.GetValue( &dwTmp, "KillPrimaryCC" ) == Success ) 
          if ( dwTmp == 1 )  {
             bKillPrimCC = true ;
             DebugPrint((0,"KillPrimaryCC is set\n")) ;
          }

       if ( Storage.GetValue( &dwTmp, "KillPrimaryCCFull" ) == Success ) 
          if ( dwTmp == 1 ) {
             bKillFullScreenPrimCC = true ;
             DebugPrint((0,"KillPrimaryCCFull is set\n")) ;
          }

       if (bKillPrimCC || (bFullScreen && bKillFullScreenPrimCC))
          bAllowCC = false ;
   }

   DebugPrint((0,"CC is%sAllowed\n",bAllowCC ? " ":" *NOT* ")) ;

   if (bOverlayMode) {
      DebugPrint((0, "Overlay Mode\n"));
   } else {
      DebugPrint((0, "Non Overlay Mode\n"));
   }

   if (bAllowCC) {
      DebugPrint((0, "Closed caption display allowed\n"));
   } else {
      DebugPrint((0, "Closed caption display NOT allowed\n"));
   }

   DebugPrint((1, "this ptr (%x)\n", this));

   for (int row = 0; row < MAX_ROW; row++) {
      for (int col = 0; col < MAX_COL; col++) {
         CCScreenArray[row][col].Init(0, row, col, this);
      }
   }

   Erase();
   SetClipping();
}

/* Method: CCDisplay::CreateFonts
 * Purpose:
 */
void CCDisplay::CreateFonts(bool bDoingInit)
{
   if (!bAllowCC && !bDoingInit) {
      return;
   }
   TRect rc;
   WORD wWidthCreate, wHeightCreate;

   DeleteFonts();

   GetClientRect( hWnd_, &rc );

   wWidthCreate   = rc.Width() * 8 / 320;

   wYOffset = rc.Height() * 10 / 120;
   wXOffset = ( rc.Width() - wWidthCreate * 34 ) / 2 ;


   wHeightCreate = rc.Height() * 4 / 80;

   wWidthCreate  = -wWidthCreate;
   wHeightCreate = -wHeightCreate;

   /*
   hFont = CreateFont( wHeightCreate, wWidthCreate, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
      FIXED_PITCH|FF_SWISS, (LPCSTR)NULL );
   hFontItalic = CreateFont( wHeightCreate, wWidthCreate , 0, 0, 0, TRUE, 0, 0, 0, 0, 0, 0,
      FIXED_PITCH|FF_SWISS, (LPCSTR)NULL );
   hFontUnder = CreateFont( wHeightCreate, wWidthCreate, 0, 0, 0, 0, TRUE, 0, 0, 0, 0, 0,
      FIXED_PITCH|FF_SWISS, (LPCSTR)NULL );
   hFontItalUnder = CreateFont( wHeightCreate, wWidthCreate, 0, 0, 0, TRUE, TRUE, 0, 0, 0, 0, 0,
      FIXED_PITCH|FF_SWISS, (LPCSTR)NULL );
   */

   /*
   hFont = CreateFont(
      wHeightCreate,    // height
      wWidthCreate,     // width
      0,                // escapement
      0,                // orientation
      0,                // weight
      0,                // italic
      0,                // underline
      0,                // strikeout
      0,                // char set
      OUT_TT_ONLY_PRECIS,           // output precision
      0,                            // clipping precision
      0,                            // output quality
      DEFAULT_PITCH | FF_DONTCARE,  // pitch and family
      "Arial"                       // typeface name
   );
   */

   hFont = CreateFont( wHeightCreate, wWidthCreate, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
      FIXED_PITCH|FF_SWISS, (LPCSTR)NULL );
   hFontItalic = CreateFont( wHeightCreate, wWidthCreate , 0, 0, 0, TRUE, 0, 0, 0, 0, 0, 0,
      FIXED_PITCH|FF_SWISS, (LPCSTR)NULL );
   hFontUnder = CreateFont( wHeightCreate, wWidthCreate, 0, 0, 0, 0, TRUE, 0, 0, 0, 0, 0,
      FIXED_PITCH|FF_SWISS, (LPCSTR)NULL );
   hFontItalUnder = CreateFont( wHeightCreate, wWidthCreate, 0, 0, 0, TRUE, TRUE, 0, 0, 0, 0, 0,
      FIXED_PITCH|FF_SWISS, (LPCSTR)NULL );

   // create the bitmaps
   BITMAP bm;
   hbmMusicNote = LoadBitmap( GetOurInstance(), MAKEINTRESOURCE( MusicNote ) );
   GetObject( hbmMusicNote, sizeof( BITMAP ), &bm );
   wMusicWidth = bm.bmWidth;
   wMusicHeight = bm.bmHeight;

   hbmTradeMark = LoadBitmap( GetOurInstance(), MAKEINTRESOURCE( TradeMark ) );
}

/* Method: CCDisplay::Erase
 * Purpose: Erases all rows
 */
void CCDisplay::Erase()
{
   if (!bAllowCC) {
      return;
   }
   for (int row = 0; row < MAX_ROW; row++) {
      for (int col = 0; col < MAX_COL; col++) {
         CCScreenArray[row][col].Erase();
      }
   }
   ReadyToDisplay_ = true ;
}

/* Method: CCDisplay::ProcessTabOffset
 * Purpose: Erases all rows
 */
void CCDisplay::ProcessTabOffset( int code )
{
   if ( wCapType_ != ( ( code & 0x08 ) ? CCMODE_CC2 : CCMODE_CC1 ) )
      return;

   int Offs = ( code & 0x0300 ) >> 8;

   if ( Offs > 3 )
      Offs = 3;

   if ( wLastControl_ == code ) {//|| !WasPAC_ ) {
      wLastControl_ = 0;
      return;
   }
   wLastControl_ = code;

   column_ += Offs;
   column_ = min(column_, (MAX_COL - 1));
}
void GetTextWidth( HDC hdc, WORD &w, WORD &h )
{
#ifdef __WIN32__
   SIZE size;
   GetTextExtentPoint32( hdc, "W", 1, &size );
   w = size.cx;
   h = size.cy;

#else

   DWORD dwExtent = GetTextExtent( hdc, "W", 1 );
   h = HIWORD( dwExtent );
   w = LOWORD( dwExtent );

#endif
}

CCDisplay::CCDisplay() : wCapStyle_( RDC ), wLastControl_( 0 ),
   hWnd_( NULL ), ReadyToDisplay_( false ), hFont( NULL ), hFontItalic( NULL ),
   hFontUnder( NULL ), hFontItalUnder( NULL ), wStreamMode_( 0 ), fp( NULL ),
   wCapType_( CCMODE_CC1 ), wRowIndex_( PRE_ROW15 ), AlertMatch_( false ), HaveAlert_( true ), tid_(0)
{
   row_ = wRowIndex_;

   memset( alarm_, '\0', sizeof( alarm_ ) );
   strcpy( alert_, "Brooktree" );
   strcpy( file_, "cc.txt" );

   ClipRgn = (RGNDATA*)&ClipRgnMem;
   ClipRect = (LRECT*)&(((RGNDATA*)&ClipRgnMem)->Buffer);
}

/* Method: CCDisplay::DeleteFonts
 * Purpose: Deletes all the GDI objects
 */
void CCDisplay::DeleteFonts()
{
   if ( hFont )          DeleteObject( hFont );
   if ( hFontItalic )    DeleteObject( hFontItalic );
   if ( hFontUnder )     DeleteObject( hFontUnder );
   if ( hFontItalUnder ) DeleteObject( hFontItalUnder );
   if ( hbmMusicNote )   DeleteObject( hbmMusicNote );
   if ( hbmTradeMark  )  DeleteObject( hbmTradeMark );

   hFont = NULL;
   hFontItalic = NULL;
   hFontUnder = NULL;
   hFontItalUnder = NULL;
   hbmMusicNote = NULL;
   hbmTradeMark = NULL;
}

CCDisplay::~CCDisplay()
{
   me = NULL ;
   if (tid_)
      KillTimer(NULL, tid_) ;
   
   DeleteFonts();
   DeleteObject(hbrColorKeyBrush);
   ReleaseDC(hWnd_, hdcScreen);
}

void CCDisplay::SetChannel( int ch )
{
   if (!bAllowCC) {
      return;
   }
   if ( wCapType_ != ch ) {
      EraseDisplay();
      wCapType_ = ch;
   }
}

/* Method:
 * Purpose: Takes care of processing special characters
 */
bool CCDisplay::ProcessSChar( char NonDisp, char Disp )
{
   WORD wControl;
   WORD wChannel;

   wChannel = (WORD)((NonDisp & 0x08) ? CCMODE_CC2 : CCMODE_CC1);

   wControl = (WORD)((NonDisp & 0x77) << 8);
   wControl |= (WORD)(Disp & 0x7F);

   if ( wLastControl_ == wControl ) {
      wLastControl_ = 0;
      return false;
   }
   wLastControl_ = wControl;

   if ( wStreamMode_ == CCMODE_TEXT1 || wStreamMode_ == CCMODE_TEXT2 ||
        wCapType_ != wChannel )
      return false;

      BYTE SpecChar = ' ';

      switch ( Disp & 0x0F ) {
      case SC0:  // Registered mark symbol
         SpecChar = 174;
         break;
      case SC1:  // Degree sign
         SpecChar = 176;
         break;
      case SC2:  // 1/2 symbol
         SpecChar = 189;
         break;
      case SC3:  // Inverse query
         SpecChar = 191;
         break;
      case SC4:  // Trademark symbol
         SpecChar = SC4;
         break;
      case SC5:  // Cents symbol
         SpecChar = 162;
         break;
      case SC6:  // Pounds Sterling symbol
         SpecChar = 163;
         break;
      case SC7:  // Music note
         SpecChar = SC7;
         break;
      case SC8:  // Lower-case a with grave accent
         SpecChar = 224;
         break;
      case SC9:  // Transparent Space, do nothing
         break;
      case SCA:  // Lower-case e with grave accent
         SpecChar = 232;
         break;
      case SCB:  // Lower-case a with circumflex
         SpecChar = 226;
         break;
      case SCC:  // Lower-case e with circumflex
         SpecChar = 234;
         break;
      case SCD:  // Lower-case i with circumflex
         SpecChar = 238;
         break;
      case SCE:  // Lower-case o with circumflex
         SpecChar = 244;
         break;
      case SCF:  // Lower-case u with circumflex
         SpecChar = 252;
         break;
      }

   AddChar( SpecChar );
   return true;
}

/* Method: CCDIsplay::ShowBitmap
 * Purpose:
 */
void CCDisplay::ShowBitmap( HDC hdc, HBITMAP hBM, WORD x, WORD y, WORD w, WORD h )
{
   HDC hMemDC;
   HBITMAP hbmOld;

   hMemDC = CreateCompatibleDC( hdc );
   hbmOld = (HBITMAP)SelectObject( hMemDC, hBM );

   StretchBlt( hdc, x, y, w, h, hMemDC, 0, 0, wMusicWidth, wMusicHeight, SRCCOPY );

   SelectObject( hMemDC, hbmOld );
   DeleteDC( hMemDC );

   return;
}

/* Method: CCDisplay::BackSpace
 * Purpose: Deletes the previous character on the current line.
 */
void CCDisplay::BackSpace()
{
   if ( column_ > 0 ) {
      column_--;
      CCScreenArray[row_][column_].Erase();
   }
}

/* Method: CCDisplay::AddChar
 * Purpose: Writes a character to the CC file
 */
void CCDisplay::AddChar( char ch )
{
   if (ch == 0) {
      return;
   }

   DebugAssert((row_ < MAX_ROW, "row = %u\n", row_));
   DebugAssert((column_ < MAX_COL, "column = %u\n", column_));

   CCScreenArray[row_][column_].Replace(ch);

   if ( column_ < (MAX_COL - 1))
      column_++;

   if ( fp )
      fwrite( &ch, 1, 1, fp );

   if ( HaveAlert_ )
      if ( ch == ' ' ) {
         if ( !strncmpi( alarm_, alert_, strlen( alert_ ) ) )
            AlertMatch_ = true;
         memset( alarm_, '\0', sizeof( alarm_ ) );
      } else
         alarm_ [ min( strlen( alarm_ ), sizeof( alarm_ ) - 1 ) ] = ch;
}

/* Method: CCDisplay::WriteCR
 * Purpose: Writes the carriage return into the file
 */
void CCDisplay::WriteCR()
{
   if ( fp )
      fputc( '\n', fp );
}

void CCDisplay::SetAttrib(char Attr)
{
   mru_hfont = hFont;

   switch ( Attr & 0x0E ) {
      case CC_GREEN:
         mru_color = CR_BGREEN;
         break;
      case CC_BLUE:
         mru_color = CR_BBLUE;
         break;
      case CC_CYAN:
         mru_color = CR_BCYAN;
         break;
      case CC_RED:
         mru_color = CR_BRED;
         break;
      case CC_YELLOW:
         mru_color = CR_BYELLOW;
         break;
      case CC_MAGENTA:
         mru_color = RGB( 0xA0,0x00,0xA0 );
         // CR_DMAGENTA; // use dark as our key color is bright magenta
         break;
      case CC_ITALICS:
         mru_hfont = hFontItalic;
         mru_color = CR_WHITE;
         break;
      default:
         mru_hfont = hFont;
         mru_color = CR_WHITE;
         break;
   }
}

bool IsEmpty( LPCSTR str )
{
   // skip leading spaces
   int i = 0, len = strlen ( str );
   while ( i < len ) {
      if ( !isspace( str [i] ) )
         break;
      i++;
   }
   if ( i == len )
      return true;

   return false;
}

/* Method: CCDisplay::SetCCFileName
 * Purpose: Opens a file handle to write CC text into
 */
void CCDisplay::SetCCFileName( LPCSTR name )
{
   // close file if exists and if names are different only
   if ( fp && strcmp( name, file_ ) )
      fclose( fp );
   // nothing to open, but open spaces
   if ( IsEmpty( name ) ) {
      *file_ = '\0';
      fp = NULL;
   } else {
     fp = fopen( name, "w" );
     strncpy( file_, name, sizeof( file_ ) - 1 );
   }
}

/* Method: CCDisplay::SetCCAlert
 * Purpose: stores the string used to trigger CC alarm
 */
void CCDisplay::SetCCAlert( LPCSTR token )
{
   // do not bother if empty token
   if ( IsEmpty( token ) ) {
      HaveAlert_ = false;
      *alert_ = '\0';
   } else {
      strncpy( alert_, token, sizeof( alert_ ) - 1 );
      HaveAlert_ = true;
   }
}

