/****************************************************************************/
/***** Copyright (c) Brooktree Corporation, 1996. All Rights Reserved *******/
/****************************************************************************/
/*                                                                          */
/*                      BROOKTREE CONFIDENTIAL                              */
/*                                                                          */
/* Filename:    FILTER.CPP                                                  */
/*                                                                          */
/* Description: Closed caption decode IIR Filter                            */
/*                                                                          */
/*                                                                          */
/* Author:      Dave Tharp                                                  */
/*                                                                          */
/****************************************************************************/
/* PVCS Muck:
 * $Header: G:/SwDev/W95/Video/pisces/dvlpmnt/CC/rcs/filter.cpp 1.4 1997/06/27 17:04:30 dtharp Released $
 * $Revision: 1.4 $
 * $Date: 1997/06/27 17:04:30 $
 * $Author: dtharp $
 * $Archive:  $
 * $Log: filter.cpp $
 * Revision 1.4  1997/06/27 17:04:30  dtharp
 * Moved global variables and functions into class in attempt to solve
 * re-entrancy problem.
 * Revision 1.3  1997/04/18 00:36:05  dtharp
 * Fix for Bug Id 433. Closed Caption Performance (speed, cpu).
 * Revision 1.2.1.1  1997/04/15 23:51:47  dtharp
 * Bug id 433: Optimization of CC decoder.
 * Revision 1.2  1997/01/17 01:19:52  dtharp
 * Merged new cc decoder branch.
 * Revision 1.1.1.2  1996/12/10 23:16:29  dtharp
 * New filter implementation: IIR filter.
 *
 */

#ifdef _MFC_VER
  #include "stdafx.h"
    #ifdef _DEBUG
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
  #endif
#endif

#include <mem.h>
#include "filter.hpp"

// 4 Tap IIR Filter

#define X_N           (data[i])
#define Y_N           (yTaps[3])
#define Y_N_MINUS_1   (yTaps[2])
#define Y_N_MINUS_2   (yTaps[1])
#define Y_N_MINUS_3   (yTaps[0])


static MathType coef[N_TAPS] = {8, 2624, -2268, 660} ;

void Filter::Apply(BYTE * data, int nPoints) 
{

  for (DWORD i = 0; i<(DWORD)nPoints; i++)
  {
    memcpy(&yTaps[0],&yTaps[1], sizeof(MathType) * (N_TAPS-1)) ;

    MathType pt = (coef[0] * (MathType) X_N) +
                (coef[1] * Y_N_MINUS_1)  +
                (coef[2] * Y_N_MINUS_2)  +
                (coef[3] * Y_N_MINUS_3) ;

    Y_N = (pt/1024) ;

    data[i] = (BYTE) Y_N ;
  }
}
// EOF