/* File : ColorSpace.cpp
 * Purpose: Contains implementation of the ColorSpace class
 * Author: sasha
 * $Log: ColSpace.cpp $
 * Revision 2.2  1996/10/03 20:59:33  sasha
 * Moved FOURCC definitions into fourcc.h.
 * Revision 2.1  1996/06/10 20:12:17  ssm
 * MarkBeta1AsRev2
 * Revision 1.11  1996/03/18 23:21:18  sasha
 * Added CF_UYVY color format. Changed width restrictions
 * Revision 1.10  1996/02/19 01:23:20  sasha
 * Removed one dummy and moved CF_VBI in its place in the arrays
 * Revision 1.9  1996/02/01 04:27:56  sasha
 * Removed dummy from arrays
 * Revision 1.8  1996/01/11 23:57:25  sasha
 * Removed mmsystem.h and just copied mmioFOURCC definition from it
 * Revision 1.7  1996/01/06 02:05:11  sasha
 * Changed ColorFormat to ColFmt
 * Revision 1.6  1995/12/22 19:17:39  sasha
 * Added YPlaneBitCount_ array and changed the way GetPitchBpp works.
 * Apparently, it had a bug for planar modes with 16 bpp
 * Revision 1.5  1995/10/13 07:44:52  sasha
 * First major check-in for testing
 * Revision 1.4  1995/10/12 23:44:52  sasha
 * Fixed values for the YUV12 horiz and vert alignment
 * Revision 1.3  1995/09/20 05:35:42  sasha
 * Played with #includes and #defines
 * Revision 1.2  1995/09/07 01:11:14  sasha
 * Start-up version, really
 * Revision 1.1  1995/08/19 02:54:26  SASHA
 * Initial revision
 */

#include "colspace.h"
#include "fourcc.h"
// copied from mmsystem.h
#define mmioFOURCC( ch0, ch1, ch2, ch3 )                                \
                ( (DWORD)(BYTE)(ch0) | ( (DWORD)(BYTE)(ch1) << 8 ) |    \
                ( (DWORD)(BYTE)(ch2) << 16 ) | ( (DWORD)(BYTE)(ch3) << 24 ) )


BYTE const ColorSpace::BitCount_ [] =
{
// RGB32 RGB24 RGB16 RGB15 YUY2 BTYUV Y8 RGB8 PL_422 PL_411 YUV9 YUV12 VBI UYVY RAW I420
   32,   24,   16,   16,   16,  12,   8, 8,   16,    12,    9,   12,   16, 16,  8,  12
};

BYTE const ColorSpace::YPlaneBitCount_ [] =
{
// RGB32 RGB24 RGB16 RGB15 YUY2 BTYUV Y8 RGB8 PL_422 PL_411 YUV9 YUV12 VBI UYVY RAW I420
   32,   24,   16,   16,   16,  12,   8, 8,   8,     8,     8,   8,    16, 16,  8,  8
};

BYTE const ColorSpace::XRestriction_ [] =
{
   1,    1,    1,    1,   2,    4,    1, 1,   8,     16,    16,  8,    2,  4,   1,  8
};

BYTE const ColorSpace::YRestriction_ [] =
{
   1,    1,    1,    1,   1,    1,    1, 1,   1,     1,     4,    2,   1,  1,   1,  2
};

FOURCC const ColorSpace::FourccArr_ [] =
{
// 32      24      16            15
   BI_RGB, BI_RGB, BI_BITFIELDS, BI_RGB, FCC_YUY2, FCC_Y41P, FCC_Y8, BI_RGB,
   FCC_422, FCC_411, FCC_YVU9, FCC_YV12, FCC_VBI, FCC_UYVY, FCC_RAW, FCC_I420
};

/* Constructor: ColorSpace::ColorSpace
 * Input: fcc: FOURCC
 */
ColorSpace::ColorSpace( FOURCC fcc, int bitCount ) : CurColor_( CF_BelowRange )
{
   if ( fcc == BI_RGB )
      switch ( bitCount ) {
      case 8:
         CurColor_ = CF_RGB8;
         break;
      case 16:
         CurColor_ = CF_RGB15;
         break;
      case 24:
         CurColor_ = CF_RGB24;
         break;
      case 32:
         CurColor_ = CF_RGB32;
         break;
      }
   else
      for ( ColFmt fccArrIdx = CF_RGB32; fccArrIdx < CF_AboveRange; fccArrIdx++ )
         if ( fcc == FourccArr_ [fccArrIdx] ) {
            CurColor_ = fccArrIdx;
            break;
         }
}

/* Method: ColorSpace::CheckDimentions
 * Purpose: This functions checks that the size of a buffer corresponds to the
 *   restrictions imposed by a color format
 * Input: size: reference to SIZE structure
 * Output: bool: True or False
 */
bool ColorSpace::CheckDimentions( const SIZE &size ) const
{
   return bool( IsDivisible( size.cx, XRestriction_ [CurColor_] ) &&
                IsDivisible( size.cy, YRestriction_ [CurColor_] ) );
}
