/* File: PsColSpc.cpp
 * Purpose: Defines PsColorSpace class
 * Author: sasha
 * $Log: pscolspc.cpp $
 * Revision 2.1  1996/06/10 20:12:18  ssm
 * MarkBeta1AsRev2
 * Revision 1.5  1996/03/18 23:23:32  sasha
 * Added support for the UYVY color format.
 * Revision 1.4  1996/02/01 04:27:11  sasha
 * Check for YUV9 and YUV12 color formats so a proper Pisces value is
 * written to registers
 * Revision 1.3  1996/01/06 02:05:13  sasha
 * Changed ColorFormat to ColFmt
 * Revision 1.2  1995/09/27 07:36:52  sasha
 * First major check-in for testing
 * Revision 1.1  1995/09/26 23:36:52  sasha
 * Initial revision
 */

#include "pscolspc.h"

/* Method: PsColorSpace::SetColorFormat
 * Purpose: Sets BtPisces color space converter to the given color
 * Input: eaColor: enum of type ColFmt
 * Output: None
 * Note: No error checking is done ( enum is checked by the compiler during compile
 *   The function writes to the XXXX register
*/
void PsColorSpace::SetColorFormat( ColFmt eColor )
{
   // save for later use...
   ColorSpace::SetColorFormat( eColor );
   ByteSwap_ = 0;
 
   if ( eColor == CF_YUV9 )
      eColor = CF_PL_411;
   else
      if ( eColor == CF_YUV12 || eColor == CF_I420 )
         eColor = CF_PL_422;
      else
         if ( eColor == CF_UYVY ) {
            eColor = CF_YUY2;
            ByteSwap_ = 1;
         }
   // set the hardware
   Format_ = eColor;
}

