/* File: Ddi.h
 * Purpose: Definition of the DDI class
 * Author: sasha
 */

#ifndef __DDI_H
#define __DDI_H


#ifndef __WINDOWS_H
#include <windows.h>
#endif

#ifdef __BORLANDC__
#include "mytypes.h"
#else  // Microsoft
#include "mstypes.h"
#endif

#ifndef __REGUTIL_H
#include "regutil.h"
#endif

#include "ddraw.h"
#include "ddi_par.h"
#include "ddiflags.h"

#include "dbg.h"

#define ClearMem( a ) ZeroMemory( &##a, sizeof( a ) )


class PS_DDCAPS : public DDCAPS
{
   public:
      PS_DDCAPS() { ClearMem( *this ); dwSize = sizeof( DDCAPS ); }
};

class PS_DDSURFACEDESC : public DDSURFACEDESC
{
   public:
      PS_DDSURFACEDESC() { ClearMem( *this ); dwSize = sizeof( DDSURFACEDESC );}
};

/* Class: DDI
 * Purpose: Encapsulates the direct draw
 */
class DDI
{
   public:
   LPDIRECTDRAW        pDirectDrawObj_;
   LPDIRECTDRAWSURFACE pOnScreenBuf_;
   LPDIRECTDRAWSURFACE pOverlaySurf_;
   LPDIRECTDRAWCLIPPER pClipper_;
   IDirectDrawPalette  *pPal_;
   COLORREF            ColorKey_;

   bool  CanOverlay_;
   bool  IsOverlaying_;
   bool  OverlayCanStretch_;
   DWORD dwSurfAddr_;
   DWORD dwPitch_;
   DWORD FourCC_;
   DWORD dwBpp_;
   DWORD dwMinScale_;
   HWND  hWin_;
   DWORD OnScreenWidth_;
   DWORD OnScreenHeight_;
   int OldWidth_;
   int OldHeight_;
   DWORD CurVidWidth_;
   DWORD CurVidHeight_;

      ErrorCode CheckCaps();

      ErrorCode Init();
      void      Release();
      ErrorCode CreatePrimSurface( COLORREF KeyColor );
      void      CreatePalette( LPPALETTEENTRY lpPal );
      void      InitSurfaceDesc(PS_DDSURFACEDESC &ddsd, TRect &Client, DWORD dwScaleFactor);
      ErrorCode CreateOverlay( bool test = false, int width = 0, int height = 0 );
      ErrorCode GetSurfacePtr( LPDIRECTDRAWSURFACE pSurf, DDSURFACEDESC &ddsd );

      void      GetData( SURF_DATA &Data );
      void      AttachWindow( HWND hDispWin ) { hWin_ = hDispWin; }
      bool      QueryClip();
      void      DoColorKey( SURF_DATA *const Data );
      DWORD     IsClipListChanged() {
         BOOL flag = FALSE;
         pClipper_->IsClipListChanged( &flag );
         return flag;
      }
      void SetVidSize( DWORD dwWidth, DWORD dwHeight )
         { CurVidWidth_ = dwWidth; CurVidHeight_ = dwHeight; }

      DDI() : pDirectDrawObj_( NULL ), pOnScreenBuf_( NULL ), hWin_( NULL ),
         pClipper_( 0 ), dwSurfAddr_( 0 ), pPal_( NULL ), CanOverlay_( false ),
         pOverlaySurf_( NULL ), FourCC_( BI_RGB ), IsOverlaying_( false ),
         OnScreenWidth_( 0 ), OnScreenHeight_( 0 ), ColorKey_( RGB( 0, 0, 0 ) ),
         OverlayCanStretch_( false ),OldWidth_( 0 ), OldHeight_( 0 ),
         dwMinScale_( 1000 ), CurVidWidth_( 0 ), CurVidHeight_( 0 ) {}

      ~DDI();

   protected:
      static HRESULT FAR PASCAL SurfEnumCallback( LPDIRECTDRAWSURFACE lpDDSurface,
         LPDDSURFACEDESC lpDDSurfaceDesc, LPVOID lpContext );
      void CheckPossibleOverlays();

};


#endif