// $Header: G:/SwDev/W95/Video/pisces/dvlpmnt/Decoder/rcs/DECODER.H 2.9 1998/02/24 22:15:36 rudyl Exp $
/////////////////////////////////////////////////////////////////////////////
// $Log: DECODER.H $
// Revision 2.9  1998/02/24 22:15:36  rudyl
// Added VSFLDALIGH reg field.
// Revision 2.8  1998/01/16 21:58:08  rudyl
// Gemini Rev D: HSFMT changes from 1 to 2 bits.
// Revision 2.7  1997/08/14 19:02:48  rudyl
// Graft from 2.6.1.2. Added Gemini support.
// Revision 2.6.1.2  1997/07/24 00:38:58  rudyl
// Added interfaces to VTOTAL register.
// Revision 2.6.1.1  1997/05/02 23:12:58  rudyl
// Duplicate revision
// Revision 2.5  1996/10/11 00:21:38  meierdw
// Grafting 2.4.1 into trunk; NTSC-Japan support added.
// Revision 2.4.1.2  1996/10/11 00:21:38  rudyl
// Changed registry key to "AltContrast".
// Revision 2.4.1.1  1996/10/11 00:21:38  rudyl
// Bug #: 351
// Problem: hardware / driver incorrectly implements contrast
// Solution: Adjust brightness when changing contrast to compensate the undesirable effect on brightness.
// Added "AdjustContrast" registry key to enable this adjustment; =0 no adjustment (default), =1 enable adjustment
// 
// Also added some Pisces+ codes. Still incomplete. Added #ifdef BT849 around all pisces+ codes.
// Revision 2.3  1996/08/12 22:30:12  rudyl
// Bt848 has white crush up and down regs. Added them temporarily long time ago. Now officiallly in our decoder set.
// Revision 2.2  1996/06/25 22:42:15  rudyl
// Removed ACCEL since it doesn't exist. Also removed functions related to HACTIVE and HSFMT since they should always be 0.
// Revision 2.1  1996/06/10 20:12:24  ssm
// MarkBeta1AsRev2
// Revision 1.11  1996/05/23 22:44:29  rudyl
// Modified calculation of U & V values in SetSaturation. Values were too small compared to default.
// Now also support SECAM. NTSC/PAL have the same U & V values. SECAM's are different.
// Revision 1.10  1996/05/20 23:44:21  rudyl
// Added support of SECAM except Chroma U & V values need to be changed.
// Not calling Scaler::Set525Lines() but calling Scaler::VideoFormatChanged() in SetVideoFormat().
// Revision 1.9  1996/05/02 16:58:31  rudyl
// Removed scaler related registers/codes.
// Revision 1.8  1996/04/18 18:16:54  sasha
// Added    RegisterB   WhiteCrushUp and  RegisterB   WhiteCrushDown
// registers. Probably will be moved into declare.h
// Revision 1.7  1996/04/17 19:32:11  rudyl
// No more Decoder base class. Bt848 has its own decoder class not derived from any other class.
// Revision 1.6  1996/01/29 00:07:48  ssm
// change SetVideoSource arg to type Connector;
// add include viddefs.h
// Revision 1.5  1996/01/19 00:32:11  sasha
// Changed return type of GetRegArrayPtr()
// Revision 1.4  1996/01/06 00:55:49  ssm
// 1st compile with scaler
// Revision 1.3  1996/01/05 19:45:50  ssm
// removed asserts to make vtoolsd happy
// Revision 1.2  1996/01/05 01:06:12  ssm
// 1st successful compile
// Revision 1.1  1996/01/04 22:49:00  ssm
// Initial revision

/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
//
//  Copyright 1996 Brooktree Corporation
//
//  Module:
//
//    Decoder.h
//
//  Abstract:
//
//    Bt848 Decoder Class API Header File
//
/////////////////////////////////////////////////////////////////////////////

#ifndef __DECODER_H
#define __DECODER_H

#ifndef __DEFS_H
#include <classlib/defs.h>
#endif

#include "regField.h"
#include "viddefs.h"
#include "retcode.h"


/////////////////////////////////////////////////////////////////////////////
// CLASS CRegInfo
//
// Description:
//    Provides min, max, and default values for a register. To use this class,
//    user will declare an object of this class and provide min, max and default
//    values of the register.
//
// Attributes:
//    int intMin - minumum value
//    int intMax - maximum value
//    int intDefault - default value
//
// Methods:
//    Min() : return minimum value of the register
//    Max() : return maximum value of the register
//    Default(): return default value of the register
//    OutOfRange() : check if an value is out of range
//
/////////////////////////////////////////////////////////////////////////////
class CRegInfo
{
   int intMin;       // minumum value
   int intMax;       // maximum value
   int intDefault;   // default value

public:
   CRegInfo( void )
   {
      intMin = 0;
      intMax = 0;
      intDefault = 0;
   }

   CRegInfo( int min, int max, int def )
   {
      intMin = min;
      intMax = max;
      intDefault = def;
   }

   // return min, max and default value of a register
   inline int Min( void ) const { return intMin; }
   inline int Max( void ) const { return intMax; }
   inline int Default( void ) const { return intDefault; }

   // check if an value is out of range of a register
   inline bool OutOfRange( int x )
   {
      if( (x > intMax) || (x < intMin) )
         return true;
      return false;
   }
};


/////////////////////////////////////////////////////////////////////////////
// CLASS Decoder
//
// Description:
//    This class encapsulates the register fields in the decoder portion of
//    the Bt848.
//    A complete set of functions are developed to manipulate all the
//    register fields in the decoder for the Bt848.
//    For Read-Write register field, "Set..." function is provided to modify
//    the content of the reigster field. And either "Get..." (for more
//    than 1 bit) or "Is..." (for 1 bit) function is provided to obtain the
//    value of the register field.
//    For Read-Only register field, only "Get..." (for more than 1 bit) or
//    "Is..." (for 1 bit) function is provided to obtain the content of the
//    register field.
//    When there are odd-field complements to the even-field register field,
//    same value is set to both odd and even register fields.
//    Several direct register content modifying/retrieval functions are
//    implemented for direct access to the register contents. They were
//    originally developed for testing purpose only. They are retained in the
//    class for convenience only and usage of these functions must be very cautious.
//
// Methods:
//    See below
//
// Note: 1) Scaling registers are not implemented.
//       2) Odd-fields are set to the same value as the even-field registers
/////////////////////////////////////////////////////////////////////////////

class Decoder
{
protected:
	#include "Declare.h"

   // used for checking if parameter out of register's range
	CRegInfo m_regHue, m_regSaturationNTSC, m_regSaturationSECAM,
            m_regContrast,	m_regBrightness;
   CRegInfo m_regSaturationDig;  // saturation reg value for digital signal

   // used for checking parameter range
   CRegInfo m_param;

   // value set to after calculations
	WORD m_satParam, m_conParam, m_hueParam, m_briParam;

   // to be used to adjust contrast
   int  regBright;      // brightness register value before adjustment
   WORD regContrast;    // contrast register value before adjustment

public:
   // constructor and destructor
	Decoder( void );
	~Decoder();

   // Device Status register (DSTATUS)
   virtual BYTE      GetDeviceStatusReg( void );
   virtual bool      IsVideoPresent( void );
   virtual bool      IsDeviceInHLock( void );
   virtual bool      IsEvenField( void );
   virtual bool      Is525LinesVideo( void );
   virtual bool      IsCrystal0Selected( void );
   virtual bool      IsPLLOutOfLock( void );
   virtual void      ResetPLLOutOfLock( void );
   virtual bool      IsLumaOverflow( void );
   virtual void      ResetLumaOverflow( void );
   virtual bool      IsChromaOverflow( void );
   virtual void      ResetChromaOverflow( void );

   // Input Format register (IFORM)
   virtual ErrorCode SetVideoInput( Connector );
   virtual int       GetVideoInput( void );
   virtual ErrorCode SetCrystal( Crystal );
   virtual int       GetCrystal( void );
   virtual ErrorCode SetVideoFormat( VideoFormat );
   virtual int       GetVideoFormat( void );

   // Temporal Decimation register (TDEC)
   virtual ErrorCode SetRate( bool, VidField, int );

   // Brightness Control register (BRIGHT)
   virtual ErrorCode SetBrightness( int );
   virtual int       GetBrightness( void );

   // Miscellaneous Control register (E_CONTROL, O_CONTROL)
   virtual void      SetLumaNotchFilter( bool );
   virtual bool      IsLumaNotchFilter( void );
   virtual void      SetCompositeVideo( bool );
   virtual bool      IsCompositeVideo( void );
   virtual void      SetLumaDecimation( bool );
   virtual bool      IsLumaDecimation( void );
   virtual void      SetCbFirst( bool );
   virtual bool      IsCbFirst( void );

   // Luma Gain register (CON_MSB, CONTRAST_LO)
   virtual ErrorCode SetContrast( int );
   virtual int       GetContrast( void );

   // Chroma Gain register (SAT_U_MSB, SAT_V_MSB, SAT_U_LO, SAT_V_LO)
   virtual ErrorCode SetSaturation( int );
   virtual int       GetSaturation( void );

   // Hue Control register (HUE)
   virtual ErrorCode SetHue( int );
   virtual int       GetHue( void );

   // SC Loop Control register (E_SCLOOP, O_SCLOOP)
   virtual void      SetChromaAGC( bool );
   virtual bool      IsChromaAGC( void );
   virtual void      SetLowColorAutoRemoval( bool );
   virtual bool      IsLowColorAutoRemoval( void );
   virtual ErrorCode SetHorizontalFilter( HorizFilter );
   virtual int       GetHorizontalFilter( void );
   virtual void      SetLumaLowPassFilter( bool );
   virtual bool      IsLumaLowPassFilter( void );
   virtual void      SetPeakFilter( PeakFilter );
   virtual int       GetPeakFilter( void );

   // Output Format register (OFORM)
   virtual void      SetFullOutputRange( bool );
   virtual bool      IsFullOutputRange( void );
   virtual ErrorCode SetLumaCoring( CoringLevel );
   virtual int       GetLumaCoring( void );

   // Vertical Scaling register (E_VSCALE_HI, O_VSCALE_HI)
   virtual void      SetChromaComb( bool );
   virtual bool      IsChromaComb( void );
   virtual void      SetVScaleMode( int );
   virtual int       GetVScaleMode( void );

   // AGC Delay register (ADELAY)
   virtual void      SetAGCDelay( BYTE );
   virtual int       GetAGCDelay( void );

   // Burst Delay register (BDELAY)
   virtual void      SetBurstDelay( BYTE );
   virtual int       GetBurstDelay( void );

   // ADC Interface register (ADC)
   virtual void      SetAnalogThresholdLow( bool );
   virtual bool      IsAnalogThresholdLow( void );
   virtual void      SetAGCFunction( bool );
   virtual bool      IsAGCFunction( void );
   virtual void      PowerDown( bool );
   virtual bool      IsPowerDown( void );
   virtual void      SetLumaADC( bool );
   virtual bool      IsLumaADC( void );
   virtual void      SetChromaADC( bool );
   virtual bool      IsChromaADC( void );
   virtual void      SetAdaptiveAGC( bool );
   virtual bool      IsAdaptiveAGC( void );

   // White Crush Down Count (WC_DN)
   virtual void      SetVerticalDetect( bool );
   virtual bool      GetVerticalDetect( void );

   // Software Reset register (SRESET)
   virtual void      SoftwareReset( void );

   // Test Control register (TEST)
   virtual void      AdjustInertialDampener( bool );

   // Video Signal Interface Format (VSIF)
   virtual ErrorCode SetVideoSignalFormat( SignalFormat format );
   virtual int       GetVideoSignalFormat( void );
   virtual ErrorCode SetSyncVideoRef( SyncVideoRef ref );
   virtual int       GetSyncVideoRef( void );
   virtual void      EnableSyncOutput( bool mode );
   virtual bool      IsSyncOutputEnabled( void );
   virtual void      EnableChromaFilterBypass( bool mode );
   virtual bool      IsChromaFilterBypassEnabled( void );
   virtual void      RemapGPDATA( bool mode );
   virtual bool      IsGPDATARemapped( void );

   // Video Timing Control (VTC)
   virtual void      SetHReset( int clk );
   virtual int       GetHReset( void );

   // Expected number of horizontal lines per frame (VTOTAL) - new for Gemini
   virtual ErrorCode SetLineCount( int lines );
   virtual int       GetLineCount( void );

   // helping functions
   virtual bool      IsDigitalSignal( void );

protected:
   // mapping function
   virtual ErrorCode Mapping( int, CRegInfo, int *, CRegInfo );

   // check registry key value to determine if contrast should be adjusted
   virtual bool IsAdjustContrast( void );
   virtual BYTE RecalcBrightness( void );

private:
   void              SelectCrystal( char );

};

#endif // __DECODER_H

