/* File: Field.h
 * Purpose: Contains definition of the Field class
 * Author: sasha
 * $Log: FIELD.H $
 * Revision 2.15  1998/05/14 23:40:39  dtharp
 * Graft from DT_VBI branch: VBI now functional.
 * Revision 2.14.1.1  1998/05/11 17:27:15  dtharp
 * Branch to restore VBI
 * Revision 2.14  1998/04/02 21:39:04  dtharp
 * Merge DT_VIDEOHDR branch. Implements a new buffer 
 * management scheme.
 * Revision 2.13.1.2  1998/04/02 21:37:50  dtharp
 * Merge DT_VIDEOHDR branch. Implements a new buffer 
 * management scheme.
 * Revision 2.13.1.1  1998/03/18 19:34:52  dtharp
 * Branch for passing videoheaders around
 * Revision 2.13  1997/11/26 00:20:06  peter
 * Graft reg key move to tip.
 * Revision 2.12.1.1  1997/11/25 17:32:01  peter
 * Revision 2.12  1997/08/08 11:18:54  dtharp
 * Fix for bug 714: NotifyArg now passed by reference.
 * Revision 2.11  1997/06/27 16:19:29  dtharp
 * Modifications to pass drop count to ring 3.
 * Revision 2.10  1997/05/16 16:18:54  dtharp
 * Vbifield now used VBIHdelay reg key for window offset.
 * Revision 2.9  1997/05/02 23:27:32  dtharp
 * 848A Merge: from 2.6.2.3
 * Revision 2.8  1997/04/25 16:43:34  dtharp
 * Grafted WST Interface Branch (2.7.1.1)
 * Revision 2.7.1.1  1997/04/03 16:46:43  dtharp
 * WST Interface. Removed the VBI_HDELAY and 
 * VBI_PACKET_SIZE overrides. These overrides are no longer
 * necessary and  were not compatible with the WST interface.
 * Revision 2.7  1997/03/20 22:08:30  dtharp
 * Grafted WDM Buffer managment branch.
 * Revision 2.6.4.3  1997/03/17 16:54:15  dtharp
 * Works @ 30fps NTSC.
 * Revision 2.6.4.2  1997/03/12 18:23:06  dtharp
 * Single field capture kinda works.
 * Revision 2.6  1996/11/19 17:02:01  dtharp
 * Merged working branch.
 * Revision 2.5.1.2  1996/11/19 16:58:31  dtharp
 * Changed sscanf() calls to strtoul().
 * Revision 2.5.1.1  1996/11/18 18:05:24  dtharp
 * Working branch.
 * Revision 2.5  1996/11/13 18:09:32  dtharp
 * Merged changes from branch 2.4.
 * Revision 2.4.1.2  1996/11/13 18:03:41  dtharp
 * VBI registry keys now accept decimal and hex values. Hex values 
 * must have "0x" prefix.
 * Revision 2.4.1.1  1996/11/13 17:48:41  dtharp
 * Working branch
 * Revision 2.4  1996/11/13 17:17:23  dtharp
 * Revision 2.3  1996/11/07 19:59:50  dtharp
 * Added static initialization of VBI regs to solve CC problem (Toshiba)
 * Register values are retrieved from Registry.
 * Revision 2.2  1996/08/22 22:09:34  rudyl
 * Pisces+: added new field FCNTR
 * Revision 2.1  1996/06/10 20:12:30  ssm
 * MarkBeta1AsRev2
 * Revision 1.17  1996/03/22 22:21:02  sasha
 * Pitch is in bytes now, so the VBI channel is changed for this
 * Revision 1.16  1996/03/18 20:58:08  sasha
 * Changed pointers to registers to references. Added 2 more registers to
 * the constructor calls ( byte and word swap registers ).
 * Revision 1.13  1996/02/27 02:05:07  sasha
 * Revision 1.12  1996/02/27 02:04:40  sasha
 * Revision 1.11  1996/02/19 00:04:19  sasha
 * Made Field's dtor virtual. Made Set/Get Window functions virtual. Made 
 * Set/Get color format virtual. Added VBIField class
 * Revision 1.10  1996/02/16 00:43:59  sasha
 * Revision 1.9  1996/01/19 00:34:33  sasha
 * Changed Field's ctror to accept a pointer to pointer to RegisterB
 * Revision 1.8  1996/01/15 20:17:06  sasha
 * Moved MaxProgs.. constant here. Added SkipCount_. Added pointer to the
 * capture_enable register; modified constructor to accept it. Added Skip() function
 * Moved Start() out of the class definition.
 * Revision 1.7  1996/01/06 02:05:40  sasha
 * Changed ColorFormat to ColFmt
 * Revision 1.6  1996/01/06 01:46:19  sasha
 * Changed constructor to accept RegisterB pointer for the scaler
 * Revision 1.5  1996/01/04 23:08:59  sasha
 * Added GetCurrentQue() function
 * Revision 1.4  1996/01/02 19:09:25  sasha
 * Changed BufQue_ to be a pointer ( moved the object into the channel class )
 * Revision 1.3  1995/12/08 22:16:42  sasha
 * Removed #pragma message
 */


#ifndef __FIELD_H
#define __FIELD_H

#ifndef __MYTYPES_H
#include "mytypes.h"
#endif

#ifndef __SCALER_H
#include "scaler.h"
#endif

#ifndef __PSCOLSPC_H
#include "pscolspc.h"
#endif

#include "vregkey.h"

const MaxProgsForField  = 2;

/* Class: Field
 * Purpose: Encapsulates the operation of a single video field provided by BtPisces
 * Attributes:
 * Operations:
 */
class Field
{
   private:
      PsColorSpace  LocalColSpace_;
//      VidBufQueue   *BufQue_;
      DWORD         dwPitch_;
      DWORD         dwOffset_;
      bool          Started_;
      int           SkipCount_;

      VideoStream   VidStrm_;

      // used to notify video channel
      ChanIface    *callback_;

      RegField     &CaptureEnable_;

      DWORD     InterruptCounter_;

   public:
      Field( RegField &CapEn, RegBase *ColReg, RegBase *WordSwap, 
         RegBase *ByteSwap ) : SkipCount_( 0 ), CaptureEnable_( CapEn ),
         LocalColSpace_( CF_RGB32, *ColReg, *WordSwap, *ByteSwap ), InterruptCounter_( 0 ),
         Started_( false ), callback_( NULL ),/* BufQue_( NULL ),*/ dwPitch_( 0 ), dwOffset_(0) {}
      virtual ~Field() {}
      void Notify( NotifyArg &na ) { if ( callback_ ) callback_->Notify( na ); }

      virtual ErrorCode SetAnalogWindow( TRect &r ) = 0;//{ return LocalScaler_.SetAnalogWin( r ); }
      virtual void      GetAnalogWindow( TRect &r ) = 0;//{ LocalScaler_.GetAnalogWin( r ); }

      virtual ErrorCode SetDigitalWindow( TRect &r ) = 0;//{ return LocalScaler_.SetDigitalWin( r ); }
      virtual void      GetDigitalWindow( TRect &r ) = 0;//{ LocalScaler_.GetDigitalWin( r ); }

      void  SetBufPitch( DWORD dwP ) { dwPitch_ = dwP; }
      DWORD GetBufPitch()            { return dwPitch_; }

      void  SetOffset( DWORD dwO ) { dwOffset_ = dwO; }
      DWORD GetOffset()            { return dwOffset_; }

      void SetStreamID( VideoStream );
      VideoStream GetStreamID();

      virtual void  SetColorFormat( ColFmt aColor )
      { LocalColSpace_.SetColorFormat( aColor ); }

      virtual ColFmt  GetColorFormat()
      { return LocalColSpace_.GetColorFormat(); }

//      PVOID GetNextBuffer() { return BufQue_->IsEmpty() ? NULL : BufQue_->Get(); }

//      void SetBufQuePtr( VidBufQueue *pQ ) { BufQue_ = pQ; }
//      VidBufQueue &GetCurrentQue() { return *BufQue_; }

      void Init( ChanIface *iface ) { callback_ = iface; }

      State  Start();
      void   Stop()  { Started_ = false; CaptureEnable_ = Off; }
      bool   IsStarted() { return Started_; }

      State  Skip();

      // called by the BtPiscess::ProcessRISCIntr()
      void GotInterrupt() { InterruptCounter_++; }

};

/* Class: FieldWithScaler
 * Purpose: Adds scaling capability to a field
 * Attributes:
 * Operations:
 */
class FieldWithScaler : public Field
{
   private:
      Scaler LocalScaler_;

   public:
      FieldWithScaler( RegField &CapEn, VidField field, RegBase *ColReg,
         RegBase *WordSwap, RegBase *ByteSwap ) : LocalScaler_( field ),
      Field( CapEn, ColReg, WordSwap, ByteSwap ) {}
         
      virtual ErrorCode SetAnalogWindow( TRect &r ) { return LocalScaler_.SetAnalogWin( r ); }
      virtual void      GetAnalogWindow( TRect &r ) { LocalScaler_.GetAnalogWin( r ); }

      virtual ErrorCode SetDigitalWindow( TRect &r ) { return LocalScaler_.SetDigitalWin( r ); }
      virtual void      GetDigitalWindow( TRect &r ) { LocalScaler_.GetDigitalWin( r ); }

      void VideoFormatChanged( VideoFormat format );
      void TurnVFilter( State s );
};


/* Class: VBIField
 * Purpose: Encapsulates the operation of a VBI data 'field'
 * Attributes:
 * Operations:
 */
class VBIField : public Field
{
   private:
      DECLARE_VBIPACKETSIZE;
      DECLARE_VBIDELAY;

#ifdef BT849
      DECLARE_VBIFIELDCOUNTER;
#endif  // #ifdef BT849

      static TRect AnalogWin_;
      static TRect DigitalWin_;
      static WORD VBI_PacketSize ;
      static BYTE VBI_HDelay ;

   public:

      VBIField( RegField &CapEn ) : Field( CapEn, NULL, NULL, NULL ),
      CONSTRUCT_VBIPACKETSIZE, CONSTRUCT_VBIDELAY
#ifdef BT849
      , CONSTRUCT_VBIFIELDCOUNTER
#endif  // #ifdef BT849
      {
      }

      virtual void  SetColorFormat( ColFmt Fmt ) {}
      virtual ColFmt  GetColorFormat() { return CF_VBI; };

      virtual ErrorCode SetAnalogWindow( TRect &r ) { AnalogWin_ = r; return Success; }
      virtual void      GetAnalogWindow( TRect &r ) { r = AnalogWin_; }

      virtual ErrorCode SetDigitalWindow( TRect &r )
      {
         char tbuf[80] ;
         DWORD nRetrieved = sizeof(tbuf) ;
         VRegistryKey vkey( PRK_LOCAL_MACHINE, "SOFTWARE\\Brooktree\\Video" );
         //VRegistryKey vkey( PRK_CLASSES_ROOT, "Bt848" );
         BYTE hOffset = 0 ;
         
         if ( vkey.getSubkeyValue( "VBI_HDelay", tbuf, (DWORD *)&nRetrieved ) )
            hOffset = (BYTE) strtoul( tbuf, NULL, 0 );

         r.left += hOffset ;
         r.right += hOffset ;

         DigitalWin_ = r;
         DWORD dwNoOfDWORDs = r.Width() / 2;
         SetBufPitch( r.Width() * ColorSpace( CF_VBI ).GetBitCount() / 8 );

         VBI_PKT_LO = (BYTE) dwNoOfDWORDs;
         VBI_PKT_HI = dwNoOfDWORDs > 0xff; // set the 9th bit
         VBI_HDELAY = r.left;

         return Success;
      }
      virtual void  GetDigitalWindow( TRect &r ) { r = DigitalWin_; }

      ~VBIField() {}
};

/* Method: Field::Start
 * Purpose: Initiates the data flow out of decoder into the FIFO
 * Input: None
 * Output: State: Off if channel was off; On if channel was on
 */
inline State Field::Start()
{
   Started_ = true;
   State RetVal = SkipCount_ >= MaxProgsForField ? Off : On;
   SkipCount_--;
   if ( SkipCount_ < 0 )
      SkipCount_ = 0;
   CaptureEnable_ = On;
   return RetVal;
}

/* Method: Field::Skip
 * Purpose: Increments the skip count and stops the data flow if it exceeds the max
 * Input: None
 * Output: State: Off if channel is stopped; On if channel remains running
 */
inline State Field::Skip()
{
   SkipCount_++;
   if ( SkipCount_ >= MaxProgsForField ) {
      Stop();
      return Off;
   }
   return On;
}
inline void FieldWithScaler::VideoFormatChanged( VideoFormat format )
{
   LocalScaler_.VideoFormatChanged( format );
}
inline void FieldWithScaler::TurnVFilter( State s )
{
   LocalScaler_.TurnVFilter( s );
}
inline void Field::SetStreamID( VideoStream st )
{
   VidStrm_ = st;
}
inline VideoStream Field::GetStreamID()
{
   return VidStrm_;
}

/* Class: FieldCounter
 * Purpose: Encapsulates the operation of the field counter in Bt848A
 * Attributes:	RegField FCNTR - field counter
 * Operations:
 *    void ResetFieldCounter() - reset counter to zero
 *    int  GetFieldCounter()   - obtain current field count
 */
class FieldCounter
{
   private:
      DECLARE_FIELDCOUNTER;

   public:
      FieldCounter() : CONSTRUCT_FIELDCOUNTER {};
      ~FieldCounter() {};

      void ResetCounter() { FCNTR = 0; }   // write anything resets counter
      int  GetCounter()   { return FCNTR; }
};
#endif
