// $Header: G:/SwDev/W95/Video/pisces/dvlpmnt/Grappler/rcs/dactl.cpp 1.1 1998/05/21 18:25:50 rudyl Exp $
// $Log: dactl.cpp $
// Revision 1.1  1998/05/21 18:25:50  rudyl
// Initial revision

/////////////////////////////////////////////////////////////////////////////
//
//  Copyright (C) 1998 Rockwell Semiconductor Systems - San Diego
//
//  All Rights Reserved
//
//  Module:
//
//    DaCtl.cpp
//
//  Abstract:
//
//    Contains all digital audio interface functions to the Grappler DLL
//
/////////////////////////////////////////////////////////////////////////////

// --- TCZ TV Digital Audio APIs

#include <windows.h>
#include <mmsystem.h>
#include <winnls.h>     // country codes
#include "regutil.h"
#include "tvctl.h"
#include "radioctl.h"
#include "tverr.h"
#include "tuner.h"
#include "audio.h"
#include "bt848api.h"
#include "bt848fcn.h"
#include "dynlink.h"


void InitDaTv( void );
void InitDaRadio( void );
void InitCountry( void );
ULONG GetTDA9874Status( void );
ULONG GetMSP3410Status( void );
ULONG SetTDA9874Status( ULONG lParm );
ULONG SetMSP3410Status( ULONG lParm );


typedef enum { I2SSRC_NONE, I2SSRC_TDA9874, I2SSRC_MSP3410 } I2SSRC;
static I2SSRC I2SSrc = I2SSRC_NONE;
static DWORD g_dwCountry = CTRY_DEFAULT;

static DAFLAVOR DaTvFlavor = { sizeof( DAFLAVOR ), DATYPE_NONE };
static DAFLAVOR DaRadioFlavor = { sizeof( DAFLAVOR ), DATYPE_NONE };


//------------------------------------
// Public country
//------------------------------------

ULONG SetCountry( ULONG lParm )
{
	// On country change, initialize both radio and tv
	g_dwCountry = (DWORD)lParm;
	InitDaTv();
	InitDaRadio();
	return 0;	// success
}

ULONG GetCountry( void )
{
	InitCountry();
	return g_dwCountry;
}

//------------------------------------
// Public Tv
//------------------------------------


ULONG SetTvDaFlavor( ULONG lParm )
{
	DAFLAVOR* lpDaFlavor = (DAFLAVOR*)lParm;

	InitDaTv();
	*lpDaFlavor = DaTvFlavor;

	return TV_ERR_SUCCESS;
}

ULONG GetTvDaFlavor( void )
{
	InitDaTv();
	return (ULONG)(LPVOID)&DaTvFlavor;
}

ULONG SetTvDaStatus( ULONG lParm )
{
	ULONG status = TV_ERR_SUCCESS;
	switch( I2SSrc )
	{
	case I2SSRC_TDA9874:
		status = SetTDA9874Status( lParm );
		break;

	case I2SSRC_MSP3410:
		status = SetMSP3410Status( lParm );
		break;
	}
	return status;
}

ULONG GetTvDaStatus( void )
{
	ULONG status = DASTATUS_OK;
	switch( I2SSrc )
	{
	case I2SSRC_TDA9874:
		status = GetTDA9874Status();
		break;

	case I2SSRC_MSP3410:
		status = GetMSP3410Status();
		break;
	}
	return status;
}

//------------------------------------
// Public Radio
//------------------------------------

ULONG SetRadioDaFlavor( ULONG lParm )
{
	DAFLAVOR* lpDaFlavor = (DAFLAVOR*)lParm;

	InitDaRadio();
	*lpDaFlavor = DaRadioFlavor;

	return RADIO_ERR_SUCCESS;
}

ULONG GetRadioDaFlavor( void )
{
	InitDaRadio();
	return (ULONG)(LPVOID)&DaRadioFlavor;
}

ULONG SetRadioDaStatus( ULONG )
{
	// TODO: implement status
	return TV_ERR_SUCCESS;
}

ULONG GetRadioDaStatus( void )
{
	// TODO: implement status
	return DASTATUS_OK;
}

//------------------------------------
// Internal
//------------------------------------


// I2S sources that we know of
BYTE bI2CMSP3410 = 0x80;	// ITT MSP 3410 D
BYTE bMSP3410RESETSUB = 0x00;
BYTE bMSP3410DEMWRITESUB = 0x10;
BYTE bMSP3410DEMREADSUB = 0x11;
BYTE bMSP3410RESETBLOCK[] = { 0x80, 0x00 };
BYTE bMSP3410RUNBLOCK[] = { 0x00, 0x00 };

BYTE bI2CTDA9874 = 0xB0;	// Philips TDA9874H
BYTE bTDA9874RESETSUB = 0x01;
BYTE bTDA9874RESETVAL = 0x08;


I2SSRC ResetAndFindI2S( void )
{
   BYTE buf;

   // Attempt to reset Philips TDA9874
   if( DlkI2CHWWrite3( bI2CTDA9874, bTDA9874RESETSUB, bTDA9874RESETVAL ) == Success )
      if( DlkI2CHWRead( bI2CTDA9874, &buf ) == Success )
         return I2SSRC_TDA9874;

	// Attempt to reset ITT MSP3410
	if( DlkI2CHWMultiWrite( bI2CMSP3410, bMSP3410RESETSUB, bMSP3410RESETBLOCK, sizeof( bMSP3410RESETBLOCK ) ) == Success )
	{
		if( DlkI2CHWMultiWrite( bI2CMSP3410, bMSP3410RESETSUB, bMSP3410RUNBLOCK, sizeof( bMSP3410RUNBLOCK ) ) == Success )
			return I2SSRC_MSP3410;
	}

	// None found, resort to soft decode
	return I2SSRC_NONE;
}

BYTE bI2STDA9874ConfigTvBGDualFM[] =
{
	0x00, 0xC4, 0x08,
	0x72, 0x95, 0x55, 0x77, 0xA1, 0x01,
	0x00, 0x00, 0x04,
	0x04, 0x04,
	0x20, 0x08, 0x14, 0x50,
	0xBB, 0x80, 0x00,
	0x17, 0x01, 0x00
};

BYTE bI2STDA9874ConfigTvBGFMNICAM[] =
{
	0x00, 0xC4, 0x08,
	0x72, 0x95, 0x55, 0x79, 0xEA, 0xAA,
	0xC8, 0x00, 0x00,
	0x04, 0x04,
	0x00, 0x03, 0x14, 0x50,
	0xBB, 0x81, 0x10,
	0x17, 0x01, 0x00
};

BYTE bI2STDA9874ConfigTvIFMNICAM[] =
{
	0x00, 0xC4, 0x08,
	0x7D, 0x00, 0x00, 0x88, 0x8A, 0xAB,
	0xC8, 0x00, 0x00,
	0x04, 0x04,
	0x00, 0x08, 0x14, 0x50,
	0xBB, 0x81, 0x10,
	0x17, 0x01, 0x00
};

BYTE bI2SMSP3410ConfigTvBGDualFM[] = { 0x00, 0x20, 0x00, 0x02 };
BYTE bI2SMSP3410ConfigTvBGFMNICAM[] = { 0x00, 0x20, 0x00, 0x08 };
BYTE bI2SMSP3410ConfigTvIFMNICAM[] = { 0x00, 0x20, 0x00, 0x0A };

void InitDaTv( void )
{
	LPBYTE lpbConfig = NULL;
	int iSizeConfig = 0;

	InitCountry();
	DaTvFlavor.DaType = DATYPE_NONE;	// Initially assume mono

	I2SSrc = ResetAndFindI2S();
// --- TCZ DEBUG
	if( I2SSRC_TDA9874 == I2SSrc )
		OutputDebugString( "Found a TDA9874\n" );
	else if( I2SSRC_MSP3410 == I2SSrc )
		OutputDebugString( "Found an MPS3410\n" );
   else
		OutputDebugString( "No Nicam chip found\n" );
// --- END
	switch( I2SSrc )
	{
	case I2SSRC_NONE:
		switch( g_dwCountry )
		{
		case CTRY_BRAZIL:
		case CTRY_CANADA:
		case CTRY_JAPAN:
		case CTRY_MEXICO:
		case CTRY_UNITED_STATES:
			// These countries use BTSC
			DaTvFlavor.DaType = DATYPE_BTSC;
			break;
		}
		break;

	case I2SSRC_TDA9874:
		switch( g_dwCountry )
		{
		case CTRY_BELGIUM:
		case CTRY_DENMARK:
		case CTRY_FINLAND:
		case CTRY_NEW_ZEALAND:
		case CTRY_NORWAY:
		case CTRY_SPAIN:
		case CTRY_SWEDEN:
			lpbConfig = bI2STDA9874ConfigTvBGFMNICAM;
			iSizeConfig = sizeof( bI2STDA9874ConfigTvBGFMNICAM );
			break;

		case CTRY_AUSTRALIA:
		case CTRY_AUSTRIA:
		case CTRY_GERMANY:
		case CTRY_ITALY:
		case CTRY_NETHERLANDS:
		case CTRY_SWITZERLAND:
			lpbConfig = bI2STDA9874ConfigTvBGDualFM;
			iSizeConfig = sizeof( bI2STDA9874ConfigTvBGDualFM );
			break;

		case CTRY_HONG_KONG:
		case CTRY_IRELAND:
		case CTRY_UNITED_KINGDOM:
			lpbConfig = bI2STDA9874ConfigTvIFMNICAM;
			iSizeConfig = sizeof( bI2STDA9874ConfigTvIFMNICAM );
			break;
		}
		if( lpbConfig )
		{
			int i;
			for( i = 0; i < iSizeConfig; ++i )
		 	{
		 		if( Fail == DlkI2CHWWrite3( bI2CTDA9874, i, lpbConfig[i] ) )
		 			break;
		 	}
			if( i == iSizeConfig )
			{
				DaTvFlavor.DaType = DATYPE_I2S;
				DaTvFlavor.DaMlb = DAMLB_MSB;
				DaTvFlavor.DaSce = DASCE_FALLING;
				DaTvFlavor.DaLri = DALRI_RIGHT;
				DaTvFlavor.AscDelay = 0;
				DaTvFlavor.CapRate = 32000;
				DaTvFlavor.CapRes = 16;
				DaTvFlavor.CapChannels = 2;
			}
		}
		break;

	case I2SSRC_MSP3410:
		switch( g_dwCountry )
		{
		case CTRY_BELGIUM:
		case CTRY_DENMARK:
		case CTRY_FINLAND:
		case CTRY_NEW_ZEALAND:
		case CTRY_NORWAY:
		case CTRY_SPAIN:
		case CTRY_SWEDEN:
			lpbConfig = bI2SMSP3410ConfigTvBGFMNICAM;
			iSizeConfig = sizeof( bI2SMSP3410ConfigTvBGFMNICAM );
			break;

		case CTRY_AUSTRALIA:
		case CTRY_AUSTRIA:
		case CTRY_GERMANY:
		case CTRY_ITALY:
		case CTRY_NETHERLANDS:
		case CTRY_SWITZERLAND:
			lpbConfig = bI2SMSP3410ConfigTvBGDualFM;
			iSizeConfig = sizeof( bI2SMSP3410ConfigTvBGDualFM );
			break;

		case CTRY_HONG_KONG:
		case CTRY_IRELAND:
		case CTRY_UNITED_KINGDOM:
			lpbConfig = bI2SMSP3410ConfigTvIFMNICAM;
			iSizeConfig = sizeof( bI2SMSP3410ConfigTvIFMNICAM );
			break;
		}
		if( lpbConfig )
		{
			if( DlkI2CHWMultiWrite( bI2CMSP3410, bMSP3410DEMWRITESUB, lpbConfig, iSizeConfig ) == Success )
			{
				DaTvFlavor.DaType = DATYPE_I2S;
				DaTvFlavor.DaMlb = DAMLB_MSB;
				DaTvFlavor.DaSce = DASCE_FALLING;
				DaTvFlavor.DaLri = DALRI_RIGHT;
				DaTvFlavor.AscDelay = 0;
				DaTvFlavor.CapRate = 32000;
				DaTvFlavor.CapRes = 16;
				DaTvFlavor.CapChannels = 2;
			}
		}
		break;
	}
}

ULONG GetTDA9874Status( void )
{
	ULONG status = 0;
	BYTE bStatus[ 3 ];
	if( Fail == DlkI2CHWMultiRead( bI2CTDA9874, 0x00, bStatus, sizeof( bStatus ) ) )
		return status;

	status = DASTATUS_OK;

	if( ( bStatus[ 0 ] & 0x08 ) && ( bStatus[ 1 ] & 0x08 ) )	// NICAM valid and sync'd
	{
		status |= DASTATUS_DIGITAL;
		if( bStatus[ 1 ] & 0x02 )
			status |= DASTATUS_STEREO;
		if( bStatus[ 1 ] & 0x01 )
			status |= DASTATUS_DUAL;
	}
	else
	{
		if( bStatus[ 0 ] & 0x10 )
			status |= DASTATUS_AUTOMUTE;
		if( bStatus[ 0 ] & 0x01 )
			status |= DASTATUS_STEREO;
		if( bStatus[ 0 ] & 0x02 )
			status |= DASTATUS_DUAL;
	}
	return status;
}

ULONG SetTDA9874Status( ULONG )
{
	// TODO: dual language
#pragma message( "To be completed." )
	return TV_ERR_SUCCESS;
}

ULONG GetMSP3410Status( void )
{
	// TODO: status and dual language
#pragma message( "To be completed." )
	return DASTATUS_OK;
}

ULONG SetMSP3410Status( ULONG )
{
	// TODO: dual language
#pragma message( "To be completed." )
	return TV_ERR_SUCCESS;
}


void InitDaRadio( void )
{
	// Radio decode done by Bt878 ;-)
	DaTvFlavor.DaType = DATYPE_BTSC;
}


void InitCountry( void )
{
	if( CTRY_DEFAULT == g_dwCountry )
	{
		char cCountry[ 10 ];
		if( GetLocaleInfo( LOCALE_USER_DEFAULT, LOCALE_ICOUNTRY, cCountry, sizeof( cCountry ) ) )
			g_dwCountry = atoi( cCountry );
		else
			g_dwCountry = CTRY_UNITED_STATES;
	}
// --- TCZ DEBUG
	char cBuf[ 256 ];
	sprintf( cBuf, "Country = %d\n", g_dwCountry );
	OutputDebugString( cBuf );
// --- END
}

// --- TCZ <end>


