// $Header: G:/SwDev/W95/Video/pisces/dvlpmnt/Grappler/rcs/tuner.cpp 2.21 1998/05/21 19:05:11 rudyl Exp $
// $Log: tuner.cpp $
// Revision 2.21  1998/05/21 19:05:11  rudyl
// Graft from 2.20.2.2.
// Revision 2.20.2.2  1998/05/21 18:59:25  rudyl
// Added Nicam support. Works with Philips TDA9874H only. ITT is next.
// Revision 2.20  1998/04/18 08:13:03  trang
// Modified to compile under Microsoft VC.
// Revision 2.19  1998/03/14 00:26:05  rudyl
// Use different algorithm for Alps FM station fine tune search to find more stations.
// Revision 2.18  1997/12/17 00:39:25  rudyl
// Fixed Alps radio tuning problem of taking long time before settle.
// Revision 2.17  1997/12/16 01:50:59  rudyl
// Removed debug print. No change in code.
// Revision 2.16  1997/11/26 00:21:06  peter
// Graft reg key move to tip.
// Revision 2.15.1.1  1997/11/20 22:46:08  peter
// Changes to move registry subkey to HKEY_LOCAL_MACHINE.
// Revision 2.15  1997/10/23 00:57:03  rudyl
// Added Tv power down mode to fix this problem:
//    // If Alps tuner was specified in the registry, invalid control codes
//    // will be sent to the Philips tuner puting it in some nonexist state causing
//    // it not able to tune correctly. Change tuner to any TV mode, e.g. power down,
//    // before regular radio tuning would fix this problem.
// Revision 2.14  1997/10/10 19:45:48  rudyl
// Alps tuner has different channels in differnet bands.
// Revision 2.13  1997/10/09 00:19:47  rudyl
// Use Temic/Philips TV AFT codes for Alps tuner.
// Only modified handling of status range 0-5.
// Revision 2.12  1997/10/08 00:59:00  rudyl
// Added Alps TSCH5 tuner support.
// Changed reg key "RFThreshold" to "FMThreshold".
// Revision 2.11  1997/09/22 23:33:43  rudyl
// Use "Bt878" as subkey for "RFThreshold".
// Revision 2.10  1997/09/18 21:40:55  rudyl
// Added code to look for registry key "RFThreshold" to set min threshold during station seek.
// Revision 2.9  1997/08/14 20:13:06  rudyl
// Use 2.7 which added Gemini support.
// Revision 2.7  1997/07/16 01:22:48  rudyl
// Added member function to get radio signal strength.
// Revision 2.6  1997/07/15 20:20:00  rudyl
// Use RF threshold level of 4 (highest). Then check AFC status to be 2, 1 or 3.
// Revision 2.5  1997/07/12 02:15:06  rudyl
// When search for FM station, check RF signal strength against threshold (currently 4, highest)
// to determine if a station is found.
// Revision 2.4  1997/07/02 18:09:54  rudyl
// Added TunerWithFMRadio class to support gemini.
// FM radio AFT codes incomplete.
// Revision 2.3  1997/05/08 17:37:02  rudyl
// Graft from 2.2.2.2:
// No default to Tv and audio on when grappler.dll is loaded.
// Revision 2.2.2.2  1997/05/05 19:52:59  rudyl
// No default to Tv and audio on when startup.
// Revision 2.2.2.1  1997/05/05 18:03:02  rudyl
// No default to Tv and audio on when startup.
// Revision 2.2  1997/03/03 21:45:32  rudyl
// Fixed cable channel 49 has wrong channel code.
// Revision 2.1.1.2  1997/02/25 17:52:58  rudyl
// Fix cable channel 49 has wrong channel code.
// Revision 2.1.1.1  1997/02/25 17:50:46  rudyl
// Cable channel 49 has wrong channel code.
// Revision 2.1  1996/06/10 20:12:37  ssm
// MarkBeta1AsRev2
// Revision 1.13  1996/05/30 19:05:27  rudyl
// 0.54 - Due to removal of vxd related calls from Bt848api.h to vxdcall.h, added include "vxdcall.h" here.
// Revision 1.12  1996/05/22 18:28:43  rudyl
// Added call to VxD when channel is about to change.
// Revision 1.11  1996/05/22 00:45:22  rudyl
// Moved VxD calls from here to Bt848_32.dll in order to hide the calls.
// Revision 1.10  1996/05/10 23:15:48  rudyl
// Implement TV_ERR_... return values from Intel. Also added exported function GetTVErrString() to retrieve our specific error msg.
// Revision 1.9  1996/05/07 19:22:44  rudyl
// Miscaulated Tuner::MaxNumberOfChannel( int mode ) by 1. Should be -1 instead of +1.
// Revision 1.8  1996/05/06 18:49:05  rudyl
// No changes; just added more comments into codes.
// Revision 1.7  1996/05/01 00:47:49  sasha
// Changed to use BT848 instead of Pisces
// Revision 1.6  1996/04/29 17:52:21  rudyl
// React to parameter type change from int to BYTE in I2CHWRead().
// Revision 1.5  1996/04/28 01:20:13  rudyl
// ver 0.50 - does not instantiate I2C and GPIO objects here. Just make API
// calls to BT848_32.DLL. Also returns error types to caller instead of just TRUE/FALSE result.
// Revision 1.4  1996/04/23 22:10:53  rudyl
// Removed codes inside SetAudio() and GetAudio() from GPIO to SetAudioSource() and GetAudioSource() here.
// Revision 1.3  1996/04/18 01:20:40  rudyl
// set audio to TV tuner when DLL starts and to mute when DLL ends
// Revision 1.2  1996/04/12 21:01:11  rudyl
// Renamed DLL from 848Tuner.dll to Grappler.dll.
// Revision 1.1  1996/04/12 18:24:43  rudyl
// Initial revision

// CUT_HERE

/////////////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 1996 Brooktree Corporation
//
//  Module:
//
//    Tuner.cpp
//
//  Abstract:
//
//    Perform channel change function through tuner for the Bt848.
//
/////////////////////////////////////////////////////////////////////////////

#include <windows.h>
#include <mmsystem.h>
#include "regutil.h"
#include "tvctl.h"
#include "radioctl.h"
#include "tverr.h"
#include "tuner.h"
#include "audio.h"
#include "bt848api.h"
#include "bt848fcn.h"
#include "dynlink.h"

/////////////////////////////////////////////////////////////////////////////
// Classes Constants
/////////////////////////////////////////////////////////////////////////////

// Temic Tuner
const WORD Tuner::PLL_SETTLE_DELAY = 30;  // check each 30 millisec to see if PLL settled
                                          // pll can take up to 150millisec to settle
const WORD Tuner::PLL_ATTEMPTS     = 5;   // # attempts to lock in PLL for each iteration
                                          // Attempts * Delay should be ~150msec

// Philips FM1236 tuner
const WORD TunerWithFMRadio::TV_POWERDOWN    = 0x8EA1;  // Philips TV power down mode
const WORD TunerWithFMRadio::RADIO_FM        = 0x8EA4;  // base Philips FM tuner code
const WORD TunerWithFMRadio::FM_FAST_TUNE    = 0x4000;  // CP = 1
const WORD TunerWithFMRadio::FM_NORMAL_TUNE  = 0x0000;  // CP = 0
const WORD TunerWithFMRadio::FM_AFC_MODE     = 0x0001;  // P0 = 1
const WORD TunerWithFMRadio::FM_RF_MODE      = 0x0000;  // P0 = 0
const WORD TunerWithFMRadio::FM_MONO         = 0x0002;  // P1 = 1
const WORD TunerWithFMRadio::FM_MUTE         = 0x0008;  // P3 = 1

// Alps TSCH5 tuner
const WORD AlpsTuner::PLL_SETTLE_DELAY = 100;      // check each 100 millisec to see if PLL settled
const WORD AlpsTuner::PLL_ATTEMPTS     = 3;        // # attempts to lock in PLL for each iteration
const WORD AlpsTuner::RADIO_FM         = 0x8605;   // Alps FM radio mode with 31.25 kHz step size

/////////////////////////////////////////////////////////////////////////////
// External variables
/////////////////////////////////////////////////////////////////////////////
extern HINSTANCE hInst;    // Handle to instance


/////////////////////////////////////////////////////////////////////////////
// External functions
/////////////////////////////////////////////////////////////////////////////
LONG SetAudioSource ( ULONG );
LONG SetAudioMute   ( ULONG );
// --- TCZ GetCountry code
LONG GetCountry		( void );
// --- TCZ <end>


/////////////////////////////////////////////////////////////////////////////
// Following are the functions communicate with I2C
/////////////////////////////////////////////////////////////////////////////
ErrorCode Bt848I2CWrite( BYTE I2CAddress, WORD msw, WORD lsw );

// --- TCZ Western European terestrial
/////////////////////////////////////////////////////////////////////////////
// Divider codes for Western Europe Air Channels (covers PAL and SECAM outside Japan)
/////////////////////////////////////////////////////////////////////////////
TunerCodes airEuroChannelCode[] = {
   0x0000,	0x2D4,	0x304,	0x374,	0x3E4,
	0xAF4,	0xB64,	0xBD4,	0xC44,	0xCB4,
	0xD24,	0xD94,	0xE04,	0x35C,	0x3E4,
	0x524,	0xAF4,	0xB74,	0xC04,	0xC94,
	0xD24,	0x1D74,	0x1DF4,	0x1E74,	0x1EF4,
	0x1F74,	0x1FF4,	0x2074,	0x20F4,	0x2174,
	0x21F4,	0x2274,	0x22F4,	0x2374,	0x23F4,
	0x2474,	0x24F4,	0x2574,	0x25F4,	0x2674,
	0x26F4,	0x2774,	0x27F4,	0x2874,	0x28F4,
	0x2974,	0x29F4,	0x2A74,	0x2AF4,	0x2B74,
	0x2BF4,	0x2C74,	0x2CF4,	0x2D74,	0x2DF4,
	0x2E74,	0x2EF4,	0x2F74,	0x2FF4,	0x3074,
	0x30F4,	0x3174,	0x31F4,	0x3274,	0x32F4,
	0x3374,	0x33F4,	0x3474,	0x34F4,	0x3574,
	0x2DC,	0x35C,	0x3DC,	0x0,	0x454,
	0x4C4,	0x534,	0x0,	0x0,	0x0,
	0x694,	0x704,	0x774,	0x7E4,	0x854,
	0x8C4,	0x934,	0x9A4,	0xA14,	0xA84,
	0xE74,	0xEE4,	0xF54,	0xFC4,	0x1034,
	0x10A4,	0x1114,	0x1184,	0x11F4,	0x1264,
	0x12F4,	0x1374,	0x13F4,	0x1474,	0x14F4,
	0x1574,	0x15F4,	0x1674,	0x16F4,	0x1774,
	0x17F4,	0x1874,	0x18F4,	0x1974,	0x19F4,
	0x1A74,	0x1AF4,	0x1B74,	0x1BF4,	0x1C74,	0x1CF4
 };

/////////////////////////////////////////////////////////////////////////////
// Divider codes for International Europe cable channels
/////////////////////////////////////////////////////////////////////////////
TunerCodes cableEuroChannelCode[] = {
	0x000,	0x304,	0x374,	0x3E4,	0x454,
	0x4C4,	0x534,	0x694,	0x704,
	0x774,	0x7E4,	0x854,	0x8C4,
	0x934,	0x9A4,	0xA14,	0xA84,
	0xAF4,	0xB64,	0xBD4,	0xC44,
	0xCB4,	0xD24,	0xD94,	0xE04,
	0xE74,	0xEE4,	0xF54,	0xFC4,
	0x1034,	0x10A4,	0x1114,	0x1184,
	0x11F4,	0x1264,	0x12F4,	0x1374,
	0x13F4,	0x1474,	0x14F4,	0x1574,
	0x15F4,	0x1034,	0x16F4,	0x1774,
	0x17F4,	0x1874,	0x18F4,	0x1974,
	0x19F4,	0x1A74,	0x1AF4,	0x1B74,
	0x1BF4,	0x1C74,	0x1CF4 };

/////////////////////////////////////////////////////////////////////////////
// Divider codes for UK cable channels
/////////////////////////////////////////////////////////////////////////////
TunerCodes cableUkChannelCode[] = {
	0x000,	0x2F4,	0x374,	0x3F4,	0x474,
	0x4F4,	0x574,	0x5F4,	0x674,
	0x6F4,	0x774,	0x7F4,	0x874,
	0x8F4,	0x974,	0x9F4,	0xA74,
	0xAF4,	0xB74,	0xBF4,	0xC74,
	0xCF4,	0xD74,	0xDF4,	0xE74,
	0xEF4,	0xF74,	0xFF4,	0x1074,
	0x10F4,	0x1174,	0x11F4,	0x1274,
	0x304,	0x374,	0x3E4,	0x454,
	0x4C4,	0x534,	0x694,	0x704,
	0x774,	0x7E4,	0x854,	0x8C4,
	0x934,	0x9A4,	0xA14,	0xA84,
	0xAF4,	0xB64,	0xBD4,	0xC44,
	0xCB4,	0xD24,	0xD94,	0xE04,
	0xE74,	0xEE4,	0xF54,	0xFC4,
	0x1034,	0x10A4,	0x1114,	0x1184,
	0x11F4,	0x1264,	0x12F4,	0x1374,
	0x13F4,	0x1474,	0x14F4,	0x1574,
	0x15F4,	0x1674,	0x16F4,	0x1774,
	0x17F4,	0x1874,	0x18F4,	0x1974,
	0x19F4,	0x1A74,	0x1AF4,	0x1B74,
	0x1BF4,	0x1C74,	0x1CF4 };
// --- TCZ <end>

/////////////////////////////////////////////////////////////////////////////
// Divider codes for NTSC USA Air Channels
/////////////////////////////////////////////////////////////////////////////
TunerCodes airNtscChannelCode[] = {
   0x0000, 0x0000, 0x0650, 0x06B0, 0x0710, 0x07B0,
   0x0810, 0x0DD0, 0x0E30, 0x0E90, 0x0EF0,
   0x0F50, 0x0FB0, 0x1010, 0x2050, 0x20B0,
   0x2110, 0x2170, 0x21D0, 0x2230, 0x2290,
   0x22F0, 0x2350, 0x23B0, 0x2410, 0x2470,
   0x24D0, 0x2530, 0x2590, 0x25F0, 0x2650,
   0x26B0, 0x2710, 0x2770, 0x27D0, 0x2830,
   0x2890, 0x28F0, 0x2950, 0x29B0, 0x2A10,
   0x2A70, 0x2AD0, 0x2B30, 0x2B90, 0x2BF0,
   0x2C50, 0x2CB0, 0x2D10, 0x2D70, 0x2DD0,
   0x2E30, 0x2E90, 0x2EF0, 0x2F50, 0x2FB0,
   0x3010, 0x3070, 0x30D0, 0x3130, 0x3190,
   0x31F0, 0x3250, 0x32B0, 0x3310, 0x3370,
   0x33D0, 0x3430, 0x3490, 0x34F0
};


/////////////////////////////////////////////////////////////////////////////
// Divider codes for NTSC USA Cable Channels
/////////////////////////////////////////////////////////////////////////////
TunerCodes cableNtscChannelCode[] = {
   0x0000, 0x0770, 0x0650, 0x06B0, 0x0710, 0x07B0,
   0x0810, 0x0DD0, 0x0E30, 0x0E90, 0x0EF0,
   0x0F50, 0x0FB0, 0x1010, 0x0A70, 0x0AD0,
   0x0B30, 0x0B90, 0x0BF0, 0x0C50, 0x0CB0,
   0x0D10, 0x0D70, 0x1070, 0x10D0, 0x1130,
   0x1190, 0x11F0, 0x1250, 0x12B0, 0x1310,
   0x1370, 0x13D0, 0x1430, 0x1490, 0x14F0,
   0x1550, 0x15B0, 0x1610, 0x1670, 0x16D0,
   0x1730, 0x1790, 0x17F0, 0x1850, 0x18B0,
   0x1910, 0x1970, 0x19D0, 0x1A30, 0x1A90,
   0x1AF0, 0x1B50, 0x1BB0, 0x1C10, 0x1C70,
   0x1CD0, 0x1D30, 0x1D90, 0x1DF0, 0x1E50,
   0x1EB0, 0x1F10, 0x1F70, 0x1FD0, 0x2030,
   0x2090, 0x20F0, 0x2150, 0x21B0, 0x2210,
   0x2270, 0x22D0, 0x2330, 0x2390, 0x23F0,
   0x2450, 0x24B0, 0x2510, 0x2570, 0x25D0,
   0x2630, 0x2690, 0x26F0, 0x2750, 0x27B0,
   0x2810, 0x2870, 0x28D0, 0x2930, 0x2990,
   0x29F0, 0x2A50, 0x2AB0, 0x2B10, 0x0890,
   0x08F0, 0x0950, 0x09B0, 0x0A10, 0x2B70,
   0x2BD0, 0x2C30, 0x2C90, 0x2CF0, 0x2D50,
   0x2DB0, 0x2E10, 0x2E70, 0x2ED0, 0x2F30,
   0x2F90, 0x2FF0, 0x3050, 0x30B0, 0x3110,
   0x3170, 0x31D0, 0x3230, 0x3290, 0x32F0,
   0x3350, 0x33B0, 0x3410, 0x3470, 0x34D0
};

// --- TCZ Calculate control word from desired freq
//#ifdef CHANNELBANDMAP
//
///////////////////////////////////////////////////////////////////////////////
//// Channel Map of frequency bands for NTSC USA air channels
///////////////////////////////////////////////////////////////////////////////
//TunerBandStruct airNtscBands[] = {
//    {  2,  6, bandVHF_LOW  },
//    {  7, 13, bandVHF_HIGH },
//    { 14, 69, bandUHF      },
//    {  0,  0, 0            } // end marker
//};
//
///////////////////////////////////////////////////////////////////////////////
//// Channel Map of frequency bands for NTSC USA cable channels
///////////////////////////////////////////////////////////////////////////////
//TunerBandStruct cableNtscBands[] = {
//    {   1,   6, bandVHF_LOW  },
//    {   7,  13, bandVHF_HIGH },
//    {  14,  20, bandVHF_LOW  },
//    {  21,  62, bandVHF_HIGH },
//    {  63,  94, bandUHF      },
//    {  95,  99, bandVHF_LOW  },
//    { 100, 125, bandUHF      },
//    {   0,   0, 0            }   // end marker
//};
//
///////////////////////////////////////////////////////////////////////////////
//// Channel Map of frequency bands for NTSC USA cable channels for Alps TSCH5
///////////////////////////////////////////////////////////////////////////////
//TunerBandStruct AlpsCableNtscBands[] = {
//    {   1,   6, bandVHF_LOW  },
//    {   7,  13, bandVHF_HIGH },
//    {  14,  15, bandVHF_LOW  },
//    {  16,  47, bandVHF_HIGH },
//    {  48,  94, bandUHF      },
//    {  95,  99, bandVHF_LOW  },
//    { 100, 125, bandUHF      },
//    {   0,   0, 0            }   // end marker
//};
//#endif
// --- TCZ <end>


/////////////////////////////////////////////////////////////////////////////
ControlWord controlPhilips[] =   // Philips FM1236 video & FM radio tuner
{
   0xCEA0, // vhf low
   0xCE90, // vhf high
   0xCE30  // uhf
};

// --- TCZ Calculate control word from desired freq
TunerCodes XOverPhilips[] =
{
	0xA04,	// vhf low - vhf high
	0x1C00,	// vhf high - uhf
};
// --- TCZ <end>

/////////////////////////////////////////////////////////////////////////////
ControlWord controlTemic[] =
{
   0x8E02, // vhf low
   0x8E04, // vhf high
   0x8E01  // uhf
};

// --- TCZ Calculate control word from desired freq
TunerCodes XOverTemic[] =
{
	0x900,	// vhf low - vhf high
	0x1D30,	// vhf low - uhf
};
// --- TCZ <end>

/////////////////////////////////////////////////////////////////////////////
ControlWord controlAlps[] =      // Alps TSCH5 tuner with FM radio
{                                // These control bytes are for video only
   0xC214, // vhf low
   0xC212, // vhf high
   0xC211  // uhf
};

// --- TCZ Calculate control word from desired freq
ControlWord XOverAlps[] =
{
	// TODO: get xover
	0xB00,	// vhf low - vhf high
	0x1940	// vhf high - ufh
};
// --- TCZ <end>

//===========================================================================
// Tuner Class
//===========================================================================


/////////////////////////////////////////////////////////////////////////////
// Constructor
//
// Method:  Tuner::Tuner( int brand, int format, BYTE I2CAddress)
// Purpose: Constructor for Tuner class
// Input:   int brand       - brand of tuner (Philips, Temic or Alps)
//          int format      - video format (NTSC or PAL)
//          BYTE I2CAddress - I2C address of the tuner
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
Tuner::Tuner( int brand, int format, BYTE I2CAddress)
{
   // there ptrs will pt to the respective look up table contains
   // all the divider and control values for all channels and modes
   m_ptrTunerCodeArray	= NULL;
   m_ptrControlWordArray = NULL;

   m_channel = 0;
   m_format = format;
   m_I2CAddress = I2CAddress;
   m_brand = brand;

// --- TCZ
   // set the brand of tuner
   if ( m_brand == brandPhilips )
   {
      m_ptrControlWordArray = controlPhilips;
      m_ptrTunerXOverArray = XOverPhilips;
   }
   else if ( m_brand == brandTemic )
   {
      m_ptrControlWordArray = controlTemic;
      m_ptrTunerXOverArray = XOverTemic;
   }
   else if ( m_brand == brandAlps )
   {
      m_ptrControlWordArray = controlAlps;
      m_ptrTunerXOverArray = XOverAlps;
   }
// --- TCZ <end>
}

/////////////////////////////////////////////////////////////////////////////
// Destructor
/////////////////////////////////////////////////////////////////////////////
Tuner::~Tuner()
{
}


/////////////////////////////////////////////////////////////////////////////
// Method:  LONG Tuner::SetChannel( int channel, int mode )
// Purpose: Set channel
// Input:   int channel - channel to set to
//          int mode    - channel mode (air or cable)
// Output:  None
// Return:  TV control error value
/////////////////////////////////////////////////////////////////////////////
LONG Tuner::SetChannel( int channel, int mode )
{
   // to determine the tuner channel codes we need to know the following:
   // 1- the channel #
   // 2- the video format (ntsc or pal)
   // 3- the mode (cable or air)
   // 4- the band setting for the tuner

   // range checking for channel number
   if ( ( channel < 0 ) || ( channel > MaxNumberOfChannel( mode ) ) )
      return ( (LONG)TV_ERR_BAD_VALUE );

   // range checking for channel mode
   if ( ( mode != modeTV ) && ( mode != modeCable ) )
      return ( (LONG)TV_ERR_BAD_VALUE );

   // notify Bt848 DLL that channel is about to change
   if ( DlkBt848PreChannelChange( channel ) != Success )
      return ( (LONG)TV_ERR_DLL_ERROR );

   // send the codes to the tuner via i2c
   LONG status = SetChannelAFT( channel, mode );
   if ( status != TV_ERR_SUCCESS )
      return ( status );

   // notify Bt848 DLL that channel has been changed
   if ( DlkBt848ChannelChanged( channel ) != Success )
      return ( (LONG)TV_ERR_DLL_ERROR );

   // store channel number
   m_channel = channel;

   return ( (LONG)TV_ERR_SUCCESS );
}


/////////////////////////////////////////////////////////////////////////////
// Method:  int GetChannel( void )
// Purpose: Get channel
// Input:   None
// Output:  None
// Return:  Channel number
/////////////////////////////////////////////////////////////////////////////
int Tuner::GetChannel( void )
{
   return ( m_channel );
}


/////////////////////////////////////////////////////////////////////////////
// Method:  WORD Tuner::MaxNumberOfChannel( int mode )
// Purpose: Return maximum number of valid channel
// Input:   int mode - air or cable
// Output:  None
// Return:  Maximum number of valid channel number
/////////////////////////////////////////////////////////////////////////////
WORD Tuner::MaxNumberOfChannel( int mode )
{
   SetCountrySpecifics( mode );
   return m_maxChannels;
}


// --- TCZ
/////////////////////////////////////////////////////////////////////////////
// Method:  void Tuner::SetCountrySpecifics()
// Purpose: Get the current country and set up arrays of stuff
// Input:   None
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
void Tuner::SetCountrySpecifics( int mode )
{
	m_ptrTunerCodeArray = NULL;
	m_ptrTunerIdArray = NULL;

	int iCountry = GetCountry();
	switch( mode )
	{
	case modeTV:
		switch( iCountry )
		{
		case CTRY_GERMANY:
		case CTRY_AUSTRALIA:
		case CTRY_NETHERLANDS:
		case CTRY_ITALY:
		case CTRY_AUSTRIA:
		case CTRY_SWITZERLAND:
		case CTRY_NORWAY:
		case CTRY_DENMARK:
		case CTRY_BELGIUM:
		case CTRY_SPAIN:
		case CTRY_NEW_ZEALAND:
		case CTRY_SINGAPORE:
		case CTRY_SWEDEN:
		case CTRY_FINLAND:

		case CTRY_GREECE:
		case CTRY_BRAZIL:
		case CTRY_BULGARIA:
		case CTRY_CZECH:
		case CTRY_FRANCE:
		case CTRY_HUNGARY:
		case CTRY_ICELAND:
		case CTRY_POLAND:
		case CTRY_PORTUGAL:
		case CTRY_ROMANIA:
		case CTRY_RUSSIA:
		case CTRY_TURKEY:

		case CTRY_HONG_KONG:
		case CTRY_IRELAND:
		case CTRY_UNITED_KINGDOM:	// Strictly the GB and HK has different VHF from above but it stopped
									// transmission in 1984 in GB (not sure about HK though)
	   		m_ptrTunerCodeArray = airEuroChannelCode;
	   		m_ptrTunerIdArray = NULL;
		    m_maxChannels = ( sizeof(airEuroChannelCode) / sizeof(*airEuroChannelCode) - 1 );
			m_fCarrier = 622; // 38.9*16
			break;
	
		case CTRY_CANADA:
		case CTRY_MEXICO:
		case CTRY_UNITED_STATES:
	   		m_ptrTunerCodeArray = airNtscChannelCode;
	   		m_ptrTunerIdArray = NULL;
		    m_maxChannels = ( sizeof(airNtscChannelCode) / sizeof(*airNtscChannelCode) - 1 );
			// TODO: this should really be 45.75 * 16, and that reduced from the divider code tables
			m_fCarrier = 0;
			break;
		}
		break;
	case modeCable:
		switch( iCountry )
		{
		case CTRY_HONG_KONG:
		case CTRY_IRELAND:
		case CTRY_UNITED_KINGDOM:
	   		m_ptrTunerCodeArray = cableUkChannelCode;
	   		m_ptrTunerIdArray = NULL;
		    m_maxChannels = ( sizeof(cableUkChannelCode) / sizeof(*cableUkChannelCode) - 1 );
			m_fCarrier = 622; // 38.9*16
			break;
	
		case CTRY_GERMANY:
		case CTRY_AUSTRALIA:
		case CTRY_NETHERLANDS:
		case CTRY_ITALY:
		case CTRY_AUSTRIA:
		case CTRY_SWITZERLAND:
		case CTRY_NORWAY:
		case CTRY_DENMARK:
		case CTRY_BELGIUM:
		case CTRY_SPAIN:
		case CTRY_NEW_ZEALAND:
		case CTRY_SINGAPORE:
		case CTRY_SWEDEN:
		case CTRY_FINLAND:

		case CTRY_GREECE:
		case CTRY_BRAZIL:
		case CTRY_BULGARIA:
		case CTRY_CZECH:
		case CTRY_FRANCE:
		case CTRY_HUNGARY:
		case CTRY_ICELAND:
		case CTRY_POLAND:
		case CTRY_PORTUGAL:
		case CTRY_ROMANIA:
		case CTRY_RUSSIA:
		case CTRY_TURKEY:
	   		m_ptrTunerCodeArray = cableEuroChannelCode;
	   		m_ptrTunerIdArray = NULL;
		    m_maxChannels = ( sizeof(cableEuroChannelCode) / sizeof(*cableEuroChannelCode) - 1 );
			m_fCarrier = 622; // 38.9*16
			break;

		case CTRY_UNITED_STATES:
	   		m_ptrTunerCodeArray = cableNtscChannelCode;
	   		m_ptrTunerIdArray = NULL;
		    m_maxChannels = ( sizeof(cableNtscChannelCode) / sizeof(*cableNtscChannelCode) - 1 );
			// TODO: this should really be 45.75 * 16, and that reduced from the divider code tables
			m_fCarrier = 0;
			break;
		}
		break;
	}
}
// --- TCZ <end>

/////////////////////////////////////////////////////////////////////////////
// Method:  WORD Tuner::GetDividerWord( int channel, int mode )
// Purpose: Obtain divider to change channel
// Input:   int channel - channel to obtain
//          int mode    - channel mode (air or cable)
// Output:  None
// Return:  Divider info if success, else 0
/////////////////////////////////////////////////////////////////////////////
WORD Tuner::GetDividerWord( int channel, int mode )
{
// --- TCZ
	SetCountrySpecifics( mode );
// --- TCZ <end>
   if ( m_ptrTunerCodeArray != NULL )
      return( m_ptrTunerCodeArray[ channel ] + m_fCarrier );
   else
      return( 0 );
}


/////////////////////////////////////////////////////////////////////////////
// Method:  void Tuner::SetDividerWord( int channel, int mode, WORD value )
// Purpose: Set divider info
// Input:   int channel - channel to obtain
//          int mode    - channel mode (air or cable)
//          WORD value  - value to set to
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
void Tuner::SetDividerWord( int channel, int mode, WORD value )
{
// --- TCZ
	SetCountrySpecifics( mode );
// --- TCZ <end>
   // now set the WORD in the selected array
   if ( m_ptrTunerCodeArray != NULL )
      m_ptrTunerCodeArray[ channel ] = value - m_fCarrier;
}


/////////////////////////////////////////////////////////////////////////////
// Method:  WORD Tuner::GetControlWord( int channel, int mode)
// Purpose: Obtain control code to change band
// Input:   int channel - channel to obtain
//          int mode    - channel mode (air or cable)
// Output:  None
// Return:  Control info if success, else 0
/////////////////////////////////////////////////////////////////////////////
WORD Tuner::GetControlWord( int channel, int mode)
{
// --- TCZ
	WORD wDivider = GetDividerWord( channel, mode );

	int band = bandVHF_LOW;
	if( wDivider > m_ptrTunerXOverArray[1] )
		band = bandUHF;
	else if( wDivider > m_ptrTunerXOverArray[0] )
		band = bandVHF_HIGH;

   // now return the control word
   if ( m_ptrControlWordArray != NULL )
      return ( m_ptrControlWordArray[ band ] );
   else
      return( 0 );
// --- TCZ <end>
}


/////////////////////////////////////////////////////////////////////////////
// Method:  LONG Tuner::SetChannelAFT( int channel, int mode )
// Purpose: Set channel with Automatic Fine Turning
// Input:   int channel - channel to obtain
//          int mode    - channel mode (air or cable)
// Output:  None
// Return:  TV control error value
// Note:    This function was provided by Intel Corp. and has been modified
//          for our need.
/////////////////////////////////////////////////////////////////////////////
LONG Tuner::SetChannelAFT( int channel, int mode )
{
    DWORD time;             // get the current time
    BYTE  status;           // Tuner status, including AFC A/D value
    int   n;                // iteration count
    int   m=0;              // how many times was PLL not locked?
    BOOL  centered = FALSE; // "tuned in" flag
    int   freq;             // our temporary copy of frequency bits

//--------------------------
// Array of frequency offset values for AFT search
// This zig-zags -- -1/4 MHz, +1/4MHz, -1/2MHz, etc., to +2-1/2 MHz
//--------------------------
    int  freqdelta[] =
       { -4,  4,  -8,  8, -12, 12, -16, 16,
        -20, 20, -24, 24, -28, 28, -32, 32,
        -36, 36, -40, 40 };
    int ITERATIONS = sizeof( freqdelta ) / sizeof( int );    // # iterations in AFT loop & in search table

//--------------------------
// Notify Capture Driver to Stop Capture while we change channels,
//   mostly to prevent garbage data from going to VBI decoder.
//--------------------------
//    StopCapture();

//--------------------------
// Grab frequency from table (2 tables -- one for cable, one for antenna).
// Write initial values to frequency and control regs in tuner
//--------------------------
   WORD div = GetDividerWord( channel, mode );
   WORD con = GetControlWord( channel, mode );
// --- TCZ DEBUG
   char cBuf[ 256 ];
   sprintf( cBuf, "Channel %d, mode %d, Divider %x, Control %x\n", channel, mode, (int)div, (int)con );
   OutputDebugString( cBuf );
// --- TCZ <end>

//--------------------------
// Send frequency to I2C to set channel
//--------------------------
   freq = div;
   if ( Bt848I2CWrite( (BYTE)m_I2CAddress, div, con ) == Fail )
      return ( TV_ERR_I2C_ERROR );

//----------------------------------------------------
// Loop to settle in on final frequency
// First, delay to allow tuner PLL to settle
//   Tuner specs claim 150 msec as settling time
//   We are delaying a shorter time, then checking for lock.
//----------------------------------------------------
    for (n=0; (n < ITERATIONS) && (m < PLL_ATTEMPTS); ) {
        time = timeGetTime(); // get current time
        while (timeGetTime() - time < PLL_SETTLE_DELAY ) {
                // SPIN waiting for PLL_SETTLE_DELAY
        }

//--------------------------
// Now take a look at status, see if we are centered on a channel.
//   (Bit 6 should be high, indicating Phase Lock).
//   (Bits 2-0 should be value 2, indicating centered frequency).
//   (if bits 2-0 < 2, then we should raise frequency)
//   (if bits 2-0 > 2, then we should lower frequency)
//   (if bits 2-0 == 4, then we also may not be near anything!)
//--------------------------
         if ( DlkI2CHWRead( (BYTE)m_I2CAddress, &status ) == Fail )
            return ( TV_ERR_I2C_ERROR );

//--------------------------
// Test Phase lock condition (bit 6)
//  (if phase lock is not achieved, we will just repeat the loop,
//   without changing frequency, giving PLL more time to settle out).
//--------------------------
         if ( status & 0x40 ) {

//--------------------------
// PLL Locked, test AFC condition (low 3 bits of status)
//--------------------------
            switch ( status & 0x07 )
            {
               case 0:     // carrier sensed, need to lower frequency
                  freq = freq - 2;
                  if ( DlkI2CHWWrite3( (BYTE)m_I2CAddress, HIBYTE( freq ), LOBYTE( freq ) ) == Fail )
                     return ( TV_ERR_I2C_ERROR );
                  break;

               case 1:     // carrier sensed, need to lower frequency
                  freq = freq - 1;
                  if ( DlkI2CHWWrite3( (BYTE)m_I2CAddress, HIBYTE( freq ), LOBYTE( freq ) ) == Fail )
                     return ( TV_ERR_I2C_ERROR );
                  break;

               case 2:     // carrier sensed, we are on correct frequency
                  centered = TRUE;
                  break;

               case 3:     // carrier sensed, need to raise frequency
                  freq = freq + 1;
                  if ( DlkI2CHWWrite3( (BYTE)m_I2CAddress, HIBYTE( freq ), LOBYTE( freq ) ) == Fail )
                     return ( TV_ERR_I2C_ERROR );
                  break;

               case 4:     // carrier not sensed, search using offset table
                  freq = div + freqdelta[n];
                  if ( DlkI2CHWWrite3( (BYTE)m_I2CAddress, HIBYTE( freq ), LOBYTE( freq ) ) == Fail )
                     return ( TV_ERR_I2C_ERROR );
                  break;

               default:
                  break;

            } // switch

//------------
// Increment n to control loop and access next frequency search offset
//------------
            n++;
            m = 0;         // Reset PLL lock attempt counter

        } // if phase locked

//--------------------------
// Else PLL not locked.
// Don't increment n.  Simply loop again until PLL is locked.
//--------------------------
        else {
            m++;
        } // else not phase locked

//--------------------------
// If we detected centered frequency,
//   update our table of frequencies (for fast tuning next time),
//   break out of loop (we are done),
// else repeat loop.
// If phase lock had not been detected, we did not change tuner settings,
//   we will just repeat the time delay, and check again for phase lock.
//--------------------------
        if (centered) {
            SetDividerWord( channel, mode, (WORD)freq );
            break; // out of for loop

        } // if centered

    } // for ITERATIONS iterations or for max PLL_ATTEMPTS to lock

//----------------------------------------------------
// Tuning loop is done, let's see how we did
//----------------------------------------------------

//--------------------------
// If we found a carrier, restart Capture.
// Notify Capture Driver of current channel
//--------------------------
   if ( centered )
   {
      return ( TV_ERR_SUCCESS );
   }

//--------------------------
// Else no carrier, return why we didn't tune right
//--------------------------
   else
   {
      // if we didn't find a carrier, restore I2C to table value
      DlkI2CHWWrite3( (BYTE)m_I2CAddress, HIBYTE( freq ), LOBYTE( freq ) );
      return ( TV_ERR_NO_CARRIER );
   }
}



//===========================================================================
//  Class TunerWithFMRadio
//===========================================================================


/////////////////////////////////////////////////////////////////////////////
// Constructor
//
// Method:  TunerWithFMRadio::TunerWithFMRadio( int brand, int format, BYTE I2CAddress)
// Purpose: Constructor for TunerWithFMRadio class
// Input:   int brand       - brand of tuner (Philips or Temic)
//          int format      - video format (NTSC or PAL)
//          BYTE I2CAddress - I2C address of the tuner
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
TunerWithFMRadio::TunerWithFMRadio( int brand, int format, BYTE I2CAddress ) :
                  Tuner( brand, format, I2CAddress ), m_FMFreq( MIN_FREQ )
{
}

/////////////////////////////////////////////////////////////////////////////
// Destructor
/////////////////////////////////////////////////////////////////////////////
TunerWithFMRadio::~TunerWithFMRadio()
{
}

/////////////////////////////////////////////////////////////////////////////
// Method:  LONG TunerWithFMRadio::SetRadioFreq( int freq )
// Purpose: Set radio frequency
// Input:   int freq - radio frequency without decimal place (e.g. 985 = 98.5MHz)
// Output:  None
// Return:  Radio control error value
/////////////////////////////////////////////////////////////////////////////
LONG TunerWithFMRadio::SetRadioFreq( int freq )
{
   LONG status = SetRadioFreqAFT( freq );
   if ( status != RADIO_ERR_SUCCESS )
      return ( status );

   // store channel number
   m_FMFreq = freq;

   return ( (LONG)RADIO_ERR_SUCCESS );
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int TunerWithFMRadio::GetRadioFreq( void )
// Purpose: Get radio frequency
// Input:   None
// Output:  None
// Return:  radio frequency without decimal place (i.e. 100.5MHz returns 1005)
/////////////////////////////////////////////////////////////////////////////
int TunerWithFMRadio::GetRadioFreq( void )
{
   return ( m_FMFreq );
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int TunerWithFMRadio::GetRadioSignalStrength( void )
// Purpose: Get radio signal strength
// Input:   None
// Output:  None
// Return:  radio signal strength: 0 (lowest) - 4 (highest) for Philips FM1236)
/////////////////////////////////////////////////////////////////////////////
int TunerWithFMRadio::GetRadioSignalStrength( void )
{
   // put tuner into RF signal level
   WORD mode = RADIO_FM | FM_NORMAL_TUNE | FM_RF_MODE | FM_MUTE;
   if ( DlkI2CHWWrite3( m_I2CAddress, HIBYTE( mode ), LOBYTE( mode ) ) == Fail )
      return ( 0 );

   // wait until PLL is locked
   DWORD time;             // get the current time
   BYTE status;            // PLL bit 6, signal strength bits[2:0]
   for ( int m = 0; m < PLL_ATTEMPTS; m++ )
   {
      time = timeGetTime(); // get current time
      while (timeGetTime() - time < PLL_SETTLE_DELAY )
         ;  // SPIN waiting for PLL_SETTLE_DELAY

      // read status and check if PLL is locked
      if ( DlkI2CHWRead( m_I2CAddress, &status ) == Fail )
         return ( 0 );

      if ( status & 0x40 )   // PLL locked?
         break;   // break out of for loop
   }

   // put tuner back to AFC mode
   mode = RADIO_FM | FM_NORMAL_TUNE | FM_AFC_MODE;
   if ( DlkI2CHWWrite3( m_I2CAddress, HIBYTE( mode ), LOBYTE( mode ) ) == Fail )
      return ( 0 );

   return ( status & 0x0007 );      // RF level at [2:0]
}

/////////////////////////////////////////////////////////////////////////////
// Method:  LONG TunerWithFMRadio::SetRadioFreqAFT( int freq )
// Purpose: Set radio frequency with Automatic Fine Turning
// Input:   int freq - frequency to set to (no decimal place; i.e. 1007 = 100.7MHz)
// Output:  None
// Return:  Radio control error value
// Note:    After tune to the specified frequency, RF signal threshold is checked
//          against the minimum acceptable level (currently 4, highest).
//          Additionally, ADC status is read and only 2, 1 or 3 are determined
//          as tune-in.
/////////////////////////////////////////////////////////////////////////////
LONG TunerWithFMRadio::SetRadioFreqAFT( int freq )
{
   int RFLevelThreshold = 3;     // min threshold to indicate a station is found;
                                 // Valid range seems to be 0-4 (highest)
   DWORD dwTemp;
   RegUtil reg( "SOFTWARE\\Brooktree\\Video" );
   if ( reg.GetValue( &dwTemp, "FMThreshold" ) == Success )
      RFLevelThreshold = (int)min( (DWORD)4, max( dwTemp, (DWORD)0 ) );	 // between 0-4

   DWORD time;             // get the current time
   BYTE  status;           // Tuner status, including AFC A/D value
   int   n;                // how many times was PLL not locked?
   int   m;                // how many times was PLL not locked?
   BOOL  centered = FALSE; // "tuned in" flag

   //--------------------------
   // range checking for frequency value
   //--------------------------
   if ( ( freq < MIN_FREQ ) || ( freq > MAX_FREQ ) )
      return ( (LONG)RADIO_ERR_BAD_VALUE );

   //--------------------------
   // Calculate program divider
   //--------------------------
   WORD div = ( freq + 107 ) * 8 / 5;

   //--------------------------
   // If Alps tuner was specified in the registry, invalid control codes
   // will be sent to the Philips tuner puting it in some nonexist state causing
   // it not able to tune correctly. Change tuner to any TV mode, e.g. power down,
   // before regular radio tuning would fix this problem.
   //--------------------------
   if ( DlkI2CHWWrite3( (BYTE)m_I2CAddress, HIBYTE( TV_POWERDOWN ), LOBYTE( TV_POWERDOWN ) ) == Fail )
      return ( RADIO_ERR_I2C_ERROR );
   time = timeGetTime(); // get current time
      while (timeGetTime() - time < PLL_SETTLE_DELAY ) {
      // SPIN waiting for PLL_SETTLE_DELAY
   }

   //--------------------------
   // Use FM band in fast tune mode using RF level search
   //--------------------------
   if ( Bt848I2CWrite( (BYTE)m_I2CAddress, RADIO_FM | FM_FAST_TUNE | FM_RF_MODE | FM_MUTE, div ) == Fail )
      return ( RADIO_ERR_I2C_ERROR );

   int RFLevel;   // RF level at status bits[2:0]

   for ( m = 0; m < PLL_ATTEMPTS; m++ ) {
       time = timeGetTime(); // get current time
       while (timeGetTime() - time < PLL_SETTLE_DELAY ) {
               // SPIN waiting for PLL_SETTLE_DELAY
       }

       //--------------------------
       // Now take a look at status, see if we are centered on a channel.
       //   (Bit 6 should be high, indicating Phase Lock).
       //--------------------------
       if ( DlkI2CHWRead( (BYTE)m_I2CAddress, &status ) == Fail )
          break;

       //--------------------------
       // Test Phase lock condition (bit 6)
       //  (if phase lock is not achieved, we will just repeat the loop,
       //   giving PLL more time to settle out).
       //--------------------------
       if ( status & 0x40 )   // PLL locked?
       {
          // get RF level and check if it is at least the level of the threshold
         RFLevel = status & 0x0007; // RF level at [2:0]

         if ( RFLevel < RFLevelThreshold )
            break;   // RF signal below threshold, not tune in
         else
         {
            // switch over to use AFC mode for maximum signal to noise ratio
            if ( Bt848I2CWrite( (BYTE)m_I2CAddress, RADIO_FM | FM_NORMAL_TUNE | FM_AFC_MODE | FM_MUTE, div ) == Fail )
               return ( RADIO_ERR_I2C_ERROR );

            for ( n=0; (n < PLL_ATTEMPTS); n++ )
            {
                time = timeGetTime(); // get current time
                while (timeGetTime() - time < PLL_SETTLE_DELAY ) {
                        // SPIN waiting for PLL_SETTLE_DELAY
                }

                //--------------------------
                // Check Bit 6 should be high, indicating Phase Lock
                //--------------------------
                if ( DlkI2CHWRead( (BYTE)m_I2CAddress, &status ) == Fail )
                   break;

                if ( status & 0x40 )               // PLL locked?
                {
                   //--------------------------
                   // PLL Locked, test AFC result (low 3 bits of status)
                   //   (Bits 2-0 should be value 2, indicating centered frequency).
                   //--------------------------
                  if ( ( ( status & 0x07 ) == 2 ) ||    // 2 = center in frequency
                       ( ( status & 0x07 ) == 3 ) ||    // 3 = very close in center
                       ( ( status & 0x07 ) == 1 ) )     // 1 = very close in center
                     centered = TRUE;
                  break;
                } // if phase locked
            }
            break;
         }
       }

      //--------------------------
      // If we detected centered frequency,
      //   break out of loop (we are done),
      // else repeat loop to wait for phase lock
      //--------------------------
       if ( centered )
           break; // out of for loop

   } // for ITERATIONS iterations or for max PLL_ATTEMPTS to lock

   //----------------------------------------------------
   // unmute and stop search mode to get best signal-to-noise ratio
   //----------------------------------------------------
   if ( Bt848I2CWrite( (BYTE)m_I2CAddress, RADIO_FM | FM_NORMAL_TUNE | FM_AFC_MODE, div ) == Fail )
      return ( RADIO_ERR_I2C_ERROR );

   //----------------------------------------------------
   // Tuning loop is done, let's see how we did
   //----------------------------------------------------
   if ( centered )
      return ( RADIO_ERR_SUCCESS );
   else
      return ( RADIO_ERR_NO_CARRIER );
}


//===========================================================================
//  Class AlpsTuner
//===========================================================================

/////////////////////////////////////////////////////////////////////////////
// Method:  LONG AlpsTuner::SetChannelAFT( int channel, int mode )
// Purpose: Set channel with Automatic Fine Turning for Alps TSCH5 tuner
// Input:   int channel - channel to obtain
//          int mode    - channel mode (air or cable)
// Output:  None
// Return:  TV control error value
// Note:    This function was provided by Intel Corp. and has been modified
//          for our need.
/////////////////////////////////////////////////////////////////////////////
LONG AlpsTuner::SetChannelAFT( int channel, int mode )
{
    DWORD time;             // get the current time
    BYTE  status;           // Tuner status, including AFC A/D value
    int   n;                // iteration count
    int   m=0;              // how many times was PLL not locked?
    BOOL  centered = FALSE; // "tuned in" flag
    int   freq;             // our temporary copy of frequency bits

//--------------------------
// Array of frequency offset values for AFT search
// This zig-zags -- -1/4 MHz, +1/4MHz, -1/2MHz, etc., to +2-1/2 MHz
//--------------------------
    int  freqdelta[] =
       { -4,  4,  -8,  8, -12, 12, -16, 16,
        -20, 20, -24, 24, -28, 28, -32, 32,
        -36, 36, -40, 40 };
    int ITERATIONS = sizeof( freqdelta ) / sizeof( int );    // # iterations in AFT loop & in search table

//--------------------------
// Grab frequency from table (2 tables -- one for cable, one for antenna).
// Write initial values to frequency and control regs in tuner
//--------------------------
   WORD div = GetDividerWord( channel, mode );
   WORD con = GetControlWord( channel, mode );

//--------------------------
// Send frequency to I2C to set channel
//--------------------------
   freq = div;
   if ( Bt848I2CWrite( (BYTE)m_I2CAddress, div, con ) == Fail )
      return ( TV_ERR_I2C_ERROR );

//----------------------------------------------------
// Loop to settle in on final frequency
// First, delay to allow tuner PLL to settle
//   Tuner specs claim 150 msec as settling time
//   We are delaying a shorter time, then checking for lock.
//----------------------------------------------------
    for (n=0; (n < ITERATIONS) && (m < PLL_ATTEMPTS); ) {
        time = timeGetTime(); // get current time
        while (timeGetTime() - time < PLL_SETTLE_DELAY ) {
                // SPIN waiting for PLL_SETTLE_DELAY
        }

//--------------------------
// Now take a look at status, see if we are centered on a channel.
//   (Bit 6 should be high, indicating Phase Lock).
//   (Bits 2-0 should be value 2 or 3, indicating centered frequency).
//   (if bits 2-0 < 2, then we should lower frequency)
//   (if bits 2-0 == 4, then we should raise frequency)
//   (if bits 2-0 == 5, then we also may not be near anything!)
//--------------------------
         if ( DlkI2CHWRead( (BYTE)m_I2CAddress, &status ) == Fail )
            return ( TV_ERR_I2C_ERROR );

//--------------------------
// Test Phase lock condition (bit 6)
//  (if phase lock is not achieved, we will just repeat the loop,
//   without changing frequency, giving PLL more time to settle out).
//--------------------------
         if ( status & 0x40 ) {

//--------------------------
// PLL Locked, test AFC condition (low 3 bits of status)
//--------------------------
            switch ( status & 0x07 )
            {
               case 0:     // carrier sensed, need to lower frequency
                  freq = freq - 2;
                  if ( DlkI2CHWWrite3( (BYTE)m_I2CAddress, HIBYTE( freq ), LOBYTE( freq ) ) == Fail )
                     return ( TV_ERR_I2C_ERROR );
                  break;

               case 1:     // carrier sensed, need to lower frequency
                  freq = freq - 1;
                  if ( DlkI2CHWWrite3( (BYTE)m_I2CAddress, HIBYTE( freq ), LOBYTE( freq ) ) == Fail )
                     return ( TV_ERR_I2C_ERROR );
                  break;

               case 2:     // carrier sensed, we are on correct frequency
               case 3:     // carrier sensed, we are on correct frequency
                  centered = TRUE;
                  break;

               case 4:     // carrier sensed, need to raise frequency
                  freq = freq + 1;
                  if ( DlkI2CHWWrite3( (BYTE)m_I2CAddress, HIBYTE( freq ), LOBYTE( freq ) ) == Fail )
                     return ( TV_ERR_I2C_ERROR );
                  break;

               case 5:     // carrier not sensed, search using offset table
                  freq = div + freqdelta[n];
                  if ( DlkI2CHWWrite3( (BYTE)m_I2CAddress, HIBYTE( freq ), LOBYTE( freq ) ) == Fail )
                     return ( TV_ERR_I2C_ERROR );
                  break;

               default:
                  break;

            } // switch

//------------
// Increment n to control loop and access next frequency search offset
//------------
            n++;
            m = 0;         // Reset PLL lock attempt counter

        } // if phase locked

//--------------------------
// Else PLL not locked.
// Don't increment n.  Simply loop again until PLL is locked.
//--------------------------
        else {
            m++;
        } // else not phase locked

//--------------------------
// If we detected centered frequency,
//   update our table of frequencies (for fast tuning next time),
//   break out of loop (we are done),
// else repeat loop.
// If phase lock had not been detected, we did not change tuner settings,
//   we will just repeat the time delay, and check again for phase lock.
//--------------------------
        if (centered) {
            SetDividerWord( channel, mode, (WORD)freq );
            break; // out of for loop

        } // if centered

    } // for ITERATIONS iterations or for max PLL_ATTEMPTS to lock

//----------------------------------------------------
// Tuning loop is done, let's see how we did
//----------------------------------------------------

//--------------------------
// If we found a carrier, restart Capture.
// Notify Capture Driver of current channel
//--------------------------
   if ( centered )
   {
      return ( TV_ERR_SUCCESS );
   }

//--------------------------
// Else no carrier, return why we didn't tune right
//--------------------------
   else
   {
      // if we didn't find a carrier, restore I2C to table value
      DlkI2CHWWrite3( (BYTE)m_I2CAddress, HIBYTE( freq ), LOBYTE( freq ) );
      return ( TV_ERR_NO_CARRIER );
   }
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int AlpsTuner::GetStatus( void )
// Purpose: Get status from tuner
// Input:   None
// Output:  None
// Return:  status if success, else -1
/////////////////////////////////////////////////////////////////////////////
int AlpsTuner::GetStatus( void )
{
   DWORD time;             // get the current time
   BYTE status;
   for ( int m = 0; m < PLL_ATTEMPTS; m++ )
   {
      time = timeGetTime(); // get current time
      while (timeGetTime() - time < PLL_SETTLE_DELAY ) {
              // SPIN waiting for PLL_SETTLE_DELAY
      }

      // get status from the tuner
      if ( DlkI2CHWRead( (BYTE)m_I2CAddress, &status ) == Fail )
         return ( -1 );    // error reading

      if ( status & 0x40 )   // PLL locked?
         return ( status & 0x07 );
   }
   return ( -1 );    // PLL never locked
}

/////////////////////////////////////////////////////////////////////////////
// Method:  LONG AlpsTuner::SetRadioFreqAFT( int freq )
// Purpose: Set radio frequency with Automatic Fine Turning
// Input:   int freq - frequency to set to (no decimal place; i.e. 1007 = 100.7MHz)
// Output:  None
// Return:  Radio control error value
// Note:    The algorithm to seek a station using fine tuning is to look for
//          signal 0 or 1 to the left or 4 or 5 to the right of the "supposed" freq.
//          If any of the status is found, station exists and search terminates.
//          Otherwise continue with same checking after adjusting the "supposed" freq
//          to left by 2 steps and to right by 2 steps.
/////////////////////////////////////////////////////////////////////////////
LONG AlpsTuner::SetRadioFreqAFT( int freq )
{
   BYTE  status;              // Tuner status, including AFC A/D value
   BOOL  centered = FALSE;    // "tuned in" flag

   //--------------------------
   // range checking for frequency value
   //--------------------------
   if ( ( freq < MIN_FREQ ) || ( freq > MAX_FREQ ) )
      return ( (LONG)RADIO_ERR_BAD_VALUE );

   //--------------------------
   // Calculate program divider
   //--------------------------
   WORD div = 32 * freq / 10 + 1320;

   //--------------------------
   // Use FM band in fast tune mode
   //--------------------------
   if ( Bt848I2CWrite( (BYTE)m_I2CAddress, RADIO_FM | FM_FAST_TUNE, div ) == Fail )
      return ( RADIO_ERR_I2C_ERROR );

   //--------------------------
   // status = 0, 1, 4 or 5 means station found
   // 0 or 1 = station is off to less than "supposed" freq; need to set freq to 1 step less
   // 4 or 5 = station is off to more than "supposed" freq; need to set freq to 1 step more
   //--------------------------
   status = GetStatus();
   switch ( status )
   {
      case 0:
      case 1:
         centered = TRUE;
         div--;
         break;
         
      case 4:
      case 5:
         centered = TRUE;
         div++;
         break;
   }
         
   //--------------------------
   // if status is 2 or 3 = no station exists; try 2 steps less
   //--------------------------
   if ( !centered )
   {
      // Use FM band in fast tune mode
      if ( Bt848I2CWrite( (BYTE)m_I2CAddress, RADIO_FM | FM_FAST_TUNE, div - 2 ) == Fail )
         return ( RADIO_ERR_I2C_ERROR );
      
      // status = 0, 1, 4 or 5 means station found
      status = GetStatus();
      switch ( status )
      {
         case 0:
         case 1:
            centered = TRUE;
            div--;
            break;
         
         case 4:
         case 5:
            centered = TRUE;
            div++;
            break;
      }
   }
         
   //--------------------------
   // if status is 2 or 3 = no station exists; try 2 steps more
   //--------------------------
   if ( !centered )
   {
      // Use FM band in fast tune mode
      if ( Bt848I2CWrite( (BYTE)m_I2CAddress, RADIO_FM | FM_FAST_TUNE, div + 2 ) == Fail )
         return ( RADIO_ERR_I2C_ERROR );
      
      // status = 0, 1, 4 or 5 means station found
      status = GetStatus();
      switch ( status )
      {
         case 0:
         case 1:
            centered = TRUE;
            div--;
            break;
         
         case 4:
         case 5:
            centered = TRUE;
            div++;
            break;
      }
   }
         
   //--------------------------
   // if status is 2 or 3, conclude that no station exists
   //--------------------------
   if ( !centered )
   {
      WORD tempDiv = 32 * ( freq - 2 ) / 10 + 1320;   // ( - 2 ) is US specific (odd only)!!!
      
      // Use FM band in fast tune mode
      if ( Bt848I2CWrite( (BYTE)m_I2CAddress, RADIO_FM | FM_FAST_TUNE, tempDiv ) == Fail )
         return ( RADIO_ERR_I2C_ERROR );
      
      GetStatus();
   }      

   //----------------------------------------------------
   // use normal tuning mode to get best signal-to-noise ratio
   //----------------------------------------------------
   if ( Bt848I2CWrite( (BYTE)m_I2CAddress, RADIO_FM | FM_NORMAL_TUNE, div ) == Fail )
      return ( RADIO_ERR_I2C_ERROR );

   //----------------------------------------------------
   // Tuning loop is done, let's see how we did
   //----------------------------------------------------
   if ( centered )
      return ( RADIO_ERR_SUCCESS );
   else
      return ( RADIO_ERR_NO_CARRIER );
}

//===========================================================================
// Following are the functions communicate with I2C
//===========================================================================

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode Bt848I2CWrite( BYTE I2CAddress, WORD msw, WORD lsw )
// Purpose: Write data to I2C
// Input:   BYTE I2CAddress - address of I2C
//          WORD msw        - most sigificant word
//          WORD lsw        - least sigificant word
// Output:  None
// Return:  Success or Fail
// Note:    This function is provided for the purpose to communicate with I2C
/////////////////////////////////////////////////////////////////////////////
ErrorCode Bt848I2CWrite( BYTE I2CAddress, WORD msw, WORD lsw )
{
   // write HIWORD to I2C
   if ( DlkI2CHWWrite3( I2CAddress, HIBYTE( msw ), LOBYTE( msw ) ) == Fail )
      return ( Fail );

   // write LOWORD to I2C
   if ( DlkI2CHWWrite3( I2CAddress, HIBYTE( lsw ), LOBYTE( lsw ) ) == Fail )
      return ( Fail );

   return ( Success );
}


