// $Header: G:/SwDev/W95/Video/pisces/dvlpmnt/Grappler/rcs/tuner.h 2.11 1998/05/21 19:05:57 rudyl Exp $
// $Log: tuner.h $
// Revision 2.11  1998/05/21 19:05:57  rudyl
// Graft from 2.10.2.2.
// Revision 2.10.2.2  1998/05/21 19:05:32  rudyl
// Added Nicam support. Works with Philips TDA9874H only. ITT is next.
// Revision 2.10  1997/10/23 00:57:04  rudyl
// Added Tv power down mode to fix this problem:
//    // If Alps tuner was specified in the registry, invalid control codes
//    // will be sent to the Philips tuner puting it in some nonexist state causing
//    // it not able to tune correctly. Change tuner to any TV mode, e.g. power down,
//    // before regular radio tuning would fix this problem.
// Revision 2.9  1997/10/10 19:43:19  rudyl
// Added member variable m_brand to store tuner brand.
// Revision 2.8  1997/10/09 00:19:48  rudyl
// Use Temic/Philips TV AFT codes for Alps tuner.
// Only modified handling of status range 0-5.
// Revision 2.7  1997/10/08 01:44:21  rudyl
// The absolute FM frequency min is 64MHz
// Revision 2.6  1997/10/08 00:57:01  rudyl
// Added Alps TSCH5 tuner support.
// Revision 2.5  1997/08/14 20:13:58  rudyl
// Use 2.3 which added Gemini support.
// Revision 2.3  1997/07/16 01:22:49  rudyl
// Added member function to get radio signal strength.
// Revision 2.2  1997/07/01 01:32:22  rudyl
// Added TunerWithFMRadio class to support gemini.
// Revision 2.1  1996/06/10 20:12:38  ssm
// MarkBeta1AsRev2
// Revision 1.5  1996/05/10 23:15:48  rudyl
// Implement TV_ERR_... return values from Intel. Also added exported function GetTVErrString() to retrieve our specific error msg.
// Revision 1.4  1996/05/06 18:49:06  rudyl
// No changes; just added more comments into codes.
// Revision 1.3  1996/04/28 01:20:14  rudyl
// ver 0.50 - does not instantiate I2C and GPIO objects here. Just make API
// calls to BT848_32.DLL. Also returns error types to caller instead of just TRUE/FALSE result.
// Revision 1.2  1996/04/12 21:01:11  rudyl
// Renamed DLL from 848Tuner.dll to Grappler.dll.
// Revision 1.1  1996/04/12 18:24:44  rudyl
// Initial revision

// CUT_HERE

/////////////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 1996 Brooktree Corporation
//
//  Module:
//
//    Tuner.h
//
//  Abstract:
//
//    Bt848 Tuner class header file
//
/////////////////////////////////////////////////////////////////////////////

#ifndef __TUNER_H
#define __TUNER_H

#include "retcode.h"

/////////////////////////////////////////////////////////////////////////////
// Constants
/////////////////////////////////////////////////////////////////////////////
// US: 87.5 - 108.0, Japan 76 - 91, Eastern Europe 64 - 72
const int MIN_FREQ = 640;     // no decimal place; i.e. 64.0MHz -> 640
const int MAX_FREQ = 1080;

/////////////////////////////////////////////////////////////////////////////
// Typedefs
/////////////////////////////////////////////////////////////////////////////
typedef WORD TunerCodes;
typedef TunerCodes* ptrTunerCodes;
typedef WORD ControlWord;



/////////////////////////////////////////////////////////////////////////////
// tuner brands
/////////////////////////////////////////////////////////////////////////////
enum { brandPhilips, brandTemic, brandAlps };


/////////////////////////////////////////////////////////////////////////////
// tuner has 2 different tuning modes: tv (ie. broadcast) or cable
/////////////////////////////////////////////////////////////////////////////
enum { modeTV, modeCable };


/////////////////////////////////////////////////////////////////////////////
// video formats
/////////////////////////////////////////////////////////////////////////////
enum { formatNTSC, formatPAL };


////////////////////////////////////////////////////////////////////
// tuner uses 3 different bands based on desired channel & mode
/////////////////////////////////////////////////////////////////////////////
enum { bandVHF_LOW, bandVHF_HIGH, bandUHF };


/////////////////////////////////////////////////////////////////////////////
// struct for associating a channel with a band
/////////////////////////////////////////////////////////////////////////////
typedef struct tagTunerBandStruct
{
   int minChannel;
   int maxChannel;
   int band;
} TunerBandStruct;
typedef TunerBandStruct* ptrTunerBandStruct;



/////////////////////////////////////////////////////////////////////////////
// Tuner Class
//
// Description:
//    This class encapsulates the tuner behavior of the Bt848. It provides
//    "SetChannel()" and "GetChannel()" to set and retrieve channel using the
//    tuner. To instantiate this class, the caller needs to supply 3 parameters
//    as defined in the constructor:
//       Tuner( int brand, int format, BYTE I2CAddress );
//          where brand can be either brandPhilips or brandTemic,
//                format can be either formatNTSC or formatPAL,
//                I2CAddress is the address of the tuner.
//    This class also incorporate Automatic Fine Tuning (AFT) when tuning a
//    channel. It will scan up to +/- 2 1/2 MHz from the predefined setting
//    in order to obtain optimum image of the channel.
//
/////////////////////////////////////////////////////////////////////////////
class Tuner
{
protected:
   // constants for PLL
   static const WORD PLL_SETTLE_DELAY; // check each 30 millisec to see if PLL settled
                                       // pll can take up to 150millisec to settle
   static const WORD PLL_ATTEMPTS;     // # attempts to lock in PLL for each iteration
                                       // Attempts * Delay should be ~150msec

   int  m_channel;      // which channel is set to
   int  m_format;       // video foramt (ntsc or pal)
   BYTE m_I2CAddress;   // I2C address
   int  m_brand;        // tuner brand ( brandPhilips, brandTemic, brandAlps )

   // ptrs to lookup tables
   ptrTunerCodes	    m_ptrTunerCodeArray;
   ControlWord *      m_ptrControlWordArray;
// --- TCZ country stuff
   int				m_fCarrier;
   int				m_maxChannels;
   ptrTunerCodes	m_ptrTunerXOverArray;
   char**			m_ptrTunerIdArray;
   virtual void     SetCountrySpecifics( int mode );
// --- <end>

   ////////////////////////////////////
   // Internal functions
   ////////////////////////////////////
   virtual WORD   GetDividerWord( int channel, int mode );
   virtual void   SetDividerWord( int channel, int mode, WORD value );
   virtual WORD   GetControlWord( int channel, int mode );
   virtual LONG   SetChannelAFT ( int channel, int mode );
   virtual WORD   MaxNumberOfChannel( int mode );

public:
   Tuner( int brand, int format, BYTE I2CAddress );
   ~Tuner();

   ////////////////////////////////////
   // Public member functions
   ////////////////////////////////////
   virtual LONG   SetChannel( int channel, int mode );
   virtual int    GetChannel( void );
// --- TCZ channel Id
//   virtual char*  GetChannelId( void );
// --- TCZ <end>
};


/////////////////////////////////////////////////////////////////////////////
// TunerWithFMRadio Class
//
// Description:
//    This class is derived from the "Tuner" class with added FM radio support
//    for Philips FM1236 video & FM tuner.
//
// Constructor:
//    TunerWithFMRadio( int brand, int format, BYTE I2CAddress );
//          where brand can be either brandPhilips or brandTemic,
//                format can be either formatNTSC or formatPAL,
//                I2CAddress is the address of the tuner.
//
// Methods:
//    LONG  SetRadioFreq( int freq )         - set radio frequency
//    int   GetRadioFreq( void )             - get radio frequency tuner was set to
//    int   GetRadioSignalStrength( void )   - get radio signal srength
//
/////////////////////////////////////////////////////////////////////////////
class TunerWithFMRadio : public Tuner
{
protected:
   // basic control code to use Philips FM tuner, bitwise OR this with desired mode
   static const WORD TV_POWERDOWN;     // Philips TV power down mode
   static const WORD RADIO_FM;         // base Philips FM tuner code

   // bitwise OR these values with RADIO_FM to turn on certain mode
   static const WORD FM_FAST_TUNE;     // CP = 1
   static const WORD FM_NORMAL_TUNE;   // CP = 0
   static const WORD FM_AFC_MODE;      // P0 = 1
   static const WORD FM_RF_MODE;       // P0 = 0
   static const WORD FM_MONO;          // P1 = 1
   static const WORD FM_MUTE;          // P3 = 1

protected:
   int   m_FMFreq;       // which radio frequency is set to

   virtual LONG  SetRadioFreqAFT( int freq );    // set radio frequency with automatic fine tuning

public:
   TunerWithFMRadio( int brand, int format, BYTE I2CAddress );
   ~TunerWithFMRadio();

   virtual LONG   SetRadioFreq( int freq );
   virtual int    GetRadioFreq( void );
   virtual int    GetRadioSignalStrength( void );
};

/////////////////////////////////////////////////////////////////////////////
// AlpsTuner Class
//
// Description:
//    This class is derived from the "TunerWithFMRadio" class designed for
//    the Alps TSCH5 tuner.
//
// Constructor:
//    AlpsTuner( int brand, int format, BYTE I2CAddress );
//          where brand is brandAlps,
//                format can be either formatNTSC or formatPAL,
//                I2CAddress is the address of the tuner.
//
/////////////////////////////////////////////////////////////////////////////
class AlpsTuner : public TunerWithFMRadio
{
protected:
   static const WORD PLL_SETTLE_DELAY; // check each 100 millisec to see if PLL settled
   static const WORD PLL_ATTEMPTS;     // # attempts to lock in PLL for each iteration
   static const WORD RADIO_FM;         // Alps FM radio mode with 31.25 kHz step size

protected:
   //==================================
   // Internal functions
   //==================================
   virtual LONG   SetChannelAFT ( int channel, int mode );
   virtual LONG   SetRadioFreqAFT( int freq );    // set radio frequency with automatic fine tuning
   virtual int    GetStatus( void );      // get status from tuner

public:
   AlpsTuner( int brand, int format, BYTE I2CAddress ) :
            TunerWithFMRadio( brand, format, I2CAddress ) {};
   ~AlpsTuner() {};

   inline virtual int GetRadioSignalStrength( void ) { return 0; }   // no signal strength ability
};
#endif // __TUNER_H
