/***************************************************************************
* Module:   bt878api.h                                                     *
*                                                                          *
* Description:                                                             *
*     Documents the API for this DLL.                                      *
*                                                                          *
* [copyright, 97]                                                          *
*                                                                          *
***************************************************************************/

/***************************************************************************
* [start cut]
* $Log: Bt878api.h $
* Revision 1.11  1998/05/15 11:48:30  trang
* Implemented for WaveIn driver that spawned out the DMA process.
* Revision 1.10  1998/02/25 23:33:47  mdimeo
* DSP calibration, fixed crackle(?), separation measurement debug mode.
* Revision 1.9  1997/10/17 23:34:05  tomz
* Fix per rudy's comments
* Revision 1.8  1997/10/17 22:33:26  tomz
* Added some codes and moved some around
* Revision 1.7  1997/07/09 16:14:29  tomz
* Get ErrorCode definitions from the right place (retcode.h)
* Revision 1.6  1997/07/02 21:06:55  tomz
* Remove dead code, cleanup
* Revision 1.5  1997/07/02 14:50:37  tomz
* Fix calling convention, continue move to single entrypoint
* Revision 1.4  1997/07/01 23:48:24  tomz
* Moving to single entrypoint, multiple command.
* Also fixing declarations to match BT848.
* Revision 1.3  1997/06/20 20:59:44  tomz
* Revision 1.2  1997/06/20 20:40:18  tomz
* Revision 1.1  1997/06/19 23:01:50  tomz
* Initial revision
* [end cut]
***************************************************************************/

#ifndef  BT878API_H     // Make sure we don't include this file twice
#define  BT878API_H

#include <windows.h>
#include "def__.h"      // define __BT878CLASS and __BT878FUNC
#include "retcode.h"    // define ErrorCode, Sucess, and Failure

#ifdef __cplusplus
extern "C"
{
#endif

typedef enum {
   MUX_IN_INVALID,
   MUX_IN_TV,
   MUX_IN_RADIO,
   MUX_IN_LINE_IN
} AUDIO_INPUT;

typedef void ( WINAPI *FN_STEREO_CALLBACK )( BOOL bStereoDetected );

/***************************************************************************
*  Interfaces in DLL expected to be called from applications               *
***************************************************************************/

#define CMD_DEVICE_OPEN             0x10
#define CMD_DEVICE_CLOSE            0x11

#define CMD_SET_INPUT_SELECTION     0x20
#define CMD_SET_STEREO_STATE        0x21
#define CMD_SET_DBX_STATE           0x22
#define CMD_SET_SAMPLE_FREQ         0x23
#define CMD_SET_SAMPLE_RES          0x24
#define CMD_SET_CAPTURE_FILE        0x25

#define CMD_GET_INPUT_SELECTION     0x30
#define CMD_GET_STEREO_STATE        0x31
#define CMD_GET_DBX_STATE           0x32
#define CMD_GET_SAMPLE_FREQ         0x33
#define CMD_GET_SAMPLE_RES          0x34
#define CMD_GET_CAPTURE_FILE        0x35

#define CMD_START_PLAYING           0x40
#define CMD_END_PLAYING             0x41
#define CMD_START_CAPTURE           0x42
#define CMD_END_CAPTURE             0x43
#define CMD_MUTE                    0x44
#define CMD_UNMUTE                  0x45

#define CMD_SET_STEREO_CALLBACK     0x50
#define CMD_GET_CAPS                0x51
#define CMD_DUMP_STATUS             0x52

#define CMD_SET_WBE_CALIB           0x60
#define CMD_SET_SE_CALIB            0x61
#define CMD_GET_SEPARATION          0x62
#define CMD_GET_WBE_CALIB           0x63
#define CMD_GET_SE_CALIB            0x64

#define CMD_START_AUDIO             0x70
#define CMD_STOP_AUDIO              0x71

#define CAPS_STEREO_SUPPORTED       0x00000001
#define CMD_CODE ULONG

typedef   DWORD ( WINAPI *ProcessCommand_t )( CMD_CODE cmd, DWORD dwHandle, DWORD dwExtra );
DWORD __BT878FUNC ProcessCommand ( CMD_CODE cmd, DWORD dwHandle, DWORD dwExtra );

#define BTDMA_EVENT_NAME			"BtDmaEvent"

/*********************************************************************
*  Radio      Stereo     Stereo    Dbx     Quality                   *
*   TV       Detected    State    State                              *
* LineIn                                                             *
**********************************************************************
* LineIn        x           x        x      Mono                     *
*    x          0           x        x      Mono                     *
*    x          x           0        x      Mono                     *
*   TV          1           1        0      Stereo, poor seperation  *
*   TV          1           1        1      Stereo                   *
*  Radio        1           1        x      Stereo                   *
*********************************************************************/

#ifdef __cplusplus
}
#endif

#endif   // BT878API_H

