/* File: Compat.h
 * Purpose: Defines compatibility data types that are common between
 *   16 and 32 bit code
 * Author: sasha
 * $Log: COMPAT.H $
 * Revision 2.4  1998/05/12 20:47:59  dtharp
 * Added default constructor for StartParms class.
 * Revision 2.3.1.2  1998/05/12 00:58:16  dtharp
 * First checkpoint: "graceful degredation" mode QS capture from overlay.
 * Revision 2.3  1998/04/06 19:42:26  dtharp
 * Vxd Frame Buffers are allocated on the fly instead of at init time.
 * Revision 2.2  1998/01/09 17:29:27  peter
 * Graft Trident fixes to tip.
 * Revision 2.1.3.1  1997/12/10 01:40:30  peter
 * Merged TRIDENT code.
 * Revision 2.1  1996/06/10 20:13:27  ssm
 * MarkBeta1AsRev2
 * Revision 1.3  1995/10/18 07:01:56  sasha
 * First major check-in for testing
 * Revision 1.2  1995/09/30 22:14:12  sasha
 * Just to make Mike happy
 * Revision 1.1  1995/08/21 04:10:40  SASHA
 * Initial revision
 */

#ifndef __COMPAT_H
#define __COMPAT_H

#ifndef __MYTYPES_H
#include "mytypes.h"
#endif

/* Class: SmallRect
 * Purpose: This class is an intermediate link between 16 bit RECT structure
 *   and 32 bits TRect. It is created to avoid manual assignment to the members
 *   of TRect. It will be used as a member of a parameters structure passed
 *   from ring-3 code ( where int is 16 bits ) to VxD.
 * Attributes: left, top, right, bottom. All 16 bit integers
 * Operations: operator Trect() - conversion to the TRect.
 * Note: The class is used in the following way:
 *   1. It is created in ring3 with RECT.
 *   2. In the VxD it is converted to TRect.
 */
class SmallRect
{
   DWORD   left;
   DWORD   top;
   DWORD   right;
   DWORD   bottom;

   public:
      operator TRect() { return TRect( (int)left, (int)top, (int)right, (int)bottom ); }

      SmallRect( RECT &r ) : left( r.left ), top( r.top ),
         right( r.right ), bottom( r.bottom ) {}

      SmallRect( int l, int t, int r, int b ) : left( l ), top( t ), right( r ),
         bottom( b ) {}
      SmallRect() : left( 0 ), top( 0 ), right( 0 ), bottom( 0 ) {}
};

class StartParms
{
   public:
      SmallRect  Analog;
      SmallRect  Digital;
      DWORD      ColFmt;
      DWORD      callback;
      DWORD      tag;
      DWORD      dwPitch;
      DWORD      FrameRate;
      DWORD      TridentAperture0 ;
      DWORD      TridentAperture1 ;
      DWORD      ImageSize ;
      StartParms( ) : dwPitch( 0 ),
         Analog(  ), Digital(  ), ColFmt( -1 ), callback( 0 ), tag( 0 ),
         TridentAperture0(0), TridentAperture1(0), ImageSize(0) {}

      StartParms( SmallRect &an, SmallRect dig, DWORD col ) : dwPitch( 0 ),
         Analog( an ), Digital( dig ), ColFmt( col ), callback( 0 ), tag( 0 ),
         TridentAperture0(0), TridentAperture1(0), ImageSize(0) {}
};

#endif