/* Regutil.h
 * Purpose: Contains definition Windows registry functions wrapper class
 * Author: sasha
 * $Log: REGUTIL.H $
 * Revision 2.13  1997/12/12 23:58:43  peter
 * Fixed REG_BINARY problem.
 * Revision 2.12  1997/12/11 17:13:13  peter
 * Added REG_BINARY support to btutil*.dll.
 * Revision 2.11  1997/12/10 00:31:44  peter
 * Revision 2.10  1997/12/09 22:18:29  peter
 * Several changes to fix 16/32-bit thunk exception.
 * Revision 2.9  1997/12/03 21:57:14  peter
 * Fixed regutil 16/32-bit incompatibility.
 * Revision 2.8  1997/11/27 02:50:14  peter
 * Implemented set/get for REG_BINARY registry types.
 * Revision 2.7  1997/11/26 19:30:52  peter
 * Keep string as member var to represent subkey from ctor.  Use this SetValue() to
 * remember current subkey.
 * Revision 2.6  1997/11/26 18:16:34  peter
 * Use return value from thunk for return value from member fn.
 * Revision 2.5  1997/11/26 02:12:57  peter
 * Allow old behavior of RegUtil if HKEY_CLASSES_ROOT explicitly 
 * specified.
 * Revision 2.4  1997/11/26 02:08:07  peter
 * Fixed MKS graft from branch problem.
 * Revision 2.2.2.2  1997/11/26 01:52:17  peter
 * Reg key move to HKEY_LOCAL_MACHINE.
 * Revision 2.2.2.1  1997/11/20 22:46:49  peter
 * Changes to move registry subkey to HKEY_LOCAL_MACHINE.
 * Revision 2.2  1997/07/10 00:20:08  trantq
 * Need to include string.h if compile with MSVC.
 * Revision 2.1  1996/06/10 20:13:29  ssm
 * MarkBeta1AsRev2
 * Revision 1.2  1995/09/30 22:12:58  sasha
 * Just to make Mike happy
 * Revision 1.1  1995/08/28 04:55:10  sasha
 * Initial revision
 */

#ifndef __REGUTIL_H
#define __REGUTIL_H

#ifndef __BORLANDC__ 	// Microsoft
#include <string.h>
#endif

#ifndef  __STDIO_H
#include <stdio.h>
#endif

#ifndef __SHELLAPI_H
#include <shellapi.h>
#endif

#ifndef __MYTYPES_H
#include "mytypes.h"
#endif

#ifndef __VIDDEFS_H
#include "viddefs.h"
#endif

#define BT_VIDEO_REG_SUBKEY	"SOFTWARE\\Brooktree\\Video"
#define BT_AUDIO_REG_SUBKEY	"SOFTWARE\\Brooktree\\Audio"


#if !defined(WIN32) && !defined(__FLAT__)
typedef int  ( WINAPI *REGUTILCREATEKEY_t )( LPCSTR );
typedef int  ( WINAPI *REGUTILSETVALUE_t )( LPCSTR, LPCSTR );
typedef int  ( WINAPI *REGUTILQUERYVALUE_t )( LPCSTR, LPSTR, LONG* );
typedef int  ( WINAPI *REGUTILQUERYBINVALUE_t )( int*, LPSTR, LPCSTR, LPCSTR );
typedef int  ( WINAPI *REGUTILSETBINVALUE_t )( int, LPSTR, LPCSTR, LPCSTR );
#endif


class RegUtil
{
private:
   HKEY	      hKey_;
   char	      m_szKeyName[ 257 ];

#if !defined(WIN32) && !defined(__FLAT__)
   static      HINSTANCE		m_hDll;
#endif

public:
               RegUtil( LPCSTR SubKeyName, HKEY = NULL );
				   ~RegUtil();
   ErrorCode	GetValue( LPDWORD lpValue, LPCSTR szSubKey = "" );
   ErrorCode	GetValue( LPSTR   lpValue, LPLONG lpByteCount, LPCSTR szSubKey = "" );
   void		   SetValue( DWORD dwValue, LPCSTR szSubKey = "" );
   void		   SetValue( LPCSTR szValue, LPCSTR szSubKey = "" );
   ErrorCode	GetBinValue( int* plSize, LPSTR cbData, LPSTR szSubKeyName );
   ErrorCode	SetBinValue( int lSize, LPSTR cbData, LPSTR szSubKeyName );
};


#endif



