/* File VidSett.h
 * Purpose: Defines video settings structure
 * Author: sasha
 * $Log: VIDSETT.H $
 * Revision 2.8  1998/02/26 20:20:42  rudyl
 * Removed QSightParam[] item from structure. Interface to VPAEC directly.
 * Revision 2.7  1998/02/24 20:41:12  rudyl
 * Graft from 2.6.1.2.
 * Revision 2.6.1.2  1998/02/24 20:40:55  rudyl
 * Changed "TrueView" to "Interface" and added QSightParam.
 * Revision 2.6.1.1  1997/12/13 01:48:08  rudyl
 * Duplicate revision
 * Revision 2.5.1.1  1997/11/27 03:10:35  rudyl
 * Moved SRAM table to dialogs.dll.
 * Revision 2.5  1997/11/26 06:28:27  rudyl
 * Added support of Scotch.
 * Revision 2.4  1997/10/29 02:18:06  rudyl
 * Added TrueView.
 * Revision 2.3  1997/10/21 00:35:20  rudyl
 * Added support to expected total line count by decoder.
 * Revision 2.2  1997/05/05 16:58:10  dtharp
 * 848A Merge: from 2.1.1.3
 * Revision 2.1.1.3  1997/02/14 23:49:37  rudyl
 * Removed all #ifdef BT848A. Use same binary files for both Bt848 and Bt848A.
 * Revision 2.1.1.2  1997/02/03 23:46:46  rudyl
 * First Pisces+ check in.
 * Revision 2.1.1.1  1996/06/10 20:13:30  rudyl
 * Start Pisces+ work.
 * Revision 2.1  1996/06/10 20:13:30  ssm
 * MarkBeta1AsRev2
 * Revision 1.3  1995/09/27 12:18:20  sasha
 * First major check-in for testing
 * Revision 1.2  1995/09/27 04:18:20  sasha
 * Just to make Mike happy
 * Revision 1.1  1995/09/07 05:54:08  sasha
 * Initial revision
 */

#ifndef __VIDSETT_H
#define __VIDSETT_H

/* Class: PLLInfo
 * Purpose: Used as interface between capture driver and VxD to pass parameters
 *          to PLL
 * Attributes:
 *     DWORD f_lo - PLL_F_LO reg value
 *     DWORD f_hi - PLL_F_HI reg value
 *     DWORD i    - PLL_I    reg value
 *     DWORD c    - PLL_C    reg value
 *     DWORD x    - PLL_X    reg value
 */
class PLLInfo
{
   public:
      DWORD f_lo;
      DWORD f_hi;
      DWORD i;
      DWORD c;
      DWORD x;
      PLLInfo( DWORD parm1, DWORD parm2, DWORD parm3, DWORD parm4, DWORD parm5 ) :
         f_lo( parm1 ), f_hi( parm2 ), i( parm3 ), c( parm4 ), x( parm5 ) {}
};

/* Class: VideoInfo
 * Purpose: Used as interface between capture driver and VxD to pass parameters
 *          to scaler
 * Attributes:
 *     DWORD HActive       - number of active horizontol pixels
 *     DWORD HDelay        - number of pixels delayed before actual signal
 *     DWORD VActive       - VActive register value
 *     DWORD VDelay        - VDelay register value
 *     DWORD linesPerField - number of active lines in each field
 */
class VideoInfo
{
   public:
      DWORD HActive;
      DWORD HDelay;
      DWORD VActive;
      DWORD VDelay;
      DWORD linesPerField;
      VideoInfo() : HActive( 1 ), HDelay( 1 ), VActive( 1 ), VDelay( 1 ),
         linesPerField( 1 ) {}
      VideoInfo( DWORD parm1, DWORD parm2, DWORD parm3, DWORD parm4, DWORD parm5 ) :
         HActive( parm1 ), HDelay( parm2 ), VActive( parm3 ), VDelay( parm4 ),
         linesPerField( parm5 ) {}
};

/* Type: VideoSettings
 * Purpose: This structure is used to pass decoder related information between
 *   capture driver and VxD
 * Note: DWORD is used for straight compatibility with the VxD land
 */
typedef struct
{
   DWORD  CurConn;
   DWORD  SVideo;
   LONG   Bright;
   LONG   Contrast;
   LONG   Hue;
   LONG   Satur;
   DWORD  VFormat;
   DWORD  SigFmt;
   DWORD  SyncRef;
   DWORD  SyncOutput;
   DWORD  InClk;
   DWORD  OutClk;
   DWORD  PLLFreq;
   DWORD  HReset;
   DWORD  RefFreq;
   VideoInfo videoInfo;
   DWORD  TotalLineCount;
   DWORD  Interlace;
   DWORD  QuartzSight;
   DWORD  TimGenSize;         // timgen are not sent to VxD
   BYTE   TimGenByte[ 100 ];
} VideoSettings;


/* Class: ChanInfo
 * Purpose: This structure is used for the operation of opening a video channel
 *   from capture driver to VxD
 * Note: It is quite possible that this method of passing data to/from a VxD is
 *   dangerous
 */
class ChanInfo
{
   private:
      ChanInfo();

   public:
      DWORD VidChan;      // VxDChannel enum is used inside capture driver to
                               // initialize this member
      LPDWORD lpHandleAddress;
      ChanInfo( VideoStream strm, LPDWORD lpAddr ) : VidChan( strm ),
         lpHandleAddress( lpAddr ) {}
};

/* Class: AllocBufStruct
 * Purpose: Used as interface between capture driver and VxD.
 * Attributes: ppBufAddr: LPBYTE * - pointer to pointer that holds address of the
 *   data buffer
 *   dwSize: - DWORD - size of the buffer to allocate
 */
class AllocBufStruct
{
   public:
      LPBYTE *ppBufAddr;
      DWORD  dwSize;
      AllocBufStruct() : ppBufAddr( NULL ), dwSize( NULL ) {}
      AllocBufStruct( LPBYTE *ppAddr, DWORD dwS ) : ppBufAddr( ppAddr ),
         dwSize( dwS ) { *ppBufAddr = NULL; }
};

#endif
