/* File : Vxdfunc.h
 * Purpose: defines functions for the VxD interface
 * Author: sasha
 * $Log: VXDFUNC.H $
 * Revision 2.21  1998/05/22 21:35:54  dtharp
 * Added size pointers to buffer instrumentation.
 * Revision 2.20  1998/04/22 16:05:22  dtharp
 * Added FNOpenDriver and FNCloseDriver
 * Revision 2.19  1998/04/16 00:11:44  dtharp
 * Added FNGetR3BufCount as a means for ring3 to tell how many buffers are
 * currently queued in the vxd.
 * Revision 2.18  1998/04/02 21:43:19  dtharp
 * Merge DT_VIDEOHDR branch. Implements a new buffer 
 * management scheme.
 * Revision 2.17.1.1  1998/03/29 18:55:30  dtharp
 * Added Instrumentation call
 * Revision 2.17  1998/02/24 20:48:35  rudyl
 * Merged 2.15.2.2 & 2.16.
 * Revision 2.16  1998/02/24 07:12:26  dtharp
 * Graft from 2.15.1.2
 * Revision 2.15.1.2  1998/02/10 03:35:01  dtharp
 * Added FN numbers for debug of VVL interface
 * Revision 2.15  1997/10/29 18:32:27  dtharp
 * Merged TrueView branch
 * Revision 2.14.1.1  1997/10/29 00:13:31  dtharp
 * TrueView Branch
 * Revision 2.14  1997/10/22 23:41:12  dtharp
 * Added hwnd member to SOUND_SYNC
 * Revision 2.13  1997/10/21 00:35:21  rudyl
 * Added support to expected total line count by decoder.
 * Revision 2.12  1997/09/16 17:08:24  dtharp
 * Added dwVsync member to interrupt count typedef.
 * Revision 2.11  1997/09/16 16:32:23  dtharp
 * Merged AVI processing branch.
 * Revision 2.10.1.1  1997/08/19 16:01:49  dtharp
 * Branch for gemini Avi post processing
 * Revision 2.10  1997/07/22 19:27:09  dtharp
 * Added #ifdef __FLAT__ to protect the BT848 PCI Header struct.
 * Revision 2.9  1997/07/22 15:40:17  dtharp
 * Added FNGetPciHdr and Type PCI_CONFIG_848.
 * Revision 2.8  1997/05/05 17:38:08  dtharp
 * 848A Merge: from 2.4.1.3
 * Revision 2.7  1997/05/02 17:23:52  dtharp
 * Added support for interrupt count instrumentation.
 * Revision 2.6  1997/04/18 00:37:29  dtharp
 * Fix for Bug Id 433. Closed Caption Performance (speed, cpu).
 * Revision 2.5.1.1  1997/04/17 22:33:47  dtharp
 * Merging Primary mode CC work with 1.5.0
 * Revision 2.5  1997/03/20 22:24:07  dtharp
 * Grafted WDM buffer managment branch.
 * Revision 2.4.3.2  1997/03/17 16:56:19  dtharp
 * Works @ 30fps NTSC.
 * Revision 2.4.3.1  1997/03/12 21:37:44  dtharp
 * Single field capture kinda works.
 * Revision 2.4  1996/09/29 23:18:52  sasha
 * Added 2 function codes to support 32 bit interrupt notifications.
 * Revision 2.3  1996/08/10 23:42:37  sasha
 * Added one more function code
 * Revision 2.2  1996/06/26 18:11:33  sasha
 * Added FNPause and FNContinue
 * Revision 2.1  1996/06/10 20:13:30  ssm
 * MarkBeta1AsRev2
 * Revision 1.13  1996/05/22 19:58:04  sasha
 * Changed channel notification names
 * Revision 1.12  1996/05/22 18:10:06  rudyl
 * Added I2CPreSetChan in FuncNumber so VxD can be notified channel is about to change.
 * Revision 1.11  1996/04/24 22:48:20  sasha
 * Added FNGetBaseAddr
 * Revision 1.10  1996/03/29 17:40:09  rudyl
 * Added FuncNumber as sub-function in VxDParam structure.
 * Revision 1.9  1996/03/28 23:21:48  rudyl
 * Added VxDParam structure to pass param to VxD
 * Revision 1.8  1996/03/25 19:30:33  rudyl
 * Added I2CSetChan function declaration to set channel through I2C.
 * Revision 1.7  1996/02/09 01:01:41  sasha
 * Added function codes for obtaining GPIO and I2C addresses
 * Revision 1.6  1996/02/08 23:44:24  sasha
 * No changes
 * Revision 1.5  1996/01/15 20:20:21  sasha
 * Added more decoder functions
 * Revision 1.4  1995/12/08 22:30:29  sasha
 * Added FNAddBufLin function code
 * Revision 1.3  1995/10/18 20:05:54  sasha
 * First major check-in for testing
 * Revision 1.2  1995/09/24 02:42:30  sasha
 * Just to make Mike happy
 * Revision 1.1  1995/09/04 23:12:48  sasha
 * Initial revision
 */

#ifndef __VXDFUNC_H
#define __VXDFUNC_H

/* Type: FuncNumber
 * Purpose: Enumerates all VxD functions
 */
typedef enum
{
   FNVer, FNOpenChan, FNCloseChan, FNStart, FNStop, FNAllocBuf, FNFreeBuf,
   FNAddBuf, FNDecoder, FNClip, FNFlushQue, FNAddBufLin, FNSkipCount, DecConn,
   DecBrt, DecSat, DecHue, DecSVid, DecContr, DecFmt, FNGetI2C, FNGetGPIO,
   FNPreSetChan, FNSetChan, FNGetBaseAddr, FNStatus, FNPause, FNContinue, FNStopFlag,
   FNIntStatusStart, FNIntStatusStop, FNDumpRiscProg, FNCCClip, FNGetIntrCounter,
   // followings are added for Bt848A
   DecSigFmt, DecSyncRef, DecSyncOutput, DecHReset, FNScaler, FNPLL,
   FNTimGenLB, FNTimGenInClk, FNTimGenOutClk, FNFldCntr,
   // End 848A
   FNGetPciHdr,
   // Gemini:
   FNRegCapEvent, FNStartCapNotify, FNEndCapNotify, DecLineCount, FNSetInterlaceMode,

   // VVL Debug 
   FNVPAEC_Get_Version, FNVPAEC_Init, FNVPAEC_Iterate, FNVPAEC_Terminate, FNVPAEC_SetParam,
   FNVPAEC_GetParam, FNVPAEC_DebugString,
   
   FNGetBufferInstrumentation,    // 32 bit only
   FNGetR3BufCount,

   FNOpenDriver, FNCloseDriver

} FuncNumber;

/* Type: FuncType
 * Purpose: Types of function requested
 */
typedef enum
{
   SetFunc, GetFunc
} FuncType;

/* Type: VxDParam
 * Purpose: Structure for VxD parameters
 */
typedef struct
{
   FuncNumber subFunc;
   FuncType funcType;
   DWORD param;
} VxDParam;

/* Type: VxDChannel
 * Purpose: Identifies a VxD channel
 */
typedef enum
{
   VS_Below, VS_Capture, VS_Preview, VS_VBIO, VS_VBIE, VS_CC, VS_EDS,
   VS_Raw, VS_Above
} VideoStream;
#define VS_Field1     VS_Capture
#define VS_Field2     VS_Preview
#define VS_VBI1       VS_VBIO
#define VS_VBI2       VS_VBIE
// From wdm:
//typedef enum
//{
//   VS_Below = -1, VS_Field1, VS_Field2, VS_VBI1, VS_VBI2, VS_CC, VS_EDS,
//   VS_Raw, VS_Above
//} VideoStream;


/* Type: Field
 * Purpose: Enumerates video fields
 *
typedef enum { Both, Even, Odd } VidField;
*/
#ifndef __VIDSETT_H
#include "vidsett.h"  // this file needs above definitions
#endif

/* Type: InterruptCounter
 * Purpose: Repository of interrupt counters. 
 * FNGetIntrCounter returns a pointer to one of these. (32 bit interface only)
 */
typedef struct InterruptCounter {
  DWORD dwRisc ;
  DWORD dwGpio ;
  DWORD dwI2c ;
  DWORD dwFbus ;
  DWORD dwFtrgt ;
  DWORD dwFdsr ;
  DWORD dwPperr ;
  DWORD dwRiperr ;
  DWORD dwPabort ;
  DWORD dwOcerr ;
  DWORD dwScerr ;
  DWORD dwVsync ;
  InterruptCounter() : 
                      dwRisc (0),
                      dwGpio (0),
                      dwI2c (0),
                      dwFbus (0),
                      dwFtrgt (0),
                      dwFdsr (0),
                      dwPperr (0),
                      dwRiperr (0),
                      dwPabort (0),
                      dwOcerr (0),
                      dwScerr (0),
                      dwVsync (0) {}
} InterruptCounter;


/* Type:
 * Purpose:
 */
typedef struct BufferInstrumentation {
  DWORD * pR3QueueCount ;
  DWORD * pAvailableQueueCount ;
  DWORD * pFilledQueueCount ;

  DWORD * pR3Size ;
  DWORD * pAvailableSize ;
  DWORD * pFilledSize ;

  DWORD   SkippedCount ;
  DWORD   NotifyCount ;
  BufferInstrumentation() : 
                      pR3QueueCount (NULL),
                      pAvailableQueueCount (NULL),
                      pFilledQueueCount (NULL),
                      SkippedCount (0),
                      NotifyCount (0) {}
} BufferInstrumentation;


#ifdef __FLAT__
/* Type: PCI_CONFIG_848
 * Purpose: Holds copy of Bt848 PCI Config header
 * FNGetPciHdr returns a pointer to one of these. (32 bit interface only)
 */
typedef struct {
    WORD VendorID ;
    WORD DeviceID ;
    WORD Command ;
    WORD Status  ;
    unsigned RevisionID : 8 ;
    unsigned ClassCode : 24 ;
    unsigned res0 : 8 ;
    unsigned LatencyTimer : 8 ;
    unsigned HeaderType0 : 8 ;
    unsigned res1 : 8 ;
    DWORD BaseAddr0 ;
    DWORD res2[10] ;
    unsigned InterruptLine : 8 ;
    unsigned InterruptPin : 8 ;
    unsigned Min_Gnt : 8 ;
    unsigned Max_Lat : 8 ;
} PCI_CONFIG_848 ;
#endif

typedef enum { SS_INVALID, SS_START, SS_STOP } SS_REASON ;

typedef struct SOUND_SYNC {
   SS_REASON why ;
   DWORD     dwVidStartTime ;
   WORD      hwnd ;             // 16 bit video window handle
   WORD      reserved ;         // required for alignment
   SOUND_SYNC() : why(SS_INVALID), dwVidStartTime(0)
     , hwnd(0) 
     {}
} SOUND_SYNC ;

#endif //__VXDFUNC_H
