//
//	File		cam_defs.h
//				commonly used camera parameter values
//
//	Created by 	S. Gillies
//	Copyright(c) 1998 VLSI Vision Ltd, all rights reserved.
//
//	Notes
//	Revision history	
//		0-01	27/01/98	SG	Framerates and mains frequencies
//		0-07	17/02/98	SG
//		

#ifndef __CAM_DEFS_H__
#define __CAM_DEFS_H__

// VVL camera control limits in setup dialog box
#define VP_MIN_BRIGHTNESS     0
#define VP_MAX_BRIGHTNESS     100
#define VP_MIN_SATURATION     0
#define VP_MAX_SATURATION     100
#define VP_MIN_HUE            0
#define VP_MAX_HUE            100
#define VP_MIN_GAMMA          0
#define VP_MAX_GAMMA          15
#define VP_MIN_SHARPNESS      0
#define VP_MAX_SHARPNESS      6

typedef enum {
	HZ_50,
	HZ_60
} T_MAINS;

typedef enum {
	FPS_7_5,
	FPS_12_5,
	FPS_15,
	FPS_25,
	FPS_30
} T_FRAMERATE;

typedef enum {
	QCIF,
	CIF
} T_SIZE;

typedef enum {
	SW_AEC,
	HW_AEC,
	NO_AEC
} T_AEC_ENABLE;

typedef enum {
	VP_AEC_TI,		//	Auto-exposure accumulater lower limit
	VP_AEC_TC,		//	Auto-exposure accumulater target
	VP_AEC_TH,		//	Auto-exposure accumulater upper limit
	VP_AEC_MAINS,	//	Mains frequency
	VP_AEC_CAM_FPS,	//	Camera framerate
	VP_AEC_APCOR_1,	//	Sharpness setting to use for a gains of 1 - 2
	VP_AEC_APCOR_2,	//	Sharpness setting to use for a gain of 2 - 3
	VP_AEC_APCOR_3,	//	Sharpness setting to use for a gain of 3 - 4
	VP_AEC_COARSE_H,
	VP_AEC_COARSE_L,
	VP_AEC_RCOMP,
	VP_AEC_G1COMP,
	VP_AEC_G2COMP,
	VP_AEC_BCOMP,
	VP_AEC_GAIN,
	VP_AEC_ENABLE,
	VP_AEC_LIGHTING,
	VP_GAMCOR,
	VP_OFFSET,
	VP_IM_SIZE,
	VP_SATURATION,
	VP_HUE,
	VP_DEFAULTS,
	VP_LOW_POWER,
	VP_SHARPNESS,

   VP_SENTINEL_LAST     // sentinel value, keep it as last item

} T_VP_PARAM;

typedef enum {
	LIGHTING_NORMAL,
	LIGHTING_BACKLIT
} T_LIGHTING;

#endif //__CAM_DEFS_H__
