/* File: ctimer.h
 * Purpose: Debug Code timing class
 * Copyright 1997 Brooktree Corporation
 * Author: dtharp
 * $Log: ctimer.h $
 * Revision 1.1  1997/04/16 17:49:21  dtharp
 * Initial revision
*/
#ifndef CTIMER_H
#define CTIMER_H
#include <windows.h>

    #if defined(_WIN32)
    //
    // Win32 Version
    //
    class CTimer {
       private:
          LARGE_INTEGER t0, t1, freq ;
          BOOL status ;
          double calcTime() {
             double cnt = t1.LowPart - t0.LowPart ;
             return (double) cnt / (double) freq.LowPart ;
          }
       public :
          CTimer(BOOL * pStatus = NULL) {
             status = QueryPerformanceFrequency (&freq);
             t0.HighPart = t0.LowPart = t1.HighPart = t1.LowPart = 0L ;
             if (pStatus)
               *pStatus = status ;
          }
          ~CTimer(){} ;
          void start() {if (status) QueryPerformanceCounter(&t0) ; }
          void stop()  {if (status) QueryPerformanceCounter(&t1) ;}
          long getResolution(){return status ? freq.LowPart : 0L ;}
          double getElapsedTime() {return status ? calcTime() : 0L ;}
    } ;
    #else
    //
    // The following code works on a Pentium or better only!!
    //

    // You'll need to change this constant to get accurate results until
    // code is implemented to detect clock freq.
    #define CLOCK_FREQ 166E6 

    class PentiumTsc {
       private:
          DWORD HiPart ;
          DWORD LowPart ;
       public:

          PentiumTsc() { Refresh() ;}
          void Refresh() ;
          PentiumTsc & operator = (PentiumTsc & x) {
             HiPart  = x.HiPart ;
             LowPart = x.LowPart ;
             return *this ;
          }
          PentiumTsc  operator - (PentiumTsc & rhs) {
             PentiumTsc res = *this ;
             res.LowPart -= rhs.LowPart ; // I'm only concerned with low 32 bits for now...
             return res ;
          }
          operator double () { return (double) LowPart ; }
          operator DWORD ()  { return LowPart ; }
    } ;

    void PentiumTsc::Refresh() {
       DWORD lsp, msp ;
       __emit__(0x0f,0x31);  // rdtsc instruction
       _asm {
          mov lsp,eax
          mov msp,edx
       } ;
       LowPart = lsp ;
       HiPart = msp ;
    }

    class CTimer {
       private:
          PentiumTsc t0, t1 ;
          DWORD freq ;
          double calcTime() {
             double cnt = (double) (t1 - t0) ;
             return (double) cnt / (double) freq ;
          }
       public :
          CTimer(BOOL * pStatus = NULL) {
             // TSC increments on every clock cycle
             // for the time being, the frequency is hard coded.
             freq = CLOCK_FREQ ;
             if (pStatus)
               *pStatus = TRUE ;
          }
          ~CTimer(){} ;
          void start() { t0.Refresh() ;}
          void stop()  { t1.Refresh() ;}
          DWORD getResolution(){ return (DWORD) freq ;}
          double getElapsedTime() {return calcTime() ;}
    } ;
    #endif

#endif
// EOF
