/* File DbgPrint.h
 * Purpose: Universal Printf style debug print.
 * Author: dtharp
 * $Log: dbgprint.h $
 * Revision 1.9  1998/02/24 07:05:44  dtharp
 * Graft from 1.8.2.1
 * Revision 1.8.2.2  1998/02/10 03:34:35  dtharp
 * Added capability to adust debug level at runtime.
 * Revision 1.8  1997/06/01 17:04:55  dtharp
 * Added ability to designate an alternate DebugBreak();
 * Revision 1.7  1997/05/14 20:28:30  dtharp
 * disabled "possible use before definition" warning message.
 * Revision 1.6  1997/05/06 17:37:45  dtharp
 * Protected DebugPrint and DebugAssert with Braces.
 * Revision 1.5  1997/04/24 21:19:53  dtharp
 * Removed File Name print from DebugPrint macro.
 * Revision 1.4  1997/04/21 21:42:59  dtharp
 * Added DebugAssert(); added file name print; added file and 
 * line number print for DebugAssert().
 * Revision 1.3  1997/04/09 17:26:38  dtharp
 * Moved debugBuffer onto stack.
 * Revision 1.2  1997/03/26 00:31:55  dtharp
 * Made DebugPrint work for any target (VxD, capdrv, etc.)
 */
#ifndef DBG_PRINT_H
#define DBG_PRINT_H

#ifndef VXD
    #include <windows.h>
    #include <stdio.h>
    #include <stdarg.h>
    #ifndef DebugLevel
      #define DebugLevel 0
    #endif
#else
  extern LONG DebugLevel ;  // VxD has a variable accessible through dot comand
#endif

#ifdef DebugPrint
    #undef DebugPrint
#endif

// Allow an alternate action for DebugBreak()
#ifndef DEBUG_BREAK
   #define DEBUG_BREAK DebugBreak()
#endif

// Note : this interface is meant to duplicate the DebugPrint interface available in the WDM driver.
// The debug level constants are not covered in their entirety here.

#ifdef DEBUG
    // Turn off "possible use before definition" warning message
    #pragma warn -def


    //#define DebugPrint(x)  PrintFile(__FILE__) ; MyDebugPrint x
    #define DebugPrint(x)  {MyDebugPrint x ;}
    #define DebugAssert(x) {PrintFileAndLine(__FILE__,__LINE__,shouldPrint x) ; MyDebugAssert x ;}
    
    #define DebugLevelVerbose 0
    #define DebugLevelPrecise 1

    #if defined VXD
       #define DEBUG_OUT Out_Debug_String
    #else
       #define DEBUG_OUT OutputDebugString
    #endif
    inline bool shouldPrint(bool LifeIsGood,  LPSTR DebugMessage, ... )
    {
       return LifeIsGood ;
    }

    inline void PrintFile(char * s)
    {
           char debugPrintBuffer[256] ;
           sprintf(debugPrintBuffer,"%s: ",s) ;
           DEBUG_OUT(debugPrintBuffer) ;
    }
    inline void PrintFileAndLine(char * f, int l, bool LifeIsGood)
    {
        if (!LifeIsGood)
        {
           char debugPrintBuffer[256] ;
           sprintf(debugPrintBuffer,"%s %d: ",f,l) ;
           DEBUG_OUT(debugPrintBuffer) ;
        }
    }

    inline void MyDebugPrint(unsigned long DebugPrintLevel, LPSTR DebugMessage, ... )
    {
       if ((LONG) DebugPrintLevel <= DebugLevel) 
       {
           char debugPrintBuffer[256] ;

           va_list marker;
           va_start( marker, DebugMessage );        // Initialize variable arguments. 
           vsprintf( debugPrintBuffer, DebugMessage, marker ); 

           DEBUG_OUT(debugPrintBuffer) ;

           va_end( marker );                        // Reset variable arguments.
       }
    }

    inline void MyDebugAssert( bool LifeIsGood,  LPSTR DebugMessage, ... )
    {
       if (!LifeIsGood) {

          char debugPrintBuffer[256];

          va_list ap;
          va_start(ap, DebugMessage);

          vsprintf(debugPrintBuffer, DebugMessage, ap);
          DEBUG_OUT(debugPrintBuffer);
          va_end(ap);

          DEBUG_BREAK;
       }
    }

#else
    #define DebugPrint(x)
    #define DebugAssert(x)
#endif

#endif
