#ifndef __MSTYPES_H
#define __MSTYPES_H

#ifndef __RETCODE_H
#include "retcode.h"
#endif

// these are lifted from mytypes.h and borland headers
typedef int bool;
typedef enum { false, true };

class TSize;
class TRect;

template<class T> inline bool ToBool(const T& t) {
   return static_cast<bool>(t != 0);
}


class  TPoint : public tagPOINT {
  public:
    // Constructors
    TPoint() {}
    TPoint(int _x, int _y) {x = _x; y = _y;}
    TPoint(const POINT far& point) {x = point.x; y = point.y;}
    TPoint(const TPoint far& point) {x = point.x; y = point.y;}
    TPoint(const SIZE far& size) {x = size.cx; y = size.cy;}
    TPoint( DWORD dw) {x = WORD(LOWORD(dw)); y = WORD(HIWORD(dw));}

    // Information functions/operators
    bool        operator ==(const TPoint& other) const;
    bool        operator !=(const TPoint& other) const;

    // Functions/binary-operators that return points or sizes
    TPoint      OffsetBy(int dx, int dy) const {return TPoint(x+dx, y+dy);}
    TPoint      operator +(const TSize& size) const;
    TSize       operator -(const TPoint& point) const;
    TPoint      operator -(const TSize& size) const;
    TPoint      operator -() const {return TPoint(-x, -y);}

    // Functions/assignement-operators that modify this point
    TPoint&     Offset(int dx, int dy);
    TPoint&     operator +=(const TSize& size);
    TPoint&     operator -=(const TSize& size);
};

//
// class TSize
// ----- -----
//
class  TSize : public tagSIZE {
  public:
    // Constructors
    TSize() {}
    TSize(int dx, int dy) {cx = dx; cy = dy;}
    TSize(const POINT far& point) {cx = point.x; cy = point.y;}
    TSize(const SIZE far& size) {cx = size.cx; cy = size.cy;}
    TSize(const TSize far& size) {cx = size.cx; cy = size.cy;}
    TSize(DWORD dw) {cx = LOWORD(dw); cy = HIWORD(dw);}

    // Information functions/operators
    bool        operator ==(const TSize& other) const;
    bool        operator !=(const TSize& other) const;

    // Functions/binary-operators that return sizes
    TSize      operator +(const TSize& size) const;
    TSize      operator -(const TSize& size) const;
    TSize      operator -() const {return TSize(-cx, -cy);}

    // Functions/assignement-operators that modify this size
    TSize&     operator +=(const TSize& size);
    TSize&     operator -=(const TSize& size);
};


class  TRect : public tagRECT {
  public:
    // Constructors
    TRect() {}
    TRect(const RECT far& rect);
    TRect(const TRect far& rect);
    TRect(int _left, int _top, int _right, int _bottom);
    TRect(const TPoint& upLeft, const TPoint& loRight);
    TRect(const TPoint& origin, const TSize& extent);

    // (re)Initializers
    void        SetNull();
    void        SetEmpty() {SetNull();}
    void        Set(int _left, int _top, int _right, int _bottom);

    // Type Conversion operators
    operator    const TPoint*() const {return (const TPoint*)this;}
    operator    TPoint*() {return (TPoint*)this;}

    // Testing functions
    bool        IsEmpty() const;
    bool        IsNull() const;
    bool        operator ==(const TRect& other) const;
    bool        operator !=(const TRect& other) const;

    // Information/access functions(const and non-const)
    const TPoint& TopLeft() const {return *(TPoint*)&left;}
    TPoint&     TopLeft() {return *(TPoint*)&left;}
    TPoint      TopRight() const {return TPoint(right, top);}
    TPoint      BottomLeft() const {return TPoint(left, bottom);}
    const TPoint& BottomRight() const {return *(TPoint*)&right;}
    TPoint&     BottomRight() {return *(TPoint*)&right;}
    int         Width() const {return right-left;}
    int         Height() const {return bottom-top;}
    TSize       Size() const {return TSize(Width(), Height());}
    long        Area() const {return long(Width())*long(Height());}

    bool        Contains(const TPoint& point) const;
    bool        Contains(const TRect& other) const;
    bool        Touches(const TRect& other) const;
    TRect       OffsetBy(int dx, int dy) const;
    TRect       operator +(const TSize& size) const;
    TRect       operator -(const TSize& size) const;
    TRect       InflatedBy(int dx, int dy) const;
    TRect       InflatedBy(const TSize& size) const;
    TRect       operator &(const TRect& other) const;
    TRect       operator |(const TRect& other) const;

    // Manipulation functions/operators
    TRect&      Offset(int dx, int dy);
    TRect&      operator +=(const TSize& delta);
    TRect&      operator -=(const TSize& delta);
    TRect&      Inflate(int dx, int dy);
    TRect&      Inflate(const TSize& delta);
    TRect&      operator &=(const TRect& other);
    TRect&      operator |=(const TRect& other);
};

//----------------------------------------------------------------------------
// Inlines
//----------------------------------------------------------------------------

inline bool TPoint::operator ==(const TPoint& other) const {
  return ToBool(other.x==x && other.y==y);
}

inline bool TPoint::operator !=(const TPoint& other) const {
  return ToBool(other.x!=x || other.y!=y);
}

inline TPoint TPoint::operator +(const TSize& size) const {
  return TPoint(x+size.cx, y+size.cy);
}

inline TSize TPoint::operator -(const TPoint& point) const {
  return TSize(x-point.x, y-point.y);
}

inline TPoint TPoint::operator -(const TSize& size) const {
  return TPoint(x-size.cx, y-size.cy);
}

inline TPoint& TPoint::Offset(int dx, int dy) {
  x += dx;
  y += dy;
  return *this;
}

inline TPoint& TPoint::operator +=(const TSize& size) {
  x += size.cx;
  y += size.cy;
  return *this;
}

inline TPoint& TPoint::operator -=(const TSize& size) {
  x -= size.cx;
  y -= size.cy;
  return *this;
}


inline bool TSize::operator ==(const TSize& other) const {
  return ToBool(other.cx==cx && other.cy==cy);
}

inline bool TSize::operator !=(const TSize& other) const {
  return ToBool(other.cx!=cx || other.cy!=cy);
}

inline TSize TSize::operator +(const TSize& size) const {
  return TSize(cx+size.cx, cy+size.cy);
}

inline TSize TSize::operator -(const TSize& size) const {
  return TSize(cx-size.cx, cy-size.cy);
}

inline TSize& TSize::operator +=(const TSize& size) {
  cx += size.cx;
  cy += size.cy;
  return *this;
}

inline TSize& TSize::operator -=(const TSize& size) {
  cx -= size.cx;
  cy -= size.cy;
  return *this;
}



inline void TRect::SetNull() {
  left = top = right = bottom = 0;
}

inline void TRect::Set(int _left, int _top, int _right, int _bottom) {
  left = _left;
  top = _top;
  right = _right;
  bottom = _bottom;
}

inline TRect::TRect(const RECT far& rect) {
  *(RECT far*)this = rect;
}

inline TRect::TRect(const TRect far& rect) {
  *(RECT far*)this = *(RECT far*)&rect;
}

inline TRect::TRect(int _left, int _top, int _right, int _bottom) {
  Set(_left, _top, _right, _bottom);
}

inline TRect::TRect(const TPoint& topLeft, const TPoint& bottomRight) {
  Set(topLeft.x, topLeft.y, bottomRight.x, bottomRight.y);
}

inline TRect::TRect(const TPoint& origin, const TSize& extent) {
  Set(origin.x, origin.y, origin.x+extent.cx, origin.y+extent.cy);
}

inline bool TRect::IsEmpty() const {
  return ToBool(left>=right || top>=bottom);
}

inline bool TRect::IsNull() const {
  return ToBool(!left && !right && !top && !bottom);
}

inline bool TRect::operator ==(const TRect& other) const {
  return ToBool(other.left==left   && other.top==top
             && other.right==right && other.bottom==bottom);
}

inline bool TRect::operator !=(const TRect& other) const {
  return ToBool(!(other==*this));
}

inline bool TRect::Contains(const TPoint& point) const {
  return ToBool(point.x >= left && point.x < right
             && point.y >= top  && point.y < bottom);
}

inline bool TRect::Contains(const TRect& other) const {
  return ToBool(other.left >= left && other.right <= right
             && other.top >= top   && other.bottom <= bottom);
}

inline bool TRect::Touches(const TRect& other) const {
  return ToBool(other.right > left && other.left < right
             && other.bottom > top && other.top < bottom);
}

inline TRect TRect::OffsetBy(int dx, int dy) const {
  return TRect(left+dx, top+dy, right+dx, bottom+dy);
}

inline TRect TRect::operator +(const TSize& size) const {
  return OffsetBy(size.cx, size.cy);
}

inline TRect TRect::operator -(const TSize& size) const {
  return OffsetBy(-size.cx, -size.cy);
}

inline TRect TRect::InflatedBy(int dx, int dy) const {
  return TRect(left-dx, top-dy, right+dx, bottom+dy);
}

inline TRect TRect::InflatedBy(const TSize& size) const {
  return InflatedBy(size.cx, size.cy);
}


inline TRect& TRect::operator +=(const TSize& delta) {
  Offset(delta.cx, delta.cy);
  return *this;
}

inline TRect& TRect::operator -=(const TSize& delta) {
  return *this += -delta;
}

inline TRect& TRect::Inflate(const TSize& delta) {
  return Inflate(delta.cx, delta.cy);
}   


#endif
