/* File: PsageBl.h
 * Purpose: Defines class PsPageBlock
 * Author: sasha
 * $Log: pspagebl.h $
 * Revision 2.1  1996/06/10 20:13:42  ssm
 * MarkBeta1AsRev2
 * Revision 1.7  1996/03/13 00:45:04  sasha
 * Added Fix() and UnFix() functions which are called from ctor and dtor
 * Revision 1.6  1996/01/29 02:34:32  sasha
 * Fixed GetPhysAddr to include lower 12 bits.
 * Revision 1.5  1995/12/08 22:31:08  sasha
 * Added GetPhysAddr() function
 * Revision 1.4  1995/11/02 17:19:04  sasha
 * First major check-in for testing
 * Revision 1.3  1995/10/12 20:14:10  sasha
 * Changed size of the array in the test version of the class
 * Revision 1.2  1995/09/30 13:59:08  sasha
 * Just to make Mike happy
 * Revision 1.1  1995/09/30 06:59:08  sasha
 * Initial revision
*/

#ifndef __PSPAGEBL_H
#define __PSPAGEBL_H

#if !defined __vtoolscp_h && !defined TESTING
#include "vtoolscp.h"
#else
#include <windows.h>
#endif



#ifndef TESTING

const PAGE_SIZE = 0x1000;
const MinPhys = 0;
const MaxPhys = -1;
const OurAllocFlags = PAGECONTIG | PAGEUSEALIGN | PAGEFIXED |  PAGEZEROINIT |
   PAGELOCKED;

/* Class: PsPageBlock
 * Purpose: Encapsulates memory allocations for the data buffers and RISC programs
 */
class PsPageBlock : public VPageBlock
{
   protected:
      DWORD     dwPhysAddr_;
      DWORD     dwBlockSize_;
      int       nPages_;
   public:
      int operator==( const PsPageBlock &rhs ) const { return m_linear == rhs.m_linear; }

      PsPageBlock( DWORD dwSize, DWORD pType = PG_SYS, DWORD flags = OurAllocFlags,
         VMHANDLE hVM = 0, DWORD alignMask = 0, DWORD minPhys = 0,
         WORD maxPhys = -1 ) : dwBlockSize_( dwSize ),
            VPageBlock( nPages_ = ( dwSize + PAGE_SIZE - 1 ) / PAGE_SIZE, pType, flags,
               hVM, alignMask, minPhys, maxPhys, &dwPhysAddr_ )
         { if ( getSize() ) Fix(); }

      inline BOOL Fix() { return lock( 0, nPages_, 0 ); }
      inline BOOL UnFix() { return unlock( 0, nPages_, 0 ); }

      DWORD GetPhysAddr()
      { return getPhysicalAddress( 0 ) | ( (DWORD)getLinearBase() & 0xFFF ); }
      ~PsPageBlock() { if ( getSize() ) UnFix(); }
};

#else // TESTING

#ifndef __VMHANDLE
typedef DWORD VMHANDLE;
#define __VMHANDLE
#endif

class PsPageBlock
{
   protected:
      PVOID     pPhysAddr_;
   public:

      PsPageBlock( DWORD dwSize, DWORD pType = 0, DWORD flags = 0,
         VMHANDLE hVM = 0, DWORD alignMask = 0, DWORD minPhys = 0,
         WORD maxPhys = -1 ) : pPhysAddr_( NULL )
         { pPhysAddr_ = new char [dwSize]; }

      ~PsPageBlock() { delete [] pPhysAddr_; }

      PVOID getLinearBase() { return pPhysAddr_; }
      DWORD GetPhysAddr() { return (DWORD)pPhysAddr_; }

};
#endif

#endif