/* Regutil.cpp
 * Purpose: Contains definition Windows registry functions wrapper class
 * Author: peter
 * $Log: regutil.cpp $
 * Revision 1.11  1998/04/22 11:56:12  trang
 * Modified to compile under MSVC 1.52.
 * Revision 1.10  1998/04/18 08:13:09  trang
 * Modified to compile under Microsoft VC.
 * Revision 1.9  1998/02/11 19:40:51  dtharp
 * Added cast to avoid compiler error.
 * Revision 1.8  1997/12/12 23:58:47  peter
 * Fixed REG_BINARY problem.
 * Revision 1.7  1997/12/12 17:43:31  peter
 * Commented-out common debug error msgs.  Removed reduntant code.
 * Revision 1.6  1997/12/11 23:11:07  dtharp
 * Removed hard coded paths to include files
 * Revision 1.5  1997/12/11 17:13:02  peter
 * Added REG_BINARY support to btutil*.dll.
 * Revision 1.4  1997/12/10 00:31:41  peter
 * Revision 1.3  1997/12/09 22:18:18  peter
 * Several changes to fix 16/32-bit thunk exception.
 * Initial revision
 */

#include <stdlib.h>
#include <windows.h>
#include "regutil.h"
#include "util_16.h"
#include "dbgprint.h"

#if !defined(WIN32) && !defined(__FLAT__)
HINSTANCE	RegUtil::m_hDll = NULL;
#endif


RegUtil::RegUtil( LPCSTR szSubKeyName, HKEY hMainKey ) : hKey_( 0 )
{
	strcpy( m_szKeyName, szSubKeyName );

#if defined(WIN32) || defined(__FLAT__)
#pragma message ("******* using 32-bit RegUtil")
	//DebugPrint(( 0, "32-bit: RegUtil::RegUtil( szSubKeyName = %s )\n", szSubKeyName ));
	if( hMainKey == HKEY_CLASSES_ROOT )
	{
		RegCreateKey( hMainKey, szSubKeyName, &hKey_ );
	}
	else
	{
		RegCreateKey( HKEY_LOCAL_MACHINE, szSubKeyName, &hKey_ );
	}
#else
#pragma message ("******* using 16-bit RegUtil")
	//DebugPrint(( 0, "16-bit: RegUtil::RegUtil( szSubKeyName = %s )\n", szSubKeyName ));

	if( hMainKey == HKEY_CLASSES_ROOT )
	{
		//	old behavior
		RegCreateKey( hMainKey, szSubKeyName, &hKey_ );
		return;
	}

	REGUTILCREATEKEY_t	pRegUtilCreateKey = NULL;
	if( !m_hDll )
	{
		m_hDll = LoadLibrary( "btutil16.dll" );
		if( (UINT)m_hDll < (UINT)HINSTANCE_ERROR )
		{
         DebugPrint(( 0, "LoadLibrary( btutil16.dll ) failed.  error = %d\n", m_hDll ));
			return;
		}
		else
		{
			//DebugPrint(( "LoadLibrary() success\n" ));
		}
	}

	if( ( pRegUtilCreateKey = (REGUTILCREATEKEY_t)GetProcAddress( m_hDll, "REGUTILCREATEKEY" ) ) == NULL )
	{
		DebugPrint(( 0, "GetProcAddress( REGUTILCREATEKEY ) failed.\n" ));
		return;
	}
	pRegUtilCreateKey( szSubKeyName );

#endif
}

RegUtil::~RegUtil()
{
	//DebugPrint(( 0, "RegUtil::~RegUtil()\n" ));
	if( hKey_ ) 
	{
		RegCloseKey( hKey_ ); 
		hKey_ = NULL;
	}
}


void RegUtil::SetValue( DWORD dwValue, LPCSTR lpszSubKey )
{
   //DebugPrint(( 0, "RegUtil::SetValue()\n" ));
   char		str[ 25 ];
   sprintf( str, "%ld", dwValue );
   SetValue( str, lpszSubKey );
}

void RegUtil::SetValue( LPCSTR szValue, LPCSTR lpszSubKey )
{
	char		szFullPathToSubKey[ 257 ];
#if defined(WIN32) || defined(__FLAT__)
	sprintf( szFullPathToSubKey, "%s\\%s", m_szKeyName, lpszSubKey );
	if( RegSetValue( HKEY_LOCAL_MACHINE, szFullPathToSubKey, REG_SZ, szValue, strlen(szValue) ) == ERROR_SUCCESS )
	{
		//DebugPrint(( 0, "RegUtil 32-bit: RegSetValue() success\n" ));
	}
	else
	{
		//DebugPrint(( 0, "RegUtil 32-bit: RegSetValue() failure\n" ));
	}
#else

	sprintf( szFullPathToSubKey, "%s\\%s", m_szKeyName, lpszSubKey );


	REGUTILSETVALUE_t	pRegUtilSetValue = NULL;

	if( ( pRegUtilSetValue = (REGUTILSETVALUE_t)GetProcAddress( m_hDll, "REGUTILSETVALUE" ) ) == NULL )
	{
		DebugPrint(( 0, "GetProcAddress( REGUTILSETVALUE ) failed.\n" ));
		return;
	}
	pRegUtilSetValue( szFullPathToSubKey, szValue );

#endif
}

ErrorCode RegUtil::GetValue( LPSTR lpValue, LPLONG lpByteCount, LPCSTR lpszSubKey )
{
	//DebugPrint(( 0, "RegUtil::GetValue()\n" ));
	char				szFullPathToSubKey[ 257 ];
	ErrorCode			nResult = Fail;
	sprintf( szFullPathToSubKey, "%s\\%s", m_szKeyName, lpszSubKey );

#if defined(WIN32) || defined(__FLAT__)

	if( RegQueryValue( HKEY_LOCAL_MACHINE, szFullPathToSubKey, lpValue, lpByteCount ) == ERROR_SUCCESS )
	{
		nResult = Success;
	}
	else
	{
		//DebugPrint(( 0, "RegUtil 32-bit: RegQueryValue() failure\n" ));
	}
#else

	REGUTILQUERYVALUE_t	pRegUtilQueryValue = NULL;
	if( ( pRegUtilQueryValue = (REGUTILQUERYVALUE_t)GetProcAddress( m_hDll, "REGUTILQUERYVALUE" ) ) == NULL )
	{
		DebugPrint(( 0, "GetProcAddress( REGUTILQUERYVALUE ) failed.\n" ));
		return nResult;
	}
	if( pRegUtilQueryValue( szFullPathToSubKey, lpValue, lpByteCount ) )
	{
		nResult = Success;
	}

#endif
	return nResult;
}

ErrorCode RegUtil::GetValue( LPDWORD lpValue, LPCSTR lpszSubKey )
{
	char		szValStr[ 16 ];
	LONG		lBC = 15;
	if ( GetValue( szValStr, &lBC, lpszSubKey ) == Success ) 
	{
		*lpValue = atol( szValStr );
		return Success;
	}
	else
	{
		//DebugPrint(( 0, "RegUtil::GetValue(DWORD,lpszSubKey) done: failure\n" ));
		return Fail;
	}
}


ErrorCode RegUtil::GetBinValue( int* plSize, LPSTR cbData, LPSTR szValueName )
{
	ErrorCode			nResult = Fail;

#if defined(WIN32) || defined(__FLAT__)
	DWORD		dwType = 0;
   HKEY     hKey = NULL;
   if( RegOpenKeyEx( HKEY_LOCAL_MACHINE, m_szKeyName, 0, KEY_ALL_ACCESS, &hKey ) == ERROR_SUCCESS )
   {
	   if( RegQueryValueEx( hKey, szValueName, NULL, &dwType, (BYTE *) cbData, (DWORD *)plSize ) == ERROR_SUCCESS )
	   {
		   if( dwType != REG_BINARY )
		   {
			   //DebugPrint(( 0, "RegUtil 32-bit: RegQueryValueEx(): type is not REG_BINARY... failed\n" ));
		   }
		   else
		   {
			   nResult = Success;
		   }
	   }
	   else
	   {
		   //DebugPrint(( 0, "RegUtil 32-bit: RegQueryValueEx() failed\n" ));
	   }
      RegCloseKey( hKey );
   }
#else
	REGUTILQUERYBINVALUE_t	   pRegUtilQueryBinValue = NULL;
	if( ( pRegUtilQueryBinValue = (REGUTILQUERYBINVALUE_t)GetProcAddress( m_hDll, "REGUTILQUERYBINVALUE" ) ) == NULL )
	{
		DebugPrint(( 0, "GetProcAddress( REGUTILQUERYBINVALUE ) failed.\n" ));
		return nResult;
	}
	if( pRegUtilQueryBinValue( plSize, cbData, m_szKeyName, szValueName ) )
	{
		nResult = Success;
	}
#endif

	return nResult;
}

ErrorCode RegUtil::SetBinValue( int lSize, LPSTR cbData, LPSTR szValueName )
{
	ErrorCode			nResult = Fail;

#if defined(WIN32) || defined(__FLAT__)
	DWORD		dwType = 0;
	DWORD		dwDisposition = 0;
   HKEY     hKey = NULL;
   if( RegCreateKeyEx( HKEY_LOCAL_MACHINE, m_szKeyName, NULL, NULL, 0, KEY_ALL_ACCESS, NULL, &hKey, &dwDisposition ) == ERROR_SUCCESS )
   {
   	if( RegSetValueEx( hKey, szValueName, 0, REG_BINARY, (CONST BYTE *) cbData, (LONG)lSize ) == ERROR_SUCCESS )
	   {
		   nResult = Success;
	   }
	   else
	   {
		   //DebugPrint(( 0, "RegUtil 32-bit: RegSetValueEx() failed\n" ));
	   }
      RegCloseKey( hKey );
   }
#else
	REGUTILSETBINVALUE_t	   pRegUtilSetBinValue = NULL;
	if( ( pRegUtilSetBinValue = (REGUTILSETBINVALUE_t)GetProcAddress( m_hDll, "REGUTILSETBINVALUE" ) ) == NULL )
	{
		DebugPrint(( 0, "GetProcAddress( REGUTILSETBINVALUE ) failed.\n" ));
		return nResult;
	}
	if( pRegUtilSetBinValue( lSize, cbData, m_szKeyName, szValueName ) )
	{
		nResult = Success;
	}
#endif

	return nResult;
}




