/* File: Strminfo.h
 * Purpose: Contains definition o the StreamInfo structure
 * Author: sasha
 * $Log: strminfo.h $
 * Revision 2.2  1997/03/20 22:24:46  dtharp
 * Grafted WDM buffer managment branch.
 * Revision 2.1.1.1  1997/03/17 16:57:53  dtharp
 * Works @ 30fps NTSC.
 * Revision 2.1  1996/06/10 20:13:43  ssm
 * MarkBeta1AsRev2
 * Revision 1.4  1995/12/08 22:31:52  sasha
 * Changed operator==()
 * Revision 1.3  1995/10/08 20:06:18  sasha
 * First major check-in for testing
 * Revision 1.2  1995/10/08 12:06:18  sasha
 * Just to make Mike happy
 * Revision 1.1  1995/10/08 05:06:18  sasha
 * Initial revision
 */

#ifndef __STRMINFO_H
#define __STRMINFO_H

#ifndef __CHANIFACE_H
#include "chanifac.h"
#endif

/* Class: StreamInfo
 * Purpose: Used as a base class in the mechanism of calling VxDVideoChannel class
 *   back from the CaptureChip class.
 * Attributes: VidChan: VideoChannel - identifies a particular channel. Used during
 *   stream allocation
 *   callback: ChanIface * -
 */
class StreamInfo
{
   public:
      VideoStream  VidStrm;
      ChanIface   *callback;
      int          StreamNo;
      bool         Interrupt_;
      RGNDATA       *clip_;
      
      bool          Paired_;
      bool          DoPause_;

      PVOID        tag;

      void SetCallback( ChanIface *iface ) { callback = iface;}

      int operator==( const StreamInfo &rhs ) const
      { return VidStrm == rhs.VidStrm;/* || StreamNo == rhs.StreamNo;*/ }

      StreamInfo( int No, ChanIface *iface ) : VidStrm( VS_Below ), clip_( NULL ),
         callback( iface ), StreamNo( No ), tag( NULL ), Interrupt_( false ), Paired_(false),
         DoPause_(false) {}

      StreamInfo( int No, VideoStream aStream ) : StreamNo( No ), Interrupt_( false ),
         VidStrm( aStream ), callback( NULL ), tag( NULL ), clip_( NULL ), Paired_(false),
         DoPause_(false) {}

      StreamInfo( int No, VideoStream aStream, PVOID p ) : StreamNo( No ), clip_( NULL ),
         VidStrm( aStream ), callback( NULL ), tag( p ), Interrupt_( false ) , Paired_(false),
         DoPause_(false){}

      StreamInfo() : VidStrm( VS_Below ), Interrupt_( false ), clip_( NULL ),
         callback( NULL ), StreamNo( 0 ), tag( NULL ) , Paired_(false),
         DoPause_(false){}

      void SetClipping( RGNDATA *const clip ) { clip_ = clip; }
      const RGNDATA *GetClipping() const { return clip_; }

      bool DoPause() { return DoPause_; }
      void SetDoPause( bool p ) {DoPause_ = p ;}

      void SetPaired( bool p ) {Paired_ = p ;}
      bool GetPaired() {return Paired_;}

//      ~StreamInfo() {};
};


#endif
