// $Header: G:/SwDev/W95/Video/pisces/dvlpmnt/pci/rcs/PCI_IO.CPP 1.1 1997/02/12 06:11:36 rudyl Released $
// $Log: PCI_IO.CPP $
// Revision 1.1  1997/02/12 06:11:36  rudyl
// Initial revision
// Revision 1.1  1996/03/04 17:48:58  rudyl
// Initial revision

#include "pci_io.h"

#pragma inline
#pragma warn -asm

//=============================================================================
// CLASS IMPLEMENTATION: PCI_IO
// Overview: This class support interaction with the PCI bus for the Bt848.
//           It provides set and get functions for all the members of the
//           registers in the Bt848.
// Usage:    User of this class must declare a PCI_IO vaeiable and must call
//           the "FindDevice(Bt848devID)" function to locate the PCI device
//           before any set or get functions can be invoked.
// Note:     The code has been compiled with Borland C++ 4.52 compiler.
//           It will also compile without any changes with the Microsoft Visual
//           C++ compiler. There are two #ifdefs in the code to take care of
//           the differences with the two assemblers.
//=============================================================================

///////////////////////////////////////////////////////////////////////////////
// Constructor
///////////////////////////////////////////////////////////////////////////////
PCI_IO::PCI_IO( int vendorID ):
        m_vID( vendorID ) { }


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::FindDevice( int devID, int devIndex )
// Purpose: To locate a PCI device
// Input:   int devID    - device ID to be located
//          int devIndex - which Bt848 to be located (default is 0)
// Output:  None
// Return:  1 if success, else 0
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::FindDevice( int devID, int devIndex )
{
  m_dID = devID;
  m_dIndex = devIndex;
  int vID = m_vID;

  char status;
  char busNumber;
  char devFunctionNumber;

  __asm
  {
    mov ax, FINDDEVICE;
    mov cx, devID;
    mov dx, vID;
    mov si, devIndex;
    int 0x1A;
    mov status, ah;
    mov busNumber, bl;
    mov devFunctionNumber, bh;
  }

  m_status = status;
  m_busNumber = busNumber;
  m_devFunctionNumber = devFunctionNumber;

  if( status == 0 )
    return 1;
  else
    return 0;
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::ReadConfigDWORD( int reg, long *retDWORD )
// Purpose: Read register content in DWORD
// Input:   int reg - register to be retrieved
// Output:  long * retDWORD - register content retrieved
// Return:  1 if success, else 0
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::ReadConfigDWORD( int reg, long *retDWORD )
{
//  int vID = m_vID;
  char busNumber = m_busNumber;
  char devFunctionNumber = m_devFunctionNumber;
  long confDWORD;
  char status;

    __asm
  {
    mov ax, READCONFWORD;
    mov bl, busNumber;
    mov bh, devFunctionNumber;
    mov di, reg;
    int 0x1A;
    mov status, ah;
  }

#ifdef __BCPLUSPLUS__   // Borland C++ compiler?
    __asm mov dword ptr confDWORD, ecx;
#else   // Microsoft compiler
    __asm _emit 0x66;
    __asm mov word ptr confDWORD, cx;
#endif

  *retDWORD = confDWORD;
  m_status = status;
  if( status == 0 )
    return 1;
  else
    return 0;
}

///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::WriteConfigDWORD( int reg, long writeDWORD )
// Purpose: Write register content in DWORD
// Input:   int reg         - register to be modified
//          long writeDWORD - register content to be set to
// Output:  None
// Return:  1 if success, else 0
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::WriteConfigDWORD( int reg, long writeDWORD )
{
//  int vID = m_vID;
  char busNumber = m_busNumber;
  char devFunctionNumber = m_devFunctionNumber;
  char status;

    __asm
  {
    mov ax, WRITECONFWORD;
    mov bl, busNumber;
    mov bh, devFunctionNumber;
    mov di, reg;
  }

#ifdef __BCPLUSPLUS__   // Borland C++ compiler?
    __asm mov ecx, dword ptr writeDWORD;
#else   // Microsoft compiler
    __asm _emit 0x66;
    __asm mov  cx, word ptr writeDWORD;
#endif

    __asm int 0x1A;
    __asm mov status, ah;

  m_status = status;
  if( status == 0 )
    return 1;
  else
    return 0;
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::SetCommandStatusBit( int whichBit, int flag )
// Purpose: Turn on/off a bit in Command and Status Register
// Input:   int whichBit - which bit to turn on/off
//          int flag     - 1 for ON, 0 for OFF
// Output:  None
// Return:  1 if success; 0 otherwise
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::SetCommandStatusBit( int whichBit, int flag )
{
  long value;
  if ( ReadConfigDWORD( cfgRegCommandStatus, &value ) == 0 )
    return( 0 );

  DWORD mask = 0x00000001L << whichBit;  // determine mask for the specified bit
  if ( flag == 0 )
    value &= ~mask;   // turn bit OFF
  else
    value |= mask;    // turn bit ON

  if ( WriteConfigDWORD( cfgRegCommandStatus, value ) == 0 )
    return( 0 );
  else
    return( 1 );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::GetCommandStatusBit( int whichBit )
// Purpose: Check if a bit in Command and Status Register is on or off
// Input:   None
// Output:  None
// Return:  -1 if error; otherwise 1 for enable or 0 for disable
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::GetCommandStatusBit( int whichBit )
{
  long value;
  if ( ReadConfigDWORD( cfgRegCommandStatus, &value ) == 0 )
    return( -1 );

  DWORD mask = 0x00000001L << whichBit;  // determine mask for the specified bit
  if ( ( value & mask ) != 0 )
    return( 1 );
  else
    return( 0 );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::GetVendorID( void )
// Purpose: Obtain vendor ID
// Input:   None
// Output:  None
// Return:  -1 if error; otherwise vendor ID (2 bytes)
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::GetVendorID( void )
{
  long value = 0L;
  if ( ReadConfigDWORD( cfgRegVenDevID, &value ) == 0 )
    return( -1 );
  else
    return( LOWORD( value ) );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::GetDeviceID( void )
// Purpose: Obtain device ID
// Input:   None
// Output:  None
// Return:  -1 if error; otherwise device ID (2 bytes)
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::GetDeviceID( void )
{
  long value = 0L;
  if ( ReadConfigDWORD( cfgRegVenDevID, &value ) == 0 )
    return( -1 );
  else
    return( HIWORD( value ) );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::SetMemorySpaceAccess( int flag )
// Purpose: Enable/disable response to memory space accesses
// Input:   int flag - 1 to enable; 0 to disable
// Output:  None
// Return:  1 if success; 0 otherwise
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::SetMemorySpaceAccess( int flag )
{
  return( SetCommandStatusBit( 1, flag ) );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::IsMemorySpaceAccess( void )
// Purpose: Check if response to memory space accesses is enable or disable
// Input:   None
// Output:  None
// Return:  -1 if error; otherwise 1 for enable or 0 for disable
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::IsMemorySpaceAccess( void )
{
  return( GetCommandStatusBit( 1 ) );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::SetBusMaster( int flag )
// Purpose: Enable/disable Bt848 to act as a bus initiator
// Input:   int flag - 1 to enable; 0 to disable
// Output:  None
// Return:  1 if success; 0 otherwise
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::SetBusMaster( int flag )
{
  return( SetCommandStatusBit( 2, flag ) );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::IsBusMaster( void )
// Purpose: Check if Bt848 is enable/disable as a bus initiator
// Input:   None
// Output:  None
// Return:  -1 if error; otherwise 1 for enable or 0 for disable
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::IsBusMaster( void )
{
  return( GetCommandStatusBit( 2 ) );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::SetParityError( int flag )
// Purpose: Enable/disable parity error reporting
// Input:   int flag - 1 to enable; 0 to disable
// Output:  None
// Return:  1 if success; 0 otherwise
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::SetParityError( int flag )
{
  return( SetCommandStatusBit( 6, flag ) );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::IsBusMaster( void )
// Purpose: Check if parity error reporting is enable or disable
// Input:   None
// Output:  None
// Return:  -1 if error; otherwise 1 for enable or 0 for disable
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::IsParityError( void )
{
  return( GetCommandStatusBit( 6 ) );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::SetSystemError( int flag )
// Purpose: Enable/disable system error driver
// Input:   int flag - 1 to enable; 0 to disable
// Output:  None
// Return:  1 if success; 0 otherwise
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::SetSystemError( int flag )
{
  return( SetCommandStatusBit( 8, flag ) );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::IsSystemError( void )
// Purpose: Check if system error driver is enable or disable
// Input:   None
// Output:  None
// Return:  -1 if error; otherwise 1 for enable or 0 for disable
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::IsSystemError( void )
{
  return( GetCommandStatusBit( 8 ) );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::IsCapableFastB2B( void )
// Purpose: Check if target capable of fast back-to-back transactions
// Input:   None
// Output:  None
// Return:  -1 if error; otherwise 1 for enable or 0 for disable
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::IsCapableFastB2B( void )
{
  return( GetCommandStatusBit( 23 ) );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::SetDataParity( int flag )
// Purpose: Enable/disable data parity error reported
// Input:   int flag - 1 to enable; 0 to disable
// Output:  None
// Return:  1 if success; 0 otherwise
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::SetDataParity( int flag )
{
  return( SetCommandStatusBit( 24, flag ) );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::IsDataParity( void )
// Purpose: Check if data parity error reported is enable or disable
// Input:   None
// Output:  None
// Return:  -1 if error; otherwise 1 for enable or 0 for disable
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::IsDataParity( void )
{
  return( GetCommandStatusBit( 24 ) );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::GetAddrDecodeTime( void )
// Purpose: Obtain DEVSEL timing
// Input:   None
// Output:  None
// Return:  -1 if error; otherwise DEVSEL timing (0 - 3)
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::GetAddrDecodeTime( void )
{
  long value;
  if ( ReadConfigDWORD( cfgRegCommandStatus, &value ) == 0 )
    return( -1 );

  DWORD mask = 0x06000000L;  // bits [26:25]
  return ( (int) ( ( value & mask ) >> 25 ) );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::SetSigTargetAbort( int flag )
// Purpose: Enable/disable target terminates transaction with Target Abort while
//          detecting an address parity error
// Input:   int flag - 1 to enable; 0 to disable
// Output:  None
// Return:  1 if success; 0 otherwise
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::SetSigTargetAbort( int flag )
{
  return( SetCommandStatusBit( 27, flag ) );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::IsSigTargetAbort( void )
// Purpose: Check if target terminates transaction with Target Abort while
//          detecting an address parity error
// Input:   None
// Output:  None
// Return:  -1 if error; otherwise 1 for enable or 0 for disable
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::IsSigTargetAbort( void )
{
  return( GetCommandStatusBit( 27 ) );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::SetRecTargetAbort( int flag )
// Purpose: Enable/disable master transaction terminated with Target Abort
// Input:   int flag - 1 to enable; 0 to disable
// Output:  None
// Return:  1 if success; 0 otherwise
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::SetRecTargetAbort( int flag )
{
  return( SetCommandStatusBit( 28, flag ) );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::IsRecTargetAbort( void )
// Purpose: Check if master transaction is terminated with Target Abort
// Input:   None
// Output:  None
// Return:  -1 if error; otherwise 1 for enable or 0 for disable
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::IsRecTargetAbort( void )
{
  return( GetCommandStatusBit( 28 ) );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::SetRecMasterAbort( int flag )
// Purpose: Enable/disable when master transaction terminated with Master Abort
// Input:   int flag - 1 to enable; 0 to disable
// Output:  None
// Return:  1 if success; 0 otherwise
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::SetRecMasterAbort( int flag )
{
  return( SetCommandStatusBit( 29, flag ) );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::IsRecMasterAbort( void )
// Purpose: Check if master transaction is terminated with Master Abort
// Input:   None
// Output:  None
// Return:  -1 if error; otherwise 1 for enable or 0 for disable
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::IsRecMasterAbort( void )
{
  return( GetCommandStatusBit( 29 ) );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::SetSigSysError( int flag )
// Purpose: Enable/disable assertion of SERR
// Input:   int flag - 1 to enable; 0 to disable
// Output:  None
// Return:  1 if success; 0 otherwise
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::SetSigSysError( int flag )
{
  return( SetCommandStatusBit( 30, flag ) );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::IsSigSysError( void )
// Purpose: Check if SERR is asserted
// Input:   None
// Output:  None
// Return:  -1 if error; otherwise 1 for enable or 0 for disable
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::IsSigSysError( void )
{
  return( GetCommandStatusBit( 30 ) );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::SetDetectParityError( int flag )
// Purpose: Enable/disable parity error detection
// Input:   int flag - 1 to enable; 0 to disable
// Output:  None
// Return:  1 if success; 0 otherwise
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::SetDetectParityError( int flag )
{
  return( SetCommandStatusBit( 31, flag ) );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::IsDetectParityError( void )
// Purpose: Check if parity error is detected
// Input:   None
// Output:  None
// Return:  -1 if error; otherwise 1 for enable or 0 for disable
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::IsDetectParityError( void )
{
  return( GetCommandStatusBit( 31 ) );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::GetRevisionID( void )
// Purpose: Obtain revision ID
// Input:   None
// Output:  None
// Return:  -1 if error; otherwise revision ID (1 byte value)
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::GetRevisionID( void )
{
  long value = 0L;
  if ( ReadConfigDWORD( cfgRegRevIDClassCode, &value ) == 0 )
    return( -1 );
  else
    return( LOBYTE( LOWORD( value ) ) );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  long PCI_IO::GetClassCode( void )
// Purpose: Obtain class code
// Input:   None
// Output:  None
// Return:  -1 if error; otherwise class code (3 bytes value)
///////////////////////////////////////////////////////////////////////////////
long PCI_IO::GetClassCode( void )
{
  long value = 0L;
  if ( ReadConfigDWORD( cfgRegRevIDClassCode, &value ) == 0 )
    return( -1 );

  DWORD mask = 0xFFFFFF00L;  // bits [31:8]
  value &= mask;
  return( value >> 8 );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::SetLatencyTimer( int timer )
// Purpose: # PCI bus clocks for LT used by bus master
// Input:   int timer - latency timer
// Output:  None
// Return:  1 if success; 0 otherwise
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::SetLatencyTimer( int timer )
{
  if ( ( timer < 0 ) || ( timer > 0x00FF ) )
    return( 0 );

  long value;
  if ( ReadConfigDWORD( cfgRegLatencyTimer, &value ) == 0 )
    return( 0 );

  DWORD mask = 0x0000FF00L;   // bits [15:8]
  value &= ~mask;             // turn bits off
  value |= ( timer << 8 );    // set bits [15:8]

  if ( WriteConfigDWORD( cfgRegLatencyTimer, value ) == 0 )
    return( 0 );
  else
    return( 1 );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::GetLatencyTimer( void )
// Purpose: Get # PCI bus clocks for LT used by bus master
// Input:   None
// Output:  None
// Return:  -1 if error; otherwise latency timer value (1 byte value)
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::GetLatencyTimer( void )
{
  long value;
  if ( ReadConfigDWORD( cfgRegLatencyTimer, &value ) == 0 )
    return( -1 );

  DWORD mask = 0x0000FF00L;   // bits [15:8]
  return( (int) ( ( value & mask ) >> 8 ) );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::GetMemUsageSpec( void )
// Purpose: Get the reserved 4 KB memory-mapped address space
// Input:   None
// Output:  None
// Return:  -1 if error; otherwise memory usage spec (12 bits value)
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::GetMemUsageSpec( void )
{
  long value;
  if ( ReadConfigDWORD( cfgRegBaseAddress, &value ) == 0 )
    return( -1 );

  DWORD mask = 0x00000FFFL;   // bits [11:0]
  return( (int)( value & mask ) );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::SetRelocateMemPtr( long ptr )
// Purpose: Determine location of the registers in 32-bits addr memory space
// Input:   long ptr - memory pointer
// Output:  None
// Return:  1 if success; 0 otherwise
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::SetRelocateMemPtr( long ptr )
{
  if ( ( ptr < 0 ) || ( ptr > 0x000FFFFFL ) )
    return( 0 );

  long value;
  if ( ReadConfigDWORD( cfgRegBaseAddress, &value ) == 0 )
    return( 0 );

  DWORD mask = 0xFFFFF000L;   // bits [31:12]
  value &= ~mask;             // turn bits off
  value |= ( ptr << 12 );     // set bits [31:12]

  if ( WriteConfigDWORD( cfgRegBaseAddress, value ) == 0 )
    return( 0 );
  else
    return( 1 );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::GetMemUsageSpec( void )
// Purpose: Obtain location of the registers in 32-bits addr memory space
// Input:   None
// Output:  None
// Return:  -1 if error; otherwise location of the registers (20 bits value)
///////////////////////////////////////////////////////////////////////////////
long PCI_IO::GetRelocateMemPtr( void )
{
  long value;
  if ( ReadConfigDWORD( cfgRegBaseAddress, &value ) == 0 )
    return( -1 );

  DWORD mask = 0xFFFFF000L;   // bits [31:12]
  return( ( value & mask ) >> 12 );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::SetInterruptLine( int line )
// Purpose: Set which line of the system interrupt controller is to be used
// Input:   int line - interrupt line to be used
// Output:  None
// Return:  1 if success; 0 otherwise
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::SetInterruptLine( int line )
{
  if ( ( line < 0 ) || ( line > 0x00FF ) )
    return( 0 );

  long value;
  if ( ReadConfigDWORD( cfgRegIntGntLat, &value ) == 0 )
    return( 0 );


  DWORD mask = 0x000000FFL;   // bits [7:0]
  value &= ~mask;             // turn bits off
  value |= line;              // set bits [7:0]

  if ( WriteConfigDWORD( cfgRegIntGntLat, value ) == 0 )
    return( 0 );
  else
    return( 1 );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::GetInterruptLine( void )
// Purpose: Get which line of the system interrupt controller is being used
// Input:   None
// Output:  None
// Return:  -1 if error; otherwise interrupt line (1 byte value)
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::GetInterruptLine( void )
{
  long value;
  if ( ReadConfigDWORD( cfgRegIntGntLat, &value ) == 0 )
    return( -1 );

  return( LOBYTE( LOWORD( value ) ) );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::GetInterruptPin( void )
// Purpose: Get interrupt pin
// Input:   None
// Output:  None
// Return:  -1 if error; otherwise interrupt pin (1 byte value)
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::GetInterruptPin( void )
{
  long value;
  if ( ReadConfigDWORD( cfgRegIntGntLat, &value ) == 0 )
    return( -1 );

  return( HIBYTE( LOWORD( value ) ) );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::GetMinGrant( void )
// Purpose: Get minimum grant brust period
// Input:   None
// Output:  None
// Return:  -1 if error; otherwise minimum grant brust period
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::GetMinGrant( void )
{
  long value;
  if ( ReadConfigDWORD( cfgRegIntGntLat, &value ) == 0 )
    return( -1 );

  return( LOBYTE( HIWORD( value ) ) );
}


///////////////////////////////////////////////////////////////////////////////
// Method:  int PCI_IO::GetMaxLatency( void )
// Purpose: Get maximum latency
// Input:   None
// Output:  None
// Return:  -1 if error; otherwise maximum latency (1 byte value)
///////////////////////////////////////////////////////////////////////////////
int PCI_IO::GetMaxLatency( void )
{
  long value;
  if ( ReadConfigDWORD( cfgRegIntGntLat, &value ) == 0 )
    return( -1 );

  return( HIBYTE( HIWORD( value ) ) );
}

