// $Header: G:/SwDev/W95/Video/pisces/dvlpmnt/pci/rcs/PCI_IO.H 1.1 1997/02/12 06:11:37 rudyl Released $
// $Log: PCI_IO.H $
// Revision 1.1  1997/02/12 06:11:37  rudyl
// Initial revision

#ifndef __PCI_IO_H
#define __PCI_IO_H

#ifdef  _MSC_VER
  #ifndef _M_I86LM
    #error "Library supports LARGE model only"
  #endif
#endif

///////////////////////////////////////////////////////////////////////////////
// Common definitions and typedefs (extracted from Windows.h)
///////////////////////////////////////////////////////////////////////////////
#define FALSE               0
#define TRUE                1

#ifndef BOOL
typedef int                 BOOL;
#endif

#ifndef BYTE
typedef unsigned char       BYTE;
#endif

#ifndef WORD
typedef unsigned short      WORD;
#endif

#ifndef DWORD
typedef unsigned long       DWORD;
#endif

#ifndef UINT
typedef unsigned int        UINT;
#endif

#ifndef LOWORD
#define LOWORD(l)           ((WORD)(l))
#endif

#ifndef HIWORD
#define HIWORD(l)           ((WORD)((DWORD)(l) >> 16))
#endif

#ifndef LOBYTE
#define LOBYTE(w)           ((BYTE)(w))
#endif

#ifndef HIBYTE
#define HIBYTE(w)           ((BYTE)((UINT)(w) >> 8))
#endif

#ifndef MAKELONG
#define MAKELONG(low, high) ((LONG)(((WORD)(low)) | (((DWORD)((WORD)(high))) << 16)))
#endif

///////////////////////////////////////////////////////////////////////////////
// Intel PCI BIOS Interrupts Functions
///////////////////////////////////////////////////////////////////////////////
const int FINDDEVICE    = 0xB102;   // find PCI device
const int READCONFWORD  = 0xB10A;   // read configuration DWORD
const int WRITECONFWORD = 0xB10D;   // write configuration DWORD


///////////////////////////////////////////////////////////////////////////////
// PCI CONFIGURATION REGISTERS
///////////////////////////////////////////////////////////////////////////////
const int cfgRegVenDevID        = 0x00;   // Vendor and Device ID Register
const int cfgRegCommandStatus   = 0x04;   // Command and Status Register
const int cfgRegRevIDClassCode  = 0x08;   // Revision ID and Class Code Register
const int cfgRegLatencyTimer    = 0x0C;   // Latency Timer Register
const int cfgRegBaseAddress     = 0x10;   // Base Address 0 Register
const int cfgRegIntGntLat       = 0x3C;   // Interrupt Line, Interrupt Pin, Min_Gnt, Max_Lat Register

//=============================================================================
// CLASS DECLARATION: PCI_IO
// Overview: This class support interaction with the PCI bus for the Bt848.
//           It provides set and get functions for all the members of the
//           registers in the Bt848.
// Usage:    User of this class must declare a PCI_IO vaeiable and must call
//           the "FindDevice(Bt848devID)" function to locate the PCI device
//           before any set or get functions can be invoked.
//=============================================================================
class PCI_IO
{
  int  m_vID;                 // vendor ID
  char m_status;              // return status of an operation
  char m_busNumber;           // bus number of the device
  char m_devFunctionNumber;   // device/function number
  int  m_dID;                 // device ID
  int  m_dIndex;              // device index

private:
  // support function to set and get bit in Command and Status Register
  int SetCommandStatusBit( int, int );
  int GetCommandStatusBit( int );

public:

  //---------------------------------------------------------------------------
  // Constructor
  //---------------------------------------------------------------------------
  PCI_IO( int vendorID );

  //---------------------------------------------------------------------------
  // Member functions
  //---------------------------------------------------------------------------

  // Following functions are provided to interact with PCI directly

  int FindDevice( int devID, int devIndex=0 );
  int ReadConfigDWORD( int reg, long *retDWORD );
  int WriteConfigDWORD( int reg, long writeDWORD );

  // Following functions are used to set or get all the available members of
  // the registers in the Bt848

  // Vendor and Device ID Register (0x00)
  int GetVendorID( void );
  int GetDeviceID( void );

  // Command and Status Register (0x04)
  int SetMemorySpaceAccess( int );
  int IsMemorySpaceAccess( void );
  int SetBusMaster( int );
  int IsBusMaster( void );
  int SetParityError( int );
  int IsParityError( void );
  int SetSystemError( int );
  int IsSystemError( void );
  int IsCapableFastB2B( void );
  int SetDataParity( int );
  int IsDataParity( void );
  int GetAddrDecodeTime( void );
  int SetSigTargetAbort( int );
  int IsSigTargetAbort( void );
  int SetRecTargetAbort( int );
  int IsRecTargetAbort( void );
  int SetRecMasterAbort( int );
  int IsRecMasterAbort( void );
  int SetSigSysError( int );
  int IsSigSysError( void );
  int SetDetectParityError( int );
  int IsDetectParityError( void );

  // Revision ID and Class Code Register (0x08)
  int GetRevisionID( void );
  long GetClassCode( void );

  // Latency Timer Register (0x0C)
  int SetLatencyTimer( int );
  int GetLatencyTimer( void );

  // Base Address 0 Register (0x10)
  int GetMemUsageSpec( void );
  int SetRelocateMemPtr( long );
  long GetRelocateMemPtr( void );

  // Interrupt Line, Interrupt Pin, Min_Gnt, Max_Lat Register (0x3C)
  int SetInterruptLine( int );
  int GetInterruptLine( void );
  int GetInterruptPin( void );
  int GetMinGrant( void );
  int GetMaxLatency( void );
};


#endif // __PCI_IO_H