/* File: Compreg.h
 * Purpose: This file contains the definition of the CompositeReg class
 * Author: sasha
 * $Log: compreg.h $
 * Revision 2.1  1996/06/10 20:13:45  ssm
 * MarkBeta1AsRev2
 * Revision 1.7  1995/11/22 18:16:00  sasha
 * Revision 1.6  1995/07/29 01:52:04  sasha
 * Included vtoolsd headers
 * Revision 1.5  1995/07/29 01:51:30  SASHA
 * Changed comments to reflect the new names
 * Revision 1.4  1995/07/29 01:42:22  SASHA
 * Changed names for LowPart_ to LSBPart_ and for
 * HighPart_ to MSBPart_
 * Revision 1.3  1995/07/18 00:36:42  SASHA
 * Added private default constructor in order to prevent an instantiation without the arguments.
 */

#ifndef __COMPREG_H
#define __COMPREG_H

#ifndef __REGFIELD_H
#include "regfield.h"
#endif

/* Class: CompositeReg
 * Purpose: This class encapsulates the registers that have their bits in two
 *          different places ( registers )
 * Attributes:
 *   LSBPart_:  Register & - least significant bits part of a composite register
 *   HighPart_: RegField & - most significant bits part of a composite register
 *   LowPartWidth_: BYTE - width of the low portion in bits
 * Operations:
 *   operator DWORD(): data access method. Returns a value of the register
 *   DWORD operator=( DWORD ): assignment operator. Used to set the register
 * Note: the error handling provided by the class is minimal. It is a responibility
 *   of the user to pass correct parameters to the constructor. The class has
 *   no way of knowing if the correct low and high registers passed in are correct,
 *   for example. If low part size in bits passed in is not less then MaxWidth ( 32 )
 *   the mask used to isolate the low portion will be 0xFFFFFFFF
 */
class CompositeReg : public RegBase
{
   private:
      Register &LSBPart_;
      RegField &MSBPart_;
      BYTE      LowPartWidth_;
      CompositeReg();
   public:
      virtual operator DWORD();
      virtual DWORD operator=( DWORD dwValue );
      CompositeReg( Register &LowReg, BYTE LowWidth, RegField &HighReg, RegisterType aType ) :
         RegBase( aType ), LSBPart_( LowReg ), MSBPart_( HighReg ),
         LowPartWidth_( LowWidth ) {}
};

#endif
