/* File: Regfield.cpp
 * Purpose: Contains implementation of the RegiField class
 * Author: sasha
 * $Log: regfield.cpp $
 * Revision 2.1  1996/06/10 20:13:47  ssm
 * MarkBeta1AsRev2
 * Revision 1.9  1995/12/08 21:23:00  sasha
 * Added code to the operator= for the RR types of registers
 * Revision 1.8  1995/12/08 20:21:57  ssm
 * replaced hard tabs with 3 spaces
 * Revision 1.7  1995/11/22 18:16:01  sasha
 * Revision 1.6  1995/07/29 01:50:26  sasha
 * Included vtoolsd headers
 * Revision 1.5  1995/07/29 01:13:54  SASHA
 * Use ReturnAllFs function now instead of explicitly returning AllFs.
 * The function can produce an error message if DEBUG is #defined
 * Revision 1.4  1995/07/24 01:54:56  SASHA
 * Revision for the code review. Should be no changes from previous check-in.
 * Revision 1.3  1995/07/18 00:32:26  SASHA
 * For all classes added code in the access functions to check the type of the register.
*/

#include "regfield.h"

/* Method: RegField::MakeAMask
 * Purpose: Computes a mask used to isolate a field withing a register based
 *   on the width of a field
 */
inline DWORD RegField::MakeAMask()
{
//   compute the mask to apply to the owner register to reset
//   all bits that are part of a field. Mask is based on the size of a field
   return ::MakeAMask( FieldWidth_ );
}

/* Method: RegField::operator DWORD()
 * Purpose: Performs the read from a field of register
*/
RegField::operator DWORD()
{
   // if write-only, get the shadow
   if ( GetRegisterType() == WO )
      return GetShadow();

   // for RO and RW do the actual read
   // get the register data and move it to the right position
   DWORD dwValue = ( Owner_ >> StartBit_ );

   DWORD dwMask = MakeAMask();

   return dwValue & dwMask;
}


/* Method: RegField::operator=
 * Purpose: performs the assignment to a field of register
 * Note:
   This function computes the mask to apply to the owner register to reset
   all bits that are part of a field. Mask is based on the start position and size
   Then it calculates the proper value from the passed argument ( moves the size
   number of bits to the starting position ) and ORs these bits in the owner register.
*/
DWORD RegField::operator=( DWORD dwValue )
{
// if a register is read-only nothing is done. This is an error
   if ( GetRegisterType() == RO )
      return ReturnAllFs();

   SetShadow( dwValue );

   // get a mask
   DWORD dwMask = MakeAMask();

   // move mask to a proper position
   dwMask = dwMask << StartBit_;

//   calculate the proper value from the passed argument ( move the size
//   number of bits to the starting position )
   DWORD dwFieldValue = dwValue << StartBit_;
   dwFieldValue &= dwMask;

   // do not perform intermediate steps on the owner; rather use a temp and update
   // the owner at once
   DWORD dwRegContent = Owner_;

   // reset the relevant bits
   if ( GetRegisterType() == RR )
      dwRegContent = 0;
   else
      dwRegContent &= ~dwMask;

   // OR these bits in the owner register.
   dwRegContent |= dwFieldValue;

   Owner_ = dwRegContent;
   return dwValue;
}
