/* File: Register.cpp
 * Purpose: Contains implementation of the RegisterX classes
 * Author: sasha
 * $Log: register.cpp $
 * Revision 2.1  1996/06/10 20:13:48  ssm
 * MarkBeta1AsRev2
 * Revision 1.9  1995/12/08 20:21:57  ssm
 * replaced hard tabs with 3 spaces
 * Revision 1.8  1995/11/22 18:16:02  sasha
 * Revision 1.7  1995/07/29 01:40:28  sasha
 * Included vtoolsd headers
 * Revision 1.6  1995/07/29 01:15:42  SASHA
 * Instead of just returning AllFs value call a function ReturnAllFs() now.
 * The function is inline and can give an error message if DEBUG is #defined
 * Revision 1.5  1995/07/21 23:26:06  SASHA
 * Revision for the code review. Should be no changes from previous check-in.
 * Revision 1.4  1995/07/21 23:14:56  SASHA
 * Synchronized function headers with their names. Simplified 'return'
 * statements by introducing a temporary pointer that holds register's address.
 * Revision 1.3  1995/07/18 00:32:02  SASHA
 * For all classes added code to check for the register type in the access functions
*/
#include "register.h"

/* Method: Register::operator DWORD()
 * Purpose: a dummy function. Always returns -1
*/
Register::operator DWORD()
{
   return ReturnAllFs();
}


/* Method: Register::operator=
 * Purpose: a dummy function. Does not perform an assignment. Always returns -1
*/
DWORD Register::operator=( DWORD )
{
   return ReturnAllFs();
}

/* Method: RegisterB::operator DWORD()
 * Purpose: Performs the read from a byte register
*/
RegisterB::operator DWORD()
{
   // if write-only return the shadow
   if ( GetRegisterType() == WO )
      return GetShadow();

   // for RO and RW do the actual read
   LPBYTE pRegAddr = GetBaseAddress() + GetOffset();
   return *pRegAddr;
}


/* Method: RegisterB::operator=
 * Purpose: performs the assignment to a byte register
*/
DWORD RegisterB::operator=( DWORD dwValue )
{
// if a register is read-only nothing is done. This is an error
   if ( GetRegisterType() == RO )
      return ReturnAllFs();

   // keep a shadow around
   SetShadow( dwValue );

   LPBYTE pRegAddr = GetBaseAddress() + GetOffset();
   *pRegAddr = (BYTE)dwValue;
   return dwValue;
}

/* Method: RegisterW::operator DWORD()
 * Purpose: Performs the read from a word register
*/
RegisterW::operator DWORD()
{
   // if write-only return the shadow
   if ( GetRegisterType() == WO )
      return GetShadow();

   // for RO and RW do the actual read
   LPWORD pRegAddr = (LPWORD)( GetBaseAddress() + GetOffset() );
   return *pRegAddr;
}


/* Method: RegisterW::operator=
 * Purpose: performs the assignment to a word register
*/
DWORD RegisterW::operator=( DWORD dwValue )
{
// if a register is read-only nothing is done. This is an error
   if ( GetRegisterType() == RO )
      return ReturnAllFs();

   // keep a shadow around
   SetShadow( dwValue );

   LPWORD pRegAddr = (LPWORD)( GetBaseAddress() + GetOffset() );
   *pRegAddr = (WORD)dwValue;
   return dwValue;
}

/* Method: RegisterDW::operator DWORD()
 * Purpose: Performs the read from a dword register
*/
RegisterDW::operator DWORD()
{
   // if write-only return the shadow
   if ( GetRegisterType() == WO )
      return GetShadow();

   // for RO and RW do the actual read
   LPDWORD pRegAddr = (LPDWORD)( GetBaseAddress() + GetOffset() );
   return *pRegAddr;
}


/* Method: RegisterDW::operator=
 * Purpose: performs the assignment to a dword register
*/
DWORD RegisterDW::operator=( DWORD dwValue )
{
// if a register is read-only nothing is done. This is an error
   if ( GetRegisterType() == RO )
      return ReturnAllFs();

   // keep a shadow around
   SetShadow( dwValue );

   LPDWORD pRegAddr = (LPDWORD)( GetBaseAddress() + GetOffset() );
   *pRegAddr = dwValue;
   return dwValue;
}
