/*
 *  File: Register.h
 *  Purpose: Contains definitions of the Register, RegisterB, RegisterW
 *           and RegisterDW classes
 *  Author: sasha
 * $Log: register.h $
 * Revision 2.1  1996/06/10 20:13:49  ssm
 * MarkBeta1AsRev2
 * Revision 1.9  1995/12/08 20:21:57  ssm
 * replaced hard tabs with 3 spaces
 * Revision 1.8  1995/11/29 00:14:15  ssm
 * made GetOffset public;  added szName to ctors
 * Revision 1.7  1995/11/22 18:16:02  sasha
 * Revision 1.6  1995/07/25 19:40:44  sasha
 * Included vtoolsd headers
 * Revision 1.5  1995/07/18 01:20:44  SASHA
 * Revision for the code review. Should be no changes from previous check-in.
 * Revision 1.4  1995/07/18 01:13:00  SASHA
 * Added private default constructor to the Register class as well 
 * ( make lint happy )
 * Revision 1.3  1995/07/18 00:32:16  SASHA
 * Added private default constructor in order to prevent an instantiation without the arguments.
*/
#ifndef __REGISTER_H
#define __REGISTER_H



#ifndef __REGBASE_H
#include "regbase.h"
#endif

/* Type: Register
 * Purpose: An intermediate class between the RegBase and actual usable classes.
 *   Actual classes are RegisterB, RegisterW, RegisterDW
 * Attributes:
 *   uOffset_: unsigned int - an offset of the register from the base
 * Operations:
 *   GetOffset(): returns the offset value. Protected
 *   operator DWORD(): data access method. Always returns -1
 *   DWORD operator=( DWORD ): assignment operator. Always returns -1
 * Note:
 *   The reason to have operators in this class is for the register field class
 *   to have a member of type 'reference to Register'. Otherwise RegField is not
 *   able to use access methods.
 */
class Register : public RegBase
{
   private:
      unsigned int uOffset_;
      Register();
    protected:

    public:
         unsigned int GetOffset() { return uOffset_; }

         virtual operator DWORD();
      virtual DWORD operator=( DWORD dwValue );

         Register( unsigned int uOff, RegisterType aType, const char *szName=NULL ) : 
            RegBase( aType , szName), uOffset_( uOff ) {}
};

/* Type: RegisterB
 * Purpose: A register that performs the BYTE I/O
 * Note:
 *   This class has no additional data members, it just overloads operators
 */
class RegisterB : public Register
{
    private:
         RegisterB();
    public:
         virtual operator DWORD();
         virtual DWORD operator=( DWORD dwValue );
         RegisterB( unsigned int uOff, RegisterType aType, const char *szName=NULL ) :
            Register( uOff, aType, szName  ) {};
};

/* Type: RegisterW
 * Purpose: A register that performs the WORD I/O
 * Note:
 *   This class has no additional data members, it just overloads operators
 */
class RegisterW : public Register
{
    private:
         RegisterW();
    public:
         virtual operator DWORD();
         virtual DWORD operator=( DWORD dwValue );
         RegisterW( unsigned int uOff, RegisterType aType, const char *szName=NULL ) : 
            Register( uOff, aType, szName  ) {};

};


/* Type: RegisterDW
 * Purpose: A register that performs the DWORD I/O
 * Note:
 *   This class has no additional data members, it just overloads operators
 */
class RegisterDW : public Register
{
    private:
         RegisterDW();
    public:
         virtual operator DWORD();
         virtual DWORD    operator=( DWORD dwValue );

         RegisterDW( unsigned int uOff, RegisterType aType, const char *szName=NULL ) : 
            Register( uOff, aType, szName  ) {};
};

#endif __REGISTER_H
