/* File: Command.cpp
 * Purpose: contains implementation of the Command class
 * Author: sasha
 * $Log: command.cpp $
 * Revision 2.7  1998/04/02 21:51:12  dtharp
 * Merge DT_VIDEOHDR branch. Implements a new buffer 
 * management scheme.
 * Revision 2.6.1.2  1998/04/01 21:20:01  dtharp
 * Incremental Checkin
 * Revision 2.6  1998/01/09 17:31:29  peter
 * Graft Trident fixes to tip.
 * Revision 2.5.1.1  1997/12/10 01:48:26  peter
 * Merged TRIDENT code.
 * Revision 2.5  1997/07/22 19:44:07  dtharp
 * This is actually rev 2.3.  There was a checkin mistake made while 
 * attempting a branch, and the ability to delete a revision from an 
 * archive seems to have been removed from the latest version of
 * SI.
 * Revision 2.3  1997/05/14 18:14:35  dtharp
 * Removed duplicate functions. Inlines are now in command.h and
 * GetPhys() is in command.cpp.
 * Revision 2.2  1997/03/20 22:25:19  dtharp
 * Grafted WDM buffer managment branch.
 * Revision 2.1.2.1  1997/03/12 18:16:17  dtharp
 * Single field capture kinda works.
 * Revision 2.1  1996/06/10 20:13:53  ssm
 * MarkBeta1AsRev2
 * Revision 1.6  1996/01/06 02:08:05  sasha
 * Revision 1.5  1995/10/17 04:20:32  sasha
 * First major check-in for testing
 * Revision 1.4  1995/10/16 22:46:06  sasha
 * Revision 1.3  1995/10/12 18:08:00  sasha
 * Added GetSyncStatus function and changed Parse to return address
 * of next instruction for JUMP case, instead of JUMP's target
 */

#include "command.h"
#include "dbgprint.h"

DWORD GetPhysAddr( DWORD dwLinAddr )
{
   DWORD PageOffset = dwLinAddr >> 12;
   DWORD PageTableEntry;
   DWORD physAddr = -1 ;
   extern DWORD gdwTridentAperture0, gdwTridentAperture1 ;

   if ( _CopyPageTable( PageOffset, 1, (PVOID*)&PageTableEntry, 0 ) 
		&& ( PageTableEntry & 1 ) ) 	// if present
   {
      physAddr = ( PageTableEntry & 0xFFFFF000UL ) + ( dwLinAddr & 0xFFF );
   }

   // Trident Secondary aperature fix: If gTridentAperture0 and gTridentAperture1
   // are non-zero, then we must check to see if we need to make the adjustment.
   if (gdwTridentAperture0 && gdwTridentAperture1)
   {
      DebugPrint((0, ">>> We have both aperture addresses : %x, %x\n",gdwTridentAperture0,gdwTridentAperture1)) ;
      // check to see if the physical address lies in the first aperature's region
      if (physAddr >= gdwTridentAperture0 && physAddr <= gdwTridentAperture0+0x3fffff)
      {
         // address lies in first aperature: translate to second aperature
         DebugPrint((0,"   Xlate: was %x, now ",physAddr)) ;
         physAddr = gdwTridentAperture1 + (physAddr - gdwTridentAperture0) ;
         DebugPrint((0,"%x\n",physAddr)) ;
      }
   }

   return physAddr;
}

BYTE Command::InstrSize_ [] =
{
   2, 1, 1, 2, 2, 5, 2, 3, 0xFF
};

BYTE Command::InstrNumber_ [] =
{
 8, 0, 1, 8, 8, 2, 8, 3, 4, 5, 6, 7
};

/* Method: Command::CreateCommand
 * Purpose: Compiles an instruction based on the input
 * Input: lpDst: PVOID - pointer to the instruction
 *   Instr: Instruction - opcode
 *   awByteCnt: WORD [] - array of byte counts for various instructions
 *   adwAddress: DWORD [] - array of addresses for various instructions
 *   SOL: bool - value of the SOL bit
 *   EOL: bool - value of the EOL bit
 *   Intr: bool - value of the interrupt bit
 */
LPVOID Command::Create(
   LPVOID lpDst, Instruction Instr, WORD awByteCnt [], DWORD adwAddress [],
    bool SafetyDevice, bool SOL, bool EOL, bool Intr )
{
//   ThisInstr_ = Instr;
//   return (PDWORD)lpDst + GetInstrSize();
   static bool ignoreSplit = false;
   // first of all, take care of physically non-contiguous memory.
   // use a simple check that should always have 32 bytes available in a page
   // if not, jump to the next page
   if ( !ignoreSplit && Need2Split( (DWORD)lpDst, 32 ) ) {
      ignoreSplit = true;
      // insert the jump
      DWORD jumpAddr [1];

   DebugPrint( ( DebugLevelVerbose,"Bt848: Command::Create - insert jump\n"));

      // obtain the phys address for the jump command target
      DWORD dwJumpAddr = ( (DWORD)lpDst + 0xFFF ) & ~0xFFF;
      jumpAddr [0] = GetPhysAddr( dwJumpAddr );
      // jump off to the new page
      Create( lpDst, JUMP, NULL, jumpAddr, true, false, false );
      // obtain the linear address of the next page
      lpDst = PVOID( dwJumpAddr );
      // now go on creating a command requested.
      ignoreSplit = false;
   }
   // this is to be retrieved later to set EOL bit when instructions are split
   // due to the non-contiguous physical memory
   pdwInstrAddr_ = (PDWORD)lpDst;

   ThisInstr_ = Instr;

   DWORD dwAssembly [5]; // maximum instruction size

   // get pointer to the first dword of a command
   LPFIRSTDWORD lpFD = (LPFIRSTDWORD)dwAssembly;

   lpFD->Initer = 0; // virgin out the command

   // bingo - started new command
   lpFD->Gen.OpCode = Instr;

   // set all the flags
   lpFD->Gen.SOL = SOL;
   lpFD->Gen.EOL = EOL;
   lpFD->Gen.IRQ = Intr;

   switch ( Instr ) {
   case WRIT:  // this command needs target address and byte count
      dwAssembly [1] = adwAddress [0]; // next DWORD is an address
      lpFD->Gen.ByteCount = awByteCnt [0];
      break;
   case SKIP: // these pair is interested in byte count only
   case WRITEC:
      lpFD->Gen.ByteCount = awByteCnt [0];
      break;
   case JUMP: // this command cares about target address only
      dwAssembly [1] = adwAddress [0];
      break;
   case SYNC: // this command has a reserved dword2
      dwAssembly [1] = 0 ;
      break;
   case WRITE123: // need everything here...
      lpFD->Gen.ByteCount = awByteCnt [0];
      LPFIRSTDWORD( &dwAssembly [1] )->CRByteCounts.ByteCountCb = awByteCnt [1];
      LPFIRSTDWORD( &dwAssembly [1] )->CRByteCounts.ByteCountCr = awByteCnt [2];
      dwAssembly [2] = adwAddress [0]; // third DWORD is an Y address
      dwAssembly [3] = adwAddress [1]; // third DWORD is an Cb address
      dwAssembly [4] = adwAddress [2]; // third DWORD is an Cr address
      break;
   case SKIP123:
      lpFD->Gen.ByteCount = awByteCnt [0];
      LPFIRSTDWORD( &dwAssembly [1] )->Gen.ByteCount = awByteCnt [1]; // second byte count is in DWORD #2
      break;
   case WRITE1S23: // this command needs Y byte count and dest. address
      lpFD->Gen.ByteCount = awByteCnt [0];
      LPFIRSTDWORD( &dwAssembly [1] )->CRByteCounts.ByteCountCb = awByteCnt [1];
      LPFIRSTDWORD( &dwAssembly [1] )->CRByteCounts.ByteCountCr = awByteCnt [2];
      dwAssembly [2] = adwAddress [0]; // third DWORD is an address
      break;
   default:
      return (LPVOID)-1;
   }
   // in case caller does not do this...
//   if ( SafetyDevice )
//      ((LPFIRSTDWORD)((PDWORD)lpDst + GetInstrSize()))->Gen.OpCode = Reserved;

   //RtlCopyMemory( lpDst, dwAssembly, GetInstrSize() * sizeof( DWORD ) );
   memcpy( lpDst, dwAssembly, GetInstrSize() * sizeof( DWORD ) );
    
   return (PDWORD)lpDst + GetInstrSize();
}

// EOF