/* File: Risceng.h
 * Purpose: Contains definition of the RISCEng class
 * Author: sasha
 * $Log: risceng.h $
 * Revision 2.5  1998/04/02 21:43:50  dtharp
 * Merge DT_VIDEOHDR branch. Implements a new buffer 
 * management scheme.
 * Revision 2.4.2.1  1998/03/18 19:37:32  dtharp
 * Branch for passing videoheaders around
 * Revision 2.4  1997/02/19 21:20:29  dtharp
 * Grafted DT_PAL_CAPTURE_RATE branch (bug id 384)
 * Revision 2.3.2.1  1997/02/10 21:53:00  dtharp
 * Branch for bug #384. Added videoFormat parameter.
 * Revision 2.3  1996/09/27 00:00:38  sasha
 * Bug ID# 254. Added bool parameter to the CreateProgram() to indicate
 * that resyncs need to be inserted.
 * Revision 2.2  1996/09/22 23:32:10  sasha
 * Added DWORD argument to the CreateProgram() to implement the 
 * functionality needed for the 640x480 YUV AVI capture.
 * Revision 2.1  1996/06/10 20:13:56  ssm
 * MarkBeta1AsRev2
 * Revision 1.8  1996/01/15 20:21:38  sasha
 * Added control registers. Modified Start() function
 * Revision 1.7  1996/01/06 02:07:04  sasha
 * Changed ColorFormat to ColFmt
 * Revision 1.6  1995/11/02 20:57:54  sasha
 * First major check-in for testing
 * Revision 1.5  1995/10/18 00:50:30  sasha
 * Changed SIZE to TSize. Removed first_ from the class.
 * Revision 1.4  1995/10/15 06:29:14  sasha
 * Added support for clipping.  
 * Revision 1.3  1995/10/12 22:48:24  sasha
 * Added int first_ to the class and added default constructor to init it.
 * Revision 1.2  1995/10/12 18:26:34  sasha
 * Changed the skipping a bit
 * Revision 1.1  1995/10/02 17:27:02  sasha
 * Initial revision
 */


#ifndef __RISCENG_H
#define __RISCENG_H

#if !defined __vtoolscp_h && !defined TESTING
#include "vtoolscp.h"
#endif

#undef FAR

#ifndef __WINDOWS_H
#include <windows.h>
#endif

#ifndef __VIDDEFS_H
#include "viddefs.h"
#endif

#ifndef __RISCPROG_H
#include "riscprog.h"
#endif

#ifndef __COMPREG_H
#include "compreg.h"
#endif

#include "preg.h"

typedef RISCProgram *RiscPrgHandle;

/* Class: RISCEng
 * Purpose: This class provides control over the BtPisces' RISC engine
 * Attributes:
 * Operations:
 */                
class RISCEng
{
   protected:
      DECLARE_RISCPROGRAMSTARTADDRESS;
      DECLARE_CONTROL;
   public:
      virtual RiscPrgHandle CreateProgram( TSize &ImageSize, DWORD dwPitch, DWORD dwOffset,
         ColFmt, LPVIDEOHDR lpVH, VideoFormat vf,bool Intr = false, 
         DWORD dwPlanarAdjust = 0, bool rsync = false );

      virtual RiscPrgHandle AddProgram( RiscPrgHandle parent, LPVIDEOHDR lpVH );
      virtual void DestroyProgram( RiscPrgHandle ToDie );
      virtual void SetClipping( RiscPrgHandle prog, const RGNDATA &const );
              void ChangeAddress( RiscPrgHandle prog, LPVIDEOHDR lpNewVH);
              void Start( RISCProgram &ToStart )
              { RISC_IPC = ToStart.GetPhysProgAddr() & ~0x3;
                FIFO_ENABLE = 1; RISC_ENABLE = 1;  
              }
              void Stop() { RISC_ENABLE = 0; FIFO_ENABLE = 0; };
              void Chain( RiscPrgHandle hParent, RiscPrgHandle hChild );
              void Skip( RiscPrgHandle hToSkip );

      RISCEng() : CONSTRUCT_RISCPROGRAMSTARTADDRESS, CONSTRUCT_CONTROL
      { PKTP = 1; }
};

#endif