/* File: Riscprog.cpp
 * Purpose: Contains implementation of the RISCProgram class
 * Author: sasha
 * $Log: riscprog.cpp $
 * Revision 2.27  1998/06/01 18:09:11  mdimeo
 * added rcs ident string
 * Revision 2.26  1998/05/14 23:22:35  dtharp
 * Adjusted some debug print priorities
 * Revision 2.25  1998/05/05 20:38:30  dtharp
 * Removed obsolete functions WriteSentinel() & SentinelPresent().
 * Revision 2.24  1998/04/07 17:25:40  dtharp
 * Removed some unused vars to get rid of compile warning.
 * Revision 2.23  1998/04/03 18:35:13  dtharp
 * General cleanup
 * Revision 2.22  1998/04/02 21:51:34  dtharp
 * Merge DT_VIDEOHDR branch. Implements a new buffer 
 * management scheme.
 * Revision 2.21.2.5  1998/04/02 21:29:00  dtharp
 * Cleanup before merge.
 * Revision 2.21.2.4  1998/04/02 16:40:31  dtharp
 * Fixed yuv9 problem
 * Revision 2.21.2.3  1998/04/01 21:20:03  dtharp
 * Incremental Checkin
 * Revision 2.21.2.2  1998/03/27 15:30:25  dtharp
 * Incremental check-in
 * Revision 2.21.2.1  1998/03/18 19:38:03  dtharp
 * Branch for passing videoheaders around
 * Revision 2.21  1997/06/27 15:59:07  dtharp
 * New sentinel based buffer management scheme.
 * Revision 2.20.1.1  1997/06/10 18:00:55  dtharp
 * Branch for new Buffer mgmnt scheme.
 * Revision 2.20  1997/05/16 16:13:08  dtharp
 * Bug Id 562: commented out PutInChain() call in setClipping(). 
 * Revision 2.19  1997/05/14 19:55:24  dtharp
 * Took care  of 3 compiler warnings. #ifdef DEBUG'd 1 and explicitly
 * casted the other two.
 * Revision 2.18  1997/05/14 18:14:37  dtharp
 * Removed duplicate functions. Inlines are now in command.h and
 * GetPhys() is in command.cpp.
 * Revision 2.17  1997/04/28 21:26:19  dtharp
 * Merged CC Primary Surface Branch from 2.15.1.1 Bug Id 475
 * Revision 2.16  1997/04/22 19:49:00  trantq
 * - Merged in Rev. 2.14.1.4 for YUV9 better subsampling using the 
 *  3rd scanline.
 * Revision 2.15  1997/03/20 22:26:19  dtharp
 * Grafted WDM buffer managment branch.
 * Revision 2.14.2.4  1997/03/20 18:13:47  dtharp
 * PAL Mode Implemented. (Increased sizes in AllocStorage)
 * Revision 2.14.2.3  1997/03/18 15:53:34  dtharp
 * Added variable frame rate.
 * Revision 2.14.2.2  1997/03/17 16:52:00  dtharp
 * Works @ 30fps NTSC.
 * Revision 2.14.2.1  1997/03/12 18:21:01  dtharp
 * Single field capture kinda works.
 * Revision 2.14.1.4  1997/04/17 22:00:38  trantq
 * - Fix green strip on the bottom of screen for YUV9 subsampling.
 * Revision 2.14.1.3  1997/03/26 00:39:36  trantq
 * - Add comments for WRITE123 and WRITE1S23.
 * Revision 2.14.1.2  1997/02/21 18:54:02  trantq
 * -Add define for YUV9 build.  If define, the third scanline is used, 
 * else it will use the fourth.
 * Revision 2.14.1.1  1997/02/21 16:47:14  trantq
 * - Add a key in Registry for YUV9 sub-sampling to read from the 3rd or 4th.
 * Revision 2.14  1997/02/19 21:21:36  dtharp
 * Grafted DT_PAL_CAPTURE_RATE branch (bug id 384)
 * Revision 2.13.2.1  1997/02/10 21:56:42  dtharp
 * Branch for bug #384. Code now checks video mode before 
 * calling CreateDistribution().
 * Revision 2.13  1997/01/30 18:08:09  dtharp
 * Merged branch 2.12.1: hauppauge blue screen fix.
 * Revision 2.12.1.1  1997/01/30 18:07:16  dtharp
 * Branch for haupauge blue screen startup problem.
 * Revision 2.12  1997/01/09 23:53:18  ssm
 * graft from revision 2.11.2.2, fix clientY < 0 problem
 * Revision 2.11.2.2  1996/12/25 03:35:11  sasha
 * Initialize the dwMilSecPerFrame_. This helps in rare cases when VfW
 * app does not start, but VBI data capture starts ( should be initialized regardless ).
 * Added AboutToCLip_. This is used to return from Create() without creating
 * a program because it will be destroyed by clipping immediately after.
 * Removed GetDataBufPhys() ( not used anymore ).
 * Use dwLinBufAddr_ in the SetClipping() instead of physical address directly
 * and obtain physical address when needed. Changed Create() so it returns
 * after storing the parameters if clipping is about to be done. There should
 * be a cleaner way of doing this, but it is probably too much things need change for that.
 * Fixed the loop variable name from 'k' to 'l' in Create().
 * Revision 2.11.2.1  1996/12/24 23:55:53  sasha
 * Branch for the above the screen fix
 * Revision 2.11  1996/10/16 05:18:57  sasha
 * Fix for the frame rate: init MapIdx_ to -1. After the mapping is created,
 * MapIdx_ is advanced past the first '1'. This allows for the frame rates
 * below 15 to be achieved using the mapping array. Do not create map
 * array if it was created already. Set MaoIdx_ to -1 in the destructor.
 * Make sure MapIdx_ is reset to start of the array in the Create().
 * Revision 2.10  1996/10/08 04:18:23  sasha
 * Extensive changes to Create()  to split lines into multiple instructions
 * when Y plane crosses non-contig. phys. page. May need more work.
 * Revision 2.9  1996/10/04 00:10:52  sasha
 * Bug ID# 254. Added code to create a map to convert 30 fps to any
 * required frame rate. Use that map to create resync instructions when needed.
 * Changed sub-sampling cheme for YVU9 and YVU12.
 * Revision 2.8  1996/10/02 04:41:56  sasha
 * ID # 254. Use the frame rate to calculate the size of risc program memory. This
 * extra memory is needed for the sync instructions.
 * Revision 2.7  1996/09/29 23:14:29  sasha
 * ID# 254. Moved the location of the interrupt to the last instruction.
 * This is needed to avoid cases when many resynchs stall 848 for a long
 * time, so the CPU has a chance to go and write over the 'live' instructions
 * when it updates the risc prog. in response to the interrupt.
 * Revision 2.6  1996/09/27 00:06:12  sasha
 * Bug ID# 254. Added code to generate resync instructions to accomodate
 * a requested frame rate.
 * Revision 2.5  1996/09/22 23:51:02  sasha
 * Made dwPlanarAdjust_ a non-static member. Create() now accepts
 * it as a parameter. This is needed fro full size planar AVI capture.
 * Cleaned up compiler warnings ( put casts ), removed an unused variable from Create().
 * Revision 2.4  1996/09/21 01:39:16  sasha
 * Now make chroma byte counts proportional to luma one when a
 * line needs to be split between 2 instructions. This reduces chances
 * of FIFO overflow. This fixes one of Intel concerns
 * Revision 2.3  1996/09/07 07:29:35  sasha
 * Added dwPlanarAdjust_. This is used to support planar modes of sizes
 * above CIF. Changed some code in Create() to implement YUV above
 * CIF. Modified Create() a bit to ensure there are no zero chroma bytes
 * even for SKIP commands.
 * Revision 2.2  1996/06/14 20:22:48  sasha
 * Changed memory size calculation for a program for clipping
 * Revision 2.1  1996/06/10 20:13:57  ssm
 * MarkBeta1AsRev2
 * Revision 1.19  1996/04/16 00:08:08  sasha
 * Made sure no byte count equals to zero for planar modes. Misc. fixes on
 * the way
 * Revision 1.18  1996/03/26 19:00:16  sasha
 * Put bpp back in in the SetClipping() - oops
 * Revision 1.17  1996/03/22 23:13:00  sasha
 * Pitch is expressed in bytes now, so removed all code that converted pitch
 * in pixels into pitch in bytes
 * Revision 1.16  1996/03/18 23:27:10  sasha
 * Added CF_UYVY cases
 * Revision 1.15  1996/03/15 19:44:48  sasha
 * New code to handle planar modes for non-contiguous memory
 * Revision 1.14  1996/02/19 02:35:39  sasha
 * Added CF_VBI to the switches
 * Revision 1.13  1996/02/14 02:43:54  sasha
 * Fixed the linear addresses being used in JUMPs. Plus changes some names
 * and too many things to mention here :)
 * Revision 1.12  1996/01/18 21:51:47  sasha
 * Changed name od the argument to GetDataBufPhys() to dwLinAddr
 * Revision 1.11  1996/01/06 02:07:05  sasha
 * Changed ColorFormat to ColFmt
 * Revision 1.10  1996/01/02 22:01:36  sasha
 * Fixed code in Create() that puts program back in chain if it was before
 * Revision 1.9  1996/01/02 18:42:53  sasha
 * Added support for scattered physical memory for ( packed modes only )
 * Revision 1.8  1995/12/18 22:20:02  sasha
 * Added flip support. Fixed image size calculation. Fixed chaining by assigning
 * to pChainAddress_ in the SetClipping. Removed commented code
 * Revision 1.7  1995/12/08 22:34:28  sasha
 * Modified the way programs are linked. Update clipping transparently.
 * Added code to get physical addresses
 * Revision 1.6  1995/11/02 20:59:20  sasha
 * First major check-in for testing
 * Revision 1.5  1995/10/21 04:05:56  sasha
 * Changed SIZE to TSize. Added protected default constructor.
 * Added CreatePrologEpilog function. Overloaded Create function.
 * Removed bool VeryFirst argument from original Create function.
 * Changed CreateLoop function to accept just a bool ( for resync ).
 * Added CreateStarter function.
 * Added code to support planar modes with pitch. Changed layout of the planar
 * modes. Optimized switch in the Create function.
 * Finished clipping code. Moved test if memory already allocated to the
 * AllocateStorage. Removed code associated with bool VeryFirst argument in Create
 * All of that needs testing
 * Revision 1.4  1995/10/16 21:37:08  sasha
 * Added support for clipping.  Moved some common code to a common function
 * ( CreatePrologEpilog ). Added support for the RESYNC and FM codes
 * Revision 1.3  1995/10/13 00:10:56  sasha
 * Fixed YUV9 and YUV12 odd number of chroma pixels bug.
 * Changed the loop for it and calculation of PitchPlane2.
 * Revision 1.2  1995/10/12 18:34:54  sasha
 * Changed the skipping a bit
 */
static char rcsid[] = "$Id: riscprog.cpp 2.27 1998/06/01 18:09:11 mdimeo Exp $" ;
#include "riscprog.h"
#define ClearMem( a ) memset( &##a, '\0', sizeof( a ) )
#include "dbg.h"

#include "dbgprint.h"

#define VARIABLE_CAPTURE_RATE

const FPS_NTSC = 30;
const FPS_PAL  = 25;

int   RISCProgram::MapIdx_ = -1;
WORD  RISCProgram::DistMap_ [30];
DWORD RISCProgram::dwMilSecPerFrame_ = 33;

/* Method:  RISCProgram::ChangeAddress
 * Purpose: Modifies existing program to use new destination address
 * Input: dwNewAddr: DWORD - new buffer address
 * Output: None
 */
void  RISCProgram::ChangeAddress( LPVIDEOHDR lpNewVH )
{
   Create( Interrupting_, lpNewVH, dwPlanarAdjust_, GenerateResync_, false );
}

/* Function: CreatePrologEpilog
 * Purpose: Called from Create function to put proper sync codes at the beginning
 *   and at the end of a RISC program
 * Input: pProgLoc: PDWORD - pointer to the instruction memory
 *    SyncBits: SyncCode
 *   CurCommand: Command & - reference to a command object
 * Output: PDWORD - address of the next instruction
 */
inline PDWORD RISCProgram::CreatePrologEpilog( PDWORD pProgLoc, SyncCode SyncBits,
   Command &CurCommand, bool Resync, bool bDataProg )
{
   if (bDataProg)
      CurCommand.Create( pProgLoc, SYNC, NULL, NULL, false );
   else
      // for jump and sync progs, SOL and EOL are reserved (0)
      CurCommand.Create( pProgLoc, SYNC, NULL, NULL, false,false,false );

   CurCommand.SetSync( pProgLoc, SyncBits, Resync );
   // advance to the next command's position
   return pProgLoc + CurCommand.GetInstrSize();
}

inline bool IsWithin( int coord, int top, int bot )
{
   return bool( coord >= top && coord < bot );
}

inline PDWORD FinishWithSkip( int pixels, int bpp, PDWORD pProgLoc, Command &com )
{
   WORD awByteCounts [1];
   awByteCounts [0] = WORD( pixels * bpp );
   return (LPDWORD)com.Create( pProgLoc, SKIP, awByteCounts, NULL,
      true, false, true, false ); // safety, SOL, EOL, Intr
}

ErrorCode RISCProgram::GetDataBufPhys( DWORD dwLinAddr )
{
   dwBufAddr_ = GetPhysAddr( dwLinAddr );                              
   if ( dwBufAddr_ == (DWORD)-1 ) {
      return Fail;
   }
   return Success;
}

/* Method:  RISCProgram::SetClipping
 * Purpose: Modifies program to take new clipping into account
 * Input: ClipList: LPRECT - pointer to the array of RECTS describing _visible_
 *   portion of the image
 *   Num: int - number of elements in the array
 * Output:
 * Note:
 */
void  RISCProgram::SetClipping( LPRECT ClipList, int Num )
{
   // first of all, make sure this program is skipped during update
//   PsPageBlock *OldProgSpace = ProgramSpace_;

   //ProgramSpace_ = NULL;

   if ( AllocateStorage( true, Num ) != Success ) {
//      ProgramSpace_ = OldProgSpace;
      return; // make the function return ErrorCode ?
   }

   int ScanLine = 0;
   int bpp = BufFormat_.GetBitCount() / 8;

   // that's were the instructions are going
   LPDWORD pProgLoc = (LPDWORD)ActiveProgramSpace_->getLinearBase();
#ifdef DEBUG
   DWORD startAddr = (DWORD) pProgLoc ;
#endif

   Command CurCommand;

   // put one of the FM codes here
   pProgLoc = CreatePrologEpilog( pProgLoc, SC_FM1, CurCommand );

   WORD  awByteCounts [2];
   DWORD adwAddresses [3];

   DWORD dwAddrTmp;

   int offset = 0;
   for ( ; ScanLine < ImageSize_.cy; ScanLine++ ) {

      int RectNumber;
      // finish off previous scan line - this is the SKIP
      if ( offset ) {
         pProgLoc = FinishWithSkip( ImageSize_.cx - offset, bpp, pProgLoc, CurCommand );
         offset = 0;
      }
      dwAddrTmp = dwBufAddr_ + dwBufPitch_ * ScanLine;

      // indicator that at least 1 rect was processed
      bool DidRect = false;

      for ( RectNumber = 0; RectNumber < Num; RectNumber++ ) {

         TRect r = ClipList [RectNumber];

         if ( IsWithin( ScanLine, r.top, r.bottom ) != true ) {
            continue;
         }
         // make a note that at least one rectangle was processed
         DidRect = true;

         Instruction WriteCom = WRIT;

         if ( r.left ) {

            bool SOL = false;
            // create a skip command
            if ( offset ) {
               // continue same line, new rectangle
               awByteCounts [0] = WORD( ( r.left - offset ) * bpp );
               WriteCom = WRITEC;
            } else { // next line, new rectangle
               // just generate skip
               awByteCounts [0] = WORD( r.left * bpp );
               SOL = true;
            }
            offset = r.left;
            pProgLoc = (LPDWORD)CurCommand.Create( pProgLoc, SKIP, awByteCounts,
               NULL, true, SOL, false );

            // do not forget to advance the address !
            dwAddrTmp += awByteCounts [0];
         }
         // create a write command
         adwAddresses [0] = dwAddrTmp;

         awByteCounts [0] = WORD( r.Width() * bpp );
         pProgLoc = (PDWORD)CurCommand.Create( pProgLoc, WriteCom, awByteCounts,
            adwAddresses, true, offset ? false : true, false ); // safety, sol, eol

         // keep track of current line
         offset += r.Width();

         // advance destination address
         dwAddrTmp += awByteCounts [0];

         if ( offset >= ImageSize_.cx ) { // should never be greater...
         // probably could get this test out of this loop...
            offset = 0;
            CurCommand.SetEOL( CurCommand.GetInstrAddr() );
            break; // finished a scan line; start all over again; actually, there should be no rects anymore at this point
         }
      } /* endfor */

      if ( DidRect == false ) {
         // skip the entire line - wow !
         awByteCounts [0] = WORD( ImageSize_.cx * bpp );
         pProgLoc = (LPDWORD)CurCommand.Create( pProgLoc, SKIP,
            awByteCounts, NULL );
      }
   } /* endfor */

   pChainAddress_ = pProgLoc;
   // take care of the case when there is a skip at the end of the last line
   if ( offset )
      pChainAddress_ = FinishWithSkip( ImageSize_.cx - offset, bpp, pProgLoc, CurCommand );

   // now the size of the program is different from the original one
   // as a result have to restore the original ending;

// We assume that this program has been skipped before this modification. It'll be
// unskipped too.
   //PutInChain();

#ifdef DEBUG
   if (((DWORD)pChainAddress_ - startAddr) > dwActiveProgramSize_)
     DebugPrint((0,">>> PROG LARGER THAN ALLOC IN SETCLIPPING! (%d > %d)\n",
                   (DWORD)pChainAddress_ - startAddr,dwActiveProgramSize_) ) ;
#endif
   // free the old program
//   delete OldProgSpace;
}

/* Method: RISCProgram::AllocateStorage
 * Purpose: Allocates a number of pages ( locked and physically contiguous ) to
 *   hold the new program
 * Input: None
 * Output: ErrorCode
 */
ErrorCode RISCProgram::AllocateStorage( bool extra, int NumViewRects)
{
   if ( ActiveProgramSpace_ && !extra)  // extra is set only when clipping
      return Success;

   // figure out size of the memory to hold the program
   // at least as many DWORDs as lines
   dwActiveProgramSize_ = ImageSize_.cy * sizeof( DWORD );

   // scale up according to the data format
   switch ( BufFormat_.GetColorFormat() ) {
   case CF_RGB32:
   case CF_RGB24:
   case CF_RGB16:
   case CF_RGB15:
   case CF_Y8:
   case CF_YUY2:
   case CF_UYVY:
   case CF_BTYUV:
   case CF_RGB8:
   case CF_RAW:
   case CF_VBI:
//      dwActiveProgramSize_ *= 2; // size of 'Write' command is 2 DWORDs
      dwActiveProgramSize_ *= 6; // size of 'Write' command is 2 DWORDs
      if ( extra == true )  // doing clipping
         dwActiveProgramSize_ *= (3 + NumViewRects);
      break;
   case CF_PL_422:
   case CF_PL_411:
   case CF_YUV9:
   case CF_YUV12:
   case CF_I420:
//      dwActiveProgramSize_ *= 5; // Planar WRITE is 5 DWORDs
      dwActiveProgramSize_ *= 8; // Planar WRITE is 5 DWORDs
   }
   // add extra for page crossings
   //dwActiveProgramSize_ += ImageSize_.cx * ImageSize_.cy * BufFormat_.GetBitCount() / 8
   //   / PAGE_SIZE * sizeof( DWORD ) * 5;

#ifdef VARIABLE_CAPTURE_RATE
   if ( GenerateResync_ ) {
      int cnt = dwMilSecPerFrame_ / 33;
      dwActiveProgramSize_ += cnt * 8;//extra memory for the synch instructions;8 bytes per instr
      if ( dwMilSecPerFrame_ <= 1000 ) { // do not use the distribution map when below 1fps
         // 'rotate' array around so it starts with a '1'
         if ( MapIdx_ == -1 ) {
            CreateDistribution( (WORD) (IsNTSCCompatible(vidFormat_) ? FPS_NTSC : FPS_PAL) , 
                                (WORD) ((DWORD) 1000 / dwMilSecPerFrame_), DistMap_ );
            while ( !DistMap_ [++MapIdx_] ) ;
            MapIdx_++;
         }
      }
   }
#endif

   // Check which program space is active. If inactive program space is smaller than
   // required size, allocate a new one
   if (ActiveProgramSpace_ == ProgramSpaceA_)
   {
     if (dwProgramSizeB_ < dwActiveProgramSize_)
     {
       DebugPrint((1,"Allocating new ProgSpaceB  (dwProgramSizeB_ %d< dwActiveProgramSize_ %d)\n",
                       dwProgramSizeB_ , dwActiveProgramSize_ )) ;
       if (ProgramSpaceB_)
         delete ProgramSpaceB_ ;
       ProgramSpaceB_ = new PsPageBlock( dwActiveProgramSize_ );
       dwProgramSizeB_ = dwActiveProgramSize_ ;
     }
     else
       DebugPrint((1,"*B\n")) ;    

     ActiveProgramSpace_ = ProgramSpaceB_ ; 
     dwActiveProgramSize_ = dwProgramSizeB_;
   }
   else
   {
     if (dwProgramSizeA_ < dwActiveProgramSize_)
     {
       DebugPrint((1,"Allocating new ProgSpaceA  (dwProgramSizeA_ %d< dwActiveProgramSize_ %d)\n",
                       dwProgramSizeA_ , dwActiveProgramSize_ )) ;
       if (ProgramSpaceA_)
         delete ProgramSpaceA_ ;
       ProgramSpaceA_ = new PsPageBlock( dwActiveProgramSize_ );
       dwProgramSizeA_ = dwActiveProgramSize_ ;
     }
     else
       DebugPrint((1,"*A\n")) ;

     ActiveProgramSpace_ = ProgramSpaceA_ ; 
     dwActiveProgramSize_ = dwProgramSizeA_;
   }

   if ( ActiveProgramSpace_ && ActiveProgramSpace_->getLinearBase() != 0 )
      return Success;
   return Fail;
}

/* Function: GetAlternateSwitch
 * Purpose: Chooses alternative instruction frequency
 * Input: AlternateSwitch: int
 *    col: ColFmt, color format
 * Output: None
*/
inline void GetAlternateSwitch( int &AlternateSwitch, ColFmt col )
{
   AlternateSwitch = col == CF_YUV9  ? 4 :
                     col == CF_YUV12 ? 2 : 1;
}

/* Function: GetSplitAddr
 * Purpose: Calculates 4K-aligned address
 * Input: dwLinBufAddr: DWORD - linear address
 * Output: DWORD
*/
inline DWORD GetSplitAddr( DWORD dwLinBufAddr )
{
   return ( dwLinBufAddr + 0x1000 ) & 0xFFFFF000L;
}

/* Function: GetSplitByteCount
 * Purpose: Calculates number of bytes before the page boundary
 * Input: dwLinBufAddr: DWORD, address
 * Output: WORD, byte count
*/
inline WORD GetSplitByteCount( DWORD dwLinBufAddr )
{
   return WORD( 0x1000 - ( dwLinBufAddr & 0xFFF ) );
}

/* Function: GetSplitNumbers
 * Purpose: Calculates addresses and byte counts when scan line crosses a page boundary
 * Input: dwLinAddr: DWORD, starting linear address
 *   wByteCount: WORD &, number of bytes to move before page crossing
 *   wByteCSplit: WORD &, number of bytes to move after page crossing
 *   SecondAddr: DWORD &, reference to the DWORD contatining address of the starting
 *   address for the second 'write' instruction
 *   FirstAddr: DWORD &,
 */
void GetSplitNumbers( DWORD dwLinAddr, WORD &wFirstByteCount, WORD &wSecondByteCount,
   DWORD &SecondAddr, DWORD &FirstAddr )
{
   FirstAddr = GetPhysAddr( dwLinAddr );

   if ( Need2Split( dwLinAddr, wFirstByteCount ) ) {

      wSecondByteCount = wFirstByteCount;

      // lin address of the second write command ( page aligned )
      SecondAddr = GetSplitAddr( dwLinAddr );

      // byte count of first write command
      wFirstByteCount = GetSplitByteCount( dwLinAddr );
      wSecondByteCount -= wFirstByteCount;

      // get the physical addresses
      SecondAddr = GetPhysAddr( SecondAddr );
   } else {
      wSecondByteCount = 0;
      SecondAddr = 0;
   }
}

/* Function: AdjustByteCounts
 * Purpose: This function is used to calculate 2 byte counts based on the given ratio
 * Purpose:
 */
void AdjustByteCounts( WORD &smaller, WORD &larger, WORD total, WORD ratio )
{
   if ( ratio <= 1 ) {
      smaller = WORD( total >> 1 );
   } else
      smaller = WORD( total / ratio );
   smaller += (WORD)3;
   smaller &= ~3;
   larger = WORD( total - smaller );
}

/* Method: RISCProgram::Create
 * Purpose: Creates a RISC program
 * Input: NeedInterrupt: bool - flag
 * Output: None
 * Note: It is likely this function is used to simply change dst addresses of
 *   an already existing program. It does not seem to make much sense to write
 *   basically the same function ( or the one that has to parse existing program)
 *   to change addresses
 */

// Planar: YUV etc
LPDWORD  RISCProgram::CreatePlanar( bool LoopOnItself )
{
   DWORD dwLinBufAddr = dwLinBufAddr_; // set in create

   // probably should create a class to handle these arrays
   WORD  awByteCounts [3];
   DWORD adwAddresses [3];

   Instruction MainInstrToUse, AltInstrToUse;

   // used to increment planes' addresses
   LONG PlanePitch1 = dwBufPitch_, 
        ChromaPitch = dwBufPitch_;

   // get size in bytes
   DWORD dwYPlaneSize = ImageSize_.cy * dwBufPitch_;

   // this is a physical address
   DWORD Plane1 = dwLinBufAddr_ + dwYPlaneSize, 
         Plane2 ;

   // initialize byte count for all planar modes
   awByteCounts [0] = (WORD)ImageSize_.cx;
   
   //	For an AltInstrToUse (WRITE123), it write pixels to memory in planar 
   //   mode from the FIFOs beginning at the specified address.

   //	For a MainInstrToUse (WRITE1S23), it write pixels to memory in planar 
   //   mode from the FIFO1 beginning at the specified address.  Skip pixels from 
   //   FIFO2 and FIFO3.  This instruction is used to achieve YUV9 and YUV12 color 
   //   modes, where the chroma components are sub-sampled.
   if ( !dwLinBufAddr_ ) { // hack to handle special case of creating a skipper for VBI streams
      MainInstrToUse = SKIP123;
      AltInstrToUse  = SKIP123;
   } else {
      MainInstrToUse = WRITE1S23;
      AltInstrToUse  = WRITE123;
   }

   // handle all planar modes here
   SyncCode  SyncBits = SC_FM3;

   // This is the control offset for YUV9 to use the nth line for sub-sampling,
   // which is initially set to zero (fourth-scanline). 	
   int nOffset = 0;

   // these guys used for the calculation of addresses
   // for different planar mode combinations ( pitch > witdh, interleaving )
   DWORD dwEqualPitchDivider ;
   DWORD dwByteCountDivider  ;
   switch ( BufFormat_.GetColorFormat() ) {
     default:
        dwEqualPitchDivider = 1;
        dwByteCountDivider  = 1;
        break ;
     case CF_PL_422:
        dwEqualPitchDivider = 2;
        dwByteCountDivider  = 2;
        break;
     case CF_PL_411:
        dwEqualPitchDivider = 4;
        dwByteCountDivider  = 4;
        break;
     case CF_YUV9:
        #ifdef YUV9LINE3
        	  nOffset = 3;
        #endif
        dwEqualPitchDivider = 16;
        dwByteCountDivider  = 4;
        break;
     case CF_I420:
     case CF_YUV12:
        dwEqualPitchDivider = 4;
        dwByteCountDivider  = 2;
        break ;
   } /*endswitch*/

   // Init AlternateSwitch.  This flag 
   int AlternateSwitch ;
   GetAlternateSwitch(AlternateSwitch,BufFormat_.GetColorFormat()) ;  // 4 for yuv9, 2 for i420 & yuv12, 1 for everything else

   awByteCounts [1] = awByteCounts [2] =  WORD( awByteCounts [0] / dwByteCountDivider );

   Plane2 = Plane1 + dwYPlaneSize / dwEqualPitchDivider;
   ChromaPitch /= dwByteCountDivider;

   // need to adjust if doing a full-size planar capture.
   Plane2 -= dwPlanarAdjust_;
   Plane1 -= dwPlanarAdjust_;
   Plane2 += dwPlanarAdjust_ / dwByteCountDivider;
   Plane1 += dwPlanarAdjust_ / dwByteCountDivider;

   // U goes first for this color format
   if ( BufFormat_.GetColorFormat() == CF_I420 ) {
      DWORD dwTmp = Plane1;
      Plane1 = Plane2;
      Plane2 = dwTmp;
   }
   // that's were the instructions are going
   LPDWORD pProgLoc = (LPDWORD)(DWORD)ActiveProgramSpace_->getLinearBase();

   //
   // 
   //

   Command CurCommand;  // this will create every command we need - yahoo !

   // put one of the FM codes here if this program is for image data only
   pProgLoc = CreatePrologEpilog( pProgLoc, SyncBits, CurCommand );


   DebugPrint((1,"   Create Planar. Y start %x,  Plane1 %x, Plane2 %x\n",
               dwLinBufAddr, Plane1, Plane2));    

   // init the destination address
   dwLinBufAddr -= PlanePitch1;

   // initial adjustment of chroma pointers
   Plane1 -= ChromaPitch;
   Plane2 -= ChromaPitch;

   // now go into a loop (up to the hight of the image) and create
   // a command for every line. Commands depend on the data format
   for (DWORD i=0;  i < (unsigned)ImageSize_.cy ;i++ ) {

      Instruction CurInstr;

      // now take care of vertically sub-sampled planar modes
      if ( (i+nOffset) % AlternateSwitch != 0 ) {
         CurInstr = AltInstrToUse;
      } else {
         CurInstr = MainInstrToUse;
         Plane2 += ChromaPitch;
         Plane1 += ChromaPitch;
      }
      // advance the linear address to the next scan line
      dwLinBufAddr += PlanePitch1;

      // these arrays contain values for the second instruction
      DWORD adwSecondAddr [3];
      WORD  FirstByteCount [3];
      WORD  SecondByteCount [3];

      adwSecondAddr   [0] = adwSecondAddr   [1] = adwSecondAddr   [2] =
      SecondByteCount [0] = SecondByteCount [1] = SecondByteCount [2] = 0;

      // initialize byte counts
      memmove( FirstByteCount, awByteCounts, sizeof( FirstByteCount ) );

      if ( dwLinBufAddr_ ) // don't bother with the addresses, if we are SKIPping them ! ( VBI hack )
         GetSplitNumbers( dwLinBufAddr, FirstByteCount [0], SecondByteCount [0],
            adwSecondAddr [0], adwAddresses [0] );

      PVOID pEOLLoc; // this is needed to set EOL bit in split instructions

      if ( AlternateSwitch > 1 && dwLinBufAddr_ ) {

         int split = 1;
         // Y plane is already done
         // now check if we better split instructions
         // just make width half of original and create 2 instructions
         if ( ImageSize_.cx > 320 && SecondByteCount [0 ] )
            split = 2;

         // temps for the loop
         DWORD dwYPlane = dwLinBufAddr;
         DWORD dwVPlane = Plane2;
         DWORD dwUPlane = Plane1;

         for ( int k = 0; k < split; k++ ) {

            // initialize byte counts
            memmove( FirstByteCount, awByteCounts, sizeof( FirstByteCount ) );
            // and split them in half
            for ( int l = 0; l < sizeof FirstByteCount / sizeof FirstByteCount [0]; l++ )
               FirstByteCount [l] /= (WORD) split; //create 2 instructions with half the pixels

            // see if any of the planes crosses a page boundary
            GetSplitNumbers( dwYPlane, FirstByteCount [0], SecondByteCount [0],
               adwSecondAddr [0], adwAddresses [0] );
            // V plane
            GetSplitNumbers( dwVPlane, FirstByteCount [1], SecondByteCount [1],
               adwSecondAddr [1], adwAddresses [1] );
            // U plane
            GetSplitNumbers( dwUPlane, FirstByteCount [2], SecondByteCount [2],
               adwSecondAddr [2], adwAddresses [2] );

            // can not have zero Y byte count
            if ( !SecondByteCount [0] && ( SecondByteCount [1] || SecondByteCount [2] ) ) {
               FirstByteCount  [0] -= max( SecondByteCount [1], SecondByteCount [2] );
               FirstByteCount  [0] &= ~3; // need to align for the second address
               SecondByteCount [0] = WORD( awByteCounts [0] / split - FirstByteCount [0] );
               // second addr starts where first ends; no page crossing
               adwSecondAddr [0] = adwAddresses [0] + FirstByteCount [0];
            }
            // now make sure that there are no zero chroma byte counts
            // adjust chroma byte counts in proportion to luma byte counts split
            if ( SecondByteCount [0] )  {
               if ( !SecondByteCount [1] ) {
                  if ( SecondByteCount [0] > FirstByteCount [0] )
                     AdjustByteCounts( FirstByteCount [1], SecondByteCount [1], FirstByteCount [1],
                        WORD( SecondByteCount [0] / FirstByteCount [0] ) );
                  else
                     AdjustByteCounts( SecondByteCount [1], FirstByteCount [1], FirstByteCount [1],
                        WORD( FirstByteCount [0] / SecondByteCount [0] ) );
                  adwSecondAddr [1] = adwAddresses [1] + FirstByteCount [1];
               }
               if ( !SecondByteCount [2] ) {
                  if ( SecondByteCount [0] > FirstByteCount [0] )
                     AdjustByteCounts( FirstByteCount [2], SecondByteCount [2], FirstByteCount [2],
                        WORD( SecondByteCount [0] / FirstByteCount [0] ) );
                  else
                     AdjustByteCounts( SecondByteCount [2], FirstByteCount [2], FirstByteCount [2],
                        WORD( FirstByteCount [0] / SecondByteCount [0] ) );
                  adwSecondAddr   [2] = adwAddresses [2] + FirstByteCount [2];
               }
            }
            // now write out the instructions
            // first command. SOL==true, EOL==false
            pProgLoc = (LPDWORD)CurCommand.Create( pProgLoc, CurInstr,
               FirstByteCount, adwAddresses, LoopOnItself, (bool) (k == 0), false );
            pEOLLoc = CurCommand.GetInstrAddr();

            if ( SecondByteCount [0] || SecondByteCount [1] || SecondByteCount [2] ) {
               // second command
               pProgLoc = (LPDWORD)CurCommand.Create( pProgLoc, CurInstr,
                  SecondByteCount, adwSecondAddr, LoopOnItself, false, false );

               pEOLLoc = CurCommand.GetInstrAddr();
            }
            // adjust starting addresses
            dwYPlane += awByteCounts [0] / 2;
            dwVPlane += awByteCounts [1] / 2;
            dwUPlane += awByteCounts [2] / 2;
         } /* endfor */
         // do not forget the EOL bit !
         CurCommand.SetEOL( pEOLLoc );

      } else {
         // first command. SOL==true, EOL==false
         pProgLoc = (LPDWORD)CurCommand.Create( pProgLoc, CurInstr,
            FirstByteCount, adwAddresses, LoopOnItself, true, false );
         pEOLLoc = CurCommand.GetInstrAddr();

         if ( SecondByteCount [0] || SecondByteCount [1] || SecondByteCount [2] ) {
            // second command
            pProgLoc = (LPDWORD)CurCommand.Create( pProgLoc, CurInstr,
               SecondByteCount, adwSecondAddr, LoopOnItself, false );
         } else
            CurCommand.SetEOL( pEOLLoc );
      } /* endif */
   } //  end for

   return pProgLoc ;
}

// Packed: RGB etc
LPDWORD  RISCProgram::CreatePacked( bool LoopOnItself )
{
   DWORD dwLinBufAddr = dwLinBufAddr_; // set in create

   // probably should create a class to handle these arrays
   WORD  awByteCounts [3];
   DWORD adwAddresses [3];

   Instruction MainInstrToUse ;

   // used to increment planes' addresses
   LONG PlanePitch1 = dwBufPitch_ ;

   // get size in bytes
   DWORD dwYPlaneSize = ImageSize_.cy * dwBufPitch_;

   // initialize byte count for packed modes
   awByteCounts [0] = (WORD)(ImageSize_.cx * BufFormat_.GetBitCount() / 8 );
   awByteCounts [1] =  
   awByteCounts [2] = 0 ;
   
   // Setup for packed mode
   SyncCode  SyncBits = SC_FM1;

   if ( !dwLinBufAddr_ ) { // hack to handle special case of creating a skipper for VBI streams
      MainInstrToUse = SKIP;
   } else {
      MainInstrToUse = WRIT;
   }

   // Are we upside down?
   bool flip ;
   switch ( BufFormat_.GetColorFormat() ) {
      case CF_RGB32:  case CF_RGB24:   case CF_RGB16:
      case CF_RGB15:  case CF_BTYUV:   case CF_RGB8:
         flip = Interrupting_;  // only flip if destination is a Ring 3 buffer??
         break;
      default:
         flip = false ;
         break ;
   }

   // that's were the instructions are going
   LPDWORD pProgLoc = (LPDWORD)(DWORD)ActiveProgramSpace_->getLinearBase();

   Command CurCommand;  // this will create every command we need - yahoo !

   // put one of the FM codes here if this program is for image data only
   pProgLoc = CreatePrologEpilog( pProgLoc, SyncBits, CurCommand );

   // init the destination address
   if ( flip ) {
      dwLinBufAddr += dwYPlaneSize;
      PlanePitch1 = -PlanePitch1;
   } else {
      dwLinBufAddr -= PlanePitch1;
   }

   // now go into a loop (up to the hight of the image) and create
   // a command for every line. Commands depend on the data format
   for (DWORD i = 0 ; i < (unsigned)ImageSize_.cy ; i++) 
   {
     Instruction CurInstr = MainInstrToUse;

     // advance the linear address to the next scan line
     dwLinBufAddr += PlanePitch1;

     // these arrays contain values for the second instruction
     DWORD adwSecondAddr   [3];
     WORD  FirstByteCount  [3];
     WORD  SecondByteCount [3];

     adwSecondAddr   [0] = adwSecondAddr   [1] = adwSecondAddr   [2] =
     SecondByteCount [0] = SecondByteCount [1] = SecondByteCount [2] = 0;

     // initialize byte counts
     memmove( FirstByteCount, awByteCounts, sizeof( FirstByteCount ) );

     if ( dwLinBufAddr_ ) // don't bother with the addresses, if we are SKIPping them ! ( VBI hack )
        GetSplitNumbers( dwLinBufAddr, FirstByteCount [0], SecondByteCount [0],
                         adwSecondAddr [0], adwAddresses [0] );

     // first command. SOL==true, EOL==false
     pProgLoc = (LPDWORD)CurCommand.Create( pProgLoc, CurInstr, FirstByteCount, adwAddresses, LoopOnItself, true, false );

     PVOID pEOLLoc = CurCommand.GetInstrAddr();    // save location 

     if ( SecondByteCount [0] || SecondByteCount [1] || SecondByteCount [2] ) {
        // second command  (SOL == false, EOL == true)
        pProgLoc = (LPDWORD)CurCommand.Create( pProgLoc, CurInstr,  SecondByteCount, adwSecondAddr, LoopOnItself, false );
     } else  {
        // only one command was needed, write the EOL bit
        CurCommand.SetEOL( pEOLLoc );
     }
   } // end for

   return pProgLoc ;
}


ErrorCode  RISCProgram::Create( bool NeedInterrupt, LPVIDEOHDR lpVH, DWORD dwPlanrAdjust, bool rsync, bool LoopOnItself )
{
   //
   // Setup common to Planar & Packed
   //
   dwPlanarAdjust_ = dwPlanrAdjust;
   Interrupting_   = NeedInterrupt;
   GenerateResync_ = rsync;

   DebugPrint((1,"RiscProg::Create...\n" )) ;

   // allocate memory for the program first
   if ( AllocateStorage() != Success ) {
      DebugPrint((0,"   AllocStorage Failed!!!!!!\n")) ;
      return Fail;
   }

   // keep the linear address around
   lpVideoHdr_ = lpVH ;
//   DWORD dwLinBufAddr = dwLinBufAddr_ = lpVH ? lpVH->dwReserved[0] : 0 ;  // linear address is stored in dwReserved[0]
   dwLinBufAddr_ = lpVH ? lpVH->dwReserved[0] : 0 ;  // linear address is stored in dwReserved[0]

   if (dwLinBufAddr_)
   {
      DebugPrint((1,"   this = %x  dwOffset_ = %d dwLinBufAddr = %x lpvh = %x\n",this, dwOffset_,dwLinBufAddr_,lpVH)) ;
//      dwLinBufAddr  += dwOffset_;
      dwLinBufAddr_ += dwOffset_;
   }

   // store the buffer address in case somebody will want to change clipping
   if ( dwLinBufAddr_ && GetDataBufPhys( dwLinBufAddr_ ) != Success ) {
      DebugPrint((0,"   GetDataBufPhys( %x ) Failed!!!!!!\n",dwLinBufAddr_)) ;
      return Fail;
   }

   //
   // Create a planar program or a packed program, depending on the color format
   //
   LPDWORD pProgLoc ;
   switch ( BufFormat_.GetColorFormat() ) {
      case CF_RGB32:
      case CF_RGB24:
      case CF_RGB16:
      case CF_RGB15:
      case CF_BTYUV:
      case CF_RGB8:
      case CF_Y8:
      case CF_YUY2:
      case CF_UYVY:
      case CF_RAW:
      case CF_VBI:
         pProgLoc = CreatePacked( LoopOnItself ) ;
         break;

      case CF_PL_422:
      case CF_PL_411:
      case CF_YUV9:
      case CF_I420:
      case CF_YUV12:
         pProgLoc = CreatePlanar( LoopOnItself ) ;
         break;
   } /*endswitch*/

   //
   //  Do common conclusion
   //
   if (pProgLoc)
   {
      #ifdef VARIABLE_CAPTURE_RATE
      Command CurCommand;  // this will create every command we need - yahoo !
      // now generate resynchs to have given frame rate
      if ( GenerateResync_ ) {
         if ( dwMilSecPerFrame_ > 1000 ) {
            int cnt = dwMilSecPerFrame_ / 33 - 1;
            for ( ; cnt > 0; cnt-- )
               pProgLoc = CreatePrologEpilog( pProgLoc, SC_VRE, CurCommand, true );
         } else {

            int limit = IsNTSCCompatible(vidFormat_) ? FPS_NTSC : FPS_PAL ;

            while ( MapIdx_ < limit && !DistMap_ [MapIdx_++] ) {
               pProgLoc = CreatePrologEpilog( pProgLoc, SC_VRE, CurCommand, true );
               MapIdx_ %= limit;
            }
            MapIdx_ %= limit;
         }
      }
      #endif

      pChainAddress_ = pProgLoc;
      pIRQAddress_ = pProgLoc;

      PutInChain();
      
      Skipped_ = false;

      // remember the creation time, so it is possible to notify of interrupt later
      SetTagEx( Get_System_Time() );
   }

   return Success ;
}

/* Method: RISCProgram::PutInChain
 * Purpose: Restores the chain of programs this program was in.
 * Input: None
 * Output: None
 * Note: The chain is destroyed when clipping is set or buffer address is changed
 */
void RISCProgram::PutInChain()
{
   if ( pChild_ )
      SetChain( pChild_ );

   if ( pParent_ )
      pParent_->SetChain( this );
}

/* Method: RISCProgram::SetChain
 * Purpose: Chains this program to another one
 * Input: dwProgAddr: DWORD - address of a first instruction in the next program
 * Output: None
 */
void  RISCProgram::SetChain( RISCProgram *ChainTo )
{
   if ( !ChainTo )
      return;

	// now we know where we are chaining to
   pChild_ = ChainTo;

   // now child knows who chains to it.Does it really want to know its parent?<g>
   pChild_->SetParent( this );
   DebugPrint((1,"Phys %x -> %x\n", GetPhysProgAddr(),pChild_->GetPhysProgAddr())) ;
   SetJump( (PDWORD)pChild_->GetPhysProgAddr() );
}

/* Method: RISCProgram::Skip
 * Purpose: Changes first instruction so programs jumps over itself and to the child
 * Input: None
 * Output: None
 * Note: This functionality is useful when there are not enough data buffers
 *   to supply for this program
 */
void RISCProgram::Skip()
{
// change first SYNC into JUMP
   PDWORD pTmpAddr = pChainAddress_;
   pChainAddress_ = (PDWORD)GetProgAddress();
   SetJump( (PDWORD)GetPhysAddr( (DWORD)pTmpAddr ) );
   pChainAddress_ = pTmpAddr;

   Skipped_ = true;
}

/* Method: RISCProgram::SetJump
 * Purpose: Creates a JUMP instruction to chain some place
 * Input: JumpAddr: PDWORD - target address
 * Output: None
 */
void RISCProgram::SetJump( PDWORD JumpAddr )
{
   Command JumpCommand;
   DWORD adwAddresses [1];
   adwAddresses [0] = (DWORD)JumpAddr;
   JumpCommand.Create( pChainAddress_, JUMP, NULL, adwAddresses, false );
   // make the last JUMP interrupt
   if ( Interrupting_ ) {
      if (!pIRQAddress_)
        asm int 3 ;

      //DebugPrint((0,"RP::SetJump -  pIRQAddress_: %08lx", pIRQAddress_)) ;

      JumpCommand.SetIRQ( pIRQAddress_ );
      if ( Counting_ )
         SetToCount();
      else
         ResetStatus();
   }
}

/* Method: RISCProgram::CreateLoop
 * Purpose: Creates a closed loop at the end of a RISC program
 * Input:  resync: bool - value of the resync bit
 * Output: None
 */
void RISCProgram::CreateLoop( bool resync )
{
   Command SyncCommand( SYNC );
   SyncCommand.SetResync( pChainAddress_, resync );
   if ( resync == true ) {
      DWORD adwAddresses [1];
      adwAddresses [0] = GetPhysAddr( (DWORD)pChainAddress_ );
      SyncCommand.Create( pChainAddress_, JUMP, NULL, adwAddresses );
   }
}

/* Method: RISCProgram::Create
 * Purpose: Creates a simple SYNC and JUMP program
 * Input: SyncBits: SyncCode - defines what code to do resync with
 * Output: None
 */
ErrorCode RISCProgram::Create( SyncCode SyncBits, bool resync )
{
   // allocate memory for the program first
   if ( AllocateStorage() != Success )
      return Fail;

   Command CurCommand;  // this will create every command we need - yahoo !

   // that's were the instructions are going
   LPDWORD pProgLoc = (LPDWORD)ActiveProgramSpace_->getLinearBase();

   // put one of the FM or VRx codes here
   pChainAddress_ = CreatePrologEpilog( pProgLoc, SyncBits, CurCommand, resync );
   CreateLoop( true );
   return Success;
}

RISCProgram::~RISCProgram()
{
   DebugPrint((1,"Deleting PS_A & B  %x %x\n",  ProgramSpaceA_,    ProgramSpaceB_)) ;
   delete ProgramSpaceA_;
   delete ProgramSpaceB_;
   ProgramSpaceA_ = NULL;
   ProgramSpaceB_ = NULL;
   ActiveProgramSpace_ = NULL;
   dwActiveProgramSize_ =  dwProgramSizeA_  = dwProgramSizeB_ = 0 ;

   if ( pParent_ )
      pParent_->SetChild( NULL );
#ifdef VARIABLE_CAPTURE_RATE
   if ( GenerateResync_ ) 
      MapIdx_ = -1;
#endif
}

