/* File: Riscprog.h
 * Purpose: Contains definition of the RISCProgram class
 * Author: sasha
 * $Log: riscprog.h $
 * Revision 2.14  1998/05/05 20:38:32  dtharp
 * Removed obsolete functions WriteSentinel() & SentinelPresent().
 * Revision 2.13  1998/04/02 21:52:04  dtharp
 * Merge DT_VIDEOHDR branch. Implements a new buffer 
 * management scheme.
 * Revision 2.12.2.5  1998/04/02 21:29:29  dtharp
 * Cleanup before merge.
 * Revision 2.12.2.4  1998/04/02 16:40:33  dtharp
 * Fixed yuv9 problem
 * Revision 2.12.2.3  1998/04/01 21:20:05  dtharp
 * Incremental Checkin
 * Revision 2.12.2.2  1998/03/27 15:30:29  dtharp
 * Incremental check-in
 * Revision 2.12.2.1  1998/03/18 19:38:35  dtharp
 * Branch for passing videoheaders around
 * Revision 2.12  1997/06/27 23:06:12  dtharp
 * Made dwMilSecPerFrame_ public.
 * Revision 2.11  1997/06/27 15:59:09  dtharp
 * New sentinel based buffer management scheme.
 * Revision 2.10.1.1  1997/06/10 18:01:10  dtharp
 * Branch for new Buffer mgmnt scheme.
 * Revision 2.10  1997/04/28 21:27:32  dtharp
 * Grafted CC Primary surface branch from 2.9.1.1 Bug ID 475.
 * Revision 2.9.1.1  1997/04/24 21:07:44  dtharp
 * Stable CC Primary < full screen.
 * Revision 2.9  1997/03/20 22:26:49  dtharp
 * Grafted WDM buffer managment branch.
 * Revision 2.8.1.3  1997/03/18 15:55:22  dtharp
 * Added variable frame rate.
 * Revision 2.8.1.2  1997/03/17 16:52:49  dtharp
 * Works @ 30fps NTSC.
 * Revision 2.8.1.1  1997/03/12 18:22:09  dtharp
 * Single field capture kinda works.
 * Revision 2.8  1997/02/19 21:22:18  dtharp
 * Grafted DT_PAL_CAPTURE_RATE branch (bug id 384)
 * Revision 2.7.2.1  1997/02/10 22:01:01  dtharp
 * Branch for bug #384. Added videoFormat param and member variable.  
 * Revision 2.7  1997/01/09 23:53:49  ssm
 * graft from revision 2.6.1.2, fix clientY < 0 problem
 * Revision 2.6.1.2  1996/12/25 03:11:29  sasha
 * Added AboutToClip_ and a method to set it. This is to avoid creation
 * of a risc program when clipping is about to be applied.
 * Revision 2.6.1.1  1996/12/25 02:53:04  sasha
 * Branch for the above the screen fix
 * Revision 2.6  1996/10/11 10:25:58  sasha
 * Added GetRate() member function.
 * Revision 2.5  1996/10/04 00:04:57  sasha
 * Bug ID# 254. Added array for holding frame rate map and index for it.
 * Revision 2.4  1996/09/27 00:05:09  sasha
 * Bug ID# 254. Added bool GenerateResync_. Added static DWORD dwMilSecPerFrame_.
 * Added bool parameter to the Create(). Added static SetRate().
 * Revision 2.3  1996/09/22 23:34:19  sasha
 * Changed dwPlanarAdjust_ to be non-static. Instead a DWORD argument
 * is passed into Create() to set that member. As a result removed 
 * SetPlanarAdjust() function. All this is needed to support AVI capture
 * of 640x480 YUV planar data.
 * Revision 2.2  1996/09/07 07:23:44  sasha
 * Added dwPlanarAdjust_ and static function to set it.
 * Revision 2.1  1996/06/10 20:13:58  ssm
 * MarkBeta1AsRev2
 * Revision 1.12  1996/02/27 02:02:14  sasha
 * Added GetDataBuffer() function
 * Revision 1.11  1996/02/14 02:45:45  sasha
 * Added extra tag data member. Added default argument to Create(). Added
 * SetResync() function. Added Set/GetTagEx().
 * Revision 1.10  1996/01/18 21:52:15  sasha
 * Changed name of the argument to GetDataBufPhys() to dwLinAddr
 * Revision 1.9  1996/01/06 02:07:06  sasha
 * Changed ColorFormat to ColFmt
 * Revision 1.8  1996/01/02 18:43:36  sasha
 * Added member function PutInChain()
 * Revision 1.7  1995/12/08 22:34:29  sasha
 * Modified the way programs are linked. Update clipping transparently. 
 * Added code to get physical addresses
 * Revision 1.6  1995/11/02 20:57:24  sasha
 * First major check-in for testing
 * Revision 1.5  1995/10/21 04:31:10  sasha
 * Changed SIZE to TSize. Added protected default constructor.
 * Added CreatePrologEpilog function. Overloaded Create function. 
 * Removed bool VeryFirst argument from original Create function. 
 * Changed CreateLoop function to accept just a bool ( for resync ).
 * Added CreateStarter function
 * Revision 1.4  1995/10/16 18:58:48  sasha
 * Added support for clipping.  
 * Revision 1.3  1995/10/12 22:45:42  sasha
 * Added a bool parameter to the Create function. It is set to true for the very
 * first RISC program and to false otherwise. Used for creation of the 
 * RESYNC command
 * Revision 1.2  1995/10/12 18:32:58  sasha
 * Changed the skipping a bit
 */

#ifndef __RISCPROG_H
#define __RISCPROG_H

#if !defined __vtoolscp_h && !defined TESTING
#include "vtoolscp.h"
#endif

#undef FAR

#ifndef __WINDOWS_H
#include <windows.h>
#endif

#ifndef __MYTYPES_H
#include "mytypes.h"
#endif

#ifndef __VIDDEFS_H
#include "viddefs.h"
#endif

#ifndef __COLSPACE_H
#include "colspace.h"
#endif

#ifndef __PSPAGEBL_H
#include "pspagebl.h"
#endif

#ifndef __COMMAND_H
#include "command.h"
#endif 

#include "dbgprint.h"

const Programs = 24;

// (VxD)
const ProgTimeNTSC = 33 ;
const ProgTimePAL  = 40 ;
// (WDM)
//const ProgTimeNTSC = 333667 ;

void CreateDistribution( WORD OrigFPS, WORD NeededFPS, PWORD lpwDistribution );

/* Class: RISCProgram
 * Purpose: Facilitates creation and maintenance of a RISC programs
 * Attributes: ImageSize_: SIZE - structure containing dimentions of the image
 *   dwBufPitch_: DWORD - buffer pitch
 *   Field_: VideoField - which field the program is for
 *   Interrupting_: bool
 *   ProgramSpace_: PsPageBlock * - pointer to the class that manages the memory
 *   occupied by a program ( allocates, deallocates, obtains information )
 *   BufFormat_: ColorSpace - format of the data in the buffer
 * Operations:
 *   void  ChangeAddress( LPVIDEOHDR lpNewVH )
 *   void  SetClipping( LPRECT pRect )
 *   void  Create( bool NeedInterrupt )
 *   void  SetChain( DWORD dwProgAddr )
 *   DWORD GetProgAddress()
 *   bool  IsInterrupting()
 */
class RISCProgram
{
   private:
      LPVIDEOHDR   lpVideoHdr_ ;
      TSize        ImageSize_;
      DWORD        dwBufPitch_;
      DWORD        dwOffset_;

      bool         Interrupting_;
      bool         Counting_;
      
      PsPageBlock *ActiveProgramSpace_;   
      PsPageBlock *ProgramSpaceA_;   
      PsPageBlock *ProgramSpaceB_;   

      DWORD        dwActiveProgramSize_ ;
      DWORD        dwProgramSizeA_ ;
      DWORD        dwProgramSizeB_ ;

      ColorSpace   BufFormat_;
      RISCProgram *pChild_;
      RISCProgram *pParent_;
      PDWORD       pChainAddress_;
      PDWORD       pIRQAddress_;
      DWORD        dwBufAddr_;
      DWORD        dwLinBufAddr_;
      bool         Skipped_;
      DWORD        SkipTime_ ;
      bool         GenerateResync_;

      PVOID         tag_;
      DWORD         ExtraTag_;

      VideoFormat  vidFormat_ ;

      DWORD        dwPlanarAdjust_;
      static   int   MapIdx_;
      static   WORD  DistMap_ [30];

      bool     bOkToNotify_ ; // for paired master
   public:
      static DWORD        dwMilSecPerFrame_ ; //  added

   protected:
      ErrorCode AllocateStorage( bool extra = false, int cnt = 1 );
      PDWORD CreatePrologEpilog( PDWORD pProgLoc, SyncCode SyncBits,
         Command &CurCommand, bool Resync = false, bool bDataProg=true );
      ErrorCode GetDataBufPhys( DWORD dwLinAddr );

      void PutInChain();

   public:
      LPVIDEOHDR   GetVideoHdr() { return lpVideoHdr_ ; }
      DWORD        GetDataBuffer() { return dwLinBufAddr_; }
      void         ChangeAddress( LPVIDEOHDR lpNewVH );
      void         SetClipping( LPRECT pRect, int count );

      PDWORD    CreatePacked( bool Loop );
      PDWORD    CreatePlanar( bool Loop );

      ErrorCode    Create( SyncCode SyncBits, bool resync = false );
      ErrorCode    Create( bool NeedInterrupt, LPVIDEOHDR lpVH, DWORD dwPlanAdjust,
         bool resync = false, bool Loop = true );

      void         MakeFault() { *((PDWORD)GetProgAddress()) = -1; }
      DWORD        GetProgAddress();
      void         SetJump( PDWORD JumpAddr );

      // chaining/unchaining group
      RISCProgram *GetParent() { return pParent_; }
      RISCProgram *GetChild() { return pChild_; }
      void         SetParent( RISCProgram *p ) { pParent_ = p; }
      void         SetChild ( RISCProgram *p ) { pChild_  = p; }
      void         CreateLoop( bool );
      void         SetChain( RISCProgram *ChainTo );
      void         Skip();
      void         SetSkipped(bool sk, DWORD dwTime );
      DWORD        GetSkipTime() { return SkipTime_; }
      void         SetToCount();
      void         ResetStatus();
      void         SetStatus( int val );
      void         SetResync( bool val );

      bool         IsSkipped();
      bool         IsInterrupting();

      void         SetTag( PVOID value );
      PVOID        GetTag();

      void         SetTagEx( DWORD val );
      DWORD        GetTagEx();

      static void  SetRate( DWORD dwMilSec ) { dwMilSecPerFrame_ = dwMilSec; }
      static DWORD GetRate() { return dwMilSecPerFrame_; }

      DWORD GetPhysProgAddr();
      DWORD GetProgramSize() {return dwActiveProgramSize_;}

      // New stuff
      void SetOkToNotify(bool bOk) { bOkToNotify_ = bOk;}
      bool OkToNotify() { return bOkToNotify_; }

      RISCProgram( TSize &size, DWORD pitch, DWORD offset, ColFmt aColor, VideoFormat  vidFormat );

      RISCProgram();

      ~RISCProgram();

      friend class RISCEng;
};

/* Method:  RISCProgram::GetProgramAddress
 * Purpose: Obtains address of this program
 * Input: None
 * Output: DWORD: program's address
 */
inline DWORD RISCProgram::GetProgAddress()
{
	return (DWORD) ActiveProgramSpace_->getLinearBase();
}

inline void RISCProgram::SetToCount()
{
   Command IRQCommand;
   IRQCommand.SetToCount( pIRQAddress_ );
   Counting_ = true;
}

inline void RISCProgram::ResetStatus()
{
   Command IRQCommand;
   IRQCommand.ResetStatus( pIRQAddress_, 0xF );
   Counting_ = false;
}

inline void RISCProgram::SetStatus( int val )
{
   Command IRQCommand;
   IRQCommand.SetStatus( pIRQAddress_, val );
}

inline void RISCProgram::SetResync( bool val )
{
   Command SyncCommand;
   SyncCommand.SetResync( (PVOID)GetProgAddress(), val );
}

inline void  RISCProgram::SetSkipped( bool sk, DWORD dwTime )
{
   SkipTime_ = dwTime ;
   Skipped_ = sk ;
}

inline bool  RISCProgram::IsSkipped()
{
   return Skipped_;
}

inline bool  RISCProgram::IsInterrupting()
{
   return Interrupting_;
}

inline void  RISCProgram::SetTag( PVOID value )
{
   tag_ = value;
}

inline PVOID RISCProgram::GetTag()
{
   return tag_;
}

inline void RISCProgram::SetTagEx( DWORD val )
{
   ExtraTag_ = val;
}

inline DWORD RISCProgram::GetTagEx()
{
   return ExtraTag_;
}

inline DWORD RISCProgram::GetPhysProgAddr()
{
   return ActiveProgramSpace_->GetPhysAddr();
}

inline RISCProgram::RISCProgram( TSize &size, DWORD pitch, DWORD offset, ColFmt aColor, VideoFormat vidFormat ) :
   ImageSize_( size ), dwBufPitch_( pitch ), dwOffset_(offset), Interrupting_( false ),
   BufFormat_( aColor ), ActiveProgramSpace_( NULL ),ProgramSpaceA_( NULL ),ProgramSpaceB_( NULL ), tag_( NULL ), 
   dwPlanarAdjust_( 0 ), pChild_( NULL ), pParent_( NULL ), 
   pChainAddress_( NULL ), GenerateResync_( false ), Skipped_( false ),
   pIRQAddress_( NULL ), dwBufAddr_( 0 ), Counting_( false ), vidFormat_(vidFormat),
   dwActiveProgramSize_(0), dwProgramSizeA_(0), dwProgramSizeB_(0), bOkToNotify_(false), SkipTime_(0), ExtraTag_(0)
{
}                                      

inline RISCProgram::RISCProgram() :
   ImageSize_( 10, 10 ), dwBufPitch_( 0 ), dwOffset_(0), Interrupting_( false ),
   Counting_( false ), BufFormat_( CF_RGB32 ), ActiveProgramSpace_( NULL ),ProgramSpaceA_( NULL ),ProgramSpaceB_( NULL ),
   tag_( NULL ), GenerateResync_( false ), pChild_( NULL ), pParent_( NULL ),
   pChainAddress_( NULL ), Skipped_( false ),
   pIRQAddress_( NULL ), dwBufAddr_( 0 ), dwLinBufAddr_( 0 ),
   dwActiveProgramSize_(0), dwProgramSizeA_(0), dwProgramSizeB_(0), bOkToNotify_(false), SkipTime_(0), ExtraTag_(0)
{
}

#endif