// $Header: G:/SwDev/W95/Video/pisces/dvlpmnt/Scaler/rcs/scaler.h 2.3 1997/05/05 17:25:12 dtharp Released $
/////////////////////////////////////////////////////////////////////////////
// $Log: scaler.h $
// Revision 2.3  1997/05/05 17:25:12  dtharp
// 848A Merge: from 2.2.1.3
// Revision 2.2.1.3  1997/02/14 23:45:37  rudyl
// Removed all #ifdef BT848A. Use same binary files for both Bt848 and Bt848A.
// Revision 2.2.1.2  1997/02/03 23:45:46  rudyl
// First Pisces+ check in.
// Revision 2.2.1.1  1996/11/11 23:47:35  rudyl
// Start Pisces+ work.
// Revision 2.2  1996/07/29 05:27:46  sasha
// Added TurnVFilter() and static VFilterFlag_ member.
// Revision 2.1  1996/06/10 20:12:42  ssm
// MarkBeta1AsRev2
// Revision 1.9  1996/05/18 01:58:34  rudyl
// Works for PAL now.
// Changed Set525Lines() and m_525Lines static so Set525Lines() is called in SetVideoFormat() in decoder.cpp
// in order to set up correct register values for different formats.
// Revision 1.8  1996/05/02 22:10:01  rudyl
// Take care of the problem of VDelay register is reversed in hardware.
// That is odd reg is really even and vice versa.
// Revision 1.7  1996/05/02 16:59:25  rudyl
// Modified to use Register class. Fixed variable scaling problem.
// Revision 1.6  1996/04/22 21:42:48  rudyl
// Added variable scaling. No testing done.
// Revision 1.5  1996/01/29 00:10:51  ssm
// add ScaleHardNTSC method
// Revision 1.4  1996/01/19 00:32:56  sasha
// Fixed accesses to the array of pointers and pointer declaration
// Revision 1.3  1996/01/17 22:28:52  sasha
// Changed argument from TRect to TRect & to Scaler functions.
// Revision 1.2  1996/01/06 01:36:17  ssm
// stubbed in code from alex's scaler to test interface only
// Revision 1.2  1996/01/06 00:55:49  ssm
// 1st compile with scaler
// Revision 1.1  1996/01/05 22:19:29  ssm
// Initial revision
// Revision 1.1  1996/01/05 21:03:43  ssm
// Initial revision
/////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////
//
//  Copyright 1996 Brooktree Corporation
//
//  Module:
//
//    scaler.h
//
//  Abstract:
//
//    Header file Bt848 Scalar Registers
//
/////////////////////////////////////////////////////////////////////////////

#ifndef __SCALER_H
#define __SCALER_H


#ifdef VXD
#include <vtoolscp.h>
#endif

#undef FAR

#ifndef __WINDOWS_H
#include <windows.h>
#endif

#ifdef VXD
#include "mytypes.h"
#endif

#ifndef __COMPREG_H
#include "compreg.h"
#endif

#define HDROP       HANDLE

#ifndef __COMPAT_H
#include "compat.h"  // this file declares data types that are shared between 16&32 bit code
#endif

#include "viddefs.h"
#include "vxdfunc.h"

// structure contains video information
struct VideoInfoStruct
{
   WORD Clkx1_HACTIVE;
   WORD Clkx1_HDELAY;
   WORD Min_Pixels;
   WORD Active_lines_per_field;
   WORD Min_UncroppedPixels;
   WORD Max_Pixels;
   WORD Min_Lines;
   WORD Max_Lines;
   WORD Max_VFilter1_Pixels;
   WORD Max_VFilter2_Pixels;
   WORD Max_VFilter3_Pixels;
   WORD Max_VFilter1_Lines;
   WORD Max_VFilter2_Lines;
   WORD Max_VFilter3_Lines;
   WORD VActive;
   WORD VDelay;
};


/////////////////////////////////////////////////////////////////
//
// for pisces, instantiate as ...
//
// Scaler evenScaler( VF_Even );
// Scaler oddScaler( VF_Odd );
//
/////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////
// CLASS Scaler
//
// Description:
//    This class encapsulates the register fields in the scaler portion of
//    the Bt848.
//    A complete set of functions are developed to manipulate all the
//    register fields in the scaler registers for the Bt848.
//
// Methods:
//    See below
//
// Remark:
//    When video signal changes, SetSignalFormat() should be called. If analog
//    signal is selected, VideoFormatChanged() should then be called to indicate
//    which analog signal standard is the input. If digital signal is selected,
//    SetVideoInfo() should be called instead to supply information of the
//    digital signal.
//
// Note:
//    For Bt848, instantiate as ...
//       Scaler evenScaler( VF_Even );
//       Scaler oddScaler( VF_Odd );
//
/////////////////////////////////////////////////////////////////////////////

class Scaler
{
	public:
		Scaler( VidField );
		~Scaler();

		static void VideoFormatChanged( VideoFormat );
      static void TurnVFilter( State st ) { VFilterFlag_ = st; }

		void      Scale( TRect & );
      ErrorCode SetAnalogWin( const TRect & );
      void      GetAnalogWin( TRect & ) const;
      ErrorCode SetDigitalWin( const TRect & );
      void      GetDigitalWin( TRect & ) const;

      // for other (non-NTSC, PAL, etc.) video signals such as digital camera
      static void SetVideoInfo( VideoInfo info );
      static void GetVideoInfo( VideoInfo * pInfo );
      static void SignalFormatChanged( SignalFormat format );

   protected:
      #include "S_declar.h"

		VideoInfoStruct * m_ptrVideoIn;
      TRect AnalogWin_;
      TRect DigitalWin_;

      // member functions to set scaling registers
		virtual void SetHActive( TRect & );
		virtual void SetHDelay( void );
		virtual void SetHScale( void );
      virtual void SetHFilter( void );
		virtual void SetVActive( void );
		virtual void SetVDelay( void );
		virtual void SetVScale( TRect & );
		virtual void SetVFilter( void );

   private:
		static VideoFormat  m_videoFormat;   // video format

      // this is to battle junk lines at the top of the video
      static State VFilterFlag_;

      WORD  m_HActive;  // calcuated intermediate value
      WORD  m_pixels;   // calcuated intermediate value
      WORD  m_lines;    // calcuated intermediate value
      WORD  m_VFilter;  // calcuated intermediate value

      // video information added for Pisces+ including digital camera
      static SignalFormat    m_signalFormat;    // video signal format
      static VideoInfoStruct anyVideoInfo;      // video info for scaling
};


#endif __SCALER_H

