/////////////////////////////////////////////////////////////////////////////
// $Header: G:/SwDev/W95/Video/pisces/dvlpmnt/TimGen/rcs/TimGen.cpp 1.2 1997/02/04 18:36:40 rudyl Released $
// $Log: TimGen.cpp $
// Revision 1.2  1997/02/04 18:36:40  rudyl
// First Pisces+ check in.
// Revision 1.1  1996/08/22 21:28:22  rudyl
// Initial revision
/////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////
//
//  Copyright 1996 Brooktree Corporation
//
//  Module:
//
//    TimGen.cpp
//
//  Abstract:
//
//    Bt848A/Bt849 TimGen Class API
//
//  DESIGN NOTES
//
//
/////////////////////////////////////////////////////////////////////////////

#include "TimGen.h"
#include "t_constr.h"


//===========================================================================
// Bt848A/Bt849 TimGen Class Implementation
//===========================================================================


/////////////////////////////////////////////////////////////////////////////
// Constructor
/////////////////////////////////////////////////////////////////////////////

TimGen::TimGen( void ):
   bytesInSRAM( 0 ),
   CONSTRUCT_TIMGEN_REGISTERS
{
}


/////////////////////////////////////////////////////////////////////////////
// Destructor
/////////////////////////////////////////////////////////////////////////////
TimGen::~TimGen()
{
}

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode TimGen::LoadSRAM( PBYTE p, int nBytes )
// Purpose: Load time generator SRAM
// Input:   PBYTE p    - points to value to load from
//          int nBytes - number of bytes to load
// Output:  None
// Return:  Fail if error loading SRAM, else Success
/////////////////////////////////////////////////////////////////////////////
ErrorCode TimGen::LoadSRAM( PBYTE p, int nBytes )
{
   EnableTimGenVideoMode( false );  // select read/write mode
   TimGenAddrReset();               // address reset

   PBYTE q = p;
   for ( int i = 0; i < nBytes; i++ )
//{
//char str [30];
//sprintf( str, "TimGen.cpp in-> %d: %X", i, *q );
//OUTPUT_MESS( str );
      timRegTGLB = *q++;
//}

   EnableTimGenVideoMode( true );   // select video mode

//EnableTimGenVideoMode( false );  // select read/write mode
//{
//char str [30];
//BYTE b;
//TimGenAddrReset();
//for ( i = 0; i < nBytes; i++ )
//{
//   b = timRegTGLB;
//   IncrTimGenReadAddr();
//   sprintf( str, "TimGen.cpp-> %d: %X", i, b );
//   OUTPUT_MESS( str );
//}
//EnableTimGenVideoMode( true );   // select video mode
//}
   return Success;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode TimGen::ReadSRAM( PBYTE p, int nBytes )
// Purpose: Load time generator SRAM
// Input:   int nBytes - number of bytes to copy
// Output:  PBYTE p    - points to value to be copied to
// Return:  Fail if error reading SRAM, else Success
// Note:    Assume *p is large enough to hold nBytes of data
/////////////////////////////////////////////////////////////////////////////
ErrorCode TimGen::ReadSRAM( PBYTE p, int nBytes )
{
   EnableTimGenVideoMode( false );  // select read/write mode
   TimGenAddrReset();               // address reset

   PBYTE q = p;
   for ( int i = 0; i < nBytes; i++ )
   {
      *q++ = timRegTGLB;
      IncrTimGenReadAddr();
   }
   EnableTimGenVideoMode( true );   // select video mode
   return Success;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  void TimGen::EnableTimGenVideoMode( bool mode )
// Purpose: Enable/Disable Timing Generator Video Mode (0-read/write mode)
// Input:   bool mode - true = Enable; false = Disable
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
void TimGen::EnableTimGenVideoMode( bool mode )
{
  timFieldTGC_VM = (mode == false) ? 0 : 1;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  bool TimGen::IsTimGenVideoModeEnabled( void )
// Purpose: Check if timing generator video mode is enable or disable
// Input:   None
// Output:  None
// Return:  true = Enable; false = Disable
/////////////////////////////////////////////////////////////////////////////
bool TimGen::IsTimGenVideoModeEnabled( void )
{
  return (timFieldTGC_VM == On) ? true : false;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  void TimGen::TimGenAddrReset( void )
// Purpose: Reset timing generator address
// Input:   None
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
void TimGen::TimGenAddrReset( void )
{
  timFieldTGC_AR = 0;
  timFieldTGC_AR = 1;
  timFieldTGC_AR = 0;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  void TimGen::IncrTimGenReadAddr( void )
// Purpose: Increment timing generator read address
// Input:   None
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
void TimGen::IncrTimGenReadAddr( void )
{
  timFieldTGC_AI = 0;
  timFieldTGC_AI = 1;
  timFieldTGC_AI = 0;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode TimGen::SetDecoderInputClock( DecInputClk clk )
// Purpose: Select decoder input clock
// Input:   Decoder input clock:
//            DecClk_Normal   - Normal Xtal0/Xtal1 mode
//            DecClk_PLL      - PLL
//            DecClk_GPCLK    - GPCLK
//            DecClk_invGPCLK - GPCLK inverted
// Output:  None
// Return:  Fail if error in parameter, else Success
/////////////////////////////////////////////////////////////////////////////
ErrorCode TimGen::SetDecoderInputClock( DecInputClk clk )
{
   if ( ( clk < DecClk_XTAL     ) ||
        ( clk > DecClk_invGPCLK ) )
      return Fail;

   timFieldTGCKI = clk;
   return Success;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int TimGen::GetDecoderInputClock( void )
// Purpose: Obtain which decoder input clock is selected
// Input:   None
// Output:  None
// Return:  int - Decoder input clock:
//            DecClk_Normal   - Normal Xtal0/Xtal1 mode
//            DecClk_PLL      - PLL
//            DecClk_GPCLK    - GPCLK
//            DecClk_invGPCLK - GPCLK inverted
/////////////////////////////////////////////////////////////////////////////
int TimGen::GetDecoderInputClock( void )
{
   return timFieldTGCKI;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode TimGen::SetGPCLKOutputClock( GPCLKOutputClk clk )
// Purpose: Select GPCLK output clock
// Input:   GPCLK output clock:
//            GPCLK_CLKx1  - CLKx1
//            GPCLK_Xtal0  - Xtal0
//            GPCLK_PLL    - PLL
//            GPCLK_invPLL - PLL inverted
// Output:  None
// Return:  Fail if error in parameter, else Success
/////////////////////////////////////////////////////////////////////////////
ErrorCode TimGen::SetGPCLKOutputClock( GPCLKOutputClk clk )
{
   if ( ( clk < GPCLK_CLKx1  ) ||
        ( clk > GPCLK_invPLL ) )
      return Fail;

   timFieldTGCKO = clk;
   return Success;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int TimGen::GetGPCLKOutputClock( void )
// Purpose: Obtain which GPCLK output clock is selected
// Input:   None
// Output:  None
// Return:  int - GPCLK output clock:
//            GPCLK_CLKx1  - CLKx1
//            GPCLK_Xtal0  - Xtal0
//            GPCLK_PLL    - PLL
//            GPCLK_invPLL - PLL inverted
/////////////////////////////////////////////////////////////////////////////
int TimGen::GetGPCLKOutputClock( void )
{
   return timFieldTGCKO;
}


