/* util_32.cpp
 * Purpose: Implementation of 32-bit registry access helper fns.
 * Author: peter
 * $Log: util_32.cpp $
 * Revision 1.8  1998/02/11 19:39:46  dtharp
 * Cast to aviod compiler error.
 * Revision 1.7  1997/12/12 23:58:04  peter
 * Fixed REG_BINARY problem.
 * Revision 1.6  1997/12/12 17:50:08  peter
 * Removed common debug error messages.
 * Revision 1.5  1997/12/11 17:11:56  peter
 * Added REG_BINARY support to btutil*.dll.
 * Revision 1.4  1997/12/09 17:38:02  peter
 * Fixed 16/32-bit thunking exception upon startup bug.
 * Initial revision
 */


#define _BUILD_BT848DLL    // all __BT848FUNC functions are exported

#include <windows.h>
#include "util_32.h"
#include "dbgprint.h"


extern "C" 
{
BOOL UTIL_ThunkConnect32( LPCSTR, LPCSTR, HINSTANCE hInst, DWORD dwReason );
}


int __BT848FUNC RegUtilCreateKey32( LPCSTR szKeyName )
{					
	HKEY		hKey = NULL;
	if( RegCreateKey( HKEY_LOCAL_MACHINE, szKeyName, &hKey ) == ERROR_SUCCESS )
	{
		//DebugPrint(( 0, "RegCreateKey() success\n" ));
		return 1;
	}
	else
	{
		DebugPrint(( 0, "RegCreateKey() failed\n" ));
		return 0;
	}
}


int __BT848FUNC RegUtilQueryValue32( LPCSTR szKeyName, LPSTR szValue, LPLONG plSize )
{					
	if( RegQueryValue( HKEY_LOCAL_MACHINE, szKeyName, szValue, plSize ) == ERROR_SUCCESS )
	{
		//DebugPrint(( 0, "RegQueryValue() success\n" ));
		return 1;
	}
	else
	{
		//DebugPrint(( 0, "failure:  RegQueryValue( %s )\n", szKeyName ));
		return 0;
	}
}


int __BT848FUNC RegUtilSetValue32( LPCSTR szKeyName, LPCSTR szValue )
{					
	if( RegSetValue( HKEY_LOCAL_MACHINE, szKeyName, REG_SZ, szValue, strlen(szValue) ) == ERROR_SUCCESS )
	{
		//DebugPrint(( 0, "RegSetValue() success\n" );
		return 1;
	}
	else
	{
		//DebugPrint(( 0, "failure:  RegSetValue( %s )\n", szKeyName ));
		return 0;
	}
}




int __BT848FUNC RegUtilQueryBinValue32( int* lpSize, LPSTR cbData, LPCSTR szKeyName, LPCSTR szValueName )
{					
	DWORD		dwType = 0;
   HKEY     hKey = NULL;
   int      nResult = 0;
   if( RegOpenKeyEx( HKEY_LOCAL_MACHINE, szKeyName, 0, KEY_ALL_ACCESS, &hKey ) == ERROR_SUCCESS )
   {
  		//DebugPrint(( 0, "32-bit: RegOpenKeyEx( %s ) success\n", szKeyName ));
	   if( RegQueryValueEx( hKey, szValueName, NULL, &dwType, cbData, (DWORD *)lpSize ) == ERROR_SUCCESS )
	   {
		   if( dwType != REG_BINARY )
		   {
            DebugPrint(( 0, "32-bit: RegUtil 32-bit: RegQueryValueEx(): type is not REG_BINARY... failed\n" ));
		   }
		   else
		   {
      		//DebugPrint(( 0, "32-bit: RegQueryValueEx( %s ) success\n", szValueName ));
			   nResult = 1;
		   }
	   }
      RegCloseKey( hKey );
   }
   else
   {
  		DebugPrint(( 0, "32-bit: RegOpenKeyEx( %s ) failed\n", szKeyName ));
   }
   return nResult;
}



int __BT848FUNC RegUtilSetBinValue32( int lSize, LPSTR cbData, LPCSTR szKeyName, LPCSTR szValueName )
{				
	DWORD		dwDisposition = 0;
   HKEY     hKey = NULL;
   int      nResult = 0;
   if( RegCreateKeyEx( HKEY_LOCAL_MACHINE, szKeyName, NULL, NULL, 0, KEY_ALL_ACCESS, NULL, &hKey, &dwDisposition ) == ERROR_SUCCESS )
   {
  		//DebugPrint(( 0, "32-bit: RegCreateKeyEx( %s ) success\n", szKeyName ));
	   if( RegSetValueEx( hKey, szValueName, 0, REG_BINARY, cbData, (LONG)lSize ) == ERROR_SUCCESS )
	   {
   		//DebugPrint(( 0, "32-bit: RegSetValueEx( %s ) success\n", szValueName ));
	   	nResult = 1;
	   }
      else
      {
   		DebugPrint(( 0, "32-bit: RegSetValueEx( %s ) failed\n", szValueName  ));
      }
      RegCloseKey( hKey );
   }
   else
   {
  		DebugPrint(( 0, "32-bit: RegCreateKeyEx( %s ) failed\n", szKeyName ));
   }
   return nResult;
}


int PASCAL DllEntryPoint( HINSTANCE hInst, DWORD dwReason, LPVOID )
{

	//DebugPrint(( 0, "DllEntryPoint():  32-bit\n" ));

	if ( !UTIL_ThunkConnect32( "btutil16.dll", "btutil32.dll", hInst, dwReason ) )
	{
		DebugPrint(( 0, "UTIL_ThunkConnect32() failed\n" ));
		return FALSE;
	}
	else
	{
		//DebugPrint(( 0, "UTIL_ThunkConnect32() success\n" ));
	}
	return TRUE;
}



