/* File: Vxdvidch.cpp
 * Purpose: Contains implementation of the VxDVideoChannel class
 * Author: sasha
 * $Log: VXDVIDCH.CPP $
 * Revision 2.15  1998/06/01 18:08:49  mdimeo
 * added rcs ident string
 * Revision 2.14  1998/05/14 23:41:50  dtharp
 * Graft from DT_VBI branch: VBI now functional.
 * Revision 2.13.1.2  1998/05/14 23:26:25  dtharp
 * Added code to place new vbi buffers in the vbi queue
 * Revision 2.13.1.1  1998/05/11 17:28:00  dtharp
 * Branch to restore VBI
 * Revision 2.13  1998/04/06 19:42:17  dtharp
 * Vxd Frame Buffers are allocated on the fly instead of at init time.
 * Revision 2.12  1998/04/05 20:28:36  dtharp
 * RestartFrameServer() no longer takes a parameter.
 * Revision 2.11  1998/04/02 21:50:00  dtharp
 * Merge DT_VIDEOHDR branch. Implements a new buffer 
 * management scheme.
 * Revision 2.10.1.3  1998/04/02 21:27:04  dtharp
 * Cleanup before merge.
 * Revision 2.10.1.2  1998/04/01 21:19:52  dtharp
 * Incremental Checkin
 * Revision 2.10.1.1  1998/03/27 15:25:44  dtharp
 * Branch for passing videohdrs around.
 * Revision 2.10  1998/03/03 21:11:54  dtharp
 * Bug Id 654:  Fixed QuartzSight split images.
 * Revision 2.9  1997/08/08 11:17:33  dtharp
 * Fix for bug 714: NotifyArg now passed by reference. Also no 'new' during
 * ISR time.
 * Revision 2.8  1997/06/27 22:50:32  dtharp
 * Commented out some unused variables in EventHandler
 * Revision 2.7  1997/06/27 16:16:34  dtharp
 * Modifications to pass drop count to ring 3.
 * Revision 2.6  1997/05/28 22:33:09  dtharp
 * Fix for BugId 608: Lock up upon modem disconnect. 
 * Added delete call in VBIVxDChannel::Interrupt().
 * Revision 2.5  1997/05/02 23:39:28  dtharp
 * 848A Merge: from 2.2.1.3
 * Revision 2.4  1997/03/26 18:03:46  dtharp
 * Grafted CC fix branch. 
 * Revision 2.3.1.1  1997/03/25 23:42:42  dtharp
 * Protected VxDVideoChannel::EventHandler() from re-entrancy.
 * Revision 2.3  1997/03/20 22:09:27  dtharp
 * Grafted WDM Buffer managment branch.
 * Revision 2.2.3.1  1997/03/12 18:24:55  dtharp
 * Single field capture kinda works.
 * Revision 2.2  1996/09/23 00:50:10  sasha
 * Added Interrupt() member functions to capture and preview channels.
 * This enables control when the capture driver is notified that a buffer 
 * is available. This is needed for the full size AVI capture.
 * Revision 2.1  1996/06/10 20:12:44  ssm
 * MarkBeta1AsRev2
 * Revision 1.18  1996/05/17 20:43:18  sasha
 * Set Started_ to true in the Start() and to false in Stop(). CallRing3Callback()
 * checks to see if channel is still started.
 * Revision 1.17  1996/05/14 23:46:33  sasha
 * Changed Open() to return Fail if opened twice. Added functions to set and
 * call ring3 callback
 * Revision 1.16  1996/05/10 19:58:17  sasha
 * Added check for open in the Stop()
 * Revision 1.15  1996/04/26 21:11:08  sasha
 * Call VBI decoder via an event now instead of directly
 * Revision 1.14  1996/03/15 22:34:54  sasha
 * Removed commented out code. Changed VBIVxDChannel's Interrupt()
 * to pass VBI decode callback buffer address.
 * Revision 1.13  1996/03/04 21:48:50  sasha
 * Removed Interrupt() function from Capture and Preview channels. Updatet
 * Interrupt in the VBIVxDChannel
 * Revision 1.12  1996/02/27 02:09:35  sasha
 * Added AddBuffer to the VBIVxDChannel
 * Revision 1.11  1996/02/19 01:24:51  sasha
 * Added VBIVxDChannel code. Will change
 * Revision 1.10  1996/01/30 19:24:09  sasha
 * Changed a bit  the way callback works.
 * Revision 1.9  1996/01/22 23:58:13  sasha
 * Removed commented out code
 * Revision 1.8  1996/01/06 02:05:58  sasha
 * Changed ColorFormat to ColFmt
 * Revision 1.7  1996/01/04 23:10:09  sasha
 * Changed VxDVideoChannel::AddBuffer() to use current queue
 * Revision 1.6  1996/01/02 18:40:52  sasha
 * Changed BufQue to be an object instead of a pointer. Use nested execution
 * to call ring3 code. Use priority VM events instead of appy time events
 * Revision 1.5  1995/12/08 22:18:17  sasha
 * Started playing with ring3 code notifications
 * Revision 1.4  1995/11/07 01:15:57  sasha
 * Forgot to put the check against NULL when calling callback in the testing
 * version of Interrupt in VxDVideoChannel.
 * Revision 1.3  1995/11/07 00:53:16  sasha
 * Added CaptureVxDVideoChannel and PreviewVxDVideoChannel classes
 * Added SetAppCallback function to the VxDVideoChannel.
 * Revision 1.2  1995/10/15 03:37:50  sasha
 * First major check-in for testing
 * Revision 1.1  1995/10/15 03:37:50  sasha
 * Initial revision
 */

static char rcsid[] = "$Id: VXDVIDCH.CPP 2.15 1998/06/01 18:08:49 mdimeo Exp $" ;
#include "vxdvidch.h"

#ifndef __COMPAT_H
#include "compat.h"
#endif
#include "dbgprint.h"

BtPiscesPlus *VxDVideoChannel::Digitizer_;
EventArg eaArray[12] ;  // one for each risc program

/* Method: VxDVideoChannel::SetDigitalWindow
 * Purpose:
 * Input:
 * Output:
 */
ErrorCode VxDVideoChannel::SetDigitalWindow( TRect &r )
{
   return Digitizer_->SetDigitalWindow( r, OurStream_ );
}

/* Method: VxDVideoChannel::SetAnalogWindow
 * Purpose:
 * Input:
 * Output:
 */
ErrorCode VxDVideoChannel::SetAnalogWindow( TRect &r )
{
   return Digitizer_->SetAnalogWindow( r, OurStream_ );
}

/* Method: VxDVideoChannel::OpenChannel
   if ( !VxDHandle_ ) {
 * Purpose: Allocates a stream from a capture chip
 * Input:
 * Output:
 * Note: It is possible that the current implementation does not require an
 *   elaborate stream allocation scheme. Nonetheless it is used as number of
 *   streams can increase in the future and their dynamics can change
 */
//Field & GetField( StreamInfo &str ) { return *(Field*)str.tag; }
ErrorCode VxDVideoChannel::OpenChannel()
{
   // can not open twice 
   if ( IsOpen() == true )
      return Fail;

   // first of all find out how many streams are out there
   int StreamsNo = Digitizer_->GetNumberOfStreams();

   // go through all in turn looking for our stream
   while ( --StreamsNo >= 0 ) {

      StreamInfo StrInf( StreamsNo, Stream_ );
      Digitizer_->GetStreamInfo( StrInf );

      // if found our stream, allocate it and return
      if ( StrInf.VidStrm == Stream_ ) {

         // let the capture chip know we want notifications
         StrInf.SetCallback( &Caller_ );
         StrInf.Interrupt_ = NeedInterrupt_;

         if ( Digitizer_->AllocateStream( StrInf ) == Success ) {
            SetPaired( false );
            // store information for all subsequent calls
            OurStream_ = StrInf;
            // flag the state
            SetOpen();
//            // this is not supposed to fail !
//            SetDefaultQue();
            return Success;
         }
         break;
      }
   }
   return Fail;
}

/* Method: VxDVideoChannel::CloseChannel
 * Purpose:
 * Input:
 * Output:
 */
ErrorCode VxDVideoChannel::CloseChannel()
{
   Stop();
   Digitizer_->FreeStream( OurStream_ );
//   BufQue_.Flush();
   SetClose();
   Callback_ = 0;
   return Success;
}

/* Method: VxDVideoChannel::SetFormat
 * Purpose:
 * Input:
 * Output:
 */
ErrorCode VxDVideoChannel::SetFormat( ColFmt aFormat )
{
   Digitizer_->SetPixelFormat( aFormat, OurStream_ );
   return Success;
}

/* Method: VxDVideoChannel::GetFormat
 * Purpose:
 * Input:
 * Output:
 */
ColFmt VxDVideoChannel::GetFormat()
{
   return Digitizer_->GetPixelFormat( OurStream_ );
}

/* Method: VxDVideoChannel::AddBuffer
 * Purpose: This function adds a buffer to a queue
 * Input: pNewBuffer: PVOID - pointer to a buffer to add
 * Output: None
 * Note: This function 'does not know' where the queue is located. It just uses
 *   a pointer to it.
 */
void VxDVideoChannel::AddBuffer( LPVIDEOHDR lpvh )
{
   Digitizer_->ProcessAddBuffer( lpvh );
}

/* Method: VxDVideoChannel::Interrupt
 * Purpose: Called by the interface class on behalf of capture chip to let know
 *   an interrupt happened. Calls a callback function to let ring3 code know too
 * Input: None
 * Output: None
 */
void VxDVideoChannel::Interrupt( NotifyArg &na )
{
}

void PreviewVxDVideoChannel::Interrupt( NotifyArg &na )
{
   DebugPrint((0, "   PrevVxDVidChan::Interrupt (Notify)\n" ));

   VxDVideoChannel::Interrupt( na );

   if ( Callback_ ) {
      eaArray[na.idx_].pRefData_ = (PVOID)cbTag_ ; 
      eaArray[na.idx_].extra_ = na.currSkipCount_ ; 
      ClientCallback_ = new CallbackEvent( EventHandler, 
                                           Callback_, 
                                           &eaArray[na.idx_]) ;
      DebugPrint((0, "call cap drv (prev)\n" ));
      ClientCallback_->call();
   }

}


void CaptureVxDVideoChannel::Interrupt( NotifyArg &na )
{
// schedule an event into the capture driver
   DebugPrint((0, "   CapVxDVidChan::Interrupt (Notify)\n" ));

   VxDVideoChannel::Interrupt( na );

   if ( Callback_ ) {
      eaArray[na.idx_].pRefData_ = (PVOID)cbTag_ ; 
      eaArray[na.idx_].extra_ = na.currSkipCount_ ; 
      ClientCallback_ = new CallbackEvent( EventHandler, 
                                           Callback_, 
                                           &eaArray[na.idx_]) ;
      DebugPrint((0, "call cap drv\n" ));
      ClientCallback_->call();
   }
}

/* Method: VxDVideoChannel::Start
 * Purpose: Starts the stream
 * Input: None
 * Output: None
 */
DWORD VxDVideoChannel::Start(DWORD dwImageSize, StartType st)
{
   Started_ = true;

   if (st != START_SLAVE)
     Digitizer_->RestartFrameServer(dwImageSize) ;

   if ( Digitizer_->Create( OurStream_ ) == Success) {
     Digitizer_->Start( OurStream_ , st);
     return ~0 ;
   }
   
   return 0 ;
}

/* Method: VxDVideoChannel::Stop
 * Purpose: Stops the stream
 * Input: None
 * Output: None
 */
ErrorCode VxDVideoChannel::Stop()
{
   if ( !IsOpen() )
      return Fail;

   Digitizer_->PauseStream( OurStream_ );
   Digitizer_->Stop( OurStream_ );
   SetAppCallback( 0, 0 );

   Started_ = false;

   return Success;
}

/* Method: VideoChanIface::Notify
 * Purpose:  Notifies the VxDVideoChannel that an interrupt happened
 * Input: None
 * Output: None
 */
void VideoChanIface::Notify( NotifyArg &na )
{
   ToBeNotified_->Interrupt( na );
}


#ifndef TESTING
/* Method: CallbackEvent::handler
 * Purpose: This function calls into ring3 code at safe points in time
 */
#pragma argsused
void CallbackEvent::handler( VMHANDLE hVM, CLIENT_STRUCT* pRegs,
   PVOID refData, BOOL bTimeOut )
{
//   DebugBreak();
   EventCallee_( dwCB_, (EventArg *) refData );
}
#endif
//static bool bInside2 = false ;
void VxDVideoChannel::EventHandler( DWORD dwCB, EventArg * pEa )
{
//   if (bInside2 == false)
   {
//     bInside2 = true ;
     CLIENT_STRUCT StateBuf;
     Save_Client_State( &StateBuf );
     Begin_Nest_Exec();


     // put 'this' as an argument
     Simulate_Push( HIWORD( pEa->pRefData_ ) );
     Simulate_Push( LOWORD( pEa->pRefData_ ) );

     // push second argument (for capture channel this is the skip count)
     Simulate_Push( HIWORD( pEa->extra_ ) );
     Simulate_Push( LOWORD( pEa->extra_ ) );

     Simulate_Far_Call( HIWORD( dwCB ), LOWORD( dwCB ) );
     Resume_Exec();
     End_Nest_Exec();
     Restore_Client_State( &StateBuf );

//     bInside2 = false ;
   }
  // else
  //   DebugPrint((0,"Inside Eventhandler")) ;
}

/* Method: VBIVxDChannel::Interrupt
 * Purpose: Called by the interface class on behalf of capture chip to let know
 *   an interrupt happened. Calls a callback function to let ring3 code know too
 * Input: None
 * Output: None
 */
void VBIVxDChannel::Interrupt( NotifyArg &na )
{
   // let the user know buffer is done
   if ( cbTag_ )
      (*(PDWORD)cbTag_)++;

   if ( Callback_ ) {
      eaArray[na.idx_].pRefData_ = (PVOID)na.idx_ ; 
      eaArray[na.idx_].extra_ = 0 ; 
      ClientCallback_ = new CallbackEvent( EventHandler, (DWORD)(PVOID)this, &eaArray[na.idx_] );
      //DebugPrint((0,"VBI Int %x, %x, %x, %x, %x\n",ClientCallback_,Callback_ , EventHandler, (DWORD)(PVOID)this, (DWORD)pTag)) ;
      //DebugAssert((ClientCallback_ != NULL,"**Bad New of ClientCallBack_\n")) ;

      if (ClientCallback_)
         ClientCallback_->call();

   }
}

/* Method:
 * Purpose: Handles interrupt caused by VBI RISC program
 * Input: dwCB: DWORD, 'this' pointer
 *   pRefData: PVOID, in reality an int - program index
 * Output: None
 */
void VBIVxDChannel::EventHandler( DWORD dwCB, EventArg * pEa )
{
   VBIVxDChannel *me = (VBIVxDChannel *)dwCB;
   DWORD dwCurBufAddr = Digitizer_->GetDataBuffer( (int)pEa->pRefData_ );

   if ( me->Callback_ ) {
      me->Ring3Tag_ = dwCurBufAddr;
      VBIIntrCallback VBICallback = (VBIIntrCallback) me->Callback_;
//   DebugPrint((0,"VBI Callback %x, %x, %x\n",me->Callback_,me->Stream_, dwCurBufAddr)) ;
      VBICallback( me->Stream_, dwCurBufAddr );
   }
}

/* Method: VBIVxDChannel::Start
 * Purpose: Starts the stream
 * Input: None
 * Output: None
 */
DWORD VBIVxDChannel::Start(DWORD dwImageSize, StartType st)
{
   SetFormat( CF_VBI );
   return VxDVideoChannel::Start(dwImageSize, st);
}

/* Method: VBIVxDChannel::AddBuffer
 * Purpose: This function adds a buffer to a queue
 * Input: pNewBuffer: PVOID - pointer to a buffer to add
 * Output: None
 */
void VBIVxDChannel::AddBuffer( LPVIDEOHDR lpvh )
{
//   GetCurrentQue().Put( lpvh );
   DebugPrint((1,"Adding to Vbi Queue = %x\n",lpvh));
   Digitizer_->GetR3VbiQueue().Add(lpvh) ;
}

/* Method: VBIVxDChannel::SetRing3Callback
 * Purpose: Stores the address of a ring3 callback and an associated tag
 * Input: func: DWORD, address of a 16 bit callback
 *   tag: DWORD, tag
 */
void VBIVxDChannel::SetRing3Callback( DWORD func, DWORD tag )
{
   Ring3CB_ = func;
   Ring3Tag_ = tag;
}

/* Method: VBIVxDChannel::CallRing3Callback
 * Purpose: Invokes the ring3 callback
 */
void  VBIVxDChannel::CallRing3Callback()
{
   // do not keep event hanging after stop
   if ( !Started_ ) {
      DEBUGTRACE( "not started !\n" );
      return;
   }
   VxDVideoChannel::EventHandler( Ring3CB_, new EventArg( (PVOID) Ring3Tag_,0 ) );
}


