/* File: Vxdvidch.h
 * Purpose: Contains definition of the VxDVideoChannel class
 * Author: sasha
 * $Log: VXDVIDCH.H $
 * Revision 2.10  1998/05/14 23:42:09  dtharp
 * Graft from DT_VBI branch: VBI now functional.
 * Revision 2.9.1.1  1998/05/11 17:28:16  dtharp
 * Branch to restore VBI
 * Revision 2.9  1998/04/06 19:42:18  dtharp
 * Vxd Frame Buffers are allocated on the fly instead of at init time.
 * Revision 2.8  1998/04/05 20:28:37  dtharp
 * RestartFrameServer() no longer takes a parameter.
 * Revision 2.7  1998/04/02 21:50:22  dtharp
 * Merge DT_VIDEOHDR branch. Implements a new buffer 
 * management scheme.
 * Revision 2.6.2.3  1998/04/01 21:19:53  dtharp
 * Incremental Checkin
 * Revision 2.6.2.2  1998/03/27 15:30:31  dtharp
 * Incremental check-in
 * Revision 2.6.2.1  1998/03/18 19:35:18  dtharp
 * Branch for passing videoheaders around
 * Revision 2.6  1997/08/08 11:16:46  dtharp
 * Fix for bug 714: NotifyArg now passed by reference.
 * Revision 2.5  1997/06/27 16:16:35  dtharp
 * Modifications to pass drop count to ring 3.
 * Revision 2.4  1997/05/05 17:12:42  dtharp
 * 848A Merge: from 2.2.1.3
 * Revision 2.3  1997/03/20 22:10:10  dtharp
 * Grafted WDM Buffer managment branch.
 * Revision 2.2.2.2  1997/03/17 16:55:31  dtharp
 * Works @ 30fps NTSC.
 * Revision 2.2.2.1  1997/03/12 18:26:40  dtharp
 * Single field capture kinda works.
 * Revision 2.2  1996/09/23 00:48:52  sasha
 * Reintroduced Interrupt() function into preview and capture channels.
 * This is needed to control when the ring3 capture driver is notified that
 * a buffer is available. All this is needed for the 640x480 AVI capture.
 * Revision 2.1  1996/06/10 20:12:44  ssm
 * MarkBeta1AsRev2
 * Revision 1.15  1996/05/14 23:47:54  sasha
 * Created VBIIntrCallback type. Added support for setting and calling ring3
 * callback from VBI channel
 * Revision 1.14  1996/04/26 21:09:56  sasha
 * Added EventHandler() to the VBI class
 * Revision 1.13  1996/03/13 20:12:06  sasha
 * No changes
 * Revision 1.12  1996/03/04 21:46:58  sasha
 * Commented out Interrupt() function in preview and capture channels
 * Revision 1.11  1996/02/27 02:10:10  sasha
 * Added AddBuffer to the VBIVxDChannel
 * Revision 1.10  1996/02/19 01:27:14  sasha
 * Added VBIVxDChannel class. Chnaged how clipping is set. Create buffer
 * quque of size 1000 ( max possible under VFW ). Removed some redundant
 * functions from derived classes
 * Revision 1.9  1996/01/30 19:23:18  sasha
 * Moved CallbackEvent class into pisces.h. Added new member function to
 * the VxDVideoChannel to be called from the event callback.
 * Revision 1.8  1996/01/15 20:19:23  sasha
 * Added GetDigitizer() function
 * Revision 1.7  1996/01/06 02:06:08  sasha
 * Changed ColorFormat to ColFmt
 * Revision 1.6  1996/01/04 23:10:56  sasha
 * Changed GetCurrentQue() to get it from Digitizer ( from field )
 * Revision 1.5  1996/01/02 18:39:15  sasha
 * Changed BufQue to be actual object instead of a pointer ( effectively moved
 * it from field class )
 * Revision 1.4  1995/12/08 22:19:54  sasha
 * Added ring3 notification code ( needs more work )
 * Added SetClipping to the Preview channel
 * Revision 1.3  1995/11/07 00:53:17  sasha
 * Added CaptureVxDVideoChannel and PreviewVxDVideoChannel classes
 * Added SetAppCallback function to the VxDVideoChannel.
 * Revision 1.2  1995/10/16 06:06:40  sasha
 * First major check-in for testing
 * Revision 1.1  1995/10/16 06:06:40  sasha
 * Initial revision
 */

#ifndef __VXDVIDCH_H
#define __VXDVIDCH_H

#if !defined __vtoolscp_h && !defined TESTING
#include <vtoolscp.h>
#endif

#undef FAR

#ifndef __WINDOWS_H
#include <windows.h>
#endif

#ifndef __MYTYPES_H
#include "mytypes.h"
#endif

#ifndef __VXDFUNC_H
#include "vxdfunc.h"
#endif

#ifndef __PISCESP_H
#include "piscesp.h"
#endif

#ifndef __VIDCHIFC_H
#include "vidchifc.h"
#endif

/* Typedef :  VBIIntrCallback
 * Purpose: Defines a function to be used as a VBI interrupt processor
*/
typedef  void (*VBIIntrCallback)( VideoStream, DWORD );

/* Class: VxDVideoChannel
 * Purpose: The base class to be used in the BtPisces capture VxD. Used for
 *   processing user requests ( comes from the device class, goes to capture
 *   chip class )
 * Attributes:
 * Methods
 */
class VxDVideoChannel
{
   protected:

      static BtPiscesPlus *Digitizer_;

      VideoStream    Stream_;

//      VidBufQueue    BufQue_;

      VideoChanIface Caller_;

      StreamInfo     OurStream_;

      bool           Open_;
      bool           Started_;
      bool           NeedInterrupt_;
      DWORD          Callback_;
      DWORD          cbTag_;

#ifndef TESTING
      CallbackEvent  *ClientCallback_;
#endif
      // from pisces.h (kludge)
      Field & GetField( StreamInfo &str ) { return *(Field*)str.tag; }

      static   void  EventHandler( DWORD dwCB, EventArg * ea );
      virtual  void  Interrupt( NotifyArg &na );

   public:
      virtual ErrorCode OpenChannel();
      virtual ErrorCode CloseChannel();
      virtual ErrorCode SetFormat( ColFmt );
              ColFmt    GetFormat();
      virtual ErrorCode SetDigitalWindow( TRect &r );
      virtual ErrorCode SetAnalogWindow( TRect &r );
      virtual DWORD     Start(DWORD dwImageSize, StartType st=START_SINGLE);
      virtual ErrorCode Stop();

      virtual  void AddBuffer( LPVIDEOHDR lpvh );

               void SetAppCallback( DWORD cb, DWORD tag )
               { Callback_ = cb; cbTag_ = tag; }

               void SetBufPitch( DWORD dwP )
               { Digitizer_->SetBufPitch( dwP, OurStream_ ); }

               void SetOffset( DWORD dwO )
               { Digitizer_->SetOffset( dwO, OurStream_ ); }

               // used to set an alternative buffer queue for full size capture
//               void  SetBufQue( VidBufQueue *aQue )
//               { Digitizer_->SetBufQuePtr( OurStream_, aQue ); }
//
//               void  SetDefaultQue()
//               { Digitizer_->SetBufQuePtr( OurStream_, &BufQue_ ); }
//
//               VidBufQueue &GetCurrentQue() { return Digitizer_->GetCurrentQue( OurStream_ ); }

               void SetPaired( bool p = false );
               void SetDoPause( bool p = false );

               void SetClipping( RGNDATA &clip )
               { OurStream_.SetClipping( &clip ); }

               RGNDATA *const GetClipping() { return (RGNDATA *)OurStream_.GetClipping(); }
               bool IsOpen()   { return Open_;  }
               void SetOpen()  { Open_ = true;  }
               void SetClose() { Open_ = false; }

               void EnableInterrupt()
               { OurStream_.Interrupt_ = NeedInterrupt_ = true; }

               void DisableInterrupt()
               { OurStream_.Interrupt_ = NeedInterrupt_ = false; }


      static void Init( BtPiscesPlus *const pCapChip ) { Digitizer_ = pCapChip; }
      static bool IsInited() { return bool( Digitizer_ != NULL ); }
      static BtPiscesPlus &GetDigitizer() { return *Digitizer_; }


      VxDVideoChannel( VideoStream aStrm, bool Intr = true ) : Stream_( aStrm ),
         NeedInterrupt_( Intr ), /*BufQue_( 1000 ),*/ Caller_( this ), OurStream_(),
         Open_( false ), Callback_( 0 ), cbTag_( 0 ), Started_( false )
#ifndef TESTING
         , ClientCallback_( NULL )
#endif
          {}

      virtual ~VxDVideoChannel() { if ( IsOpen() ) CloseChannel(); }

      friend class VideoChanIface;
};


inline void VxDVideoChannel::SetPaired( bool p )
{
//   OurField_->SetPaired( p );  orig wdm
   OurStream_.SetPaired( p );
}
inline void VxDVideoChannel::SetDoPause( bool p )
{
//   OurField_->SetPaired( p );  orig wdm
   OurStream_.SetDoPause( p );
}

/* Class: PreviewVxDVideoChannel
 * Purpose: Specializes the base class to implement the video preview
 *   functionality
 * Attributes:
 * Methods:
 */
class PreviewVxDVideoChannel : public VxDVideoChannel
{
   protected:
      virtual void  Interrupt( NotifyArg &na );

   public:

//      virtual void      AddBuffer( LPVIDEOHDR lpvh );
              ErrorCode SetClipping( PVOID pData );

      PreviewVxDVideoChannel( VideoStream aStrm, bool Intr = false ) :
         VxDVideoChannel( aStrm, Intr ) {}
};

/* Class: CaptureVxDVideoChannel
 * Purpose: Specializes the base class to implement the video capture
 *   functionality
 * Attributes:
 * Methods
 */
class CaptureVxDVideoChannel : public VxDVideoChannel
{
   protected:
      virtual void  Interrupt( NotifyArg &na );

   public:

      CaptureVxDVideoChannel( VideoStream aStream ) : VxDVideoChannel( aStream ) {}
};

/* Class: VBIVxDChannel
 * Purpose: Channel to capture VBI data
 */
class VBIVxDChannel : public VxDVideoChannel
{
private:
   DWORD   Ring3CB_;
   DWORD   Ring3Tag_;
protected:
   virtual void  Interrupt( NotifyArg &na );
   static  void  EventHandler( DWORD dwCB, EventArg * ea );

public:
   VBIVxDChannel( VideoStream aStream ) : VxDVideoChannel( aStream ),
   Ring3CB_( 0 ), Ring3Tag_( 0 ) {}

   virtual  void AddBuffer( LPVIDEOHDR lpvh );

   void  SetRing3Callback( DWORD func, DWORD tag );

   void  CallRing3Callback();
   
   virtual DWORD     Start(DWORD dwImageSize, StartType st=START_SINGLE);
};

#endif __VXDVIDCH_H
