/* File: DevIOCtl.cpp
 * Purpose: Contains DeviceIOControl handler
 * Author: sasha
 * $Log: DevIOCtl.cpp $
 * Revision 2.12  1998/05/14 23:21:50  dtharp
 * Added support to dump Starter program in FNDumpRisc
 * Revision 2.11  1998/04/03 18:34:32  dtharp
 * General clean up
 * Revision 2.10  1998/04/02 21:50:49  dtharp
 * Merge DT_VIDEOHDR branch. Implements a new buffer 
 * management scheme.
 * Revision 2.9.1.3  1998/04/01 21:19:55  dtharp
 * Incremental Checkin
 * Revision 2.9.1.2  1998/03/27 16:33:35  dtharp
 * Commit to new prog array and no skipper array.
 * Revision 2.9.1.1  1998/03/27 15:29:12  dtharp
 * Branch for passing videohdrs around.
 * Revision 2.9  1998/02/24 17:42:38  dtharp
 * Graft from 2.8.2.2
 * Revision 2.8.2.2  1998/02/10 03:38:41  dtharp
 * Added debug interface for VVl stuff. Currently #if0'd out.
 * Revision 2.8  1997/09/16 16:41:41  dtharp
 * Merged AVI processing branch.
 * Revision 2.7.1.1  1997/08/19 16:00:23  dtharp
 * Branch for gemini Avi post processing
 * Revision 2.7  1997/07/22 15:36:18  dtharp
 * Added FNGetPciHdr functionality via DevIOCtl and VxD Service.
 * Revision 2.6  1997/05/02 17:28:15  dtharp
 * Added support for interrupt count instrumentation.
 * Revision 2.5  1997/04/25 16:45:08  dtharp
 * Grafted WST interface branch (2.4.1.1)
 * Revision 2.4.1.1  1997/04/03 17:07:59  dtharp
 * WST interface. Added VBI_Change_Processor() call.
 * Revision 2.4  1997/03/26 00:26:21  dtharp
 * Removed Pause and Continue (weren't needed anymore 
 * for riscprog dump). Dumper now has capability to dump
 * Skippers_ array.
 * Revision 2.3  1997/03/20 22:11:34  dtharp
 * Grafted WDM Buffer managment branch.
 * Revision 2.2.3.1  1997/03/17 16:45:34  dtharp
 * Added support for dumping a riscprog. (still needs polishing)
 * Revision 2.2  1996/09/29 23:16:41  sasha
 * Added code to support 32 bit interrupt notifications. 
 * Revision 2.1  1996/06/10 20:12:46  ssm
 * MarkBeta1AsRev2
 * Revision 1.1  1996/05/11 23:46:25  sasha
 * Initial revision
 */

#include "device.h"
#include "dbgprint.h"
extern "C" {
#include "vpaec.h"
}
#include "bt848vxd.h"

InterruptCounter gIntrCntr ;
BufferInstrumentation gBufInstrumentation ;

bool PsDevice::ChangingChannel_ = false;

/* Method: PsDevice::OnW32DeviceIoControl
 * Purpose: Handles DeviceIOControl calls
 */
DWORD PsDevice::OnW32DeviceIoControl( PIOCTLPARAMS pDIOCParams )
{
   DWORD dwData = *( (PDWORD)pDIOCParams->dioc_InBuf );

   switch ( pDIOCParams->dioc_IOCtlCode ) {                   // select on control code
   case DIOC_CLOSEHANDLE:
      return 0;
   case DIOC_GETVERSION:
      return 0;
   case 1: {
      PDWORD pRetAddr = (PDWORD)pDIOCParams->dioc_OutBuf;
      DWORD  PhysAddr = *( (PDWORD)pDIOCParams->dioc_InBuf );

      if ( pDIOCParams->dioc_cbOutBuf < 4 )
         return -1;          // invalid parameters

       *pRetAddr = (DWORD)MapPhysToLinear( (const void *)PhysAddr, 0xFFF, 0 );
      if ( pDIOCParams->dioc_bytesret )
         *pDIOCParams->dioc_bytesret = 4;
       break;
    }
   case 2:
   case FNGetBaseAddr: {
      PDWORD pRetAddr = (PDWORD)pDIOCParams->dioc_OutBuf;
      if ( pDIOCParams->dioc_cbOutBuf < 4 )
         return -1;          // invalid parameters

      *pRetAddr = (DWORD)RegBase::GetBaseAddress();
      if ( pDIOCParams->dioc_bytesret )
         *pDIOCParams->dioc_bytesret = 4;
      break;
    }
   case FNSetChan:
      VBISetChannel( dwData );
      break;
   case FNPreSetChan:
      ChangingChannel_ = true;
      break;
   case FNIntStatusStart: {
      if ( pDIOCParams->dioc_cbOutBuf < 4 )
         return -1;          // invalid parameters

      BtPisces::StartIntNotification( *(HANDLE*)pDIOCParams->dioc_InBuf );

      PDWORD *ppRetAddr = (PDWORD *)pDIOCParams->dioc_OutBuf;
      *ppRetAddr = BtPisces::GetIntStatLocation();

      if ( pDIOCParams->dioc_bytesret )
         *pDIOCParams->dioc_bytesret = 4;
      break;
    }
   case FNIntStatusStop:
      {
         BtPisces::StopIntNotification();
         break;
      }
   case FNDecoder:
      {
         VxDParam *param = (VxDParam *)pDIOCParams->dioc_InBuf;
         dwData = param->param;

         switch ( param->subFunc ) {
         case DecSat:
            if ( param->funcType == SetFunc )
               SetSaturation( VideoIn_, &dwData );
            else
               GetSaturation( &param->param );
            break;
         case DecHue:
            if ( param->funcType == SetFunc )
               SetHue( VideoIn_, &dwData );
            else
               GetHue( &param->param );
            break;
         case DecBrt:
            if ( param->funcType == SetFunc )
               SetBrightness( VideoIn_, &dwData );
            else
               GetBrightness( &param->param );
            break;
         case DecSVid:
            if ( param->funcType == SetFunc )
               SetSVideo( VideoIn_, &dwData );
            else
               GetSVideo( &param->param );
            break;
         case DecContr:
            if ( param->funcType == SetFunc )
               SetContrast( VideoIn_, &dwData );
            else
               GetContrast( &param->param );
            break;
         case DecFmt:
            if ( param->funcType == SetFunc )
            {
               SetFormat( VideoIn_, &dwData );
               VBI_Change_Processor( VIDEO_FORMAT_CHANGE, dwData ) ;
            }
            else
               GetFormat( &param->param );
            break;
         case DecConn:
            if ( param->funcType == SetFunc )
               SetConnector( VideoIn_, &dwData );
            else
               GetConnector( &param->param );
            break;
         }
         break;
      }

      case FNDumpRiscProg:
      {
      // Input 
      //    DWORD : RiscProg ndx
      //       CreatedProgs : 12 elements     ndx 0..11
      //       ActiveProgs  : 12 elements     ndx 12..23
      //       Starter      : 1  elements     ndx 30
      // Output
      //    Buffer filled with riscprog
        int i = 0;

        if ( !pDIOCParams->dioc_cbOutBuf || (pDIOCParams->dioc_cbInBuf != 4))
          return -1;          // invalid parameters

        // pause
        // CaptureContrll_->Pause() ;

        // dump a prog
        DWORD WhichProg = *((PDWORD) pDIOCParams->dioc_InBuf);

        RiscPrgHandle hProg = NULL;

        if (WhichProg < 12)            // CreatedProgs
        {
          hProg = CaptureContrll_->CreatedProgs_[WhichProg] ;
        }
        else if (WhichProg < 24)       // Active
        {
          hProg = CaptureContrll_->ActiveProgs_[WhichProg % 12] ;
        }
        else if (WhichProg == 30)
        {
          hProg = &CaptureContrll_->Starter_ ;
        }

        if(hProg)
        {
          char * pRetAddr = (char *)pDIOCParams->dioc_OutBuf;
          DWORD physAddr = hProg->GetPhysProgAddr() ;
          DWORD progSize = hProg->GetProgramSize();
          char * linBuf = (char *) MapPhysToLinear((void *)physAddr, progSize, 0) ;

          *((DWORD *) pRetAddr) = physAddr ;
          pRetAddr+=4 ;
          for (i = 0 ; i < progSize && i < pDIOCParams->dioc_cbOutBuf ; i++)
          {
             *pRetAddr++ = linBuf[i] ;
          }
        }
        if ( pDIOCParams->dioc_bytesret )
          *pDIOCParams->dioc_bytesret = i;

        // and resume
        // CaptureContrll_->Continue() ;
      }
        break;

      case FNGetIntrCounter:
      {
        PDWORD pRetAddr = (PDWORD)pDIOCParams->dioc_OutBuf;
        if ( pDIOCParams->dioc_cbOutBuf < 4 )
           return -1;          // invalid parameters

        *pRetAddr = (DWORD) &gIntrCntr;
        if ( pDIOCParams->dioc_bytesret )
           *pDIOCParams->dioc_bytesret = 4;
      }
      break ;

      case FNGetPciHdr:
      {
        PDWORD pRetAddr = (PDWORD)pDIOCParams->dioc_OutBuf;
        if ( pDIOCParams->dioc_cbOutBuf < 4 )
           return -1;          // invalid parameters

        *pRetAddr = (DWORD) &pciCfgHdr_;
        if ( pDIOCParams->dioc_bytesret )
           *pDIOCParams->dioc_bytesret = 4;
      }
      break ;

      case FNRegCapEvent :
      {
        //    in      : 32 bit Ring 0 Event
        //    returns : pointer to SOUND_SYNC structure
        extern HANDLE gCapEvent ;
        extern SOUND_SYNC gSoundSync ;

        // parameter check
        if ( pDIOCParams->dioc_cbOutBuf < 4 ) {
          DebugPrint((0,"FNRegCapEvent : BAD PARAMS\n")) ;
          return -1;          // invalid parameters
        }

        // Process input
        gCapEvent = *(HANDLE*)pDIOCParams->dioc_InBuf ;
        DebugPrint((0,"FNRegCapEvent : gCapEvent %x\n",gCapEvent)) ;

        // Set up return
        PDWORD pRetAddr = (PDWORD)pDIOCParams->dioc_OutBuf;
        *pRetAddr = (DWORD) &gSoundSync;

        DebugPrint((0,"FNRegCapEvent : returned %x\n",&gSoundSync)) ;

        if ( pDIOCParams->dioc_bytesret )
           *pDIOCParams->dioc_bytesret = 4;
      }
      break ;

      case FNGetBufferInstrumentation:
      {
        DebugPrint((1,"FNGetBufferInstrumentation\n")) ;
        PDWORD pRetAddr = (PDWORD)pDIOCParams->dioc_OutBuf;
        if ( pDIOCParams->dioc_cbOutBuf < 4 )
           return -1;          // invalid parameters

        *pRetAddr = (DWORD) &gBufInstrumentation;
        if ( pDIOCParams->dioc_bytesret )
           *pDIOCParams->dioc_bytesret = 4;
      }
      break ;
#if 0
// The following code is left here (commented out) in case 
// we need to debug the VVL Interface ever again (not anticipated).
      //
      // VVL Debug follows
      //
      case FNVPAEC_Get_Version :
      {
        // DWORD _cdecl VPAEC_Get_Version()
        DebugPrint((0,"+++ Calling VPAEC_Get_Version\n")) ;
        DWORD ver = VPAEC_Get_Version() ;

        PDWORD pRetAddr = (PDWORD)pDIOCParams->dioc_OutBuf;
        if ( pDIOCParams->dioc_cbOutBuf < 4 )
           return -1;          // invalid parameters

        *pRetAddr = ver ;
        if ( pDIOCParams->dioc_bytesret )
           *pDIOCParams->dioc_bytesret = 4;
      }
      break ;

      case FNVPAEC_Init :
      {
        // void _cdecl VPAEC_Init( BOOL doAEC , BOOL doACB , WORD VxDID , WORD ReadServiceID , WORD WriteServiceID )
        WORD * pArg = (WORD *) pDIOCParams->dioc_InBuf ;

        // low byte is AEC, hi byte = ACB
        BOOL doAEC = *pArg & 0xff ;
        BOOL doACB = (*pArg & 0xff00) >> 8 ;

        DebugPrint((0,"+++ Calling FNVPAEC_Init : doAEC= %d  doACB = %d\n",doAEC,doACB)) ;
        VPAEC_Init( doAEC,                   //  doAEC  
                  doACB,                   //  doACB  
                  PISCES_DEVICE_ID,       //  VxDID  
                  __BT848_VVL_Read_Byte,  //  ReadServiceID 
                  __BT848_VVL_Write_Byte  //  WriteServiceID 
                  ) ;
      }
      break ;

      case FNVPAEC_Iterate :
      {
        // void _cdecl VPAEC_Iterate( void )
        DebugPrint((0,"+++ Calling VPAEC_Iterate\n")) ;
        VPAEC_Iterate() ;
      }
      break ;

      case FNVPAEC_Terminate :
      {
        // void _cdecl VPAEC_Terminate( void )

        DebugPrint((0,"+++ Calling VPAEC_Terminate\n")) ;
        VPAEC_Terminate() ;
      }
      break ;

      case FNVPAEC_SetParam :
      {
        // BOOL _cdecl VPAEC_SetParam( T_VP_PARAM param , BYTE value )
        DebugPrint((0,"+++ Calling VPAEC_SetParam\n")) ;
      }
      break ;

      case FNVPAEC_GetParam :
      {
        // BOOL _cdecl VPAEC_GetParam( T_VP_PARAM param , BYTE *value )
        DebugPrint((0,"+++ Calling VPAEC_GetParam\n")) ;

      }
      break ;

      case FNVPAEC_DebugString :
      {
        // void _cdecl VPAEC_DebugString( char *str )
        DebugPrint((0,"+++ Calling FNVPAEC_DebugString\n")) ;

        // Process input
        LPSTR dbg = (LPSTR) pDIOCParams->dioc_InBuf ;
        VPAEC_DebugString( dbg ) ;
      }
      break ;
#endif

   } /* endswitch */

   return 0;
}


