/* File: Device.cpp
 * Purpose: main module for VxD PISCES
 * Author: sasha
 *$Log: device.cpp $
 *Revision 2.68  1998/06/01 18:08:52  mdimeo
 *added rcs ident string
 *Revision 2.67  1998/05/30 19:28:05  dtharp
 *Added fetch of VVL i2c Values from registry.
 *Revision 2.66  1998/05/19 18:48:31  dtharp
 *Removed inappropriate SetPlanarAdjust() calls in StartQuartzSight().
 *Revision 2.65  1998/05/19 17:44:23  dtharp
 *Fixed sequencing of SetPlanarAdjust() call in StartInterlaced().
 *Revision 2.64  1998/05/14 23:42:55  dtharp
 *Merge from DT_VBI branch: VBI now functional.
 *Revision 2.62.1.3  1998/05/14 23:27:50  dtharp
 *Changes to queue access function names
 *Revision 2.62.1.2  1998/05/13 16:55:05  dtharp
 *Merged QS overlay cap graceful degradation.
 *Revision 2.63  1998/05/12 22:56:03  dtharp
 *Changes needed to implement capture from overlay in QuartzSight mode.
 *Revision 2.62.2.3  1998/05/12 22:55:13  dtharp
 *Document and clean up
 *Revision 2.62.2.2  1998/05/12 00:58:19  dtharp
 *First checkpoint: "graceful degredation" mode QS capture from overlay.
 *Revision 2.62  1998/04/22 16:07:59  dtharp
 *Added support for FNOpenDriver and FNCloseDriver. Also disabled
 *handler for FNCCClip (will reinstate after Camera only release).
 *Revision 2.61  1998/04/21 17:07:04  dtharp
 *Fix for Quartzsight Clipping .
 *Revision 2.60  1998/04/17 22:55:43  dtharp
 *Initialized HWInt pointer to NULL
 *Revision 2.59  1998/04/17 18:44:07  dtharp
 *Protected against adding a NULL videoheader pointer to buffer queue.
 *Revision 2.58  1998/04/16 04:04:26  dtharp
 *Removed invalid debugprint
 *Revision 2.57  1998/04/16 00:12:17  dtharp
 *Added Handling for FNGetR3BufCount.
 *Revision 2.56  1998/04/06 20:25:51  dtharp
 *Reset default debug level to 0
 *Revision 2.55  1998/04/06 19:42:19  dtharp
 *Vxd Frame Buffers are allocated on the fly instead of at init time.
 *Revision 2.54  1998/04/05 20:28:39  dtharp
 *RestartFrameServer() no longer takes a parameter.
 *Revision 2.53  1998/04/04 03:34:43  rudyl
 *Merged 2.52 & 2.50.1.2.
 *Revision 2.52  1998/04/03 18:34:57  dtharp
 *Added call to FlushR3Queue
 *Revision 2.51  1998/04/03 06:13:56  dtharp
 *Merge DT_VIDEOHDR branch. Implements a new buffer 
 *management scheme.
 *Revision 2.50  1998/03/28 01:44:07  rudyl
 *Graft from 2.49.2.2.
 *Revision 2.49.2.2  1998/03/28 01:43:31  rudyl
 *Removed Scotch init codes from stream start to scotch.cpp when sys boots up.
 *Removed scotch flush after every write.
 *Revision 2.49.2.1  1998/03/13 00:47:04  rudyl
 *Duplicate revision
 *Revision 2.49.1.6  1998/04/02 21:28:00  dtharp
 *Cleanup before merge.
 *Revision 2.49.1.5  1998/04/02 18:07:36  dtharp
 *Fixed full screen overlay
 *Revision 2.49.1.4  1998/04/01 21:19:57  dtharp
 *Incremental Checkin
 *Revision 2.49.1.3  1998/03/27 16:33:32  dtharp
 *Commit to new prog array and no skipper array.
 *Revision 2.49.1.2  1998/03/27 15:30:34  dtharp
 *Incremental check-in
 *Revision 2.49.1.1  1998/03/18 19:35:36  dtharp
 *Branch for passing videoheaders around
 *Revision 2.49  1998/03/13 00:47:04  dtharp
 *Fixed temporal capture problem in QuartzSight Mode.
 *Revision 2.48  1998/03/13 00:10:37  dtharp
 *Renamed variable TrueViewMode to QuartzSightMode
 *Revision 2.47  1998/03/12 00:55:36  dtharp
 *Merged HP8180 shutdown fix.
 *Revision 2.46  1998/03/10 00:57:12  dtharp
 *Protected call of HWInt.unhook(), since HWInt could be NULL at this point.
 *Revision 2.45  1998/03/03 21:11:55  dtharp
 *Bug Id 654:  Fixed QuartzSight split images.
 *Revision 2.44  1998/02/27 18:24:43  dtharp
 *Capture controller constructor now takes devId as param. AEC algorithm
 *is no longer set to defaults on stream start.
 *Revision 2.43  1998/02/24 21:23:27  rudyl
 *Merged 2.41.1.2 & 2.42.
 *Revision 2.42  1998/02/24 17:57:08  dtharp
 *Merge VVL AEC Code
 *Revision 2.40.1.4  1998/02/13 00:39:55  dtharp
 *Re-worked VPAEC Init() and iterate() call conditions
 *Revision 2.40.1.3  1998/02/10 07:19:06  dtharp
 *Cleaned up some compiler warnings
 *Revision 2.40.1.2  1998/02/10 03:37:19  dtharp
 *Added VVl Read and Write functions. Also added vxd debug interface.
 *Revision 2.41  1998/01/09 17:11:09  peter
 *Graft Trident fixes to tip.
 *Revision 2.40  1997/12/17 00:20:28  dtharp
 *Fix for Scotch preview and capture problem.
 *Revision 2.39  1997/12/11 02:09:29  dtharp
 *Bug id 923. Fix for scotch in primary surface mode clipping 1/2 vertical.
 *Revision 2.38  1997/11/26 22:34:28  rudyl
 *Removed _TrueView reg key since QuartzSight camera is supported now.
 *Revision 2.37  1997/11/03 05:35:14  dtharp
 *Moved _TrueView reg key to PRK_LOCAL_MACHINE\SOFTWARE\Brooktree\Bt848.
 *Revision 2.36  1997/10/31 16:27:48  dtharp
 *Fixed trueview quartzsight video jumping in 1/2 and 1/4 size. Also
 *added a temp reg key for enabling trueview mode.
 *Revision 2.35  1997/10/29 18:35:52  dtharp
 *Merged TrueView branch.
 *Revision 2.34.1.2  1997/10/29 00:12:17  dtharp
 *Removed #ifdefs,  found and fixed non-trueview mode bug.
 *Revision 2.34.1.1  1997/10/28 16:07:03  dtharp
 *TrueView Branch
 *Revision 2.34  1997/10/22 23:43:23  dtharp
 *FNStartCapNotify now loads the window handle into the struct
 *Revision 2.33  1997/10/21 00:38:59  rudyl
 *Added support to expected total line count by decoder.
 *Revision 2.32  1997/09/16 16:52:54  dtharp
 *Merged AVI processing branch.
 *Revision 2.31  1997/09/04 22:45:21  dtharp
 *Merged VSYNC Callback changes
 *Revision 2.30.3.1  1997/08/21 01:38:26  dtharp
 *Branch for vsync callback
 *Revision 2.30  1997/07/22 15:36:16  dtharp
 *Added FNGetPciHdr functionality via DevIOCtl and VxD Service.
 *Revision 2.29  1997/07/21 23:21:53  dtharp
 *Fix for Bug ID 675: Suspend/Resume IRQ problems.
 *Revision 2.28  1997/06/27 16:23:20  dtharp
 *Set paired for videoin channel.
 *Revision 2.27  1997/06/17 22:18:19  dtharp
 *Modification for Intel Intercast VBI. Also added SELF_DESTRUCT
 *capability.
 *Revision 2.26  1997/06/01 17:08:00  dtharp
 *Improved memory allocation during CC Clips.
 *Revision 2.25  1997/05/16 20:07:41  dtharp
 *Fix for Bug ID 555: Video displayed outside of app. 
 *Revision 2.24  1997/05/14 17:29:44  dtharp
 *Cast some  bools to avoid compiler warnings.
 *Revision 2.23  1997/05/05 17:06:58  dtharp
 *848A Merge: from 2.16.3.3
 *Revision 2.22  1997/04/28 21:11:48  dtharp
 *Merged CC Primary branch from 2.20.2.5. Bug ID 475
 *Revision 2.21  1997/04/25 16:47:09  dtharp
 *Grafted WST Interface branch (2.20.1.1)
 *Revision 2.20.1.1  1997/04/03 17:05:50  dtharp
 *WST Interface. Added VBI_Change_Processor() call.
 *Revision 2.20  1997/04/02 00:19:43  dtharp
 *Grafted ReadConfig branch (driver reads and stores DevId and 
 *Revision info from PCI config space).
 *Revision 2.19.1.1  1997/03/31 18:25:58  dtharp
 *Added static vars wDevId_ & byRev_.
 *Revision 2.19  1997/03/20 22:19:16  dtharp
 *Merged WDM buffer management branch.
 *Revision 2.18  1997/03/18 19:35:16  rudyl
 *Now PAL uses 288 lines per field.
 *Revision 2.17.3.2  1997/03/20 18:15:07  dtharp
 *Added dwVxdSkipCount.
 *Revision 2.17.3.1  1997/03/17 16:50:34  dtharp
 *Works @ 30fps NTSC.
 *Revision 2.17.2.2  1997/03/08 02:47:25  rudyl
 *Now PAL uses 288 lines per field.
 *Revision 2.17.2.1  1997/03/06 20:04:54  rudyl
 *Use 288 lines for PAL/SECAM
 *Revision 2.17  1996/11/08 18:54:48  meierdw
 *Grafting 2.16.1 back into trunk; NTSC-Japan support.
 *Revision 2.16.1.2  1996/11/08 18:54:48  rudyl
 *Added NTSC-Japan, PAL-M & PAL-N formats.
 *Revision 2.16.1.1  1996/11/07 23:43:27  rudyl
 *Start work to add NTSC-Japan, PAL-M & PAL-N formats.
 *Revision 2.16  1996/10/25 00:35:00  sasha
 *Removed SetRate() member function ( not needed ).
 *Revision 2.15  1996/10/22 18:16:51  dtharp
 *Bug # 358: GPF when attempting to capture video in:  
 *640 x 480, Overlay (primary surface),
 *VXD API FNStop now sets pSkipCount (address registered
 *by the capture driver during input channel stream start)  to a 
 *safe dummy location.
 *Revision 2.14  1996/10/02 05:21:40  sasha
 *Changed PISCES to BT848 in the Declare_Virtual_Device.
 * Added APIs to set ring 0 I2C and GPIO callbacks.
 *Revision 2.13  1996/09/27 00:08:37  sasha
 *Disable interrupts in the AddBuffer(). Removed global bool stopped.
 *Bug ID# 254. In the Start() call RISCProgram::SetRate().
 *Revision 2.12  1996/09/23 00:13:32  sasha
 *Modified AddBuffer(). SetPlanarAdjust() is a member of BtPisces now
 *instead of RISCProgram. All this is done to support 640x480 planar
 *AVI capture.
 *Revision 2.11  1996/09/17 08:08:21  sasha
 *Fixed code in AssignQuePointer() as it was reversing buffers.
 *Added code to Start() to take raw mode size into account when
 *making decision to go both fields.
 *Revision 2.10  1996/09/07 07:18:40  sasha
 *Added a bool argument to the AssignQuePointer() to control odd/even
 *buffer addresses. Make a call to the RISC program code to set adjustment
 *for the YUV formats above CIF
 *Revision 2.9  1996/08/15 07:54:34  sasha
 *Make tops of the clip rectangles even for full-size capture.
 *Revision 2.8  1996/08/10 23:44:54  sasha
 *Added code to set a flag depending if there is a full-screen DOS box.
 *Revision 2.7  1996/08/01 19:31:41  rudyl
 *Moved the call TurnVFilter(off) so it is called before Scaler::SetVFilter is being called.
 *Revision 2.6  1996/07/29 05:07:45  sasha
 *Modified clipping code to remove the bottom empty line. Improved 
 *how video looks in resolutions above CIF ( use same scaling for both
 *fields ).
 *Revision 2.5  1996/07/19 01:42:53  rudyl
 *Fixed DOS box completely covered video freeze problem.
 *Revision 2.4  1996/06/26 19:09:23  sasha
 *Added FNPause and FNContinue APIs. Divide top of the analog
 *rectangle in half
 *Revision 2.3  1996/06/17 21:10:32  sasha
 *Added PsVM::OnSetDeviceFocus to handle DOS boxes. Create system VM
 *object when loaded for the same purpose
 *Revision 2.2  1996/06/15 00:27:55  sasha
 *Added code to process VM creation and destruction to restart Pisces
 *properly after a full screen DOS box goes away. Still do not handle
 *alt-tab properly
 *Revision 2.1  1996/06/10 20:12:48  ssm
 *MarkBeta1AsRev2
 *Revision 1.19  1996/05/23 03:29:45  sasha
 *Set VBI buffer pointers to NULL when stopping VBI capture from ring 3
 *Revision 1.18  1996/05/21 18:51:17  rudyl
 *Per Ben's suggestion, since we don't have that many lines available to display,
 *changed lines for PAL/SECAM from 288 to 284 for default size.
 *Revision 1.17  1996/05/17 20:44:01  sasha
 *Set the same ring3 callback for both VBI channels in Start().
 *Revision 1.16  1996/05/14 23:58:23  ssm
 *move 1.15.1.3 to head
 *Revision 1.15.1.3  1996/05/14 23:38:39  sasha
 *Added support for callback to ring 3 in the Start() for VBI channels.
 *Properly Stop() VBI channels from ring3.
 *Revision 1.15.1.2  1996/05/12 00:27:53  sasha
 *Split into 3 parts. Moved devioctl into a devioctl.cpp. Moved PnP-related
 *code into PsPnP.cpp.
 *Added support for opening and working with the VBI channels from ring3
 *capture driver. Not tested at all ( except old stuff seems to be still working ).
 *Revision 1.15.1.1  1996/05/10 23:36:23  sasha
 *Revision 1.15  1996/04/26 22:36:50  sasha
 *Added function headers. Removed buffer allocation/deallocation.
 *Added Pisces suspention/resume to power events
 *Revision 1.14  1996/04/24 22:51:34  sasha
 *Added FNGetBaseAddr. Added initial power events code. Added one
 *more DWORD when getting GPIO pointer. Added SetRate() - do not call yet
 *Revision 1.13  1996/03/29 21:07:46  sasha
 *Changed the IOcontrol to do decoder functions via func/subfunc. Added
 *Get...() functions.
 *Revision 1.12  1996/03/27 00:36:14  sasha
 *Added a bunch of functions to the Win32IOControl ( channel, decoder )
 *Revision 1.11  1996/03/22 23:04:08  sasha
 *Changed to use supplied pitch directly ( in bytes already )
 *Revision 1.10  1996/03/22 00:00:00  sasha
 *Added support for detecting full-screen dos boxes and stopping Pisces
 *Revision 1.9  1996/03/15 23:04:44  sasha
 *Changed types of the VBI buffers to PVOID
 *Revision 1.8  1996/03/05 23:31:18  sasha
 *Added support for VBI capture. Added support for the interrupt sharing.
 *Changed interrupt handler. Added handling of failure to hook an interrupt.
 *Revision 1.7  1996/02/27 02:11:27  sasha
 *Nothing important
 *Revision 1.6  1996/02/19 02:39:45  sasha
 *Do not forget to close the channels in the Finish(). Added support to I2C.
 *Added initial support for the VBI channels
 *Revision 1.5  1996/02/08 23:39:31  sasha
 *Added code to set clipping for the full-size capture. Changed SetClipping()'s
 *argument type. Changed the way channels are open a bit to make sure the
 *preview channel is open even when VFW driver does not explicitly open it.
 *Revision 1.4  1996/02/06 01:41:38  sasha
 *Added DIOC entry point. Removed timer-related code. Changed order of
 *channel Start()s. Use Detach() to delete allocated buffers. Made Finish()
 *out-of line. Changed OnPnpNewDevnode to return CR_.. values instead of BOOL
 *Added CreateCaptureController() member function.
 *Revision 1.3  1996/01/23 00:01:02  sasha
 *Added decoder functions.
 *Revision 1.2  1996/01/12 00:02:49  sasha
 *Renamed file to be device.cpp. Bunch of changes.
 *Revision 1.1  1995/09/06 02:14:34  sasha
 *Initial revision
 */

static char rcsid[] = "$Id: device.cpp 2.68 1998/06/01 18:08:52 mdimeo Exp $" ;
#define DEVICE_MAIN
#include "device.h"
Declare_Virtual_Device(BT848)
#undef DEVICE_MAIN
#include <vdd.h>

#include "dbgprint.h"
#include "cliplist.h"

#ifdef SELF_DESTRUCT
#include "boomdate.h"
#endif

extern "C" {
  #include "vpaec.h"
}
#include "vxdid.h"
#include "bt848vxd.h"

#define SIZE( arr ) ( sizeof( arr ) / sizeof( arr##[0] ) )

LONG DebugLevel = 0; // Default DEBUG level is 0

const I2C_Offset = 0x110;
const GPIO_Cntl_Offset  = 0x10D;
const GPIO_OutputOffset = 0x118;
const GPIO_DataOffset   = 0x200;

DWORD gdwStreamStartTime = 0 ;

DWORD gdwTridentAperture0 = 0;
DWORD gdwTridentAperture1 = 0;

extern  VDevice* TheDevice;
PsInterrupt      *HWInt = NULL;
DWORD dummy;
PDWORD pSkipCount = &dummy;
PDWORD  pStopFlag;

PCI_CONFIG_848 PsDevice::pciCfgHdr_ ;

LPVIDEOHDR PsDevice::EvenVBIBuf_ [2];
LPVIDEOHDR PsDevice::OddVBIBuf_  [2];

RGNDATA * pViewList = NULL ;      // DD3 "clip" list
RGNDATA * pClipList = NULL ;      // CC  clip list
RGNDATA * pCombinedList = NULL ;  // merged clip list
DWORD *viewListBuf = NULL;        // buffer for saving current "clip"List
DWORD cbViewListBuf = 0 ;
DWORD *clipListBuf = NULL;        // buffer for saving current viewList
DWORD cbClipListBuf = 0 ;

enum {SINGLE_FIELD, FULL_SCREEN} lastMode ;

SOUND_SYNC gSoundSync ;
HANDLE gCapEvent = NULL ;

// holds pointers to VMs
PsVM *VMArray [25];
// index into the above array
static int VMIdx = 0;

static bool PowerDownInEffect = false;

static char displayFocus = 0;   //RL who has display device focus (0=sys VM, 1=DOS box)

#define DO_CLIP_ON_PROG(a,b)           \
if(pa[a]) {                            \
  CaptureContrll_->SkipProgram(a) ;    \
  rEng.SetClipping( pa[a], *b) ;       \
  CaptureContrll_->UnSkipProgram(a) ; }

#define DO_CLIP_PAIR(a,b,c)                   \
if(pa[a]) CaptureContrll_->SkipProgram(a) ;   \
if(pa[b]) CaptureContrll_->SkipProgram(b) ;   \
if(pa[a]) rEng.SetClipping( pa[a], *c) ;      \
if(pa[b]) rEng.SetClipping( pa[b], *c) ;      \
if(pa[a]) CaptureContrll_->UnSkipProgram(a) ; \
if(pa[b]) CaptureContrll_->UnSkipProgram(b) ;

int run_risc( LPDWORD *StartInstruction, BtPiscesPlus &Digitizer,
   unsigned char *buffer_start, int exit_on_jump, int exit_on_buffer );

void AdjustRegionForFullScreen(RGNDATA &rgnData)
{
   int i;
   for ( i = 0; i < rgnData.rdh.nCount; i++ ) {
      TRect *lpR = (TRect *)rgnData.Buffer + i;

      // make all even
      lpR->top++;
      lpR->top &= ~1;

      lpR->bottom++;
      lpR->bottom &= ~1;

      lpR->top    /= 2;
      lpR->bottom /= 2;
   }
}
void AdjustRegionForSingleField(RGNDATA &rgnData)
{
   int i;
   for ( i = 0; i < rgnData.rdh.nCount; i++ ) {
      TRect *lpR = (TRect *)rgnData.Buffer + i;

      lpR->top    *= 2;
      lpR->bottom *= 2;
   }
}
VMHANDLE GetVMHandle() { return ((PsDevice*)TheDevice)->GetVMHandle(); }

PsDevice *PsDevice::GetPsDevice() { return (PsDevice*)TheDevice; }

PsVM::PsVM( VMHANDLE hVM ) : VVirtualMachine( hVM ) //RL , FullScreen_( false )
{
}

//RL The following info is provided by Intel
/*.......................................................................


   The concept is as follows: the only time any virtual machine besides the
   system virtual machine gets the Virtual Display Device (VDD) focus is
   when a full-screen DOS box is in use. Therefore, we trap on the Set
   Device Focus system message, and then check to see if the VDD is being
   set to something besides the system VM. This indicates a switch to
   a full-screen dos box is underway. If it's being set to the system VM,
   and DisplayFocus was a 1, this indicates we are switching out of the
   dos box back to windows.

   The basic problem I found was that this message is one of convenience:
   Win95 appears to send it when it 'gets the time'.  Therefore, most of
   the time, it appears randomly in the flow of commands controlling
   the switching. This makes it unreliable, at best.

   The code below relied on a weirdness with the S3 driver -- after it
   had performed the switchover, the overlay surface was un-mapped,
   therefore it showed as all FF's. Since my notice was too late to
   prevent the problem, I attempted to glom onto this un-mapped state
   as a possible solution -- but to no avail.  Keep it here as a
   reminder of all the bad things that can happen in this state.

   I am also going to copy the entire discussion from drvproc.c to here, to
   indicate the problems I ran into. I'm also going to keep all of this
   grotesque code here, but commented out, since we may have to resort
   to it for Video cards that either do not realize they have a problem
   with full-screen dos boxes, or have no solution for it that they'll
   tell us about.  With this code in, I managed to get up to 60-70
   switches before we crashed and burned with the S3.

..................................................................*/

/* Method: PsVM::OnSetDeviceFocus
 * Purpose: Used to pause/restart Pisces when switching to/from DOS boxes
 */
VOID PsVM::OnSetDeviceFocus( DWORD devID, DWORD flags, VMHANDLE aVM )
{
//RL   if ( Test_Sys_VM_Handle( getHandle() ) || !FullScreen_ )
//RL      PsDevice::GetPsDevice()->RestartCapture();
//RL   else
//RL      if ( FullScreen_ )
//RL         PsDevice::GetPsDevice()->PauseCapture();

   if ( devID == VDD_DEVICE_ID ) {   // display device
      switch ( displayFocus ) {   // who has the focus now?
      case 0:                    // sys VM owns the display
         if ( Test_Sys_VM_Handle( getHandle() ) )  // setting to system VM
            break;
         displayFocus = 1;
         PsDevice::GetPsDevice()->PauseCapture();
         *pStopFlag = true;
         break;
      case 1:                    // some DOS box owns the display
         if ( !Test_Sys_VM_Handle( getHandle() ) ) // setting to some other dos box
            break;
         displayFocus = 0;
         PsDevice::GetPsDevice()->RestartCapture();
         *pStopFlag = false;
         break;
      } // switch
   }
}

/* Function: GetVMIdx
 * Purpose: Searches for an entry in the VMArray with the given VM handle
 * Input: hVM: VMHANDLE
 */
int GetVMIdx( VMHANDLE hVM )
{
   for ( int i = 0; i < VMIdx; i++ )
      if ( VMArray [i] && hVM == VMArray [i]->getHandle() )
         return i;
   return  0;
}

/* Method: PsVM::OnDestroyVM
 * Purpose: Handles destroy VM message to see if Pisces needs to be restarted
 */
VOID PsVM::OnDestroyVM()
{
   int idx = GetVMIdx( m_handle );

   // at this point always want to restart Pisces as even if other full-screen
   // VMs exist they are minimized after another full-screen VM ( this one ) exists
   // this function will take care of not restarting of was not paused
   PsDevice::GetPsDevice()->RestartCapture();
   delete VMArray [idx];
   VMArray [idx] = NULL;
   VMIdx--;
}

PsThread::PsThread(THREADHANDLE hThread) : VThread(hThread) {}

PsInterrupt::PsInterrupt(int irq ) : VHardwareInt( irq, VPICD_OPT_CAN_SHARE, 0, 0 )
{}

/* Method: PsInterrupt::OnHardwareInt
 * Purpose: Handles HW interrupt on Pisces IRQ
 * Input: VMNAHDLE
 * Output: void
 */
VOID PsInterrupt::OnHardwareInt( VMHANDLE )
{
   if ( ((PsDevice*)TheDevice)->Interrupt(PowerDownInEffect) == On ) {
      sendPhysicalEOI();
      ClearCarry();
      return;
   }
   SetCarry();
}

/* Method: PsDevice::PauseCapture
 * Purpose: Pauses Pisces
 * Input: None
 * Output: None
*/
void PsDevice::PauseCapture()
{
   if ( CaptureContrll_ )
      CaptureContrll_->Pause();
}

/* Method: PsDevice::RestartCapture
 * Purpose: Restarts Pisces after it was stopped by PauseCapture
 * Input: None
 * Output: None
*/
void PsDevice::RestartCapture()
{
   if ( CaptureContrll_ )
      CaptureContrll_->Continue();
}

/* Method: PsDevice::OnPowerEvent
 * Purpose: Handles different power events in the system
 * Input: dwEvent: DWORD, type of power event
 *   pResult: PDWORD, pointer to the return value
 * Output: BOOL
 */
BOOL PsDevice::OnPowerEvent( DWORD dwEvent, PDWORD pResult )
{
   if ( dwEvent == PWR_SUSPENDREQUEST ) {
      PowerDownInEffect = true;
      PauseCapture();
   } else
      if ( dwEvent == PWR_SUSPENDRESUME ) {
         PowerDownInEffect = false;
         RestartCapture();
      }
   return true;
}

/* Method: PsDevice::OnCreateVM
 * Purpose: creates a VM object so it can restart Pisces when gets destroyed
 */
BOOL  PsDevice::OnCreateVM( VMHANDLE hVM )
{
   int i;
   for ( i = 0; i < SIZE( VMArray ); i++ ) // find first empty entry
      if ( !VMArray [i] )
         break;
   if ( i == SIZE( VMArray ) ) // somebody has a lot of DOS boxes
      return true;

   VMArray [i] = new PsVM( hVM );
   VMIdx++;
   return true;
}

/* Method: PsDevice::OnSysDynamicDeviceInit
 * Puspose: Performs start-up initialization
 * Input: None
 * Output: BOOL
 */
BOOL PsDevice::OnSysDynamicDeviceInit()
{
   DebugPrint((1, "dyna-init\n\r" ));

   OnCreateVM( Get_Sys_VM_Handle() );
   VEvent::initEvents();
//RL   Hook_Device_Service_Ex( __VDD_Set_VMType, FullScreenChecker, &FullScreenThunk );

#ifdef SELF_DESTRUCT
//
// Self destruct: The macro is enabled by the makefile if the environment
//                variable SELF_DESTRUCT is present at compile time.
//                If enabled (should only be for desktop builds), the header file
//                "boomdate.h" will have been created by the makefile
//                and included above. This header file contains a number (dwKillDate) which
//                represents the number of days since 1/1/80 + the default
//                lifespan of this vxd (30 days).
//
   DWORD dwDate ;   // days since Jan 1 1980
   VTD_Get_Date_And_Time(&dwDate);
   DebugPrint((0,"Today's Date is %d; kill date is %d\n",dwDate,dwKillDate)) ;

   if (dwDate >= dwKillDate)
   {
     DebugPrint((0,"Life Expired...\n")) ;
     return FALSE;
   }
#endif

   return TRUE;
}

/* Method: PsDevice::OnSysDynamicDeviceExit()
 * Purpose: Performs clean-up
 * Input: None
 * Output: BOOL
 */
BOOL PsDevice::OnSysDynamicDeviceExit()
{
   Finish();
   return TRUE;
}

/* Method: PsDevice::OnCritRebootNotify
 * Purpose: Stops Pisces before a reboot
 * Input: None
 * Output: None
 */
VOID PsDevice::OnCritRebootNotify()
{
   PauseCapture();
}

/* Method: PsDevice::OnSystemExit
 * Purpose: Finishes when system shuts down
 * Input: None
 * Output: None
 */
VOID PsDevice::OnSystemExit( VMHANDLE )
{
   Finish();
}

/* Method: BtPisces::CreateCaptureController
 * Purpose:
 * Input: None
 * Output: None
 */
void PsDevice::CreateCaptureController( WORD devId )
{
   delete CaptureContrll_;
   CaptureContrll_ = new BtGemini(devId);
   if ( QuartzSightVP_ )
      delete QuartzSightVP_;
   QuartzSightVP_ = new BtScotch();
   VxDVideoChannel::Init( CaptureContrll_ );
   VideoIn_.OpenChannel();
   VideoOut_.OpenChannel();

   if ( VBIConnect() == Success )
       if ( VBISetParams() == Success )
          return;
}

/* Function: GetVidChan
 * Purpose: Used to retrieve address of a video channel passed by the
 *   capture driver
 */
inline VxDVideoChannel &GetVideoChannel( CLIENT_STRUCT *pRegs )
{
   return *(VxDVideoChannel*)_clientEBX;
}

/* Function: GetData
 * Purpose: A shortcut wrapper to get data passed by capture driver
 * Input: ptr16_16: DWORD - ring 3 16:16 type pointer
 *   DX: WORD - client's DX register
 *   hVM: VMHANDLE - handle of the current VM
 */
inline PVOID PsDevice::GetData( DWORD ptr16_16 )
{
   return MapPointerToFlat( HIWORD( ptr16_16 ), LOWORD( ptr16_16 ), hCurVM_ );
}

/* Method: PsDevice::AddBuf
 * Purpose: Adds a buffer to one of the VBI channels
 * Input: VideoChan: VxDVideoChannel &
 *   pBufAddr: PVOID - address of the next buffer
 * Output: None
 * Note:
 */
void PsDevice::AddBufVBI( VxDVideoChannel &VBIChan, LPVIDEOHDR lpvh )
{
   if ( &VBIChan == &VBIEven_ ) {
      if ( !EvenVBIBuf_ [0] )
         EvenVBIBuf_ [0] = lpvh;
      else
         EvenVBIBuf_ [1] = lpvh;
   } else {
      if ( !OddVBIBuf_ [0] )
         OddVBIBuf_ [0] = lpvh;
      else
         OddVBIBuf_ [1] = lpvh;
   }
}

#define  BEGIN_CRITICAL { \
   asm pushfd;            \
   asm cli;               \
}

#define END_CRITICAL { \
   asm popfd;          \
}

/* Method: PsDevice::AddBuf
 * Purpose: Adds next buffer to be used to the queue
 * Input: VideoChan: VxDVideoChannel &
 *   pBufAddr: PVOID - address of the next buffer
 * Output: None
 */
void PsDevice::AddBuffer( VxDVideoChannel &VideoChan, LPVIDEOHDR pVidHdr )
{
   // bogus channel, bye-bye
   if ( !IsOurChannel( VideoChan ) )
      return;
   if ( IsVBIChannel( VideoChan ) ) {
      AddBufVBI( VideoChan, pVidHdr );
      return;
   }

   BEGIN_CRITICAL;

   DebugPrint((1,"\nPsDeviceAddBuffer: Adding %x to %s\n",pVidHdr, &VideoChan == &VideoIn_ ? "Video In" : "Video out")) ;
   VideoChan.AddBuffer( pVidHdr );

   END_CRITICAL;
}      

/* Function: GetEvenAndOddSizes
 * Purpose: Calculates sizes of even and odd fields given size of full image
 * Input: SizeFull: TSize &
 *   SizeOdd: TSize &
 *   SizeEven: TSize &
 */
void GetEvenAndOddSizes( TSize &SizeFull, TSize &SizeFirst, TSize &SizeSecond )
{
   // 1. have to divide image in half first ( vertically )
   // 2. set sizes accordingly for each field
   // first field has one line more if size is odd

   SizeFull.cy++;
   SizeFull.cy &= ~1;
   int HalfHight = SizeFull.cy / 2;
   SizeFirst  = TSize( SizeFull.cx, HalfHight );// + ( SizeFull.cy & 1 ) );
   SizeSecond = TSize( SizeFull.cx, HalfHight );
}

/* Method: PsDevice::SetChannelsAnalogWindows
 * Purpose: Sets sizes of analog windows for capture and preview channels
 * Input: r: TRect & - reference to the rectangle defining the window
 * Output: ErrorCode
 */
ErrorCode PsDevice::SetChannelsAnalogWindows( TRect &r )
{
   TSize SizeFirst;
   TSize SizeSecond;
   r.top /= 2; // top is the half of original
   TSize tmpFullSize = r.Size();
   GetEvenAndOddSizes( tmpFullSize, SizeFirst, SizeSecond );

   if ( FullSizeChannel_->SetAnalogWindow( TRect( r.TopLeft(), SizeFirst ) ) != Success ||
        SlaveChannel_->SetAnalogWindow( TRect( r.TopLeft(), SizeSecond  ) ) != Success ) {
      FullSizeChannel_ = NULL;
      return Fail;
   }
   return Success;
}

/* Method: PsDevice::SetChannelsDigitalWindows
 * Purpose: Sets sizes of digital windows for capture and preview channels
 * Input: r: TRect & - reference to the rectangle defining the window
 * Output: ErrorCode
 */
ErrorCode PsDevice::SetChannelsDigitalWindows( TRect &r )
{
   TSize SizeFirst;
   TSize SizeSecond;
   TSize tmpFullSize = r.Size();
   GetEvenAndOddSizes( tmpFullSize, SizeFirst, SizeSecond );

   if ( FullSizeChannel_->SetDigitalWindow( TRect( r.TopLeft(), SizeFirst  ) ) != Success ||
        SlaveChannel_   ->SetDigitalWindow( TRect( r.TopLeft(), SizeSecond ) ) != Success ) {
      FullSizeChannel_ = NULL;
      return Fail;
   }
   return Success;
}

/* Method: PsDevice::AssignQuePointer
 * Purpose: Changes queue pointer of the 'slave' channel to the temporary one
 * Input: dwOffset: DWORD, offset between odd and even buffers
 *   swap: bool, indicates if even/odd needs to be swapped
 * Output: None
 */
void PsDevice::AssignQuePointer( DWORD dwOffset, bool swap )
{
      if ( swap ) {
         VideoOut_.SetOffset(dwOffset) ;
         VideoIn_.SetOffset(0) ;
      } else {
         VideoIn_.SetOffset(dwOffset) ;
         VideoOut_.SetOffset(0) ;
      }
}


void PsDevice::PreStart(VxDVideoChannel &VidChan, StartParms *pParms)
{
   if      (&VidChan==&VideoIn_ ) {
       DebugPrint((0,"Start : VideoIn_\n")) ;
   }
   else if (&VidChan==&VideoOut_ ) {
       DebugPrint((0,"Start : VideoOut_\n"));
   }
   else if (&VidChan==&VBIEven_ ) {
       DebugPrint((0,"Start : VBIEven_\n")) ;
   }
   else if (&VidChan==&VBIOdd_ ) {
       DebugPrint((0,"Start : VBIOdd_\n"))  ;
   }
   else {
       DebugPrint((0,"Start : Unknown VidChan\n")) ;
   }

   // Get trident alternate aperture address
   if (&VideoIn_ == &VidChan )
   {
     gdwTridentAperture0 = pParms->TridentAperture0;
     gdwTridentAperture1 = pParms->TridentAperture1;
   }
   DebugPrint((1,"** Start: apertures  %x, %x\n", gdwTridentAperture0 , gdwTridentAperture1)) ;

   DWORD * pDVSIFReg        = (DWORD *)( RegBase::GetBaseAddress() + 0xFC );
   DWORD * pEvenMiscCtrlReg = (DWORD *)( RegBase::GetBaseAddress() + 0x2C );
   DWORD * pOddMiscCtrlReg  = (DWORD *)( RegBase::GetBaseAddress() + 0xAC );

   if (bQuartzSightMode_)
   {
     // Turn off filtering for Camera
     *pDVSIFReg        |= BIT(6) ;
     *pEvenMiscCtrlReg |= (BIT(5) | BIT(7)) ;
     *pOddMiscCtrlReg  |= (BIT(5) | BIT(7)) ;
   }
   else
   {
     // make sure filtering is on if not camera
     *pDVSIFReg        &= ~(BIT(6)) ;
     *pEvenMiscCtrlReg &= ~(BIT(5) | BIT(7)) ;
     *pOddMiscCtrlReg  &= ~(BIT(5) | BIT(7)) ;
   }
}

ErrorCode PsDevice::StartQuartzSight(VxDVideoChannel &VidChan, StartParms *pParms)
{
    if ( VPAEC_Is_Initialized() )
    {
       if (CaptureContrll_)
          CaptureContrll_->SetVsyncTaken( true ) ;
    }
    else     
       if (CaptureContrll_)
          CaptureContrll_->SetVsyncTaken( false ) ;

   // 
   // Set up risc engine
   //

   // we're using both fields, but they're independant of each other (no master/slave)
   VideoOut_.SetPaired(false) ;
   VideoIn_.SetPaired(false) ;

   // VideoOut is associated with odd field which is first
   FullSizeChannel_ = &VidChan;  //[bugbug] is this necessary?

   if ( &VidChan == &VideoIn_ ) {
      // make buffer processing possible. Do that for the preview or
      // capture modes only. We do not interrupt for overlay.
      VideoIn_.EnableInterrupt();
      VideoOut_.EnableInterrupt();
      SlaveChannel_ = &VideoOut_;
      DebugPrint((0,"Both interrupts enabled. Slave is Video Out\n")) ;
   } else {
      SlaveChannel_ = &VideoIn_;
      VideoIn_.DisableInterrupt();
      VideoOut_.DisableInterrupt();
      DebugPrint((0,"Both interrupts disabled. Slave is Video in\n")) ;
   }

   // turn off the vertical filter to get rid of the junk lines on top
   Scaler::TurnVFilter( Off );

   // Set video size in HW
   VideoIn_.SetAnalogWindow ( pParms->Analog  )  ;      // odd field
   VideoOut_.SetAnalogWindow ( pParms->Analog  )  ;     // even field

   VideoIn_.SetDigitalWindow( pParms->Digital )  ;
   VideoOut_.SetDigitalWindow( pParms->Digital )  ;

   // make both channels produce the same color format !
   if ( VideoIn_.SetFormat ( ColFmt( pParms->ColFmt ) ) != Success ||
        VideoOut_.SetFormat( ColFmt( pParms->ColFmt ) ) != Success ) {
      FullSizeChannel_ = NULL;
      return Fail;
   }

   // set the proper pitch 
   dwFullSizePitch_ = pParms->dwPitch ;
   VideoIn_.SetBufPitch ( dwFullSizePitch_ );
   VideoOut_.SetBufPitch( dwFullSizePitch_ );

//   // set slave to grab bufffers from master's queue
//   SlaveChannel_->SetBufQue(&FullSizeChannel_->GetCurrentQue()) ;

   // Ensure that there is no offset (possible if last start was for an interlaced image)
   VideoOut_.SetOffset(0) ;
   VideoIn_.SetOffset(0) ;

   // Camera mode needs both fields to Notify
   VideoIn_.SetAppCallback( pParms->callback, pParms->tag );
   VideoOut_.SetAppCallback( pParms->callback, pParms->tag );

   RISCProgram::SetRate( pParms->FrameRate );
   BtPisces::SetPlanarAdjust( 0 );

   // important to start master first
   if ( VideoIn_.Start(pParms->ImageSize, START_MASTER ) )
   {
      if ( VideoOut_.Start(pParms->ImageSize, START_SLAVE ) ) {
         DebugPrint((-1,"start QuartzSight (FrameRate = %d) @ %d\n",pParms->FrameRate,Get_System_Time()));
         lastMode = FULL_SCREEN;
         return Success;
      } else {
         VideoIn_.Stop();
         VideoOut_.Stop();
//         SlaveChannel_->SetDefaultQue();
         FullSizeChannel_ = NULL;
         SlaveChannel_    = NULL;
      }
   }
   return Fail ;
}

ErrorCode PsDevice::StartInterlaced(VxDVideoChannel &VidChan, StartParms *pParms)
{
   DebugPrint((1,"\nStartInterlaced\n")) ;

   VideoOut_.SetPaired(true) ;
   VideoIn_.SetPaired(true) ;

   // VideoOut is associated with odd field which is first
   FullSizeChannel_ = &VidChan;

   if ( &VidChan == &VideoIn_ ) {
      // make buffer processing possible. Do that for the preview or
      // capture modes only. We do not interrupt for overlay.
      VideoIn_.EnableInterrupt();
      VideoOut_.EnableInterrupt();
      SlaveChannel_ = &VideoOut_;
      DebugPrint((1,"   Both interrupts enabled. Slave is Video Out\n")) ;
   } else {
      SlaveChannel_ = &VideoIn_;
      VideoIn_.DisableInterrupt();
      VideoOut_.DisableInterrupt();
      DebugPrint((1,"   Both interrupts disabled. Slave is Video in\n")) ;
   }

   // turn off the vertical filter to get rid of the junk lines on top
   Scaler::TurnVFilter( Off );

   TRect r = pParms->Analog;

   if ( SetChannelsAnalogWindows( r ) != Success )
      return Fail;

   if ( SetChannelsDigitalWindows( pParms->Digital ) != Success )
      return Fail;

   // make both channels produce the same color format !
   if ( VideoIn_.SetFormat ( ColFmt( pParms->ColFmt ) ) != Success ||
        VideoOut_.SetFormat( ColFmt( pParms->ColFmt ) ) != Success ) {
      FullSizeChannel_ = NULL;
      return Fail;
   }
   // set the proper pitch ( twice the original pitch )
   ColorSpace tmp( ColFmt( pParms->ColFmt ) );

   dwFullSizePitch_ = pParms->dwPitch * 2; // write to the every second line

   VideoIn_.SetBufPitch ( dwFullSizePitch_ );
   VideoOut_.SetBufPitch( dwFullSizePitch_ );

   // now the channel that has initiated this mode will supply data buffers
   // have to switch other channel's buffer queue to make it
   // use master channel's buffers
   // swap when starting preview channel and doing planar mode
   bool swap = (bool) (FullSizeChannel_ == &VideoIn_ && 
      !( tmp.GetColorFormat() > CF_RGB8 && tmp.GetColorFormat() < CF_VBI ));

   // For interlaced mode, the dwPitch offset is given to one
   // of the fields in AssignQuePointer.
   DWORD dwOffset = pParms->dwPitch ;
      
   // AssignQuePointer flushes both stream queues and builds a FullSize que.
   DebugPrint((1,"   Offseting a field... %d\n",dwOffset)) ;
   AssignQuePointer( dwOffset, swap );

   // second field will generate the callback
   VideoIn_.SetAppCallback( pParms->callback, pParms->tag );

   // take care of setting clipping for the second field
   RGNDATA *const clip = VidChan.GetClipping();
   if ( clip )
   {
     if(lastMode != FULL_SCREEN) 
        AdjustRegionForFullScreen(*clip) ;
      SetClipping( VidChan, *clip );
   }

   RISCProgram::SetRate( pParms->FrameRate );

   // important to start master first
   BtPisces::SetPlanarAdjust( pParms->dwPitch );  // master gets planar adjust!
   if ( VideoIn_.Start(pParms->ImageSize, START_MASTER ) )
   {
      BtPisces::SetPlanarAdjust( 0 );
      if ( VideoOut_.Start(pParms->ImageSize, START_SLAVE ) ) {
         DebugPrint((0,"start double (FrameRate = %d)\n",pParms->FrameRate));
         BtPisces::SetPlanarAdjust( 0 );
         lastMode = FULL_SCREEN;
         return Success;
      } else {
         VideoIn_.Stop();
         VideoOut_.Stop();
//         SlaveChannel_->SetDefaultQue();
         FullSizeChannel_ = NULL;
         SlaveChannel_    = NULL;
         BtPisces::SetPlanarAdjust( 0 );
      }
   }
   return Fail ;
}

ErrorCode PsDevice::StartSingle(VxDVideoChannel &VidChan, StartParms *pParms)
{
  VideoOut_.SetPaired(false) ;
  VideoIn_.SetPaired(false) ;

  RISCProgram::SetRate( pParms->FrameRate);
  VidChan.SetBufPitch( pParms->dwPitch );
  VidChan.SetOffset(0) ;

  VidChan.SetAppCallback( pParms->callback, pParms->tag );
  VideoIn_.EnableInterrupt(); // restore the default which could've been changed for full size capture

  // enable the vertical filter
  Scaler::TurnVFilter( On );

  if(lastMode == FULL_SCREEN)
  {
    if ( pViewList )
       AdjustRegionForSingleField(*pViewList) ;
    if ( pClipList )
       AdjustRegionForSingleField(*pClipList) ;
  }

  TRect r = pParms->Analog;

  if ( VidChan.SetAnalogWindow ( pParms->Analog  ) == Success && //<-must be set first !
       VidChan.SetDigitalWindow( pParms->Digital ) == Success &&
       VidChan.SetFormat( ColFmt( pParms->ColFmt ) ) == Success &&
       VidChan.Start(pParms->ImageSize, START_SINGLE ) != NULL ) {
     DebugPrint((-1, "start single (FrameRate = %d, pitch = %d) @ %d\n",
                    pParms->FrameRate,pParms->dwPitch, Get_System_Time()));
     lastMode = SINGLE_FIELD ;
     return Success;
  }
  return Fail ;
}

LPVIDEOHDR lpvhOverlay = NULL ;
StartParms spLastOverlay ;

ErrorCode PsDevice::Start( VxDVideoChannel &VidChan, PVOID pData )
{
   StartParms *pParms = (StartParms*)pData;

   if (&VidChan == & VideoOut_)
      spLastOverlay = *pParms ;

   PreStart(VidChan, pParms) ;

   // Vbi Channel ?
   if ( IsVBIChannel( VidChan ) ) {
      VBIProcessor_ = CapDrvVBIIntrCallback;
      VBIEven_.SetRing3Callback( pParms->callback, pParms->tag );
      VBIOdd_.SetRing3Callback( pParms->callback, pParms->tag );
      StartVBI(VBI_INTERNAL);
      return Success;
   }

   if ( FullSizeChannel_ )  
   {
      DebugPrint((-1,"\nFullSizeChannel is not NULL (both fields occupied)\n"));
      //
      // If we're in QuartzSight mode at this point, then the reason that FullSizeChannel is
      // non-null is because we're currently in overlay mode and we're streaming both
      // fields to the screen in order to achieve 30fps.
      //
      // If the channel to start is VideoIn_, then we have just requested a capture from
      // overlay mode. In the case of the Quartzsight camera, we have a requirement to 
      // "gracefully degrade" instead of simply refusing the request (refusal on the basis 
      // that both fields are already commited to support the overlay and there is no
      // field available to support capture). The graceful degradation implemented here is:
      // 1. If the requested capture frame rate is 15 fps or less, restart overlay in single 
      //    field mode (essentially 15 fps instead of 30) and start up a single field capture
      //    at the requested frame rate. In this mode, capture monitor is updated on the
      //    overlay destination (at a reduced frame rate -- no longer 30 fps). 
      // 2. If the requested frame rate is above 15 fps (max 30 fps), then we need to
      //    make use of both fields for capture. We must therefore discontinue the overlay
      //    and configure both fields for capture. This means that there will be no capture
      //    monitoring on the overlay, but the capture frame rate can go to 30fps.
      // 3. Original overlay mode is re-established after capture is completed.
      //
      // This is probably the best compromise since:
      // 1. We always satisfy the requested capture frame rate.
      // 2. We never fail capturing from overlay on QuartzSight.
      //
      if (bQuartzSightMode_ && &VidChan == &VideoIn_) // qs mode and video in requested (capture)
      {
         DebugPrint((-1,"   QuartzSight mode and video in requested\n"));

         // save overlay state for later restore
         CProgArray & pa = CaptureContrll_->GetCreatedProgs () ;
         if (pa[1] != NULL)
            lpvhOverlay = pa[1]->GetVideoHdr() ;

         // as there will already be buffers in the queue at this point, we need to preserve them
         // across the stop
         CQueue tmp(10) ;
         CQueue &r3q = CaptureContrll_->GetR3VidInQueue() ;
         LPVIDEOHDR lpvh = r3q.Remove() ;
         while (lpvh) {
            tmp.Add(lpvh) ;
            lpvh = r3q.Remove() ;
         }

         // stop the overlay
         pSkipCount = (PDWORD)&dummy;
         Stop( VideoOut_ );  // flushes video hdr queue since all streams have stopped

         // if fps <= 15
         if (pParms->FrameRate >= 66)  // 66 ms == 15 fps
         {
            // restart the overlay using one field only
            r3q.Add(lpvhOverlay) ;
            r3q.Add(lpvhOverlay) ;
            StartSingle(VideoOut_, &spLastOverlay) ;

            // adjust framerate
            pParms->FrameRate  /= 2 ;

            // start single field capture
            lpvh = tmp.Remove() ;  // restore the buffers
            while (lpvh) {
               r3q.Add(lpvh) ;
               lpvh = tmp.Remove() ;
            }

            DebugPrint((-1,"   Starting Single from Overlay mode (QS)(requested frame rate <= 15)\n"));
            return StartSingle(VidChan, pParms) ;
         }
         else // fps > 15 : need more than one field to capture
         {
            // restore the buffers 
            lpvh = tmp.Remove() ;
            while (lpvh) {
               r3q.Add(lpvh) ;
               lpvh = tmp.Remove() ;
            }

            DebugPrint((-1,"   Starting QuartzSightMode from Overlay mode (requested frame rate > 15)\n"));
            return StartQuartzSight(VidChan, pParms) ;
         }
      }
      else
      {
         // full size image capture is in progress, can not accomodate more
         DebugPrint((-1,"   *** Start returns fail because we're already in 2 field mode\n"));
         return Fail;
      }
   }
   
   if (bQuartzSightMode_)
   {
      DebugPrint((1,"\nStarting QuartzSightMode\n"));
      return StartQuartzSight(VidChan, pParms) ;
   }

   //
   // Get max field height. Determines interlaced mode or not
   //
   VideoFormat VidFmt;
   GetFormat ( (PDWORD)&VidFmt );

   int height;
   SignalFormat sigFmt;
   GetSignalFormat( (PVOID)&sigFmt );
   if ( sigFmt != Signal_Analog )
   {  // digital signal; get video info about the signal
      VideoInfo vInfo;
      GetVideoInfo( (PVOID)&vInfo );
      height = vInfo.linesPerField;
   }
   else
      // height of PAL-NC is the same as PAL
      height = ( ColFmt( pParms->ColFmt ) == CF_RAW ? 434 :
               ( IsNTSCCompatible( VidFmt ) ? 240 : 288 ) );

   // get requested height for comparison
   TRect r = pParms->Analog;

   if ( r.Height() > height ) 
   {
      DebugPrint((1,"\nStarting Interlaced\n"));
      return StartInterlaced(VidChan, pParms) ;
   } 
   else 
   {
      DebugPrint((1,"\nStarting Single\n"));
      return StartSingle(VidChan, pParms) ;
   }
}

/* Method: PsDevice::Stop
 * Purpose: Adds next buffer to be used to the queue
 * Input: VideoChan: VxDVideoChannel &
 * Output: None
 */
void PsDevice::Stop( VxDVideoChannel &VidChan )
{
   DebugPrint((1, "stopped\n" ));

   if ( FullSizeChannel_ && FullSizeChannel_ == &VidChan ) 
   {
      // this forces the ProcessRISCInterrupt to shut the both video streams
      VideoIn_.SetDoPause( true );

      DWORD dwStart = Get_System_Time();
      while ( Get_System_Time() - dwStart < 33 )
         ; // give the ISR a chance to stop the 848

      asm int 1;

      VideoIn_.SetDoPause( false );

      // stop both channels
      FullSizeChannel_->Stop();
      SlaveChannel_->Stop();

//      // restore buffer queue
//      SlaveChannel_->SetDefaultQue();
      FullSizeChannel_ = NULL;

      VideoIn_.DisableInterrupt();
      VideoOut_.DisableInterrupt();
   } 
   else 
   {
      if ( IsVBIChannel( VidChan ) ) 
      {
         StopVBI();
         // reset the buffers for the next capture
         EvenVBIBuf_ [0] = NULL;
         EvenVBIBuf_ [1] = NULL;
         OddVBIBuf_  [0] = NULL;
         OddVBIBuf_  [1] = NULL;
      }
      else
      {
         VidChan.Stop();
      }
   }

   // if lpvhOverlay is non-null, then the normal QS overlay was interrupted
   // to perform a capture. We need to restore the overlay to normal operation.
   if (lpvhOverlay && bQuartzSightMode_ && &VidChan == &VideoIn_)
   {
      DebugPrint((-1,"Restarting interrupted QS overlay\n")) ;
      LPVIDEOHDR lpvhO = lpvhOverlay ;
      lpvhOverlay = NULL; // prevent recursion

      // stop the existing overlay (if running)
      pSkipCount = (PDWORD)&dummy;
      Stop( VideoOut_ );  // flushes video hdr queue since all streams have stopped

      // fill up the queue with the proper addr
      CQueue &r3q = CaptureContrll_->GetR3VidInQueue() ;
      r3q.Add(lpvhO) ;
      r3q.Add(lpvhO) ;

      // kick it off
      Start( VideoOut_, &spLastOverlay ) ;
   }

}

/* Method: PsDevice::SetClipping
 * Purpose: Propagates the call down a video channel
 * Input: VideoChan: VxDVideoChannel & - reference
 *   dwData: DWORD - a pointer to RGNDATA in reality
 * Output: None
 */


void PsDevice::SetClipping( VxDVideoChannel &VidChan, RGNDATA &const rgnData )
{
   if ( !rgnData.rdh.nCount )
      return;

   if ( FullSizeChannel_ ) {
      // have to decrese hight of all rectangles in half and decrease top in half
      //AdjustRegionForFullScreen(rgnData) ; //(now done earlier)

      FullSizeChannel_->SetClipping( rgnData );
      SlaveChannel_   ->SetClipping( rgnData );
   } else
      VidChan.SetClipping( rgnData );
}

/* Method: PsDevice::IsVideoChannel
 * Purpose:
 */
bool PsDevice::IsVideoChannel( VxDVideoChannel &aChan )
{
   return bool( &aChan == &VideoIn_ || &aChan == &VideoOut_ );
}

/* Method: PsDevice::IsVideoChannel
 * Purpose:
 */
bool PsDevice::IsVBIChannel( VxDVideoChannel &aChan )
{
   return bool( &aChan == &VBIEven_ || &aChan == &VBIOdd_ );
}

/* Method: PsDevice::OurChannel
 * Purpose: Verifies the channel
 * Input: aChan: VxDVideoChannel &, reference to a channel
 * Output: true if our, false otherwise
 */
bool PsDevice::IsOurChannel( VxDVideoChannel &aChan )
{
   return (bool) (IsVideoChannel( aChan ) || IsVBIChannel( aChan ));
}

/* Method::PsDevice::FlushQue
 * Purpose: Called when capture driver recieves RESET command
 * Input: VidChan: VxDVideoChannel & - reference to a video channel
 * Output: None
 */
void PsDevice::FlushQue( VxDVideoChannel &VidChan )
{
   if ( !IsOurChannel( VidChan ) )
      return;
   // [bugbug] We must complete the VIDEOHDRS and send them to ring 3 instead of 
   // just dumping them as we currently do.
//   VidChan.GetCurrentQue().Flush();
   CaptureContrll_->FlushR3VidInQueue() ;
}

/* Method: PsDevice::PM_API_Entry
 * Purpose: This is the main entry point for the VxD. 16 bit capture driver
 *   communicates with VxD through this function
 * Input: hVM: VMHANDLE - handle of the VM making a call
 *   pRegs: CLIENT_STRUCT * - pointer to the structure with VM's registers
 * Note: calling convention between capture driver and VxD:
 *   AX - function number
 *   SI:DX - pointer to data. Meaning of the data is defined by the
 *   function code in AX
 */
 #define API_DEBUG_LEVEL 1
VOID PsDevice::PM_API_Entry( VMHANDLE hVM, CLIENT_STRUCT* pRegs )
{
   // save for the use for everybody else
   hCurVM_ = hVM;
   PVOID pData = GetData( _clientESI );
   DWORD dwData = _clientEDX;

   VxDVideoChannel &aVidChan = GetVideoChannel( pRegs );

   switch ( _clientAX ) {

   case FNOpenDriver:
      DebugPrint((API_DEBUG_LEVEL,"FNOpenDriver")) ;
      break;

   case FNCloseDriver:
      DebugPrint((API_DEBUG_LEVEL,"FNCloseDriver")) ;
      CaptureContrll_->DeallocVxdFrames() ;
      break;

   case FNVer:
      DebugPrint((API_DEBUG_LEVEL,"FNVer\n")) ;
      _clientAX = 0;
      break;
   case FNOpenChan:
      DebugPrint((API_DEBUG_LEVEL,"FNOpenChan\n")) ;
      OpenChannel( pData );
      break;
   case FNCloseChan:
      DebugPrint((API_DEBUG_LEVEL,"FNCloseChan\n")) ;
      CloseChannel( dwData );
      break;
   case FNStart:
      DebugPrint((API_DEBUG_LEVEL,"FNStart %s\n",&aVidChan == &VideoIn_ ? "VideoIn_" :
                                    &aVidChan == &VideoOut_ ? "VideoOut_" : "??")) ;
      *pSkipCount = 0 ;
      gdwStreamStartTime = Get_System_Time() ;
      _clientAX = (WORD)Start( aVidChan, pData );
      break;
   case FNStop:
      DebugPrint((API_DEBUG_LEVEL,"FNStop %s\n",&aVidChan == &VideoIn_ ? "VideoIn_" :
                                   &aVidChan == &VideoOut_ ? "VideoOut_" : "??")) ;
      pSkipCount = (PDWORD)&dummy;
      Stop( aVidChan );
      break;

   case FNStartCapNotify:
      DebugPrint((API_DEBUG_LEVEL,"FNStartCapNotify\n")) ;
//      if (&aVidChan == &VideoIn_)
//      {
         gSoundSync.why = SS_START ;
         gSoundSync.dwVidStartTime = 0 ;
         gSoundSync.hwnd = *((WORD*)pData);

         // signal sound dll if event has been registered
         if ( gCapEvent ) {
            DebugPrint((0,"FNStart: Pulsing event %x\n",gCapEvent)) ;
            _VWIN32_PulseWin32Event(gCapEvent) ;
         }
//      }
      break ;

   case FNEndCapNotify:
      DebugPrint((API_DEBUG_LEVEL,"FNEndCapNotify\n")) ;
//      if (&aVidChan == &VideoIn_)
  //    {
         gSoundSync.why = SS_STOP ;
         // signal sound dll if event has been registered
         if ( gCapEvent ) {
            DebugPrint((0,"FNStart: Pulsing event %x\n",gCapEvent)) ;
            _VWIN32_PulseWin32Event(gCapEvent) ;
         }
//      }
      break ;


   case FNAddBuf:
      DebugPrint((API_DEBUG_LEVEL,"FNAddBuf\n")) ;
      // We are now getting a pointer to a VideoHdr structure (pData).
      // since this structure has a 16:16 pointer for it's lpData member, we must
      // translate it. We store the translated linear address in  the dwReserved[0] member.
      {
        LPVIDEOHDR lpvh = (LPVIDEOHDR) pData ;
        if (lpvh)
        {
           lpvh->dwReserved[0] = (DWORD) GetData((DWORD) lpvh->lpData) ;
           lpvh->dwReserved[1] = (DWORD) _clientESI ; // save 16:16 addr for notify later
           lpvh->dwReserved[2] = 0L ;
           lpvh->dwReserved[3] = 0L ;
           DebugPrint((0,"Adding buffer 16:16 %x xlated %x from lpvh %x\n",lpvh->lpData,lpvh->dwReserved[0],lpvh)) ;
           AddBuffer( aVidChan, lpvh );
        }
        else {
           DebugPrint((-1,"Attempt to add a null LPVH\n",lpvh)) ;
        }
      }
      break;

   case FNAddBufLin:
      DebugPrint((API_DEBUG_LEVEL,"FNAddBufLin\n")) ;
      // No translation of the lpData member since it originated from 32 bit code (ddi.cpp).
      // We still store the value in dwReserved[0] for consistency. 
      {
        LPVIDEOHDR lpvh = (LPVIDEOHDR) pData ;
        if (lpvh)
        {
           lpvh->dwReserved[0] = (DWORD) lpvh->lpData ;
           lpvh->dwReserved[1] = (DWORD) _clientESI ; // save 16:16 addr for notify later
           lpvh->dwReserved[2] = 0L ;
           lpvh->dwReserved[3] = 0L ;
           DebugPrint((0,"Adding buffer Lin %x from lpvh %x\n",lpvh->dwReserved[0],lpvh)) ;
           AddBuffer( aVidChan, lpvh );
        }
        else {
           DebugPrint((-1,"Attempt to add (lin) a null LPVH\n",lpvh)) ;
        }
      }
      break;


   case FNClip:
      DebugPrint((API_DEBUG_LEVEL,"FNClip\n")) ;
   {
      // save new view list pointer
      pViewList = (RGNDATA*)dwData ; // DD3 "clip" list

      DWORD ViewListSizeNeeded = sizeof(RGNDATAHEADER) + pViewList->rdh.nCount * sizeof(RECT) ;
      if (cbViewListBuf < ViewListSizeNeeded) 
      {
        if (viewListBuf)
           delete [] viewListBuf;
        viewListBuf = (DWORD *) new DWORD[ViewListSizeNeeded] ;
        cbViewListBuf = ViewListSizeNeeded ;
      }
      DebugAssert((bool(viewListBuf !=NULL),"Bad alloc of ViewListBuf")) ;
      memcpy(viewListBuf,(void *)dwData,ViewListSizeNeeded) ;

      pViewList = (RGNDATA*)viewListBuf ; // DD3 "clip" list now stored in our buffer

      DumpList("View (before)", pViewList) ;
      if(lastMode == FULL_SCREEN && !bQuartzSightMode_) 
      {
         AdjustRegionForFullScreen(*pViewList) ;
         DumpList("View (after)", pViewList) ;
      }
      else
         DebugPrint((1,"Not Full Screen")) ;

      // if CC List is present
      if (pClipList)
      {
//         if (pCombinedList)
//            delete [] pCombinedList ;

         // Combine Lists : Note that *pClipList has already been adjusted
         // for full screen (if applicable) at this point

         pCombinedList = CombineLists(pCombinedList, *pViewList, *pClipList) ;

         DebugPrint((1,"Clipping with combined list\n")) ;

         DumpList("Combined", pCombinedList) ;

         // send result to RiscEng
         if (pCombinedList) 
            SetClipping( aVidChan, *pCombinedList );
         else
            SetClipping( aVidChan, *pViewList );
      }
      else
      {
         DebugPrint((1,"Clipping with view list\n")) ;
         // send view list to RiscEng
         if (bQuartzSightMode_)       // (Don't worry about CC Clipping (above) for QuartzSight)
         {
           SetClipping( VideoIn_, *pViewList );
           SetClipping( VideoOut_, *pViewList );
         }
         else
         {
           SetClipping( aVidChan, *pViewList );
         }
      }
   }
      break;

   case FNCCClip:
      DebugPrint((API_DEBUG_LEVEL,"FNCCClip\n")) ;
      {
#if 0
//[bugbug] disabled primary surface mode CC  (no more clipping)
         DebugPrint((1,"FN CC Clip\n")) ;
         pClipList = (RGNDATA*)dwData ; // DD3 "clip" list
         if (pClipList)
         {
            DWORD ClipListSizeNeeded = sizeof(RGNDATAHEADER) + pClipList->rdh.nCount * sizeof(RECT) ;
            if (cbClipListBuf < ClipListSizeNeeded) 
            {
              if (clipListBuf)
                 delete [] clipListBuf;
              clipListBuf = (DWORD *) new DWORD[ClipListSizeNeeded] ;
              cbClipListBuf = ClipListSizeNeeded ;
            }
            DebugAssert((bool(clipListBuf!=NULL),"Bad alloc of ClipListBuf")) ;
            memcpy(clipListBuf,(void *)dwData,ClipListSizeNeeded) ;

            pClipList = (RGNDATA*)clipListBuf ; // DD3 "clip" list
         }

//         break ;

         if (pViewList)
         {
//            if (pCombinedList)
//               delete [] pCombinedList ;

            RISCEng & rEng =CaptureContrll_->GetRiscEngine() ;
            CProgArray & pa = CaptureContrll_->GetCreatedProgs () ;

            if (pClipList)
            {
              // Combine Lists
              //DumpList("View", pViewList) ;
              //DumpList("Clip", pClipList) ;

              // If we're in full screen overlay (not capturing), we need
              // to adjust the incoming rectangles
              if(FullSizeChannel_ && !bQuartzSightMode_) 
                AdjustRegionForFullScreen(*pClipList) ;

              //DumpList("Combined (before)", pCombinedList) ;
              pCombinedList = CombineLists(pCombinedList, *pViewList, *pClipList) ;
              //DumpList("Combined (after)", pCombinedList) ;

              DebugPrint((1,"Clipping with combined list\n")) ;
              // send result to RiscEng
              if (pCombinedList)
              {
                 DO_CLIP_PAIR(1,7,pCombinedList) ;
                 DO_CLIP_PAIR(4,10,pCombinedList) ;
              }
              else
              {
                 DO_CLIP_PAIR(1,7,pViewList) ;
                 DO_CLIP_PAIR(4,10,pViewList) ;
              }
            } 
            else
            {
              DebugPrint((1,"Clipping with default view list\n")) ;

              DO_CLIP_PAIR(1,7,pViewList) ;
              DO_CLIP_PAIR(4,10,pViewList) ;
            }
         }
#endif
      }
      break;
   case FNFlushQue:
      DebugPrint((API_DEBUG_LEVEL,"FNFlushQue %s\n",&aVidChan == &VideoIn_ ? "VideoIn_" :
                                   &aVidChan == &VideoOut_ ? "VideoOut_" : "??")) ;

      FlushQue( aVidChan );
      break;
   case FNSkipCount:
      DebugPrint((API_DEBUG_LEVEL,"FNSkipCount\n")) ;
      pSkipCount = (PDWORD)pData;
      *pSkipCount = 0;
      break;
   case FNStopFlag:
      DebugPrint((API_DEBUG_LEVEL,"FNStopFlag\n")) ;
      pStopFlag = (PDWORD)pData;
      break;
   case FNGetI2C:
      DebugPrint((API_DEBUG_LEVEL,"FNGetI2C\n")) ;
      *(PDWORD)pData = DWORD( RegBase::GetBaseAddress() + I2C_Offset );
      break;
   case FNGetGPIO:
      DebugPrint((API_DEBUG_LEVEL,"FNGetGPIO\n")) ;
      *(PDWORD)pData = DWORD( RegBase::GetBaseAddress() + GPIO_Cntl_Offset );
      *( (PDWORD)pData + 1 ) = DWORD( RegBase::GetBaseAddress() + GPIO_OutputOffset );
      *( (PDWORD)pData + 2 ) = DWORD( RegBase::GetBaseAddress() + GPIO_DataOffset );
      break;
   case FNGetBaseAddr:
      DebugPrint((API_DEBUG_LEVEL,"FNGetBaseAddr\n")) ;
      *(PDWORD)pData = DWORD( RegBase::GetBaseAddress() );
      _clientAX = 0;
      break;
   case FNStatus:
      DebugPrint((API_DEBUG_LEVEL,"FNStatus\n")) ;
      *(PDWORD)pData = *(PDWORD) RegBase::GetBaseAddress();
      break;
   case FNPause:
      DebugPrint((API_DEBUG_LEVEL,"FNPause\n")) ;
      PauseCapture();
      break;
   case FNContinue:
      DebugPrint((API_DEBUG_LEVEL,"FNContinue\n")) ;
      RestartCapture();
      break;
   case FNDecoder:
      DebugPrint((API_DEBUG_LEVEL+1,"FNDecoder\n")) ;
      _clientAX = Fail;
      if ( !IsVideoChannel( aVidChan ) )
         return;
      switch ( dwData ) {
      case DecSat:
         SetSaturation( aVidChan, pData );
         break;
      case DecHue:
         SetHue( aVidChan, pData );
         break;
      case DecBrt:
         SetBrightness( aVidChan, pData );
         break;
      case DecSVid:
         SetSVideo( aVidChan, pData );
         break;
      case DecContr:
         SetContrast( aVidChan, pData );
         break;
      case DecFmt:
         SetFormat( aVidChan, pData );
         VBI_Change_Processor( VIDEO_FORMAT_CHANGE, *(PDWORD)pData ) ;
         break;
      case DecConn:
         SetConnector( aVidChan, pData );
         break;
      case DecSigFmt:
         SetSignalFormat( aVidChan, pData );
         break;
      case DecSyncRef:
         SetSyncRef( aVidChan, pData );
         break;
      case DecSyncOutput:
         SetSyncOutput( aVidChan, pData );
         break;
      case DecHReset:
         SetHReset( aVidChan, pData );
         break;
      case DecLineCount:
         SetTotalLineCount( pData );
         break;
      }
      _clientAX = Success;
      break;
   case FNScaler:
      DebugPrint((API_DEBUG_LEVEL+1,"FNScaler\n")) ;
      SetVideoInfo( pData );
      break;
   case FNPLL:
      DebugPrint((API_DEBUG_LEVEL+1,"FNPLL\n")) ;
      SetPLL( pData );
      break;
   case FNTimGenLB:
      DebugPrint((API_DEBUG_LEVEL+1,"FNTimGenLB\n")) ;
      SetTimGenLoadByte( pData, dwData );
      break;
   case FNTimGenInClk:
      DebugPrint((API_DEBUG_LEVEL+1,"FNTimGenInClk\n")) ;
      SetDecInputClock( pData );
      break;
   case FNTimGenOutClk:
      DebugPrint((API_DEBUG_LEVEL+1,"FNTimGenOutClk\n")) ;
      SetGPCLKOutputClock( pData );
      break;
   case FNFldCntr:
      DebugPrint((API_DEBUG_LEVEL+1,"FNFldCntr\n")) ;
      switch ( dwData )
      {
         case SetFunc:
            ResetFieldCounter();
            break;
         case GetFunc:
            GetFieldCounter( pData );
            break;
      }

    case FNSetInterlaceMode :
      DebugPrint((API_DEBUG_LEVEL,"FNSetInterlaceMode\n")) ;
      {
         SetVScaleMode( dwData );

         //VRegistryKey vkey( PRK_CLASSES_ROOT, "Bt848" );
         //VRegistryKey vkey( PRK_LOCAL_MACHINE, "SOFTWARE\\Brooktree\\Bt848" );
         //DWORD nRetrieved ;
         //char tbuf[20] ;
         //if ( vkey.getSubkeyValue( "_TrueView", tbuf, (DWORD *)&nRetrieved ) && tbuf[0] == '1') {

            // dwData == true  --> interlace is ON,  TrueView then Off
            //        == false --> interlace is Off, TrueView then On
            bQuartzSightMode_ = dwData ? false : true;
            DebugPrint((0,"*** SetQuartzSight [%x]\n",bQuartzSightMode_)) ;
         //}
         //else
         //   DebugPrint((0,"*** SetQuartzSight Disallowed\n")) ;
      }
      break ;

   case FNGetR3BufCount:
      DebugPrint((API_DEBUG_LEVEL,"FNGetR3BufCount\n")) ;
      {
         extern BufferInstrumentation gBufInstrumentation ;
         *(PDWORD)pData = *((DWORD *)gBufInstrumentation.pR3QueueCount);
         _clientAX = 0;
      }
      break;

   }
}

/* Method: PsDevice::OpenChannel
 * Purpose: This function opens a channel requested by the capture driver
 * Input: hVM: VMHANDLE - handle of the VM making a call
 *   pRegs: CLIENT_STRUCT * - pointer to the structure with VM's registers
 * Output: None
 */
void PsDevice::OpenChannel( PVOID pData )
{
   // get the address of the structure first. It is in the SI:DX pair
   ChanInfo *parameters = (ChanInfo*)pData;

   PDWORD ChannAddr = (PDWORD)GetData( (DWORD)parameters->lpHandleAddress );

   // fail initially
   *ChannAddr = NULL;

   if ( !VideoIn_.IsOpen() )
      VideoIn_.OpenChannel();
   if ( !VideoOut_.IsOpen() )
      VideoOut_.OpenChannel();

   switch ( parameters->VidChan ) {
   case VS_Capture:
      if ( VideoIn_.IsOpen() )
         *ChannAddr = (DWORD)&VideoIn_;
      break;
   case VS_Preview:
      if ( VideoOut_.IsOpen() )
         *ChannAddr = (DWORD)&VideoOut_;
      break;
   case VS_VBIE:
      if ( VBIEven_.OpenChannel() == Success )
         *ChannAddr = (DWORD)&VBIEven_;
      break;
   case VS_VBIO:
      if ( VBIOdd_.OpenChannel() == Success )
         *ChannAddr = (DWORD)&VBIOdd_;
      break;
   }
}

/* Method: PsDevice::CloseChannel
 * Purpose: This function opens a channel requested by the capture driver
 * Input: hVM: VMHANDLE - handle of the VM making a call
 *   pRegs: CLIENT_STRUCT * - pointer to the structure with VM's registers
 * Output: None
 */
void PsDevice::CloseChannel( DWORD dwData )
{
   // get the address of the channel. It is in the SI:DX pair
   VxDVideoChannel *ToClose = (VxDVideoChannel*)dwData;

   if ( !IsOurChannel( *ToClose ) )
      return;
   ToClose->CloseChannel();

   // this is the easiest way to prevent all kinds of problems. don't want to add
   // yet another VxD function and even more code.
   if ( ToClose == &VideoOut_ ) 
      pStopFlag = &dummy;
}

/* Method:
 * Purpose:
 * Input:
 * Output: None
 */
void PsDevice::SetSaturation( VxDVideoChannel &aVidChan, PVOID pData )
{
   VxDVideoChannel::GetDigitizer().SetSaturation( *(PDWORD)pData );
}

/* Method:
 * Purpose:
 * Input:
 * Output: None
 */
void PsDevice::SetHue( VxDVideoChannel &aVidChan, PVOID pData )
{
   VxDVideoChannel::GetDigitizer().SetHue( *(PDWORD)pData );
}

/* Method:
 * Purpose:
 * Input:
 * Output: None
 */
void PsDevice::SetBrightness( VxDVideoChannel &aVidChan, PVOID pData )
{
   VxDVideoChannel::GetDigitizer().SetBrightness( *(PDWORD)pData );
}

/* Method:
 * Purpose:
 * Input:
 * Output: None
 */
void PsDevice::SetSVideo( VxDVideoChannel &aVidChan, PVOID pData )
{
   VxDVideoChannel::GetDigitizer().SetSVideo( *(PDWORD)pData );
}

/* Method:
 * Purpose:
 * Input:
 * Output: None
 */
void PsDevice::SetContrast( VxDVideoChannel &aVidChan, PVOID pData )
{
   VxDVideoChannel::GetDigitizer().SetContrast( *(PDWORD)pData );
}

/* Method:
 * Purpose:
 * Input:
 * Output: None
 */
void PsDevice::SetFormat( VxDVideoChannel &aVidChan, PVOID pData )
{
   VxDVideoChannel::GetDigitizer().SetFormat( *(PDWORD)pData );
}

/* Method:
 * Purpose:
 * Input:
 * Output: None
 */
void PsDevice::SetConnector( VxDVideoChannel &aVidChan, PVOID pData )
{
   VxDVideoChannel::GetDigitizer().SetConnector( *(PDWORD)pData );
}

/* Method:
 * Purpose:
 * Input:
 * Output: None
 */
void PsDevice::SetVScaleMode( DWORD dwData )
{
   VxDVideoChannel::GetDigitizer().SetVScaleMode( dwData );
}

/* Method:
 * Purpose:
 * Input: pData: PDWORD
 * Output: None
 */
void PsDevice::GetSaturation( PDWORD pData )
{
   CaptureContrll_->GetSaturation( pData );
}

/* Method:
 * Purpose:
 * Input: pData: PDWORD
 * Output: None
 */
void PsDevice::GetHue( PDWORD pData )
{
   CaptureContrll_->GetHue( pData );
}

/* Method:
 * Purpose:
 * Input: pData: PDWORD
 * Output: None
 */
void PsDevice::GetBrightness( PDWORD pData )
{
   CaptureContrll_->GetBrightness( pData );
}

/* Method:
 * Purpose:
 * Input: pData: PDWORD
 * Output: None
 */
void PsDevice::GetSVideo( PDWORD pData )
{
   CaptureContrll_->GetSVideo( pData );
}

/* Method:
 * Purpose:
 * Input: pData: PDWORD
 * Output: None
 */
void PsDevice::GetContrast( PDWORD pData )
{
   CaptureContrll_->GetContrast( pData );
}

/* Method:
 * Purpose:
 * Input: pData: PDWORD
 * Output: None
 */
void PsDevice::GetFormat( PDWORD pData )
{
   CaptureContrll_->GetFormat( pData );
}

/* Method:
 * Purpose:
 * Input: pData: PDWORD
 * Output: None
 */
void PsDevice::GetConnector( PDWORD pData )
{
   CaptureContrll_->GetConnector( pData );
}

/* Method:
 * Purpose:
 * Input: pData: PDWORD
 * Output: None
 */
void PsDevice::GetVScaleMode( PDWORD pData )
{
   CaptureContrll_->GetVScaleMode( pData );
}

void PsDevice::OnKernel32Shutdown()
{
   DebugPrint((0, "OnKernel32Shutdown\n" ));
   Finish() ;
}

/* Method: PsDevice::Finish
 * Purpose: Shuts everything down
*/
void PsDevice::Finish()
{
   if ( !IsInited() )
      return;
//RL   Unhook_Device_Service_Ex( __VDD_Set_VMType, FullScreenChecker, &FullScreenThunk );

   if ( VideoIn_.IsOpen() )
      VideoIn_.CloseChannel();
   if ( VideoOut_.IsOpen() )
      VideoOut_.CloseChannel();
   if ( VBIEven_.IsOpen() )
      VBIEven_.CloseChannel();
   if ( VBIOdd_.IsOpen() )
      VBIOdd_.CloseChannel();

   delete CaptureContrll_;
   CaptureContrll_ = NULL;

   delete QuartzSightVP_;
   QuartzSightVP_ = NULL;

   if (HWInt) {
      HWInt->unhook();
      delete HWInt;
      HWInt = NULL;
   }

   if (viewListBuf)
     delete [] viewListBuf ;
   if (clipListBuf)
     delete [] clipListBuf ;
}

/* Method: PsDevice::BT848_Get_Version
 * Purpose: Returns the driver version
 */
DWORD _cdecl PsDevice::BT848_Get_Version()
{
   return ( BT848_Major << 16 ) | BT848_Minor;
}

/* Method: PsDevice::BT848_Set_VSYNC_Callback
 * Purpose: API to set the ring 0 VSYNC interrupt callback
 */
void  _cdecl PsDevice::BT848_Set_VSYNC_Callback( DWORD cb )
{
   BtPisces::SetVSYNCCB( cb );
}

/* Method: PsDevice::BT848_Set_GPIO_Callback
 * Purpose: API to set the ring 0 GPIO interrupt callback
 */
void  _cdecl PsDevice::BT848_Set_GPIO_Callback( DWORD cb )
{
   BtPisces::SetGPIOCB( cb );
}

/* Method: PsDevice::BT848_Set_I2C_Callback
 * Purpose: API to set the the ring 0 I2C interrupt callback
 */
void  _cdecl PsDevice::BT848_Set_I2C_Callback( DWORD cb )
{
   BtPisces::SetI2CCB( cb );
}

/* Method: PsDevice::Bt848_Get_PCI_Config_Header
 * Purpose: API to fetch PCI header acquired during PnP time
 * Returns pointer to a copy of the PCI Configuration Header (in EAX)
 */
PCI_CONFIG_848 *  _cdecl PsDevice::Bt848_Get_PCI_Config_Header()
{
   return &pciCfgHdr_ ;
}

//// return timeout value in microsec
typedef enum {VVL_WRITE, VVL_READ} TimeoutType ;
DWORD GetTimeoutValue( TimeoutType t )
{
  static DWORD readTimeout = 0 ;
  static DWORD writeTimeout = 0 ;
  DWORD ret = 600 ;

  switch (t)
  {
     case VVL_WRITE :
        if (writeTimeout)
           ret = writeTimeout ;
        else
        {
          VRegistryKey vkey( PRK_LOCAL_MACHINE, "SOFTWARE\\Brooktree\\Video\\QuartzSight" );
          // make sure the key exists
          if ( vkey.lastError() == ERROR_SUCCESS )
          {
             char * keyName = "I2CWriteTimeout_uS";
             char   key[9];
             DWORD  keyLen = 8;    // up to 8 chars + null

             // get the registry value and verify
             if ( vkey.getSubkeyValue( keyName, key, (DWORD *)&keyLen ) )
             {
                ret = (DWORD)atol( key );
                DebugPrint((-1,">>> Got %d for I2C Write Timeout\n",ret)) ;
             }
          }
        }
        writeTimeout = ret ;
     break ;

     case VVL_READ :
        if (readTimeout)
           ret = readTimeout ;
        else
        {
          VRegistryKey vkey( PRK_LOCAL_MACHINE, "SOFTWARE\\Brooktree\\Video\\QuartzSight" );
          // make sure the key exists
          if ( vkey.lastError() == ERROR_SUCCESS )
          {
             char * keyName = "I2CReadTimeout_uS";
             char   key[9];
             DWORD  keyLen = 8;    // up to 8 chars + null

             // get the registry value and verify
             if ( vkey.getSubkeyValue( keyName, key, (DWORD *)&keyLen ) )
             {
                ret = (DWORD)atol( key );
                DebugPrint((-1,">>> Got %d for I2C Read Timeout\n",ret)) ;
             }
          }
        }
        readTimeout = ret ;
     break ;
  }

  return ret ;
}

/* Method: WaitForI2C( DWORD * pIsr, DWORD timeout )
 * Purpose: Wait until I2C bit is set or timeout expires
 */
BOOL WaitForI2C( DWORD * pIsr, DWORD timeout )
{
   DWORD hiStart, loStart, hiNow, loNow;
   
   VTD_Get_Real_Time( &hiStart, &loStart );
   while( 1 )
   {
      if ( *pIsr & BIT(8) )   // I2C done bit set?
         return TRUE;

      VTD_Get_Real_Time( &hiNow, &loNow );

      if ( loStart > loNow )   // current loword rollover?
         loStart = loNow;

      // VTD_Get_Real_Time() returns current system time based on an unit of 0.8 microseconds
      if ( loNow - loStart > timeout * 5 / 4 )   // timeout expires?
      {
DebugPrint((0,">>>>>>>>>> WaitForI2C Timeout=%ldus elapsed=%ldus\n", timeout, (loNow - loStart)*4/5));
         return FALSE;
      }
   }
}      

/* Method: PsDevice::BT848_VVL_Read_Byte(BYTE reg)
 * Purpose: Read byte from Scotch
 * Return:  Byte read if success or 0xFFFF if fail
 */
WORD  _cdecl PsDevice::BT848_VVL_Read_Byte(BYTE reg)
{
// Ok cowboys: we're gonna control the i2c directly since:
//  a) we don't want to do major surgery and bring ring 3 i2c code to ring 0
//  b) this is time critical stuff
//  c) existing i2c code is not compatible with scotch
  WORD st = 0 ;
  const BYTE cameraAddr   = 0x88;
  const WORD ERROR        = 0xFFFF;
  DWORD timeout =  GetTimeoutValue(VVL_READ);   // I2C timeout period (in microsec)

  DWORD * pI2c = (DWORD *) DWORD( RegBase::GetBaseAddress() + I2C_Offset );
  DWORD * pIsr = (DWORD *) DWORD( RegBase::GetBaseAddress() + 0x100 );
  DebugPrint((1,"VVL Read : pI2c = %x  pIsr = %x\n",pI2c,pIsr)) ;

  //
  // Set up 2 byte write to set subaddress pointer
  //
  DWORD i2c_data  = ((cameraAddr ) << 24) ; // set camera addr in high order byte
  i2c_data |= (reg << 16) ;           // set register addr in next byte
  i2c_data |= (0x8b) ;                // control: 2 byte write

  *pIsr = BIT(8);           // clear I2c done bit
  *pI2c =  i2c_data ;       // write the data
  if ( !WaitForI2C( pIsr, timeout ) )    // wait for i2c done bit
    return ERROR;

  //
  //
  //
  i2c_data |= 0x01000000 ;  // Set read bit
  i2c_data &= 0xffffff00 ;  // nuke old control
  i2c_data |= 0xaf ;        // control:no stop
  i2c_data |= 0x00ffff00 ;  // ??

  *pIsr = BIT(8);           // clear I2c done bit
  *pI2c =  i2c_data ;       // write the data
  if ( !WaitForI2C( pIsr, timeout ) )    // wait for i2c done bit
    return ERROR;

  //
  //
  //
  i2c_data &= 0xffffff00 ;  // nuke old control
  i2c_data |= 0x9b ;        // control: 1 byte read without start

  *pIsr = BIT(8);           // clear I2c done bit
  *pI2c =  i2c_data ;       // write the data
  if ( !WaitForI2C( pIsr, timeout ) )    // wait for i2c done bit
    return ERROR;

  //
  // Extract the data
  //
  DWORD regVal = *pI2c ;  // Byte 2 is what we want

  DebugPrint((1,"VVL Read = %x\n",regVal)) ;

  return st | (WORD)((regVal >> 8) & 0xff ) ;
}

/* Method: PsDevice::BT848_VVL_Write_Byte(BYTE reg, BYTE val)
 * Purpose: Write byte to Scotch
 * Return:  0 if success or 0xFFFF if fail
 */
WORD  _cdecl PsDevice::BT848_VVL_Write_Byte(BYTE reg, BYTE val)
{
  const WORD ERROR = 0xFFFF;
  DWORD timeout = GetTimeoutValue(VVL_WRITE);   // I2C timeout period (in microsec)
  DWORD * pI2c = (DWORD *) DWORD( RegBase::GetBaseAddress() + I2C_Offset );
  DWORD * pIsr = (DWORD *) DWORD( RegBase::GetBaseAddress() + 0x100 );
  const BYTE cameraAddr   = 0x88;

  //
  // Write the data
  //
  DWORD i2c_data  = ((DWORD)(cameraAddr ) << 24) ; // set camera addr in high order byte
  i2c_data |= ((DWORD) reg << 16) ;          // set register addr in next byte
  i2c_data |= ((DWORD) val << 8) ;           // set val in next byte
  i2c_data |= (0x8F) ;                       // control: 3 byte write

  *pIsr = BIT(8);           // clear I2c done bit
  *pI2c =  i2c_data ;       // write the data
  if ( !WaitForI2C( pIsr, timeout ) ){    // wait for i2c done bit
    DebugPrint((1,"Timeout VVL Write\n")) ;
    return ERROR;
  }

  DebugPrint((1,"End VVL Write\n")) ;
  return 0 ;
}

/****************************************************************************
 * Method:  PsDevice::SetSignalFormat
 * Purpose: Select input signal format
 ***************************************************************************/
void PsDevice::SetSignalFormat( VxDVideoChannel &aVidChan, PVOID pData )
{
   CaptureContrll_->SetSignalFormat( *(PDWORD)pData );
}

/****************************************************************************
 * Method:  PsDevice::SetSyncRef
 * Purpose: Select sync video reference
 ***************************************************************************/
void PsDevice::SetSyncRef( VxDVideoChannel &aVidChan, PVOID pData )
{
   CaptureContrll_->SetSyncRef( *(PDWORD)pData );
}

/****************************************************************************
 * Method:  PsDevice::SetSyncOutput
 * Purpose: Enable/Disable syncs output for synchronizing video input
 ***************************************************************************/
void PsDevice::SetSyncOutput( VxDVideoChannel &aVidChan, PVOID pData )
{
   CaptureContrll_->SetSyncOutput( *(PDWORD)pData );
}

/****************************************************************************
 * Method:  PsDevice::SetHReset
 * Purpose: Enable/Disable HRESET to be shortened from 64 pixels to 32 pixels
 ***************************************************************************/
void PsDevice::SetHReset( VxDVideoChannel &aVidChan, PVOID pData )
{
   CaptureContrll_->SetHReset( *(PDWORD)pData );
}

/****************************************************************************
 * Method:  PsDevice::SetPLL
 * Purpose: Set PLL to synthesize desired frequency
 ***************************************************************************/
void PsDevice::SetPLL( PVOID pData )
{
   CaptureContrll_->SetPLL( *(PLLInfo *)pData );
}

/****************************************************************************
 * Method:  PsDevice::SetVideoInfo
 * Purpose: Set video size parameters
 ***************************************************************************/
void PsDevice::SetVideoInfo( PVOID pData )
{
   CaptureContrll_->SetVideoInfo( *(VideoInfo *)pData );
}

/****************************************************************************
 * Method:  PsDevice::SetTimGenLoadByte
 * Purpose: Load time generator bytes into SRAM
 ***************************************************************************/
void PsDevice::SetTimGenLoadByte( PVOID pData, DWORD dwData )
{
   CaptureContrll_->SetTimGenLoadByte( (PBYTE)pData, dwData );
}

/****************************************************************************
 * Method:  PsDevice::SetDecInputClock
 * Purpose: Select decoder input clock
 ***************************************************************************/
void PsDevice::SetDecInputClock( PVOID pData )
{
   CaptureContrll_->SetDecInputClock( *(PDWORD)pData );
}

/****************************************************************************
 * Method:  PsDevice::SetGPCLKOutputClock
 * Purpose: Select GPCLK output clock
 ***************************************************************************/
void PsDevice::SetGPCLKOutputClock( PVOID pData )
{
   CaptureContrll_->SetGPCLKOutputClock( *(PDWORD)pData );
}

/****************************************************************************
 * Method:  PsDevice::SetTotalLineCount
 * Purpose: Set expected total line count per frame for the decoder
 ***************************************************************************/
void PsDevice::SetTotalLineCount( PVOID pData )
{
   CaptureContrll_->SetTotalLineCount( *(PDWORD)pData );
}

/****************************************************************************
 * Method:  PsDevice::GetSignalFormat
 * Purpose: Get current video signal format
 ***************************************************************************/
void PsDevice::GetSignalFormat( PVOID pData )
{
   CaptureContrll_->GetSignalFormat( (PDWORD)pData );
}

/****************************************************************************
 * Method:  PsDevice::GetSyncRef
 * Purpose: Get sync video reference
 ***************************************************************************/
void PsDevice::GetSyncRef( PVOID pData )
{
   CaptureContrll_->GetSyncRef( (PDWORD)pData );
}

/****************************************************************************
 * Method:  PsDevice::GetSyncOutput
 * Purpose: Checks if sync output for sync video input is enabled or disabled
 ***************************************************************************/
void PsDevice::GetSyncOutput( PVOID pData )
{
   CaptureContrll_->GetSyncOutput( (PDWORD)pData );
}

/****************************************************************************
 * Method:  PsDevice::GetHReset
 * Purpose: Checks if HRESET is shortened from 64 to 32 pixels
 **************************************************************************/
void PsDevice::GetHReset( PVOID pData )
{
   CaptureContrll_->GetHReset( (PDWORD)pData );
}

/****************************************************************************
 * Method:  PsDevice::GetPLL
 * Purpose: Obtains the settings of PLL registers
 ***************************************************************************/
void PsDevice::GetPLL( PVOID pData )
{
   CaptureContrll_->GetPLL( (PLLInfo *)pData );
}

/****************************************************************************
 * Method:  PsDevice::GetVideoInfo
 * Purpose: Get video size parameters
 ***************************************************************************/
void PsDevice::GetVideoInfo( PVOID pData )
{
   CaptureContrll_->GetVideoInfo( (VideoInfo *)pData );
}

/****************************************************************************
 * Method:  PsDevice::GetTimGenLoadByte
 * Purpose: Get timing generator load byte
 ***************************************************************************/
void PsDevice::GetTimGenLoadByte( PVOID pData, PDWORD pdwData )
{
   CaptureContrll_->GetTimGenLoadByte( (PBYTE)pData, pdwData );
}

/****************************************************************************
 * Method:  PsDevice::GetDecInputClock
 * Purpose: Get decoder input clock
 ***************************************************************************/
void PsDevice::GetDecInputClock( PVOID pData )
{
   CaptureContrll_->GetDecInputClock( (PDWORD)pData );
}

/****************************************************************************
 * Method:  PsDevice::GetGPCLKOutputClock
 * Purpose: Get GPCLK output clock
 ***************************************************************************/
void PsDevice::GetGPCLKOutputClock( PVOID pData )
{
   CaptureContrll_->GetGPCLKOutputClock( (PDWORD)pData );
}

/****************************************************************************
 * Method:  PsDevice::GetTotalLineCount
 * Purpose: Get expected total line count per frame for the decoder
 ***************************************************************************/
void PsDevice::GetTotalLineCount( PVOID pData )
{
   CaptureContrll_->GetTotalLineCount( (PDWORD)pData );
}

/****************************************************************************
 * Method:  PsDevice::ResetFieldCounter
 * Purpose: Reset field counter
 ***************************************************************************/
void PsDevice::ResetFieldCounter( void )
{
   CaptureContrll_->ResetFieldCounter();
}

/****************************************************************************
 * Method:  PsDevice::GetFieldCounter
 * Purpose: Obtain field counter
 ***************************************************************************/
void PsDevice::GetFieldCounter( PVOID pData )
{
   CaptureContrll_->GetFieldCounter( (PDWORD)pData );
}

// Function
//  dgets - get a string from the debug console
//
// Input
//  buf   buffer to receive string
//  maxchar   number of characters not including terminating nul that
//      buf can accommodate
//
// Returns
//  Returns the number of characters read
//
// Remarks
//  Input is terminated by a CR 
//
int dgets(char* buf, int maxchar)
{
  WORD ch;
  int  i;

  for (i=0; i < maxchar; i++, buf++)
  {
    ch  = In_Debug_Chr() & (WORD) 0x00ff;
    if ( ((ch & 0xff) == 13) || ((ch & 0xff00) == 0xff00) )
      break;
    else  
    {
      *buf = ch;
      Out_Debug_Chr((BYTE)(ch & (WORD) 0x00ff));
    }
  }

  *buf = 0;
  return i;
}


BOOL validDebugLevel(BYTE ch)
{
  switch (ch) {
     case '0' :
     case '1' :
     case '2' :
     case '3' :
     case '4' :
     case 'X' :
     case 'x' :
       return TRUE ;
     default:
       return FALSE ;
  }
}

VOID PsDevice::OnDebugQuery()
{
  WORD ch;
  BOOL done = FALSE ;
  extern InterruptCounter gIntrCntr ;


  DebugPrint((-1,"******************\n")) ;
  DebugPrint((-1,"* Bt Video Debug *\n")) ;
  DebugPrint((-1,"******************\n")) ;

  while (!done)
  {
    DebugPrint((-1,"\nSelect Option:\n")) ;
    DebugPrint((-1,"   [A] Dump Interrupt counts\n")) ;
    DebugPrint((-1,"   [B] Set Debug Level\n")) ;
    DebugPrint((-1,"   [C] Show RiscProg Arrays\n")) ;
    DebugPrint((-1,"   [Q] Exit:\n")) ;

    ch  = In_Debug_Chr() & (WORD) 0x00ff;
    switch ((BYTE) ch)
    {
       case 'Q': case 'q':
         done = TRUE ;
         break ;

       case 'A': case 'a':
         DebugPrint((-1,"Interrupt count dump:\n")) ;
         DebugPrint((-1,"   dwRisc = %d\n",gIntrCntr.dwRisc));
         DebugPrint((-1,"   dwGpio = %d\n",gIntrCntr.dwGpio));
         DebugPrint((-1,"   dwI2c = %d\n",gIntrCntr.dwI2c));
         DebugPrint((-1,"   dwFbus = %d\n",gIntrCntr.dwFbus));
         DebugPrint((-1,"   dwFtrgt = %d\n",gIntrCntr.dwFtrgt));
         DebugPrint((-1,"   dwFdsr = %d\n",gIntrCntr.dwFdsr));
         DebugPrint((-1,"   dwPperr = %d\n",gIntrCntr.dwPperr));
         DebugPrint((-1,"   dwRiperr = %d\n",gIntrCntr.dwRiperr));
         DebugPrint((-1,"   dwPabort = %d\n",gIntrCntr.dwPabort));
         DebugPrint((-1,"   dwOcerr = %d\n",gIntrCntr.dwOcerr));
         DebugPrint((-1,"   dwScerr = %d\n",gIntrCntr.dwScerr));
         DebugPrint((-1,"   dwVsync = %d\n",gIntrCntr.dwVsync));
         DebugPrint((-1,"\nClear?\n"));
         ch  = In_Debug_Chr() & (WORD) 0xff;
         if (ch == 'y' || ch == 'Y')
         {
            gIntrCntr.dwRisc    =  gIntrCntr.dwGpio    =  gIntrCntr.dwI2c     =  gIntrCntr.dwFbus    =
            gIntrCntr.dwFtrgt   =  gIntrCntr.dwFdsr    =  gIntrCntr.dwPperr   =  gIntrCntr.dwRiperr  =
            gIntrCntr.dwPabort  =  gIntrCntr.dwOcerr   =  gIntrCntr.dwScerr   =  gIntrCntr.dwVsync   = 0 ;
            DebugPrint((-1,"Cleared Interrupt counts.\n"));
         }

         break;

       case 'B': case 'b':
         DebugPrint((-1,"Enter Debug Level (0-4 or X to turn off): ")) ;
         ch = -1 ;  // invalid
         while (!validDebugLevel(ch))
           ch  = In_Debug_Chr() & (WORD) 0xff;

         DebugPrint((-1,"\n")) ;
         switch((BYTE)ch)
         {
            case '0': DebugLevel = 0 ; DebugPrint((-1,"Setting debug level %d\n",'0' - (BYTE) ch)) ;break ;
            case '1': DebugLevel = 1 ; DebugPrint((-1,"Setting debug level %d\n",'0' - (BYTE) ch)) ;break ;
            case '2': DebugLevel = 2 ; DebugPrint((-1,"Setting debug level %d\n",'0' - (BYTE) ch)) ;break ;
            case '3': DebugLevel = 3 ; DebugPrint((-1,"Setting debug level %d\n",'0' - (BYTE) ch)) ;break ;
            case '4': DebugLevel = 4 ; DebugPrint((-1,"Setting debug level %d\n",'0' - (BYTE) ch)) ;break ;
            case 'X': 
            case 'x': DebugLevel = -1 ; DebugPrint((-1,"Turning DEBUG OFF\n")) ;break ;
         }
         break ;

       case 'C': case 'c':
         {
            DebugPrint((-1,"Created\n"));
            for (int i = 0 ; i < 6 ; i ++)
              DebugPrint((-1,"   [%d] = %x    [%d] = %x\n",i,
                                                           CaptureContrll_->CreatedProgs_[i],
                                                           i+6,
                                                           CaptureContrll_->CreatedProgs_[i+6]));
              
            DebugPrint((-1,"Active\n"));
            for ( i = 0 ; i < 6 ; i ++)
              DebugPrint((-1,"   [%d] = %x    [%d] = %x\n",i,
                                                           CaptureContrll_->ActiveProgs_[i],
                                                           i+6,
                                                           CaptureContrll_->ActiveProgs_[i+6]));
              
         }
         break ;
    }
  }
}
