/* File: Vbi.cpp
 * Purpose: Contains implementation of the interface to the Intel VBI decoder VxD
 * Author: sasha
 * $Log: vbi.cpp $
 * Revision 2.15  1998/06/03 03:28:19  dtharp
 * Initialized dwReserved[0] member of ring0 allocated 
 * videoheaders. This is the 'ring 0' address of the data buffer, and
 * it was not being initialized correctly.
 * Revision 2.14  1998/06/01 18:08:53  mdimeo
 * added rcs ident string
 * Revision 2.13  1998/05/14 23:37:01  dtharp
 * Graft from DT_VBI branch: VBI now functional.
 * Revision 2.12.1.1  1998/05/11 17:28:55  dtharp
 * Branch to restore VBI
 * Revision 2.12  1998/04/06 19:42:21  dtharp
 * Vxd Frame Buffers are allocated on the fly instead of at init time.
 * Revision 2.11  1998/04/02 21:40:04  dtharp
 * Merge DT_VIDEOHDR branch. Implements a new buffer 
 * management scheme.
 * Revision 2.10.2.1  1998/03/23 18:11:51  dtharp
 * Vbi streams now use videoinfohdr
 * Revision 2.10  1997/07/28 23:36:57  dtharp
 * Removed checking for 849 in WST Open Call.
 * Revision 2.9  1997/07/22 15:36:20  dtharp
 * Added FNGetPciHdr functionality via DevIOCtl and VxD Service.
 * Revision 2.8  1997/07/21 23:23:53  dtharp
 * Added some debug prints and a code switch for FAIL when not an
 * 849.
 * Revision 2.7  1997/06/20 20:30:17  dtharp
 * Intel VBI Mods; reworked oversample selection, added call to VBISetMostParams()
 * Revision 2.6  1997/06/17 22:19:46  dtharp
 * Modification for Intel Intercast VBI. 
 * Revision 2.5.1.1  1997/06/10 17:59:26  dtharp
 * Branch for new Buffer mgmnt scheme.
 * Revision 2.5  1997/05/16 16:17:51  dtharp
 * Moved HDelay adjustment to field.h.
 * Revision 2.4  1997/04/25 16:49:44  dtharp
 * Grafted WST Interface branch (2.3.3.1).
 * Revision 2.3.3.1  1997/04/03 17:11:29  dtharp
 * Merged in the DT_VBI_API branch to the current (V1_4_1) stuff.
 * Solved PAL/NTSC coexistance problem.
 * Revision 2.3  1996/06/19 18:14:04  sasha
 * Changed VBIConnect() a bit to circumvent register trashing by the VxDCall()
 * Revision 2.2  1996/06/18 22:50:19  sasha
 * Fixed VBI decoder callback to properly handle channel sharing
 * Revision 2.1  1996/06/10 20:12:51  ssm
 * MarkBeta1AsRev2
 * Revision 1.6  1996/05/21 21:27:38  sasha
 * Changed CallRing3Callback() to call for odd field only
 * Revision 1.5  1996/05/14 23:55:09  ssm
 * move 1.4.1.5 to head
 * Revision 1.4.1.5  1996/05/14 23:40:39  sasha
 * Use new names for the VBI decoder functions. Use new callback conventions
 * Added support for calling back ring3 code.
 * Fixed bug when a channel change was propagated to VBI VxD without
 * checking its presence.
 * Revision 1.4.1.4  1996/05/12 00:31:03  sasha
 * Changed VBIDecoderCallback ( allocate and free buffers in it. Added stub
 * for the VBI interrupt callback which will notify ring 3 code.
 * Revision 1.4.1.3  1996/05/11 02:03:36  sasha
 * Make sure our type of callback from VBI VxD is set ( when starting )
 * Revision 1.4.1.2  1996/05/11 01:56:52  sasha
 * Moved opening of the channels out of the VBIConnect into VBIDecoderCallback.
 * Now try to open channel only when VBI VxD asked for data. Upon stop
 * from VBI VxD close channels. Use interrupt callback pointer in the call to
 * SetAppCallback instead of direct function name ( this lets to have same code
 * be used for 16 bit interface )
 * Revision 1.4  1996/03/27 00:54:12  sasha
 * Removed commented out code. Added VBISetChannel() function
 * Revision 1.3  1996/03/22 01:21:02  sasha
 * Commented out the 'check' function
 * Revision 1.2  1996/03/15 22:43:10  sasha
 * Much closer to the final form
 * Revision 1.1  1996/03/04 22:28:10  sasha
 * Initial revision
 */

static char rcsid[] = "$Id: vbi.cpp 2.15 1998/06/03 03:28:19 dtharp Exp $" ;
#define TASM_AVAILABLE
#include "device.h"
#include "vbidec.h"
#include "dbgprint.h"

const SamplesNTSC = 768;
const SamplesPAL  = 960;

const Lines   = 20;
const BytesPerSample = 2;

enum {PROTOCOL_NABTS, PROTOCOL_WST} CurrentIntercastProtocol = PROTOCOL_NABTS;

#define lbl( x ) } x##: asm {
#define CALL_EAX  asm { db 0xFF; db 0xD0; }

DWORD dwVbiLinesMask = 0x003FFC00; // default lines 10-21

/* Method: PsDevice::VBIConnect
 * Purpose: Establishes the connection to the Intel VBI decoder
 * Input: None
 * Output: ErrorCode
 */
ErrorCode PsDevice::VBIConnect()
{
   VxDCall( ICVBIDEC_Get_Version );
   asm {
      jnc  end
   }
   return Fail;
end:
   // the function call is needed because value of eax is trashed by the VxDCall
   GetPsDevice()->VBIDecoderInited_ = true;
   return Success;
}

/* Method: PsDevice::VBISetMostParams
 * Purpose: Sets various VBI decoder parameters
 * Input: None
 * Output: ErrorCode
 */
ErrorCode PsDevice::VBISetMostParams()
{
   PsDevice *me = GetPsDevice();
   // set the oversample
   //   For WST:
   //      data bit ratio is 6.9375
   //      only valid in PAL
   //      PAL  : sample freq is 35.46895
   //             Oversample is 35.46895/6.9375 = 5.11      = 0x051CD611
   //       
   //   For NABTS:
   //      data bit ratio is 5.7272
   //      NTSC : sample freq is 28.636
   //             Oversample is 28.636/5.7272 = 5.0         = 0x05000000
   //      PAL  : sample freq is 35.46895
   //             Oversample is 35.46895/5.7272 = 6.19306   = 0x06316c61
   //
   // Oversample matrix
   //            NTSC          PAL
   //   WST      N/A           0x051CD611
   //   NABTS    0x05000000    0x06316c61
   DWORD dwOversample ;
   switch (CurrentIntercastProtocol)
   {
      default:
      case PROTOCOL_NABTS:
         if(IsNTSCCompatible(me->CaptureContrll_->GetFormat())) { 
            dwOversample = 0x05000000 ;        // NTSC
         } else {
            dwOversample = 0x06316c61 ;        // PAL
         }
         break ;

      case PROTOCOL_WST:
         dwOversample = 0x051CD611 ;           // WST : PAL Only
         break ;
   }

   asm   mov  eax, dwOversample
   VxDCall( ICVBIDEC_Set_Oversample );
   asm   jnc do_pitch
   return Fail;

do_pitch:
   // We need to provide pitch based on current protocol
   // Possible Combinations:
   // ======================
   // NABTS    Line pitch = 1536  (768*2) or SamplesNTSC * BytesPerSample
   // WST      Line pitch = 1920  (960*2) or SamplesPal * BytesPerSample
   if(CurrentIntercastProtocol == PROTOCOL_NABTS){ // NTSC (cc)
      asm   mov   eax, SamplesNTSC * BytesPerSample
   } else {
      asm   mov   eax, SamplesPAL * BytesPerSample
   }

   VxDCall( ICVBIDEC_Set_Line_Pitch );
   asm   jnc   set_channel
   return Fail;

set_channel:
   asm   mov   eax, 13
   VxDCall( ICVBIDEC_Set_Channel );
   asm   jnc   done
   return Fail;

done:
   return Success ;

}

ErrorCode PsDevice::VBISetParams()
{
  if (VBISetMostParams() == Fail)
     return Fail ;

set_callback:
   // register the callback
   PVOID pCB = VBIDecoderCallback;
   asm   mov  eax, dword ptr pCB

   VxDCall( ICVBIDEC_Register_Callback );
   asm   jc  end
   asm   or  eax, eax
   asm   jz  bye

   StartVBI(VBI_INTERCAST);
bye:
   return Success;
end:
   return Fail;
}

static bool can_check = false;

/* Method: PsDevice::VBIIntrCallback()
 * Purpose: Called by the ISR when VBI program interrupts
 * Input: aStrm: VideoStream
 * Output: None
 */
void PsDevice::VxDVBIIntrCallback( VideoStream aStrm, DWORD dwBufAddr )
{
   if ( !can_check || ChangingChannel_ )
      return; // bah-bahi

   _ECX = aStrm != VS_VBIO;
   asm {
      mov   eax, dword ptr dwBufAddr
      mov   ebx, dwVbiLinesMask
      xor   edx, edx
   }
   VxDCall( ICVBIDEC_Decode_Field );

   asm   jnc  end
   OUTPUT_MESS( "error decoding" );
end:
}

/* Method: PsDevice::GrabVBIChannels
 * Purpose: Tries to allocate VBI channels and sets buffers for them
 * Input: None
 * Output: ErrorCode
 */
ErrorCode PsDevice::GrabVBIChannels()
{
   if ( VBIOdd_.OpenChannel() == Success ) {
      if ( VBIEven_.OpenChannel() != Success ) {
         VBIOdd_.CloseChannel();
         return Success;
      }
   } else
      return Fail; // somebody else has the VBI channels open

   // always allocate just before using
   DWORD dwBufSize = IsNTSCCompatible(CaptureContrll_->GetFormat()) ? 
                        SamplesNTSC * Lines * BytesPerSample :            // NTSC (cc)
                        SamplesPAL * Lines * BytesPerSample ;             // PAL  (ttx)

   #define CLEAR_VIDEO_HDR(p) memset((PVOID)p,0,sizeof(VIDEOHDR)) ; 

   OddVBIBuf_ [0]          = new VIDEOHDR ;
   CLEAR_VIDEO_HDR(OddVBIBuf_ [0]) ;
   OddVBIBuf_ [0]->lpData  = new BYTE [ dwBufSize ];
   OddVBIBuf_ [0]->dwReserved[0] = (DWORD)OddVBIBuf_ [0]->lpData ;

   OddVBIBuf_ [1]          = new VIDEOHDR ;
   CLEAR_VIDEO_HDR(OddVBIBuf_ [1]) ;
   OddVBIBuf_ [1]->lpData  = new BYTE [ dwBufSize ];
   OddVBIBuf_ [1]->dwReserved[0] = (DWORD)OddVBIBuf_ [1]->lpData ;

   EvenVBIBuf_ [0]         = new VIDEOHDR ;
   CLEAR_VIDEO_HDR(EvenVBIBuf_ [0]) ;
   EvenVBIBuf_ [0]->lpData = new BYTE [ dwBufSize ];
   EvenVBIBuf_ [0]->dwReserved[0] = (DWORD)EvenVBIBuf_ [0]->lpData ;

   EvenVBIBuf_ [1]         = new VIDEOHDR ;
   CLEAR_VIDEO_HDR(EvenVBIBuf_ [1]) ;
   EvenVBIBuf_ [1]->lpData = new BYTE [ dwBufSize ];
   EvenVBIBuf_ [1]->dwReserved[0] = (DWORD)EvenVBIBuf_ [1]->lpData ;

   if ( OddVBIBuf_  [0] && OddVBIBuf_ [1] && EvenVBIBuf_ [0] && EvenVBIBuf_ [1] )
      return Success;
   else
      return Fail;
}

/* Method: PsDevice::FreeVBIChannels
 * Purpose: Frees VBI channels
 * Input: None
 * Output: None
 */
void PsDevice::FreeVBIChannels()
{
   VBIEven_.CloseChannel();
   VBIOdd_.CloseChannel();

   // always free after using
   delete [] OddVBIBuf_ [0]->lpData;
   delete    OddVBIBuf_ [0];

   delete [] OddVBIBuf_ [1]->lpData;
   delete    OddVBIBuf_ [1];

   delete [] EvenVBIBuf_ [0]->lpData;
   delete    EvenVBIBuf_ [0];

   delete [] EvenVBIBuf_ [1]->lpData;
   delete    EvenVBIBuf_ [1];

   OddVBIBuf_ [0] = NULL;
   OddVBIBuf_ [1] = NULL;

   EvenVBIBuf_ [0] = NULL;
   EvenVBIBuf_ [1] = NULL;
}

/* Method:
 * Purpose:
 * Input: None
 * Output: None
 */
void PsDevice::VBIDecoderCallback()
{
// what does VBI decoder want ?
// Possible Combinations:
// ======================
// NTSC -> NABTS    Line pitch = 1536  (768*2)
// PAL  -> WST      Line pitch = 1920  (960*2)
// PAL  -> NABTS
// 
// Interface:
// ==========
// We will call the capture driver through the CAP_Callback given to us 
// using Register_Callback with:
//         EBX = 0 for NABTS
//             = 1 for WST
// EAX remains the same (Used for captured VBI lines bitmask)

   dwVbiLinesMask   = _EAX;   // eax contains lines bitmask
   DWORD dwProtocol = _EBX;   // ebx contains protocol code

   PsDevice *me = GetPsDevice();
   if ( dwVbiLinesMask ) { // at least one line to capture
      if ( me->GrabVBIChannels() == Success ) {
         // Set the protocol
         switch (dwProtocol)
         {
            default:
            case 0: CurrentIntercastProtocol = PROTOCOL_NABTS ; break ;
            case 1: CurrentIntercastProtocol = PROTOCOL_WST ; break ;
         }

         VBISetMostParams() ; // Realign Intel's VxD with the new settings

         // opened channels O.K. Can set out callback safely
         me->VBIProcessor_ = VxDVBIIntrCallback;
         StartVBI(VBI_INTERCAST);
      }
   } else {
      if ( me->VBIEven_.IsOpen() && me->VBIOdd_.IsOpen() &&
         // make sure stopping the VBI channels ( intercast, WST, etc ), not ring3 CC
         me->VBIProcessor_ && me->VBIProcessor_ != CapDrvVBIIntrCallback ) {
         StopVBI();
         me->FreeVBIChannels();
      }
   }
}

/* Method: PsDevice::StartVBI
 * Purpose: Starts VBI capture
 * Input: None
 * Output: None
 */
void PsDevice::StartVBI(VBI_INITIATOR_TYPE vbiInitiator)
{
   PsDevice *me = GetPsDevice();

   me->VBIOdd_.AddBuffer ( OddVBIBuf_ [0] );
   me->VBIOdd_.AddBuffer ( OddVBIBuf_ [1] );

   me->VBIEven_.AddBuffer( EvenVBIBuf_ [0] );
   me->VBIEven_.AddBuffer( EvenVBIBuf_ [1] );

   me->VBIOdd_.EnableInterrupt();
   me->VBIEven_.EnableInterrupt();

   me->VBIOdd_.SetAppCallback( (DWORD)(PVOID)me->VBIProcessor_, 0 );
   me->VBIEven_.SetAppCallback( (DWORD)(PVOID)me->VBIProcessor_, 0 );

   //
   // Set up window for Intercast
   //
   if (vbiInitiator == VBI_INTERCAST)
   {
      // calculate vertical window based on lines mask
      // find bit number of lowest bit ; 0-9 are reserved: illegal
      int i ;

      DWORD dwLoLine = 0 ;
      DWORD mask = 1 ;
      for ( i = 0 ; i < 32 ; i++)
      {
         if (dwVbiLinesMask & mask) 
           break ;

         mask <<= 1 ;
         dwLoLine++ ;
      }
      
      // find bit number of highest bit set
      DWORD dwHiLine = 31 ;
      mask = 0x80000000 ;
      for ( i = 0 ; i < 32 ; i++)
      {
         if (dwVbiLinesMask & mask) 
           break ;

         mask >>= 1 ;
         dwHiLine-- ;
      }

      // Both must be positive
      DWORD y0 = max ((int) dwLoLine - 10,0) ;
      DWORD y1 = max ((int) dwHiLine - 9, 0) ;

      // calculate horizontal size of window based on current protocol
      DWORD dwWidth = (CurrentIntercastProtocol == PROTOCOL_NABTS) ? SamplesNTSC : SamplesPAL ;

      TRect r( 0, y0, dwWidth, y1 );
      me->VBIOdd_.SetDigitalWindow( r );
      me->VBIEven_.SetDigitalWindow( r );
   }
   else
   //
   // Set up window for internal (CC) or external "WST" Interface
   //
   {
      if(IsNTSCCompatible(me->CaptureContrll_->GetFormat())) // NTSC (cc)
      {
         TRect r( 0, 0, 768, 12 );
         me->VBIOdd_.SetDigitalWindow( r );
         me->VBIEven_.SetDigitalWindow( r );
      }
      else // PAL (ttx)
      {
         TRect rOdd ( 0, 0, 960, 17 );
         TRect rEven( 0, 0, 960, 18 );
         me->VBIOdd_.SetDigitalWindow( rOdd );
         me->VBIEven_.SetDigitalWindow( rEven );
      }
   }
   me->VBIOdd_.Start(0);
   me->VBIEven_.Start(0);

   can_check = true;
}

/* Method: PsDevice::StopVBI
 * Purpose: Stops VBI capture
 * Input: None
 * Output: None
 */
void PsDevice::StopVBI()
{
   can_check = false;
   PsDevice *me = GetPsDevice();
   me->VBIOdd_.Stop();
   me->VBIEven_.Stop();

//   me->VBIOdd_.GetCurrentQue().Flush();
//   me->VBIEven_.GetCurrentQue().Flush();
   me->VBIProcessor_ = NULL;
}

/* Method: PsDevice::VBISetChannel
 * Purpose:
 * Input: dwChannel: DWORD, number of a channel
 */
ErrorCode PsDevice::VBISetChannel( DWORD dwChannel )
{
   ChangingChannel_ = false;

   // call into the current VBI decoder
   if ( VBI_Change_Processor( CHANNEL_CHANGE, dwChannel ) )
      return Success ;

   // this code is Intercast-specific
   if ( VBIDecoderInited_ ) {
      asm   mov   eax, dwChannel
      VxDCall( ICVBIDEC_Set_Channel );
      asm   jc error
      return Success;
   }
error:
   return Fail;
}

/* Method: PsDevice::CapDrvVBIIntrCallback
 * Purpose: This function is called when a VBI program generates the interrupt
 * Input: aStrm: VideoStream, even or odd
 *   dwBufAddr: DWORD, address of a buffer associated with this interrupt
 */
void PsDevice::CapDrvVBIIntrCallback( VideoStream aStrm, DWORD dwBufAddr )
{
   PsDevice *me = GetPsDevice();
   if ( can_check && aStrm == VS_VBIO ) {
      me->VBIOdd_.CallRing3Callback();
   }
}

//\\\\\\\\\\\\  functions below implement the proposed VBI interface \\\\\\\\
//\\\\\\\\\\\\  to be used by such decoders as WST VBI decoder       \\\\\\\\

/* API: Bt848_Open_VBI_Channel
 * Purpose: Opens VBI channels for the use by a ring 0 VBI decoder
 * Input: EDX contains a pointer to the session parameters structure
 * Output: CF is set on error.
 */
void _cdecl PsDevice::Bt848_Open_VBI_Channel()
{
   VBI_DECODE_SESSION *sess = (VBI_DECODE_SESSION *)_EDX;
   PsDevice *me = GetPsDevice();

   DebugPrint((0,"BT848: Bt848_Open_VBI_Channel: sess = %x\n",sess)) ;
   // somebody else ( ring 0 ) has claimed resource ?
   if ( me->CurrVBIDec_ ) {
      DebugPrint((0,"BT848: Open FAILED (another decoder is present)\n")) ;
      _EAX = 0;
      SetCarry();
      return;
   }
   if ( me->GrabVBIChannels() != Success ) { // somebody else has channel open ( ring3 ) ?
      DebugPrint((0,"BT848: Open FAILED (another ring3 decoder is present)\n")) ;
      _EAX = 0;
      SetCarry();
      return;
   }
   me->VBIProcessor_ = VBI_API_Processor;

   // this thing has info about the current client
   me->CurrVBIDec_ = new VBI_PARMS( *sess );
   if ( !me->CurrVBIDec_ ) {
      DebugPrint((0,"BT848: Open FAILED (no decoder given)\n")) ;
      _EAX = 0;
      SetCarry();
      return;
   }
   // provide the info back to the client

   /////////////////////////////////////////////////////////////////////////////////
   // For PAL, 
   //   pitch: 960*2=1920
   //   lines: For ODD field=17, EVEN field=18. For the quick fix, we'll just set it
   //          to the greater of the two, 18.
   // For NTSC,
   //   pitch: 768*2=1536
   //   lines: 12. This includes the VBI area and line 21 (CC Data)
   //
   // Oversampling,
   //   This member now reflects the sample rate:
   //   28,640,000 for NTSC, 35,470,000 for PAL 

   if(IsNTSCCompatible(me->CaptureContrll_->GetFormat())) // NTSC (cc)
   {
     sess->dwPitch = 1536;
     sess->dwLines = 12;
     sess->dwOversampling = 28640000 ;
   }
   else  // PAL (ttx)
   {
     sess->dwPitch = 1920;
     sess->dwLines = 18;
     sess->dwOversampling = 35470000;
   }

   DebugPrint((0,"BT848: Open Succeeded (handle = %x)\n",me->CurrVBIDec_)) ;
   ClearCarry(); // indicate no error
   _EAX = (DWORD)me->CurrVBIDec_; // to be used in all other calls
}

/* API: Bt848_Start_VBI_Capture
 * Purpose: Starts VBI capture
 * Input: EDX contains a handle returned by the open API
 * Output: CF is set on error.
 */
void _cdecl PsDevice::Bt848_Start_VBI_Capture()
{
   VBI_PARMS *client = (VBI_PARMS *)_EDX;
   PsDevice *me = GetPsDevice();
   // make sure the real thing is talking to us
   if ( me->CurrVBIDec_ != client || !client ) {
      DebugPrint((0,"BT848: NOT Starting VBI [%x != %x]\n",me->CurrVBIDec_,client)) ;
      SetCarry();
      return;
   }
   DebugPrint((0,"BT848: Starting VBI\n")) ;
   StartVBI(VBI_WST);
   ClearCarry();
}

/* API: Bt848_Stop_VBI_Capture
 * Purpose: Stops VBI capture
 * Input: EDX contains a handle returned by the open API
 * Output: CF is set on error.
 */
void _cdecl PsDevice::Bt848_Stop_VBI_Capture()
{
   VBI_PARMS *client = (VBI_PARMS *)_EDX;
   PsDevice *me = GetPsDevice();
   // make sure real thing is talking to us
   if ( me->CurrVBIDec_ != client || !client ) {
      SetCarry();
      return;
   }
   StopVBI();
   ClearCarry();
}

/* API: Bt848_Close_VBI_Channel
 * Purpose: Starts VBI capture
 * Input: EDX contains a handle returned by the open API
 * Output: CF is set on error.
 */
void _cdecl PsDevice::Bt848_Close_VBI_Channel()
{
   VBI_PARMS *client = (VBI_PARMS *)_EDX;
   PsDevice *me = GetPsDevice();
   // make sure real thing is talking to us
   if ( me->CurrVBIDec_ != client || !client  ) {
      SetCarry();
      return;
   }
   delete me->CurrVBIDec_;
   me->CurrVBIDec_ = NULL;
   me->FreeVBIChannels();
   ClearCarry();
}

/* Method: PsDevice::VBI_API_Processor
 * Purpose: This function is called to process VBI interrupt. It calls into the
 *    currently active VBI decoder
 * Input: aStrm: VideoStream
 *   dwBufAddr - address of the buffer
 * Output: None
 */
void PsDevice::VBI_API_Processor( VideoStream aStrm, DWORD dwBufAddr )
{
   PsDevice *me = GetPsDevice();
   DebugPrint((0,"BT848: Calling VBI decoder\n")) ;
   asm   pushad; // just in case
   _EDX = me->CurrVBIDec_->dwTag;
   _ECX = aStrm != VS_VBIO;
   _EBX = dwBufAddr;
   _EAX = me->CurrVBIDec_->dwDecoder;
   CALL_EAX;
   asm  popad;//
}

/* Method:  PsDevice::VBI_Change_Processor
 * Purpose: This function is called when either the Video Format changes or
 *          the channel changes. It calls the callback registered by the 
 *          currently active decoder.
 * Input:   VBI_CHANGE_TYPE ct - Either CHANNEL_CHANGE or VIDEO_FORMAT_CHANGE.
 *          DWORD arg          - If CHANNEL_CHANGE, arg is the new channel;
 *                               If VIDEO_FORMAT_CHANGE, arg is the new video 
 *                               format.
 * Output:  None
 */
BOOL PsDevice::VBI_Change_Processor( VBI_CHANGE_TYPE ct, DWORD arg )
{
   // ECX : what is changing : 0=channel, 1=video format
   // EDX : if channel change, edx gets new channel. If video formant change
   //       edx gets Video format value. (same as used by the capture driver 
   //       and decoder)
   BOOL st = FALSE ;
   PsDevice *me = GetPsDevice();
   asm   pushad; // just in case
   if ( me->CurrVBIDec_ )
   {
     if (me->CurrVBIDec_->dwChange)
     {
       _ECX = ct == CHANNEL_CHANGE ? 0 : 1 ;
       _EDX = arg ;
       _EAX = me->CurrVBIDec_->dwChange;
       CALL_EAX;
     }
     st = TRUE ;
   }
   asm  popad;
   return st ;
}
